//---------------------------------------------------------------------
// Copyright 2002-2005, Avocent Corporation
//---------------------------------------------------------------------
// FILE         : ds2mib.h
//
// DESCRIPTION :  This modules provides an interface between the DS appliance
//                code and the MIB structure.
//
//---------------------------------------------------------------------
// AUTHOR       : Jamie Collins
//
// DATE         : January 2002
//
//---------------------------------------------------------------------
// HISTORY
//
// DATE         PROGRAMMER        MODIFICATIONS
// January 2002  Jamie Collins    Initial version
//
//---------------------------------------------------------------------

#ifndef __DS2MIB_H
#define __DS2MIB_H

//#include "defines.h"
#include "dstypes.h"
#include "utf8.h"
//#include "eeparm.h"

//--------------------------------------------------------------------
// Struct AggSvrEntry
//
// Contains 10 bytes of sever information for a single server in an
// aggregate status message
//--------------------------------------------------------------------
typedef struct 
{
  unsigned char	b1Start;   // always zero
  unsigned char	b2Version; // version of the aggregate object
} AggSvrHeader;

typedef struct 
{
  unsigned char	b1AriPort;
  unsigned char	b2SwitchChannel;
  unsigned char	b3_b8RipEID[6];

  /*  left most four bits : power status   */
  /* right most four bits : session status */
  unsigned char b9Status;                          
  unsigned char b10Status;                          
} AggSvrEntry;

#if 0
typedef struct 
{
  AggSvrHeader	header;
  AggSvrPath    path;
} AggSvrEntry;
#endif

/*
 * struct used to pass session info to DsmibAddSession()
 */
typedef struct DsmibSession
{
  unsigned int    inputPort;
  RIP_ID          ripId; 
  unsigned int    switchChannel;
  UTF8            serverName;
  UTF8            userName;
  IP_ADDR         remoteIp;
  unsigned int    preemptionLevel; // local user preemption level
  unsigned int    accessLevel;     // local user access rights
  unsigned int    serverIdx;       // index into the server table
  unsigned int    mode;            // session mode
  unsigned int    owner;           // session owner
  
  unsigned int    type;            // session type
#define DS_SESSION_TYPE_KVM 1
#define DS_SESSION_TYPE_VM  2
  
  unsigned int    lockIndex;       // index of session this sess is locked to
  unsigned long   duration;
  unsigned long   sessionId;
  unsigned long   vidChan;
} DsmibSession;


/*
 * struct used to pass image file table info when using 
 *    DsmibSetImageFileTable() and DsmibGetImageFileTable()
 */
typedef struct DsmibImageFileTable
{

  unsigned long    index;
  UTF8             description;
  UTF8             version;
  UTF8             updateFileName;
  unsigned long    tftpIPAddress;
  int              upgradeControl;
  int              upgradeStatus;
  int              upgradeFailReason;
  UTF8             creationDate;
  UTF8             creationTime;
  UTF8             countryCode;
  UTF8             languageCode;
  unsigned long    headerVersion;

} DsmibImageFileTable;

typedef struct DsmibPppParms
{
  IP_ADDR  localIpAddress;
  IP_ADDR  remoteIpAddress;
  IP_ADDR  subnetAddress;
  IP_ADDR  gatewayAddress;
} DsmibPppParms;


/*
 * struct used to pass server table info when using 
 *    DsmibSetServerTable() and DsmibGetServerTable()
 */
typedef struct DsmibServerTable
{
  unsigned int  valid;  /* TRUE for valid entry */
  unsigned int  inputPort;
  RIP_ID        ripId;
  unsigned int  switchChannel;
  unsigned int  pemChannel;
#define DS_SERVER_NAME_MAXLEN  64
  UTF8          serverName;
  unsigned int  exitMacroIndex;

// DEBUGreh--DON'T know that there should be both status and session status--they
// are the same thing.  Initial glance at 

  unsigned int  status; // called powerStatus in DSView3x
#define DS_SERVER_STATUS_NOPOWER       1
#define DS_SERVER_STATUS_IDLE          2
#define DS_SERVER_STATUS_ACTIVE        3
#define DS_SERVER_STATUS_PATHBLOCKED   4

  unsigned int  sessionStatus;
#define DS_SESSION_STATUS_IDLE      1
#define DS_SESSION_STATUS_ACTIVE    2
#define DS_SESSION_STATUS_BLOCKED   3    /* also OFFLINE */

  unsigned int  switchType; 
#define DSRIPTABLESWITCHTYPE_NONE         1
#define DSRIPTABLESWITCHTYPE_UNKNOWN      2
#define DSRIPTABLESWITCHTYPE_AUTOVIEW2000 3
  
#define USR_ACL_SIZE 64
  char accessControlList[ USR_ACL_SIZE ];

  unsigned int  vmSessionStatus;
#define DS_VM_SESSION_STATUS_IDLE      1
#define DS_VM_SESSION_STATUS_ACTIVE    2
#define DS_VM_SESSION_STATUS_BLOCKED   3    /* also OFFLINE */

} DsmibServerTable ;

/*
 * struct used to pass RIP table info when using 
 *    DsmibSetRipTable() and DsmibGetRipTable()
 */
typedef struct DsmibRipTable
{
  unsigned int  valid;  /* TRUE for valid entry */
  unsigned int  inputPort;
  unsigned int  inputChan;
  RIP_ID        ripId;

  unsigned int  ripType;
#define DS_RIP_TYPE_PS2  1
#define DS_RIP_TYPE_SUN  2
#define DS_RIP_TYPE_USB  3
#define DS_RIP_TYPE_CCO  4
#define DS_RIP_TYPE_KCO  5
#define DS_RIP_TYPE_CMC  6 // Dell Shredder
#define DS_RIP_TYPE_SRL  7
#define DS_RIP_TYPE_UCO  8
#define DS_RIP_TYPE_USB2 9
#define DS_RIP_TYPE_PS2M 10

  unsigned int  switchType;
#define dSW_TYPE_NONE    1 // just a single server
#define dSW_TYPE_UNKNOWN 2 // any non-ACI cascade switch
#define dSW_TYPE_AV2000  3 // unique because it runs on ACI

  unsigned int  switchChannels;
  unsigned int  ripVendor;
#define DS_RIP_VENDOR_AVR  1
#define DS_RIP_VENDOR_DSR  2
#define DS_RIP_VENDOR_HP   3
#define DS_RIP_VENDOR_DELL 4
#define DS_RIP_VENDOR_IBM  5
#define DS_RIP_VENDOR_FS   6
#define DS_RIP_VENDOR_WL   7
#define DS_RIP_VENDOR_BB   8

  unsigned int  status;
#define DS_RIP_STATUS_OFFLINE  1
#define DS_RIP_STATUS_ONLINE   2
  unsigned long dwBootImageVersion;
  unsigned long dwAppImageVersion;
  unsigned long dwAppAvailableVersion;
  UTF8          bootImageVersion;
  UTF8          appImageVersion;
  UTF8          appAvailableVersion;
  UTF8          hardwareVersion;
  unsigned int  upgradeControl;
    // 1 = none
    // 2 = upgrade
  unsigned int  upgradeStatus;
#define DS_RIP_UPGRADE_STATUS_NA           0
#define DS_RIP_UPGRADE_STATUS_READY        1
#define DS_RIP_UPGRADE_STATUS_IN_PROGRESS  2
#define DS_RIP_UPGRADE_STATUS_SUCCEEDED    3
#define DS_RIP_UPGRADE_STATUS_FAILED       4

  unsigned int  upgradeFailReason;
#define DS_RIP_UPGRADE_FAIL_REASON_NO_FIRMWARE_IMAGE 1
#define DS_RIP_UPGRADE_FAIL_REASON_LOST_CONTACT      2
#define DS_RIP_UPGRADE_FAIL_REASON_FAILED_RESTART    3
#define DS_RIP_UPGRADE_FAIL_REASON_NOSTATUS          9999
  UTF8          name;             // name of server or name of cascade switch
  unsigned int  switchDetected;   // TRUE if detected; else FALSE
  unsigned int  keyboardLayout;
  UTF8          FPGAVersion;
    // Not used for PS/2 RIP
    // Sun layout codes for Sun RIP
    // USB layout codes for USB RIP
  unsigned long addSequence;      // used to determine which old RIP to discard 
  unsigned long presentInMainBoardList;  // used to determine if main board has seen this one

  // not in MIB
  unsigned int upgradable;               // used to determine if the RIP is upgradable or not
} DsmibRipTable;


/*
 * struct used to pass RIP table info when using 
 *    DsmibSetRipUpgradeTable() and DsmibGetRipUpgradeTable()
 */
typedef struct DsmibRipUpgradeTable
{
  unsigned long dwVersionAvailable;
  UTF8          versionAvailable;
  unsigned int  versionMismatches;
  unsigned int  control;
  unsigned int  status;           // use DS_RIP_UPGRADE_STATUS defines
  unsigned int  failReason;
} DsmibRipUpgradeTable;


/*
 * struct used to pass User table info when using 
 *    DsmibSetUserTable() and DsmibGetUserTable()
 */
typedef struct DsmibDsUserTable
{
  UTF8          name;
  UTF8          password;
  unsigned int  preemptionLevel;
  unsigned int  accessLevel;
  unsigned int  lockStatus;
} DsmibDsUserTable;
#define DS_USER_LOCKSTATUS_UNLOCKED 1
#define DS_USER_LOCKSTATUS_LOCKED   2

#define DS_USER_ACCESS_USER             1
#define DS_USER_ACCESS_ADMIN            2
#define DS_USER_ACCESS_APPLIANCE_ADMIN  3

/*
 * struct used to pass SNMP communities when using 
 *    DsmibSetCommunities() and DsmibGetCommunities()
 */
typedef struct DsmibCommunities
{
  UTF8    read;
  UTF8    write;
  UTF8    trap;
} DsmibCommunities;


/*
 * struct used to pass Encryption table info when using 
 *    DsmibSetKmEncrypTable() and DsmibGetKmEncrypTable()
 */
typedef struct DsmibEncrypTable
{
  unsigned encrypdes;      // TRUE when enabled; else FALSE
  unsigned encryp3des;     // TRUE when enabled; else FALSE
  unsigned encryp128;      // TRUE when enabled; else FALSE
  unsigned encrypAes;
} DsmibEncrypTable; 

/*
 * struct used to pass Encryption table info when using 
 *    DsmibSetKmEncrypTable() and DsmibGetKmEncrypTable()
 */
typedef DsmibEncrypTable DsmibKmEncrypTable;

/*
 * struct used to pass Encryption table info when using 
 *    DsmibSetVidEncrypTable() and DsmibGetVidEncrypTable()
 */
typedef DsmibEncrypTable DsmibVidEncrypTable;

/*
** Structure used to pass Authentication table info when using
** DsmibSetAuthenticationTable() and DsmibInitAuthenticationTable()
*/
typedef struct DsmibAuthenticationTable
{
  unsigned int authenticationOrder;
} DsmibAuthenticationTable; 

/*
** Structure used to pass Licensed Options table info when using
** DsmibGetOptionsTable() and DsmibInitOptionsTable()
*/
typedef struct DsmibOptionsTable
{
  UTF8         optionsName;
  unsigned int optionsStatus;
} DsmibOptionsTable; 

/*
** Structure used to pass LDAP table info when using
** DsmibGetLdapServerTable() and DsmibInitLdapServerTable()
*/
typedef struct DsmibLdapServerTable
{
  UTF8         serverName;
  unsigned int serverPort;
  unsigned int serverAccess;
} DsmibLdapServerTable; 

/*
** Structure used to pass LDAP parameter info when using
** DsmibGetLdapServerTable() and DsmibInitLdapServerTable()
*/
typedef struct DsmibLdapParameters
{
  UTF8   searchDN;
  UTF8   searchPasswd;
  UTF8   searchBase;
  UTF8   searchUid;
  unsigned int queryModeAppliance;
  unsigned int queryModeServer;
  UTF8   groupContainer;
  UTF8   groupContainerMask;
  UTF8   targetMask;
  UTF8   accessControl;
} DsmibLdapParameters; 

/*
** Structure used to pass Exit Macro table entries when
** using DsmibGetExitMacroTable() and DsmibInitExitMacroTable()
*/

#define DSMIB_EXIT_MACRO_MAX             16     // Maximum number of exit macros in MIB
#define DSMIB_EXIT_MACRO_MAX_SIZE       512     // Maximum number of bytes in one exit macro
#define DSMIB_EXIT_MACRO_NAME_MAX_SIZE   32     // Maximum size of exit macro name
 
typedef struct DsmibExitMacroTable
{
    unsigned char *     pExitMacro;
    int                 ExitMacroLength;
    UTF8                ExitMacroName;
} DsmibExitMacroTable;


typedef struct DsmibCascadeSwitch
{
  unsigned int  valid;  /* TRUE for valid entry */
  unsigned int  inputPort;
  RIP_ID        ripId;
  
#define DS_CASCADESWITCH_TYPE_NONE         1
#define DS_CASCADESWITCH_TYPE_UNKNOWN      2
#define DS_CASCADESWITCH_TYPE_AUTOVIEW2000 3
  unsigned int  type;

  unsigned int  switchChannels;

#define DS_CASCADESWITCH_STATUS_OFFLINE 1
#define DS_CASCADESWITCH_STATUS_ONLINE  2
  unsigned int  status;

#define DS_CASCADESWITCHNAME_MAXLEN 64
  UTF8          switchName;
} DsmibCascadeSwitch;


typedef struct DsmibSerialTable
{
  unsigned int typeIndex;
  unsigned int portIndex;
  unsigned int baudRate;
  unsigned int flowControl;
  #define DS_SERIAL_TABLE_PORTNAME_MAXLEN 64
  UTF8         portName;
  unsigned int timeout;
  unsigned int protocol;
} DsmibSerialTable;


/*
 * structs and symbols used to generate SNMP traps 
 * from outside the MIB. 
 */
#define TRAP_coldStart   xx
#define TRAP_warmStart
#define TRAP_userAuthFailure
#define TRAP_userLogin
#define TRAP_userLogout


typedef struct TrapInfo_coldStart
{
  UTF8    userName;   // name of user that caused it
} TrapInfo_coldStart;


typedef struct TrapInfo_warmStart
{
  UTF8    userName;   // name of user that caused it
} TrapInfo_warmtart;


typedef struct TrapInfo_userAuthFailure
{
  UTF8    userName;   // name of user that caused it
} TrapInfo_userAuthFailure;


typedef struct TrapInfo_userLogin
{
  UTF8    userName;   // name of user that caused it
} TrapInfo_userLogin;


typedef struct TrapInfo_userLogout
{
  UTF8    userName;   // name of user that caused it
} TrapInfo_userLogout;

/*
 * network status functions
 */
int dsNetworkConfigured( void );
void dsSetNetworkConfigured( int status );

/*
 * Initialization and access functions for the MIB data store 
 * from outside of the MIB code. 
 */

/* initializer functions */

void DsmibInitDigitizerTotal( unsigned numDigitizers );
void DsmibInitInputPortTotal( unsigned numInputPorts );
void DsmibInitSerialNumber( char *serialNumber );

void DsmibGetSysName( UTF8 * name );
void DsmibSetSysName( UTF8 * name );

void DsmibGetSysContact( UTF8 * contact );
void DsmibSetSysContact( UTF8 * contact );

void DsmibGetSysLocation( UTF8 * location );
void DsmibSetSysLocation( UTF8 * location );

unsigned DsmibGetSnmpControl( void );
void DsmibInitSnmpControl( unsigned control );

void DsmibGetCpqGUID( UTF8 * cpqGUID );
void DsmibSetCpqGUID( UTF8 * cpqGUID );

void DsmibGetCommunities( DsmibCommunities * ptr );
void DsmibInitCommunities( DsmibCommunities * ptr );


extern int DSR_COMMANAGERTABLE_MAXROW;
#define SNMP_MNGR_ADDR_COUNT    DSR_COMMANAGERTABLE_MAXROW
void DsmibGetMngrAddr( unsigned inx, IP_ADDR * ptr );
void DsmibInitMngrAddr( unsigned inx, IP_ADDR * ptr );

extern int DSR_COMTRAPHOSTTABLE_MAXROW;
#define SNMP_TRAP_ADDR_COUNT    DSR_COMTRAPHOSTTABLE_MAXROW
void DsmibGetTrapAddr( unsigned inx, IP_ADDR * ptr );
void DsmibInitTrapAddr( unsigned inx, IP_ADDR * ptr );

extern int DSR_COMMAXTRAPTYPES;
#define SNMP_TRAP_TYPE_COUNT    DSR_COMMAXTRAPTYPES 
unsigned DsmibGetTrapCfgEnable( unsigned inx );
void DsmibInitTrapCfgEnable( unsigned inx, unsigned enable );


/* access functions */
#define DSMIB_SESSION_MODE_NORMAL     1
#define DSMIB_SESSION_MODE_EXCLUSIVE  2
#define DSMIB_SESSION_MODE_SHARINGINTERACTIVE 3
#define DSMIB_SESSION_MODE_SHARINGPASSIVE     4
#define DSMIB_SESSION_MODE_SCAN       5
#define DSMIB_SESSION_MODE_STEALTH    6

#define DSMIB_SESSION_TABLE_COUNT  20
int DsmibGetSessionTableId( int inx );
int DsmibGetSessionTableByIndex( int inx, DsmibSession * ptr );
int DsmibGetSessionTableById( unsigned long sessionId, DsmibSession * ptr );
unsigned long DsmibAddSession( DsmibSession * ptr );
void DsmibDeleteSession( unsigned long sessionId );
void DsmibIncrementSessionDurations( void );
unsigned long DsmibGetSessionDuration( unsigned long sessionId );
int DsmibLockSession( int sessionId, int lockId );
int DsmibUnlockSession( unsigned long sessionId );

unsigned int getNumActiveKvmSessions(int port);

//void DsmibGetNetworkParms( EEpromNetParms * ptr );
//void DsmibInitNetworkParms( EEpromNetParms * ptr );

void DsmibGetImageFileTable( DsmibImageFileTable * ptr );
void DsmibSetImageFileTable( DsmibImageFileTable * ptr );
void DsmibGetImageTableTftpIpAddress( IP_ADDR * ipAddr );
void DsmibSetImageTableTftpIpAddress( IP_ADDR * ipAddr );
void DsmibGetImageTableFileName( UTF8 * fileName );
void DsmibSetImageTableFileName( UTF8 * fileName );
int  DsmibGetImageFileTableUpgradeControl(void);

#define DSMIB_SERVER_TABLE_NAME_MAXLEN   (15+1)
#ifndef TRIDENT_PLATFORM
 #define DSMIB_SERVER_TABLE_PORT_COUNT    16
#else
 #define DSMIB_SERVER_TABLE_PORT_COUNT    32
#endif
#define DSMIB_SERVER_TABLE_CHAN_COUNT    24
#define DSMIB_SERVER_TABLE_TOTAL_COUNT   ( DSMIB_SERVER_TABLE_PORT_COUNT * DSMIB_SERVER_TABLE_CHAN_COUNT )

void DsmibGetServerNameByPort( int port, int chan, char * name, int bufLen );
void DsmibGetServerTable( unsigned inx, DsmibServerTable * ptr );
void DsmibInitServerTable( unsigned inx, DsmibServerTable * ptr );
void DsmibSetServerTable( unsigned inx, DsmibServerTable * ptr );
int DsmibFindServerTable( DsmibServerTable * ptr );
int DsmibAddServerTable( DsmibServerTable * ptr );
void DsmibSetServerTable_Status( unsigned inx, unsigned status );

#ifndef TRIDENT_PLATFORM
 #define DSMIB_RIP_TABLE_PORT_COUNT  16
#else
 #define DSMIB_RIP_TABLE_PORT_COUNT  32
#endif
#define DSMIB_RIP_TABLE_CHAN_COUNT  (16+1)
#define DSMIB_RIP_TABLE_TOTAL_COUNT ( DSMIB_RIP_TABLE_PORT_COUNT * DSMIB_RIP_TABLE_CHAN_COUNT )

void DsmibGetRipTable( unsigned inx, DsmibRipTable * ptr );
void DsmibInitRipTable( unsigned inx, DsmibRipTable * ptr );
void DsmibSetRipTable( unsigned inx, DsmibRipTable * ptr );
int DsmibFindRipTable( DsmibRipTable * ptr );
int DsmibAddRipTable( DsmibRipTable * ptr );
void DsmibSetRipTable_Status( unsigned inx, unsigned status );

#define DSMIB_RIP_UPG_TABLE_COUNT       DSRIPUPGRADETABLE_MAXROW
void DsmibGetRipUpgradeTable( unsigned inx, DsmibRipUpgradeTable * ptr );
void DsmibSetRipUpgradeTable( unsigned inx, DsmibRipUpgradeTable * ptr );

void DsmibGetKBLanguage( unsigned char * langCode );
void DsmibSetKBLanguage( unsigned char langCode );

#define DSMIB_USER_TABLE_COUNT 64
int DsmibGetUserTable( unsigned inx, DsmibDsUserTable * ptr );
void DsmibInitUserTable( unsigned inx, DsmibDsUserTable * ptr );
void DsmibSetUserTable( unsigned inx, DsmibDsUserTable * ptr );
int DsmibFindUserTable( DsmibDsUserTable * ptr );
int DsmibAddUserTable( DsmibDsUserTable * ptr );

void DsmibGetKmEncrypTable( DsmibKmEncrypTable * ptr );
void DsmibInitKmEncrypTable( DsmibKmEncrypTable * ptr );
void DsmibSetKmEncrypTable( DsmibKmEncrypTable * ptr );

void DsmibGetVidEncrypTable( DsmibVidEncrypTable * ptr );
void DsmibInitVidEncrypTable( DsmibVidEncrypTable * ptr );

#ifndef TRIDENT_PLATFORM
#define DSMIB_CS_TABLE_COUNT 16
#else
#define DSMIB_CS_TABLE_COUNT 32
#endif
void DsmibGetCascadeSwitch( unsigned inx, DsmibCascadeSwitch * ptr );
void DsmibInitCascadeSwitch( unsigned inx, DsmibCascadeSwitch * ptr );
void DsmibSetCascadeSwitch( unsigned inx, DsmibCascadeSwitch * ptr );
//int DsmibFindCascadeSwitch( DsmibCascadeSwitch * ptr );
int DsmibFindCascadeSwitchByEID( DsmibCascadeSwitch * ptr );
int DsmibFindCascadeSwitchByPort( DsmibCascadeSwitch * ptr );
int DsmibAddCascadeSwitch( DsmibCascadeSwitch * ptr );
void DsmibSetSwitchTable_Status( unsigned inx, unsigned status );
int DsmibUpdateCascadeSwitch( DsmibRipTable * rip );

#define DSMIB_EXIT_MACRO_COUNT  16
void DsmibGetExitMacroTable ( unsigned inx, DsmibExitMacroTable * ptr );
void DsmibInitExitMacroTable( unsigned inx, DsmibExitMacroTable * ptr );
void dsmibSendExitMacroTable( void );
void dsmibSendExitMacroIndexTable( void );

unsigned long DsmibGetSessionTimeout( void );
void DsmibInitSessionTimeout( unsigned long timeout );
           
#define DSMIB_AUTHSERVER_ADDRESS_COUNT  4
void DsmibGetAuthServerAddress( unsigned inx, IP_ADDR * ptr );
void DsmibInitAuthServerAddress( unsigned inx, IP_ADDR * ptr );

void DsmibInitSessionOwner ( unsigned long sessionId, unsigned long sessionOwnerId );
void DsmibInitSessionMode ( unsigned long sessionId, unsigned long sessionMode );

unsigned DsmibGetLocalAuthSetting( void );
void DsmibInitLocalAuthSetting( unsigned initialSetting );

unsigned DsmibGetDefaultPreemptionLevel( void );
void DsmibInitDefaultPreemptionLevel( unsigned initialLevel );
unsigned DsmibGetPppPreemptionControl( void );
void DsmibInitPppPreemptionControl( unsigned control );
unsigned DsmibGetVideoNoiseControl( void );
void DsmibInitVideoNoiseControl( unsigned control );

unsigned long DsmibGetSessionPreemptionLevel( unsigned long sessionId );
unsigned long DsmibGetSessionAccessLevel( unsigned long sessionId );

unsigned long DsmibGetSessionPreemptionTimeout( void );
void DsmibInitSessionPreemptionTimeout( unsigned long timeout );

#define DSMIB_AUTHENTICATION_TABLE_COUNT  2
void DsmibGetAuthenticationTable( unsigned inx, DsmibAuthenticationTable * ptr );
void DsmibInitAuthenticationTable( unsigned inx, unsigned order );

#define DSMIB_OPTIONS_TABLE_COUNT 1
void DsmibGetOptionsTable( unsigned inx, DsmibOptionsTable * ptr );
void DsmibInitOptionsTable( unsigned inx, DsmibOptionsTable * ptr );

#define DSMIB_LDAP_SERVER_TABLE_COUNT 2
void DsmibGetLdapServerTable( unsigned inx, DsmibLdapServerTable * ptr );
void DsmibInitLdapServerTable( unsigned inx, DsmibLdapServerTable * ptr );
void DsmibGetLdapParameters( DsmibLdapParameters * ptr );
void DsmibInitLdapParameters(DsmibLdapParameters * ptr );

byte DsmibGetRcmChan( const DsmibServerTable * ptr );

int  DsmibGetApplianceStatus();
void DsmibSetApplianceStatus( unsigned status );
void DsmibTerminateASMPSessions( void );

unsigned long DsmibGetUserLockDuration( void );
void DsmibInitUserLockDuration( unsigned long duration );
void DsmibLockUser( unsigned inx );
void DsmibUnlockUser( unsigned inx );

int DsmibAuthenticateUser( char * username, int userNameLen, char * password, int passwordLen );

void dsBuildSingleAggSvrStsEntry(int index, AggSvrEntry * entry);
void dsBuildSingleAggSPCsocketStsEntry(int spcIndex, int socketIndex, AggSvrEntry * entry);
void dsBuildAggregateServerStatus(void);

int  DsmibGetPingResponse( void );
void DsmibInitPingResponse( int status );
void DsmibSetPingResponse( int status );

unsigned long DsmibGetPppAuthTimeout( void );
void DsmibInitPppAuthTimeout( unsigned long timeout );
void DsmibSetPppAuthTimeout( unsigned long timeout );

unsigned long DsmibGetPppInactivityTimeout( void );
void DsmibInitPppInactivityTimeout( unsigned long timeout );
void DsmibSetPppInactivityTimeout( unsigned long timeout );

void DsmibInitPppParms( DsmibPppParms * ptr );
void DsmibGetPppParms( DsmibPppParms * ptr );

#define DSMIB_SECURITY_MODE_TRUSTALL 1
#define DSMIB_SECURITY_MODE_SECURE   2
int  DsmibGetSecurityMode( void );
void DsmibInitSecurityMode( unsigned long mode );
void DsmibSetSecurityMode( unsigned long mode );

#define DSMIB_VM_LOCKED_TO_KVM_UNLOCKED 1
#define DSMIB_VM_LOCKED_TO_KVM_LOCKED   2
int  DsmibGetVmLockedToKvm( void );
void DsmibInitVmLockedToKvm( unsigned long locked );
void DsmibSetVmLockedToKvm( unsigned long locked );

#define DSMIB_VM_ACCESS_MODE_READONLY  1
#define DSMIB_VM_ACCESS_MODE_READWRITE 2
int  DsmibGetVmAccessMode( void );
void DsmibInitVmAccessMode( unsigned long mode );
void DsmibSetVmAccessMode( unsigned long mode );

void DsmibGetVmEncrypTable( DsmibEncrypTable * ptr );
void DsmibInitVmEncrypTable( DsmibEncrypTable * ptr );
void DsmibSetVmEncrypTable( DsmibEncrypTable * ptr );

#define DS_NAME_CHANGE_SERVER          0
#define DS_NAME_CHANGE_CASCADE_SWITCH  1
#define DS_NAME_CHANGE_SPC_SOCKET      2
void DsmibBumpNameChangedCounter( int type );
void DsmibInitNameChangedCounter( int type, int counter );
int DsmibGetNameChangedCounter( int type );

#define DS_TOPOLOGY_CHANGE_RIP         0
#define DS_TOPOLOGY_CHANGE_SERIAL      1
#define DS_TOPOLOGY_CHANGE_SPC         2
void DsmibBumpTopologyChangedCounter( int type );
void DsmibInitTopologyChangedCounter( int type, int counter );
int DsmibGetTopologyChangedCounter( int type );

#define SERIAL_PROTOCOL_UNSPECIFIED 1
#define SERIAL_PROTOCOL_AVOCENT     2    
#define SERIAL_PROTOCOL_SERVERTECH  3
#define SERIAL_PROTOCOL_BAYTECH     4
#define SERIAL_PROTOCOL_CYCLADES    5

#if HAVE2SPC
#define DSMIB_SERIAL_TABLE_COUNT 6 //DSSERIALTABLE_MAXROW * DSSERIALTABLE_MAX_PORTS
#else
#define DSMIB_SERIAL_TABLE_COUNT 3 //DSSERIALTABLE_MAXROW * DSSERIALTABLE_MAX_PORTS
#endif
void DsmibInitSerialTable( unsigned inx, DsmibSerialTable * ptr );
int DsmibGetSerialTable( unsigned inx, DsmibSerialTable * ptr );
void DsmibSetSerialTable_Protocol( unsigned inx, unsigned protocol );

#define LONG_NAME_DISPLAY_MODE_FIRSTN 1
#define LONG_NAME_DISPLAY_MODE_LASTN  2
int DsmibGetLongNameDisplayMode( void );
void DsmibInitLongNameDisplayMode( unsigned mode );


//-----------------------------------------------------------------
// Traps
//-----------------------------------------------------------------

void DsmibTrapColdStart( void );
void DsmibTrapLinkUp( unsigned ifNum );
void DsmibTrapLinkDown( unsigned ifNum );

void DsmibTrapUserAdded( PUTF8 userName, PUTF8 targetUserName );
void DsmibTrapUserDeleted( PUTF8 userName, PUTF8 targetUserName );
void DsmibTrapUserModified( PUTF8 userName, PUTF8 targetUserName );
void DsmibTrapUserLogin(PUTF8 userName );
void DsmibTrapUserLogout( PUTF8 userName );
void DsmibTrapUserAuthenticationFailure(PUTF8 userName );

void DsmibTrapColdStart( void );
void DsmibTrapLinkUp( unsigned ifNum );
void DsmibTrapLinkDown( unsigned ifNum );

void DsmibTrapRebootStarted( PUTF8 userName );
void DsmibTrapVideoSessionStarted(unsigned int sessionId);
void DsmibTrapVideoSessionStopped(unsigned int sessionId);
void DsmibTrapVideoSessionTerminated(PUTF8 preemptingUserName, unsigned int sessionId);
void DsmibTrapLocalPortViewingStarted(unsigned int sessionId);
void DsmibTrapLocalPortViewingStopped(unsigned int sessionId);
void DsmibTrapLocalPortViewingTerminated(PUTF8 preemptingUserName, unsigned int sessionId);
void DsmibTrapRipAdded  (PUTF8 ripEid, unsigned chan);
void DsmibTrapRipRemoved(PUTF8 ripEid, unsigned chan);
void DsmibTrapRipMoved  (PUTF8 ripEid, unsigned chan, unsigned newInputPort, unsigned oldInputPort);
void DsmibTrapScreenResolutionChanged(PUTF8 prevRes, PUTF8 currRes, int sessionId);
void DsmibTrapAggregateSvrStatusChg(unsigned int length, 
                                    unsigned char * aggregateServerStatus);

/* These are used for the imageType in imageFile and ripImage upgrades */
#define BOOT 1
#define APP  2
#define FPGA 3
void DsmibTrapImageFileUpgradeStarted(PUTF8 userName,int imageType,
                                      PUTF8 newVers, PUTF8 curVers);
void DsmibTrapImageFileUpgradeResult(int result, PUTF8 userName, int imageType,
                                     PUTF8 newVers, PUTF8 curVers);
void DsmibTrapRipImageUpgradeStarted(PUTF8 userName, int imageType, int ripTableIndx);
void DsmibTrapRipImageUpgradeResult(int result, PUTF8 userName, 
                                    int imageType, int ripTableIndx );
void DsmibTrapRipRestarted(PUTF8 ripEid);

void DsmibTrapUserLogin(PUTF8 userName );
void DsmibTrapUserLogout(PUTF8 userName );
void DsmibTrapFactoryDefaultsSet();
void DsmibTrapServerNameChanged( UTF8 * ripId, UTF8 * newName, UTF8 * oldName );
void DsmibTrapCascadeSwitchNameChanged( UTF8 * ripId, UTF8 * newName, UTF8 * oldName );
void DsmibTrapCascadeSwitchAdded( UTF8 * ripId, UTF8 * swName );
void DsmibTrapCascadeSwitchRemoved( UTF8 * ripId, UTF8 * swName );

#ifdef AVWORKS_CLIENT
void DsmibTrapVideoSessionTerminated( UTF8 * userName, UTF8 * targetName, UTF8 * serverName );
#endif

void DsmibTrapVirtualMediaSessionStarted(unsigned int sessionId);
void DsmibTrapVirtualMediaSessionStopped(unsigned int sessionId);
void DsmibTrapVirtualMediaSessionTerminated(PUTF8 preemptingUserName, unsigned int sessionId);
void DsmibTrapLocalPortVirtualMediaSessionStarted(unsigned int sessionId);
void DsmibTrapLocalPortVirtualMediaSessionStopped(unsigned int sessionId);
void DsmibTrapLocalPortVirtualMediaSessionTerminated(PUTF8 preemptingUserName, unsigned int sessionId);

typedef struct _dsAuthServerAddress {
	unsigned long	dsAuthServerAddressIpAddress;
} dsAuthServerAddress;

typedef struct _dsAppliance {
	int dsSecurityMode;
	dsAuthServerAddress dsAuthServerAddressEntry[ DSMIB_AUTHSERVER_ADDRESS_COUNT ];
} dsAppliance;

#endif /* __DS2MIB_H */

/* end */
