/*
 * Author: Tatu Ylonen <ylo@cs.hut.fi>
 * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
 *                    All rights reserved
 * Code for manipulating FIFO buffers.
 *
 * As far as I am concerned, the code I have written for this software
 * can be used freely for any purpose.  Any derived versions of this
 * software must be clearly marked as such, and if the derived work is
 * incompatible with the protocol description in the RFC file, it must be
 * called by a name other than "ssh" or "Secure Shell".
 */

/* RCSID("$OpenBSD: buffer.h,v 1.9 2001/06/26 17:27:23 markus Exp $"); */

#ifndef LIN_SSH_BUFFER_H
#define LIN_SSH_BUFFER_H

// SSH stuff

#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/md5.h>
#include <openssl/evp.h>

#define SSH_BUFFER_INCREMENT	4096

typedef struct SSH_Buffer
{
	char			*buf;	/* SSH_Buffer for data. */
	unsigned int	alloc;	/* Number of bytes allocated for data. */
	unsigned int	offset;	/* Offset of first byte containing data. */
	unsigned int	end;	/* Offset of last byte containing data. */
}	SSH_Buffer;

void ssh_buffer_init(SSH_Buffer *buffer);
u_int ssh_buffer_len(SSH_Buffer *buffer);
char *ssh_buffer_ptr(SSH_Buffer *buffer);
void ssh_buffer_get(SSH_Buffer *buffer, char *buf, u_int len);
void ssh_buffer_consume(SSH_Buffer *buffer, u_int bytes);
void ssh_buffer_free(SSH_Buffer *buffer);
int buffer_get_char(SSH_Buffer *buffer);
u_int buffer_get_int(SSH_Buffer *buffer);
char *buffer_get_string(SSH_Buffer *buffer, u_int *length_ptr);
void ssh_buffer_append(SSH_Buffer *buffer, const char *data, u_int len);
void ssh_buffer_append_space(SSH_Buffer *buffer, char **datap, u_int len);
void buffer_put_int(SSH_Buffer *buffer, u_int value);
void buffer_put_string(SSH_Buffer *buffer, const void *buf, u_int len);
void buffer_put_cstring(SSH_Buffer *buffer, const char *s);
void buffer_put_bignum(SSH_Buffer *buffer, BIGNUM *value);
int buffer_get_bignum(SSH_Buffer *buffer, BIGNUM *value);
void buffer_put_bignum2(SSH_Buffer *buffer, BIGNUM *value);
int buffer_get_bignum2(SSH_Buffer *buffer, BIGNUM *value);


#endif	/* LIN_SSH_BUFFER_H */
