//---------------------------------------------------------------------
// Copyright 2002, Avocent Corporation
//---------------------------------------------------------------------
// FILE         : utf8.h 
//
// DESCRIPTION  : UTF8 text string manipulation functions
//
//---------------------------------------------------------------------
// AUTHOR       : David Stafford
// DATE         : Jan-04-2002
//
//---------------------------------------------------------------------
// HISTORY
//
// DATE         PROGRAMMER        MODIFICATIONS
// Jan-04-2002  David Stafford    Initial version
//
//---------------------------------------------------------------------

#ifndef __UTF8_H
#define __UTF8_H

#include "dstypes.h"

#define UTF8_MAX_LENGTH  256	  /* max number of UTF8 encodings in string */
#define UTF8_MAX_BYTES   (UTF8_MAX_LENGTH*6)


/* defines type for UTF8 encoded strings */

typedef struct UTF8 
{
  unsigned int 	  numBytes;	    /* number of bytes in string */
  unsigned int	  numChars;	    /* number of characters in string */
  unsigned char 	bytes[ UTF8_MAX_BYTES ];
} UTF8 ;

typedef char * PUTF8;

#define ascii_to_utf8( utf, str ) utf8_init( utf, str )
char * utf8_to_ascii( char * str, UTF8 * utf, int maxlen );

void unicode_to_utf8( UTF8 * utf, UniCode * uni );
void utf8_to_unicode( UniCode * uni, UTF8 * utf, int maxlen );

int utf8_cmp( UTF8 * utf1, UTF8 * utf2 );
void utf8_cpy(  UTF8 * utf1, UTF8 * utf2 );

unsigned utf8_len( UTF8 * utf );
unsigned utf8_size( UTF8 * utf );



void utf8_nInit( UTF8 * utf, unsigned char * bytes, int numBytes );
void utf8_init( UTF8 * utf, unsigned char * bytes );


#endif /* __UTF8_H */

/* end */


