/*
 *  Module:  aidp.h (Avocent Install and Discovery Protocol)
 *
 *  Author:  Marcelo Peccin <marcelo.peccin@avocent.com>
 *
 *  From the original:  Avocent Install and Discovery Protocol (aidp.h file)
 *             Author:  John M. Welch (Built from EspUdp.h)
 *
 *  Copyright:  (c) 2006 Avocent Corporation
 *
 *  Date:  May 10, 2006
 *
 *  Comments:  Structures and definitions used by Avocent Install and Discovery Protocol.
 *             See also aidp.c and aidpaux.c files.
 */

#ifndef __AIDP_H__
#define __AIDP_H__

#undef  FALSE
#define FALSE      0
#undef  TRUE
#define TRUE    (!FALSE)

extern int debug;
#define printDEBUG	if (debug) printlog
void printlog(const char *fmt, ...);

// These defines are from the document AIDP_Model_Types.txt file archived at
// Sunrise SourceSafe Database in \DEPT\Development\Projects\Protocol Specifications
#define APP_CPS810       1
#define APP_CPS1610      2
#define APP_DS1800       3
#define APP_DSR1161      4
#define APP_DSR2161      5
#define APP_DSR4160      6
#define APP_COMPAQ1116   7
#define APP_COMPAQ3116   8
#define APP_DSR1000      9
#define APP_DSR2000     10
#define APP_DSR4000     11
#define APP_DELL2161DS  12
#define APP_IBMRCM2161  13
#define APP_BBOXKV120   14
#define APP_BBOXKV121   15
#define APP_FSCS21611   16
#define APP_FSCS22611   17
#define APP_DSR800      18
#define APP_DSR1010     19
#define APP_DSR2010     20
#define APP_DSR4010     21
#define APP_DSR1021     25
#define APP_DSR1020     26
#define APP_DSR2020     27
#define APP_DSR4020     28
#define APP_DSR8020     29
#define APP_DSR1022     30
#define APP_DSR1024     31

#define APP_DSR1030     48
#define APP_DSR2030     49
#define APP_DSR4030     50
#define APP_DSR8030     51

#define APP_DSR1031     61

#define APP_DSR2035     64
#define APP_DSR8035     65

#define APP_SVIP1020    69

#define APP_AV3100      80
#define APP_AV3200      81


///////////////////////////// These model codes were informed by Ken Harper.
#define APP_ACS1        0x52
#define APP_ACS4        0x53
#define APP_ACS8        0x54
#define APP_ACS16       0x55
#define APP_ACS32       0x56
#define APP_ACS48       0x57
#define APP_TS100       0x5c
#define APP_TS400       0x5d
#define APP_TS800       0x5e
#define APP_TS1000      0x5f
#define APP_TS2000      0x60
#define APP_TS3000      0x61
#define APP_TS110       0x62
///////////////////////////// (they are DSView official)


#define UDP_PROTOCOL   0x11
#define AIDP_PORT      3211

#define RXMAX_UDPBUF  0x8000 //size of RX data buffers for UDP
#define TXMAX_UDPBUF  0x8000 //size of TX data buffers for UDP


#define MAKE_WORD(h, l)  ((((word)(h))<<8)|(l))


// Data Normalization

#define RX_NORM_STRING(ppbuf, pstring, lcnt, binc) \
{ \
memcpy((pstring), (*(ppbuf)), (lcnt)); \
if (binc) \
	(*(ppbuf))+=(lcnt); \
}

#define RX_NORM_BYTE(ppbuf, xbyte, binc) \
{ \
(xbyte) = (*(ppbuf))[0]; \
if (binc) \
	(*(ppbuf))++; \
}

#define RX_NORM_WORD(ppbuf, pword, binc) \
{ \
((byte*)(pword))[0] = (*(ppbuf))[0]; \
((byte*)(pword))[1] = (*(ppbuf))[1]; \
if (binc) \
	(*(ppbuf))+=2; \
}

#define RX_NORM_DWORD(ppbuf, pdword, binc) \
{ \
((byte*)(pdword))[0] = (*(ppbuf))[0]; \
((byte*)(pdword))[1] = (*(ppbuf))[1]; \
((byte*)(pdword))[2] = (*(ppbuf))[2]; \
((byte*)(pdword))[3] = (*(ppbuf))[3]; \
if (binc) \
	(*(ppbuf))+=4; \
}

#define TX_NORM_STRING(pstring, ppbuf, lcnt, binc) \
{ \
memcpy((*(ppbuf)), (pstring), (lcnt)); \
if (binc) \
  (*(ppbuf))+=(lcnt); \
}

#define TX_NORM_BYTE(xbyte, ppbuf,  binc) \
{ \
(*(ppbuf))[0] = (xbyte); \
	if (binc) \
	  (*(ppbuf))++; \
}

#define TX_NORM_WORD(pword, ppbuf,  binc) \
{ \
(*(ppbuf))[0] = ((byte*)(pword))[0]; \
(*(ppbuf))[1] = ((byte*)(pword))[1]; \
if (binc) \
  (*(ppbuf))+=2; \
}

#define TX_NORM_DWORD(pdword, ppbuf,  binc) \
{ \
(*(ppbuf))[0] = ((byte*)(pdword))[0]; \
(*(ppbuf))[1] = ((byte*)(pdword))[1]; \
(*(ppbuf))[2] = ((byte*)(pdword))[2]; \
(*(ppbuf))[3] = ((byte*)(pdword))[3]; \
if (binc) \
  (*(ppbuf))+=4; \
}


typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

// FIELD Types

typedef struct /* MAC ADDR */
{
	byte  byFieldID;
    word  wFieldLen;
    byte  byMacAddr[6]; /* Address */
} F_MACADDR;

typedef struct /* IP ADDR */
{
	byte  byFieldID;
    word  wFieldLen;
    byte  byIPAddr[4]; /* Address */
} F_IPADDR;

typedef struct /* MASK */
{
	byte  byFieldID;
    word  wFieldLen;
    byte  byMask[4]; /* Mask */
} F_MASK;

typedef struct /* GW ADDR */
{
	byte  byFieldID;
    word  wFieldLen;
    byte  byGWAddr[4]; /* GW Address */
} F_GWADDR;

typedef struct /* STATUS */
{
	byte  byFieldID;
    word  wFieldLen;
    word  wStatus; /* Status Code */
} F_STATUS;
#define STAT_SUCCESS         0x00
#define STAT_IP_ADDR_EXISTS  0x01
#define STAT_INV_REQ         0x02
#define STAT_APP_ERR         0x03

typedef struct /* Appliance */
{
	byte  byFieldID;
    word  wFieldLen;
    word  wAppliance; /* Appliance Code */
} F_APP;


// REQUEST Messages
enum enum_NT_AIDPMessageType
{
    REQ_DISCOVER    = 0x01,
    REQ_TESTIPADDR  = 0x02,
    REQ_SETIPADDR   = 0x03,
	REQ_SNMPGET     = 0x10,
	REQ_SNMPGETNEXT = 0x11
};


// REPLY Messages
enum enum_APP_AIDPMessageType
{
    REPLY_DISCOVER    = 0x81,
    REPLY_TESTIPADDR  = 0x82,
    REPLY_SETIPADDR   = 0x83,
	REPLY_SNMPGET     = 0x90,
	REPLY_SNMPGETNEXT = 0x91
};


///////////////////////////////////////////////////////////////////////////
// NT_MSG_DISCOVER - Discover Request
typedef struct
{
    dword  dwUnused __attribute__ ((packed));
} NT_DISCOVER;

///////////////////////////////////////////////////////////////////////////
// NT_MSG_TESTIPADDR - IP Test Request
typedef struct
{
	F_MACADDR  MAC_Addr __attribute__ ((packed));
    F_IPADDR   IP_Addr __attribute__ ((packed));
	F_MASK	   Mask __attribute__ ((packed));
	F_GWADDR   GW_Addr __attribute__ ((packed));
} NT_TESTIPADDR;

///////////////////////////////////////////////////////////////////////////
// NT_MSG_SETIPADDR - IP Set Request
typedef struct
{
	F_MACADDR  MAC_Addr __attribute__ ((packed));
    F_IPADDR   IP_Addr __attribute__ ((packed));
	F_MASK	   Mask __attribute__ ((packed));
	F_GWADDR   GW_Addr __attribute__ ((packed));
} NT_SETIPADDR;

///////////////////////////////////////////////////////////////////////////
// APP_MSG_DISCOVER - Discover Reply
typedef struct
{
    F_APP      Appliance __attribute__ ((packed));
	F_MACADDR  MAC_Addr __attribute__ ((packed));
    F_IPADDR   IP_Addr __attribute__ ((packed));
	F_MASK	   Mask __attribute__ ((packed));
	F_GWADDR   GW_Addr __attribute__ ((packed));
} APP_DISCOVER;

///////////////////////////////////////////////////////////////////////////
// APP_MSG_TESTIPADDR - IP Test Reply
typedef struct
{
	F_STATUS  APP_Status __attribute__ ((packed));
} APP_TESTIPADDR;

///////////////////////////////////////////////////////////////////////////
// APP_MSG_SETIPADDR - IP Set Reply
typedef struct
{
	F_STATUS  APP_Status __attribute__ ((packed));
} APP_SETIPADDR;


#define APP_AIDP_SIGNATURE       "AIDP" //actual Signature string
#define APP_AIDP_SIGNATURE_SIZE      4  //length of APP_AIDP_SIGNATURE not including NULL
///////////////////////////////////////////////////////////////////////////
// AIDP Message Header
typedef struct
{
    byte   bySOH __attribute__ ((packed));
	char   szUniqueID[APP_AIDP_SIGNATURE_SIZE] __attribute__ ((packed));
	word   wSequenceNumber __attribute__ ((packed));
    byte   byMsgType __attribute__ ((packed));
    dword  dwMsgLen __attribute__ ((packed));
} APP_AIDP_HEADER;


///////////////////////////////////////////////////////////////////////////
// AIDP Data Message
typedef union
{
	NT_DISCOVER     NTDiscover __attribute__ ((packed));
	NT_TESTIPADDR   NTTestIpAddr __attribute__ ((packed));
	NT_SETIPADDR    NTSetIpAddr __attribute__ ((packed));
	APP_DISCOVER    APPDiscover __attribute__ ((packed));
	APP_TESTIPADDR  APPTestIpAddr __attribute__ ((packed));
	APP_SETIPADDR   APPSetIpAddr __attribute__ ((packed));
} APP_AIDP_DATA;


///////////////////////////////////////////////////////////////////////////
// AIDP Message
typedef struct
{
    APP_AIDP_HEADER  MsgHdr __attribute__ ((packed));  /* Header for Message */
    APP_AIDP_DATA    MsgData __attribute__ ((packed)); /* Union of Message Types */
} APP_AIDP_MESSAGE;


///////////////////////////////////////////////////////////////////////////
// AIDP Socket Message
typedef struct
{
	int                 udp_socket;
	unsigned char       recvfrom_ip[4];
	int                 recvfrom_port;
//	struct sockaddr_in  port;
    char                *MsgData;    /* Message */
	int                 reclen;   /* length of data */
} AIDP_SOCKET_MESSAGE;


#define PACK_HDR_LEN 42
///////////////////////////////////////////////////////////////////////////
// Raw Ethernet Packet (Header)
typedef struct
{
	// DLC Header           // byte
	byte destEthAddr[6];    // 0-5
	byte sourceEthAddr[6];  // 6-11
	byte ethType[2];        // 12-13
	// IP Header
	byte vers_hdr_len;      // 14
	byte tos;               // 15
	byte len[2];            // 16-17
	byte ID[2];             // 18-19
	byte flags_offs[2];     // 20-21
	byte ttl;               // 22
	byte protocol;          // 23
	byte checksum[2];       // 24-25
	byte sourceIpAddr[4];   // 26-29
	byte destIpAddr[4];     // 30-33
	// UDP Header
	byte srcPort[2];        // 34-35
	byte destPort[2];       // 36-37
	byte pktLen[2];         // 38-39
	byte UDPchecksum[2];    // 40-41
} UDP_IP_PacketHdr;

#endif //__AIDP_H__
