/***************************************************************************
 	Copyright (C) Cyclades Corporation, 1999.
 	All rights reserved
 
 	PR-3000 Router
 
 	This file:	cyccrc.c
 	Description:
   Called as cyccrc.exe <binary.file> <header of file>
   This utility will put a crc and total lenght in a binary file to warranty
 the total integrity in FTP transferences between Hosts and PR3000.
---------------------------------------------------------------------------
  	Change History
---------------------------------------------------------------------------
   mar/15/99	Elias Henrique Ferreira
  	Initial Implementation
***************************************************************************/

// CYCCRC EHF mar/99

#include	<fcntl.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<stdio.h>
#include	"cyccrc.h"

unsigned short 
calc_crcfile(unsigned long byte_n, unsigned short crc, char *buffer)
{
	unsigned int i,j,flag;

	for (i = 0; i < byte_n; i++) {
		crc = (((crc >> 8) ^ buffer[i]) << 8) | (0xFF & crc) ; 
        for(j = 0; j < 8; j++) {
			flag = (crc & 0x8000) >> 15;
			crc <<= 1;
			if (flag) {
				crc ^= 0x1021;
			}
		}
	}
	return(crc);
}
/* Converts strings in numbers */
int 
conv(char *pbuf){
	int ret = 0;
	int aux;

	while((*pbuf >= 0x30)&&(*pbuf <= 0x39)){
		aux = *pbuf++;
		aux -= 0x30;
		ret *= 10;
		ret += aux;
	}
	return (ret);
}

char	buffer[1024];

main(int argc, char *argv[])
{
	int source, target;
	unsigned int	bytes_read, crc_new, crc_old = 0;
	long	byte_cnt = 0;
	long	entry_point;
	long	seek_ret;
	int		offs, skip;
	struct {
		unsigned long st;
		unsigned char id[24];
		unsigned long ld;
	} x;

// Apr/24/00 Livio: Opcode ID
	if (argc != 6){
		fprintf(stderr, "%s: You must specify: source file, Cyclades Hdr len, Opcode ID,\n"
			"          output file and skip size\n", argv[0]);
		exit(-1);
	}
	
	offs = conv(argv[2]);	// Main header of binary file 
	skip = conv(argv[5]);	// ELF header of binary file 

	// Open the source input file
	if ((source = open(argv[1], O_RDWR)) == -1) {
		fprintf(stderr, "%s: Error openning %s\n",argv[0], argv[1]);
		exit(-2);
	}
	
	// open the output file
	if ((target = creat(argv[4], 0666)) == -1) {
		fprintf(stderr, "%s: Error openning %s\n",argv[0], argv[4]);
		exit(-3);
	}

	// Skip the ELF header
	seek_ret = lseek(source, (long)skip, SEEK_SET);
	if (seek_ret != (long)skip) {
		fprintf(stderr, "%s: Error skiping source file \n", argv[0]);
		exit(-4);
	}
	
	while ((bytes_read = read(source, buffer, sizeof(buffer))) > 0) {

		if (offs) {
			memcpy(&x, buffer, sizeof(x));
		}
		
		crc_new = calc_crcfile( (bytes_read - offs), crc_old, buffer + offs);
		byte_cnt += bytes_read;
		crc_old = crc_new;

		if (write(target, buffer, bytes_read) != bytes_read) {
			fprintf(stderr, "%s: Error writting target file \n", argv[0]);
			exit(-6);
		}

		offs = 0;	// used only in the first time

	}
	close(source);
	close(target);

	if ((target = open(argv[4], O_RDWR)) == -1) {
		fprintf(stderr, "%s: Error openning %s\n",argv[0], argv[4]);
		exit(-9);
	}
	
	// Building  the buffer to write
	buffer[4] = buffer[0] = (char)((byte_cnt >> 24) & 0xFF);
	buffer[5] = buffer[1] = (char)((byte_cnt >> 16) & 0xFF);
	buffer[6] = buffer[2] = (char)((byte_cnt >> 8) & 0xFF);
	buffer[7] = buffer[3] = (char)(byte_cnt & 0xFF);

	buffer[8] = (char)((crc_new >> 8) & 0xFF);
	buffer[9] = (char)(crc_new & 0xFF);

	// Apr/24/00 Livio: Opcode ID >> begin
	memcpy(&buffer[10], argv[3], 2);

	seek_ret = lseek(target, (long)(SIZE_OPCODE_OFFSET), SEEK_SET);
	if (seek_ret != (long)(SIZE_OPCODE_OFFSET)) {
		fprintf(stderr, "%s: Error seeking target file \n", argv[0]);
		exit(-7);
	}
	
	if (write(target, buffer, 12) != 12) {
		fprintf(stderr, "%s: Error writting target file \n", argv[0]);
		exit(-8);
	}

	close(target);
	x.id[11] = 0;

	x.st = ntohl(x.st);
	x.ld = ntohl(x.ld);

	printf("\n%s %s : %08x Load : %08x\n",argv[4], x.id, x.st, x.ld);
	return 0;
}
