# include <signal.h>
# include <errno.h>
# include <netdb.h>
# include <stdio.h>
# include <termios.h>
# include <unistd.h>
# include <fcntl.h>
# include <time.h>
# include <stdlib.h>

# include <sys/time.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <sys/socket.h>
# include <sys/ioctl.h>
# include <netinet/in.h>

// media & 0x1 = 1 ==> RS485
// media & 0x1 = 0 ==> RS232
// media & 0x6 = 6 ==> loopback
// media & 0x6 = 4 ==> Half Duplex
// media & 0x9 = 1 ==> RS485 terminator
//
#define RS232                   0       // 0000
#define RS232_HALF              4       // 0100
#define RS232_HALF_CTS          12      // 1100
#define RS485                   1       // 0001
#define RS485_HALF              13      // 1101
#define RS485_FULL              9      // 1001
#define RS485_HALF_TERM         5       // 0101
#define RS485_FULL_TERM         1       // 0001
#define RS232_LOOPBACK          6       // 0110
#define RS485_LOOPBACK          7       // 0111

#define MIN(x,y) ((x) > (y) ? (y) : (x))
#define MAX(x,y) ((x) < (y) ? (y) : (x))

#define MAX_LINE_PAGE	18
#define MAX_RUN_TABLE	128
#define MAX_DATA_RX		1024
#define MAX_DATA_TX		255

#define XDEBUG( x, P... ) if (debug & x) fprintf(stderr, ##P )
#define XDEBUG_ERR( x, y) if (debug & x) perror(y)

#define TmpStrLen 64

typedef struct _RUN_TABLE {
	struct in_addr ip;
	int		socket;
	ushort	port;
	time_t	time_rx_cmd;
	time_t	time_tx_cmd;
	int		id;
	short	status;
	short	flag_run;
	short	media;
	short	phase;
	int		pkt_rx;
	int		pkt_tx;
	int		tst_cycle;
	int		pkt_err;
	int		cts_err;
	int		dcd_err;
	int		dsr_err;
	int		len_cmd_tx;
	unsigned char	rfc2217_cmd_tx[TmpStrLen];
	int		len_cmd_rx;
	unsigned char	rfc2217_cmd_rx[TmpStrLen];
	int		len_data_rx;
	unsigned char	data_rx[MAX_DATA_TX+1];
	time_t	time_tx;
	int		iac_st;
	int		iac_sig;
} RUN_TABLE;

#define ST_INACT		0
#define ST_LOOPBACK		1
#define ST_SOCK_INI		2
#define ST_SOCK_ERR		3
#define ST_RUNNING		4
#define ST_PAUSE		5
#define ST_SOCK_RST		6
#define ST_WAIT_RST		7
#define ST_WAIT_STP		8
#define ST_CHG_MEDIA	9
#define ST_COM_LOST		10

#define MAX_ST_TXT		11

static char *st_table[MAX_ST_TXT] = {
	 ""
	,"No LpBk"
	,"Ini Sock"
	,"Err Sock"
	,"Running"
	,"Pause"
	,"Rst Sock"
	,"Wait Rst"
	,"Wait Stp"
	,"Ch Media"
	,"Com Lost"
};

#define CLS() printf("\x1b[2J\x1b[1;1H")

#define IACNormal		0
#define IACReceived		1
#define IACComReceiving	2

#define C *buf
#define IACEscape pt->iac_st
#define IACCommand pt->rfc2217_cmd_rx
#define IACPos pt->len_cmd_rx
#define IACSigEscape pt->iac_sig

/* Base Telnet protocol constants (STD 8) */
#define TNSE ((unsigned char) 240)
#define TNNOP ((unsigned char) 241)
#define TNBREAK ((unsigned char) 243)
#define TNSB ((unsigned char) 250)
#define TNWILL ((unsigned char) 251)
#define TNWONT ((unsigned char) 252)
#define TNDO ((unsigned char) 253)
#define TNDONT ((unsigned char) 254)
#define TNIAC ((unsigned char) 255)

/* Base Telnet protocol options constants (STD 27, STD 28, STD 29) */
#define TN_TRANSMIT_BINARY ((unsigned char) 0)
#define TN_ECHO ((unsigned char) 1)
#define TN_SUPPRESS_GO_AHEAD ((unsigned char) 3)
#define TN_LINEMODE ((unsigned char) 34)

/* Base Telnet Com Port Control (CPC) protocol constants (RFC 2217) */
#define TNCOM_PORT_OPTION ((unsigned char) 44)

/* CPC Client to Access Server constants */
#define TNCAS_SIGNATURE ((unsigned char) 0)
#define TNCAS_SET_BAUDRATE ((unsigned char) 1)
#define TNCAS_SET_DATASIZE ((unsigned char) 2)
#define TNCAS_SET_PARITY ((unsigned char) 3)
#define TNCAS_SET_STOPSIZE ((unsigned char) 4)
#define TNCAS_SET_CONTROL ((unsigned char) 5)
#define TNCAS_NOTIFY_LINESTATE ((unsigned char) 6)
#define TNCAS_NOTIFY_MODEMSTATE ((unsigned char) 7)
#define TNCAS_FLOWCONTROL_SUSPEND ((unsigned char) 8)
#define TNCAS_FLOWCONTROL_RESUME ((unsigned char) 9)
#define TNCAS_SET_LINESTATE_MASK ((unsigned char) 10)
#define TNCAS_SET_MODEMSTATE_MASK ((unsigned char) 11)
#define TNCAS_PURGE_DATA ((unsigned char) 12)
#define TNCAS_SET_MEDIA ((unsigned char) 20)

/* CPC Access Server to Client constants */
#define TNASC_SIGNATURE ((unsigned char) 100)
#define TNASC_SET_BAUDRATE ((unsigned char) 101)
#define TNASC_SET_DATASIZE ((unsigned char) 102)
#define TNASC_SET_PARITY ((unsigned char) 103)
#define TNASC_SET_STOPSIZE ((unsigned char) 104)
#define TNASC_SET_CONTROL ((unsigned char) 105)
#define TNASC_NOTIFY_LINESTATE ((unsigned char) 106)
#define TNASC_NOTIFY_MODEMSTATE ((unsigned char) 107)
#define TNASC_FLOWCONTROL_SUSPEND ((unsigned char) 108)
#define TNASC_FLOWCONTROL_RESUME ((unsigned char) 109)
#define TNASC_SET_LINESTATE_MASK ((unsigned char) 110)
#define TNASC_SET_MODEMSTATE_MASK ((unsigned char) 111)
#define TNASC_PURGE_DATA ((unsigned char) 112)
#define TNASC_SET_MEDIA ((unsigned char) 120)

/* Modem state effective change mask */
#define ModemStateECMask ((unsigned char) 240)

unsigned char data_tx[MAX_DATA_TX+1];

int max_tbl	= 0;
int sel_all	= 0;
int sel_tbl	= 0;
int ix_tbl	= 0;
int ix_line;
int confirm_command = 0;
int flag_change = 0;

RUN_TABLE run_table[MAX_RUN_TABLE];

typedef struct _CONFMSG {
	ulong	id;
	ulong   op;
	ulong   ip;
	ushort  port;
	ushort  f1;
} CONFMSG;

ushort COMMAND_PORT = 0x7cf3;

#define MAX_REFRESH_DEF		5
#define MAX_INTERV_TST_DEF	4

int MAX_REFRESH			= MAX_REFRESH_DEF;
int MAX_INTERV_TST 		= MAX_INTERV_TST_DEF;

CONFMSG cmd;

char *			IPlocal;

char *			Pgname;

int				MpSocket;

int debug = 0;

struct sockaddr_in	Mp_loc;

int len_sockaddr = sizeof(struct sockaddr_in);

int max_fd;

fd_set InFdSet;
fd_set ErrFdSet;

struct termios raw_term, std_term;

int get_command_sock(void);
void free_command_sock(void);
void handle_command_udp(void);
void handle_command_udp_timeout(void);
void handle_socket_input(void);
void handle_socket_output(void);
void handle_keyboard_command(void);
void wait_on_select(void);
void PrintStatsHeader (void);
int check_run_table(void);
void print_run_table(void);
void reset_control(RUN_TABLE *pt, int flag);
void set_raw_mode(void);
void ExitFunction(void);
void send_chg_media(RUN_TABLE *pt);

void HandleCPCCommand(RUN_TABLE *pt);
void SendTelnetOption(int SockFd, unsigned char Command, char Option);
void HandleIACCommand(RUN_TABLE *pt);
void HandleChar(unsigned char *buf, int len, RUN_TABLE *pt);
void SendCPCIntCommand(RUN_TABLE *pt, int flag, unsigned char Command, unsigned long int BR);
void SendCPCCommand(RUN_TABLE *pt, int flag, unsigned char Command);
void SendCPCByteCommand(RUN_TABLE *pt, int flag, unsigned char Command, unsigned char Parm);

void show_usage(void)
{
	fprintf(stderr, "Usage : %s [-d N] [-i interv] [-r refresh] [-p TCPport] IPlocal\n", Pgname);
	exit(2);
}

int main(int argc, char **argv)
{
	int fl, ret;
	unsigned long i = 0;
	extern int optind;
	time_t last = 0, now;

	for(fl=0; fl < MAX_DATA_TX; fl++) data_tx[fl] = fl+1;
	data_tx[MAX_DATA_TX] = 255;		/* two IAC is required */

	signal (SIGPIPE, SIG_IGN);
	signal(SIGHUP, SIG_IGN);

	Pgname = argv[0];

	while ((ret = getopt(argc, argv, "d:p:r:i:")) != EOF) {
		switch(ret) {
		case 'd':
			debug = atoi(optarg);
			break;
        case 'p':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			COMMAND_PORT = (ushort)atoi(optarg);
			break;
        case 'r':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			if ((MAX_REFRESH = atoi(optarg)) < 1 || MAX_REFRESH > 30) {
				MAX_REFRESH = MAX_REFRESH_DEF;
			}
			break;
        case 'i':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			if ((MAX_INTERV_TST = atoi(optarg)) < 1 || MAX_INTERV_TST > 15) {
				MAX_INTERV_TST = MAX_INTERV_TST_DEF;
			}
			break;
        default:
			show_usage();
		}
	}

	if (argc - optind != 1) {
		show_usage();
	}

	IPlocal  = argv[optind+0];

	XDEBUG(1, "get_command_sock\n");

	if (get_command_sock() != 0) {
		exit(1);
	}

	atexit(ExitFunction);

	set_raw_mode();

	while (1) {

		XDEBUG(1, "wait_on_select\n");
		wait_on_select();

		XDEBUG(1, "handle_command_udp\n");
		handle_command_udp();

		XDEBUG(1, "handle_command_udp_timeout\n");
		handle_command_udp_timeout();

		XDEBUG(1, "handle_socket_input\n");
		handle_socket_input();

		XDEBUG(1, "handle_socket_output\n");
		handle_socket_output();

		XDEBUG(1, "handle_keyboard_command\n");
		handle_keyboard_command();

		now  = time(NULL);

		if (!confirm_command && ((last + MAX_REFRESH < now) || flag_change)) {
			flag_change = 0;
			PrintStatsHeader();
			print_run_table();
			last = now;
		}
	}

	XDEBUG(1, "free_command_sock\n");
	free_command_sock();

}

void ExitFunction(void)
{
	tcsetattr(0, TCSANOW, &std_term);
}

void set_raw_mode(void)
{
    tcgetattr (0, &std_term);
    memset (&raw_term, 0, sizeof (raw_term));
    raw_term.c_cflag = (std_term.c_cflag & CBAUD) | CS8 | CREAD;
    raw_term.c_iflag = IGNPAR | IGNBRK;
	tcsetattr(0, TCSANOW, &raw_term);
}

void handle_socket_input()
{
	RUN_TABLE	*pt;
	int i, count;
	unsigned char data_rx[MAX_DATA_RX];

	for (i=0, pt=&run_table[0]; i < max_tbl; i++, pt++) {
		switch (pt->status) {
		case ST_PAUSE:			/* pause */
		case ST_RUNNING:		/* running */
		case ST_SOCK_INI:		/* Telnet negotiation */
		case ST_CHG_MEDIA:		/* Changing media */
			if (!FD_ISSET(pt->socket,&InFdSet)) {
				break;
			}

			if ((count = read(pt->socket, data_rx,sizeof(data_rx))) <= 0) {
				close(pt->socket);
				pt->status = ST_SOCK_ERR;
				break;
			}

			HandleChar(data_rx, count, pt);
			break;
		default:
			break;
		}
	}
}

void handle_socket_output()
{
	RUN_TABLE	*pt;
	int i, count;
	time_t now = time(NULL);

	for (i=0, pt=&run_table[0]; i < max_tbl; i++, pt++) {
		switch (pt->status) {
		case ST_RUNNING:		/* running */
			if (pt->time_tx + MAX_INTERV_TST > now) {
				break;
			}
			pt->time_tx = now;
			pt->tst_cycle++;

			if (pt->flag_run) {
				switch (pt->phase) {
				case 0:
					pt->pkt_err++;
					break;
				case 1:
				case 2:
					pt->dcd_err++;
					pt->cts_err++;
					pt->dsr_err++;
					break;
				}
			}

			if (!pt->media) {
				pt->phase = (pt->phase + 1) % 3;
			}

			switch (pt->phase) {
			case 1:
				SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK,0);
				SendCPCByteCommand(pt,0,TNCAS_SET_CONTROL,9);	/* DTR OFF */
				SendCPCByteCommand(pt,0,TNCAS_SET_CONTROL,11);	/* RTS ON */
				/* 0x80 dcd, 0x20 dsr, 0x10 cts */ 
				SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK, 0xb0);
				SendCPCByteCommand(pt,1,TNCAS_NOTIFY_MODEMSTATE, 0x10);
				pt->flag_run = 1;
				break;
			case 2:
				SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK,0);
				SendCPCByteCommand(pt,0,TNCAS_SET_CONTROL,8);	/* DTR ON */
				SendCPCByteCommand(pt,0,TNCAS_SET_CONTROL,12);	/* RTS OFF */
				/* 0x80 dcd, 0x20 dsr, 0x10 cts */ 
				SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK, 0xb0);
				SendCPCByteCommand(pt,1,TNCAS_NOTIFY_MODEMSTATE, 0xa0);
				pt->flag_run = 1;
				break;
			case 0:
				SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK,0);
				count = write(pt->socket, data_tx, MAX_DATA_TX+1);
				if (count != MAX_DATA_TX+1) {
					pt->pkt_err++;
				} else {
					pt->pkt_tx++;
					pt->flag_run = 1;
				}
				break;
			}
			break;
		default:
			break;
		}
	}
}

void handle_keyboard_command()
{
	int ch, i, imax;
	RUN_TABLE *pt;
	struct sockaddr_in	Mp_rem;	

	if (!FD_ISSET(0,&InFdSet)) {
		return;
	}

	if (sel_all) {
		i = 0;
		imax = max_tbl;
	} else {
		i = sel_tbl;
		imax = i + 1;
	}

	pt=&run_table[i];
	ch = getchar();

	if (confirm_command) {
		flag_change = 1;
		if (ch != 'y' && ch != 'Y') {
			confirm_command = 0;
			return;
		} else {
			ch = confirm_command;
		}
	}

	switch (ch) {
	case 'u':
	case 'U':
		for ( ; i < imax; i++, pt++) {
			if (pt->status == ST_PAUSE) {
				flag_change = 1;
				reset_control(pt, 0);
			}
		}
		break;
	case 't':
	case 'T':
		if (!max_tbl) {
			break;
		}

		if (!confirm_command) {
			confirm_command = 't';
			CLS();
			printf("This command will reboot ");
			if (sel_all) {
				printf("all TS100's !!!");
			} else {
				printf("the TS100 %s !!!", inet_ntoa(pt->ip));
			}
			printf("\r\n\n Do you confirm your option (Y)es or (N)o [n] : ");
			fflush(stdout);
			break;
		} else {
			confirm_command = 0;
		}

		for ( ; i < imax; i++, pt++) {
			flag_change = 1;
			pt->status = ST_WAIT_STP;
			pt->time_tx_cmd = time(NULL);
   			memset((char *)&cmd, 0, sizeof(cmd));
   			cmd.id = 0;
   			cmd.op = htonl(13);
   			cmd.ip = Mp_loc.sin_addr.s_addr;
   			cmd.port = pt->port;

			Mp_rem.sin_family = AF_INET;
			Mp_rem.sin_addr.s_addr = pt->ip.s_addr;
			Mp_rem.sin_port = htons(COMMAND_PORT);

			if (sendto (MpSocket, &cmd, sizeof(CONFMSG), 0,
				(struct sockaddr *) &Mp_rem, len_sockaddr) == -1) {
				XDEBUG(4, "%s : Can't send message\n", Pgname);
				XDEBUG_ERR (4, "sendto");
			}
		}
		break;
	case 'r':
	case 'R':
		for ( ; i < imax; i++, pt++) {
			if (pt->status == ST_PAUSE) {
				flag_change = 1;
				reset_control(pt, 1);
			}
		}
		break;
	case 'p':
	case 'P':
		for ( ; i < imax; i++, pt++) {
			if (pt->status == ST_RUNNING) {
				pt->status = ST_PAUSE;
				flag_change = 1;
				reset_control(pt, 2);
			}
		}
		break;
	case 's':
	case 'S':
		sel_all ^= 1;
		flag_change = 1;
		break;
	case '3':
		for ( ; i < imax; i++, pt++) {
			if (pt->media && pt->status == ST_PAUSE) {
				flag_change = 1;
				send_chg_media(pt);
			}
		}
		break;
	case '8':
		for ( ; i < imax; i++, pt++) {
			if (!pt->media && pt->status == ST_PAUSE) {
				send_chg_media(pt);
				flag_change = 1;
			}
		}
		break;
	case 0x03:		/* CTRL-C */
		if (!confirm_command) {
			confirm_command = 3;
			CLS();
			printf("This command will finish this ts1h_mon session !!! ");
			printf("\r\n\n Do you confirm your option (Y)es or (N)o [n] : ");
			fflush(stdout);
			break;
		} else {
			confirm_command = 0;
		}
		printf("\r\n");
		exit(0);
		break;
	case 10:		/* CTRL-J */
		if (sel_tbl + 1 < ix_tbl + MAX_LINE_PAGE &&
			sel_tbl + 1 < max_tbl) {
			flag_change = 1;
			++sel_tbl;
		}
		break;
	case 11:		/* CTRL-K */
		if (ix_tbl < sel_tbl) {
			flag_change = 1;
			--sel_tbl;
		}
		break;
	case 6:			/* CTRL-F */
		if (ix_tbl + MAX_LINE_PAGE < max_tbl) {
			ix_tbl += MAX_LINE_PAGE;
			sel_tbl = ix_tbl;
			flag_change = 1;
		}
		break;
	case 2:			/* CTRL-B */
		if (ix_tbl >= MAX_LINE_PAGE) {
			ix_tbl -= MAX_LINE_PAGE;
			sel_tbl = ix_tbl;
			flag_change = 1;
		}
		break;
	}
}

void send_chg_media(RUN_TABLE *pt)
{
	pt->status = ST_CHG_MEDIA;
	SendCPCIntCommand(pt, 1, TNCAS_SET_MEDIA, (pt->media ? RS232 : RS485_FULL));
	pt->flag_run = 1;
	pt->phase = 1;
	pt->time_tx_cmd = time(NULL);
}

void reset_control(RUN_TABLE *pt, int flag)
{
	if (flag == 3 && pt->media) {
		pt->media = 0;
		send_chg_media(pt);
		return;
	}

	if (flag == 1 || flag == 3) {
		pt->dsr_err = 0;
		pt->dcd_err = 0;
		pt->cts_err = 0;
		pt->pkt_err = 0;
		pt->pkt_rx = 0;
		pt->pkt_tx = 0;
		pt->tst_cycle = 0;
	}

	pt->flag_run = 0;
	pt->phase = 0;

	pt->time_tx = 0;
	pt->len_cmd_tx = 0;
	pt->len_cmd_rx = 0;
	pt->len_data_rx = 0;
	pt->iac_st = 0;
	pt->iac_sig = 0;

	if (flag != 2) {
		pt->status = ST_RUNNING;
	}

	flag_change = 1;
	SendCPCByteCommand(pt,0,TNCAS_SET_MODEMSTATE_MASK,0);
	SendCPCIntCommand(pt, 0, TNCAS_SET_MEDIA, 0x20001);

}


void start_test(RUN_TABLE *pt)
{
	struct sockaddr_in	sock_client;
	int fd, flags;

	flag_change = 1;

	sock_client.sin_family = AF_INET;
	sock_client.sin_addr.s_addr = pt->ip.s_addr;
	sock_client.sin_port = pt->port;

	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		pt->time_tx_cmd = time(NULL);
		pt->status = ST_SOCK_ERR;
		return;
	}

	if (connect(fd,(struct sockaddr *)&sock_client,len_sockaddr) < 0) {
		pt->time_tx_cmd = time(NULL);
		pt->status = ST_SOCK_ERR;
		return;
	}

	pt->time_tx_cmd = time(NULL);
	pt->socket = fd;
	pt->status = ST_SOCK_INI;
}

int check_run_table(void)
{
	int i;
	RUN_TABLE	*pt;

	for (i=0, pt=&run_table[0]; i < max_tbl; i++, pt++) {
		if (cmd.ip == pt->ip.s_addr) {
			break;
		}
	}

	if (i >= max_tbl) {
		if (i >= MAX_RUN_TABLE) {
			return(1);
		}
		memset(pt, 0, sizeof(RUN_TABLE));
		pt->ip.s_addr = cmd.ip;
		pt->status = (cmd.op == 1 ? ST_LOOPBACK : ST_INACT);
		pt->port = cmd.port;
		++max_tbl;
		flag_change = 1;
	}

	pt->time_rx_cmd = time(NULL);

	if (pt->status == ST_WAIT_STP) {
		cmd.op = 3;
		return(0);
	}

	switch (cmd.op) {
		case 1:
			pt->status = ST_LOOPBACK;
			break;
		case 2:
			if (pt->status == ST_INACT ||
				pt->status == ST_COM_LOST) {
				pt->status = ST_SOCK_RST;
				pt->time_tx_cmd = time(NULL);
				cmd.op = 4;
			}
			if (pt->status == ST_SOCK_RST) {
				pt->id = cmd.id;
				return(1);
			}
			if (pt->status == ST_LOOPBACK) {
				start_test(pt);
			}
			break;
		case 4:
			pt->time_tx_cmd = time(NULL);
			pt->status = ST_WAIT_RST;
			return(1);
			break;
	}

	return(0);
}

int get_command_sock()
{

	unsigned long inaddr;

	struct hostent *hp;

	if ((inaddr = inet_addr (IPlocal)) == INADDR_NONE) {
		if ((hp = gethostbyname (IPlocal)) == (struct hostent *) NULL) {
			XDEBUG(8, "%s : no such host : %s\n", Pgname,
				IPlocal);
			return(1);
		}
		memcpy ((char *) &inaddr, hp->h_addr, sizeof (long));
	}

	Mp_loc.sin_family = AF_INET;
	Mp_loc.sin_addr.s_addr = inaddr;
	Mp_loc.sin_port = ntohs(COMMAND_PORT);

	if ((MpSocket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
		XDEBUG(4, "%s : Can't alocate a socket in\n", 
				Pgname);
		XDEBUG_ERR (4, "socket");
		return (-1);
	}

	if (bind(MpSocket, (struct sockaddr *)&Mp_loc, len_sockaddr) < 0) {
		XDEBUG(4, "%s : Can't bind socket %s\n", 
				Pgname, IPlocal);
		XDEBUG_ERR (4,"bind");
		return (-1);
	}

	return (0);
}

void INC_FD_SET(int fd, fd_set *FdSet)
{
	FD_SET(fd, FdSet);

	if (fd >= max_fd) {
		max_fd = fd + 1;
	}
}

void wait_on_select(void)
{
	struct timeval BTimeout;
	RUN_TABLE	*pt;
	int i;

	FD_ZERO(&InFdSet);

	max_fd = 0;

	INC_FD_SET(0,&InFdSet);
	INC_FD_SET(MpSocket,&InFdSet);

	BTimeout.tv_sec = MAX_INTERV_TST;
	BTimeout.tv_usec = 0;

	for (i=0, pt=&run_table[0]; i < max_tbl; i++, pt++) {
		switch (pt->status) {
		case ST_PAUSE:			/* pause */
		case ST_RUNNING:		/* running */
		case ST_SOCK_INI:		/* Telnet negotiation */
		case ST_CHG_MEDIA:		/* Changing media */
			INC_FD_SET(pt->socket,&InFdSet);
			break;
		default:
			break;
		}
	}

	ErrFdSet = InFdSet;

	if (select(max_fd,&InFdSet,NULL,&ErrFdSet,&BTimeout) <= 0) {
		FD_ZERO(&InFdSet);
	}
}

void handle_command_udp_timeout(void)
{
	RUN_TABLE	*pt;
	int i;
	time_t now = time(NULL);
	struct sockaddr_in	Mp_rem;	

	for (i=0, pt=&run_table[0]; i < max_tbl; i++, pt++) {
		if (pt->status == ST_COM_LOST) {
			continue;
		}
		if (pt->time_rx_cmd + 30 < now) {
			pt->status = ST_COM_LOST;
			continue;
		}
		switch (pt->status) {
		case ST_SOCK_INI:		/* Telnet negotiation */
			if (pt->time_tx_cmd + 4 > now) {
				break;
			}
			close(pt->socket);
			pt->status = ST_SOCK_ERR;
			pt->time_tx_cmd  = now;
			break;
		case ST_SOCK_ERR:		/* connect error */
			if (pt->time_tx_cmd + 10 > now) {
				break;
			}
			start_test(pt);
			break;
		case ST_SOCK_RST:		/* reseting socket */
			if (pt->time_tx_cmd + 3 > now) {
				break;
			}

    		memset((char *)&cmd, 0, sizeof(cmd));

    		cmd.id = pt->id;
    		cmd.op = htonl(14);
    		cmd.ip = Mp_loc.sin_addr.s_addr;
    		cmd.port = pt->port;

			Mp_rem.sin_family = AF_INET;
			Mp_rem.sin_addr.s_addr = pt->ip.s_addr;
			Mp_rem.sin_port = htons(COMMAND_PORT);

			if (sendto (MpSocket, &cmd, sizeof(CONFMSG), 0,
				(struct sockaddr *) &Mp_rem, len_sockaddr) == -1) {
				XDEBUG(4, "%s : Can't send message\n", Pgname);
				XDEBUG_ERR (4, "sendto");
			}
			break;
		case ST_WAIT_RST:		/* waiting reseting socket */
			if (pt->time_tx_cmd + 5 > now) {
				break;
			}

			start_test(pt);
			break;
		case ST_WAIT_STP:
			if (pt->time_tx_cmd + 15 > now) {
				break;
			}
			if (i + 1 < max_tbl) {
				memcpy(pt, pt+1, (max_tbl-i-1) * sizeof(RUN_TABLE));
			}
			--max_tbl;
			if (sel_tbl && sel_tbl >= i) {
				--sel_tbl;
			}
			if (ix_tbl > sel_tbl) {
				ix_tbl = (ix_tbl > MAX_LINE_PAGE ? ix_tbl - MAX_LINE_PAGE : 0);
			}
			break;
		case ST_CHG_MEDIA:
			if (pt->time_tx_cmd + 3 > now) {
				break;
			}
			send_chg_media(pt);
			break;
		default:
			break;
		}
	}
}

void handle_command_udp(void)
{
	int		ret;
	struct sockaddr_in	Mp_rem;	

	if (!FD_ISSET(MpSocket,&InFdSet)) {
		return;
	}

	ret = recvfrom (MpSocket, (char *) &cmd, sizeof (cmd), 0,
		(struct sockaddr *) &Mp_rem, &len_sockaddr);

	if (ret == -1) {
		if (errno == EINTR) {
			return;	/* User abort */
		}
		XDEBUG(4, "\n\n%s : Can't receive message\n", Pgname);
		XDEBUG_ERR (4, "recvfrom");
		return;
	}

	cmd.op = ntohl(cmd.op);

	if (check_run_table()) {
		return;
	}

	cmd.op = htonl(cmd.op+10);

	cmd.ip = Mp_loc.sin_addr.s_addr;
	
	if (sendto (MpSocket, &cmd, sizeof(CONFMSG), 0,
		(struct sockaddr *) &Mp_rem, len_sockaddr) == -1) {
		XDEBUG(4, "%s : Can't send message\n", Pgname);
		XDEBUG_ERR (4, "sendto");
		return;
	}
}

void free_command_sock()
{
	close (MpSocket);
}

void PrintStatsHeader (void)
{
	CLS();

	printf (
		"    %02d to %02d of %02d         "
		"<---PACKETS-->         <---- ERRORS ---->\n\r",
		 ix_tbl+1, MIN(ix_tbl+MAX_LINE_PAGE, max_tbl), max_tbl);

	printf (
		"    IP address      Media TX      RX     "
		" Cycles  Data CTS  DCD  DSR  Status\n\r");

	printf (
		"    --------------------------------------"
		"------------------------------------\n\r");

	ix_line = 3;
}

void print_run_table(void)
{
    int i, imax;
	RUN_TABLE	*pt;

	if (!max_tbl) {
		printf("\n\rNo TS100 to be tested !!!\n\r");
		fflush(stdout);
		ix_line += 2;
		return;
	}

	imax = MIN(ix_tbl+MAX_LINE_PAGE+1, max_tbl);
 
    for (i=ix_tbl, pt=&run_table[ix_tbl]; i < imax; i++, ++pt) {
		printf("%s", (i == sel_tbl && sel_all == 0) ? "--> " : "    ");

		printf("%-15s ", inet_ntoa(pt->ip));

        printf("%s ", pt->media ? "RS485" : "RS232");

		printf("%-7d %-7d %-7d %-4d %-4d %-4d %-4d ",
			pt->pkt_tx,
			pt->pkt_rx,
			pt->tst_cycle,
			pt->pkt_err,
			pt->cts_err,
			pt->dcd_err,
			pt->dsr_err);

		if (pt->status && pt->status < MAX_ST_TXT) {
			printf("%s", st_table[pt->status]);
		}

		printf("\n\r");
		++ix_line;
    }
	while (ix_line < 23) {
		++ix_line;
		printf("\n\r");
	}

	printf("(S)elected[%s], RS2(3)2, RS4(8)5, (P)ause, "
			"(R)estart, Res(u)me, S(t)op : ",
			sel_all ? "all" : "-->");
	fflush(stdout);

}

void HandleChar(unsigned char *buf, int len, RUN_TABLE *pt)
{

	for ( ; len; len--, buf++) {

		/* Check the IAC escape status */
		switch (IACEscape) {
		case IACNormal: /* Normal status */
			if (C == TNIAC) {
		  		IACEscape = IACReceived;
				break;
			}
			if (pt->len_data_rx < MAX_DATA_TX) {
				pt->data_rx[pt->len_data_rx++] = C;
			} else {
				pt->flag_run = 0;
				pt->pkt_err++;
				pt->len_data_rx = 0;
			}
			break;

		case IACReceived: /* IAC previously received */
			if (C == TNIAC) {
				IACEscape = IACNormal;
				if (pt->len_data_rx < MAX_DATA_TX) {
					pt->data_rx[pt->len_data_rx++] = C;
				}
				if (pt->len_data_rx != MAX_DATA_TX ||
					memcmp(pt->data_rx, data_tx, MAX_DATA_TX)) {
					pt->pkt_err++;
				} else {
					pt->pkt_rx++;
				}
				pt->flag_run = 0;
				pt->len_data_rx = 0;
				break;
			}

			IACCommand[0] = TNIAC;
			IACCommand[1] = C;
			IACPos = 2;
			IACEscape = IACComReceiving;
			IACSigEscape = IACNormal;
			if (C == TNBREAK) {
				HandleIACCommand(pt);
			}
			break;

		/* IAC Command reception */
		case IACComReceiving:
			/* Telnet suboption, could be only CPC */
			if (IACCommand[1] == TNSB) {
				/* Get the suboption signature */
				if (IACPos < 4) {
					IACCommand[IACPos] = C;
					IACPos++;
					break;
				}

				/* Check which suboption we are dealing with */
				switch (IACCommand[3]) {
				/* Signature, which needs further escaping */
				case TNASC_SIGNATURE:
					switch (IACSigEscape) {
					case IACNormal:
						if (C == TNIAC)
							IACSigEscape = IACReceived;
						else {
							if (IACPos < TmpStrLen) {
								IACCommand[IACPos] = C;
								IACPos++;
							}
						}
						break;

					case IACComReceiving:
						IACSigEscape = IACNormal;
						break;

					case IACReceived:
						if (C == TNIAC) {
							if (IACPos < TmpStrLen) {
								IACCommand[IACPos] = C;
								IACPos++;
							}
							IACSigEscape = IACNormal;
						} else {
							if (IACPos < TmpStrLen) {
								IACCommand[IACPos] = TNIAC;
								IACPos++;
				  			}

							if (IACPos < TmpStrLen) {
								IACCommand[IACPos] = C;
								IACPos++;
				  			}

							HandleIACCommand(pt);
						}
			  			break;
					}
					break;

				case TNASC_SET_MEDIA:
				case TNASC_SET_BAUDRATE: /* Set baudrate */
					IACCommand[IACPos] = C;
					IACPos++;

					if (IACPos == 10) {
						HandleIACCommand(pt);
					}
					break;

				/* Flow control command */
				case TNASC_FLOWCONTROL_SUSPEND:
				case TNASC_FLOWCONTROL_RESUME:
					IACCommand[IACPos] = C;
					IACPos++;

					if (IACPos == 6) {
						HandleIACCommand(pt);
					}
					break;

				/* Normal CPC command with single byte parameter */
				default:
					IACCommand[IACPos] = C;
					IACPos++;

					if (IACPos == 7) {
			  			HandleIACCommand(pt);
					}
					break;
				}					   

			} else {
				/* Normal 3 byte IAC option */
				IACCommand[IACPos] = C;
				IACPos++;

				if (IACPos == 3) {
					HandleIACCommand(pt);
				}
			}
			break;
		}
	}
}

void HandleIACCommand(RUN_TABLE *pt)
{
	IACEscape = IACNormal;

	/* Check which command */
	switch(IACCommand[1]) {
	case TNBREAK:
		break;

	/* Suboptions */
	case TNSB:
		switch (IACCommand[2]) {
		/* RFC 2217 COM Port Control Protocol option */
		case TNCOM_PORT_OPTION:
			HandleCPCCommand(pt);
			break;
		}
		break;

	/* Requests for options */
	case TNWILL:
		switch (IACCommand[2]) {
		/* COM Port Control Option */
		case TNCOM_PORT_OPTION:
			SendTelnetOption(pt->socket,TNDO,TNCOM_PORT_OPTION);
			reset_control(pt, 3);
			break;

		/* Telnet Binary mode */
		case TN_TRANSMIT_BINARY:
			break;

		/* Echo request not handled */
		case TN_ECHO:
			break;

		/* No go ahead needed */
		case TN_SUPPRESS_GO_AHEAD:
			SendTelnetOption(pt->socket,TNDO,TN_TRANSMIT_BINARY);
			break;

		/* Reject everything else */
		default:
			SendTelnetOption(pt->socket,TNDONT,IACCommand[2]);
			break;
		}
		break;

	/* Confirmations for options */
	case TNDO:
		switch (IACCommand[2]) {
		/* COM Port Control Option */
		case TNCOM_PORT_OPTION:
			SendTelnetOption(pt->socket,TNWILL,TNCOM_PORT_OPTION);
			break;

		/* Telnet Binary mode */
		case TN_TRANSMIT_BINARY:
			break;

		/* Echo reques1t not handled */
		case TN_ECHO:
			break;

		/* No go ahead needed */
		case TN_SUPPRESS_GO_AHEAD:
			break;

		/* Reject everything else */
		default:
			SendTelnetOption(pt->socket,TNWONT,IACCommand[2]);
			break;
		}
		break;
	
	/* Notifications of rejections for options */
	case TNWONT:
	case TNDONT:
		break;
	}
}

static unsigned char IACseq[4] = {TNIAC, 0,0,0};

void SendTelnetOption(int SockFd, unsigned char Command, char Option)
  {
	IACseq[1] = Command;
	IACseq[2] = Option;

	write(SockFd, IACseq, 3);
  }

void HandleCPCCommand(RUN_TABLE *pt)
{
	XDEBUG(2, "Command TX=%x RX=%x\r\n", 
	pt->rfc2217_cmd_tx[3], pt->rfc2217_cmd_rx[3]);
	XDEBUG(16, "flag_run\r\n");
	if (pt->flag_run == 0) {
		return;
	}
	XDEBUG(16, "len cmd\r\n");
	if (pt->len_cmd_rx != pt->len_cmd_tx) {
		return;
	}
	XDEBUG(16, "command\r\n");
	if (pt->rfc2217_cmd_tx[3]+0x64 != pt->rfc2217_cmd_rx[3]) {
		return;
	}
	XDEBUG(16, "phase\r\n");
	if (!pt->phase) {
		return;
	}

	switch(pt->rfc2217_cmd_tx[3]) {
	case TNCAS_NOTIFY_MODEMSTATE:
		pt->flag_run = 0;
		if ((pt->rfc2217_cmd_tx[4] & 0x80) != (pt->rfc2217_cmd_rx[4] & 0x80)) {
			pt->dcd_err++;
		}
		if ((pt->rfc2217_cmd_tx[4] & 0x20) != (pt->rfc2217_cmd_rx[4] & 0x20)) {
			pt->dsr_err++;
		}
		if ((pt->rfc2217_cmd_tx[4] & 0x10) != (pt->rfc2217_cmd_rx[4] & 0x10)) {
			pt->cts_err++;
		}
		break;
	case TNCAS_SET_MEDIA:
		if (pt->status != ST_CHG_MEDIA) {
			break;
		}
		if (*(ulong *)&pt->rfc2217_cmd_tx[4] !=
			*(ulong *)&pt->rfc2217_cmd_rx[4]) {
			send_chg_media(pt);
			break;
		}
		pt->media ^= 1;
		reset_control(pt, 1);
		break;
	}
}

static unsigned char char_cmd_byte[7] =
    {TNIAC,TNSB,TNCOM_PORT_OPTION, 0, 0, TNIAC,TNSE};

void SendCPCByteCommand(RUN_TABLE *pt, int flag, unsigned char Command, unsigned char Parm)
{
	char_cmd_byte[3] = Command;
	char_cmd_byte[4] = Parm;
	write(pt->socket,char_cmd_byte,7);
	if (flag) {
		pt->len_cmd_tx = 7;
		memcpy(pt->rfc2217_cmd_tx, char_cmd_byte, 7);
	}
}


static unsigned char char_cmd[6] =
    {TNIAC,TNSB,TNCOM_PORT_OPTION, 0, TNIAC,TNSE};

void SendCPCCommand(RUN_TABLE *pt, int flag, unsigned char Command)
{
	char_cmd[3] = Command;
	write(pt->socket,char_cmd,6);
	if (flag) {
		pt->len_cmd_tx = 6;
		memcpy(pt->rfc2217_cmd_tx, char_cmd, 6);
	}
}

static unsigned char char_cmd_int[10] =
    {TNIAC,TNSB,TNCOM_PORT_OPTION, 0, 0, 0, 0, 0, TNIAC,TNSE};

void SendCPCIntCommand(RUN_TABLE *pt, int flag, unsigned char Command, unsigned long int BR)
{
	*(int *)&char_cmd_int[4] = htonl(BR);
	char_cmd_int[3] = Command;
	write(pt->socket, char_cmd_int, 10);
	if (flag) {
		pt->len_cmd_tx = 10;
		memcpy(pt->rfc2217_cmd_tx, char_cmd_int, 10);
	}
}

