#!/bin/sh 
# Upgrade script for device's software V 3.0.0

DATE=`date '+%D %H:%M:%S'`
TIMESTAMP=`date '+%Y%m%d%H%M%S'`
TEMP=/tmp/.upgrade
NEEDSAVE=0

#activate shadow
if [ -z "`grep -s 'root:x:0:0:' /etc/passwd`" ] ; then
  echo -n "Translating /etc/passwd entries to /etc/shadow..."
  rm -f /etc/shadow
  pwconv
  if ! grep -q /etc/shadow /etc/config_files; then
    echo "/etc/shadow" >> /etc/config_files
  fi
  NEEDSAVE=1
  echo " Done."
fi

#create group wheel
if ! grep -q "wheel::" /etc/group; then
  echo "wheel::100:" >> /etc/group
  echo "Updated /etc/group" 
  NEEDSAVE=1
fi

# Include the events notification in the syslog-ng.conf file
if [ -f /etc/syslog-ng/syslog-ng.conf ]; then 
  if ! grep -q evt_syslog_pipe /etc/syslog-ng/syslog-ng.conf; then
    echo "# events notification " >> /etc/syslog-ng/syslog-ng.conf
    echo "source s_evt { pipe(\"/var/run/evt_syslog_pipe\"); };" >> /etc/syslog-ng/syslog-ng.conf
    echo "log { source(s_evt); destination(console); };" >> /etc/syslog-ng/syslog-ng.conf
    echo "Updated /etc/syslog-ng/syslog-ng.conf" 
    NEEDSAVE=1
  fi
fi

# Include the events notification in the syslog-ng.conf file
if [ -f /etc/config_files ]; then 

  if grep -q "/etc/network/ifcfg" /etc/config_files; then
    cp /etc/config_files /tmp/config_files
    grep -v "/etc/network/ifcfg" /tmp/config_files > /etc/config_files
  fi

  # DLA & ADSAP2 stuff
  if ! grep -q event_notif.conf /etc/config_files; then
    echo -e \
      "/etc/event_notif.conf\n" \
      "/etc/event_notif.lnk\n" \
      "/etc/logpipe/logpipe.conf\n" \
      "/etc/adsap2.conf\n" \
      "/etc/CA/ssh_appliance_key" >> /etc/config_files
    echo "Updated /etc/config_files" 
    NEEDSAVE=1
  fi

  # Replace cron with crond
  if  grep -q /etc/inittab /etc/config_files; then
    if ! grep -q crond /etc/inittab; then
      cp /etc/inittab /tmp/inittab
      awk '$0 == "::once:/sbin/cron" {print "::once:/sbin/crond"; next}; {print $0};' \
          /tmp/inittab > /etc/inittab
      echo "Updated /etc/inittab (crond)" 
      NEEDSAVE=1
    fi
    if ! grep -q gettty.sh /etc/inittab; then
      cp /etc/inittab /tmp/inittab
      awk '$0 ~ /^ttyS0::respawn/ {print "::once:/bin/getty.sh init"; next}; \
           $0 ~ /cy_buffering/ {next}; \
           $0 == "::once:/bin/webs" {next}; \
           {print $0};' \
          /tmp/inittab > /etc/inittab
      echo "Updated /etc/inittab (gettty.sh)" 
      NEEDSAVE=1
    fi
  fi

  if  grep -q rc.sysinit /etc/config_files; then
    # the rc.sysinit was customized by the client
    if ! grep -q /var/run/evt_syslog_pipe /etc/rc.sysinit; then
      echo -e \
        "# create the fifo devices for event notifications\n" \
        "mkfifo -m 666 /var/run/snmp_pipe\n" \
        "mkfifo -m 666 /var/run/evt_syslog_pipe\n" \
        "mkfifo -m 666 /dev/evt_pipe_logpipe" >> /etc/rc.sysinit
      echo "Updated 'mkfifo' in /etc/rc.sysinit" 
      NEEDSAVE=1
    fi

    # New interface initialization method
    if ! grep -q "/etc/network/intf_cfg" /etc/rc.sysinit; then
      cp /etc/rc.sysinit /tmp/rc.sysinit
      awk '$1 == "/bin/hostname" {print $0 "\n\n# Create shared memory\n/sbin/cy_shm"; next};\
           $0 == "  /sbin/sh \$file" {print $0 " start"; next}; \
           $4 == "/etc/network/ifcfg\*" {print "for file in /etc/network/intf_cfg* /etc/network/ifcfg_lo"; next}; \
           $0 == "/sbin/cy_shm" {next}; \
           $0 == "# Create shared memory" {next}; \
           {print $0};' /tmp/rc.sysinit > /etc/rc.sysinit
      echo "Updated 'interface initialization' in /etc/rc.sysinit" 
    fi
  fi

  # more DLA & ADSAP2 stuff
  if  grep -q daemon_list /etc/config_files; then
    # the daemon_list was customized by the client
    if ! grep -q EVTGEN /etc/daemon_list; then
      echo -e \
        "DB          /etc/cy_buffering.sh\n" \
        "WEB         /etc/webui.sh\n" \
        "EVTGEN      /etc/event_gen.sh\n" \
        "LOGPIPE     /etc/logpipe.sh\n" \
        "ADSAP2      /etc/adsap2.sh" >> /etc/daemon_list
      echo "Updated /etc/daemon_list" 
      NEEDSAVE=1
    fi
  fi

  # include the root directory
  if ! grep -q "/root" /etc/config_files; then
    echo -e "/root" >> /etc/config_files
    NEEDSAVE=1
  fi

fi

# verify the "admin" user and the "admin" group
if ! grep -q "^admin:" /etc/passwd; then
  #echo admin_password ADMPAS="AD"${1}
  ADMPAS="AD"${1}
  adduser -g admin -G admin -P $ADMPAS admin
  echo '***************************************************************************'
  echo 'For DSView the "admin" user account was created with one randomly password.'
  echo 'After log-in as root, run the "passwd" command to change it.'
  echo '***************************************************************************'
else
  if ! grep -q "[,:]admin" /etc/group; then
    echo '**************************************************************'
    echo 'For DSView the "admin" user must be a member of "admin" group.'
    echo 'After log-in as root, run the "addadmin" command.'
    echo '**************************************************************'
  fi
fi

if ! grep -q "^PIDF" /etc/sshd.sh; then
  echo "PIDF=/var/run/sshd.pid" >> /etc/sshd.sh
  NEEDSAVE=1
fi

#Validate all modification
if [ $NEEDSAVE = 1 ]
then
    /bin/saveconf 
fi
