#!/bin/sh

VALUE=0              # working variable
MAX_TEMP=45          # maximum temperature in Celsius 
H_VCC33=231          # VCC +3.3V high limit (110%) 
L_VCC33=189          # VCC +3.3V low limit  (90%)
H_VCC5=231           # VCC +5V high limit   (110%)
L_VCC5=189           # VCC +5V low limit    (90%)
H_VCC12=231          # VCC +12V high limit  (110%)
L_VCC12=189          # VCC +12V low limit   (90%)

TMP=2                # Number of times Temperature warning will be sent
V33=2                # Number of times VCC+3.3 warning will be sent 
V5=2                 # Number of times VCC+5 warning will be sent 
V12=2                # Number of times VCC+12 warning will be sent 

if [ -f /etc/sm_counter ]; then
  grep -v ^\# /etc/sm_counter | grep counters > /tmp/.sm_aux
  while read LOC TMP V33 V5 V12 < /tmp/.sm_aux
  do
    if [ -z "$LOC" ]; then
      break
    fi
    if [ "$LOC" = "counters" ]; then
      break
    fi
  done
fi

# Check Temperature
VALUE=`/bin/sm s r 01`
if [ $TMP != 0 ] && [ $VALUE -gt $MAX_TEMP ]; then
  [ $TMP -gt 0 ] && TMP=`expr $TMP - 1`

  #Print a Warning message.
  echo "WARNING! Temperature inside server's case above the limit. It is currently at [C]: $VALUE" > /dev/console

  #Send a Warning e-mail.
  #sendmail -t t@none.com -f a@none.com -s "SM100 temperature" -m "Temperature inside the server's case above the limite. It is currently at [C]: $VALUE" -h 10.0.0.2
fi

# Check VCC +3.3V 
VALUE=`/bin/sm s r 30`
if [ $V33 != 0 ] && ( [ $VALUE -gt $H_VCC33 ] || [ $VALUE -lt $L_VCC33 ] ); then
  [ $V33 -gt 0 ] && V33=`expr $V33 - 1`
  K=0  
  [ $VALUE -ge 12 ] && K=12
  VALUE=`expr $VALUE + $K`
  VALUE=`expr $VALUE \* 157`
  INT=`expr $VALUE / 10000`
  DEC=`expr $VALUE % 10000`
  #Print a Warning message.
  echo "WARNING! VCC +3.3V below/above limit. It is currently at [V]: $INT.$DEC" > /dev/console

  #Send a Warning e-mail.
  #sendmail -t t@none.com -f a@none.com -s "SM100 VCC +3.3V" -m "VCC +3.3V below/above limit. It is currently at [V]: $INT.$DEC" -h 10.0.0.2
fi

# Check VCC +5V 
VALUE=`/bin/sm s r 2e`
if [ $V5 != 0 ] && ( [ $VALUE -gt $H_VCC5 ] || [ $VALUE -lt $L_VCC5 ] ); then
  [ $V5 -gt 0 ] && V5=`expr $V5 - 1`
  K=0  
  [ $VALUE -ge 12 ] && K=12
  VALUE=`expr $VALUE + $K`
  VALUE=`expr $VALUE \* 238`
  INT=`expr $VALUE / 10000`
  DEC=`expr $VALUE % 10000`
  #Print a Warning message.
  echo "WARNING! VCC +5V below/above limit. It is currently at [V]: $INT.$DEC" > /dev/console

  #Send a Warning e-mail.
  #sendmail -t t@none.com -f a@none.com -s "SM100 VCC +5V" -m "VCC +5V below/above limit. It is currently at [V]: $INT.$DEC" -h 10.0.0.2
fi

# Check VCC +12V 
VALUE=`/bin/sm s r 2f`
if [ $V12 != 0 ] && ( [ $VALUE -gt $H_VCC12 ] || [ $VALUE -lt $L_VCC12 ] ); then
  [ $V12 -gt 0 ] && V12=`expr $V12 - 1`
  K=0  
  [ $VALUE -ge 12 ] && K=12
  VALUE=`expr $VALUE + $K`
  VALUE=`expr $VALUE \* 571`
  INT=`expr $VALUE / 10000`
  DEC=`expr $VALUE % 10000`
  #Print a Warning message.
  echo "WARNING! VCC +12V below/above limit. It is currently at [V]: $INT.$DEC" > /dev/console

  #Send a Warning e-mail.
  #sendmail -t t@none.com -f a@none.com -s "SM100 VCC +12V" -m "VCC +12V below/above limit. It is currently at [V]: $INT.$DEC" -h 10.0.0.2
fi

echo "#Server Management - Working counters" > /etc/sm_counter
echo "#        TEMP V+33 V+5 V+12" >> /etc/sm_counter
echo "counters $TMP $V33 $V5 $V12" >> /etc/sm_counter
