
// the max size of syslog message 
// message that exceeds this size will be truncated 
#define MAX_SYSLOG_MSG	1024

#define MAX_EVENT_NUMBER 33

typedef enum {
	TYPE_NUMBER,
	TYPE_STRING
} VARIABLE_TYPE;

// link : internal event number (sent by applications) and
// its correspondent TRAP number and Syslog message
unsigned char event_internal[] = {
	// reboot.c (busybox-100)
	1,	// reboot	
	// busybox-100 and OpenSSH
	2,	// log in to the appliance
	// busybox-100 and OpenSSH
	3,	// log out of the appliance
	// busybox-100 and OpenSSH
	4,	// authentication fail - appliance session
	// OpenSSH and Ras
	5,	// serial port session started
	// OpenSSH and RAS
	6,	// serial port session stopped
	// OpenSSH and RAS
	7,	// authenticaiton fail - serial port session
	// cy_buffering
	8,	// port alert
	// cy_buffering
	9,	// serial port : DCD On 
	// cy_buffering
	10,	// serial port : DCD Off
	// daemon.sh
	11,	// daemon started
	// daemon.sh
	12,	// daemon stopped
	// WebApi
	13,	// admin session terminated by another
	// saveconf
	14,	// configuration is saved
	// signal_ras
	15,	// configuration is activated
	// busybox-100
	16,	// adduser
	// busybox-100
	17,	// deluser
	// logpipe
	18,	// low threshold
	// logpipe
	19,	// medium threshold
	// logpipe
	20,	// discard data
	// telnetd and OpenSSH
	21,	// data lost
	// pmd
	22,	// IPDU is ON 
	// pmd
	23,	// IPDU is OFF
	// pmd
	24,	// outlet was turned ON
	// pmd
	25,	// outlet was turned OFF
	// pmd
	26,	// command cycle
	// pmd
	27,	// exceeded max threshold
	// pmd
	28,	// beneath min threshold
	// telnetd and OpenSSH
	29,	// alert string in the appliance
	// 
	30,	// firmware upgrade started
	//
	31,	// done the firmare upgrade 
	// pmfwupgrade
	32,	// PM firmware upgrade started
	// pmfwupgrade
	33,	// done the PM firmware upgrade
	-1, // EVT 34
	-1, // EVT 35
	-1, // EVT 36
	-1, // EVT 37
	-1, // EVT 38
	-1, // EVT 39
	-1, // EVT 40
	34, // Cycle completed EVT 41
	35, // Session stopped Port In use EVT 42
};

#define MAX_ARGUMENTS	6
struct EVENT_SNMPTrapOID {
	int variables_len;
	int variables[MAX_ARGUMENTS];
	VARIABLE_TYPE variable_type[MAX_ARGUMENTS];
} event_snmptrap[] = {
// EVT-01 : Appliance is in process of rebooting.
 { 1, { cyAcsTrapObjectUserName }, { TYPE_STRING } },

// EVT-02 :  User logged into the appliance.
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectSessionType },
 	{ TYPE_STRING, TYPE_NUMBER }  },

// EVT-03:  User logged out of the appliance.
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectSessionType },
	{ TYPE_STRING, TYPE_NUMBER } },

// EVT-04: A user failed to authenticate with the appliance.
 { 1, { cyAcsTrapObjectUserName }, { TYPE_STRING } },

// EVT-05: A serial session has started on particular port.
 { 4, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPortName,
	cyAcsTrapObjectSerialPort, cyAcsTrapObjectSerialSessionType },
	{ TYPE_STRING, TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-06: A serial session has stopped.
 { 4, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPortName,
	cyAcsTrapObjectSerialPort, cyAcsTrapObjectSerialSessionType },
	{ TYPE_STRING, TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-07: serial session authentication fails.
 { 4, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPortName,
	cyAcsTrapObjectSerialPort, cyAcsTrapObjectSerialSessionType },
	{ TYPE_STRING, TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-08: The appliance detected a port alert string on a serial port.
 { 5, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectAlertIndex, cyAcsTrapObjectAlertString, cyAcsTrapObjectAlertMessage},
	{ TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_STRING, TYPE_STRING } },

// EVT-09: The appliance has detected that the power on/off control signal 
// (RS232C - DCD signal) is high on a serial port indicating that the  server 
// attached to the port is powered on.
 { 2, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort },
	{ TYPE_STRING, TYPE_NUMBER} },

// EVT-10: The appliance has detected that the power on/off control signal 
// (RS232C - DCD signal) is low on a serial port indicating that the  server 
// attached to the port is powered off.
 { 2, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort },
	 { TYPE_STRING, TYPE_NUMBER} },

// EVT-11: Daemon in the appliance started
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectDaemonName},
	 { TYPE_STRING, TYPE_STRING} },

// EVT-12: Daemon in the appliance stopped
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectDaemonName},
	 { TYPE_STRING, TYPE_STRING} },

// EVT-13: This event is generated when one previous admin session is terminated 
// by another admin user.
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectTargetUserName },
	 { TYPE_STRING, TYPE_STRING} },

// EVT-14: This event is generated when 'saveconf' command is successful.
 { 1, { cyAcsTrapObjectUserName}, { TYPE_STRING } },

// EVT-15: This event is generated when the configuration is activated.
 { 1, { cyAcsTrapObjectUserName}, { TYPE_STRING } },

// EVT-16: A new user has been added to the local user database.
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectTargetUserName },
	 { TYPE_STRING, TYPE_STRING} },

// EVT-17: A user has been deleted from the local user database.
 { 2, { cyAcsTrapObjectUserName, cyAcsTrapObjectTargetUserName },
	 { TYPE_STRING, TYPE_STRING} },

// EVT-18: This event is generated when the appliance is managed by DsView 
// and  global data buffer has reached configured Low level Threshold.
 { 1, { cyAcsTrapObjectDataBufferSize }, { TYPE_NUMBER } },

// EVT-19: This event is generated when the appliance is managed by DsView 
// and  global data buffer has reached configured Med level Threshold.
 { 1, { cyAcsTrapObjectDataBufferSize }, { TYPE_NUMBER } },

// EVT-20: This event is generated when the appliance is managed by DsView 
// and global data buffer has exceeded beyond the size of buffer and data 
// lost has occurred.
 { 0, {0} },

// EVT-21: This event is generated when the appliance is managed by DsView 
// and Appliance data lost occurred on ssh or telnet session.
 { 2, { cyAcsTrapObjectProcessID, cyAcsTrapObjectUserName},
	 { TYPE_NUMBER, TYPE_STRING } },

// EVT-22: A chain of power device attached to the appliance is detected to be online.
 { 4, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMCount, cyAcsTrapObjectPMSocketCount },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_NUMBER } },

//EVT-23: A chain of power device attached to the appliance is detected to be offline.
 { 2, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort },
	 { TYPE_STRING, TYPE_NUMBER} },

// EVT-24: An outlet on a chain of power device has been turned on.
 { 3, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMSocketNumber },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-25: An outlet on a chain of power device has been turned off.
 { 3, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMSocketNumber },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-26: cycle finished
 { 3, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMSocketNumber },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-27: A chain of power device attached to the appliance has exceeded 
// the maximum  threshold for total load.
 { 5, { cyAcsTrapObjectSerialPortName,  cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMNumber, cyAcsTrapObjectPMPhase,
        cyAcsTrapObjectPMCurrentValue },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_STRING, TYPE_STRING } },

// EVT-28: A chain of power device attached to the appliance has gone 
// beneath the minimum threshold for total load.
 { 5, { cyAcsTrapObjectSerialPortName,  cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMNumber, cyAcsTrapObjectPMPhase,
	cyAcsTrapObjectPMCurrentValue },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_STRING, TYPE_STRING} },

// EVT-29: An alert string was detected on a ssh or telnet session into the appliance.
 { 4, { cyAcsTrapObjectUserName, cyAcsTrapObjectAlertIndex, cyAcsTrapObjectAlertString,
	cyAcsTrapObjectAlertMessage },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_STRING, TYPE_STRING } },

// EVT-30: The appliance has started an upgrade of a software image by DSView.
 { 3, { cyAcsTrapObjectUserName, cyAcsTrapFirmwareVersion,
        cyAcsTrapNewFirmwareVersion },
	 { TYPE_STRING, TYPE_STRING, TYPE_STRING } },

// EVT-31: This trap is generated when Firmware upgraded is done to indicate the result.
 { 2, { cyAcsTrapFirmwareUpgradeResult, cyAcsTrapNewFirmwareVersion },
	 { TYPE_NUMBER, TYPE_STRING } },

// EVT-32: The appliance has started an upgrade of the PM software.
 { 6, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPortName,
	cyAcsTrapObjectSerialPort, cyAcsTrapObjectPMNumber,
	cyAcsTrapFirmwareVersion, cyAcsTrapNewFirmwareVersion },
	 { TYPE_STRING, TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_STRING, TYPE_STRING } },

// EVT-33: This trap is generated when Firmware upgraded is done to indicate the result.
 { 5, { cyAcsTrapObjectSerialPortName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMNumber, cyAcsTrapFirmwareUpgradeResult,
	cyAcsTrapNewFirmwareVersion },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER, TYPE_NUMBER, TYPE_STRING } },

// EVT-41: Cycle started
 { 3, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPort,
	cyAcsTrapObjectPMSocketNumber },
	 { TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

// EVT-42: A serial session has stopped. Port in use.
 { 4, { cyAcsTrapObjectUserName, cyAcsTrapObjectSerialPortName,
	cyAcsTrapObjectSerialPort, cyAcsTrapObjectSerialSessionType },
	{ TYPE_STRING, TYPE_STRING, TYPE_NUMBER, TYPE_NUMBER } },

};

struct EVENT_SYSLOG {
	unsigned char 	*summary;
	unsigned char 	arguments[MAX_ARGUMENTS];
} event_syslog[] = {
// EVT-01 : Appliance is in process of rebooting.
 {"The appliance is rebooting. Command issued by user: %s." , {0} },

// EVT-02 :  User logged into the appliance.
 {"A user logged into the appliance. User: %s. Session type: %d.", {0,1} },

// EVT-03:  User logged out of the appliance.
 {"A user logged out of the appliance. User: %s. Session type: %d.", {0,1} },

// EVT-04: A user failed to authenticate with the appliance.
 {"User authentication failed. User: %s.", {0} },

// EVT-05: A serial session has started on particular port.
 {"Serial session started. User: %s. Port name: %s. Port Number: %d.  Session Type: %d.", {0,1,2,3} },

// EVT-06: A serial session has stopped.
 {"Serial session stopped. User: %s. Port name: %s. Port Number: %d. Session type: %d.", {0,1,2,3} },

// EVT-07: serial session authentication fails.
 {"Serial session authentication failed. User: %s. Port name: %s. Port Number: %d. Session type: %d.", 
	{0,1,2,3} },

// EVT-08: The appliance detected a port alert string on a serial port.
 {"A port alert was detected on a serial port. Port name: %s. Port number: %d. Port alert index: %d. Port alert string: %s. Line text: %s.",
    	{0,1,2,3,4} },

// EVT-09: The appliance has detected that the power on/off control signal 
// (RS232C - DCD signal) is high on a serial port indicating that the  server 
// attached to the port is powered on.
 {"Power on detected on a serial port. Port name: %s. Port number: %d.", {0,1} },

// EVT-10: The appliance has detected that the power on/off control signal 
// (RS232C - DCD signal) is low on a serial port indicating that the  server 
// attached to the port is powered off.
 {"Power off detected on a serial port. Port name: %s. Port number: %d.", {0,1} },

// EVT-11: Daemon in the appliance started
 {"A daemon started. User: %s. Daemon name: %s.", {0,1} },

// EVT-12: Daemon in the appliance stopped
 {"A daemon stopped. User: %s. Daemon name: %s.", {0,1} },

// EVT-13: This event is generated when one previous admin session is terminated 
// by another admin user.
 {"Admin session terminated. Command issued by user: %s. Terminated user: %s.", {0,1} },

// EVT-14: This event is generated when 'saveconf' command is successful.
 {"Configuration saved to flash. User: %s.", {0} },

// EVT-15: This event is generated when the configuration is activated.
 {"Configuration activated. User: %s.", {0} },

// EVT-16: A new user has been added to the local user database.
 {"A user was added to the local user database. Command issued by user: %s. New user: %s.", 
	{0,1} },

// EVT-17: A user has been deleted from the local user database.
 {"A user was deleted to the local user database. Command issued by user: %s. Deleted user: %s.",
	{0,1} },

// EVT-18: This event is generated when the appliance is managed by DsView 
// and  global data buffer has reached configured Low level Threshold.
 {"Data Buffering for DSView data log is getting full (low threshold). Buffer size: %d.", {0} },

// EVT-19: This event is generated when the appliance is managed by DsView 
// and  global data buffer has reached configured Med level Threshold.
 {"Data Buffering for DSView data log is almost full (medium threshold). Buffer size: %d.", {0} },

// EVT-20: This event is generated when the appliance is managed by DsView 
// and global data buffer has exceeded beyond the size of buffer and data 
// lost has occurred.
 {"Data Buffering for DSView data log had data loss because buffer full.", {0} },

// EVT-21: This event is generated when the appliance is managed by DsView 
// and Appliance data lost occurred on ssh or telnet session.
 {"Appliance data log loss. Process ID: %d. User: %s.", {0,1} },

// EVT-22: A chain of power device attached to the appliance is detected to be online.
 {"A power device is online. Port name: %s. Port number: %d. Power Devices: %d. Number of sockets: %d.",
	{0,1,2,3} },

//EVT-23: A chain of power device attached to the appliance is detected to be offline.
  {"A power device is offline. Port name: %s. Port number: %d.", {0,1} },

// EVT-24: An outlet on a chain of power device has been turned on.
  {"The on command was issued to a power device socket.  Command issued by user: %s. Power device port number: %d.  Socket number: %d.",
	{0,1,2} },

// EVT-25: An outlet on a chain of power device has been turned off.
  {"The off command was issued to a power device socket. Command issued by user: %s. Power device port number: %d. Socket number: %d.",
	{0,1,2} },

// EVT-26: An outlet on a chain of power device has been issued the cycle command.
  {"The cycle command has completed to a power device socket. Command issued by user: %s. Power device port number: %d.Socket number: %d.",
	{0,1,2} },

// EVT-27: A chain of power device attached to the appliance has exceeded 
// the maximum  threshold for total load.
  {"Power device exceeded the total load maximum threshold. Power device port name: %s. Power device port number: %d. Power device number: %d. Phase: %s. Current detected: %s amps",
	{0,1,2,3,4} },

// EVT-28: A chain of power device attached to the appliance has gone 
// beneath the minimum threshold for total load.
  {"Power device is back to below the alarm threshold. Power device port name: %s. Power device port number: %d. Power device number: %d. Phase: %s. Current detected: %s amps",
	{0,1,2,3,4} },

// EVT-29: An alert string was detected on a ssh or telnet session into the appliance.
  {"An appliance alert was detected. User: %s. Alert index: %d. Alert string: %s. Line text: %s.",
	{0,1,2,3} },

// EVT-30: The appliance has started an upgrade of a software image by DSView.
  {"Image upgrade started by DSView. User: %s. Current version: %s. New version: %s.",
	{0,1,2} },

// EVT-31: This trap is generated when Firmware upgraded is done to indicate the result.
  {"Image upgrade result. Result: %d. New firmware version: %s.", {0,1} },

// EVT-32: The appliance has started an upgrade of the PM software.
  {"PM image upgrade started. User: %s. Power device port name: %s. Power device port number: %d. Power device number: %d. Current version: %s. New version: %s.",
	{0,1,2,3,4,5} },

// EVT-33: This trap is generated when Firmware upgraded is done to indicate the result.
  {"PM image upgrade result. Power device port name: %s. Power device port number: %d. Power device number: %d. Result: %d. New Firmware version: %s.",
	{0,1,2,3,4} },

// EVT-41: Cycle started
  {"The cycle command has started to a power device socket. Command issued by user: %s. Power device port number: %d.Socket number: %d.",
	{0,1,2} },

// EVT-42: A serial session has stopped. Port in use
 {"Serial session stopped. Port In use. User: %s. Port name: %s. Port Number: %d. Session type: %d.", {0,1,2,3} },

};
