
/* MPC8xx CPM SPI interface. 
 * Copyright (c) 2000 Embedded Planet <mlocke@jps.net>.
 * in progress, can't get microcode patch to work
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/poll.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/wait.h>
#include <asm/uaccess.h>
#include <asm/pgtable.h>
#include <asm/mpc8xx.h>

#include "commproc.h"

/* This is the SPI device number, but I use the minors to indicate
 * the SPI device address.
 */
#define CPM_SPI_CDEV_MAJOR 89

void	cpm_spi_init(void);
extern ssize_t cpm_spi_read(struct file *, char *, size_t, loff_t *);
extern ssize_t cpm_spi_write(struct file *, char *, size_t, loff_t *);

static struct file_operations cpm_spi_fops = {
	NULL,		/* lseek */
	NULL,		/* read */
	NULL,		/* write */
	NULL,		/* readdir */
	NULL,		/* select */
	NULL,		/* ioctl */
	NULL,		/* mmap */
	NULL, /*cpm_spi_open,*/
	NULL, /*cpm_spi_close,*/
	NULL		/* fsync */
};

/* Only one user at a time.
*/
static int	cpm_spi_flags;

static	dev_t	spi_device;
static	struct wait_queue *spi_wait;

static	ushort r_tbase, r_rbase;

void
cpm_spi_init()
{
	uint	mem_addr, dp_addr, reloc;
	volatile cbd_t		*bdp;
	volatile cpm8xx_t	*cp;
	volatile spi_t		*spp;
	volatile immap_t	*immap;

	cp = cpmp;	/* Get pointer to Communication Processor */
	immap = (immap_t *)IMAP_ADDR;	/* and to internal registers */

printk("SPI init\n");
	spp = (spi_t *)&cp->cp_dparam[PROFF_SPI];

	/* Check for and use a microcode relocation patch.
	*/
	if (reloc = spp->spi_rpbase)
		spp = (spi_t *)&cp->cp_dpmem[spp->spi_rpbase];
		

	/* Initialize Port B SPI pins.
	*/
	cp->cp_pbpar |= 0x0000000E;
	cp->cp_pbdir |= 0x0000000E;

	/* Initialize the parameter ram.
	 * We need to make sure many things are initialized to zero,
	 * especially in the case of a microcode patch.
	 */
	spp->spi_rstate = 0;
	spp->spi_rdp = 0;
	spp->spi_rbptr = 0;
	spp->spi_rbc = 0;
	spp->spi_rxtmp = 0;
	spp->spi_tstate = 0;
	spp->spi_tdp = 0;
	spp->spi_tbptr = 0;
	spp->spi_tbc = 0;
	spp->spi_txtmp = 0;


	/* Allocate space for one transmit and one receive buffer
	 * descriptor in the DP ram.
	 */
	dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * 2);

	/* Allocate space for receive buffer in the host memory.
	 * 
	 */

	/* Set up the SPI parameters in the parameter ram.
	*/
	spp->spi_tbase = r_tbase = dp_addr;
	spp->spi_rbase = r_rbase = dp_addr + sizeof(cbd_t);

	/*
	 *  manually init if using microcode patch
	 */
	if(reloc)
	{
		printk("SPI microcode patch is running\n");
		spp->spi_rbptr = spp->spi_rbase;
		spp->spi_tbptr = spp->spi_tbase;
	}

	/* Set the buffer address.
	*/
	mem_addr = m8xx_cpm_hostalloc(64);
	bdp = (cbd_t *)&cp->cp_dpmem[spp->spi_rbase];
	bdp->cbd_bufaddr = __pa(mem_addr);

	mem_addr = m8xx_cpm_hostalloc(64);
	bdp = (cbd_t *)&cp->cp_dpmem[spp->spi_tbase];
	bdp->cbd_bufaddr = __pa(mem_addr);

	spp->spi_tfcr = SMC_EB;
	spp->spi_rfcr = SMC_EB;

	/* Set maximum receive size.
	*/
	spp->spi_mrblr = 16;

	/* Initialize Tx/Rx parameters.
	*/
	if (reloc == 0) {
		cp->cp_cpcr =
			mk_cr_cmd(CPM_CR_CH_SPI, CPM_CR_INIT_TRX) | CPM_CR_FLG;
		while (cp->cp_cpcr & CPM_CR_FLG);
	}
	else {
		cp->cp_cpcr =
			mk_cr_cmd(CPM_CR_CH_SPI, CPM_CR_INIT_TX) | CPM_CR_FLG;
		while (cp->cp_cpcr & CPM_CR_FLG);

		cp->cp_cpcr =
			mk_cr_cmd(CPM_CR_CH_SPI, CPM_CR_INIT_RX) | CPM_CR_FLG;
		while (cp->cp_cpcr & CPM_CR_FLG);
	}

	/* Clear event
	*/
	cp->cp_spie = ~0;

	/* No events 
	*/
	cp->cp_spim = 0;



/*
 *  NOTE:  for a real spi driver these need to be set for 
 *	   each peripheral. These are the Touch Panel settings
 */

   /*
     * Initialize SPMODE
     *
     * Settings:
     *     Reverse Data      -- Normal Operation (TX/RX MSB first)
     *     Master/Slave Mode -- Master Mode
     *     Character Length  -- 8 Bits
     */

    cp->cp_spmode = SPMODE_REV | SPMODE_MSTR;

    //
    // Calculate the SPI clock divisor to provide the desired frequency
    // to the A to D converter
    //

    cp->cp_spmode |= SPMODE_DIV16 | ((8-1) << 4);
    cp->cp_spmode &= ~SPMODE_PMMSK;
printk("SPI done\n");
}

/* Read from SPI.
 * This is a two step process.  First, we send the "dummy" write
 * to set the device offset for the read.  Second, we perform
 * the read operation.
 */
static	u_char	iitemp[32];

ssize_t
cpm_spi_read(struct file *file, char *buf, size_t count, loff_t *ppos)
{
	volatile spi_t *spp;
	volatile cbd_t	*tbdf, *rbdf;
	volatile cpm8xx_t	*cp;
	volatile uint		*hcsr;
	unsigned long flags;
	uint	reloc;

	cp = cpmp;	/* Get pointer to Communication Processor */
	spp = (spi_t *)&cp->cp_dparam[PROFF_SPI];
	hcsr = (volatile uint *)HIOX_CSR0_ADDR;

// HIOX - turn SPI on
	*hcsr |= HIOX_CSR0_TSELSPI;

	/* Check for and use a microcode relocation patch.
	*/
	if (reloc = spp->spi_rpbase) {
		spp = (spi_t *)&cp->cp_dpmem[spp->spi_rpbase];
	}


	tbdf = (cbd_t *)&cp->cp_dpmem[spp->spi_tbase];
	rbdf = (cbd_t *)&cp->cp_dpmem[spp->spi_rbase];

	memset(__va(tbdf->cbd_bufaddr),0,count);
	tbdf->cbd_datlen = count;
	tbdf->cbd_sc |= BD_SC_READY;

	rbdf->cbd_sc = BD_SC_WRAP;
	rbdf->cbd_sc |= BD_SC_EMPTY;

	/* Chip bug, set enable here.
	*/
	save_flags(flags); cli();

	cp->cp_spcom = 0x80;	/* Start Transmit */

	/* Wait for SPI transfer.
	*/
	while(rbdf->cbd_sc & BD_SC_EMPTY);
	//interruptible_sleep_on(&spi_wait);
	restore_flags(flags);
	if (signal_pending(current))
		return -ERESTARTSYS;

	if (rbdf->cbd_sc & BD_SC_EMPTY)
		printk("SPI read complete but rbuf empty\n");
	
	memcpy(buf,__va(tbdf->cbd_bufaddr), count);
//	if (copy_to_user(buf, __va(rbdf->cbd_bufaddr), count))
//		return -EFAULT;
	
	*ppos += count;

// HIOX - turn SPI off
	*hcsr &= ~HIOX_CSR0_TSELSPI;

	return count;
}

////////////////////////////////////////////////////////////////////
ssize_t
cpm_spi_write(struct file *file, char *buf, size_t count, loff_t *ppos)
{
	volatile spi_t *spp;
	volatile cbd_t	*tbdf, *rbdf;
	volatile cpm8xx_t	*cp;
	volatile uint		*hcsr;
	u_char			*tb;
	loff_t	offset;
	unsigned long flags;
	uint	reloc;

	cp = cpmp;	/* Get pointer to Communication Processor */
	spp = (spi_t *)&cp->cp_dparam[PROFF_SPI];
	hcsr = (volatile uint *)HIOX_CSR0_ADDR;

// HIOX - turn SPI on
	*hcsr |= HIOX_CSR0_TSELSPI;

	/* Check for and use a microcode relocation patch.
	*/
	if (reloc = spp->spi_rpbase) {
		spp = (spi_t *)&cp->cp_dpmem[spp->spi_rpbase];
	}


	tbdf = (cbd_t *)&cp->cp_dpmem[spp->spi_tbase];

//	if (copy_from_user(__va(tbdf->cbd_bufaddr), buf, count))
//		return -EFAULT;

	memcpy(__va(tbdf->cbd_bufaddr), buf, count);

	tbdf->cbd_datlen = count;		/* Length */

	tbdf->cbd_sc = BD_SC_WRAP|BD_SC_LAST; 
	tbdf->cbd_sc |= BD_SC_READY; 

	save_flags(flags); cli();
	cp->cp_spcom = 0x80;

	/* Wait for SPI transfer.
	*/
	while(tbdf->cbd_sc & BD_SC_READY);

//	interruptible_sleep_on(&spi_wait);

	restore_flags(flags);
	if (signal_pending(current))
		return -ERESTARTSYS;

printk("transfer complete\n");

	if (tbdf->cbd_sc & BD_SC_READY)
		printk("SPI ra complete but tbuf ready\n");

// HIOX - turn SPI off
	*hcsr &= ~HIOX_CSR0_TSELSPI;

  return count;
}
