/*
 * arch/ppc/kernel/multi_pci.c
 * 
 * Support for multiple indirectly addressed PCI host bridges
 *
 * Author: Matt Porter
 *         mporter@mvista.com
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/pci.h>
#include <asm/io.h>
#include <asm/system.h>
#include <asm/multi_pci.h>

#include "pci.h"

decl_config_access_method(indirect);

pci_interface pci_interfaces[MAX_PCI_INTERFACES];

int multi_find_pci_interface(unsigned char bus)
{
	int i;

	/* Attempt to find the correct interface for a given bus */
	for (i=0; i<MAX_PCI_INTERFACES; i++)
		if ( (pci_interfaces[i].top_bus <= bus) &&
				(pci_interfaces[i].sub_bus >= bus) )
			return i;
	return 0;
}

void multi_set_pci_interface(unsigned char bus)
{
	int index;

	/* Set a default pci interface */
	pci_config_address = pci_interfaces[0].pci_cfga;
	pci_config_data = pci_interfaces[0].pci_cfgd;

	/* Try to find the real pci interface and set it */
	index = multi_find_pci_interface(bus);
	pci_config_address = pci_interfaces[index].pci_cfga;
	pci_config_data = pci_interfaces[index].pci_cfgd;
}

int multi_pcibios_read_config_byte(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned char *val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_read_config_byte(bus, dev_fn, offset, val);
}

int multi_pcibios_read_config_word(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned short *val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_read_config_word(bus, dev_fn, offset, val);

}

int multi_pcibios_read_config_dword(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned int *val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_read_config_dword(bus, dev_fn, offset, val);
}

int multi_pcibios_write_config_byte(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned char val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_write_config_byte(bus, dev_fn, offset, val);
}

int multi_pcibios_write_config_word(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned short val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_write_config_word(bus, dev_fn, offset, val);
}

int multi_pcibios_write_config_dword(unsigned char bus, unsigned char dev_fn,
			     unsigned char offset, unsigned int val)
{
	multi_set_pci_interface(bus);
	return indirect_pcibios_write_config_dword(bus, dev_fn, offset, val);

}
