/*
 * $Id: pcore_pci.c,v 1.1.1.1 2000/12/06 23:51:27 root Exp $
 * Originally by Gary Thomas
 * rewritten and updated by Cort Dougan (cort@cs.nmt.edu)
 * rewritten for Force PowerCore by Christian Zankel (chris@mvista.com)
 */

#include <linux/types.h>
#include <linux/pci.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/version.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/ptrace.h>
#include <asm/prom.h>
#include <asm/pci-bridge.h>
#include <asm/residual.h>
#include <asm/processor.h>
#include <asm/irq.h>
#include <asm/machdep.h>
#include <asm/pcore.h>


#include "../kernel/pci.h"


extern void w83c553_init(struct pci_dev*);

decl_config_access_method(indirect);


/* address and data port to access the config space */

extern unsigned int *pci_config_address;
extern unsigned char *pci_config_data;

/* Which PCI interrupt line does a given device [slot] use? */
/* Note: This really should be two dimensional based in slot/pin used */
unsigned char *Motherboard_map;
unsigned char *Motherboard_map_name;

/* How is the 82378 PIRQ mapping setup? */
unsigned char *Motherboard_routes;

/* Tables for known hardware */   

struct bridge_data **bridges;

/* Bases (TOP) for remaping */
static unsigned int io_base = PCORE_PCI_IO_TOP_ADDR;
static unsigned int mem_base = PCORE_PCI_MEM_TOP_ADDR;


/*
 * Force Computer PowerCore IRQ-maps
 */


#if defined(CONFIG_PCORE680)
/* Force Computers PowerCore PCORE680 IRQ-map */

/* The first table contains an entry number 
   for the second table the based on the device number.
   The second table describes the INTx interrupt wireing */

static char PowerCoreCPCI680_pci_IRQ_map[] =
{
        0,      /* Slot 0  - Hostrbridge MPC107 */
        0,      /* Slot 1  - unused */
        0,      /* Slot 2  - unused */
        0,      /* Slot 3  - unused */
        0,      /* Slot 4  - unused */
        0,      /* Slot 5  - unused */
        0,      /* Slot 6  - unused */
        0,      /* Slot 7  - unused */
        0,      /* Slot 8  - unused */
        0,      /* Slot 9  - unused */
        0,      /* Slot 10 - unused */
        0,      /* Slot 11 - unused */
        0,      /* Slot 12 - unused */
        0,      /* Slot 13 - unused */
        0,      /* Slot 14 - unused */
        0,      /* Slot 15 - unused */
        0,      /* Slot 16 - unused */
        0,      /* Slot 17 - unused */
        0,      /* Slot 18 - unused */
        0,      /* Slot 19 - unused */
        0,      /* Slot 20 - unused */
        0,      /* Slot 21 - unused */
        0,      /* Slot 22 - unused */
        0,      /* Slot 23 - unused */
        4*2+0,  /* Slot 24 - PCI-CPCI bridge (Sentinel) */
        4*1+1,  /* Slot 25 - EEPRO-100 Ethernet controller 1 */
        4*3+0,  /* Slot 26 - PMC I */
        4*4+0,  /* Slot 27 - PMC II */
        4*1+0,  /* Slot 28 - EEPRO-100 Ethernet controller 2 */
        0,      /* Slot 29 - unused */
        0,      /* Slot 30 - PCI-ISA bridge and IDE */
        0       /* Slot 31 - unused */
};
static char PowerCoreCPCI680_pci_IRQ_routes[] =
{
      0,  0,  0,  0,    /* unused                      */
      9, 10,  0,  0,    /* PCI:    INTA,INTB,INTC,INTD */
      9, 10, 11, 12,    /* CPCI0:  INTA,INTB,INTC,INTD */
     11, 12,  9, 10,    /* PMCI:   INTC,INTD,INTA,INTB */ 
     12,  9, 10, 11     /* PMCII:  INTD,INTA,INTB,INTC */
};
#endif /* CONFIG_PCORE680 */



#ifdef CONFIG_PCORE6750
/* Force Computers PowerCore PCORE IRQ-map */

/* The first table contains an entry number 
   for the second table the based on the device number.
   The second table describes the INTx interrupt wireing */

static char PowerCoreCPCI6750_pci_IRQ_map[] =
{
    0,       /* Slot 0  - unused */
        0,       /* Slot 1  - unused */
        0,       /* Slot 2  - unused */
        0,       /* Slot 3  - unused */
        0,       /* Slot 4  - unused */
        0,       /* Slot 5  - unused */
        0,       /* Slot 6  - unused */
        0,       /* Slot 7  - unused */
        0,       /* Slot 8  - unused */
        0,       /* Slot 9  - unused */
        0,       /* Slot 10 - unused */
        0,       /* Slot 11 - unused */
        0,       /* Slot 12 - unused */
        0,       /* Slot 13 - unused */
        0,       /* Slot 14 - unused */
        0,       /* Slot 15 - unused */
        0,       /* Slot 16 - unused */
        0,       /* Slot 17 - unused */
        0,       /* Slot 18 - unused */
        0,       /* Slot 19 - unused */
        0,       /* Slot 20 - unused */
        0,       /* Slot 21 - unused */
        0,       /* Slot 22 - unused */
        0,       /* Slot 23 - unused */
        1*4+0,   /* Slot 24 - PCI-PCI */
        1*4+1,   /* Slot 25 - DEC21143 */
        2*4,     /* Slot 26 - PMC I */
        3*4,     /* Slot 27 - PMC II */
        0,       /* Slot 28 - unused */
        0,       /* Slot 29 - unused */
        0,       /* Slot 30 - PCI-ISA */
        0        /* Slot 31 - unused */
};
static char PowerCoreCPCI6750_pci_IRQ_routes[] =
{
      0,  0,  0,  0,    /* unused */
      9, 10, 11, 15,    /* PCI0:  INTA,INTB,INTC,INTD */
     11, 15,  9, 10,    /* PMCI:  INTC,INTD,INTA,INTB */ 
     12, 11, 15,  9     /* PMCII: INTB,INTC,INTD,INTA */
     /* PMCII mapping is suspect.  The manual says
        it whould be 10,11,15,9, but all my single 
	function INTA devs show up on INT12.  I could
	not ntest INTB,INTC,INTD so 11,15,9 maybe
	correct, it may be wrong too.  */
};
#endif /* CONFIG_PCORE6750 */


#ifdef CONFIG_PCOREVME
/* Force Computers PowerCore 6750 VME IRQ-map */
static char PowerCore6750VME_pci_IRQ_map[] =
{
        0,   /* Slot 0  - unused */
        0,   /* Slot 1  - unused */
        0,   /* Slot 2  - unused */
        0,   /* Slot 3  - unused */
        0,   /* Slot 4  - unused */
        0,   /* Slot 5  - unused */
        0,   /* Slot 6  - unused */
        0,   /* Slot 7  - unused */
        0,   /* Slot 8  - unused */
        0,   /* Slot 9  - unused */
        0,   /* Slot 10 - unused */
        0,   /* Slot 11 - unused */
        0,   /* Slot 12 - unused */
        0,   /* Slot 13 - unused */
        0,   /* Slot 14 - unused */
        0,   /* Slot 15 - unused */
        0,   /* Slot 16 - unused */
        0,   /* Slot 17 - unused */
        0,   /* Slot 18 - unused */
        0,   /* Slot 19 - unused */
        0,   /* Slot 20 - unused */
        0,   /* Slot 21 - unused */
        0,   /* Slot 22 - unused */
        0,   /* Slot 23 - unused */
        1*4+1,   /* Slot 24  - UNIVERSE II */
        0,   /* Slot 25 - unused */
        1*4,   /* Slot 26 - PCI-ISA */
        1*4+0,   /* Slot 27 - DEC21143 */
        2*4+0,   /* Slot 28 - PMC I */
        3*4+0,   /* Slot 29 - PMC II */
        0,   /* Slot 30 - unused */
        0    /* Slot 31 - unused */
};
static char PowerCore6750VME_pci_IRQ_routes[] =
{
      0,  0,  0,  0,    /* unused */
     10, 11, 14,  5,    /* PCI0:  INTA,INTB,INTC,INTD */
     14,  5, 10, 11,    /* PMCI:  INTC,INTD,INTA,INTB */ 
     11, 14,  5, 10     /* PMCII: INTB,INTC,INTD,INTA */
}; 
#endif /* CONFIG_PCOREVME */


/* Known devices */

struct pcore_pcidevice {
    unsigned short vendor, device;
    unsigned char irq;
    unsigned short cmd;
    unsigned char cache_line, latency;
    void (*init)(struct pci_dev *dev);
};

static struct pcore_pcidevice pcore_map[] = {
  { 0x1011, 0x0046, 9, 0, 32, 0, NULL },  /* PPC2PCI Bridge */
};

#define PCORE_MAP_NUM ((sizeof( pcore_map )/sizeof( pcore_map[0] )))


/*  Setup the known on-board devices */

static void __init mapin_device( struct pci_dev *dev )
{
    struct pcore_pcidevice *p;
    unsigned short cmd;
    int i;
  

    for( i=0; i < PCORE_MAP_NUM; i++ ) {
        p = &pcore_map[i];
    
        if ( p->vendor == dev->vendor && 
             p->device == dev->device ) {
     
            if (p->irq != 0xff) {
                pci_write_config_byte( dev, PCI_INTERRUPT_LINE, p->irq );
                dev->irq = p->irq;
            }

            if (p->cmd) {
                pci_read_config_word( dev, PCI_COMMAND, &cmd );
                pci_write_config_word( dev, PCI_COMMAND, (p->cmd|cmd));
            }

            if (p->cache_line)
                pci_write_config_byte( dev, PCI_CACHE_LINE_SIZE, p->cache_line );
            
            if (p->latency)
                pci_write_config_byte( dev, PCI_LATENCY_TIMER, p->latency );
        }
    }
}


/* remap the PCI bus */
   
__init void layout_dev( struct pci_dev *dev )
{
    struct pci_bus *bus;
    unsigned short cmd;
    unsigned int reg, base, size, type;
    int i;
    
    bus = dev->bus;
    
    /* ???? */
    /* MPIC already done */
    if (dev->vendor == 0x10ad && dev->device == 0x0105)
        return;
    
    /*
     * Do not remap buses beyong bus 0 since we're not doing it right anyway.
     * The "bios" on this board will, hopefully, assign the resources properly.
     * The bios seems to assume that all downstream pci-pci bridges support
     * over 64KB IO space.  Thus, if you have a pci-pci bridge which only supports
     * the lower 64KB of IO space, the bridge will not be configured properly and none
     * of the devices beyond the bridge will work.
     */
    if (bus->number != 0) {
        return;
    }

    /* make any known settings happen */
    mapin_device( dev );

    indirect_pcibios_read_config_word( bus->number, dev->devfn, PCI_COMMAND, &cmd );

    for( reg = PCI_BASE_ADDRESS_0, i=0; reg <= PCI_BASE_ADDRESS_5; reg += 4, i++ ) {
    
        indirect_pcibios_write_config_dword( bus->number, dev->devfn, reg, 0xffffffff );
        indirect_pcibios_read_config_dword( bus->number, dev->devfn, reg, &base );
        if (!base) {
#if (LINUX_VERSION_CODE >= 0x020300)
            /*            dev->resource[i].start = 0;
                        dev->resource[i].end = 0; */
#else
            dev->base_address[i] = 0;
#endif
            continue;
        }
        
        if (base & PCI_BASE_ADDRESS_SPACE_IO) {
            cmd |= PCI_COMMAND_IO;

            size = base & PCI_BASE_ADDRESS_IO_MASK;
            io_base = (io_base & size) + size;
            indirect_pcibios_write_config_dword( bus->number, dev->devfn, reg, 
                                 ((io_base) & 0x00ffffff) | 0x1);

#if (LINUX_VERSION_CODE >= 0x020300)
            dev->resource[i].end = io_base+ ~size;
            printk("remapping IO (0x%0x:0x%0x BAR %d): 0x%x -> 0x%x:0x%x\n",
                   dev->vendor, dev->device, i, dev->resource[i].start,io_base,dev->resource[i].end);
            dev->resource[i].start = io_base; // | 0x1;
#else
            dev->base_address[i] = io_base | 0x1;
            printk("remapping IO (0x%0x:0x%0x BAR %d): 0x%x -> 0x%x\n",
                   dev->vendor, dev->device, i, (unsigned int)dev->base_address[i],io_base);
#endif
        } else {
            cmd |= PCI_COMMAND_MEMORY;
            type = size & PCI_BASE_ADDRESS_MEM_TYPE_MASK;
            size = base & PCI_BASE_ADDRESS_MEM_MASK;
            switch( type ) {

            case PCI_BASE_ADDRESS_MEM_TYPE_32:
                break;
            case PCI_BASE_ADDRESS_MEM_TYPE_64:
                printk("Warning: Ignoring 64-bit device; slot %d, function %d.\n",
                       PCI_SLOT( dev->devfn ), PCI_FUNC( dev->devfn ));
                reg += 4;
                continue;
            }
            mem_base = (mem_base & size) + size;
            indirect_pcibios_write_config_dword( bus->number, dev->devfn,
                                 reg, (mem_base));

#if (LINUX_VERSION_CODE >= 0x020300)
            dev->resource[i].end = mem_base + ~size;
            printk("remapping MEM(0x%x:0x%x BAR %d): %x -> 0x%x:0x%x\n",
                   dev->vendor, dev->device, i, dev->resource[i].start,mem_base,dev->resource[i].end); 
            dev->resource[i].start = mem_base;
#else
            dev->base_address[i] = mem_base;
            printk("remapping MEM(0x%x:0x%x BAR %d): %x -> 0x%x\n",
                   dev->vendor, dev->device, i, (unsigned int)dev->base_address[i],mem_base); 
#endif
        }
    }

    if ((dev->class >> 8) == PCI_CLASS_DISPLAY_VGA)
        cmd |= PCI_COMMAND_IO;

    indirect_pcibios_write_config_word( bus->number, dev->devfn, PCI_COMMAND,
                        (cmd|PCI_COMMAND_MASTER));
}

__init void pcore_layout_bus( struct pci_bus *bus )
{
    struct pci_dev *dev;
    if (!bus->devices && !bus->children)
        return;

    for( dev = bus->devices; dev; dev = dev->sibling ) 
    {
        if (((dev->class >> 16) != PCI_BASE_CLASS_BRIDGE) ||
            ((dev->class >> 8) == PCI_CLASS_BRIDGE_OTHER))
            {
                layout_dev( dev );
            }
    }
}


__init void pcore_pcibios_fixup(void)
{
    struct pci_bus *bus;
    struct pci_bus *sysbus;
    struct pci_dev *dev;

#if defined(CONFIG_PCORE680)
    Motherboard_routes = PowerCoreCPCI680_pci_IRQ_routes;
    Motherboard_map = PowerCoreCPCI680_pci_IRQ_map;
#endif
#if defined(CONFIG_PCORE6750)
    Motherboard_routes = PowerCoreCPCI6750_pci_IRQ_routes;
    Motherboard_map = PowerCoreCPCI6750_pci_IRQ_map;
#endif

    /* re-layout the bus */


    for (bus= &pci_root; bus; bus = bus->children)
    {
        pcore_layout_bus(bus);
    }

    /* re-route the interrupt for the devices */

    for(dev=pci_devices; dev; dev=dev->next)
    {
      if ( dev->bus->number == 0 ) 
      {
        /* the root bus we know via the documented mapping */
        dev->irq = Motherboard_routes[Motherboard_map[PCI_SLOT(dev->devfn)]];
        pcibios_write_config_byte( dev->bus->number, dev->devfn, PCI_INTERRUPT_LINE, dev->irq );
      }
      else  /* fix up for sub-buses */
      {
        unsigned char biosirq;

        pcibios_read_config_byte( dev->bus->number, dev->devfn, PCI_INTERRUPT_LINE, &biosirq );

        if (biosirq > 3) {
            /*
             * Something has changed in the bios since we put this hack in.
             *
             * The bios assigns interrupt lines 0 through 3, where 0 is mapped
             * to INTA and 3 is INTD.
             */
            printk("device vendor 0x%x at bus %d, unable to remap irq line\n",
                    dev->vendor, dev->bus->number);
        }
        else {
            dev->irq += 9;
            pcibios_write_config_byte( dev->bus->number, dev->devfn, PCI_INTERRUPT_LINE, dev->irq );
        }
      }
    }

    /* initialize WINBOND 83C553 Device */

    if (!(dev = pci_find_device( PCI_VENDOR_ID_WINBOND,PCI_DEVICE_ID_WINBOND_83C553,0)))
        printk("cannot find W83C553!!\n");
    else
        w83c553_init(dev);

}


__init void pcore_setup_pci_ptrs(void)
{
    /* we use indirect pci access */
    set_config_access_method(indirect);
    pci_config_address= (unsigned *) 0xfec00000;
    pci_config_data= (unsigned char *) 0xfee00000;

    /* we need to re-layout the PCI bus */
    ppc_md.pcibios_fixup = pcore_pcibios_fixup;
}


