/*
 *  linux/arch/ppc/force/setup.c
 *
 *  Copyright (C) 1995  Linus Torvalds
 *  Adapted from 'alpha' version by Gary Thomas
 *  Modified by Cort Dougan (cort@cs.nmt.edu)
 *  Modified by Christian Zankel (zankel@mailserv.rz.fh-muenchen.de)
 */

/*
 * bootup setup stuff..
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/malloc.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/major.h>
#include <linux/interrupt.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <linux/blk.h>
#include <linux/ioport.h>
#include <linux/console.h>
#include <linux/timex.h>
#include <linux/pci.h>
#include <linux/openpic.h>

#include <asm/mmu.h>
#include <asm/processor.h>
#include <asm/residual.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/cache.h>
#include <asm/dma.h>
#include <asm/machdep.h>
#include <asm/mk48t59.h>
#include <asm/pcore.h>
#include "time.h"
#include "local_irq.h"
#include "open_pic.h"
#include "i8259.h"


extern unsigned long mk48t59_get_rtc_time(void);
extern int mk48t59_set_rtc_time(unsigned long nowtime);
extern void pcore_setup_pci_ptrs(void);


void pcore_setup_pci_ptrs(void);

unsigned long empty_zero_page[1024];

extern PTE *Hash, *Hash_end;
extern unsigned long Hash_size, Hash_mask;
extern int probingmem;
extern unsigned long loops_per_sec;


/* FIXME: extern unsigned char aux_device_present; */

extern char saved_command_line[256];

#ifdef CONFIG_BLK_DEV_RAM
extern int rd_doload;		/* 1 = load ramdisk, 0 = don't load */
extern int rd_prompt;		/* 1 = prompt for ramdisk, 0 = don't prompt */
extern int rd_image_start;	/* starting block # of image */
#endif


/* NVRAM */

unsigned char pcore_nvram_rd (int addr)
{
	outb((char)(addr & 0xff), PCORE_NVRAM_ADDR_LO);
	outb((char)(addr >> 8), PCORE_NVRAM_ADDR_HI);
	return inb(PCORE_NVRAM_DATA);
}

void pcore_nvram_wr (int addr, unsigned char val)
{
	outb((char)(addr & 0xff), PCORE_NVRAM_ADDR_LO);
	outb((char)(addr >> 8), PCORE_NVRAM_ADDR_HI);
	outb(val,PCORE_NVRAM_DATA);
}


/* FROM PReP */
/* We use the NVRAM RTC to time a second to calibrate the decrementer. */
__initfunc(void pcore_mk48t59_calibrate_decr(void))
{
        unsigned long freq, divisor;
        unsigned long t1, t2;
        unsigned char save_control;
        long i;
        unsigned char sec;
	
	
        /* Make sure the time is not stopped. */
        save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLB);
	
        ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			       (save_control & (~MK48T59_RTC_CB_STOP)));
	
        /* Now make sure the read bit is off so the value will change. */
        save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
        save_control &= ~MK48T59_RTC_CA_READ;
        ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);
	
	
        /* Read the seconds value to see when it changes. */
        sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
        for (i = 0 ; i < 1000000 ; i++) { /* may take up to 1 second... */
		if (ppc_md.nvram_read_val(MK48T59_RTC_SECONDS) != sec) {
			break;
		}
        }
        t1 = get_dec();
	
        sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
        for (i = 0 ; i < 1000000 ; i++) { /* Should take up 1 second... */
		if (ppc_md.nvram_read_val(MK48T59_RTC_SECONDS) != sec) {
			break;
		}
        }
	
        t2 = t1 - get_dec();
	
        freq = t2 * 60; /* try to make freq/1e6 an integer */
        divisor = 60;
        printk("time_init: decrementer frequency = %lu/%lu (%luMHz)\n",
               freq, divisor,t2>>20);
        decrementer_count = freq / HZ / divisor;
        count_period_num = divisor;
        count_period_den = freq / 1000000;
}

__init void w83c553_init(struct pci_dev* dev)
{
        unsigned char irq_mode1 = 0, irq_mode2 = 0;

        pci_write_config_byte(dev, 0x43,0xff);   /* reroute interupts of IDE0 and IDE1 to 15 */
        pci_write_config_word(dev, 0x44,0x9abc); /* route INTA(9), INTB(10), INTC(11), INTD(12) */

	/* bit on = level, bit off = edge */
        irq_mode1 = 0x00; /*            - irq07=0x80, irq00=0x01 */
        irq_mode2 = 0x1e; /* 12,11,10,9 - irq15=0x80, irq08=0x01 */
        outb( irq_mode1 , 0x4d0 );
        outb( irq_mode2 , 0x4d1 );
}

static inline int get_msr()
{
        int msr;
        asm volatile("mfmsr %0" : "=r" (msr):);
        return msr;
}


int
pcore_get_cpuinfo(char *buf)
{
    int len;

#if defined(CONFIG_PCORE680G3)
    len = sprintf( buf, "machine\t\t: Force Computers PowerCore 680/G3\n" );
#elif defined(CONFIG_PCORE680G4)
    len = sprintf( buf, "machine\t\t: Force Computers PowerCore 680/G4\n" );
#elif defined(CONFIG_PCORE6750)
    len = sprintf( buf, "machine\t\t: Force Computers PowerCore 6750\n" );
#else 
    len = sprintf( buf, "machine\t\t: Force Computers PowerCore <unknown>\n" );
#endif


    len += sprintf( buf+len, "PVR\t\t: 0x%08x, ", _get_PVR() );
    len += sprintf( buf+len, "cpu=" );
    switch ( ( _get_PVR() & 0xffff0000 ) >> 16 )
    {
      case 0x0008: { len += sprintf( buf+len, "750/G3"  ); break; }
      case 0x000C: { len += sprintf( buf+len, "7400/G4" ); break; }
      default:     { len += sprintf( buf+len, "unknown" ); break; }
    }
    len += sprintf( buf+len, ", rev=%d.%d\n", 
      ( ( _get_PVR() & 0x0000ff00 ) >> 8 ),
        ( _get_PVR() & 0x000000ff ) );


    len += sprintf( buf+len,"HID0\t\t: 0x%08x", (unsigned)_get_HID0() );
    len += sprintf( buf+len,", ICE=" );
    if ( ( _get_HID0() & HID0_ICE ) == HID0_ICE )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, ", DCE=" );
    if ( ( _get_HID0() & HID0_DCE ) == HID0_DCE )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, ", SGE=" );
    if ( ( _get_HID0() & HID0_SGE ) == HID0_SGE )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, ", BTIC=" );
    if ( ( _get_HID0() & HID0_BTIC ) == HID0_BTIC )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, ", BHTE=" );
    if ( ( _get_HID0() & HID0_BHTE ) == HID0_BHTE )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, ", BTCD=" );
    if ( ( _get_HID0() & HID0_BTCD ) == HID0_BTCD )
    {
      len += sprintf( buf+len, "1" );
    }
    else
    {
      len += sprintf( buf+len, "0" );
    }
    len += sprintf( buf+len, "\n" );


#if defined(CONFIG_PCORE680)
    len += sprintf( buf+len,"DCCRA\t\t: 0x%08x", inb(0x308) );
    sprintf( buf+len, ", L2=" );
    switch ( ( inb(0x308) & 0xc0 ) >> 6 )
    {
      case 0:  { len += sprintf( buf+len, ", 2MB"      ); break; }
      case 1:  { len += sprintf( buf+len, ", reserved" ); break; }
      case 2:  { len += sprintf( buf+len, ", 1MB"      ); break; }
      case 3:  { len += sprintf( buf+len, ", 512KB"    ); break; }
      default: { len += sprintf( buf+len, ", err"      ); break; }
    }
    sprintf( buf+len, ", SDRAM=" );
    switch ( ( inb(0x308) & 0x0f ) )
    {
      case 0:  { len += sprintf( buf+len, ", reserved" ); break; }
      case 1:  { len += sprintf( buf+len, ", 64M"      ); break; }
      case 2:  { len += sprintf( buf+len, ", 128MB"    ); break; }
      case 3:  { len += sprintf( buf+len, ", 256MB"    ); break; }
      default: { len += sprintf( buf+len, ", err"      ); break; }
    }
#endif
#if defined(CONFIG_PCORE6750)
    len += sprintf( buf+len,"DCCRA\t\t: 0x%08x", inb(0x308) );
    sprintf( buf+len, ", L2=" );
    switch ( ( inb(0x308) & 0xc0 ) >> 6 )
    {
      case 0:  { len += sprintf( buf+len, ", nocache"  ); break; }
      case 1:  { len += sprintf( buf+len, ", 256KB"    ); break; }
      case 2:  { len += sprintf( buf+len, ", 1024KB"   ); break; }
      case 3:  { len += sprintf( buf+len, ", 512KB"    ); break; }
      default: { len += sprintf( buf+len, ", err"      ); break; }
    }
    if ( ( inb(0x308) & 0x10 ) )
    {
      sprintf( buf+len, ", EDO=" );
      switch ( ( inb(0x308) & 0x0f ) )
      {
        case 0x7: { len += sprintf( buf+len, ", 16MB"          ); break; }
        case 0xd: { len += sprintf( buf+len, ", 64MB"          ); break; }
        case 0x9: { len += sprintf( buf+len, ", 196MB(64+128)" ); break; }
        default:  { len += sprintf( buf+len, ", err"           ); break; }
      }
    }
    else
    {
      sprintf( buf+len, ", SDRAM=" );
      switch ( ( inb(0x308) & 0x0f ) )
      {
        case 0x3:  { len += sprintf( buf+len, ", 64MB"           ); break; }
        case 0x1:  { len += sprintf( buf+len, ", 128MB"          ); break; }
        case 0x2:  { len += sprintf( buf+len, ", 196MB(64+128)"  ); break; }
        case 0x0:  { len += sprintf( buf+len, ", 256MB(128+128)" ); break; }
        default: { len += sprintf( buf+len, ", err"              ); break; }
      }
    }
#endif
    len += sprintf( buf+len,"\n" );


    len += sprintf( buf+len,"L2CR\t\t: 0x%08x", (unsigned)_get_L2CR() );
    if ( _get_L2CR() & 0x80000000 )
    {
      len += sprintf( buf+len, ", enabled" );
    }
    else
    {
      len += sprintf( buf+len, ", disabled" );
    }
    if ( _get_L2CR() & 0x40000000 )
    {
      len += sprintf( buf+len, ", parity" );
    }
    else
    {
      len += sprintf( buf+len, ", noparity" );
    }
    switch (  ( _get_L2CR() & 0x30000000 ) >> 28 )
    {
      case 0:  { len += sprintf( buf+len, ", 2MB"   ); break; }
      case 1:  { len += sprintf( buf+len, ", 256KB" ); break; }
      case 2:  { len += sprintf( buf+len, ", 512KB" ); break; }
      case 3:  { len += sprintf( buf+len, ", 1MB"   ); break; }
      default: { len += sprintf( buf+len, ", err"   ); break; }
    }
    switch (  ( _get_L2CR() & 0x0e000000 ) >> 25 )
    {
      case 0:  { len += sprintf( buf+len, ", noclock"  ); break; }
      case 1:  { len += sprintf( buf+len, ", 1x"       ); break; }
      case 2:  { len += sprintf( buf+len, ", 1.5x"     ); break; }
      case 3:  { len += sprintf( buf+len, ", 3.5x"     ); break; }
      case 4:  { len += sprintf( buf+len, ", 2.0x"     ); break; }
      case 5:  { len += sprintf( buf+len, ", 2.5x"     ); break; }
      case 6:  { len += sprintf( buf+len, ", 3.0x"     ); break; }
      case 7:  { len += sprintf( buf+len, ", 4.0x"     ); break; }
      default: { len += sprintf( buf+len, ", err"      ); break; }
    }
    switch ( ( _get_L2CR() & 0x01800000 ) >> 23 )
    {
      case 0:  { len += sprintf( buf+len, ", f/t sync burst"      ); break; }
      case 1:  { len += sprintf( buf+len, ", reserved"            ); break; }
      case 2:  { len += sprintf( buf+len, ", p/l sync burst"      ); break; }
      case 3:  { len += sprintf( buf+len, ", p/l sync late-write" ); break; }
      default: { len += sprintf( buf+len, ", err"                 ); break; }
    }
    len += sprintf( buf+len, "\n" );


    len += sprintf( buf+len,"MSR\t\t: 0x%08x", (unsigned)get_msr() );
    len += sprintf( buf+len, "\n" );

    /* FIXME:provide more information about the board */

    return len;
}

void __init
pcore_setup_arch(unsigned long * memory_start_p, unsigned long * memory_end_p)
{
	extern char cmd_line[];

	/* init to some ~sane value until calibrate_delay() runs */
	loops_per_sec = 50000000;

	printk("Boot arguments: %s\n", cmd_line);
}

void
pcore_restart(char *cmd)
{
	int i=1;

	/* soft reset of the CPU */
	_disable_interrupts();
	
	/* set exception prefix high - to the prom */
	_nmask_and_or_msr(0, MSR_IP);
       	*(char*)0xfe000332 = (char) 0x22;
	
	while ( i != 0 ) i++;
	panic("soft restart failed\n");
       	*(char*)0xfe000332 = (char) 0x11;
	
}

void
pcore_power_off(void)
{
	pcore_restart(NULL);
}

void
pcore_halt(void)
{
	while(1)
		;
}


u_int
pcore_irq_cannonicalize(u_int irq)
{
	if (irq == 2) {
		return 9;
	} else {
		return irq;
	}
}


void
pcore_do_IRQ(struct pt_regs *regs, int cpu, int isfake)
{
	int irq;

	if ( (irq = i8259_irq(0)) < 0 )
	{
		printk(KERN_DEBUG "Bogus interrupt from PC = %lx\n",
		       regs->nip);
		ppc_spurious_interrupts++;
		return;
	}
        ppc_irq_dispatch_handler( regs, irq );
}

#if 1
__init void pcore_init_IRQ(void)
{
	int i;
	/* just in case there's a second proc.. */
	ppc_cached_irq_mask[0] = ppc_cached_irq_mask[1] = ~0UL;
	//	i8259_pic.irq_offset = 0;
        for ( i = 0 ; i < 16  ; i++ )
                irq_desc[i].ctl = &i8259_pic;

	i8259_init();
	/*
	 * According to the Carolina spec from ibm irqs 0,1,2, and 8
	 * must be edge triggered.  Also, the pci intrs must be level
	 * triggered and _only_ isa intrs can be level sensitive
	 * which are 3-7,9-12,14-15. 13 is special - it can be level.
	 *
	 * power on default is 0's in both regs - all edge.
	 *
	 * These edge/level control regs allow edge/level status
	 * to be decided on a irq basis instead of on a PIC basis.
	 * It's still pretty ugly.
	 * - Cort
	 */
}
#endif


void __init
pcore_init(unsigned long r3, unsigned long r4, unsigned long r5,
	       unsigned long r6, unsigned long r7) 
{
	isa_io_base = PCORE_ISA_IO_BASE;
	isa_mem_base = PCORE_ISA_MEM_BASE;
	pci_dram_offset = PCORE_PCI_DRAM_OFFSET;
	ISA_DMA_THRESHOLD = 0x00ffffff; /* 16MB ISA space */
/* ??? */
	DMA_MODE_READ = 0x44;
	DMA_MODE_WRITE = 0x48;
	pcore_setup_pci_ptrs();


#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if ( r4 ) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */

	/* take care of cmd line */
        if ( r6 )
        {
                *(char *)(r7+KERNELBASE) = 0;
                strcpy(cmd_line, (char *)(r6+KERNELBASE));
        }

	ppc_md.setup_arch     = pcore_setup_arch;
	ppc_md.setup_residual = NULL;
	ppc_md.get_cpuinfo    = pcore_get_cpuinfo;

	ppc_md.irq_cannonicalize = pcore_irq_cannonicalize;
	ppc_md.init_IRQ       = pcore_init_IRQ;
	ppc_md.do_IRQ         = pcore_do_IRQ;

	ppc_md.init           = NULL;
	ppc_md.restart        = pcore_restart;
	ppc_md.power_off      = pcore_power_off;
	ppc_md.halt           = pcore_halt;

	ppc_md.nvram_read_val = pcore_nvram_rd;
	ppc_md.nvram_write_val = pcore_nvram_wr;
	
	ppc_md.time_init      = NULL;
	ppc_md.set_rtc_time   = mk48t59_set_rtc_time;
	ppc_md.get_rtc_time   = mk48t59_get_rtc_time;
	ppc_md.calibrate_decr = pcore_mk48t59_calibrate_decr;

	/* no keyboard */
	ppc_md.kbd_setkeycode    = NULL;
	ppc_md.kbd_getkeycode    = NULL;
        ppc_md.kbd_translate     = NULL;
	ppc_md.kbd_unexpected_up = NULL;
	ppc_md.kbd_leds          = NULL;
	ppc_md.kbd_init_hw       = NULL;
#ifdef CONFIG_MAGIC_SYSRQ
        ppc_md.ppc_kbd_sysrq_xlate = NULL;
#endif
}

