/*
 * arch/ppc/boot/console.c
 *
 *  Copyright (C) 1998 Gabriel Paubert, paubert@iram.es
 *
 *  This file is based on arch/ppc/boot/{kbd,vreset}.c in previous versions 
 * of Linux/PPC, mostly contributed by Gary Thomas and Cort Dougan.
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of this archive
 *  for more details.
 */

#include <linux/types.h>
#include <linux/string.h>
#include <asm/byteorder.h>
#include <linux/kernel.h>
#include <linux/config.h>
#include <asm/page.h>
#include <asm/processor.h>
#include <asm/mmu.h>
#include <asm/system.h>
#include <asm/io.h>
#include <stdarg.h>


/* 8250 polling mode */

/*
 * COM1 NS16550 support
 */

#define RBR 0
#define IER 1
#define FCR 2
#define LCR 3
#define MCR 4
#define LSR 5
#define MSR 6
#define SCR 7
#define THR RBR
#define IIR FCR
#define DLL RBR
#define DLM IER

#define LSR_DR   0x01  /* Data ready */
#define LSR_OE   0x02  /* Overrun */
#define LSR_PE   0x04  /* Parity error */
#define LSR_FE   0x08  /* Framing error */
#define LSR_BI   0x10  /* Break */
#define LSR_THRE 0x20  /* Xmit holding register empty */
#define LSR_TEMT 0x40  /* Xmitter empty */
#define LSR_ERR  0x80  /* Error */

#define COM1	0x3F8
#define COM2	0x2F8

#ifdef CONFIG_KGDB
int kgdb_debugport;

void putDebugChar(u_char c)
{
	while ((inb(kgdb_debugport+LSR) & LSR_THRE) == 0) 
		;
	outb(c, kgdb_debugport+THR);
}
 
int getDebugChar(void)
{
	while ((inb(kgdb_debugport+LSR) & LSR_DR) == 0) ;
	return (inb(kgdb_debugport+RBR));
}

void kgdb_interruptible(int enable)
{

}
#endif /* CONFIG_KGDB */


/* Serial Polling-Mode */

int sercon_port;

int sercon_getc(void)
{
	if ((inb(sercon_port+LSR) & LSR_DR) == 0)
		return -1;
	return (int)(inb(sercon_port+RBR));
}

int sercon_getc_w(void)
{
	while ((inb(sercon_port+LSR) & LSR_DR) == 0)
		;
	return (int)(inb(sercon_port+RBR));
}

void sercon_putc(u_char c)
{
	while ((inb(sercon_port+LSR) & LSR_THRE) == 0) 
		;
	outb(c, sercon_port+THR);
}
 
int sercon_tstc(void)
{
	return ((inb(sercon_port+LSR) & LSR_DR) != 0);
}

void sercon_puts(const u_char *s)
{
        char c;

        while ( ( c = *s++ ) != '\0' ) {
                sercon_putc(c);
                if ( c == '\n' )
                        sercon_putc('\r');
        }
}

int sercon_init(void) 
{
	sercon_port = COM1;
	return 0;
}

/* endof 8250 support file */

/* generic sercon */



/* Based on linux/lib/vsprintf.c and modified to suit our needs,
 * bloat has been limited since we basically only need %u, %x, %s and %c.
 * But we need 64 bit values !
 */

int printf(const char *fmt, ...) {
	va_list args;
	int i;
	/* Should not be a problem with 8kB of stack */
	char buf[256];
	
	va_start(args, fmt);
	i = vsprintf(buf, fmt, args);
	va_end(args);
	sercon_puts(buf);
	return  i;
}
