/*
 * arch/ppc/pcoreboot/head.S
 *
 * Copyright (C) 2000 Christian Zankel (chris@mvista.com)
 *
 * This code should be loaded to a 1MB aligned address above 2MB.
 * It can also be started out of ROM.
 */

#include "../kernel/ppc_asm.tmpl"
#include <asm/processor.h>
#include <asm/page.h>
#include <linux/config.h>
#include <asm/mmu.h>

/* LED codes: LED<U1><U2> U1/U2: 0=off, G=green, R=red H=hw */
#define LED00	0x0
#define LED0G	0x1
#define LED0R	0x2
#define LED0H	0x3
#define LEDG0	0x4
#define LEDGG	0x5
#define LEDGR	0x6
#define LEDGH	0x7
#define LEDR0	0x8
#define LEDRG	0x9
#define LEDRR	0xa
#define LEDRH	0xb
#define LEDH0	0xc
#define LEDHG	0xd
#define LEDHR	0xe
#define LEDHH	0xf

#define SETLEDS(x) \
	lis	r10,0xfe00; \
	ori	r10,r10,0x0300; \
	li	r11,x; \
	stb	r11,0(r10);

#define STACKSIZE	4096*10
#define MEM_SIZE	0x80000

.extern cmd_line
.extern cmd_line_e
			
.extern	start
		.text
	.globl	_stext
_stext:

	.text
	.globl	_start
_start:

/* We need two memory areas, one for the boot code, and one to where we copy the kernel. */
/* The 'booter' runs at BOOTBASE, the kernel is copied to KERNELBASE. */

	SETLEDS(LEDHH)		/* set user-led 1 and 2 to h/w */
	
/* Turn off MMU */

	li	r3,0
	mtmsr	r3
	isync

/* delete BAT registers that we don't need */
	li	r3,0
	mtdbatl	2,r3
	mtdbatu	2,r3
	mtdbatl	3,r3
	mtdbatu	3,r3
	mtibatl	2,r3
	mtibatu	2,r3
	mtibatl	3,r3
	mtibatu	3,r3

/*
 * Setup MMU:	size	virt		phys
 * BAT0:	1MB	80000000	<load address>			(boot loader)
 * BAT1:	1MB	81000000	<load address> + 200000		(boot data/bss)
 * BAT2:	4MB	C0000000	0				(kernel space)
 * BAT3:	2MB	FE000000	FE000000			(io-space)
 */

	bl	1f
1:      mflr	r8
	subis	r8,r8,(1b-_start)@ha
	subi	r8,r8,(1b-_start)@l		/* r8:	this is where _start is physically right now, should be 1 MB aligned */

	srwi	r10,r8,31			/* r9:	 1:  running in ROM, 0: running in RAM */

	/* BAT0 I/D:	.text phys: r8, virt: BOOTBASE */
	ori	r3,r8,2			/* r8: physical address (= BPRN), R/W */
	lis	r4,BOOTBASE@h		/* virtual address (= BEPI) */
	ori	r4,r4,0x1e		/* 1MB, supervisor */

	mtibatl	0,r3
	mtibatu 0,r4
	mtdbatl	0,r3
	mtdbatu 0,r4

	/* BAT1 D:	.data/.bss phys: r8 + 200000 (aligned) / 0x400000 (ROM), virt BOOTDATA */

	addis	r7,r8,0x002fffff@ha
	addi	r7,r7,0x002fffff@l
	andis.	r7,r7,0xfff0
	
	/* running in ROM? */
	cmpwi	r10,0
	beq	3f
	/* running in ROM: physical address is 0x00400000 (instead of 'current address') */
	lis	r7,0x0040
3:
	/* copy data segment */

	/* r7: physical address of the data segment */
	/* r3: get the physical address of where the datas are located right now */
	lis	r3,__data_reloc@h
	ori	r3,r3,__data_reloc@l
	subis	r3,r3,_start@ha
	subi	r3,r3,_start@l
	add	r3,r8,r3
	
	subi	r3,r3,4
	subi	r4,r7,4

	lis	r6,__data_end@h
	ori	r6,r6,__data_end@l
	subis	r5,r6,__data_start@ha
	subi	r5,r5,__data_start@l
	srwi	r5,r5,2
	mtctr	r5
5:	lwzu	r5,4(r3)
	stwu	r5,4(r4)
	bdnz	5b

	ori	r3,r7,0x2
	lis	r4,BOOTDATA@h
	ori	r4,r4,0x1e		/* 1MB, supervisor */
	
	mtdbatl	1,r3
	mtdbatu 1,r4

	/* kernel */
	li	r3,2			/* physical address = 0, R/W */
	lis	r4,KERNELBASE@h		/* virtual address */
	ori	r4,r4,0x7e		/* 4MB, supervisor */

	mtdbatl	2,r3
	mtdbatu 2,r4

	/* IO */
	lis	r3,0xfe00
	ori	r4,r3,0x0e
	ori	r3,r3,0x32
	mtdbatl	3,r3
	mtdbatu	3,r4
	
	li	r3,0x30			/* IR, DR (MMU on ) */
	mtspr	SRR1,r3
	
	/* address */
	lis	r3,BOOTBASE@h
	addi	r3,r3,(4f-_start)
/* r3 should be the same as 4f!!) */
	mtspr	SRR0,r3
	rfi
4:

	/* clear all of BSS */
	lis	r3,__bss_start-4@h
	ori	r3,r3,__bss_start-4@l
	lis	r4,__bss_end-4@h
	ori	r4,r4,__bss_end-4@l
	sub	r4,r4,r3
	srwi	r4,r4,2
	mtctr	r4
	li	r0,0
7:	stwu	r0,4(r3)
	bdnz	7b
		
	/* stack is at r1 */
	lis	r1,.stack+STACKSIZE-4@h
	ori	r1,r1,.stack+STACKSIZE-4@l

	mr	r3,r10			/* romboot flag */
	mr	r4,r8			/* base */
	bl	uncompress

	lis	r6,cmd_line@h
	ori	r6,r6,cmd_line@l
	lis	r7,cmd_line_e@h
	ori	r7,r7,cmd_line_e@l

	lwz	r6,0(r6)
	lwz	r7,0(r7)
	addis	r6,r6,-KERNELBASE@h
	addis	r7,r7,-KERNELBASE@h
	li	r3,_MACH_pcore@l
	
	li	r9,0
	mtspr	SRR0,r9
	mtspr	SRR1,r9
	rfi
	
.comm	.stack, STACKSIZE,4
.comm	_memory, MEM_SIZE, 4
