#include <asm/io.h>
#include <asm/page.h>
#include <asm/processor.h>
#include <asm/mmu.h>
#include <asm/mk48t59.h>
#include <asm/processor.h>
#include <asm/pcore.h>
#include <linux/config.h>

extern void _image;
extern void _image_end;
extern void _memory;

extern int sercon_init(void);
extern int sercon_tstc(void);
extern void sercon_putc(char);
extern void sercon_puts(char*);
extern int sercon_getc(void);
extern int sercon_getc_w(void);
extern void strcpy(char*,char*);
extern int strlen(char*);
extern int printf(char*,...);
extern void gunzip(void *dst, int dstlen, unsigned char *src, int *len);

unsigned long isa_io_base=0xfe000000;

char	*avail_ram;
char	*end_avail;
char	*cmd_line = (char *)0xc03f0000;
char    *cmd_line_e;

unsigned char pcore_nvram_rd (int addr)
{
	outb((char)(addr & 0xff), PCORE_NVRAM_ADDR_LO);
	outb((char)(addr >> 8), PCORE_NVRAM_ADDR_HI);
	return inb(PCORE_NVRAM_DATA);
}

void pcore_nvram_wr (int addr, unsigned char val)
{
	outb((char)(addr & 0xff), PCORE_NVRAM_ADDR_LO);
	outb((char)(addr >> 8), PCORE_NVRAM_ADDR_HI);
	outb(val,PCORE_NVRAM_DATA);
}


#if 0
#define I2C_ADDR  ((volatile char*)0xfe000301)
#define I2C_S()	  ( *I2C_ADDR = (data | 0x40), *I2C_ADDR = (0xc0 | data) )
#define I2C_R()	  ( *I2C_ADDR = data, *I2C_ADDR = (0x80 | data), (char)((*I2C_ADDR) >> 6)&1 )
#define I2C_W(x)  ( *I2C_ADDR = (data | ((x)<<6)), *I2C_ADDR = (0x80 | data | ((x)<<6)))
#define I2C_E()	  ( *I2C_ADDR = data )
#define I2C_RD	  0x2
#define I2c_WR    0x1

int read_i2c(int addr)
{
	int i;
	int val, ret;
	int data;

	data = *I2C_ADDR & 0x3f;
	val = (I2C_RD << 6) | (addr & 0x3f);
	
	/* start */
	I2C_S();

	/* command and address */
	for (i=(1<<7); i>0 ; i>>=1)
		I2C_W(val&i?1:0);


	printf("addr should be low now %02x\n",*I2C_ADDR);

	/* read 16 bit */
	for (i=0, ret=0; i<16; i++,ret<<=1)
		ret |= I2C_R();

	/* stop */
	I2C_E();

	return ret>>1;

}
#endif 

void uncompress(int romstart, int base)
{
	unsigned char save_control;
	char sec_s,sec_e;
	int ch;
	char *cp;
	int len;

	/* this will be our command line passed to the kernel */
        #ifdef CONFIG_CMDLINE
	  strcpy( cmd_line, CONFIG_CMDLINE );
        #else
          *cmd_line = '\0';
        #endif


	/* let's show something */
	sercon_init();
	printf("\n\n\nUncompressing kernel...");
	
	/* uncompress kernel */
	avail_ram = (char *)PAGE_ALIGN((unsigned long)&_memory);
	end_avail = &_memory+0x80000;
	len = (unsigned long)&_image_end- (unsigned long)&_image;
	gunzip ((void*)0xc0000000, 0x3f0000, &_image, &len);
	
	printf("done.\n");

	/* flush buffer */
	sercon_getc();sercon_getc();sercon_getc();
	sercon_getc();sercon_getc();sercon_getc();
	
	printf("Initializing clock. \n");

	/* Make sure the time is not stopped. */
	save_control = pcore_nvram_rd(MK48T59_RTC_CONTROLB);
       	pcore_nvram_wr(MK48T59_RTC_CONTROLB,
		       (save_control & (~MK48T59_RTC_CB_STOP)));
	
	/* Now make sure the read bit is off so the value will change. */
	save_control = pcore_nvram_rd(MK48T59_RTC_CONTROLA);
	save_control &= ~MK48T59_RTC_CA_READ;
	pcore_nvram_wr(MK48T59_RTC_CONTROLA, save_control);
	
	cp = &cmd_line[strlen(cmd_line)];
	printf("Load/PPC : %s",cmd_line);
	
	sec_e = pcore_nvram_rd(MK48T59_RTC_SECONDS)&0x0f;
	while ((sec_s = pcore_nvram_rd(MK48T59_RTC_SECONDS)&0x0f) == sec_e)	/* wait for clock to be ready */
		;
	do {
		if (sercon_tstc()) {
			while (((ch = sercon_getc_w()) != '\n') && (ch != '\r')) {
				if (ch == '\b') {
					if (cp != cmd_line) {
						cp--;
						sercon_puts("\b \b");
					} 
				} else {
					*cp++ = ch;
					sercon_putc(ch);
				}
			}
			break;
		}
		if((sec_e = pcore_nvram_rd(MK48T59_RTC_SECONDS)&0x0f) < sec_s)
			sec_e += 60;
	} while (sec_e-sec_s < 5);
	
	
	/* Let the time values change again. */
	/*	pcore_nvram_wr(MK48T59_RTC_CONTROLA, save_control);*/
	cmd_line_e = cp;
	*cmd_line_e = '\0';
	printf("\nBooting kernel...\n");
}
