/*************************************************************************
 *** Intel Confidential                                      
 *** Copyright (C) Intel Corporation 2000                 
 *** All Rights Reserved.                                    
 *** --------------------------------------------------------------------
 ***
 *** Project Name: Flash Templates
 ***
 *** Module Name: Template
 ***
 *** File: Template.C - Common template C source file
 ***
 *** File Revision: $Revision: 1.2 $, $JustDate:  4/21/00 $, $Author: arnaldo $
 ***
 *** Purpose:
 ***
 ***    The purpose of this file is to implement the common application 
 ***    programmers interface for flash template software.  This module
 ***    implements all flash template subroutines that do not have
 ***    a device specific implementation in the device.c module for
 ***    the supported device.  Macros are used to determine if
 ***    the template.c common implementation or the device.c specific
 ***    implementation is used by the template for each subroutine.
 ***
 ***    Note that some common template functions implemented in this
 ***    module will return STATUS_UNSUPPORTED since no common
 ***    implementation is possible due to the differences between 
 ***    supported devices for those functions.  The device.c module
 ***    must contain implementations for these particular functions
 ***    when the function is in fact supported for that device.
 ***
 *** $NoKeywords: $
 *************************************************************************/

/*
***************************************************************
* INTEL OEM SOFTWARE LICENSE AGREEMENT
* BETWEEN INTEL CORPORATION AND OEM/Licensee ("You")
*  (Rev 10/04/99)
*
* BY USING THIS SOFTWARE, YOU ("You" or "Licensee") ARE AGREEING 
* TO BE BOUND BY THE TERMS OF THIS AGREEMENT.  DO NOT USE THE 
* SOFTWARE UNTIL YOU HAVE CAREFULLY READ AND AGREED TO THE 
* FOLLOWING TERMS AND CONDITIONS.  IF YOU DO NOT AGREE TO THE 
* TERMS OF THIS AGREEMENT, PROMPTLY RETURN THE SOFTWARE AND ANY 
* ACCOMPANYING ITEMS. LICENSEE MUST BE AN ORIGINAL EQUIPMENT 
* MANUFACTURER ("OEM") SYSTEM DEVELOPER TO ACQUIRE ANY RIGHTS 
* IN THE SOFTWARE UNDER THIS LICENSE AGREEMENT.
*
* LICENSE: Intel Corporation ("Intel") grants the Licensee the 
* non-exclusive right to use the software program ("Software") 
* in source and / or object code form on the terms set forth 
* below.  Licensee will not use, copy, modify, rent, sell or 
* transfer the Software or any portion thereof, except as 
* provided in this Agreement.  
*
* USE RESTRICTION:
* THIS ROYALTY FREE LICENSE ALLOWS LICENSEE TO USE THE SOFTWARE 
* WITH INTEL FLASH PRODUCTS ONLY. USE OF THE SOFTWARE WITH ANY 
* OTHER FLASH PRODUCTS IS EXPRESSLY PROHIBITED UNLESS AND UNTIL 
* LICENSEE APPLIES FOR, AND IS GRANTED, IN INTEL'S SOLE DISCRETION.
* A SEPARATE WRITTEN SOFTWARE LICENSE AGREEMENT BETWEEN LICENSEE 
* AND INTEL IS NECESSARY BEFORE ANY SUCH USE.
*
* LICENSEE MAY:
* 1. Copy the Software for support, backup or archival purposes;
* 2. Install or distribute the Software in object code form only;
* 3. Install, use, modify, distribute, and/or make or have made 
* Derivatives based on the Software subject to the terms and 
* conditions in this Agreement;
* 4. Modify and/or use Software source code that Intel provides to
* the Licensee, provided that Licensee does not remove or modify 
* the "Compatibility" module, if any, in the Software or in any 
* Derivatives and subject to the other restrictions contained 
* herein;
* 5. Allow authorized contractors ("Subcontractors") engaged by 
* the Licensee for the sole purpose of product development work to
* have access to the Software solely for that purpose. 
* Subcontractors do NOT acquire any of the Licensee rights to the 
* Software provided in this Agreement.
*
* RESTRICTIONS:  
* LICENSEE IS NOT ALLOWED TO: 
* 1.  Copy, disclose or distribute the Software, in whole or in 
* part, except as provided for in this Agreement;
* 2.  Remove or modify the "Compatibility" module, if any, in 
* the Software or in any Derivatives;
* 3.  Decompile or reverse engineer any Software delivered in 
* object code form.
*
* LICENSEE MUST:
* Execute the "Compatibility" module if provided with the 
* Software.
*
* TRANSFER:  Except as provided above, Licensee may not 
* transfer or disclose the Software to any other party.
*
* OWNERSHIP AND COPYRIGHT OF SOFTWARE: Title to the Software 
* and all copies thereof remain with Intel.  The Software is 
* copyrighted and is protected by United States and 
* international copyright laws.  Licensee will not remove 
* the copyright notice from the Software.  Licensee agrees 
* to prevent the unauthorized copying of the Software.
*
* DERIVATIVES: "Derivatives" are any work that is based upon the 
* Software, such as a revision, modification, translation, or 
* compilation in which the Software is recast, transformed or 
* adapted; any adaptation, subset, addition, improvement or 
* combination of or including the Software; and any new material, 
* information or data derived from the Software, including new 
* material which may be protectable by copyright, patent or other
* proprietary rights.  Licensee is not required to provide Intel 
* with a copy of the source or object code for any Derivatives 
* created by Licensee. Licensee may use, market, sell, and/or 
* distribute Derivatives in object code form only at Licensee's own
* risk and expense.  Licensee may use, but may not market, sell or 
* distribute, source code for any Derivatives, subject to the Use 
* Restriction and other restrictions contained in this Agreement. 
* Title to Derivatives, other than the portion(s) of the 
* Derivatives consisting of any portion of the Software, shall 
* remain with Licensee.
*
* CONFIDENTIALITY: Licensee will maintain the confidentiality of 
* the source code for the Software with at least the same degree 
* of care that Licensee uses to protect Licensee's own confidential 
* and proprietary information, but with no less than a reasonable 
* degree under the circumstances. Disclosure of source code will 
* only be made to Licensee's employees on a need-to-know basis. 
* Subject to the licenses granted hereunder, Licensee shall 
* maintain the Software source code and all other proprietary 
* information relating to the Software in confidence and shall 
* not disclose to others any such source code or other Intel 
* proprietary information relating to the Software. Any 
* Subcontractors to whom Licensee discloses the source code for 
* the Software must sign a written confidentiality agreement 
* which contains terms regarding the Software no less restrictive 
* than those set forth in this Agreement.
*
* WARRANTY:  The Software is provided "AS IS". Intel warrants 
* that the media on which the Software is furnished will be free 
* from defects in material and workmanship for a period of one 
* year upon receipt. 
*
* THE ABOVE WARRANTIES ARE THE ONLY WARRANTIES OF ANY KIND GIVEN 
* BY INTEL UNDER THIS AGREEMENT. INTEL SPECIFICALLY DISCLAIMS ANY 
* OTHER WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WARRANTIES OF 
* MERCHANTABILITY, NONINFRINGEMENT OR FITNESS FOR ANY PARTICULAR 
* PURPOSE.
*
* LIMITATION OF LIABILITY: NEITHER INTEL NOR ITS VENDORS OR AGENTS 
* SHALL BE LIABLE FOR ANY LOSS OF PROFITS, LOSS OF USE, LOSS OF 
* DATA, INTERRUPTION OF BUSINESS, NOR FOR INDIRECT, SPECIAL, 
* INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY KIND WHETHER UNDER 
* THIS AGREEMENT OR OTHERWISE, EVEN IF ADVISED OF THE POSSIBILITY 
* OF SUCH DAMAGES.
*
* TERMINATION OF THIS LICENSE: Intel reserves the right to conduct 
* or have conducted audits to verify Licensee's compliance with 
* this Agreement.  Intel may terminate this Agreement at any 
* time if Licensee is in breach of any of its terms and conditions.
* Upon termination, Licensee will immediately destroy, and certify 
* in writing the destruction of, the Software and return all copies 
* of the Software and documentation to Intel.
*
* U.S. GOVERNMENT RESTRICTED RIGHTS: The Software and documentation
* were developed at private expense and are provided with 
* "RESTRICTED RIGHTS".  Use, duplication or disclosure by the 
* Government is subject to restrictions as set forth in 
* FAR52.227-14 and DFAR252.227-7013 et seq. or its successor.
*
* EXPORT LAWS: The distribution and export/re-export of the 
* Software shall be in compliance with the laws, regulations, 
* orders or other restrictions of the U.S. Export Administration 
* Regulations.
*
* APPLICABLE LAW: This Agreement is governed by the laws of the 
* State of Delaware and the United States, including patent and
* copyright laws.  Any claim arising out of this Agreement will be 
* brought in Delaware.  
*
* Intel reserves the right to use and/or include Licensee's name
* in public relations activities and marketing material after the 
* product is publicly announced.
*****************************************************************
*/
/* Page Break */

#include <linux/config.h>

#ifdef CONFIG_TS1H

/* Include Files */
#include "cyc_cfiflash_aux.h"
//#include "..\drvrs\quicc.h"
//#include	<global.h>
//#include "..\..\sw\boot\boot.h"

/*
 * Global Functions
 */ 

#if !DEVICE_CLEAR_STATUS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ClearStatus
 *
 * Description:   
 *    This procedure is called to clear the status register on the flash
 *    device.  See the flash device datasheet for specific details on 
 *    this command.
 *
 * Parameters:
 *    NONE
 *
 * Returns:   
 *    NONE
 *
 * Assumptions:
 *    NONE
 *
 ***************************************************************************/
void TMPL_ClearStatus ( void )
{

   TMPL_WriteF( TMPL_BASE_FLASH_ADDRESS,  TMPL_CLEAR_STATUS_REGISTER );

}
#endif /* !DEVICE_CLEAR_STATUS */


#if !DEVICE_ERASE_BLOCK /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_EraseBlock
 *
 * Description:   
 *
 *    This procedure is called to erase a data block on the flash 
 *    device.  See the flash device datasheet for specific details on
 *    this command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.  
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_EraseBlock ( UINT16 blocknum,
                              UINT8  returnSR )
{

   TMPL_Status   stat;
   UINT32 blockadd;
  
   stat = TMPL_GetBlockAddress( blocknum, &blockadd );

   if ( stat.Result != StatCompleted )
   {
	  return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   TMPL_WriteF(blockadd, TMPL_BLOCK_ERASE);

   TMPL_WriteF(blockadd, TMPL_CONFIRM );

   if ( !TMPL_WaitUntilReady( TMPL_ERASE_TIMEOUT ) )
   {
      stat.Result = StatTimeout;
   }
   else
   {
      stat.Result = StatCompleted;
   }

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );

}
#endif /* !DEVICE_ERASE_BLOCK */

#if !DEVICE_GET_BLOCK_ADDRESS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_GetBlockAddress
 *
 * Description:   
 *
 *    This procedure is called to get the flash starting address for the
 *    specified block number.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    OUT     address  - the starting flash address for the specified
 *                       block.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_GetBlockAddress ( UINT16     blocknum, 
                                   UINT32_PTR address )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_GET_BLOCK_ADDRESS */


#if !CLIENT_GET_FPTR /* if there is no client provided implementation */
/****************************************************************************
 *
 * TMPL_GetFptr
 *
 * Description:   
 *
 *    This procedure is called to return a flash ptr given a 
 *    specified device address.  This routine will likely need to be
 *    provided externally by the template client in order to properly
 *    deal with addressing details that are specific to the HW platform.
 *
 * Parameters:
 *
 *    IN      address  - the flash address 
 *
 * Returns:   
 *
 *    TMPL_FDATA_PTR  - address returned in ptr form
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
extern unsigned char *flash_addr;
TMPL_FDATA_PTR  TMPL_GetFptr ( UINT32 address )
{
   return( (TMPL_FDATA_PTR) (address + flash_addr) );
}
#endif /* !CLIENT_GET_FPTR */

#if !DEVICE_WAIT_UNTIL_READY /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_WaitUntilReady
 *
 * Description:   
 *
 *    This procedure is called to wait until the flash device status 
 *    indicates a ready state or times out.  See the flash device datasheet
 *    for specific details on reading status data.
 *
 * Parameters:
 *
 *    IN      timeout  - timeout value specified as number of times
 *                       to read the status register before giving up.
 *
 * Returns:   
 *
 *    UINT8          - boolean flag indicating whether the ready state was
 *                     reached before the timeout number of reads 
 *                     occured.
 *
 * Assumptions:
 *
 *    NONE.
 *
 ***************************************************************************/
UINT8 TMPL_WaitUntilReady ( UINT32 timeout )
{

   UINT32 count=0;
   TMPL_FDATA status;
   TMPL_FDATA_PTR  fptr = TMPL_GetFptr(TMPL_BASE_FLASH_ADDRESS);
   
   *fptr = (TMPL_FDATA) TMPL_READ_STATUS_REGISTER;

   while( timeout )
   {
      status = *fptr;
      if ( mIsStatusReady( status ) )
	  {
	     return( TRUE );
	  }

      timeout--;
	  count++;

	  if (!(count & 3)) {
		schedule();
	  }
   }

   return( FALSE );

}
#endif /* !DEVICE_WAIT_UNTIL_READY */


#if !DEVICE_WRITEF /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_WriteF
 *
 * Description:   
 *
 *    This procedure is called to write a single data item directly to the 
 *    specified device address.  This function is used internally by the
 *    flash template api.  Clients that need to write data to the flash
 *    device should use the TMPL_ProgramFlash or TMPL_ProgramFlashBuffered
 *    command.  See the flash device datasheet for specific details on
 *    writing flash data.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be written to.
 *
 *    IN      value    - the flash data to write to the device.
 *
 * Returns:   
 *
 *    NONE
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
void TMPL_WriteF ( UINT32     address, 
                   TMPL_FDATA value )
{

   TMPL_FDATA_PTR  fptr;

   fptr = TMPL_GetFptr(address);

   *fptr = value; 

}
#endif /* !DEVICE_WRITEF */

#if !DEVICE_READ_STATUS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ReadStatus
 *
 * Description:   
 *
 *    This procedure is called to read the status register value from
 *    the flash device.  See the flash device datasheet for specific details
 *    on this command.
 *
 * Parameters:
 *
 *    NONE
 *
 * Returns:   
 *
 *    TMPL_FDATA - the status register value read from the device.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_FDATA TMPL_ReadStatus ( void )
{
    TMPL_FDATA status;

	TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_STATUS_REGISTER );

	TMPL_ReadF(TMPL_BASE_FLASH_ADDRESS, &status );

    return( status );

}
#endif /* !DEVICE_READ_STATUS */

#if !DEVICE_READF /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ReadF
 *
 * Description:   
 *
 *    This procedure is called to read a single data item directly from the 
 *    specified device address.  This function is used internally by the
 *    flash template api.  Clients that need to read data from the flash
 *    device should use the TMPL_ReadFlash command.  See the flash device 
 *    datasheet for specific details on reading flash data.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be read from.
 *
 *    OUT     value    - the flash data read from the device.
 *
 * Returns:   
 *
 *    NONE
 *
 * Assumptions:
 *
 *    The flash device is in the READ_ARRAY mode when this function is
 *    called.
 *
 ***************************************************************************/
void TMPL_ReadF ( UINT32         address, 
                  TMPL_FDATA_PTR value )
{

   TMPL_FDATA_PTR  fptr;

   fptr = TMPL_GetFptr(address);

   *value = *fptr; 

}
#endif /* !DEVICE_READF */

#if !DEVICE_ERASE_ALL_BLOCKS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_EraseAllBlocks
 *
 * Description:   
 *
 *    This procedure is called to erase all data blocks on the flash 
 *    device.  See the flash device datasheet for specific details on
 *    the block erase command.
 *
 * Parameters:
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_EraseAllBlocks ( UINT8 returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_ERASE_ALL_BLOCKS */


#if !DEVICE_ERASE_SUSPEND /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_EraseSuspend
 *
 * Description:   
 *
 *    This procedure is called to issue the erase suspend command to
 *    the flash device.  See the flash device datasheet for specific details 
 *    on this command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    When this function is called the device is currently in the erase 
 *    mode for the block identified.
 *
 ***************************************************************************/
TMPL_Status TMPL_EraseSuspend ( UINT16 blocknum,
                                UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_ERASE_SUSPEND */


#if !DEVICE_EXTENDED_QUERY
/****************************************************************************
 *
 * TMPL_ExtendedQuery
 *
 * Description:   
 *
 *    This procedure is called to retrieve the extended query
 *    data from the flash device.  See the flash device datasheet for
 *    specific details on this command.
 *
 * Parameters:
 *
 *    OUT      *extquery - pointer to extended query structure
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 ***************************************************************************/
TMPL_Status TMPL_ExtendedQuery ( struct TMPL_ExtQueryData *extquery )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );

}
#endif /* !DEVICE_EXTENDED_QUERY */


#if !DEVICE_GET_QUERY_ADDRESS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_GetQueryAddress
 *
 * Description:   
 *
 *    This procedure is called to get the flash address for a given
 *    query offset.
 *
 *   Parameters:
 *
 *    IN      offset   - query offset location
 *
 *    OUT     address  - the flash address for the specified query
 *                       offset.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_GetQueryAddress ( UINT32 offset, 
                                   UINT32_PTR address )
{
   TMPL_Status stat;

#if ( X_8 )
   *address = offset;
#elif ( X_16 )
   *address = offset << 1;
#elif( PX_16 || X_32 )
   *address = offset << 2;
#endif

   *address += TMPL_BASE_FLASH_ADDRESS;

   stat.Result = StatCompleted;

   return( stat );
}
#endif /* !DEVICE_GET_QUERY_ADDRESS */


#if !DEVICE_LOCK_BLOCK /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_LockBlock
 *
 * Description:   
 *
 *    This procedure is called to lock the specified block on the flash
 *    device.  See the flash device datasheet for specific details on this 
 *    command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_LockBlock ( UINT16 blocknum,
                             UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_LOCK_BLOCK */


#if !DEVICE_LOCK_DOWN_BLOCK /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_LockDownBlock
 *
 * Description:   
 *
 *    This procedure is called to lockdown the specified block on the flash
 *    device.  See the flash device datasheet for specific details on this 
 *    command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_LockDownBlock ( UINT16 blocknum,
                                 UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_LOCK_DOWN_BLOCK */


#if !DEVICE_LOCK_PROTECTION /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_LockProtection
 *
 * Description:   
 *
 *    This procedure is called to program the protection register user lock
 *    bit on the flash device.  See the flash device datasheet for specific
 *    details on this command.
 *
 * Parameters:
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_LockProtection ( UINT8 returnSR )
{
   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_LOCK_PROTECTION */


#if !DEVICE_PAGE_MODE /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_PageMode 
 *
 * Description:   
 *
 *    This procedure is called to enable or disable page mode read to
 *    the device.
 *
 * Parameters:
 *
 *    IN    enable - flag "1" to indicate the page mode is anable and 
 *                   flag "0" to indicate the page mode is disable to 
 *                   the device.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum 
 *                  TMPL_CommandStat and optionally the flash device 
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE 
 *
 ***************************************************************************/
TMPL_Status TMPL_PageMode ( UINT16 enable )
{
   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_PAGE_MODE */


#if !DEVICE_PROGRAM_FLASH /* if there is no device.c implementation */
#if X_16
/****************************************************************************
 *
 * TMPL_ProgramFlash
 *
 * Description:   
 *
 *    This procedure is called to program the flash device at the specified 
 *    address with the single specified data value.  See the flash device 
 *    datasheet for specific details on this command.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be programmed.
 *
 *    IN      item     - the data value to be programmed.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramFlash ( UINT32 address, 
                                UINT8  item, 
                                UINT8  returnSR )
{

   TMPL_Status stat;

   TMPL_FDATA_PTR  fptr;
   TMPL_FDATA writedata;

   if ( address >= TMPL_TOTAL_SIZE )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   writedata = (TMPL_FDATA)( item );

   if ( ( address & 0x1 ) != 0 )
   {
      address--;
#if (BIG_ENDIAN_ARCHITECTURE)
	  writedata |= 0xFF00;
#else /* little endian */
      writedata = ( writedata << 8 ) | 0x00ff;
#endif
   }
   else
   {
#if BIG_ENDIAN_ARCHITECTURE
      writedata |= 0x00ff;
#else /* little endian */
      writedata |= 0xff00;
#endif /* BIG_ENDIAN_ARCHITECTURE */
   } 

   TMPL_WriteF(address, TMPL_PROGRAM_SETUP );

   fptr = TMPL_GetFptr(address);

   *fptr = writedata;

   if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
   {
      stat.Result = StatTimeout;
   }
   else
   {
      stat.Result = StatCompleted;
   }

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}
#endif /* X_16 */
#endif /* !DEVICE_PROGRAM_FLASH */

#if !DEVICE_PROGRAM_FLASH /* if there is no device.c implementation */
#if X_8
/****************************************************************************
 *
 * TMPL_ProgramFlash
 *
 * Description:   
 *
 *    This procedure is called to program the flash device at the specified 
 *    address with the single specified data value.  See the flash device 
 *    datasheet for specific details on this command.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be programmed.
 *
 *    IN      item     - the data value to be programmed.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramFlash ( UINT32 address, 
                                UINT8  item, 
                                UINT8  returnSR )
{

   TMPL_Status stat;

   TMPL_FDATA_PTR  fptr;
   TMPL_FDATA writedata;

   if ( address >= TMPL_TOTAL_SIZE )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   writedata = (TMPL_FDATA)( item );

   TMPL_WriteF(address, TMPL_PROGRAM_SETUP );

   fptr = TMPL_GetFptr(address);

   *fptr = writedata;

   if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
   {
      stat.Result = StatTimeout;
   }
   else
   {
      stat.Result = StatCompleted;
   }

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}
#endif /* X_8 */
#endif /* !DEVICE_PROGRAM_FLASH */


#if !DEVICE_PROGRAM_FLASH_BUFFERED /* if there is no device.c implementation */
#if X_16
/****************************************************************************
 *
 * TMPL_ProgramFlashBuffered
 *
 * Description:   
 *
 *    This procedure is called to program the flash device at the specified 
 *    starting address contiguously with the specified buffer data.  See
 *    the flash device datasheet for specific details on the program
 *    command.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be programmed.
 *
 *    IN      buffer   - the buffer containing data to be programmed.
 *
 *    IN      numitems - the number of data items contained in the buffer.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE.
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramFlashBuffered ( UINT32    address, 
                                        UINT8_PTR buffer, 
                                        UINT32    numbytes,
                                        UINT8     returnSR )
{

   UINT32 count;
   TMPL_Status stat;

   TMPL_FDATA_PTR  fptr;
   TMPL_FDATA writedata;

   if ( ( address + numbytes ) > TMPL_TOTAL_SIZE )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   if ( numbytes > 0 ) /* while more data to write */
   {
      while ( numbytes > 0 ) /* if 2 or more bytes still to write */
      {

         if ( ( address & 0x1 ) != 0 ) /* if destination address is odd */
         {
            address--;

#if (BIG_ENDIAN_ARCHITECTURE)
            writedata = (FDATA) *buffer;
			writedata |= 0xff00;
#else /* little endian */
            writedata = *((TMPL_FDATA_PTR)buffer);
            writedata = ( writedata << 8 ) | 0x00ff;
#endif
			numbytes--;

			buffer++;
		 }
         else /* destination address is even */
		 {

#if BIG_ENDIAN_ARCHITECTURE

            /* grab first byte */
            writedata = (TMPL_FDATA)( *buffer );

			writedata = ( ( writedata << 8 ) & 0xFF00 );

            /* grab second byte */
			writedata = writedata | ( (TMPL_FDATA) *(buffer+1) );

#else /* little endian architecture */

            /* grab 2 bytes */
            writedata = *( (TMPL_FDATA_PTR)buffer ); 


#endif /* BIG_ENDIAN_ARCHITECTURE */


	        if ( numbytes == 1 ) 
	        {

#if BIG_ENDIAN_ARCHITECTURE
               writedata |= 0x00ff;
#else
               writedata |= 0xff00;
#endif

               numbytes--;
            }
	        else
	        {
	           numbytes -= sizeof(TMPL_FDATA);
	        }

		    buffer += sizeof(TMPL_FDATA);

         }

	     TMPL_WriteF(address, TMPL_PROGRAM_SETUP );

         fptr = TMPL_GetFptr(address);

         *fptr = writedata;

         if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
	     {
            stat.Result = StatTimeout;

            if ( returnSR )
            {
               stat.SR = TMPL_ReadStatus();
            }
 
	        return( stat );
	     }

         address += sizeof(TMPL_FDATA);
      }
   }

   stat.Result = StatCompleted;

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}
#endif /* X_16 */


#if X_8
/****************************************************************************
 *
 * TMPL_ProgramFlashBuffered
 *
 * Description:   
 *
 *    This procedure is called to program the flash device at the specified 
 *    starting address contiguously with the specified buffer data.  See
 *    the flash device datasheet for specific details on the program
 *    command.
 *
 * Parameters:
 *
 *    IN      address  - the flash address to be programmed.
 *
 *    IN      buffer   - the buffer containing data to be programmed.
 *
 *    IN      numitems - the number of data items contained in the buffer.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE.
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramFlashBuffered ( UINT32    address, 
                                        UINT8_PTR buffer, 
                                        UINT32    numbytes,
                                        UINT8     returnSR )
{

   TMPL_Status stat;

   TMPL_FDATA_PTR  fptr;
   TMPL_FDATA writedata;

   if ( ( address + numbytes ) > TMPL_TOTAL_SIZE )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   if ( numbytes > 0 ) /* while more data to write */
   {
      while ( numbytes > 0 ) /* if 2 or more bytes still to write */
      {

         writedata = (TMPL_FDATA) *buffer;
		 numbytes--;

		 buffer++;

	     TMPL_WriteF( address, TMPL_PROGRAM_SETUP );

         fptr = TMPL_GetFptr(address);

         *fptr = writedata;

         if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
	     {
            stat.Result = StatTimeout;

            if ( returnSR )
            {
               stat.SR = TMPL_ReadStatus();
            }
 
	        return( stat );
	     }

         address += sizeof(TMPL_FDATA);
      }
   }

   stat.Result = StatCompleted;

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}

/* This is an optimized version of the function above for char access */

TMPL_Status TMPL_ProgramFlashBufferedFast ( UINT32    address, 
                                        	UINT8_PTR buffer, 
                                        	UINT32    numbytes,
                                        	UINT8     returnSR )
{

   TMPL_Status stat;
   UINT32 numwritebytes;
   UINT32 byteswritten;
   TMPL_FDATA_PTR  cmdaddress;
   TMPL_FDATA_PTR  fptr = TMPL_GetFptr(address);

   if ( ( address & 0x01 ) || (( address + numbytes ) > TMPL_TOTAL_SIZE ) )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
   {
      stat.Result = StatTimeout;
      if ( returnSR )
      {
         stat.SR = TMPL_ReadStatus();
      }

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   if ( numbytes > 0 ) /* while more data to write */
   {

      while ( numbytes != 0 )
      {

         if ( numbytes > TMPL_BUFFER_SIZE )
		 {
		    /* write full TMPL_BUFFER_SIZE-byte buffer */
		    numwritebytes = TMPL_BUFFER_SIZE;
		 }
		 else
		 {
		    /* write partial buffer */
		    numwritebytes = numbytes;
         }

         byteswritten = numwritebytes;

		 cmdaddress = fptr;
         *cmdaddress = (TMPL_FDATA) TMPL_WRITE_TO_BUFFER;
         *cmdaddress = (TMPL_FDATA) (numwritebytes-1);
	
	     while ( numwritebytes > 0 ) /* if more bytes still to write */ 
		 { /* address is known even at this point */
	         *fptr = (TMPL_FDATA) *buffer;
		     numwritebytes--;
		     buffer++;
		     fptr++;
	     }

		 *cmdaddress = (TMPL_FDATA) TMPL_CONFIRM;

         if ( !TMPL_WaitUntilReady( TMPL_PROGRAM_TIMEOUT ) )
         {
            stat.Result = StatTimeout;
            if ( returnSR )
            {
               stat.SR = TMPL_ReadStatus();
            }

            return( stat );
         }

         numbytes -= byteswritten;

      } 

   }

   stat.Result = StatCompleted;

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus();
   }

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}
#endif /* X_8 */
#endif /* !DEVICE_PROGRAM_FLASH_BUFFERED */


#if !DEVICE_PROGRAM_PROTECTION /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ProgramProtection
 *
 * Description:   
 *
 *    This procedure is called to program the protection register on
 *    the flash device at the specified location with the specified data
 *    value.  See the flash device datasheet for specific details on this
 *    command.
 *
 * Parameters:
 *
 *    IN      location - the protection register location on the flash
 *                       device to be programmed.
 *
 *    IN      value    - the data item to be programmed.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramProtection ( UINT32     location, 
                                     TMPL_FDATA value,
                                     UINT8      returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_PROGRAM_PROTECTION */


#if !DEVICE_PROGRAM_SUSPEND /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ProgramSuspend
 *
 * Description:   
 *
 *    This procedure is called to issue the program suspend command to
 *    the flash device.  See the flash device datasheet for specific details 
 *    on this command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    When this function is called the device is currently in the program
 *    mode for the block identified.
 *
 ***************************************************************************/
TMPL_Status TMPL_ProgramSuspend ( UINT16 blocknum,
                                  UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_PROGRAM_SUSPEND */


#if !DEVICE_QUERY /* if there is no device.c implementation */
#if X_16
/****************************************************************************
 *
 * TMPL_Query
 *
 * Description:   
 *
 *    This procedure is called to issue the query command to
 *    the flash device.  See the flash device datasheet for specific details 
 *    on this command.
 *
 * Parameters:
 *
 *    OUT      *query - pointer to query structure
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_Query ( struct TMPL_QueryData *query )
{

   TMPL_Status stat;
   UINT32 add;
   UINT32 offset;
   TMPL_FDATA item;
   UINT32 longitem;
   UINT32 i;

   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_QUERY );
 
   offset = TMPL_QUERY_START_OFFSET;

   /* read query string */ 
   for ( i=0; i < 3; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );
      item &= 0xff;
      query->QueryStr[i] = (char)item;
	  offset++;
   }
   query->QueryStr[3] = '\0'; /* null terminate string */

   /* read vendor id */
   query->VendorId = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif

      query->VendorId = (UINT16)( query->VendorId | item ); 
      offset++;
   }

   /* read extended table ptr */
   query->ExtTablePtr = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif

      query->ExtTablePtr = (UINT16)( query->ExtTablePtr | item ); 
      offset++;
   }
 
   /* read alternate vendor id */
   query->AltVendorId = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif
      query->AltVendorId = (UINT16)( query->AltVendorId | item ); 
      offset++;
   }

   /* read secondary extended table ptr */
   query->SecExtTablePtr = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif
      query->SecExtTablePtr = (UINT16)
                                 ( query->SecExtTablePtr | item ); 
      offset++;
   }

   /* read minimum voltage */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->VccMin = (UINT8)item;
   offset++;

   /* read maximum voltage */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->VccMax = (UINT8)item;
   offset++;

   /* read supply voltage */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->VppMin = (UINT8)item;
   offset++;

   /* read supply max voltage */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->VppMax = (UINT8)item;
   offset++;

   /* read typical program timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->TypicalProgTimeout = (UINT8)item;
   offset++;

   /* read typical buffer timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->TypicalBufferTimeout = (UINT8)item;
   offset++;

   /* read block erase timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->BlockEraseTimeout = (UINT8)item;
   offset++;

   /* read chip erase timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->ChipEraseTimeout = (UINT8)item;
   offset++;

   /* read max single timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->MaxSingleTimeout = (UINT8)item;
   offset++;

   /* read max buffer timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->MaxBufferTimeout = (UINT8)item;
   offset++;

   /* read max block erase timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->MaxBlockEraseTimeout = (UINT8)item;
   offset++;

   /* read max chip erase timeout */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->MaxChipEraseTimeout = (UINT8)item;
   offset++;

   /* read device size */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->DeviceSize = (UINT8)item;
   offset++;

   /* read device interface */
   query->DeviceInterface = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif

      query->DeviceInterface = (UINT16)
                                 ( query->DeviceInterface | item ); 
      offset++;
   }

   /* read max write buffer bytes */
   query->MaxWriteBufferBytes = 0;
   for ( i=0; i < 2; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  item = item << (8*(1-i) );
#else
      item &= 0xff;
	  item = item << (8*i);
#endif

      query->MaxWriteBufferBytes = (UINT16)
                                 ( query->MaxWriteBufferBytes | item ); 
      offset++;
   }

   /* read num erase blocks */
   stat = TMPL_GetQueryAddress(offset, &add);

   if ( stat.Result != StatCompleted )
   {
      return( stat );
   }

   TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
   item &= 0x00ff;
#else
   item &= 0xff;
#endif

   query->NumEraseBlocks = (UINT8)item;
   offset++;

   /* read erase block information */
   query->EraseBlockInformation = 0;
   for ( i=0; i < 4; i++ )
   {
      stat = TMPL_GetQueryAddress(offset, &add);

      if ( stat.Result != StatCompleted )
      {
         return( stat );
      }

      TMPL_ReadF( add, &item );

#if BIG_ENDIAN_ARCHITECTURE
      item &= 0x00ff;
	  longitem = (UINT32)((UINT32)item << (8*(3-i) ));
#else
      item &= 0xff;
	  longitem = (UINT32)((UINT32)item << (8*i));
#endif

      query->EraseBlockInformation = (UINT32)
                                 ( query->EraseBlockInformation | longitem ); 
      offset++;
   }

   stat.Result = StatCompleted;

   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   return( stat );
}
#endif /* X_16 */
#endif /* !DEVICE_QUERY */


#if !DEVICE_READ_BLOCK_STATUS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ReadBlockStatus
 *
 * Description:   
 *
 *    This procedure is called to read the status for the specified block
 *    from the flash device.  See the flash device datasheet for specific 
 *    details on this command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number on the device.
 *
 *    OUT     blockstat- the status of the block as: unlocked, locked, 
 *                       or locked down.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ReadBlockStatus ( UINT16         blocknum,
                                   TMPL_FDATA_PTR blockstat );
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_READ_BLOCK_STATUS */


#if !DEVICE_READ_DEVICE_ID /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ReadDeviceID
 *
 * Description:   
 *
 *    This procedure is called to read the status for the specified block
 *    from the flash device.  See the flash device datasheet for specific 
 *    details on this command.
 *
 * Parameters:
 *
 *    OUT     mcode    - the manufacturer code.
 *
 *    OUT     deviceid - the device id.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ReadDeviceID ( UINT16_PTR mcode, 
                                UINT16_PTR deviceid )
{
   TMPL_Status stat;
   UINT32 idaddress;

#if X_8
   idaddress = ( TMPL_BASE_FLASH_ADDRESS + 2 );
#endif

#if PX_16
   idaddress = ( TMPL_BASE_FLASH_ADDRESS + sizeof( TMPL_FDATA ) );
#endif

#if X_16
   idaddress = ( TMPL_BASE_FLASH_ADDRESS + sizeof( TMPL_FDATA ) );
#endif

#if X_32
   idaddress = ( TMPL_BASE_FLASH_ADDRESS + sizeof( TMPL_FDATA ) );
#endif

#if ILX_32
   idaddress = ( TMPL_BASE_FLASH_ADDRESS + ( sizeof( TMPL_FDATA ) * 2 ) );
#endif

   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ID_CODES);

   TMPL_ReadF( TMPL_BASE_FLASH_ADDRESS, mcode);

   TMPL_ReadF( idaddress, deviceid);

   /* return device to read array mode */
   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   stat.Result = StatCompleted;

   return( stat );

}
#endif /* !DEVICE_READ_DEVICE_ID */


#if !DEVICE_READ_FLASH /* if there is no device.c implementation */
#if X_16
/****************************************************************************
 *
 * TMPL_ReadFlash
 *
 * Description:   
 *
 *    This procedure is called to read one or more data items from 
 *    the flash device at the specified starting address.  See the flash 
 *    device datasheet for specific details on reading flash data.
 *
 * Parameters:
 *
 *    IN      address  - the starting device address.
 *
 *    OUT     buffer   - the buffer to contain the data items read.
 *
 *    IN      numitems - the number of data items to read.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ReadFlash ( UINT32    address, 
                             UINT8_PTR buffer, 
                             UINT32    numbytes,
                             UINT8     returnSR )
{
   TMPL_Status stat;
   TMPL_FDATA readdata;
   TMPL_FDATA readdatatemp;

   TMPL_FDATA_PTR fptr;

   if ( ( address + numbytes ) > TMPL_TOTAL_SIZE )
   {
      stat.Result = StatBadAddress;

      return( stat );
   }

   if ( returnSR )
   {
      TMPL_ClearStatus();
   }

   TMPL_WriteF(TMPL_BASE_FLASH_ADDRESS, TMPL_READ_ARRAY );

   if ( ( address & 0x1 ) != 0 ) 
   {
      address--;

      fptr = TMPL_GetFptr(address);

	  readdata = *fptr;

	  *buffer++ = mGetHighMemoryByte(readdata);

      numbytes--;

      address += sizeof(TMPL_FDATA);
   }

   while ( numbytes > 1 )
   {

      fptr = TMPL_GetFptr(address);

      readdata = *fptr;

	  *buffer++ = mGetLowMemoryByte(readdata);
      *buffer++ = mGetHighMemoryByte(readdata);

      address += sizeof(TMPL_FDATA);
	  numbytes -= sizeof(TMPL_FDATA);
   }	

   if ( numbytes > 0 ) /* 1 more byte left */
   {
      fptr = TMPL_GetFptr(address);

	  readdata = *fptr;

	  *buffer = mGetLowMemoryByte(readdata);

   }

   stat.Result = StatCompleted;

   if ( returnSR )
   {
      stat.SR = TMPL_ReadStatus(); 
   }

   return( stat );

}
#endif /* X_16 */
#endif /* !DEVICE_READ_FLASH */


#if !DEVICE_READ_PROTECTION /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_ReadProtection
 *
 * Description:   
 *
 *    This procedure is called to read the protection register value on
 *    the flash device from the specified location.  See the flash device 
 *    datasheet for specific details on this command.
 *
 * Parameters:
 *
 *    IN      location - the protection register location on the flash
 *                       device to be read from. 
 *
 *    OUT     value    - the data item read from the register.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_ReadProtection ( UINT32         location, 
                                  TMPL_FDATA_PTR value )
{

   STATUS stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_READ_PROTECTION */


#if !DEVICE_RESUME /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_Resume
 *
 * Description:   
 *
 *    This procedure is called to issue the resume command to the flash
 *    device for the specified block. See the flash device datasheet for
 *    specific details on this command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number to resume.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    The block indicated was previously program suspended or erase 
 *    suspended.
 *
 ***************************************************************************/
TMPL_Status TMPL_Resume ( UINT16 blocknum,
                          UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_RESUME */


#if !DEVICE_SEC_EXTENDED_QUERY /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_SecExtendedQuery
 *
 * Description:   
 *
 *    This procedure is called to retrieve the secondary extended query
 *    data from the flash device.  See the flash device datasheet for
 *    specific details on this command.
 *
 * Parameters:
 *
 *    OUT      *secextquery - pointer to secondary extended query structure
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat.
 *
 * Assumptions:
 *
 *    NONE
 *
 ***************************************************************************/
TMPL_Status TMPL_SecExtendedQuery ( struct TMPL_SecExtQueryData *secextquery )
{
   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_SEC_EXTENDED_QUERY */


#if !DEVICE_UNLOCK_ALL_BLOCKS /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_UnlockAllBlocks
 *
 * Description:   
 *
 *    This procedure is called to unlock all blocks on the flash
 *    device.  See the flash device datasheet for specific details on this 
 *    command.
 *
 * Parameters:
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum 
 *                  TMPL_CommandStat and optionally the flash device 
 *                  status register value.
 *
 * Assumptions:
 *
 *    The blocks were previously locked.
 *
 ***************************************************************************/
TMPL_Status TMPL_UnlockAllBlocks ( UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_UNLOCK_ALL_BLOCKS */


#if !DEVICE_UNLOCK_BLOCK /* if there is no device.c implementation */
/****************************************************************************
 *
 * TMPL_UnlockBlock
 *
 * Description:   
 *
 *    This procedure is called to unlock the specified block on the flash
 *    device.  See the flash device datasheet for specific details on this 
 *    command.
 *
 * Parameters:
 *
 *    IN      blocknum - the block number to unlock.
 *
 *    IN      returnSR - flag to indicate whether the device status register
 *                       value should be returned by this function.
 *
 * Returns:   
 *
 *    TMPL_Status - includes function return status defined by enum
 *                  TMPL_CommandStat and optionally the flash device
 *                  status register value.
 *
 * Assumptions:
 *
 *    The block indicated was previously locked.
 *
 ***************************************************************************/
TMPL_Status TMPL_UnlockBlock ( UINT16 blocknum,
                               UINT8  returnSR )
{

   TMPL_Status stat;

   stat.Result = StatUnsupported;

   return( stat );
}
#endif /* !DEVICE_UNLOCK_BLOCK */


#endif /* CONFIG_TS1H */

