 /*****************************************************************
 * File: linux/fs/proc/cyc_flash.c
 *
 * Copyright (C) 2003 Cyclades Corporation
 *
 * Maintainer: www.cyclades.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * Description: proc Flash FS handling functions
 *
 ****************************************************************/

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/minix_fs.h>
#include <linux/ext2_fs.h>
#include <linux/romfs_fs.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/hdreg.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/malloc.h>
#include <linux/ioctl.h>
#include <linux/fd.h>
#include <linux/module.h>
#include <linux/init.h>

#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/byteorder.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel_stat.h>
#include <linux/tty.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <linux/pagemap.h>
#include <linux/swap.h>
#include <linux/slab.h>
#include <linux/smp.h>
#include <linux/signal.h>

#include <asm/pgtable.h>
#include <asm/io.h>
#include <linux/blk.h>

#include <asm/delay.h>

#include <asm/mpc8xx.h>
#include <asm/8xx_immap.h>
#include <asm/flash.h>
#ifdef CONFIG_TSxK
#include <asm/cpld.h>
#endif

#ifdef CONFIG_TSxK
/* prototypes */
static int cf_send_cmd(int command);
static int cf_detect (void);
static int cf_read (unsigned char *buf, int lba_addr, int size);
static int cf_write (unsigned char *buf, int lba_addr, int size);
static void cf_print_info(void);
int cf_get_size(void);
static inline void cf_led_on (void);
static inline void cf_led_off (void); 
#endif

#define MAJOR_NR CYC_FLASH_MAJOR

#define CARD_FLASH	1
#define CARD_IDE	2

#define CONFIG_FILE_SIZE 512

#define BUILD_CORE

static int n_shareds = 0;

struct SHARED_SECTORS {
	int sector;
	int state;
};

static struct SHARED_SECTORS sh_sectors[1] = {{0}};

#define MAX_SH_ITEMS (sizeof(sh_sectors)/sizeof(struct SHARED_SECTORS ))

struct CycFlashPartition {
	int	sector_start;
	int	sector_end;
	int	current_sector;
	int	initial_offset;
	int	end_offset;
	int	flags;
	unsigned char	*buf;
	unsigned short crc_flag;
	unsigned short crc_calc;
	struct CycFlashCtr *flash_ctr;
	int ix_word_size;
};

struct CycFlashCtr {
	int flash_locked;
	int flash_type;
	int cfg_addr1;
	int cfg_addr2;
	int FlashNblkChip;
	int flash_length;
	int FlashNblkTotal;
	unsigned char *flash_vma_addr;
	int (*NumOfSubSector)(struct CycFlashCtr *cf, int sector);
	char *(*SubSectorAddr)(struct CycFlashCtr *cf, int sector, int sub_sector);
	char *(*FlashChipAddr)(struct CycFlashCtr *cf, int sector);
	int (*FlashWrite)(struct CycFlashPartition *fp);
	int (*EraseSector)(struct CycFlashPartition *fp, int sector);
	int (*FlashRead)(struct CycFlashPartition *fp, int sector);
	int (*FlashFileSize)(struct CycFlashPartition *fp, int wr, int val);
};

static struct CycFlashCtr FlashControl = { 0 };

#ifdef CONFIG_TSxK
static struct CycFlashCtr CF_Control = { 0 };
#endif

static int cyc_flash_grava(struct CycFlashCtr *cf, char *buf, int sector);
static int do_flash_erase_sector(struct CycFlashPartition *fp, int sector);
static int do_flash_write(struct CycFlashPartition *fp);
static int sector_erase_command_sequence(struct CycFlashCtr *cf, int sector);
#ifdef CONFIG_TS1H
static int cyc_cfiflash_grava(struct CycFlashCtr *cf, char *buf, int sector);
static int cyc_cfiflash_erase(struct CycFlashCtr *cf, int sector);
#endif

static struct CycFlashPartition cyc_flash_ori_partition = { crc_flag: 2, };

static struct CycFlashPartition cyc_flash_alt_partition = { crc_flag: 3, };

static struct CycFlashPartition cyc_flash_cnf_partition = { crc_flag: 2, };

static struct CycFlashPartition cyc_flash_ope_partition = { crc_flag: 3, };

static struct CycFlashPartition cyc_flash_scr_partition = { crc_flag: 2, };

#ifdef BUILD_CORE
static struct CycFlashPartition cyc_flash_core_partition = { crc_flag: 4, };
#endif

static ssize_t cyc_flash_write(struct file * filp, const char * buf,
			size_t count, loff_t *ppos);
static ssize_t cyc_flash_read(struct file * filp, char * buf,
			size_t count, loff_t *ppos);
static int cyc_flash_release(struct inode * inode, struct file * filp);
static int cyc_flash_open(struct inode * inode, struct file * filp);
static int fill_sector_flash(int sector, int pos, char *buf, int count, struct CycFlashPartition *fp, int flag);
static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer);

static struct file_operations cyc_flash_fops = {
	NULL,			/* lseek  */
	cyc_flash_read,		/* read */
	cyc_flash_write,	/* write */
	NULL,			/* readdir */
	NULL,			/* poll */
	NULL,			/* ioctl */
	NULL,			/* mmap */
	cyc_flash_open,		/* open */
	NULL,			/* flush */
	cyc_flash_release	/* release */
};

struct inode_operations cyc_flash_inode_operations = {
	&cyc_flash_fops,	/* default file-ops */
	NULL,			/* create */
	NULL,			/* lookup */
	NULL,			/* link */
	NULL,			/* unlink */
	NULL,			/* symlink */
	NULL,			/* mkdir */
	NULL,			/* rmdir */
	NULL,			/* mknod */
	NULL,			/* rename */
	NULL,			/* readlink */
	NULL,			/* follow_link */
	NULL,			/* readpage */
	NULL,			/* writepage */
	NULL,			/* bmap */
	NULL,			/* truncate */
	NULL			/* permission */
};

#define FLASH_PERMISSION (S_IFREG | S_IRUSR | S_IWUSR)
//[RK]Aug/15/06 - RW for group admin
#define FLASH_PERMISSION_GROUP (S_IFREG | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP)
#define FLASH_PERMISSION_CORE (S_IFREG | S_IRUSR | S_IWUSR)

struct proc_dir_entry proc_root_flash = {
	PROC_FLASH, 5, "flash",
	S_IFDIR | S_IRUGO | S_IXUGO, 2, 0, 0,
	0, &proc_dir_inode_operations
};
struct proc_dir_entry proc_flash_config = {
	PROC_FLASH+1, 6, "config",
	FLASH_PERMISSION, 1, 0, 0, 0, 
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_cnf_partition
};
struct proc_dir_entry proc_flash_zImage = {
	PROC_FLASH+2, 6, "zImage",
	FLASH_PERMISSION_GROUP, 1, 0, 104, 0,  //[RK]Aug/15/06 - group admin
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_ope_partition
};
struct proc_dir_entry proc_flash_boot_ori = {
	PROC_FLASH+3, 8, "boot_ori",
	FLASH_PERMISSION, 1, 0, 0, 0,
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_ori_partition
};
struct proc_dir_entry proc_flash_boot_alt = {
	PROC_FLASH+4, 8, "boot_alt",
	FLASH_PERMISSION_GROUP, 1, 0, 104, 0,	//[RK]Aug/16/06 - group admin
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_alt_partition
};
struct proc_dir_entry proc_flash_scr = {
	PROC_FLASH+6, 6, "script",
	FLASH_PERMISSION, 1, 0, 0, 0,
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_scr_partition
};
#ifdef BUILD_CORE
struct proc_dir_entry proc_flash_core = {
	PROC_FLASH+7, 6, ".flash",
	FLASH_PERMISSION_CORE, 1, 0, 0, 0,
	&cyc_flash_inode_operations,
	NULL, NULL,
	NULL, NULL, NULL,
	&cyc_flash_core_partition
};
#endif

#ifdef CONFIG_TSxK
static int lba_sector_sizes;
#endif

/*---------------------------------------------------
 *
 * Routines that handle shared flash sectors
 *
 *---------------------------------------------------*/

static void add_shared_sector(int sector)
{
	if (n_shareds >= MAX_SH_ITEMS) {
		panic("Flash - Shared sectors overflow");
	}
	sh_sectors[n_shareds++].sector = sector;
}

static int check_shared_sector(struct CycFlashPartition *fp, int start, int end)
{
	int i;

	if ((fp->crc_flag & 4)) {
		return(0);
	}
	for (i=0; i < n_shareds; i++) {
		if (sh_sectors[i].sector >= start &&
			sh_sectors[i].sector <= end) {
			return(1);
		}
	}
	return(0);
}
 
static int lock_shared_sector(int start, int end)
{
	int i;

	for (i=0; i < n_shareds; i++) {
		if (sh_sectors[i].sector >= start &&
			sh_sectors[i].sector <= end) {
			if (sh_sectors[i].state) {
				return(1);
			}
			sh_sectors[i].state = 1;
		}
	}
	return(0);
}
 
static void unlock_shared_sector(int start, int end)
{
	int i;

	for (i=0; i < n_shareds; i++) {
		if (sh_sectors[i].sector >= start &&
			sh_sectors[i].sector <= end) {
			sh_sectors[i].state = 0;
		}
	}
}
 
/*--------------------------------------------------------------------------
 *
 * at49bv1614_NumOfSubSector
 *
 * Returns the number of sub sectors inside a 64k flash block
 *
 *---------------------------------------------------------------------------*/
static int at49bv1614_NumOfSubSector(struct CycFlashCtr *cf, int sector)
{
	switch(sector % cf->FlashNblkChip) {
	case 0:			/* num of sub-sectors on first sector */
		return(8);
	case 1:			/* num of sub-sectors on second sector*/
		return(2);
	}
	/* no sub sector */
	return(1);
}

/*--------------------------------------------------------------------------
 *
 * at49bv1614_SubSectorAddr
 *
 * Returns the sub sector address inside the 64k flash block
 *
 *---------------------------------------------------------------------------*/
static char *at49bv1614_SubSectorAddr(struct CycFlashCtr *cf,
					 int sector, int sub_sector)
{
	char *addr = cf->flash_vma_addr + sector * SECTOR_SIZE;

	switch(sector % cf->FlashNblkChip) {
	case 0:			/* size of sub-sectors on first sector */
		return(addr + sub_sector * 0x2000);
	case 1:			/* size of sub-sectors on second sector*/
		return(addr + sub_sector * 0x8000);
	}
	/* no sub sector */
	return(addr);
}

#define at49bv_321_161_SubSectorAddr	at49bv1614_SubSectorAddr
#define at49bv_321_161_NumOfSubSector	at49bv1614_NumOfSubSector

/*--------------------------------------------------------------------------
 *
 * FlashChipAddr
 *
 * Returns the correct FLASH chip address (1st, 2nd or 3rd)
 *
 *---------------------------------------------------------------------------*/
static char *FlashChipAddr(struct CycFlashCtr *cf, int sector)
{
	int div;

	if ((div = (sector / cf->FlashNblkChip)) == 0) {
		return(cf->flash_vma_addr);
	}

	return(div * cf->FlashNblkChip * SECTOR_SIZE + cf->flash_vma_addr);
}

#ifdef CONFIG_TSxK
/*--------------------------------------------------------------------------
 *
 * sst39lf020_NumOfSubSector
 *
 * Returns the number of sub sectors inside a 64k flash block
 *
 *---------------------------------------------------------------------------*/
static int sst39lf020_NumOfSubSector(struct CycFlashCtr *cf, int sector)
{
	/* all 64k sector have the same structure */
	return(16);
}

/*--------------------------------------------------------------------------
 *
 * sst39lf020_SubSectorAddr
 *
 * Returns the sub sector address inside the 64k flash block
 *
 *---------------------------------------------------------------------------*/
static char *sst39lf020_SubSectorAddr(struct CycFlashCtr *cf,
					 int sector, int sub_sector)
{
	/* all sub-sector have 4k bytes */
	return(cf->flash_vma_addr + sector * SECTOR_SIZE + sub_sector * 0x1000);
}
#endif

/*--------------------------------------------------------------------------
 * This routine generates the 16 bits CRC (X^16+X^12+X^5+1) for a
 * stream of bytes.
 *
 *	byte_n	   number of bytes
 *	crc		  initial value of crc
 *	buffer	   first address of buffer
 *
 *---------------------------------------------------------------------------
 */

static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
	int j;

	while (byte_n--) {
		crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
		for(j = 0; j < 8; j++) {
			if (crc & 0x8000)  {
				crc <<= 1;
				crc ^= 0x1021;
			} else {
				crc <<= 1;
			}
		}
	}
	return(crc);
}

/*--------------------------------------------------------------------------
 *
 * unlock_sequence
 *
 * Provides 2 write cycles in order to "unlock" the flash
 *
 *---------------------------------------------------------------------------*/
static void unlock_sequence(struct CycFlashCtr *cf, volatile unsigned char *pflash)
{
	*(pflash + cf->cfg_addr1) = UNLOCK_VALUE_A;
	*(pflash + cf->cfg_addr2) = UNLOCK_VALUE_B;
}

/*--------------------------------------------------------------------------
 *
 * data_polling
 *
 * Provides the necessary command sequence demanded by the Flash (Am29f400AT)
 * in order to check if the command (program/erase) over an address given as
 * argument was completed.  Does the necessary switching on the window register
 * to guarantee that the interrupts (when enabled) will be pointing to I/O 
 * window space.
 *
 * Returns -	PASS - completed with succes
 *				FAIL
 *---------------------------------------------------------------------------*/

static unsigned char data_polling(volatile unsigned char *verify_addr, unsigned char data)
{
	unsigned char aux;
	int	flash_retries;

	for (flash_retries=1; flash_retries < 36000000; flash_retries++) {
		if (((*verify_addr) & DQ7) == (data & DQ7)) {
			return(PASS);
		}
#if defined(CONFIG_PR1K) || defined(CONFIG_TSxK)
		if (((bd_t *)res)->hw_info.board_type == BOARD_TS3K) {
			aux = ((*(verify_addr) & DQ5) == DQ5);
		} else {
			// PA1 - flash busy (PR1000, TS1000, TS2000, TS400, and TS800)
			aux = (((immap_t *)IMAP_ADDR)->im_ioport.iop_padat) & 0x4000;
		}
#else
		aux = ((*(verify_addr) & DQ5) == DQ5);
#endif
		if (aux) {
			/* flash ready*/
			if ((*(verify_addr) & DQ7) == (data & DQ7)) {
				return(PASS);
			} else {
				return(FAIL);
			}
		}
		if (!(flash_retries & 3)) {
			//current->state = TASK_INTERRUPTIBLE;
			schedule();
		}
	}
	return(FAIL);
}

/*--------------------------------------------------------------------------
 *
 * sector_erase_command_sequence
 *
 * Provides the necessary command sequence demanded by the Flash (?)
 * in order to erase a sector given as argument. Does the necessary switching
 * on the window register to guarantee that the interrupts (when enabled)
 * will be pointing to I/O window space.
 *
 *---------------------------------------------------------------------------*/
static int sector_erase_command_sequence(struct CycFlashCtr *cf, int sector)
{
	volatile unsigned char 	*pflash;
	volatile unsigned char 	*sector_addr;
	int	sub_sector, max_sector;

	if (cf->NumOfSubSector) {
		max_sector = (*cf->NumOfSubSector)(cf, sector);
	} else {
		max_sector = 1;
	}

	for (sub_sector=0; sub_sector < max_sector; sub_sector++) {
		/* spots the correct FLASH chip (1st, 2nd or 3rd) */
		pflash = (*cf->FlashChipAddr)(cf, sector);
		if (cf->SubSectorAddr && sub_sector) {
			sector_addr = (*cf->SubSectorAddr)(cf, sector, sub_sector);
		} else {
			sector_addr = cf->flash_vma_addr + sector * SECTOR_SIZE;
		}
		*(pflash + 0x0000) = RESET_READ_VALUE;
		unlock_sequence(cf, pflash);
		*(pflash + cf->cfg_addr1) = SETUP_VALUE;	/* setup command */
		unlock_sequence(cf, pflash);
		*(sector_addr) = SECTOR_ERASE_VALUE;

		/* waits for the sector erased confirmation */
		if (data_polling(sector_addr, DQ7) != PASS) {
			return(FAIL);
		}
		*(pflash + 0x0000) = RESET_READ_VALUE;
	}
	return(PASS);
}

#ifdef CONFIG_TSxK

static int do_cflash_read(struct CycFlashPartition *fp, int sector)
{
	if (!fp->buf) {
		if ((fp->buf = kmalloc(SECTOR_SIZE, GFP_KERNEL)) == NULL) {
			return(-ENOMEM);
		}
		fp->current_sector = -1;
	}

	if (!cf_read (fp->buf, sector * 128, SECTOR_SIZE)) {
		return(-EIO);
	} 
	fp->current_sector = sector;
	return(0);
}

static int do_cflash_file_size(struct CycFlashPartition *fp, int wr, int val)
{
	int buf[256];

	if (!(fp->crc_flag & 2)) {
		return(0);
	}

	if (!cf_read((char *)buf, lba_sector_sizes, 1024)) {
		return(0);
	}

	if (wr) {
		buf[fp->ix_word_size] = val;
		cf_write((char *)buf, lba_sector_sizes, 1024);
	}

	return(buf[fp->ix_word_size]);
}

static int do_cflash_write(struct CycFlashPartition *fp)
{
	if (!cf_write (fp->buf, fp->current_sector * 128, SECTOR_SIZE)) {
		return(-EIO);
	} 
	return(0);
}

#endif


#ifdef CONFIG_TS1H
static int do_cfiflash_write(struct CycFlashPartition *fp)
{
	return(cyc_cfiflash_grava(fp->flash_ctr, fp->buf, fp->current_sector));
}

static int do_cfiflash_erase_sector(struct CycFlashPartition *fp, int sector)
{
	return (cyc_cfiflash_erase(fp->flash_ctr, sector));
}

#endif

static int do_flash_write(struct CycFlashPartition *fp)
{
	return(cyc_flash_grava(fp->flash_ctr, fp->buf, fp->current_sector));
}

static int do_flash_erase_sector(struct CycFlashPartition *fp, int sector)
{
	if (sector_erase_command_sequence(fp->flash_ctr, sector) != PASS) {
		return(-EIO);
	}
	return(0);
}

static int do_flash_file_size(struct CycFlashPartition *fp, int wr, int val)
{
	unsigned char *p; 
	int i, j;

	if (wr) {
		return(0);
	}

	if (fp->crc_flag & 4) {
		return((fp->sector_end - fp->sector_start) * SECTOR_SIZE);
	}

	if (fp->crc_flag & 2) {
		i = (fp->sector_end - fp->sector_start) * SECTOR_SIZE -
			fp->initial_offset - fp->end_offset;
		p = fp->flash_ctr->flash_vma_addr +
				 fp->sector_end * SECTOR_SIZE - fp->end_offset;
		while ((i -= 4) >= 0 && *(int *)((char *)p -= 4) == -1);

		for (j=0; j < 4 && *p != 0xff; j++, p++, i++);

		if (i) --i;

		if (i >= 2 && fp->crc_flag & 1) i -= 2;

		if (i < 0) {
			return((fp->sector_end - fp->sector_start) * SECTOR_SIZE -
					fp->initial_offset - fp->end_offset);
		}
		return(i);
	}
	return(0);
}


static int cyc_flash_grava(struct CycFlashCtr *cf, char *buf, int sector)
{
	volatile unsigned char	*pflash, *pchip;
	char *p = buf;
	unsigned int	i;

	if (sector_erase_command_sequence(cf, sector) != PASS) {
		return(-EIO);
	}

	/* spots the correct FLASH chip (1st, 2nd or 3rd) */
	pchip = (*cf->FlashChipAddr)(cf, sector);
	pflash = cf->flash_vma_addr + sector * SECTOR_SIZE;

	for (i = 0; i < SECTOR_SIZE; i++, buf++, pflash++) {
		/* write one byte */
		unlock_sequence(cf, pchip);
		*(pchip + cf->cfg_addr1) = PROGRAM_VALUE; /* setup command */
		*pflash = *buf;

		/* waits until program sequence completion */
		if  (data_polling(pflash, *buf) != PASS) {
			/* something went wrong : return with fail status */
			return(-EIO);
		}
	}
	if (memcmp(cf->flash_vma_addr+sector*SECTOR_SIZE, p, SECTOR_SIZE)) {
		return(-EIO);
	}

	return (0);
}

#ifdef CONFIG_TS1H
extern int cfi_erase(short);
extern int cfi_program(char*, short);
extern unsigned char* TMPL_GetFptr(int);

static int cyc_cfiflash_grava(struct CycFlashCtr *cf, char *buf, int sector)
{
	volatile unsigned char	*pflash, *pchip;
	char *p = buf;
	unsigned int	i;

	//printk("FLASH erase... sector: %d\n", sector);
		
	if (cfi_erase((short)sector)) {
		return(-EIO);
	}

	//printk("FLASH program... address: %08x   sector_address: %08x\n",
	//	(int)FlashControl.flash_vma_addr, TMPL_GetFptr(sector * SECTOR_SIZE));

	if (cfi_program(buf, (short)sector)) {
		return(-EIO);
	}

	//printk("FLASH done... check memory\n");
	//printk("%08x: %08x %08x %08x %08x\n", cf->flash_vma_addr+(sector*SECTOR_SIZE), 
	//		* (int*)(cf->flash_vma_addr+(sector*SECTOR_SIZE)),
	//		* (int*)(cf->flash_vma_addr+(sector*SECTOR_SIZE) + 4),
	//		* (int*)(cf->flash_vma_addr+(sector*SECTOR_SIZE) + 8),
	//		* (int*)(cf->flash_vma_addr+(sector*SECTOR_SIZE) + 12));

	if (memcmp(cf->flash_vma_addr+(sector*SECTOR_SIZE), p, SECTOR_SIZE)) {
		return(-EIO);
	}

	//printk("FLASH check done\n");

	return (0);
}
static int cyc_cfiflash_erase(struct CycFlashCtr *cf, int sector)
{
	//printk("FLASH erase only... sector: %d\n", sector);

	if (cfi_erase((short)sector)) {
		return(-EIO);
	}

	return (0);
}
#endif


static int cyc_flash_open(struct inode * inode, struct file * filp)
{
	struct proc_dir_entry *dp;
	struct CycFlashPartition *fp;

	dp = (struct proc_dir_entry *) inode->u.generic_ip;
	fp = (struct CycFlashPartition *)dp->data;

	if (fp->flags) {
		return(-EIO);
	}
	
	if (check_shared_sector(fp, fp->sector_start, fp->sector_end)) {
		if (lock_shared_sector(fp->sector_start, fp->sector_end)) {
			return(-EIO);
		}
	}

	fp->flags = 1;
	fp->buf = NULL;
	fp->current_sector = -1;
	fp->crc_calc = 0;

	return 0;
}

static int cyc_flash_release(struct inode * inode, struct file * filp)
{
	struct proc_dir_entry *dp;
	struct CycFlashPartition *fp;
	unsigned char	buf[4];
	int i, i_max;
	struct CycFlashCtr *cf;

	dp = (struct proc_dir_entry *) inode->u.generic_ip;
	fp = (struct CycFlashPartition *)dp->data;

	if (!fp->flags) {
		return(-EIO);
	}

	cf = fp->flash_ctr;

//	if ((fp->crc_flag & 4)) {
//		goto clean_up1;
//	}

	if (!(fp->flags & 4)) {
		goto clean_up;
	}

	while (test_and_set_bit(0, &cf->flash_locked)) {
		schedule();
	}

	if (fp->flags & 2) {
		dp->size = filp->f_pos;
		inode->i_size = dp->size;
		if (fp->crc_flag & 1) {
			buf[0] = fp->crc_calc / 256;
			buf[1] = fp->crc_calc % 256;
			buf[2] = 0x7e;
			fill_sector_flash(
				fp->sector_start+(filp->f_pos+fp->initial_offset)/SECTOR_SIZE,
				(filp->f_pos+fp->initial_offset)%SECTOR_SIZE, buf, 2, fp, 1);
			filp->f_pos += 2;
		}
		if (fp->crc_flag & 2) {
			buf[0] = 0x7e;
			fill_sector_flash(
				fp->sector_start+(filp->f_pos+fp->initial_offset)/SECTOR_SIZE,
				(filp->f_pos+fp->initial_offset)%SECTOR_SIZE, buf, 1, fp, 1);
			filp->f_pos++;

			i = (filp->f_pos + fp->initial_offset) % SECTOR_SIZE;
			i_max = fp->current_sector+1 != fp->sector_end ? SECTOR_SIZE :
				SECTOR_SIZE - fp->end_offset;
			for ( ; i < i_max; ) fp->buf[i++] = 0xff;

			(*cf->FlashWrite)(fp);
		
			if (fp->flash_ctr->FlashFileSize) {
				(*fp->flash_ctr->FlashFileSize)(fp, 1, dp->size);
			}

			if (cf->EraseSector) {
				for (i=fp->current_sector+1; i < fp->sector_end; i++) {
					(*cf->EraseSector)(fp, i);
				}
			}
		} else {
			(*cf->FlashWrite)(fp);
		}
	}

clean_up:
	unlock_shared_sector(fp->sector_start, fp->sector_end);

clean_up1:
	if (fp->buf) {
		kfree(fp->buf);
	}

	fp->flags = 0;
	fp->buf = NULL;
	fp->current_sector = -1;
	clear_bit(0, &cf->flash_locked); 
	return 0;
}

static ssize_t cyc_flash_read(struct file * filp, char * buf,
			size_t count, loff_t *ppos)
{
	unsigned char *p;
	struct inode *inode;
	struct proc_dir_entry *dp;
	struct CycFlashPartition *fp;
	int offset, max_ppos, block, ix_block, count_aux, ret;
	int save_count = 0, read_count = 0;

	inode = filp->f_dentry->d_inode;
	dp = (struct proc_dir_entry *) inode->u.generic_ip;
	fp = (struct CycFlashPartition *)dp->data;
	offset = fp->sector_start * SECTOR_SIZE + fp->initial_offset;

	if (fp->crc_flag & 2) {
		max_ppos = dp->size;
	} else {
		max_ppos = fp->sector_end * SECTOR_SIZE - fp->end_offset - offset;
	}
	if ((max_ppos -= *ppos) <= 0) {
		return(0);
	}

	if (count > max_ppos) {
		count = max_ppos;
	}

get_from_mem:

	block = (offset + *ppos)/SECTOR_SIZE;

	if (fp->flash_ctr->flash_type != CARD_FLASH) {
		if (!fp->buf || block != fp->current_sector) {
			if ((ret = (*fp->flash_ctr->FlashRead)(fp, block))) {
				return (ret);
			}
		}
	}

	if (fp->buf && block == fp->current_sector) {
		ix_block =  *ppos % SECTOR_SIZE;
		count_aux = SECTOR_SIZE - ix_block;
		if (count_aux > count) {
			count_aux = count;
		}
		copy_to_user(buf, fp->buf+ix_block, count_aux);
		read_count += count_aux;
		*ppos += count_aux;
		if ((count -= count_aux) == 0) {
			return(read_count);
		}
		buf += count_aux;
	}

	if (fp->flash_ctr->flash_type != CARD_FLASH) {
		goto get_from_mem;
	}

	if (fp->buf &&
		block < fp->current_sector &&
		fp->current_sector <= (offset + *ppos + count)/SECTOR_SIZE) {
		save_count = count;
		count = SECTOR_SIZE * fp->current_sector - *ppos;
		save_count -= count;
	}

	p = fp->flash_ctr->flash_vma_addr + *ppos + offset;
	copy_to_user(buf, p, count);
	*ppos += count;
	read_count += count;

	if (save_count) {
		count = save_count;
		save_count = 0;
		goto get_from_mem;
	}

	return(read_count);
}

static int fill_sector_flash(int sector, int pos, char *buf, int count, struct CycFlashPartition *fp, int flag)
{
	int result;
	struct CycFlashCtr *cf = fp->flash_ctr;

	if (fp->buf && sector != fp->current_sector && (fp->flags & 2)) {
		fp->flags &= ~2;
		if ((result = (*cf->FlashWrite)(fp))) {
			return(result);
		}
	}

	if (!fp->buf) {
		if ((fp->buf = kmalloc(SECTOR_SIZE, GFP_KERNEL)) == NULL) {
			return(-ENOMEM);
		}
		fp->current_sector = -1;
	}

	if (sector != fp->current_sector) {
		if (check_shared_sector(fp, sector, sector)) {
			if (fp->flash_ctr->flash_type != CARD_FLASH) {
				if ((result = (*fp->flash_ctr->FlashRead)(fp, sector))) {
					return(result);
				}
			} else {
				memcpy(fp->buf, cf->flash_vma_addr + sector * SECTOR_SIZE, SECTOR_SIZE);
			}
		}
	}

	if (count + pos > SECTOR_SIZE) {
		count = SECTOR_SIZE - pos;
	}

	if (flag) {
		memcpy(fp->buf+pos, buf, count);
	} else {
		copy_from_user(fp->buf+pos, buf, count);
	}

	fp->current_sector = sector;
	fp->flags = 7;
	return(count);
}

static ssize_t cyc_flash_write(struct file * filp, const char * buf,
			size_t count, loff_t *ppos)
{
	char * p = buf;
	struct inode *inode;
	struct proc_dir_entry *dp;
	struct CycFlashPartition *fp;
	int sector, max_ppos, count_aux;

	inode = filp->f_dentry->d_inode;
	dp = (struct proc_dir_entry *) inode->u.generic_ip;
	fp = (struct CycFlashPartition *)dp->data;

//	if ((fp->crc_flag & 4)) {
//		return(-EIO);
//	}

	max_ppos = (fp->sector_end - fp->sector_start) * SECTOR_SIZE -
				fp->initial_offset - fp->end_offset;

	if ((max_ppos -= *ppos) <= 0) {
		return(-EINVAL);
	}

	if (count > max_ppos) {
		count = max_ppos;
	}

	max_ppos = 0;

	while (test_and_set_bit(0, &fp->flash_ctr->flash_locked)) {
		schedule();
	}

	while (count > 0) {
		sector = fp->sector_start + (*ppos+fp->initial_offset) / SECTOR_SIZE;
		if ((count_aux = fill_sector_flash(sector, (*ppos+fp->initial_offset) % SECTOR_SIZE, p, count, fp, 0)) <= 0) {
			clear_bit(0, &fp->flash_ctr->flash_locked); 
			return(count_aux);
		}
		if (fp->crc_flag & 1) {
			fp->crc_calc = calc_crc(count_aux, fp->crc_calc, p);
		}
		count -= count_aux;
		*ppos += count_aux;
		p += count_aux;
		max_ppos += count_aux;
	}

	clear_bit(0, &fp->flash_ctr->flash_locked); 
	return(max_ppos);
}

static void cyc_flash_proc_register(struct proc_dir_entry *proc_entry)
{
	struct CycFlashPartition *fp = (struct CycFlashPartition *)proc_entry->data;

	proc_register(&proc_root_flash, proc_entry);

	fp->current_sector = -1;

	if (fp->flash_ctr->FlashFileSize) {
		proc_entry->size = (*fp->flash_ctr->FlashFileSize)(fp, 0, 0);
	} else {
		proc_entry->size = 0;
	}

	printk("Flash partition:%s size: %d\n", proc_entry->name, (fp->sector_end - fp->sector_start) * SECTOR_SIZE - fp->initial_offset - fp->end_offset);
}

static int SetFlashSectors(struct CycFlashPartition *pf, int st, int sz,
			int init_offset, int end_offset, struct CycFlashCtr *cf)
{
	sz += st;

	if ((st < 0 || st >= cf->FlashNblkTotal) ||
		(sz < 0 || sz >  cf->FlashNblkTotal)) {
		printk("Flash partition invalid sector range %d - %d\n", st, sz);
		return(-1);
	}

	pf->flash_ctr = cf;

	pf->sector_start = st;
	pf->sector_end   = sz;
	pf->initial_offset = init_offset;
	pf->end_offset = end_offset;

	return(sz);
}

/*
 * This is the registration and initialization section of
 * the FLASH disk driver
 */

__initfunc(int cyc_flash_init(void))
{
	bd_t	*bd = (bd_t *)res;
	int i;

	FlashControl.FlashWrite	 = do_flash_write;
	FlashControl.EraseSector	= do_flash_erase_sector;
	FlashControl.FlashFileSize  = do_flash_file_size;

#ifdef CONFIG_PR3K
	/* Flash AM29f016B do not have sub-sectors */
	FlashControl.flash_type = CARD_FLASH;
	FlashControl.FlashNblkChip = 32;	/* number of 64k block per chip */
	FlashControl.FlashChipAddr = FlashChipAddr;

	FlashControl.cfg_addr1 = CFG_ADDR;		// 0x5555
	FlashControl.cfg_addr2 = CFG_ADDR >> 1; // 0x2AAA
#endif

#ifdef CONFIG_TSxK
	if (bd->hw_info.board_type == BOARD_TS3K) {

		if (cf_detect()) {
			cf_print_info();
			CF_Control.flash_type	  = CARD_IDE;
			CF_Control.flash_length   = cf_get_size();
			CF_Control.FlashNblkTotal = CF_Control.flash_length / SECTOR_SIZE;
			/* number of 64k block per chip */
			CF_Control.FlashNblkChip  = CF_Control.FlashNblkTotal;
			CF_Control.flash_vma_addr = cflash_addr;
			CF_Control.FlashWrite	  = do_cflash_write;
			CF_Control.FlashRead	  = do_cflash_read;
			//CF_Control.EraseSector	= do_cflash_erase_sector;
			CF_Control.FlashFileSize  = do_cflash_file_size;
		} else {
			printk("Could not detect CF+\n");
		}

		FlashControl.flash_type = CARD_FLASH;
		FlashControl.FlashNblkChip = 4;	/* number of 64k block per chip */

		FlashControl.NumOfSubSector = sst39lf020_NumOfSubSector;
		FlashControl.SubSectorAddr = sst39lf020_SubSectorAddr;
		FlashControl.FlashChipAddr = FlashChipAddr;

		FlashControl.cfg_addr1 = CFG_ADDR;		// 0x5555
		FlashControl.cfg_addr2 = CFG_ADDR >> 1; // 0x2AAA
	} else {
		FlashControl.flash_type = CARD_FLASH;
		FlashControl.FlashNblkChip = 32;	/* number of 64k block per chip */

		FlashControl.NumOfSubSector = at49bv1614_NumOfSubSector;
		FlashControl.SubSectorAddr = at49bv1614_SubSectorAddr;
		FlashControl.FlashChipAddr = FlashChipAddr;

		FlashControl.cfg_addr2 = CFG_ADDR;		// 0x5555
		FlashControl.cfg_addr1 = CFG_ADDR << 1; // 0xAAAA
	}
#endif

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	FlashControl.flash_type = CARD_FLASH;

#if defined(CONFIG_SM100)
	// PC8 - chip 2 or 4 mega (32 or 64 blocks of 64k per chip)
	FlashControl.FlashNblkChip = bd->hw_info.flash_size /
		bd->hw_info.flash_chips / 0x10000;
#else
	FlashControl.FlashNblkChip = bd->hw_info.flash_size / SECTOR_SIZE;
#endif

	FlashControl.NumOfSubSector = at49bv_321_161_NumOfSubSector;
	FlashControl.SubSectorAddr = at49bv_321_161_SubSectorAddr;
	FlashControl.FlashChipAddr = FlashChipAddr;

	FlashControl.cfg_addr2 = CFG_ADDR;			// 0x5555
	FlashControl.cfg_addr1 = CFG_ADDR << 1;		// 0xAAAA
#endif

#ifdef CONFIG_PR1K
	FlashControl.flash_type = CARD_FLASH;
	FlashControl.FlashNblkChip = 32;	/* # of 64k block per chip */
	FlashControl.NumOfSubSector = at49bv1614_NumOfSubSector;
	FlashControl.SubSectorAddr = at49bv1614_SubSectorAddr;
	FlashControl.FlashChipAddr = FlashChipAddr;

	FlashControl.cfg_addr2 = CFG_ADDR;			// 0x5555
	FlashControl.cfg_addr1 = CFG_ADDR << 1;		// 0xAAAA
#endif

	FlashControl.flash_length = bd->hw_info.flash_size;
	FlashControl.FlashNblkTotal = FlashControl.flash_length / SECTOR_SIZE;
	FlashControl.flash_vma_addr = flash_addr;

	switch (bd->hw_info.board_type) {
		case BOARD_TS3K:
		case BOARD_TS110:
		case BOARD_SM100:
			i = SetFlashSectors(&cyc_flash_ori_partition, 0, 2, 0, 0, &FlashControl);
			i = SetFlashSectors(&cyc_flash_alt_partition, i, 1, 0, 0, &FlashControl);
			break;
		case BOARD_TS1H:
			if (bd->hw_info.board_version == 2) {	// HW v2.0 (new board with more flash)
#ifdef CONFIG_TS1H
				FlashControl.FlashWrite	 = do_cfiflash_write;
				FlashControl.EraseSector = do_cfiflash_erase_sector;
#endif
				i = SetFlashSectors(&cyc_flash_ori_partition, 0, 1, 0, 0, &FlashControl);
				add_shared_sector(i);
				i = SetFlashSectors(&cyc_flash_alt_partition, i, 1, 0, (SECTOR_SIZE/2), &FlashControl);
			}
			else {	// HW v1.0
				i = SetFlashSectors(&cyc_flash_ori_partition, 0, 2, 0, 0, &FlashControl);
				i = SetFlashSectors(&cyc_flash_alt_partition, i, 1, 0, 0, &FlashControl);
			}
			printk("CFI FLASH detected.\n");
			break;
		default:
			i = SetFlashSectors(&cyc_flash_ori_partition, 0, 1, 0, 0, &FlashControl);
			i = SetFlashSectors(&cyc_flash_alt_partition, i, 1, 0, 0, &FlashControl);
			break;		
	}
	FlashControl.flash_vma_addr = flash_addr;

	if (bd->hw_info.board_type != BOARD_TS3K) {
		/* WARNING config and script will share the current sector */
		if ((bd->hw_info.board_type == BOARD_TS1H) && (bd->hw_info.board_version == 2)) {
			i = SetFlashSectors(&cyc_flash_cnf_partition, i-1, 1, (SECTOR_SIZE/2), (SECTOR_SIZE/2)-CONFIG_FILE_SIZE, &FlashControl);
			i = SetFlashSectors(&cyc_flash_scr_partition, i-1, 2, (SECTOR_SIZE/2)+CONFIG_FILE_SIZE, 0, &FlashControl);
		} 
		else {
			add_shared_sector(i);
			i = SetFlashSectors(&cyc_flash_cnf_partition, i, 1, 0, SECTOR_SIZE-CONFIG_FILE_SIZE, &FlashControl);
			i = SetFlashSectors(&cyc_flash_scr_partition, i-1, 2, CONFIG_FILE_SIZE, 0, &FlashControl);
		}
#ifdef BUILD_CORE
		SetFlashSectors(&cyc_flash_core_partition, i,
			FlashControl.FlashNblkTotal-i, 0, 0, &FlashControl);
#endif
		i = SetFlashSectors(&cyc_flash_ope_partition, i,
			FlashControl.FlashNblkTotal-i, 0, 0, &FlashControl);
	}
#ifdef CONFIG_TSxK
	else {
		i = SetFlashSectors(&cyc_flash_cnf_partition, i, 1, 0, 0, &FlashControl);
		cyc_flash_ope_partition.ix_word_size = 0;
		cyc_flash_scr_partition.ix_word_size = 1;
		i = SetFlashSectors(&cyc_flash_ope_partition, 0,
			CF_Control.FlashNblkTotal - i - 5, 0, 0, &CF_Control);
		lba_sector_sizes = i * 128;
		i = SetFlashSectors(&cyc_flash_scr_partition, i+1, 4, 0, 0, &CF_Control);
	}
#endif

	printk("FLASH disk driver initialized:  VMA %08x size %dkb blocks %d\n",
		(int)FlashControl.flash_vma_addr,
		FlashControl.flash_length/1024, FlashControl.FlashNblkTotal);

	proc_register(&proc_root, &proc_root_flash);

	cyc_flash_proc_register(&proc_flash_config);
	cyc_flash_proc_register(&proc_flash_boot_ori);
	cyc_flash_proc_register(&proc_flash_boot_alt);
	cyc_flash_proc_register(&proc_flash_scr);
	cyc_flash_proc_register(&proc_flash_zImage);
#ifdef BUILD_CORE
	if (bd->hw_info.board_type != BOARD_TS3K) {
		cyc_flash_proc_register(&proc_flash_core);
	}
#endif

	return 0;
}

#ifdef CONFIG_TSxK

/*-----------------------------------------------------------------------------
 *
 * Copyright (C) Cyclades Corporation, 1997 - 2000
 * All rights reserved.
 *
 * TS3000
 *
 * This file : 		cf_ata.c
 *
 * Description: 	This file contains the routines to access compact flash
 *
 * Complies with Cyclades SW coding Standard rev 1.2
 *-----------------------------------------------------------------------------
 */

 /*----------------------------------------------------------------------------
  * Change History
  *----------------------------------------------------------------------------
  * 08/8/01  Arnaldo Zimmermann		Initial implementation
  *----------------------------------------------------------------------------
  */

/* structure */
typedef struct sCF {
	unsigned short data;
	unsigned char  dummy1;
	unsigned char  error_and_features;
	unsigned char  dummy2;
	unsigned char  sec_cnt;
	unsigned char  dummy3;
	unsigned char  sec_num;
	unsigned char  dummy4;
	unsigned char  cyl_low;
	unsigned char  dummy5;
	unsigned char  cyl_high;
	unsigned char  dummy6;
	unsigned char  sel_card_head;
	unsigned char  dummy7;
	unsigned char  status_and_command;
	unsigned char  dummy8[0xd];
	unsigned char  alt_status_and_dev_ctl;
	unsigned char  dummy9;
	unsigned char  drv_card_addr;
} CF;

/* static variables */
static volatile CF *cf;
static char dummy_status;
static unsigned short cf_buf[256];
static int max_lba_addr;

/* CF Status and Alternate Status Register */
#define CF_STATUS_BUSY		0x80
#define CF_STATUS_RDY		0x40
#define CF_STATUS_DSC		0x10
#define CF_STATUS_DRQ		0x08
#define CF_STATUS_ERROR		0x01

/* CF Device Control Register */
#define CF_INT				0x02
#define CF_INT_ENABLE		0x00

/* CF Drive/Head Register */
#define CF_LBA				0x40

/* FPGA MISC Bit Map */
#define CF_CARD_DETECT_2	0x80
#define CF_CARD_DETECT_1	0x40
#define CF_RESET			0x02
#define CF_MISC_INT			0x04

#define CF_NLOOP			100000		
#define CF_DELAY			100		/* in usec */
#define CF_CMD_DELAY		30000	/* in usec */	
#define CF_CLASS_MASK		0xff00
#define CF_CMD_MASK			0x00ff
#define CF_CLASS_1			0x0100
#define CF_CLASS_2			0x0200
#define CF_CLASS_3			0x0300
#define CF_SIGNATURE		0x848a	/* CF general configuration signature */
#define IDE_SIGNATURE	   0x45A	/* IDE HD general configuration signature */
#define CF_CLS				0x1		/* clear status register */
#define CF_NO_CLS			0x0		/* no clear status register */
#define CF_LBA_MASK			0x200	/* LBA capability support */

/* CF Class and Code of Command Set */
#define CF_CMD_CHECK_POWER_MODE		(CF_CLASS_1 | 0x98)		/* check power mode */
#define CF_CMD_EXE_DRV_DIAG			(CF_CLASS_1 | 0x90)		/* execute driver diagnostic */
#define CF_CMD_ID_DRV				(CF_CLASS_1 | 0xEC)		/* identify driver */
#define CF_CMD_RD_SECTOR			(CF_CLASS_1 | 0x20)		/* read sectors */
#define CF_CMD_WR_SECTOR			(CF_CLASS_2 | 0x30)		/* write sectors */
#define CF_IDLE						(CF_CLASS_1 | 0xE3)		/* put CF in idle mode */
#define CF_CMD_ERASE_SECTOR			(CF_CLASS_1 | 0xC0)		/* erase sectors */
#define CF_CMD_RDM_SECTOR			(CF_CLASS_1 | 0xC4)		/* read multiple sectors */
#define CF_CMD_MULTIPLE_MODE		(CF_CLASS_1 | 0xC6)		/* multiple mode */
#define CF_CMD_WR_ERASED_SECTOR		(CF_CLASS_2 | 0x38)		/* write sectors w/o erase */


/* CF Command Response */
#define CF_ST_POWER_MODE_OK			0xFF	/* status power mode ok */
#define CF_ST_NO_ERR_DET			0x01	/* status no error detected */

/* CF Identify Driver Mask */
#define CF_GEN_INFO					0		/* general info signature, 2 bytes */
#define CF_VENDOR_ID				9		/* vendor ID, 2 bytes */
#define CF_SN						10		/* serial number, 20 bytes */
#define CF_MODEL_NUM				27		/* model number, 40 bytes */
#define CF_CAPABILITIES				49		/* capabilities */
#define CF_MAX_LBA					60		/* max num of sectors in LBA mode, 4 bytes */

/* CF Power mode */
#define CF_POWER_DOWN_MODE_DIS		0		/* power down mode disable */

/*************************************************************************
 * get_dec_timer - gets the decrementer timer register value
 *************************************************************************/

static int get_dec_timer(void)
{
	asm(" mfspr	3,22");
}

#define INITIAL_VALUE   0x270F

static int flag_ide_init = 1;

#ifdef CF_IDE_INT
static int ide_intr_pend = 0;

static void cf_int_handler(int irq, void *dev_id, struct pt_regs * regs)
{
	//printk("I got a IDE intr !!!\n");
	ide_intr_pend = 0;
	dummy_status = cf->status_and_command;	/* clear status */
}
#endif

static int calc_delay(int start_real_time, int start_dec_time)
{
	int end_dec_time, delta, delta_ticks;

	end_dec_time = get_dec_timer();
	delta_ticks = jiffies - start_real_time;
	if (delta_ticks == 0) {
		delta = start_dec_time - end_dec_time;
	} else {
		delta_ticks--;
		delta = start_dec_time + INITIAL_VALUE - end_dec_time;
	}
	if (delta < 0) {
		delta = 0;
	}
	return(delta_ticks * (INITIAL_VALUE + 1) + delta);
}

/*-----------------------------------------------------------------------------
 *	delay_us()
 *-----------------------------------------------------------------------------
 *	Description: this routine performs a delay in microseconds.
 *-----------------------------------------------------------------------------
 */
static void delay_us (int usec)
{
	int delay, start_real_time, start_dec_time;

	start_real_time = jiffies;
	start_dec_time = get_dec_timer();
	delay = 0;
	while (delay < usec){
		if (!flag_ide_init) {
			schedule();
		}
		delay = calc_delay(start_real_time, start_dec_time);
	}
}

static int cf_check_status(int usec, unsigned char bits, int flag)
{
	int delay, start_real_time, start_dec_time;

	start_real_time = jiffies;
	start_dec_time = get_dec_timer();
	delay = 0;
	while (delay < usec){
		if (flag) {
			if  ((cf->status_and_command & bits)) {
				return(1);
			}
		} else {
			if  (!(cf->status_and_command & bits)) {
				return(1);
			}
		}
		if (!flag_ide_init) {
			schedule();
		}
		delay = calc_delay(start_real_time, start_dec_time);
	}
	cf_led_off();
	return 0;
}

/*-----------------------------------------------------------------------------
 *	cf_send_cmd()
 *-----------------------------------------------------------------------------
 *	Description: this routine sends command to compact flash and return only 
 *			   when command is done.
 *-----------------------------------------------------------------------------
 */
static int cf_send_cmd(int command)
{
	if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
		/* timeout waiting CF get ready */
		return 0;
	}

#ifdef CF_IDE_INT
	ide_intr_pend = 1;
#endif

	cf->status_and_command = (command & CF_CMD_MASK);

	if ((command & CF_CLASS_MASK) > CF_CLASS_2) {
		delay_us(CF_CMD_DELAY);
	}
	else {
		delay_us(CF_DELAY);
	}

	if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
		/* timeout waiting CF get ready */
		return 0;
	}

#ifdef CF_IDE_INT
	if (ide_intr_pend) {
		printk("Ide interrupt pending !!!\n");
	}
#endif

	/* clear interrupt pending status */
	if (cf->status_and_command & CF_STATUS_ERROR) {
		return 0;
	}	
	return 1;
}

/*-----------------------------------------------------------------------------
 *	cf_detect()
 *-----------------------------------------------------------------------------
 *	Description: this routine detects and program compact flash 
 *-----------------------------------------------------------------------------
 */
static int cf_detect (void) 
{
	int loop, i;
	int warm_up;
	volatile cpm8xx_t *pcpm = &(((immap_t *)IMAP_ADDR)->im_cpm);

	flag_ide_init = 1;
	max_lba_addr = 0;

	pcpm->cp_pbpar &= ~(0x0002);
	pcpm->cp_pbdir |= 0x0002;

	cf = (CF *)cflash_addr;

	cf_led_on();

	/* activate Orange Light for HD/CF */
	cpld->cpld_misc |= 0x08;

	if (cpld->cpld_misc & (CF_CARD_DETECT_1|CF_CARD_DETECT_2)) {
		/* IDE HD present */
		warm_up = 1000000;			/* 1s warm up delay */
	}
	else {
		/* compact flash present */
		warm_up = 200000;			/*  200ms warm up delay */
	}
	
	cpld->cpld_misc |= CF_RESET;	/* compact flash reset */
	delay_us(500000);				/* 500ms */
	cpld->cpld_misc &= ~CF_RESET;	/* compact flash released */
	delay_us(warm_up);

#ifdef CF_IDE_INT
	if (!request_irq(SIU_IRQ3, cf_int_handler, SA_INT_LEVEL, "IDE", NULL)) {
		cf->alt_status_and_dev_ctl = CF_INT_ENABLE;	/* enable CF interrupt */
		cpld->cpld_misc |= CF_MISC_INT;
	}
#endif

	loop = CF_NLOOP;
	while ((cf->alt_status_and_dev_ctl & (CF_STATUS_BUSY|CF_STATUS_DSC|CF_STATUS_RDY)) != (CF_STATUS_DSC|CF_STATUS_RDY)) {
		loop--;
		if (!loop) {
			/* timeout waiting CF get ready */
			cf_led_off();
			return 0;
		}
		delay_us(CF_DELAY);
	}
	dummy_status = cf->status_and_command;	/* clear status */

	/* all following access are for Driver #0 */

	cf->sec_cnt = CF_POWER_DOWN_MODE_DIS;
	/* set power mode idle */
	if (!cf_send_cmd(CF_IDLE)) {
		cf_led_off();
		return 0;						/* command failed */
	}

	/* check power mode */
	if (!cf_send_cmd(CF_CMD_CHECK_POWER_MODE)) {
		cf_led_off();
		return 0;						/* command failed */
	}
	if (cf->sec_cnt != CF_ST_POWER_MODE_OK) {
		cf_led_off();
		return 0;						/* power mode not ok */
	}

	/* execute drive diagnostic */
	if (!cf_send_cmd(CF_CMD_EXE_DRV_DIAG)) {
		cf_led_off();
		return 0;						/* command failed */
	}
	if (cf->error_and_features != CF_ST_NO_ERR_DET) {
		cf_led_off();
		return 0;						/* diagnostic not ok */
	}

	/* identify drive */
	if (!cf_send_cmd(CF_CMD_ID_DRV)) {
		cf_led_off();
		return 0;						/* command failed */
	}
	for (i=0; i<256; i++) {
		cf_buf[i] = cf->data;			/* read identification */
	}

	if (!(cf_buf[CF_CAPABILITIES] & CF_LBA_MASK)) {
		cf_led_off();
		return 0; 						/* no LBA capabilities */
	}

	max_lba_addr = cf_buf[CF_MAX_LBA] + (cf_buf[CF_MAX_LBA+1] << 16);

	cf_led_off();
	flag_ide_init = 0;
	return 1;
}

/*-----------------------------------------------------------------------------
 *	cf_read()
 *-----------------------------------------------------------------------------
 *	Description: this routine performs a read. Buffer address must have  
 *				 alignment 2 and size must be multiple of 512 bytes.
 *-----------------------------------------------------------------------------
 */
static int cf_read (unsigned char *buf, int lba_addr, int size) 
{
	int fifo_size, n_sectors, m_sectors;
	unsigned short *buffer;

	if ((size % 512)  || (((int)buf) % 2)) {
		return 0;
	}
	
	cf_led_on();
	
	/* set variables to word mode */
	buffer = (unsigned short*) buf;
	n_sectors = size / 512;

	if ((n_sectors + lba_addr) > max_lba_addr) {
		cf_led_off();
		return 0;				/* LBA address overflow */
	} 

	while (n_sectors) {

		if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
			/* timeout waiting CF get ready */
			return(0);
		}

		/* set LBA address and LBA mode */
		cf->sec_num = lba_addr & 0xff;
		cf->cyl_low = (lba_addr >> 8) & 0xff;
		cf->cyl_high = (lba_addr >> 16) & 0xff;
		cf->sel_card_head &= ~0x0f;			/* erase HS3-HS0 */
		cf->sel_card_head |= (((lba_addr >> 24) & 0x0f) | CF_LBA);

		/* read sectors */
		if (n_sectors >= 256) {
			m_sectors = 256;
			cf->sec_cnt = 0;
		}
		else {
			m_sectors = n_sectors;
			cf->sec_cnt = n_sectors;
		}
	
		/* send command to read */
		if (!cf_send_cmd(CF_CMD_RD_SECTOR)) {
			cf_led_off();
			return 0; 			/* command failed */
		}

		/* next loop */
		lba_addr += m_sectors;
		n_sectors -= m_sectors;

		while (m_sectors--) {

			/* wait for NOT BUSY */
			if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_DRQ, 1)) {
				/* timeout waiting CF get ready */
				return 0;
			}

			/* read 256 words */
			fifo_size = 256;
			while (fifo_size--) {
				*buffer++ = cf->data;			
			}

			delay_us(CF_DELAY);
		}
	}
	cf_led_off();
	return 1;
}

/*-----------------------------------------------------------------------------
 *  cf_write()
 *-----------------------------------------------------------------------------
 *  Description: this routine performs a write. Buffer address must have
 *			   alignment 2 and size must be multiple of 512 bytes.
 *-----------------------------------------------------------------------------
 */
static int cf_write (unsigned char *buf, int lba_addr, int size) 
{
	int fifo_size, n_sectors, m_sectors, lba, cmd;
	unsigned short *buffer;

	if ((size % 512)  || (((int)buf) % 2)) {
		return 0;
	}

	cf_led_on();

	/* set variables */
	buffer = (unsigned short*) buf;
	n_sectors = size / 512;
	lba = lba_addr;

	/* check if there is room */
	if ((n_sectors + lba_addr) > max_lba_addr) {
		cf_led_off();
		return 0;				/* LBA address overflow */
	} 

	if (cf_buf[CF_GEN_INFO] == CF_SIGNATURE) {

		if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
			/* timeout waiting CF get ready */
			return(0);
		}

		/* compact flash device */
		while (n_sectors) {
			/* set LBA address and LBA mode */
			cf->sec_num = (lba & 0xff);
			cf->cyl_low = (lba >> 8) & 0xff;
			cf->cyl_high = (lba >> 16) & 0xff;
			cf->sel_card_head &= ~0x0f;			/* erase HS3-HS0 */
			cf->sel_card_head |= (((lba >> 24) & 0x0f) | CF_LBA);

			/* erase multiples sectors */
			if (n_sectors >= 256) {
				m_sectors = 256;
				cf->sec_cnt = 0;
			}
			else {
				m_sectors = n_sectors;
				cf->sec_cnt = n_sectors;
			}
	
			/* send command to erase */
			if (!cf_send_cmd(CF_CMD_ERASE_SECTOR)) {
				cf_led_off();
				return 0; 			/* command failed */
			}

			/* next loop */
			lba += m_sectors;
			n_sectors -= m_sectors;
		}
		cmd = CF_CMD_WR_ERASED_SECTOR;
	}
	else {
		/* HD device */
		cmd = CF_CMD_WR_SECTOR;
	}
	
	n_sectors = size / 512;
	while (n_sectors) {

		if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
			/* timeout waiting CF get ready */
			return(0);
		}

		/* set LBA address and LBA mode */
		cf->sec_num = (lba_addr & 0xff);
		cf->cyl_low = (lba_addr >> 8) & 0xff;
		cf->cyl_high = (lba_addr >> 16) & 0xff;
		cf->sel_card_head &= ~0x0f;			/* erase HS3-HS0 */
		cf->sel_card_head |= (((lba_addr >> 24) & 0x0f) | CF_LBA);

		/* write multiples sectors */
		if (n_sectors >= 256) {
			m_sectors = 256;
			cf->sec_cnt = 0;
		}
		else {
			m_sectors = n_sectors;
			cf->sec_cnt = n_sectors;
		}
	
		/* send command to write */
		if (!cf_send_cmd(cmd)) {
			cf_led_off();
			return 0; 			/* command failed */
		}

		/* next loop */
		lba_addr += m_sectors;
		n_sectors -= m_sectors;

		while (m_sectors--) {

			/* wait for DRQ */
			if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_DRQ, 1)) {
				/* timeout waiting CF get ready */
				return 0;
			}

			/* write 256 words */
			fifo_size = 256;
			while (fifo_size--) {
				cf->data = *buffer++;			
			}

			/* wait for writing data on device */
			delay_us(CF_DELAY);
			if (!cf_check_status(CF_NLOOP * CF_DELAY, CF_STATUS_BUSY, 0)) {
				/* timeout waiting CF get ready */
				return(0);
			}
		}
		dummy_status = cf->status_and_command;
	}
	cf_led_off();
	return 1;
}
/*-----------------------------------------------------------------------------
 *	cf_print_info()
 *-----------------------------------------------------------------------------
 *	Description: this routine prints the string with some Compact Flash info
 *-----------------------------------------------------------------------------
 */
static void cf_print_info (void) 
{
	unsigned int aux;
	char *cdev;

	aux = (max_lba_addr * 8) / 15625; // 512 / 1000000

	if (cf_buf[CF_GEN_INFO] == CF_SIGNATURE) {
		cdev = "Compact Flash"; 
	} else {
		cdev = "IDE HD";
	}
	printk("IDE: %s %s Size %u Mbytes\n", cdev, (char *)&cf_buf[CF_MODEL_NUM], aux);
}

/*-----------------------------------------------------------------------------
 *	cf_get_size()
 *-----------------------------------------------------------------------------
 *	Description: this routine returns the Compact Flash size in bytes
 *-----------------------------------------------------------------------------
 */
int cf_get_size(void)
{
	return(max_lba_addr * 256 * 2);
}

/*-----------------------------------------------------------------------------
 *	cf_led_on()
 *-----------------------------------------------------------------------------
 *	Description: this routine turns HD/CF LED on
 *-----------------------------------------------------------------------------
 */
static inline void cf_led_on (void) 
{
	volatile cpm8xx_t *pcpm = &(((immap_t *)IMAP_ADDR)->im_cpm);

	pcpm->cp_pbdat |= 0x0002;
}

/*-----------------------------------------------------------------------------
 *	cf_led_off()
 *-----------------------------------------------------------------------------
 *	Description: this routine turns HD/CF LED off
 *-----------------------------------------------------------------------------
 */
static inline void cf_led_off (void) 
{
	volatile cpm8xx_t *pcpm = &(((immap_t *)IMAP_ADDR)->im_cpm);

	pcpm->cp_pbdat &= ~0x0002;
}

#endif

