/*-----------------------------------------------------------------------------
 *	Copyright (C) Cyclades Corporation, 1997.
 *	All rights reserved
 *
 *	PR-3000 Router
 *
 *	This file:	fpga.h
 *	Description:	This file contains the definitions and prototypes 
 *			related to the FPGA programming
 *			
 *	
 *	Complies with Cyclades SW Coding Standard rev 1.2.
 *-----------------------------------------------------------------------------
 */
 
/*-----------------------------------------------------------------------------
 *	Change History
 *-----------------------------------------------------------------------------
 * 05/21/97	V 1.0.0		Helio Fujimoto
 *	Initial Implementation
 * 09/16/97 V 1.0.1		Helio Fujimoto
 *	Adapted for the new PR-3000 board
 *	ZBUS definitions added.
 *	Included the structures for the mother board FPGA, SWAN and ZBUS registers.
 *-----------------------------------------------------------------------------
 */

/*--------------------------------------------------------------------
 * Daughter board definitions
 *--------------------------------------------------------------------
 */
/* Daughter card detect values */
#define	NO_CARD_DET			0xf8
#define SWAN_CARD_DET		0xb8
#define SWAN_REV_CARD_DET	0xb0
#define T1_CARD_DET			0xe0
#define E1_CARD_DET			0xe8
#define BRI_ST_CARD_DET 	0x88
#define BRI_U_CARD_DET		0x80
#define ZBUS_CARD_DET   	0xc0
#define DATA_CARD_DET		0xd0
#define ETHE_CARD_DET   	0xd8
#define VOICE_CARD_I		0xA0
#define VOICE_CARD_II		0xA8

/* Number of the board */
#define	BOARD_0				0
#define	BOARD_1				1
#define	BOARD_2				2
#define BOARD_3 			3

#ifdef CONFIG_PR4K
#define	NUMBER_OF_BOARDS	1
#define	MB_EPLD_ADDR		((((immap_t *)IMAP_ADDR)->im_memctl.memc_br0 & 0xffff8000L) + 0x800000L)
#endif

#if defined(CONFIG_PR2K) || defined(CONFIG_PR3K)
#define	NUMBER_OF_BOARDS	3
#define	MB_EPLD_ADDR		(((immap_t *)IMAP_ADDR)->im_memctl.memc_br1 & 0xffff8000L)
#endif

#ifdef CONFIG_PR1K
#define	NUMBER_OF_BOARDS	1
#endif


#define	MB_FPGA_ADDR		(((immap_t *)IMAP_ADDR)->im_memctl.memc_br1 & 0xffff8000L)
#define DB_FPGA_ADDR		(((immap_t *)IMAP_ADDR)->im_memctl.memc_br6 & 0xffff8000L)
#define DB_FPGA_ADDR_OFF(board)	(board * 0x20000L)
#define DB_FPGA_ADDR_SIZE	(NUMBER_OF_BOARDS * 0x20000L)

/*--------------------------------------------------------------------
 * Mother board FPGA definitions
 *--------------------------------------------------------------------
 */
 
/* Mother board EPLD registers */ 
typedef struct
{
	unsigned char dsp_irq;
	unsigned char db_pdr;
	unsigned char dsp_id0;
	unsigned char dsp_id1;
	unsigned char dsp_id2;
	unsigned char dsp_id3;
	unsigned char dram_id;
	unsigned char vrm_sw;
} EPLD_REG_PR4K;

/* Mother board FPGA registers */
typedef struct
{
	unsigned char	mb_level;
	unsigned char	ecr;
	unsigned char	t1e1_leds;
	unsigned char	dsp_prg;
	unsigned char	swch_reg;
	unsigned char	clust_ctrl;
	unsigned char	hw_mon;
	unsigned char	db_rst;
	unsigned char	gpio;
	unsigned char	t1e1_rst;
	unsigned char	t1e1_sync;
	unsigned char	id_version;
} FPGA_REG_PR4K;

#define	T1_OR_E1			((((FPGA_REG_PR4K *)MB_FPGA_ADDR)->mb_level) & 0x10)
#define FPGA_ID_MASK		0xE0	/* motherboard fpga ID */
#define FPGA_VERSION_MASK	0x1F	/* motherboard fpga version */
#define FPGA_207			0x80	/* 2.0.7a fpga code */
#define FPGA_209			0xA0	/* 2.0.9  fpga code */
#define PWR_RESET			0xA5	/* Id for Power reset */
#define FETH_RESET			0x5A	/* Id for FETH reset */
#define FALCa_RESET			0x66	/* Id for FALC A reset */
#define FALCb_RESET			0x99	/* Id for FALC B reset */
#define PWR_RELEASE			0x81	/* Id for Power release */
#define FETH_RELEASE		0x7E	/* Id for FETH release */
#define FALCa_RELEASE		0x3C	/* Id for FALC A release */
#define FALCb_RELEASE		0x42	/* Id for FALC B release */
#define ALL_RELEASE			0xC3	/* Id for all devices release */

/* Mother board FPGA registers */
typedef struct
{
	unsigned short	pdr;
	unsigned char	space1[2];

	unsigned char	cpu_led;
	unsigned char	space2[3];

	unsigned short	ecr;
	unsigned char	space3[2];

	unsigned char	epr;
	unsigned char	space4[3];

	unsigned short	rcr;
	unsigned char	space5[2];

	unsigned char	dbgir;
	unsigned char	space6[3];

	unsigned char	dbgor;
	unsigned char	space7[3];

	unsigned char	mlcr;
} FPGA_REG;

typedef struct
{
	unsigned short	mb_level;
	unsigned short	space1;

	unsigned short	wan_conf;
	unsigned short	space2;
	
	unsigned short	reset;
	unsigned short	space3;
} EPLD_REG;

/*--------------------------------------------------------------------
 * DSP board FPGA definitions
 *--------------------------------------------------------------------
 */

#define	MAX_DSP_DIMM		8	/* Number of the board */
#define MAX_DSP_PER_DIMM 	8	/* Number of DSPs per DIMM board */
#define NO_DSP_CARD_DET		0x0F
#define NO_DSP_ID_DET		0x1F
#define DSPID_MODEM8		0x8A
#define DSPID_MODEM6		0x6A
#define TIMER1				0
#define TIMER2				1	/* allocated for DSP */
#define TIMER3				2
#define TIMER4				3
#define TIME_OUT			10	/* max number of retries in a delay loop */
#define	START_TIMER2		((immap_t *)IMAP_ADDR)->im_cpmtimer.timer_tgcr ^= 0x0011;	/* free timer2 to run */
#define DIMM_BOARD_ADDR(x)	(x*0x00400000UL)
#define DIMM_BANK_ADDR(x)	((x / (MAX_DSP_DIMM / 2)) * 0x10000000UL)
#define LATCH_ADDR(x)		(x*4 + 0x80)
#define IDMA_ADDR(x)		(x*4 + 0xC0)

#define DSPBOARD_DETECTED	((n_dsp_boards) ? 0 : 1)

/* Structures */
typedef struct 
{
	unsigned char   reserved:3,
					dsp_done:1,
					dsp_init:1,
					dsp_reset:1,
					dsp_clk:1,
					dsp_data:1;
} DSPPRG;

typedef struct
{
	unsigned char	dummy1;
	unsigned char	idreg;
	unsigned char	dummy2[3];
	unsigned char	verreg;
	unsigned char	dummy3[3];
	unsigned char	flagout;
	unsigned char	dummy4[3];
	unsigned char	irqstat;
	unsigned char	dummy5[3];
	unsigned char	dsprst;
	unsigned char 	dummy6[3];
	unsigned char	dspsync;
	unsigned char	dummy7[3];
	unsigned char	fpgarst;
	unsigned char	dummy8[3];
	unsigned char	scratch;
} DSP_FPGA;

/*--------------------------------------------------------------------
 * SWAN card FPGA definitions
 *--------------------------------------------------------------------
 */
 
/* FPGA ID */
#define	SWAN_ID			0xDE
#define	SWAN_TEST_ID	0xD4
#define SWAN_REV_ID		0xCF

/* Cable Detect values */
#define RS232_OR_V35_CABLE	0
#define RS232_CABLE			1
#define V35_CABLE			2
#define X21_CABLE			3
#define NO_CABLE			0xf

/* Type of Terminal: */
#define	DTE_TERMINAL	0
#define	DCE_TERMINAL	1

/* Internal configuration vector */
typedef struct {
	unsigned char	clock_source,	/* internal/external*/
					clock_frequency,
					tx_clock_dce,
					tx_clock_dte,
					rx_clock,
					interface,	/* RS-232, V35,...*/
					crossing,	/* DTE/DCE */
					line_code;	/* NRZ/NRZI - SNA/SDLC - Jul/16/98 */
} SWAN_CONFIG[NUMBER_OF_BOARDS];

/* SWAN registers */
typedef struct 
{
	unsigned char	id;
	unsigned char	version;
	unsigned char	detect;
	unsigned char	config;
	unsigned char	clk_oe;
	unsigned char	clk_out_sel;
	unsigned char	clk_mux;
	unsigned char	scratch;
} SWAN_REGS;

/* SWAN test registers */
typedef struct 
{
	unsigned char	id;
	unsigned char	version;
	unsigned char	status;
	unsigned char	error1;
	unsigned char	error2;
	unsigned char	clrerr;
	unsigned char	space;
	unsigned char   scratch;
} SWAN_TEST_REGS;

/*--------------------------------------------------------------------
 * ZBUS card FPGA definitions
 *--------------------------------------------------------------------
 */
 
/* FPGA ID */
#define ZBUS_ID				0x56

/* Serial Expander ID */
#define SEX_ID				0x29

// SSE8-V174
/* Synchronous Serial Expander defines */
#define SSEX_ID             0x40
#define SSEX_VERSION        0x01

/* Maximum number of SEXs allowed */
#define	MAX_NUMBER_OF_SEXS	16

/* ZBUS FPGA registers */
typedef struct 
{
	unsigned char	id;
	unsigned char	version;
	unsigned char	irq;
	unsigned char	sense;
	unsigned char	reset;
	unsigned char	power;
	unsigned char	wait_states;
	unsigned char	debug;
} ZBUS_REGS;

/* Serial Expander UART registers */
typedef union 
{
	struct uart_read  {
		unsigned char	rhr;
		unsigned char	space;
		unsigned char	isr;		// SSE8-V174
		unsigned char	lcr;
		unsigned char	mcr;
		unsigned char	lsr;
		unsigned char	msr;
		unsigned char	spr;
	} r;
											
	struct uart_write  {
		unsigned char	thr;
		unsigned char	ier;
		unsigned char	fcr;
		unsigned char	lcr;
		unsigned char	mcr;
		unsigned char	space[2];
		unsigned char	spr;
    } w;
	
	struct uart_special {
		unsigned char	dll;
		unsigned char	dlm;
		unsigned char	efr;
		unsigned char	space;
		unsigned char	xon1;
		unsigned char	xon2;
		unsigned char	xoff1;
		unsigned char	xoff2;
	} s;
} UART_REGS;

/* Serial Expander registers */
typedef struct 
{
	unsigned char	id;
	unsigned char	version;
	unsigned char	isr_low;
	unsigned char	isr_high;
	unsigned char	space[0x80 - 4];
	UART_REGS		uart[16];
} SEX_REGS;

// SSE8-V174 >> begin
/* Syncronous Serial Expander Registers:

0000 - ID (Identification).	Value = 0x40 [.....]
0001 - Version Current value = 0x01 [.....]
0002 - ISR - ISR deve espelhar os INT dos diversos Siemens, da seguinte
forma: [.....]
		bit 0 - Siemens correspondente as portas 1 e 2
		bit 1 - portas 3 e 4
		bit 2 - portas 5 e 6
		bit 3 - portas 7 e 8
		bits 4 a 7 - zero
0003 - not used [.....]
0004 - Leds-low	 
0005 - Leds-high
0006 - Leds-extra
0007 - 001FF - not used [......]
0200 - 03FF - registradores das 8 serial controllers (4 Siemens)
			[64 bytes por porta x 8 portas == 0x200 bytes]
*/
typedef union {
	struct usart_read {
		unsigned char fifo;
		unsigned char reser[31];
		unsigned char star;
		unsigned char rsta;									/*HDLC*/
		unsigned char mode;
		unsigned char timr;
		unsigned char xon;		// or	unsigned char xad1;	/*HDLC*/
		unsigned char xoff;		// or	unsigned char xad2;	/*HDLC*/
		unsigned char tcr;		// or	unsigned char rah1;	/*HDLC*/
		unsigned char dafo;		// or	unsigned char rah2;	/*HDLC*/
		unsigned char rfc;		// or	unsigned char ral1;	/*HDLC*/
		unsigned char rhcr;									/*HDLC*/
		unsigned char rbcl;    
		unsigned char rbch;    
		unsigned char ccr0;    
		unsigned char ccr1;    
		unsigned char ccr2;    
		unsigned char ccr3;    
		unsigned char space30_33[4];
		unsigned char vstr;    
		unsigned char space35_37[3];
		unsigned char gis;    
		unsigned char ipc;    
		unsigned char isr0;    
		unsigned char isr1;    
		unsigned char pvr;    
		unsigned char pis;    
		unsigned char pcr;    
		unsigned char ccr4;									/*HDLC*/
	} r;

	struct usart_write{
		unsigned char fifo;
		unsigned char reser[31];
		unsigned char cmdr;
		unsigned char pre;									/*HDLC*/
		unsigned char mode;
		unsigned char timr;
		unsigned char xon;		// or	unsigned char xad1;	/*HDLC*/
		unsigned char xoff;		// or	unsigned char xad2;	/*HDLC*/
		unsigned char tcr;		// or	unsigned char rah1;	/*HDLC*/
		unsigned char dafo;		// or	unsigned char rah2;	/*HDLC*/
		unsigned char rfc;		// or	unsigned char ral1;	/*HDLC*/
		unsigned char ral2;									/*HDLC*/
		unsigned char xbcl;    
		unsigned char xbch;    
		unsigned char ccr0;    
		unsigned char ccr1;    
		unsigned char ccr2;    
		unsigned char ccr3;    
		unsigned char tsax;    
		unsigned char tsar;    
		unsigned char xccr;    
		unsigned char rccr;    
		unsigned char bgr;
		unsigned char tic;		// or	unsigned char rlcr; /*HDLC*/
		unsigned char mxn;		// or	unsigned char aml;
		unsigned char mxf;		// or	unsigned char amh;
		unsigned char iva;    
		unsigned char ipc;    
		unsigned char imr0;    
		unsigned char imr1;    
		unsigned char pvr;    
		unsigned char pim;    
		unsigned char pcr;    
		unsigned char ccr4;									/*HDLC*/
	} w;
} USART_REGS;

typedef struct {
	unsigned char	id;
	unsigned char	version;
	unsigned char	isr;
	unsigned char	filler;
	unsigned char 	low_led;
	unsigned char 	hi_led;
	unsigned char 	extra_led;
	unsigned char	space[0x200 - 7];
	USART_REGS		usart[8];
} SSEX_REGS;

// SSE8-V174 << end

#ifdef CONFIG_PR4K

#define	PWR_IRQ_MASK	0x80000000L
#define	FALC1_IRQ_MASK	0x20000000L
#define	FALC2_IRQ_MASK	0x08000000L
#define	DB_IRQ_MASK		0x02000000L
#define	DSP_IRQ_MASK	0x00800000L
#define	MON_IRQ_MASK	0x00200000L
#define	FPGA_IRQ_MASK	0x00080000L

#else

#define	IRQ0_MASK	0x80000000L
#define	IRQ1_MASK	0x20000000L
#define	IRQ2_MASK	0x08000000L
#define	IRQ3_MASK	0x02000000L

#define	BRI_ST_IT	0
#define	BRI_U_IT	1
#define	E1_IT		2
#define	T1_IT		3
#define	ZBUS_IT		4
#define	NO_IT		-1

#endif

extern FPGA_REG *fpga;

extern unsigned char *board_addr;

extern unsigned char board_detect[NUMBER_OF_BOARDS];

