/*
 * include/asm-ppc/k2.h
 * 
 * Definitions for SBS K2 board support
 *
 * Author: Matt Porter
 *         mporter@mvista.com
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ASMPPC_K2_H
#define __ASMPPC_K2_H

#include <asm/m1543c.h>

/*
 * SBS K2 definitions
 */

#define	K2_PCI64_BAR		0xff400000
#define	K2_PCI32_BAR		0xff500000

#define K2_PCI64_CONFIG_ADDR	(K2_PCI64_BAR + 0x000f8000)
#define K2_PCI64_CONFIG_DATA	(K2_PCI64_BAR + 0x000f8010)

#define K2_PCI32_CONFIG_ADDR	(K2_PCI32_BAR + 0x000f8000)
#define K2_PCI32_CONFIG_DATA	(K2_PCI32_BAR + 0x000f8010)

#define K2_PCI64_PHY_MEM_BASE	0xc8000000
#define K2_PCI64_MEM_BASE	0xe8000000
#define K2_PCI64_PHY_IO_BASE	0x88000000
#define K2_PCI64_IO_BASE	K2_PCI64_PHY_IO_BASE

#define K2_PCI32_PHY_MEM_BASE	0xc0000000
#define K2_PCI32_MEM_BASE	0xe0000000
#define K2_PCI32_PHY_IO_BASE	0x80000000
#define K2_PCI32_IO_BASE	K2_PCI32_PHY_IO_BASE

#define K2_PCI32_SYS_MEM_BASE	0x80000000

#define K2_PCI64_MEM_TOP	0x10000000
#define	K2_PCI64_IO_TOP		0x10000000

#define K2_PCI32_MEM_TOP	0x08000000
#define K2_PCI32_IO_TOP		0x08000000

#define K2_ISA_IO_BASE		K2_PCI32_IO_BASE
#define K2_ISA_MEM_BASE		K2_PCI32_MEM_BASE

#define	K2_ISAPNP_CONFIG	(K2_ISA_IO_BASE + M1543C_ISAPNP_CONFIG_OFFSET)
#define K2_ISAPNP_INDEX		K2_ISAPNP_CONFIG
#define K2_ISAPNP_DATA		(K2_ISA_IO_BASE + M1543C_ISAPNP_DATA_OFFSET)

#define K2_BOARD_ID_REG		(K2_ISA_IO_BASE + 0x800)
#define K2_MISC_REG		(K2_ISA_IO_BASE + 0x804)
#define K2_MSIZ_GEO_REG		(K2_ISA_IO_BASE + 0x808)
#define K2_HOT_SWAP_REG		(K2_ISA_IO_BASE + 0x80c)
#define K2_PLD2_REG		(K2_ISA_IO_BASE + 0x80e)
#define K2_PLD3_REG		(K2_ISA_IO_BASE + 0x80f)

#define K2_BUS_SPD(board_id)	(board_id >> 2) & 3

#define K2_RTC_BASE_OFFSET	0x97ff8

#define K2_MEM_SIZE_MASK	0xe0
#define K2_MEM_SIZE(size_reg)	(size_reg & K2_MEM_SIZE_MASK) >> 5
#define	K2_MEM_SIZE_1GB		0x40000000
#define K2_MEM_SIZE_512MB	0x20000000
#define K2_MEM_SIZE_256MB	0x10000000
#define K2_MEM_SIZE_128MB	0x08000000

#define K2_L2CACHE_MASK		0x03	/* Mask for 2 L2 Cache bits */
#define K2_L2CACHE_512KB	0x00	/* 512KB */
#define K2_L2CACHE_256KB	0x01	/* 256KB */
#define K2_L2CACHE_1MB		0x02	/* 1MB */
#define K2_L2CACHE_NONE		0x03	/* None */

#define K2_GEO_ADR_MASK		0x1f

#define K2_SYS_SLOT_MASK	0x80

#endif /* __ASMPPC_K2_H */
