/*----------------------------------------------------------------------------
 *      Copyright (C) Cyclades Corporation, 1998.
 *      All rights reserved
 *      SM100/TS110
 *
 *      This file:      cy_sm.h
 *      Description:    Routines to handle Server Management and IO commands.
 *---------------------------------------------------------------------------
 *---------------------------------------------------------------------------
 *      Change History
 *---------------------------------------------------------------------------
 *      Aug/20/2002     V 1.0.0   Arnaldo Zimmermann - Initial Implementation
 *--------------------------------------------------------------------------*/

/* I2C Device address */
#define ADC_I2C_ADDR_FIRST	0x90	// possible physical addresses are: 
									// 0x90, 0x92, 0x094, 0x96, 0x98, 0x9a, 0x9c, 0x9e
#define ADC_I2C_ADDR_LAST	0x9e
#define CY_SLAVE_ADDR		0x10

#define CMD_SM_MODE_ATX		0
#define CMD_SM_MODE_IPMI	1
#define CMD_SM_MODE_SENSOR	2

#define POWER_MODE_STATUS	 (0x0040)
#define POWER_ON_MB			 (0x0020)
#define SM100_POWER_ON		 (0x0010)
#define SM100_POWER_ENABLE	 (0x0008)
#define MB_POWER_ENABLE		 (0x0004)
#define SM100_MB_POWER		 (SM100_POWER_ON | POWER_ON_MB)
#define POWER_MODE_MASK		 (POWER_MODE_STATUS|POWER_ON_MB|SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE)

/* MAX6656 Sensor Chip */
#define SENSOR_PWR_OFFSET(x) ((x<12) ? 0 : 12)
#define SENSOR_VCC_3_3(x)	 ((x + SENSOR_PWR_OFFSET(x)) * 157)
#define SENSOR_VCC_5(x)		 ((x + SENSOR_PWR_OFFSET(x)) * 238)
#define SENSOR_VCC_12(x)	 ((x + SENSOR_PWR_OFFSET(x)) * 571)
#define SENSOR_DEC_TEMP(x) 	 (x / 0x20 * 125)
#define SENSOR_SLAVE_ADDRESS (0x30)

/* MAX6656 Sensor Chip - Register Map */
#define SENSOR_RRTE_REG			(0x01)
#define SENSOR_RSL_REG			(0x02)
#define SENSOR_RCL_REG			(0x03)
#define SENSOR_WCA_REG			(0x09)
#define SENSOR_WCRW_REG			(0x0a)
#define SENSOR_WRHA_REG			(0x0d)
#define SENSOR_RRTE1_REG		(0x10)
#define SENSOR_WV0HL_REG		(0x1e)
#define SENSOR_WV0LL_REG		(0x1f)
#define SENSOR_WV1HL_REG		(0x20)
#define SENSOR_WV1LL_REG		(0x21)
#define SENSOR_WV2HL_REG		(0x22)
#define SENSOR_WV2LL_REG		(0x23)
#define SENSOR_RV0_REG			(0x2e)
#define SENSOR_RV1_REG			(0x2f)
#define SENSOR_RV2_REG			(0x30)
#define SENSOR_RSL2_REG			(0x32)
#define SENSOR_RCL2_REG			(0x33)
#define SENSOR_WCA2_REG			(0x34)

/* MAX6656 Sensor Chip - Conf Byte 1 Bit Assigments */
#define SENSOR_MASK_Vcc			(0x01)
#define SENSOR_MASK_Vin1		(0x02)
#define SENSOR_MASK_Vin2		(0x04)
#define SENSOR_MASK_Vin3		(0x08)
#define SENSOR_MASK_RT2			(0x10)
#define SENSOR_MASK_RT1			(0x20)
#define SENSOR_RUN_STOP			(0x40)
#define SENSOR_MASK_ALL			(0x80)

/* MAX6656 Sensor Chip - Conf Byte 2 Bit Assigments */
#define SENSOR_DIS_Vcc			(0x02)
#define SENSOR_DIS_Vin1			(0x04)
#define SENSOR_DIS_Vin2			(0x08)
#define SENSOR_DIS_Vin3			(0x10)
#define SENSOR_DIS_IT			(0x20)
#define SENSOR_DIS_RT2			(0x40)
#define SENSOR_DIS_RT1			(0x80)

/* MAX6656 Sensor Chip - Status Byte 1 Bit Assigments */
#define SENSOR_R2LOW			(0x01)
#define SENSOR_R2HIGH			(0x02)
#define SENSOR_DODS1			(0x04)
#define SENSOR_RLOW				(0x08)
#define SENSOR_RHIGH			(0x10)
#define SENSOR_LLOW				(0x20)
#define SENSOR_LHIGH			(0x40)
#define SENSOR_BUSY				(0x80)

/* MAX6656 Sensor Chip - Status Byte 2 Bit Assigments */
#define SENSOR_Vcca				(0x01)
#define SENSOR_Va1				(0x02)
#define SENSOR_Va2				(0x04)
#define SENSOR_Va3				(0x08)
#define SENSOR_DODS2			(0x10)
#define SENSOR_R2O				(0x20)
#define SENSOR_R1O				(0x40)
#define SENSOR_LO				(0x80)

/* I2C read/write mask */
#define I2C_OFFSET_MASK			(0x01)
#define I2C_CLOCK_MASK			(0x02)

/* I2C read/write values */
#define I2C_NO_OFFSET			(0x00)
#define I2C_SET_OFFSET			(0x01)
#define I2C_SET_MASTER			(0x02)
#define I2C_SET_SLAVE			(0x00)

#define I2C_MAX_RETRY			 0x03

/* IPMI */
#define BMC_SA                   0x20    // BMC slave address
#define BMC_LUN                  0x00    // BMC only responds to this LUN
#define CHASSIS_NETFN            0x00
#define APP_NETFN                0x06
#define BMC_MSG_MOD              0x40
#define RQ_SA_OFFSET             0x03    // rqSA offset
#define RQ_SEQ_OFFSET            0x04    // rqSeq/rqLUN offset

/* IMB packet defines and utility macros */
#define IMB_HEAD_SIZE            3                       // rsSa, netFnLun, csum
#define IMB_CHECK1               2

#define NETFN_OF(x)              (((x)>>2) & 0x3f)       // extract netFn from nfLn
#define LUN_OF(x)                ((x) & 0x3)             // extract lun from nfLn
#define SEQ_OF(x)                (((x)>>2) & 0x3f)       // extract seqNo from seqLn

#define SEQ_LUN(s,l)             ((((s) & 0x3f) << 2) | ((l) & 0x3))     // construct seqLn from seq and lun
#define NETFN_LUN(n,l)           ((((n) << 2) | ((l) & 0x3)))            // construct nfLun from netFn and lun

#define RESPONSE_NETFN(nf)       ((nf) | 1)              // create response netFn from request netFn
#define IS_REQUEST(nf)           (((nf) & 1) == 0)       // test for request netFn
#define IS_RESPONSE(nf)          (((nf) & 1) == 1)       // test for response netFn

/* IMB commands used by the driver */
#define CHASSIS_CONTROL          0x02
#define RESET_WATCHDOG_TIMER     0x22
#define SET_WATCHDOG_TIMER       0x24

#define BMC_MAX_MSG_SIZE         32

typedef struct {
	u_char    rqSA;
	u_char    rqSeq;
	u_char    cmd;
	u_char    data[1];
} BMC_Rq_rq;

typedef struct {
	u_char    rsSA;
	u_char    netFn;
	u_char    chk1;
	BMC_Rq_rq rq;
} BMC_Rq;

typedef struct {
	u_char    rsSA;
	u_char    rqSeq;
	u_char    cmd;
	u_char    data[1];
} BMC_Rs_rs;

typedef struct {
	u_char    rqSA;
	u_char    netFn;
	u_char    chk1;
	BMC_Rs_rs rs;
} BMC_Rs;

/* IPMI POWER CONTROL */
#define                         WD_DONT_LOG             0x80
#define                         WD_BIOS_FRB2            0x01
#define                         WD_BIOS_POST            0x02
#define                         WD_OS_LOAD              0x03
#define                         WD_SMS                  0x04

#define                         WD_PRE_ACT_NONE         0x00
#define                         WD_PRE_ACT_SMI          0x10
#define                         WD_PRE_ACT_SCI          0x20

#define                         WD_ACT_NONE             0x00
#define                         WD_ACT_RESET            0x01
#define                         WD_ACT_PWR_DOWN         0x02
#define                         WD_ACT_PWR_CYCLE        0x03


/* IPMI CHASSIS CONTROL */
#define                         CH_PWR_DOWN            0x00
#define                         CH_PWR_UP              0x01
#define                         CH_PWR_CYCLE           0x02
#define                         CH_HW_RESET            0x03
#define                         CH_NMI                 0x04
#define                         CH_SW_SHUTDOWN         0x05
