/* -*- linux-c -*- */
/*
    This file is part of llconf2

    Copyright (C) 2004  Oliver Kurth <oku@debian.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* copied from llconf, modified, and will hopefully be returned to llconf sometime
   to avoid duplication... */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "slist.h"
#include "lines.h"

//FIXME: create function to destry whole list of lines, and call that in all parsers...

struct line *create_line(const char *line)
{
	struct line *l;
  
	l = malloc(sizeof(struct line));
	if(l){
		memset(l, 0, sizeof(struct line));
		if(line)
			l->line = strdup(line);
	}
	return l;
}

void destroy_line(struct line *l)
{
	if(l){
		if(l->line) free(l->line);
		free(l);
	}
}

void destroy_line_list(struct line *l_list)
{
	struct line *l, *l_next;

	for(l = l_list; l; l = l_next){
		l_next = l->next;
		destroy_line(l);
	}
}

struct line *append_line(struct line *l_list, struct line *l)
{
	return (struct line *)slist_append(
		(struct slist_hdr *)l_list, (struct slist_hdr *)l);
}

struct line *prepend_line(struct line *l_list, struct line *l)
{
	return (struct line *)slist_prepend(
		(struct slist_hdr *)l_list, (struct slist_hdr *)l);
}

int line_setval(struct line *l, const char *value)
{
	if(l->line){
		free(l->line);
	}
	l->line = strdup(value);
	return 0;
}

struct line *read_lines(FILE *fptr)
{
	char line[MAX_LINE];
	struct line *l = NULL, *l_next, *l_root = NULL;

	while(fgets(line, MAX_LINE-1, fptr)){
		l_next = create_line(line);
		if(l)
			l->next = l_next;
		else
			l_root = l_next;
		l = l_next;
	}
	return l_root;
}

static
int getline(const char **pp, int buf_size, char *dst, int max_len)
{
	const char *p = *pp;
	char *q = dst;
	int len = 0;

	while(*p && (*p != '\n') && (q < dst + max_len - 1) && (p < *pp + buf_size)){
		if(*p >= ' ' || *p == '\t')
			*q++ = *p++;
		else
			p++;
	}
	if((*p == '\n') && (q < dst + max_len - 1) && (p < *pp + buf_size))
		*q++ = *p++;
	*q = 0;

	len = p - *pp;
	*pp = p;

	return len;
}

struct line *convert_lines(const char *buf, int size)
{
	char line[MAX_LINE];
	struct line *l = NULL, *l_next, *l_root = NULL;
	const char *p;

	if(buf){
		p = buf;
		while(size > 0 && *p){
			int sz;
			
			sz = getline(&p, size, line, MAX_LINE);
			if(sz > 0){
				l_next = create_line(line);
				if(l)
					l->next = l_next;
				else
					l_root = l_next;
				l = l_next;
				size -= sz;
			}else
				break;
		}
	}
	return l_root;
}
