/* -*- linux-c -*- */
/*
    This file is part of cacpd

    Copyright (C) 2005 Cyclades Corp.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* Author: Oliver Kurth <oliver.kurth@cyclades.com> */

/* NOTE: this code is partially inspired by the man page select_tut(2)
   and the httpd example from http://threading.2038bug.com/httpd.c.html.gz.
   Both references are from Paul Sheer. (oliver) */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <sys/time.h>

#include <libdaemon/dlog.h>

#include "slist.h"

#include "loop.h"

#undef max
#define max(x,y) ((x) > (y) ? (x) : (y))

struct fd_callback *g_cb_list = NULL;
struct fd_callback *g_timecb_list = NULL;

struct fd_callback *create_fd_callback(int fd, int type, int (*callback)(int, void *), void *data)
{
	struct fd_callback *cb;

	cb = (struct fd_callback *)malloc(sizeof(struct fd_callback));
	if(cb){
		memset(cb, 0, sizeof(struct fd_callback));
		cb->type = type;
		cb->fd = fd;
		cb->cb = callback;
		cb->data = data;
	}
	return cb;
}

void loop_prepend_callback(struct fd_callback *cb)
{
	g_cb_list = (struct fd_callback *)slist_append(
		(struct slist_hdr *)cb, (struct slist_hdr *)g_cb_list);
}

void loop_add_callback(struct fd_callback *cb)
{
	g_cb_list = (struct fd_callback *)slist_append(
		(struct slist_hdr *)g_cb_list, (struct slist_hdr *)cb);
}

void loop_del_callback(struct fd_callback *cb)
{
	g_cb_list = (struct fd_callback *)slist_unlink(
		(struct slist_hdr *)g_cb_list, (struct slist_hdr *)cb);
	free(cb);
}

void loop_add_time_callback(struct fd_callback *cb)
{
	g_timecb_list = (struct fd_callback *)slist_append(
		(struct slist_hdr *)g_timecb_list, (struct slist_hdr *)cb);
}

void loop_del_time_callback(struct fd_callback *cb)
{
	g_timecb_list = (struct fd_callback *)slist_unlink(
		(struct slist_hdr *)g_timecb_list, (struct slist_hdr *)cb);
	free(cb);
}

void loop(struct timeval *timeout0)
{
	struct fd_callback *cb;
	fd_set write_fd_set, read_fd_set;
	struct timeval timeout;

	if(timeout0)
		memcpy(&timeout, timeout0, sizeof(timeout));

	while(1){
		int sel_ret, nfds = 0;

		
		FD_ZERO(&read_fd_set);
		FD_ZERO(&write_fd_set);

		for(cb = g_cb_list; cb; cb = cb->next){
			if(cb->fd >= 0){
				if(cb->type == FDCB_READ)
					FD_SET(cb->fd, &read_fd_set);
				if(cb->type == FDCB_WRITE)
					FD_SET(cb->fd, &write_fd_set);
				nfds = max(nfds, cb->fd);
			}
		}

		sel_ret = select(nfds+1, &read_fd_set, &write_fd_set, NULL, timeout0 ? &timeout : NULL);
		
		if(sel_ret > 0){
			struct fd_callback *cb_next;

			for(cb = g_cb_list; cb; cb = cb_next){
				cb_next = cb->next;
				if((cb->state != FDCB_CLOSING) && (cb->fd >= 0)){
					if((cb->type & FDCB_READ) && FD_ISSET(cb->fd, &read_fd_set))
						cb->cb(cb->fd, cb->data);
					if((cb->type & FDCB_WRITE) && FD_ISSET(cb->fd, &write_fd_set))
						cb->cb(cb->fd, cb->data);
				}
			}

			for(cb = g_cb_list; cb; cb = cb_next){
				cb_next = cb->next;
				if(cb->state == FDCB_CLOSING){
					if(cb->fd >= 0){
						int ret = 0;

						ret = close(cb->fd);
						if(ret != 0)
							daemon_log(LOG_ERR, "close fd=%d: %s", cb->fd, strerror(errno));
					}
					loop_del_callback(cb);
					cb->fd = -1;
				}
			}

			if(g_cb_list == NULL){
				daemon_log(LOG_NOTICE, "loop: no more sockets to listen, returning");
				return;
			}
		}else if(sel_ret == 0){
			struct fd_callback *cb_next;
			
			for(cb = g_timecb_list; cb; cb = cb_next){
				cb_next = cb->next;
				cb->cb(-1, cb->data);
			}

			for(cb = g_timecb_list; cb; cb = cb_next){
				cb_next = cb->next;
				if(cb->state == FDCB_CLOSING){
					if(cb->fd >= 0){
						int ret;

						ret = close(cb->fd);
						if(ret != 0)
							daemon_log(LOG_ERR, "close fd=%d: %s", cb->fd, strerror(errno));
					}
					loop_del_time_callback(cb);
					cb->fd = -1;
				}
			}

			if(timeout0)
				memcpy(&timeout, timeout0, sizeof(timeout));
		}else{
			if (errno == EINTR)
				continue;
		}
	}
}
