#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/mount.h>

#include <linux/mtd/mtd.h>

typedef struct erase_info er_info;

int non_region_erase(int Fd, int start, int count, int unlock)
{
	mtd_info_t meminfo;
	region_info_t erase;

	if (ioctl(Fd,MEMGETINFO,&meminfo) == 0)
	{

		erase.offset = start; //+ 0x006e2200;

		erase.erasesize = meminfo.erasesize;
		erase.numblocks = 1;

		for (; count > 0; count--) {
			printf("\rPerforming Flash Erase of length %u at offset 0x%x",
					count, erase.offset);
			fflush(stdout);

			if(unlock != 0)
			{
				//Unlock the sector first.
				printf("\rPerforming Flash unlock at offset 0x%x",erase.offset);
				if(ioctl(Fd, MEMUNLOCK, &erase) != 0)
				{
					perror("\nMTD Unlock failure");
					close(Fd);
					return 8;
				}
			}

			if (ioctl(Fd,MEMERASE,&erase) != 0)
			{      
				perror("\nMTD Erase failure");
				close(Fd);
				return 8;
			}
			erase.offset += meminfo.erasesize;
		}
		printf(" done\n");
	}
	return 0;
}

int main(int argc,char *argv[])
{
	int regcount;
	int Fd;
	int start;
	int count;
	int unlock;
	int res = 0; 

	if (1 >= argc)
	{
		fprintf(stderr,"You must specify a device\n");
		return 16;
	}

	if (argc > 2)
		start = strtol(argv[2], NULL, 0);
	else
		start = 0;

	if (argc > 3)
		count = strtol(argv[3], NULL, 0);
	else
		count = 1;

	if(argc > 4)
		unlock = strtol(argv[4], NULL, 0);
	else
		unlock = 0;


	// Open and size the device
	if ((Fd = open(argv[1],O_RDWR)) < 0)
	{
		fprintf(stderr,"File open error\n");
		return 8;
	}

	printf("Erase Total %d Units\n", count);

	res = non_region_erase(Fd, start, count, unlock);
	//res = region_erase(Fd, start, count, unlock, regcount);
			
	return res;
}
