/*****************************************************************
* File: pmCommand.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
* This program treats the commands pass from user to pm menu and 
* pmCommand. It takes the command, check the arguments and pass to
* pmd daemon, by coping command to shared memory and sending a signal
* to pmd indicating that there is a command to be executed.
*
****************************************************************/

#define PMCOMMAND
#include <grp.h>
#include <pwd.h>
#include "pmCommand.h"

#define ADMIN_GROUP             "admin"
#define ADMIN_USER              "admin"
#define alias serverfarm

#define INFO 1

void FillPmcOutlets(pmcInfo *pmc);
static int admin, ipdu, pmcindex;
char username[NAME]={0};

#ifdef ONS
int auxflag = 0;
#endif
//static struct pmcInfo *pmc;
pmcInfo *pmc=0;

/* Which software interrupts were detected */
int got_sig;

# define GOT_SIGUSR1	1	/* It is signalled by PMD, indicating that the
				   PMD is exiting */
# define GOT_SIGUSR2	2	/* It is signalled by PMD, indicating that 
				   there is data to be read from the shared
				   memory. */
# define GOT_SIGINT	4	/* It exits the pm/pmCommand process */

#define MAX_DEV         cy_shm->max_dev

static char pmCommand_header2[] =
	"Type  help  for help  \r\n"
	"Type  menu  for menu driven interface  \r\n"
	"Type  exit  to exit \r\n"
	"-----------------------------------------------------------------------\r\n";

static char pmCommand_syntax[] =
	"\r\n"
	"Use: pmCommand <serial port number> <command> <arguments> \r\n"
	"where: <serial port number>  is the serial port number configured as IPDU \r\n"
	"	<command> <arguments> are the PM command and its arguments. \r\n";

char *pmCommand_error_msgs[4] = {
	"The user does not have permission to manage this PDU.\r\n",
	"The command is not accepted by PDU, type \"help\" to see the command list.",
	"You do not have permission to execute this command, type \"help\" to see the command list.",
	"The arguments typed are not valid for this command, type \"help\" to see the arguments."
	};

static int call_pm(char **command);
static int pm_show_help(char **command);
static int pm_treat_exit(char **command); 
static int pm_treat_whoami(char **command);
static int pm_treat_server_name(char **command); 

struct _commands pmBasicCommands[] = {
	{0, "Exit", "exit", NULL, 
		"Exit", 
		NULL, pm_treat_exit},
	{0, "Help", "help", NULL, 
		"Show this help", 
		NULL, pm_show_help},
	{0, "Menu", "menu", NULL, 
		"Start the menu driven text interface(pm)", 
		NULL, call_pm},
	{0, "Who Am I", "whoami", NULL, 
		"Display the username currently logged in", 
		NULL, pm_treat_whoami},
	{1, "Get Name", "getname", "<outlet>",
		"Get the name of the server",
		NULL, pm_treat_server_name}
};

#define NUM_BASIC_COMMANDS	(sizeof(pmBasicCommands)/sizeof(struct _commands))

static void usage() {
		puts(pmCommand_syntax);
}

static int check_pmd() 
{
	if (!cy_shm->PMD_pid) {
		printf ("\r\n[Error] - PMD has not started.\r\n");
		return 0;
	} 
	return 1;	
}

int check_ipdu(int ipdu) {
	if (cy_shm->line_conf[ipdu-1].protocol != P_IPDU) {
			printf("\r\n[Error] - Protocol IPDU is not set.\r\n");
			return 0;
	} 
	return 1;
}

/* Check if user is admin, and if not, if there is an users allowed list,
 * if is, check if user are in list. 1 is success, 0 is fail */

static int check_user() 
{
	int uid;
	char *str, *token="\0";
	struct passwd *user;
	struct group *grpa;
	
	uid=getuid();
	user=getpwuid(uid);

	strcpy(username,user->pw_name);

	grpa = getgrnam(ADMIN_GROUP);

	if (grpa) {
		while (*grpa->gr_mem) {
			if (!strcmp(*grpa->gr_mem, username)) {
				admin=1;
				return 1;
			}
			grpa->gr_mem++;
		}
	}
	admin=0;

	if (lineconf[ipdu-1].pmusers) { //[RK]Aug/13/04 - check the pmusers not users
		str=malloc(strlen(lineconf[ipdu-1].pmusers)+1);
		strcpy(str,lineconf[ipdu-1].pmusers);
		token=strtok(str," :;");
		while (token != NULL) {
			if(strcmp(username,token)==0) 
				return 1;
			else 
				token=strtok(NULL," :;");
		}
		free(str);
	}
	return 0;
}

// treatSignals : set got_sig with corret signal received SIGUSR1 or SIGUSR2
static void treatSignals( int sig ) 
{
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));
	switch (sig) {
		case SIGUSR1:
			got_sig |= GOT_SIGUSR1;
			break;
		case SIGUSR2:
			got_sig |= GOT_SIGUSR2;
			break;
		case SIGTERM:
		case SIGINT:
			got_sig |= GOT_SIGINT;
			break; 
	}
        act.sa_handler = treatSignals;
        sigaction(sig, &act, 0);
        unblock(sig);
}

static void setSignals() 
{
	struct sigaction act;

	memset((char*)&act,0,sizeof(struct sigaction));
	
	// Setting up the act structure
	act.sa_handler = treatSignals;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	
	sigaction(SIGUSR1, &act, 0);
	sigaction(SIGUSR2, &act, 0);
	sigaction(SIGINT, &act, 0);
	sigaction(SIGTERM, &act, 0);
	unblock(SIGUSR1);
	unblock(SIGUSR2);
	unblock(SIGINT);
	unblock(SIGTERM);
}


static int  waitAnswer() {

	while(TRUE) {
		sleep(1);
		if (got_sig & GOT_SIGUSR1) {
			got_sig &= ~GOT_SIGUSR1;
			return 0;
		}
		if (got_sig & GOT_SIGUSR2) {
			got_sig &= ~GOT_SIGUSR2;
			return 1;
		}
		if (got_sig & GOT_SIGINT) {
			Exit();
		}
	}

}

static void initPmc() {

	char * buf;
	int sizeB;	// answer buffer size = 400 + (80 * num. of outlets)

	for (pmcindex=0;pmcindex<MAXPMDACCESS; pmcindex++) {
		if (cy_shm->pmc_list[pmcindex] == 0)
			break;
		pmc = cy_shm->pmc_list[pmcindex];
		if (kill(pmc->pmc_pid, 0) < 0) {
			if (errno == ESRCH) {
				cy_shm->pmc_list[pmcindex] = NULL;
				if (pmc->pmc_answerBuf) {
					shm_free_str(pmc->pmc_answerBuf);
				}
				shm_free_str((char *)pmc);
				break;
			}
		}
	}
	if (pmcindex==MAXPMDACCESS) {
		puts("Cannot find an empty position at pmc_list");
		exit(0);
	}

	sizeB = 400 + (80 * cy_shm->PMD_outlets[ipdu-1]);

	pmc = (pmcInfo *)shm_malloc(sizeof(pmcInfo), TYPE_SHM_PMC);
	buf = shm_malloc(sizeB,TYPE_SHM_PMCBUF);

	if (!pmc || !buf) {
		if (pmc) shm_free_str((char *)pmc);
		if (buf) shm_free_str(buf);
		pmc = NULL;
		puts("Memory allocation error");
		exit(0);
	}
	shm_signal(CY_SEM_SHM);
	pmc->idx_pmclist = pmcindex;
	pmc->pmc_pid = getpid();
#ifdef ONS
	pmc->pmc_aux = auxflag;
#endif
	pmc->pmc_ipdu = ipdu-1; 
	pmc->pmc_status = PMC_IDLE;
	pmc->pmc_sizeAnswerBuf = sizeB;
	pmc->pmc_answerBuf = buf;
	pmc->next = NULL;

	//[RK]Sep/14/04 - parent PID to be used in W command
	if (getppid() == cy_shm->vet_pid[ipdu-1]) {
		pmc->pmc_ppid = cy_shm->vet_pid[ipdu-1];
	}

	cy_shm->pmc_list[pmcindex] = pmc;
	strcpy(pmc->pmc_username,username); //[RK]Sep/02/04 - w command
	shm_unsignal(CY_SEM_SHM);
}

static void Exit() {

	shm_signal(CY_SEM_SHM);
	if (pmc) {
		cy_shm->pmc_list[pmcindex]=NULL;
		shm_free_str(pmc->pmc_answerBuf);
		shm_free_str((char *)pmc);
	}
	shm_unsignal(CY_SEM_SHM);
	exit(0);
}

static void sendCommand(char **command) 
{
	char scmd[100]={0};

	while(*command) {
		if (scmd[0]) strcat(scmd," ");
		strcat(scmd,*command);
		command++;
	}
	strcpy(pmc->pmc_username,username); //[RK]Aug/13/04 - PMD syslog message
	strcpy(pmc->pmc_cmd,scmd);
	pmc->pmc_status = PMC_REQUEST;
	if (!waitAnswer()) {
		puts("Error while sending command");
	}
	else {
		printf("\r\n");
		printf(pmc->pmc_answerBuf);
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
}

#if 0
static int check_outlet_name(char *command) {
	int ok;
	char *nout;
	mountStatusCommand(pmc->pmc_cmd, command);
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;
	if(waitAnswer()) {
		if((nout=strstr(pmc->pmc_answerBuf,command))==NULL) {
			ok=0;
		} else {
			ok=getOutletNumber(nout);
			pmc->pmc_outlet[0] = ok;
			pmc->pmc_outlet[1] = 0;
			ok=1;
		}
	}
	else {
		printf("Error checking outlet string.\n");
		ok=0;
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	return ok;
}

static int expandOutlets(char *command) 
{

	char *pb, *pe;
	long first,last;
	int count=0,index,ok;

	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	 if(strcmp(command,"all")!=0){
		pb = command;
		pe = command;
		last = strtol(pb,&pe,10);
		if(last == 0) {
			ok=check_outlet_name(command);
			if (!ok) {
				printf("\r\nPlease type a valid outlet name.\r\n");
				return (-1);
			} else {
				 return 1;
			}
		}
		while(*pb) {
			last = first = strtol(pb,&pe,10);
			if (first < 1 || first > 128) {
				printf("\r\nPlease type a valid first outlet number.\r\n");
				return (-1);
			}
			if(*pe == '-') {
				last=strtol(pe+1,&pb,10);
				if (last < 1 || last > 128) {
					printf("\r\nPlease type a valid last outlet number.\r\n");
					return (-1);
				}
				pe = pb;
			}
			if (first < 1 || last > 128 || first > last) {
				printf("\r\nPlease type a valid outlet range.\r\n");
				return (-1);
			}
			for (index=first;index<=last && count < PMC_OUTLETS;index++) {
				pmc->pmc_outlet[count] = index;
				count++;
			}
			if (*pe == ',') pe++;
			pb = pe;
		}
	 }
	return count;
}
#else
static int expandOutlets(char *command)
{
	int count = -1;
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	if(! strcmp(command,"all")){
		return 0;
	}
	mountStatusCommand(pmc->pmc_cmd, command);
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;
	if(waitAnswer()) {
		count = string2outlets(pmc->pmc_answerBuf, pmc->pmc_outlet);
		if (count == 0) {
			printf("\r\nInvalid outlet string.\r\n");
			return(-1);
		}
	} else {
		printf("Error checking outlet string.\n");
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	return count;
}

#endif

static pmcInfo *getPmc (void)
{
	return pmc;
}

static void cleanOutlets (void)
{
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
}

static int isAdmin(void)
{
	return admin;
}

static int pm_treat_exit(char **command) 
{
	Exit();
	return 1;
}

static void show_long_help (struct _commands *pcomm)
{
	char *s;
	putchar('\r');
	putchar('\n');
	for (s =pcomm->commandName; *s; s ++) putchar(*s);
	putchar(' ');
	if (pcomm->usage) {
		for (s = pcomm->usage; *s; s ++) putchar (*s);
	}
	putchar('\r');
	putchar('\n');
	if (pcomm->longHelp) {
		for (s = pcomm->longHelp; *s; s ++) putchar (*s);
	} else {
		for (s = pcomm->shortHelp; *s; s ++) putchar (*s);
		putchar('\r');
		putchar('\n');
	}
}

static void show_short_help (struct _commands *pcomm)
{
	int i;
	char *s;
	for (s = pcomm->commandName; *s; s ++) putchar(*s);
	putchar(' ');
	for (i = 0; 
		i < (27 - strlen(pcomm->commandName)); 
		i ++) {
		putchar('-');
	}
	putchar(' ');
	for (s = pcomm->shortHelp; *s; s ++) putchar(*s);
	putchar('\r');
	putchar('\n');
}

static int pm_show_help(char **command) 
{
	int i;
	if (*(command+1)) {
		for (i = 0; i < NUM_BASIC_COMMANDS; i ++) {
			if (pmBasicCommands[i].restricted && ! admin) {
				continue;
			}
			if (! strcmp(*(command + 1), 
				pmBasicCommands[i].commandName)) {
				if (strcmp(*(command + 1), "help")) {
					show_long_help (&pmBasicCommands[i]);
				}
				return 1;
			}
		}
		for (i = 0; i < NUMBER_OF_COMMANDS; i ++) {
			if (pmCommands[i].restricted && ! admin) {
				continue;
			}
			if (! strcmp(*(command + 1), 
				pmCommands[i].commandName)) {
				show_long_help (&pmCommands[i]);
				return 1;
			}
		}
 		printf("\n\rpmCommand: command %s not found\r\n",*(command+1)); 
	} else {
		putchar('\r');
		putchar('\n');
		for (i = 0; i < NUM_BASIC_COMMANDS; i ++) {
			if (pmBasicCommands[i].restricted && ! admin) {
				continue;
			}
			show_short_help (&pmBasicCommands[i]);
		}
		for (i = 0; i < NUMBER_OF_COMMANDS; i ++) {
			if (pmCommands[i].restricted && ! admin) {
				continue;
			}
			show_short_help (&pmCommands[i]);
		}
		printf("Type 'help <command>' to see details of <command>\r\n");
	}
	return 1;
}

static int check_name_outlet(char *outlet, char * name) 
{
	int index;
	char aux[100]={0};
	char *token,*aux2;

	aux2=malloc(100);
	for(index=0;index < MAX_DEV; index++) {
		if (lineconf[index].pmoutlet) {
			strcpy(aux,lineconf[index].pmoutlet);
			token=strtok(aux," ");
#ifdef ONS
			if (lineconf[index].auxflg) {
				sprintf(aux2,"a%d.%s", (ipdu-AUX_OFFSET), outlet);
			} else {
				sprintf(aux2,"s%d.%s",ipdu,outlet);
			}
#else
			sprintf(aux2,"%d.%s",ipdu,outlet);
#endif
			while (token) {
				if (strcmp(token,aux2)==0) {
					if (name) {
						if (lineconf[index].alias) {
							strcpy(name, lineconf[index].alias);
						} else {
							sprintf(name, "PM%s",aux2);
						}
						strcat(name,"\r\n");
					}
					return 1;
				}
				token=strtok(NULL," ,");
			}
		}
	}
	return 0;
}

static int pm_treat_server_name(char **command) 
{
	char name[100];

	if (*(command+1)) {
		memset(name,0,100);
	
		if (check_name_outlet(*(command+1),name)) {
			puts(name);
		} else {
			puts("No Server\r\n");
		}
		return 1;
	}
	return 2;
}

static int pm_treat_whoami(char **command) 
{

	printf("\n\r");
	system("whoami");
	return 1;
}


static int call_pm(char **command) 
{
	char pmmenu[50]; //[RK]Aug/13/04 - memory problem

#ifdef ONS
	sprintf(pmmenu,"pm %1s%d",auxflag ? "a" : "s",
		auxflag ? (ipdu - AUX_OFFSET) : ipdu);
#else
	sprintf(pmmenu,"pm %d",ipdu);
#endif
	system(pmmenu);
	return 1;

}

void call_pmCommand_header(){

	printf("%s",pmCommand_header);
	if(lineconf[ipdu-1].alias) {
		printf("Power Name: %s\r\n\n",lineconf[ipdu-1].alias);  //[GY]2005/Nov/30 BUG#5821
	} 
	else {
#ifdef ONS
        printf("Power Name: %s%d\r\n",
        auxflag ? "AuxPort" : "SerialPort",
        auxflag ? (ipdu-AUX_OFFSET) : ipdu);
		printf("Number of units: %d\r\n",cy_shm->PMD_pms[ipdu-1]);
		printf("%s: %d\r\n\n", auxflag ? "Aux Port" : "Serial Port",
			auxflag ? (ipdu-AUX_OFFSET) : ipdu);
#else
	printf("Power Name: SerialPort%d\r\n",ipdu);
	printf("Number of units: %d\r\n",cy_shm->PMD_pms[ipdu-1]);
	printf("Serial Port: %d\r\n\n",ipdu);
#endif
//	puts(pmCommand_header2);
	}
	puts(pmCommand_header2);  //[GY]2005/Nov/30 BUG#5821
}

//This function cleans backspaces from input
// so ssh/telnet conections don't have problems

void clean_command(char ** command) {

	char * pcmd;

	//while are tokens
	while(*command) {
		//for all caracters in token
		pcmd = *command;
		while (*pcmd) {
			// if the caracter is backspace
			if ( *pcmd == '\b' ) {
				if (pcmd == *command) {
					strcpy(pcmd,pcmd+1);
				} else {
					strcpy(pcmd-1,pcmd+1);
					pcmd --;
				}
				continue;
			} 
			pcmd++;
		}
		command++;
	}
}


int check_pmCommand_command(char **command) 
{
	int value = 1, i;

	clean_command(command);
	if (!*command) {
		return 1;
	} else {
		for (i = 0; i < NUM_BASIC_COMMANDS; i ++) {
			if (! strcmp(*command, pmBasicCommands[i].commandName)) {
				if (pmBasicCommands[i].restricted && !admin) {
					puts("Admin access only");
					if (! strcmp(*command, "getname")) {
						return 0;
					} else {
						return 1;
					}
				}
				if (pmBasicCommands[i].pmCommandTreat) {
					value = (pmBasicCommands[i].pmCommandTreat)(command);
					if (value == 2) {
						show_long_help (&pmBasicCommands[i]);
						return 1;
					}
				}
				return 1;
			}
		}
		for (i = 0; i < NUMBER_OF_COMMANDS; i ++) {
			if (! strcmp(*command, pmCommands[i].commandName)) {
				if (pmCommands[i].restricted && !admin) {
					puts("Admin access only");
					return 1;
				}
				if (pmCommands[i].pmCommandTreat) {
					value = (pmCommands[i].pmCommandTreat)(command);
					if (value == 2) {
						show_long_help (&pmCommands[i]);
						return 1;
					}
					return value;
				}
				sendCommand(command);
				return 1;
			}
		}
		printf("pmCommand: command %s not found\r\n", *command);
	}
	return value;
}

static void call_pmCommand_prompt() 
{
	char *vcmd[10] = {0};
	char command[100];
	char *pcmd=command;
	char *ptoken;
	int quits=0,i;
	while(!quits) {
		memset(pcmd,0,100);
		if(lineconf[ipdu-1].alias) {
			printf("\n\r[%s] ",lineconf[ipdu-1].alias);
		} 
		else {
#ifdef ONS
			printf("\n\r[%s%d] ",auxflag ? "AuxPort" : "SerialPort",
				auxflag ? (ipdu - AUX_OFFSET) : ipdu);
#else
			printf("\n\r[SerialPort%d] ",ipdu);
#endif
		}
		
		if (fgets(pcmd, 100, stdin) == NULL) {
			Exit();
		}

		pcmd[strlen(pcmd) - 1] = 0;
		if (!pcmd[0]) {
			continue;
		}
		ptoken = pcmd;
		i = 0;
		while (*ptoken && i < 10) {
			while (isspace(*ptoken)) ptoken++;
			if (*ptoken == '"') {
				char *s, ch;
				ptoken ++;
				s = vcmd[i ++] = ptoken;
				while ((ch = *ptoken) != 0) {
					ptoken++;
					if (ch == '"') {
                                                if( *ptoken == ':' ) {  //[GY]2005/Dec/08  BUG#5884
                                                   while( *ptoken && !isspace(*ptoken) ) {
                                                      *s++ = *ptoken;
                                                      ptoken++;
                                                   }
                                                }
				     		*s++ = 0;
						break;
					}
					if (ch == '\\') {
						if (! *ptoken) {
							break;
						}
						switch (*ptoken++) {
						case '\\':
							ch = '\\';
							break;
						case '"':
							ch = '"';
							break;
						default:
							ch = 0;
							break;
						}
					}
					if (ch) {
						*s++ = ch;
					}
				}
			} else {
				vcmd[i ++] = ptoken;
				while (*ptoken && !isspace(*ptoken)) ptoken ++;
			}
			if (*ptoken) *ptoken ++ = 0;
		}
		for (; i < 10; i ++) {
			vcmd[i] = 0;
		}
		quits=!check_pmCommand_command(vcmd);
	}
}

int main (int argc, char **argv) 
{
	
	int userPerm;

	if ( argc < 2 ) {
		usage();
		exit(0);
		}

	// Starting the shared memory
	if (shm_init(-1) < 0) {
		printf ("\r\n[Error] - Could not start shared memory.\r\n");
		exit(0);
	}
	if (!check_pmd()) exit(0);

	argv++; /* argv now points to ipdu port number */
#ifndef ONS
	ipdu=atoi(*argv);
#else
	if (!memcmp(*argv, "a", 1)) {
		auxflag = 1;
		ipdu=atoi(*argv+1) + AUX_OFFSET;
	} else if (!memcmp(*argv, "s", 1)) {
		auxflag = 0;
		ipdu=atoi(*argv+1);
	} else {
		printf("error in syntax\n");
		exit(0);
	}
#endif
	if (!check_ipdu(ipdu)) exit(0);
	setSignals();
	userPerm=check_user();
	if (!userPerm) {
		puts("User have no access\r\n");
		exit(0);	
	}
	else {
		atexit(Exit);
		initPmc();
		setbuf(stdout,0);
		if (argc == 2) { 
			call_pmCommand_header();
			call_pmCommand_prompt(); 
		} else {
			argv++;
			check_pmCommand_command(argv); 
			printf("\r\n");
		}
	}
	return 1;
}

/***************************************************************************************************
 FillPmcOultets - this function fills the array pmc->pmc_outlets, which is supposed to contain, in
 each position, the numerical value corresponding to each outlet that the command stored in 
 pmc->pmc_command is using

 pmc -> pointer to the structure containing pmc_command, which is the command for the ipdu in
 string format, and pmc_outlets, which is the list we are going to fullfill with each outlet
 number 
 **************************************************************************************************/
void FillPmcOutlets(pmcInfo *pmc)
{
	char obuf[5] = "";
	int pmcoi = 0; //pmc_outlet index
	char *pcom = pmc->pmc_cmd; 
	char *pbuf = obuf;
	memset(pmc->pmc_outlet,0,sizeof(pmc->pmc_outlet));		
	while(!isdigit(*pcom)){ //find the first number 
		pcom++;
		if(!*pcom) return; //what? no digit found? is there any outlet number?
	}
	while(*pcom){
		while(isdigit(*pcom)){ //copies the outlet number to the temporary buffer
			*pbuf = *pcom;
			pbuf++;
			pcom++;
		}
		//found other thing than number, so we have the outlet number
		//closes the string with the necessary null terminator
		*pbuf = '\0';

		//converts it to integer and puts it into its proper location in 
		//pmc->pmc_outlets
		pmc->pmc_outlet[ pmcoi ] = atoi(obuf);

		//go to next position to insert outlet number in this array
		pmcoi++;

		//reset our buffer pointer
		pbuf = obuf;

		//and looks for the next digit, what will mark the beggining of
		//the next outlet number
		while(!isdigit(*pcom))
			if(*pcom) pcom++;
			else return; //we are finished with the outlets when this line is reached
	}

}


