
/*****************************************************************
* File: pmCommand.h
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
* This program treats the commands pass from user to pm menu and 
* pmCommand. It takes the command, check the arguments and pass to
* pmd daemon, by coping command to shared memory and sending a signal
* to pmd indicating that there is a command to be executed.
*
****************************************************************/

# include <stdio.h>
# include <pwd.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <errno.h>
# include <unistd.h>
# include <termios.h>
# include <linux/ioctl.h>
# include <stdlib.h>
# include <signal.h>
# include <string.h>
# include <ctype.h>

# include <linux/shm.h>
# include <linux/sem.h>
# include <server.h>

/*PMC_CMDSIZE - pmc->pmc_cmd size */
#define CMD 64
/*404 is the maximum # chars used if user choose the all outlets allowed in chain */
#define OUTL 404
/*Tinylogin user structure has a 32-lenght char for username,but pmc->pmc_username uses 31 */
#define NAME 31
#define OPT 10

struct _commands;

struct _commands {
	int restricted;
	char *menuName;
	char *commandName;
	char *usage;
	char *shortHelp;
	char *longHelp;
#ifdef PMCOMMAND
	int (*pmCommandTreat) (char **);
#else
	int (*pmTreat) (struct _commands *);
#endif
};

#ifdef PMCOMMAND
static char pmCommand_header[] = 
	"\r\n"
	"-----------------------------------------------------------------------\r\n" 
//	"Cyclades Corporation \r\n"  [GY]2005/Nov/22  BUG#5658
	"Power Management Command Prompt v1.1 \r\n" 
	"-----------------------------------------------------------------------\r\n";

#else
static char pm_header[] = 
	"\r\n"
        "--------------------------------------------------------------------- \r\n"
//	"Cyclades Corporation \r\n"  [GY]2005/Nov/22  BUG#5658
	"Power Management Menu v1.0\r\n"
        "--------------------------------------------------------------------- \r\n";
static char pm_prompt_header[] =
	"\r\n"
        "--------------------------------------------------------------------- \r\n"
	"Power Management Menu ";  //[GY]2005/Nov/22  BUG#5658

//[GY]2005/Nov/11 
char pm_menu_multi_1[] =
        "--------------------------------------------------------------------- \r\n"
        "1. Return\t4. Cycle\t6. Unlock\t\t8. Show \r\n"
        "2. On\t\t5. Lock\t\t7. Status\t\t9. Info \r\n"
        "3. Off";

//[GY]2005/Nov/11 
char pm_menu_multi_1_desc[] =
        "--------------------------------------------------------------------- \r\n"
        "1. Return: takes you back to the last menu.\r\n"
        "2. On: issue an \"on\" on a specified multi-device outlet. Its name will\r\n"
        "   be asked if not passed already as an argument (a string with the\r\n"
        "   alias name). This command will be translated into multiple on\r\n"
        "   commands respecting the sequence given in the sXX.outlets parameter\r\n"
        "   for the outlets in the same ipdu.\r\n"
        "3. Off: will turn off all the outlets belonging to this multi-outlet.\r\n"
        "4. Cycle: executes a cycle on the multi-outlet device, which has the\r\n"
        "   same effect of an off followed by an on at the multi-outlet.\r\n"

        "5. Lock:  will  block this  multi-outlet  from   issuing any  of  prior\r\n"
        "   commands by an individual lock on each of the outlets.\r\n"
        "6. Unlock: will unlock all the outlets belonging tothis multi-outlet.\r\n"
        "7. Status: will issue individual status commands on each of the outlets\r\n"
        "   belonging to this multi-outlet.\r\n"
        "8. Show:  will  show  which  outlets  in  which  ipdus belong  to  this\r\n"
        "   multi-outlet.\r\n"
        "9. Info: will show this help text explaining each one of the options.";

#endif

static char pmCommand_help_alarm[] =
	"Set and read the current threshold. The current threshold is the maximum \r\n"
	"allowed current on the power strip. \r\n"
	"If the IPDU number is specified, shows only the current threshold of this IPDU. \r\n"
	"If the threshold is specified, sets the current threshold of the IPDU. ";

static char pmCommand_help_buzzer[] =
	"Display or change the buzzer notification status. ";

static char pmCommand_help_current[] =
	"Display the total current draw by power distribution units and the maximum \r\n"  //[GY]2005/Nov/22  BUG#5658
	"current the unit has drawn.";  //[GY]2005/Nov/22  BUG#5658

static char pmCommand_help_currseg[] =
	"Display the current drawn by the IPDU in one of the segments. ";

static char pmCommand_help_currentprotection[] =
	"Turn on and off the over current protection feature. The over current protection don't let any outlet be turned on if the current draw by the unit is bigger then the current threshold configured with the command alarm. This command can also be used to display the status of this feature. ";

static char pmCommand_help_cycle[] =
	"Power cycle an outlet or groups of outlets. You can use the outlet number or the outlet name. ";

static char pmCommand_help_display[] =
	"Display or change the LED display mode. ";

static char pmCommand_help_factory_default[] =
	"Bring the unit to its factory configuration. ";

static char pmCommand_help_id[] =
	"Display or change the IPDU identification name. ";

static char pmCommand_help_interval[] =
	"Configure the power up interval for each outlet in the system. The power up interval of an outlet is the time the unit will wait, after turning this outlet on, to turn on any other outlet. ";

static char pmCommand_help_lock[] =
	"Lock an outlet or group of outlets in the current state. ";

//[GY]2005/Dec/20  BUG#4727
static char pmCommand_help_name[] =
//   	"Name an outlet. A maximum of eight characters (not case sensitive) is allowed. ";
        "Name an outlet. For PM firmware version until V_1.6.0 a maximum of  eight\r\n"         
        "characters is allowed. From version V_1.6.0 inclusive, sixteen characters\r\n"
        "are allowed.";

static char pmCommand_help_on[] =
	"Turn an outlet or group of outlets on. ";

static char pmCommand_help_off[] =
	"Turn an outlet or group of outlets off. ";

static char pmCommand_help_reboot[] =
	"Reboot the specified unit or reboot the master unit if no argument is provided. ";

static char pmCommand_help_restore[] =
	"Restore the configuration currently saved in flash. \r\n";

static char pmCommand_help_save[] =
	"Saves the current configuration in the flase device built in to all AltherPath PM units. ";

static char pmCommand_help_status[] =
	"Display the status of an outlet or group of outlets.";

static char pmCommand_help_syslog[] =
	"Display or change the syslog notification status.";

static char pmCommand_help_temperature[] =
	"Display the temperature in AltherPath PM unit surrounding and the maximum temperature the unit has registered (temperature peak).";

static char pmCommand_help_unlock[] =
	"Unlock the state of an outlet or group of outlets.";

#ifdef PMCOMMAND
static int isAdmin(void);
static void cleanOutlets (void);
static void Exit();
static int check_name_outlet(char *outlet, char * name);
static void sendCommand(char **command);
static int expandOutlets(char *command);
static pmcInfo *getPmc (void);
static int  waitAnswer();

#define pm_treat_threshold	NULL
#define pm_treat_id		NULL
#define pm_treat_display	NULL

static int pm_treat_outlet(char **command) 
{
	int outs = 0;              //[GY]2005/Dec/06  BUG#5884
        pmcInfo *pmc = getPmc ();  //[GY]2005/Dec/06  BUG#5884

	if (*(command+1)) {
           outs = expandOutlets(*(command+1));
           if( outs == -1 ) return 1;
           if( outs == 1 ) sprintf(*(command+1), "%d", pmc->pmc_outlet[0]);  //[GY]2005/Dec/06  BUG#5884 (end)
	   sendCommand(command);
	   cleanOutlets();
	   return 1;
	} else {
	   return 2;
	}
}

static int pm_treat_name(char **command) 
{
	int outs = 0, ret = 1;
	if (*(command+1)) {
		outs = expandOutlets(*(command+1));
	}
	if((outs == 1) && *(command+1) && *(command+2)) {
		if (!check_name_outlet(*(command+1),NULL)) {
                   if( isdigit(*(*(command+2))) ) {  //[GY]2005/Dec/06 BUG#5884
                      puts("\r\nPlease type a valid outlet string.\r\n");
                      return ret;
                   }
			sprintf(*(command+1), "%d", (getPmc())->pmc_outlet[0]);
			sendCommand(command);	
		}
		else {
			puts("\r\nThis outlet is associated to a port server\r\n");
		}
	}
	else {
	 	if (outs>0 || !*(command+1))
			ret = 2;
	}
	cleanOutlets();
	return ret;
}

static int pm_treat_status(char **command) 
{
	int outs = 0;
        pmcInfo *pmc = getPmc ();  //[GY]2005/Dec/05  BUG#5884

	if (*(command+1)) {
	   if( !isAdmin() ) {  //[GY]2005/Dec/05  BUG#5884 (start)
	      outs = expandOutlets(*(command+1));
	   }
           if( outs == -1 ) return 1;
           if( outs == 1 ) sprintf(*(command+1), "%d", pmc->pmc_outlet[0]);  //[GY]2005/Dec/05  BUG#5884 (end)
	   sendCommand(command);
	   return 1;
	}
	return 2;
}

static int pm_treat_curr_temp(char **command) 
{
	if (*(command+1) && (strcmp(*(command+1),"reset") == 0 && 
		! isAdmin() )) {
		puts("Admin access only");
	} else {
		sendCommand(command);
	}
	return 1;
}
#define pm_treat_currseg	pm_treat_curr_temp

static int pm_treat_reboot(char **command) 
{
	pmcInfo *pmc = getPmc ();
	puts("Sending reboot");
	strcpy(pmc->pmc_cmd,*command);
	if (*(command+1)) {
		strcat(pmc->pmc_cmd," ");
		strcat(pmc->pmc_cmd,*(command+1));
	}
	pmc->pmc_status = PMC_REQUEST;
	while(1) {
		if(! waitAnswer()) {
			puts("Error while waiting answer");
		} else if(strstr(pmc->pmc_answerBuf,"not available")!=NULL ||
			strstr(pmc->pmc_answerBuf,"not ready")!=NULL) {
			pmc->pmc_answerBuf[0]=0;
			strcpy(pmc->pmc_cmd,"status all");
			pmc->pmc_status = PMC_REQUEST;
		} else {
			pmc->pmc_status=PMC_IDLE;
			pmc->pmc_answerBuf[0]=0;
			puts("Reboot done");
			return 1;
		}
	}
	return 1;
}

static int pm_treat_on_off(char **command) 
{
	if(! strcmp(*command,"buzzer") ||
		! strcmp(*command,"currentprotection")) {
		if( *(command + 1) == NULL ) {
			*(command + 1) = "status";
			*(command + 2) = NULL;
		}
	}
	sendCommand(command);
	return 1;
}

#else
static pmcInfo *getPmc (void);
static int isAdmin(void);
static void cleanOutlets (void);
static void Exit();
static int check_name_outlet(char *outlet);
static void sendCommand(char *command);
static int expandOutlets(char *command);
static int  waitAnswer();
static char *read_string(char *string,int size);
static int check_outlet_name(char *command); 

static char pm_outlet_request[] = 
	"\r\n"
	"Outlet name or outlet number(? for help, m for main menu): ";

static char pm_outlet_desc_n[] =
	"\r\n"
	"Please enter an outlet number, an outlet list or an outlet name. \r\n"
	"Outlet number is the outlet's number in IPDU. \r\n"
	"Outlet name is the name associated to an outlet in IPDU or the alias \r\n"
	"for server connected to this outlet\r\n"
	"An outlet list is a list of outlet numbers separated by comas \r\n"
	"(indicating outlets) or dash (indicating a range). \r\n"
	"Ex: 2,3,5 - indicates outlets 2, 3 and 5 \r\n"
       	"	2,3-5 - indicates outlets 2 and 3 through 5 (3, 4 and 5) ";

static char pm_status_outlet_request[] = "\r\n"
	"Outlet name or outlet number(all for all, ? for help, m for main menu): ";

static char pm_status_desc[] =
	"\r\n"
	"Please enter an outlet number or an outlet list. \r\n"
	"Outlet number is the outlet's number in IPDU. \r\n"
	"An outlet list is a list of outlet numbers separated by comas \r\n"
	"(indicating outlets) or dash (indicating a range). \r\n"
	"Ex: 2,3,5 - indicates outlets 2, 3 and 5 \r\n"
	"	2,3-5 - indicates outlets 2 and 3 through 5 (3, 4 and 5) ";

static char pm_outlet_desc[] =
	"\r\n"
	"Please enter an outlet number or an outlet name. \r\n"
	"Outlet number is the outlet's number in IPDU. ";

static unsigned char pm_on_off_request[] = "\r\n"
	"On|Off(none for check actual state, on to activate, off to deactivate): ";
static char pm_threshold_request[] = "\r\n"
	"Threshold value(enter to check current value): ";

static char pm_id_request[] = "\r\n"
	"IPDU identification name(enter to check current ID): ";

static char pm_displaymode1_request[] = "\r\n"
	"Display mode(0, 180, enter to check current mode): ";

static char pm_displaymode2_request[] = "\r\n"
	"Display mode(Imanual, Iauto, enter to check current mode): ";

static char pm_displaytime_request[] = "\r\n"
	"Display cycle time(in seconds): ";

static char pm_ipdu_request[] = "\r\n"
	"IPDU number(none for all, reset to reset all units, ? for help, m for main menu): ";

static char pm_ipdu_request2[] = "\r\n"
	"IPDU number(none for all, ? to help, m for main menu): ";

//[RK]Nov/29/05 
static char pm_ipdu_request3[] = "\r\n"
	"IPDU number(none for the master unit, ? to help, m for main menu): ";

static char pm_reboot_desc[] =
	"\r\n"
	"Please enter the IPDU number in the chain for a specific unit or blank if you want reboot the master unit. ";

static char pm_alarm_desc_1[] =
	"\r\n"
	"Please enter the IPDU number in chain or blank for check all units.";

static char pm_current_desc[] =
	"\r\n"
	"Please enter blank if you want check the value for all\r\n"
	" units or IPDU number in chain for a specific unit. \r\n"
	"Reset option is allowed for admin user only.";

static unsigned char pm_phase_request[] = "\r\n"
	"Phase (A, B, C, none for all phases): ";

static unsigned char pm_segment_request[] = "\r\n"
	"Segment(A, B, reset to reset all segments, m for main menu): ";

static int getNumberOfPhases(char *ipdu)
{
	int nphases = 1;
	char *nout;
        pmcInfo *pmc = getPmc ();
	snprintf(pmc->pmc_cmd, CMD-1, "ver %s", ipdu);
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;
	if(waitAnswer()) {
		nout = strstr(pmc->pmc_answerBuf, "no IPDU");
		if (nout) {
			printf(pmc->pmc_answerBuf);
		}
		nout = strstr(pmc->pmc_answerBuf, " phases");
		if (nout) {
			nout --;
			while (*nout && !isspace(*nout) && *nout != '(') 
				nout --;
			nout ++;
			if (isdigit(*nout)) {
				nphases = atoi(nout);
			}
		}
	}
	else {
		printf("Error checking IPDU model.\n");
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	return nphases;
}

static int pm_treat_outlet(struct _commands *pcomm) 
{
	char command[CMD]={0};
	char outlet[OUTL]={0},interval[OPT]={0};
//	int outs=0;  //[GY]2005/Dec/06  warning: unused variable
//	long int last;  //[GY]2005/Dec/06  warning: unused variable

	printf(pm_outlet_request);
	if (read_string(outlet, OUTL) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(outlet,"?",1)==0) {
		puts(pm_outlet_desc_n);
	} else if( (strncmp(outlet,"m",1) == 0) && (outlet[1] == '\0') ) {  //[GY]2005/Nov/22
		return 1;
	} else if(outlet[0]) {
		strcpy(command, pcomm->commandName);
		if (! strcmp(pcomm->commandName, "interval")) {
			printf("\r\n%s (in seconds): ", pcomm->menuName);
			if (read_string(interval, OPT) == NULL) {
				Exit();
			}
		}
		strncat(command," ",CMD-strlen(command)-1);
		strncat(command,outlet,CMD-strlen(command)-1);
		if (interval[0]) {
			strncat(command," ",CMD-strlen(command)-1);
			strncat(command,interval,CMD-strlen(command)-1);
		}
		sendCommand(command);
		cleanOutlets();
		return 1;
	}
	return 0;
}

static int pm_treat_status(struct _commands *pcomm) {
	char command[CMD]={0};
	char outlet[OUTL];

	printf(pm_status_outlet_request);
	if (read_string(outlet, OUTL) == NULL) {
		Exit();
	}
	if (strncmp(outlet,"?",1)==0) {
		puts(pm_status_desc);
	} else if( (strncmp(outlet,"m",1) == 0) && (outlet[1] == '\0') ) {  //[GY]2005/Nov/22
		return 1;
	} else if (outlet[0]) {
		strcpy(command, pcomm->commandName);
		strncat(command," ",CMD-strlen(command)-1);
		strncat(command,outlet,CMD-strlen(command)-1);
		sendCommand(command);
		return 1;
	}
	return 0;
}

static int pm_treat_name(struct _commands *pcomm) 
{
	int outs;
	char outlet[OUTL],name[NAME],command[CMD];
	
	printf(pm_outlet_request);
	if (read_string(outlet, OUTL) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(outlet,"?",1)==0) {
		puts(pm_outlet_desc);
	} else if( (strncmp(outlet,"m",1) == 0) && (outlet[1] == '\0') ) {  //[GY]2005/Nov/22
		return 1;
        } else if (outlet[0]) {   //[GY]2005/Nov/22
//   	} else if(!check_name_outlet(outlet)) {
	    if(!check_name_outlet(outlet)) {
              do {  //[GY]2005/Nov/22
                 printf("\r\nOutlet name: ");
                 if (read_string(name, NAME) == NULL) {
                         Exit();
                         break;
                 }
              } while( isspace(name[0]) || isdigit(name[0]) || name[0] == '\0' );
	      outs=expandOutlets(outlet);
	      if(outs==1) {
		sprintf(command,"%s %s %s",pcomm->commandName,outlet,name);
		sendCommand(command);
	      } else {
		puts(pm_outlet_desc);
	      }
              return 1;
	    } else {
	      printf("\r\nThis port is associated to a server\r\n");
              return 1;
	    }
        }
	return 0;
}

static int pm_treat_curr_temp(struct _commands *pcomm)
{
	char command[CMD], ipdu[OPT], phase[OPT];
	int nphases;

	printf(pm_ipdu_request);
	if (read_string(ipdu, OPT) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(ipdu,"?",1)==0) {
		puts(pm_current_desc);
		return 0;
	}
	if (strncmp(ipdu,"m",1)==0) {
		return 1;
	}
	if(strcmp(ipdu,"reset")==0 && !isAdmin()) {
		puts("\n\rAdmin access only");
		return 1;
	}
	strcpy(command, pcomm->commandName);
	if (ipdu[0]) {
		strcat(command," ");
		strcat(command,ipdu);

		if (! strcmp(pcomm->commandName, "current")
			&& ((nphases = getNumberOfPhases(ipdu)) == 3)) {

			printf(pm_phase_request);
			if (read_string(phase, OPT) == NULL) {
				Exit();
				return 1;
			}
			if (phase[0]) {
				strcat(command," ");
				strcat(command,phase);
			}
		}
	}
	sendCommand(command);
	return 1;
}

static int pm_treat_currseg(struct _commands *pcomm)
{
	char command[CMD], segm[OPT];
	printf(pm_segment_request);
	if (read_string(segm, OPT) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(segm,"m",1)==0) {
		return 1;
	}
	if(strcmp(segm,"reset")==0 && !isAdmin()) {
		puts("\n\rAdmin access only");
		return 1;
	}
	strcpy(command, pcomm->commandName);
	strcat(command," ");
	strcat(command,segm);
	sendCommand(command);
	return 1;
}

static int pm_treat_on_off(struct _commands *pcomm)
{
	char command[CMD];
	char onoff[OPT]={0};

   	printf(pm_on_off_request);
	if (read_string(onoff, OPT) == NULL) {
		Exit();
	}	
	strcpy(command, pcomm->commandName);
	if(onoff[0]) {
		strcat(command, " ");
		strcat(command, onoff);
	} else if (! strcmp(pcomm->commandName, "buzzer") || 
		! strcmp(pcomm->commandName, "currentprotection")) {
		strcat(command, " status");
	}
	sendCommand(command);
	return 1;
}

static int pm_treat_reboot(struct _commands *pcomm)
{
	char ipdu[OUTL]={0}, command[CMD];
	pmcInfo *pmc = getPmc ();

	printf(pm_ipdu_request3); //[RK]Nov/29/05
	if (read_string(ipdu, OUTL) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(ipdu,"?",1)==0) {
		puts(pm_reboot_desc);
	} else if (strncmp(ipdu,"m",1)==0) {
		return 1;
	} else {
		strcpy(command, pcomm->commandName);
		if (ipdu[0]) {
			strcat(command," ");
			strcat(command,ipdu);
		}
		strcpy(pmc->pmc_cmd,command);
		pmc->pmc_status = PMC_REQUEST;
		while (1) {
			if (! waitAnswer()) {
				puts("Error while waiting answer");
			} else if (strstr(pmc->pmc_answerBuf,"not available")!=NULL || 
				strstr(pmc->pmc_answerBuf,"not ready")!=NULL) {
				pmc->pmc_answerBuf[0]=0;
				strcpy(pmc->pmc_cmd,"status all");
				pmc->pmc_status = PMC_REQUEST;
			} else {
				pmc->pmc_status=PMC_IDLE;
				pmc->pmc_answerBuf[0]=0;
				puts("Reboot done");
				return 1;
			}
		}
	}
	return 0;
}

static int pm_treat_threshold(struct _commands *pcomm)
{
	char command[CMD], tshold[OPT], ipdu[OPT]={0}, phase[OPT];
	int nphases;

	printf(pm_ipdu_request2);
	if (read_string(ipdu, OPT) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(ipdu,"?",1)==0) {
		puts(pm_alarm_desc_1);
		return 0;
	}
	if (strncmp(ipdu,"m",1)==0) {
		return 1;
	}
	strcpy(command, pcomm->commandName);
	if (ipdu[0]) {
		strcat(command, " ");
		strcat(command, ipdu);
		printf(pm_threshold_request);
		if (read_string(tshold, OPT) == NULL) {
			Exit();
			return 1;
		}
		if (*tshold) {
			strcat(command," ");
			strcat(command,tshold);
			if ((nphases = getNumberOfPhases(ipdu)) == 3) {
				printf(pm_phase_request);
				if (read_string(phase, OPT) == NULL) {
					Exit();
					return 1;
				}
				if (phase[0]) {
					strcat(command," ");
					strcat(command,phase);
				}
			}
		}
	}
	sendCommand(command);
	return 1;
}

static int pm_treat_id(struct _commands *pcomm)
{
	char command[CMD], id[OPT], ipdu[OPT]={0};

	printf(pm_ipdu_request2);
	if (read_string(ipdu, OPT) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(ipdu,"?",1)==0) {
		puts(pm_alarm_desc_1);
		return 0;
	}
	if (strncmp(ipdu,"m",1)==0) {
		return 1;
	}
	strcpy(command, pcomm->commandName);
	if (ipdu[0]) {
		strcat(command, " ");
		strcat(command, ipdu);
		printf(pm_id_request);
		if (read_string(id, OPT) == NULL) {
			Exit();
			return 1;
		}
		if (id[0]) {
			strcat(command, " ");
			strcat(command, id);
		}
	}
	sendCommand(command);
	return 1;
}

static int pm_treat_display(struct _commands *pcomm)
{
	char command[CMD], mode[OPT]={0}, time[OPT]={0}, ipdu[OPT]={0};
	int nphases;

	printf(pm_ipdu_request2);
	if (read_string(ipdu, OPT) == NULL) {
		Exit();
		return 1;
	}
	if (strncmp(ipdu,"?",1)==0) {
		puts(pm_alarm_desc_1);
		return 0;
	}
	if (strncmp(ipdu,"m",1)==0) {
		return 1;
	}
	strcpy(command, pcomm->commandName);

	if (ipdu[0]) {
		strcat(command, " ");
		strcat(command, ipdu);
		if ((nphases = getNumberOfPhases(ipdu)) == 3) {
			printf(pm_displaymode2_request);
		} else {
			printf(pm_displaymode1_request);
		}
		if (read_string(mode, OPT) == NULL) {
			Exit();
			return 1;
		}
		if (mode[0]) {
			strcat(command," ");
			strcat(command,mode);
			if (nphases == 3 && ! strcmp(mode, "Iauto")) {
				printf(pm_displaytime_request);
				if (read_string(time, OPT) == NULL) {
					Exit();
					return 1;
				}
				if (time[0]) {
					strcat(command," ");
					strcat(command,time);
				}
			}
		}
	}

	sendCommand(command);
	return 1;
}
#endif

struct _commands pmCommands[] = {
	{0, "On", "on", "<outlet>", 
		"Turn on outlets", 
		pmCommand_help_on, pm_treat_outlet},
	{0, "Off", "off", "<outlet>", 
		"Turn off outlets", 
		pmCommand_help_off, pm_treat_outlet},
	{0, "Cycle", "cycle", "<outlet>", 
		"Power cycle outlets", 
		pmCommand_help_cycle, pm_treat_outlet},
	{0, "Lock", "lock", "<outlet>", 
		"Lock outlets in current state", 
		pmCommand_help_lock, pm_treat_outlet},
	{0, "Unlock", "unlock", "<outlet>", 
		"Unlock outlets", 
		pmCommand_help_unlock, pm_treat_outlet},
	{0, "Status", "status", "<outlet>", 
		"Display state of the outlets", 
		pmCommand_help_status, pm_treat_status},
	{0, "Power Up Interval", "interval", "[<outlet>] [<time>]", 
		"Set/read the power up interval", 
		pmCommand_help_interval, pm_treat_outlet},
	{0, "Name", "name", "<outlet> <new name>", 
		"Name an outlet", 
		pmCommand_help_name, pm_treat_name},
	{0, "Current", "current", "[<IPDU number in the chain>]", 
		"Set/Read/Reset the current", 
		pmCommand_help_current, pm_treat_curr_temp},
	{0, "Temperature", "temperature", "[<IPDU number in the chain>]", 
		"Set/Read/Reset the temperature", 
		pmCommand_help_temperature, pm_treat_curr_temp},
	{0, "Version", "ver", NULL, 
		"Show the software and hardware version", 
		NULL, NULL},
	{1, "Buzzer", "buzzer", "[<on|off>]",
		"Set/read the buzzer", 
		pmCommand_help_buzzer, pm_treat_on_off},
	{1, "Current Protection", "currentprotection", "[<on|off>]",
		"Set/read the over current protection", 
		pmCommand_help_currentprotection, pm_treat_on_off},
	{1, "Factory Default", "factory_defaults", NULL,
		"Bring the unit to factory configuration",
		pmCommand_help_factory_default, NULL},
	{1, "Reboot", "reboot", "[<IPDU number in the chain>]",
		"Reboot the units in chain",
		pmCommand_help_reboot, pm_treat_reboot},
	{1, "Restore", "restore", NULL,
		"Restore the configuration in flash",
		pmCommand_help_restore, NULL},
	{1, "Save", "save", NULL,
		"Save the current configuration in flash",
		pmCommand_help_save, NULL},
	{1, "Syslog", "syslog", "[<on|off>]",
		"Set/read the syslog", 
		pmCommand_help_syslog, pm_treat_on_off},
	{1, "Alarm", "alarm", "[<IPDU number in the chain> [<threshold>]]",
		"Set/read the alarm status", 
		pmCommand_help_alarm, pm_treat_threshold},
	{1, "ID", "id", "[<IPDU number in the chain> [<ID>]]",
		"Set/read the IPDU ID",
		pmCommand_help_id, pm_treat_id},
	{0, "Current(segment)", "currseg", "<segment>",
		"Read/reset the current in each segment",
		pmCommand_help_currseg, pm_treat_currseg},
	{1, "Display", "display", "[<IPDU number in the chain>[ <mode>]]",
		"Set/read the display mode",
		pmCommand_help_display, pm_treat_display},
};

#define NUMBER_OF_COMMANDS	(sizeof(pmCommands)/sizeof(struct _commands))

void mountStatusCommand(char *command, char *outlet)
{
	strcpy(command, "status ");
	strncat(command, outlet, CMD-strlen(command)-1);
}

int string2outlets(char *buf, char *outlets)
{
	char *bl = buf, *el, *pl;
	int count = 0;
	while (*bl) {
		/* Check for end of line */
		for (el = bl; *el && *el != '\n'; el ++);
		for (pl = bl; isspace(*pl); pl ++);
		if (isdigit(*pl)) {
			outlets[count ++] = (char)atoi(pl);
		}
		if (* el) {
			el ++;
		}
		bl = el;
	}
	return count;
}
