/*****************************************************************
* File: pmd.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
*
****************************************************************/

# define PMD

// Linux headers
# include <stdio.h>
# include <stdlib.h>
#include <fcntl.h>
# include <signal.h>
# include <unistd.h>
# include <string.h>
# include <linux/shm.h>
# include <linux/sem.h>
# include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <signal.h>
#include <errno.h>

// Cyclades Headers
# include <server.h>	// Global definitions for cy_buffering shared memory
# include <pmd.h>	// Definition for PMD software

/* Which software interrupts were detected */                                   
static int last_sig; //[RK]Aug/09/04
static int got_sig;
#define	GOT_SIGIO	1	/* This signal comes either from the 
				   pm/pmCommand, or from the socket session, 
				   requesting the pmd to treat a command. */
#define	GOT_SIGINT	2	/* This signal causes the PMD to exit. */
#define	GOT_SIGHUP	4	/* This signal comes from cy_ras, when a 
				   configuration related to a IPDU port is 
				   changed. */
#define GOT_SIGGEN	8 	//[RK]Aug/09/04
#define	GOT_ALARM	16

#define MAX_DEV 	cy_shm->max_dev

#ifdef ON_SEQ
static void IpduBreakOn2(int index);
static char *queuedons(int index, char *username);
static void comma2space( char *string);

struct Tonqueue *vet_onq[MAXPMDACCESS] = {NULL};
#endif

/******************************************************************************
 * It is the handler routine for the signals. If it receives a SIGIO it treat
 * the data receive in the shared memory. If it receives a SIGINT it set the
 * variable fim to 1, what means that the program will finish.
 ******************************************************************************/
static void treatSignals( int sig ) {
	struct sigaction act;

	memset((char*)&act,0,sizeof(struct sigaction));

	switch (sig) {
	case SIGIO:
		got_sig |= GOT_SIGIO;
		break;
	case SIGINT:
		got_sig |= GOT_SIGINT;
		break;
	case SIGALRM:
		got_sig |= GOT_ALARM;
		break;
	case SIGUSR2:
		got_sig |= GOT_SIGHUP;
		break;
	case SIGUSR1:
		debugFlg = (debugFlg) ? 0 : 1;
		break;
	default://[RK]Aug/09/04
		last_sig = sig;
		got_sig |= GOT_SIGGEN;
	}
	act.sa_handler = treatSignals;
	sigaction(sig, &act, 0);
	unblock(sig);
}

/******************************************************************************
 * This routine initializes the signal treatment. It starts up the reception
 * of SIGINT, SIGALRM and SIGIO.
 ******************************************************************************/
static int initSigTreatment() {
	int i;
	struct sigaction act;

	memset((char*)&act,0,sizeof(struct sigaction));
	// Initializing the struct.	
	act.sa_handler = treatSignals;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;

	// Requiring SIGINT's and SIGIO's
/* [RK]Aug/09/04
	if (sigaction(SIGIO, &act, 0)) return -1;
	if (sigaction(SIGINT, &act, 0)) return -1;
	if (sigaction(SIGUSR2, &act, 0)) return -1;
	if (sigaction(SIGUSR1, &act, 0)) return -1; //[RK]Aug/09/04 - debug/trace
	signal(SIGHUP, SIG_IGN);
	unblock(SIGIO);
	unblock(SIGINT);
	unblock(SIGUSR2);
	unblock(SIGUSR1); //[RK]Aug/09/04 - debug/trace
	unblock(SIGHUP);
*/
	for (i=1; i < 64; i++) {
		if (i == SIGPIPE) continue;
		if (i == SIGWINCH) continue;
		if (i == SIGSEGV) continue;
		if (i == SIGFPE) continue;
		if (i == SIGILL) continue;
		if (i == SIGKILL) continue;
		if (i == SIGCHLD) continue;

		sigaction(i,&act,0);
		unblock (i);	
	}
	return 0;
}

/******************************************************************************
 * This routine check all ipduList structures that are not initialized to
 * check if someone plugged an ipdu on the port after the pmd had started.
 * It checks all the serial ports configured as ipdu.
 ******************************************************************************/

static IpduUserList * initIpduUserList(char *pmuser)
{
	int first,last;
	int j,nusr;
	char *pi,*pn;
	char aux[50];
	IpduUserList *plist,*info;

	info = plist = (IpduUserList *)malloc(sizeof(IpduUserList) * 128);
	memset((char *)plist,0,sizeof(IpduUserList) * 128);
	
	nusr=0;
	pi = pmuser;
	while (*pi) {
		while (*pi && (*pi == ' ')) pi++;
		pn = pi;
		while(*pi && (*pi!=':')) pi++;
		if (!*pi) break;
		strncpy(plist->username,pn,pi-pn);
		pi++;	// skip :
		while (*pi && (*pi == ' ')) pi++;
		pn = pi;
		while (*pi && (*pi != ';')) pi++;
		memset(aux,0,50);
		strncpy(aux,pn,pi-pn);
		if (*pi) pi++;
		pn = aux; 
		while (*pn) {
			while(*pn && (*pn == ' ')) pn++;
			if (sscanf(pn,"%d-%d",&first,&last) == 2) {
				for (j=first; j <=last ; j++) {
					plist->outlets[j] = 1;
				}
			} else {
				sscanf(pn,"%d",&first);
				plist->outlets[first] = 1;
			}
			while(*pn && (*pn != ',')) pn++;
			if (*pn) pn++;
		}
		plist++;
		if (++nusr > 128) {
			printf ("\r\n[PMD] Error - Configuration problem - exceeded the max users in pmusers.\r\n");
			break;
		}
	}
	return info;
}

static int alarm_set = 0;

static void set_alarm(void)
{
	if (alarm_set) return;

	alarm_set = 1;
	alarm(10);
}

static void start_ipdu (int port) 
{
	struct line_cfg * line;
	ipduInfo *ipdu = ipduList[port];
	ipduInfo2 *ipdu2;

	line = &cy_shm->line_conf[port];
	if (ipdu) {
		//dolog(LOG_NOTICE, "pmd.c:main() - IPDU already initialized in port %d", port + 1);
#ifndef ONS
		dodebug("PMD[%d]:start_ipdu - IPDU already initialized\n", port + 1);
#else
		dodebug("PMD[%s%d]:start_ipdu - IPDU already initialized\n", 
			line->auxflg ? "a" : "s", line->auxflg ?
			(port+1-AUX_OFFSET) : (port+1));
#endif
		return;
	}

	if (line->protocol != P_IPDU ||
		!line->tty || !*line->tty) {
		return;
	}
	//dolog(LOG_INFO, "pmd.c main() - Starting IPDU %d", port + 1);
#ifndef ONS
	dodebug("PMD[%d]:start_ipdu - Starting IPDU\n", port + 1);
#else
	dodebug("PMD[%s%d]:start_ipdu - Starting IPDU\n",
		line->auxflg ? "a" : "s", line->auxflg ? (port+1-AUX_OFFSET) : (port+1));
#endif

	if (cy_shm->vet_pid[port] && (cy_shm->vet_pid[port] != getpid())) {
		if ((kill(cy_shm->vet_pid[port],0) < 0) && (errno == ESRCH)) {
			cy_shm->vet_pid[port] = 0;
		} else {
			cy_shm->vet_chg[port] |= 2;
			set_alarm();
			return;
		}
	}

	cy_shm->vet_pid[port] = getpid();
        cy_shm->tty_user[port] = shm_strdup("IPDU");

	if (cy_shm->sess_list[port]) {
		//dolog(LOG_NOTICE, "Parameter sess_list is different of zero for the port %d" , port + 1);
		dodebug("Parameter sess_list is different of zero\n");
		shm_free_str((char *)cy_shm->sess_list[port]);
		cy_shm->sess_list[port] = NULL;
	}
	// Allocating memory for this struct
	ipdu = malloc(sizeof(ipduInfo));
	if (!ipdu) {
		//dolog(LOG_ERR, "Could not allocate memory for the IPDU %d.", port + 1);
		dodebug("Could not allocate memory for the IPDU.\n");
		return;
	}
	memset(ipdu,0,sizeof(ipduInfo));
	// Getting the configuration for this IPDU
	if ( initIpduStruct(ipdu, cy_shm->line_conf[port].pmtype) == -1 ) {
		//dolog(LOG_ERR, "Could not read configuration for the IPDU %d.", port + 1);
#ifndef ONS
		dodebug("PMD[%d]: Could not read configuration for the IPDU.\n", port + 1);
#else
		dodebug("PMD[%s%d]: Could not read configuration for the IPDU.\n",
		line->auxflg ? "a" : "s", line->auxflg ? (port+1-AUX_OFFSET) : (port+1));
#endif
			
		free(ipdu);
		return;
	}

	if ((ipdu2 = initIpdu2Struct(ipdu)) == NULL) {
		//dolog(LOG_ERR, "Could not allocate shared memory.");
		dodebug("Could not allocate shared memory.\n");
		cleanIpduStruct(ipdu);
		return;
	}
	// Setting up the serialPort variable
#ifdef ONS
	ipdu->auxflg = line->auxflg;
#endif
	ipdu->serialPort = port + 1;
	// Initialize bufindex;
	ipdu->bufindex = 0;
	// Initialize pmindex;
	ipdu->pmindex = -1;
	ipdu->pmupgidx = -1;
	// Initialize the command indexes
	ipdu->command_put = ipdu->command_get = 0;
	// Initializaing the serial port
	ipdu->fd = serialStart(ipdu);

	if ( ipdu->fd == -1 ) {
		//dolog(LOG_ERR, "Could not init serial port.");
		dodebug("Could not init serial port.\n");
		cleanIpduStruct(ipdu);
		cleanIpdu2Struct(ipdu2);
		return;
	}

	// check pmusers 
	if (cy_shm->line_conf[port].pmusers && *cy_shm->line_conf[port].pmusers) {
		ipdu->userlist = initIpduUserList(cy_shm->line_conf[port].pmusers);
	} else {
		ipdu->userlist = 0;
	}

	ipdu->flgSendEnter = 0;

	ipduList[port] = ipdu;
#ifdef NUMBER_OF_OUTLETS
	cy_shm->PMD_outlets[port] = NUMBER_OF_OUTLETS;
	cy_shm->PMD_pms[port] = 1;
#else
	cy_shm->PMD_outlets[port] = 1;
	cy_shm->PMD_pms[port] = 0;
#endif
	cy_shm->sess_list[port] = (CY_SESS *)ipdu2;
}

/******************************************************************************
 * This routine check all ipduList structures that are initialized and resets
 * them. It consists of sending exit commands, resetting the port and 
 * deallocating the IPDU structures.
 ******************************************************************************/
static void stop_ipdu (int port)
{
	ipduInfo *ipdu = ipduList[port];
	if (!ipdu) {
		return;
	}

	//dolog(LOG_INFO, "pmd.c main() - Stopping IPDU %d", port + 1);
#ifdef ONS
	dodebug("PMD[%s%d]:stop_ipdu - Stopping IPDU\n",
		ipdu->auxflg ? "a" : "s", ipdu->serialPort);
#else
	dodebug("PMD[%d]:stop_ipdu - Stopping IPDU\n", port + 1);
#endif
	// Closing the serial port
	serialFinish(ipdu);

	// Freeing the ipdu struct
	cleanIpduStruct(ipdu);
	ipduList[port] = NULL;
	shm_signal(CY_SEM_SHM);
	cleanIpdu2Struct((ipduInfo2 *)(cy_shm->sess_list[port]));
	cy_shm->sess_list[port] = NULL;
	cy_shm->PMD_pms[port] = 0;
	shm_unsignal(CY_SEM_SHM);
}

void pmdStatus(pmcInfo *pmc)
{
	ipduInfo *ipdu = ipduList[(int)pmc->pmc_ipdu];
	struct _pmdStatus *buf = (struct _pmdStatus*)pmc->pmc_answerBuf;

	buf->state = ipdu->state;
	buf->start_time = ipdu->start_time;
	buf->retry = ipdu->retry;
	buf->cmd_get = ipdu->command_get;
	buf->cmd_put = ipdu->command_put;
	memcpy(buf->comm,ipdu->comm,IPDUCOMMSIZE);
	memcpy(buf->commands,ipdu->commands, sizeof(commItem)*MAXIPDUQUEUE);
	buf->pmindex = ipdu->pmindex;
}

static void handle_sigio(void)
{
	int pmIndex;
	pmcInfo *pmc;

#ifdef ON_SEQ
	char *todo, userbuf[20];
#endif

	got_sig &= ~GOT_SIGIO;
	for (pmIndex = 0; pmIndex < MAXPMDACCESS; 
		pmIndex ++) {
		if ((pmc = cy_shm->pmc_list[pmIndex]) == NULL) {
			continue;
		}		
		if (pmc->pmc_status == PMD_STATUS) {
			if (!strcmp(pmc->pmc_cmd,"pmdcycl")) {
				pmdStatus(pmc);
			}
			pmc->pmc_status = PMC_ANSOK;
			kill(pmc->pmc_pid, SIGUSR2);
			continue;
		}

		// Upgrade firmware request
		if ((pmc->pmc_status == PMC_UPGRADE) || (pmc->pmc_status == PMC_UPGRADE_END)){
			//dolog(LOG_INFO, "pmd- Got SIGIO[%d] ==> IpduSendUpgrade",pmc->pmc_pid);
			dodebug("PMD[%d]:handle_sigio: PMC_UPGRADE from %s/%d\n",pmc->pmc_ipdu,pmc->pmc_username,pmc->pmc_pid);
			IpduSendUpgrade(ipduList[pmc->pmc_ipdu], pmc);
			continue;
		}
		
		if (pmc->pmc_status == PMC_PASSWDOK) {
			//dolog(LOG_INFO, "pmd - Got SIGIO from proc[%d]",pmc->pmc_pid);
			dodebug("PMD[%d]:handle_sigio : PMC_PASSWDOK from %s/%d\n",pmc->pmc_ipdu,pmc->pmc_username,pmc->pmc_pid);
			IpduSendPassword(ipduList[pmc->pmc_ipdu], pmc);
			continue;
		}

		if ((pmc->pmc_status & 0x7fffffff) == PMC_REQUEST) { //[RK]Aug/13/04 - Syslog Message
			//dolog(LOG_INFO, "pmd - Got SIGIO from proc[%d]",pmc->pmc_pid);
			dodebug("PMD[%d]:handle_sigio: PMC_REQUEST from %s/%d\n",pmc->pmc_ipdu,pmc->pmc_username,pmc->pmc_pid);

			if(! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", 
				strlen(CYCLE_COMMAND)+1)&& \
//				(user_not_admin(pmc->pmc_username))&& \  [GY]2006/Mar,07  retrieve r1.43
				!strstr(pmc->pmc_cmd,"all")&& \
				!strstr(pmc->pmc_cmd,"-")) {
				pmc->iscycle = 1;
			} else {
				pmc->iscycle = 0;
			}
			if(! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", strlen(CYCLE_COMMAND) + 1)) pmc->iscycle |= 2;

#ifdef ON_SEQ
			if((! strncmp(pmc->pmc_cmd, ON_COMMAND" ", \
					strlen(ON_COMMAND) + 1) || \
				! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", \
					strlen(CYCLE_COMMAND) + 1)) && \
//				(user_not_admin(pmc->pmc_username)) && \  [GY]2006/Mar,07  retrieve r1.43
				(!strstr(pmc->pmc_cmd, "all")) && \
				(!strstr(pmc->pmc_cmd, "-"))) { 

				//if the outlets came separated by comma
				//change all then for spaces
				//this happens when individual ipdus menu is chosen in pm
				comma2space(pmc->pmc_cmd);
					
				//this function breaks an on command over many outlets
				//int many on commands, each one over a single outlet
				//this is used to guarantee that sequence will be 
				//respected
				IpduBreakOn2(pmIndex);
					
				//takes one of the "on"s commands from the queue
				todo = queuedons(pmIndex, userbuf);
				strcpy(pmc->pmc_cmd, todo);
				strcpy(pmc->pmc_username, userbuf);

			}
#endif
			IpduSendCommand(ipduList[pmc->pmc_ipdu], pmc);
			continue;				 
		}
#ifdef ON_SEQ
		else {
			if (pmc->pmc_status == PMC_WAITANS) {
				continue;
			}
			if (vet_onq[pmIndex]) {
				vet_onq[pmIndex]->index = -1;
				vet_onq[pmIndex]->number = 0;
			}
		}
#endif

	}
}		

static void handle_sighup(void)
{
	int portIndex, pmIndex;
	struct line_cfg * line;
	ipduInfo *ipdu;

	//dolog(LOG_INFO, "pmd.c main() - Got SIGHUP");
	dodebug("PMD:handle_sighup\n");
	got_sig &= ~GOT_SIGHUP;

	if (alarm_set) {
		alarm(0);
		got_sig &= ~GOT_ALARM;
		alarm_set = 0;
	}

	for ( portIndex = 0; portIndex < MAX_DEV; portIndex++) {
		if ((cy_shm->vet_chg[portIndex] & 2) == 0) {
			continue;
		}
		
		cy_shm->vet_chg[portIndex] &= ~2;

		line = &cy_shm->line_conf[portIndex];
		if (line->protocol != P_IPDU || line->this != portIndex) {
			// IPDU was deleted
			for (pmIndex = 0; pmIndex < MAXPMDACCESS; pmIndex ++) {
				pmcInfo *pmc = cy_shm->pmc_list[pmIndex];
				if (! pmc) {
					continue;
				} 
				if (pmc->pmc_ipdu != portIndex) {
					continue;
				}
				kill(pmc->pmc_pid, SIGUSR1); 
			}
			stop_ipdu(portIndex);
			continue;
		}
		
		ipdu = ipduList[portIndex];
		if (!ipdu) {
			// IPDU was inserted
			start_ipdu(portIndex);
			continue;
		}

		// configuration was changed : check pmusers
		if (ipdu->userlist) free(ipdu->userlist);
		if (line->pmusers && *line->pmusers) {
			ipdu->userlist = initIpduUserList(line->pmusers);
		} else {
			ipdu->userlist = 0;
		}
	}
}

/******************************************************************************
 * This routine is run when the PMD is exiting.
 *****************************************************************************/
static void ExitPMD (void)
{
	int portIndex, pmIndex;
	cy_shm->PMD_pid = 0;
	
	for (pmIndex = 0; pmIndex < MAXPMDACCESS ; pmIndex ++) {
		pmcInfo *pmc = cy_shm->pmc_list[pmIndex];
		if (! pmc) {
			continue;
		}
		//dolog(LOG_INFO, "SIGUSR1 sent to process ID %d", pmc->pmc_pid);
		dodebug("SIGUSR1 sent to process ID %d\n", pmc->pmc_pid);
		kill(pmc->pmc_pid, SIGUSR1); /* Finishes the client
						process */
	}
	for ( portIndex = 0; portIndex < MAX_DEV; portIndex++) {
		stop_ipdu(portIndex);
	}
        //dolog(LOG_INFO, "PMD Stopped");
        dodebug("PMD Stopped\n");
}

/******************************************************************************
 * This is the main routine. 
 *****************************************************************************/
int main (int argc, char **argv) {
	fd_set InFdSet, AuxFdSet;
	struct timeval Timeout;
	int aux, fd, max_fd;
	int portIndex, pmIndex;
	pmcInfo *pmc;
        char *pn;
	extern char *optarg;
	int opt;
	extern int maxflgsendenter;
#ifdef ON_SEQ
	char *todo, userbuf[20];
#endif

        if ((pn = strrchr(argv[0], '/')) == NULL) {
                pn = argv[0];
        } else {
                ++pn;
        }

	while ((opt = getopt(argc, argv, "e:sd")) != EOF) {
		switch (opt) {
			case 'd':
				debugFlg = 1;
				break;
			case 'e':
				aux = atoi(optarg);
				if (aux >= 30) {
					maxflgsendenter = aux;
				} else 
					printf("\r\n[PMD] The minimun value is 30 sec");
				break;
			case 's':
				syslogFlg = 0;
				break;
		}
	}
	    
	// Starting the shared memory
	if (shm_init(-1) < 0) {
		printf ("\r\n[PMD] Error - Could not start shared memory.\r\n");
		return -1;
	}

	// Initializing the signal treatment
	aux = initSigTreatment();
	if (aux) {
		printf ("\r\n[PMD] Error - Could not init signals treatment.\r\n");
		return -1;
	}

	// Wait for the cy_ras to read all the configuration.
        while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
                sleep(1);
                continue;
        }

	// Initializing the syslog
        //dolog(LOG_INFO, "PMD Started");
        dodebug("PMD Started\n");

	for (pmIndex = 0; pmIndex < MAXPMDACCESS; pmIndex ++) {
		if ((pmc = cy_shm->pmc_list[pmIndex]) == NULL) {
			continue;
		}
		kill(pmc->pmc_pid, SIGUSR1);
		//shm_signal(CY_SEM_SHM);
		//if (pmc->pmc_answerBuf) shm_free_str(pmc->pmc_answerBuf);
		//shm_free_str((char *)pmc);
		//cy_shm->pmc_list[pmIndex] = NULL;
		//shm_unsignal(CY_SEM_SHM);
	}

	// Initializing the ipduList vector
	bzero(ipduList, sizeof(ipduList));
	for ( portIndex = 0; portIndex < MAX_DEV; portIndex++) {
		start_ipdu(portIndex);
	}
	// Initialize pmd PID
	cy_shm->PMD_pid = getpid();

	atexit(ExitPMD);

	// The main loop
	while (TRUE) {
		if (got_sig & GOT_SIGGEN) {
			//dolog(LOG_INFO, "pmd.c received signal=%d",last_sig);
			dodebug("PMD:main received signal=%d\n",last_sig);
			break;
		}
		if (got_sig & GOT_SIGINT) {
			//dolog(LOG_INFO, "pmd.c main() - Got SIGINT");
			dodebug("PMD:main received SIGINT\n");
			break;
		}
		// Wait for the cy_ras to read all the configuration.
	        while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) 
			!= 1) {
        	        sleep(1);
                	continue;
	        }
		if ((got_sig & GOT_SIGHUP) ||
			((got_sig & GOT_ALARM) && alarm_set)) {
			handle_sighup();
		}
		if (got_sig & GOT_SIGIO) {
			handle_sigio();
		}
		/* Mount the File Descriptors for IPDU */
		max_fd = -1;
		FD_ZERO(&InFdSet);
		for ( portIndex = 0; portIndex < MAX_DEV; portIndex++) {
			if (ipduList[portIndex]) {
				fd = ipduList[portIndex]->fd;
				FD_SET(fd, &InFdSet);
				if (fd > max_fd) {
					max_fd = fd;
				}
			}
		}
		AuxFdSet = InFdSet;
		Timeout.tv_sec = 1;
		Timeout.tv_usec = 0;
		if (select (max_fd + 1, &AuxFdSet, NULL, NULL, &Timeout) < 0) {
			continue;
		}

		// Handle ports
		for ( portIndex = 0; portIndex < MAX_DEV; portIndex++) {
			if (ipduList[portIndex]) {
				IpduHandlePort(ipduList[portIndex]);
				if ((ipduList[portIndex]->state == IPDU_UPGRADE) &&
					(cy_shm->pmc_list[ipduList[portIndex]->pmindex] == NULL)){
					ipduList[portIndex]->state = IPDU_READY;
				}
			}
		}
		for (pmIndex = 0; pmIndex < MAXPMDACCESS; 
			pmIndex ++) {
			if ((pmc = cy_shm->pmc_list[pmIndex]) == NULL) {
				continue;
			}
#ifdef ON_SEQ
			if((todo = queuedons(pmIndex, userbuf))){
				//makes sure that there will be a minival interval for the first
				//outlet
				if( todo == vet_onq[pmIndex]->queue[1] && strstr(pmc->pmc_cmd,OFF_COMMAND)){
					sleep(4);
				}
				strcpy(pmc->pmc_cmd,todo);
				strcpy(pmc->pmc_username, userbuf);
				//got from the queue the command pmc->pmc_cmd
				//[RK]Oct/18/05 - get the original status
				pmc->pmc_status = vet_onq[pmIndex]->orig_status;
				//Im gonna send the command
				IpduSendCommand(ipduList[pmc->pmc_ipdu], pmc);
			}
			else if ((pmc->pmc_status & 0x7fffffff) == PMC_REQUEST) { //[RK]Aug/13/04 - Syslog Message
				dodebug("PMD[%d]:main : PMC_REQUEST from %s/%d cmd [%s]\n",pmc->pmc_ipdu,pmc->pmc_username,pmc->pmc_pid, pmc->pmc_cmd);
				if(! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", strlen(CYCLE_COMMAND) + 1)&& \
//					(user_not_admin(pmc->pmc_username))&& \  [GY]2006/Mar/07  retrieve r1.41
					!strstr(pmc->pmc_cmd,"all")&& \
					!strstr(pmc->pmc_cmd,"-")) {
					pmc->iscycle = 1;
				} else {
					pmc->iscycle = 0;
				}
				if(! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", strlen(CYCLE_COMMAND) + 1)) pmc->iscycle |= 2;
				
				//if((strstr(pmc->pmc_cmd,ON_COMMAND) || strstr(pmc->pmc_cmd,CYCLE_COMMAND)) 
				if((! strncmp(pmc->pmc_cmd, ON_COMMAND" ", strlen(ON_COMMAND) + 1) ||
					! strncmp(pmc->pmc_cmd,CYCLE_COMMAND" ", strlen(CYCLE_COMMAND) + 1)) 
//					&& (user_not_admin(pmc->pmc_username))  [GY]2006/Mar/07  retrieve r1.41
					&& (!strstr(pmc->pmc_cmd, "all"))
					&& (!strstr(pmc->pmc_cmd, "-"))) { 

					//if the outlets came separated by comma
					//change all then for spaces
					//this happens when individual ipdus menu is chosen in pm
					comma2space(pmc->pmc_cmd);
					
					//this function breaks an on command over many outlets
					//int many on commands, each one over a single outlet
					//this is used to guarantee that sequence will be 
					//respected
					IpduBreakOn2(pmIndex);
					
					//takes one of the "on"s commands from the queue
					todo = queuedons(pmIndex, userbuf);
					strcpy(pmc->pmc_cmd, todo);
					strcpy(pmc->pmc_username, userbuf);
				} else {
					if (vet_onq[pmIndex]) {
						vet_onq[pmIndex]->index = -1;
						vet_onq[pmIndex]->number = 0;
					}
				}
				//send the command
				IpduSendCommand(ipduList[pmc->pmc_ipdu], pmc);
			}
#else
			if ((pmc->pmc_status & 0x7fffffff) == PMC_REQUEST) { 
				//send the command
				IpduSendCommand(ipduList[pmc->pmc_ipdu], pmc);
			}
#endif
			// Upgrade firmware request
			if (!(strcmp(pmc->pmc_username,"root")) && 
			    ((pmc->pmc_status == PMC_UPGRADE) || 
			     (pmc->pmc_status == PMC_UPGRADE_END))){
				//dolog(LOG_INFO, "pmd.c main() - Got SIGIO ==> IpduSendUpgrade");
				dodebug("PMD[%d]:main : PMC_UPGRADE from %s/%d\n",pmc->pmc_ipdu,pmc->pmc_username,pmc->pmc_pid);
				IpduSendUpgrade(ipduList[pmc->pmc_ipdu], pmc);
			}
		
		}
	}
	// We're done
	return 0;
}



#ifdef ON_SEQ
/***************************************************************************************************
 IpduBreakOn2 - break a on command in multiple outlets into many individual commands, or a cycle
 command into a sequel of an unique off at all outlets, followed by individual ons

index - index of pmc and onq

 **************************************************************************************************/
static void IpduBreakOn2(int index)
{
	pmcInfo *pmc = cy_shm->pmc_list[index];

	int i = 0,j=strlen(ON_COMMAND)+1;
	char *p;
	struct Tonqueue *onq = vet_onq[index];

	if (onq == NULL)  {
		onq = malloc(sizeof(struct Tonqueue));
		vet_onq[index] = onq;
	}

	if (onq == NULL) exit(-1);

	strcat(pmc->pmc_cmd," ");
	p = strstr(pmc->pmc_cmd," "); //starts scan after the on command
	while(*p == ' ') p++; //skips possible additional spaces
	memset(onq->queue,0,sizeof(onq->queue));
	strcpy(onq->queue[0],ON_COMMAND" "); //first on command of the array
	onq->index = 0;
	onq->number = 0;
	strncpy(onq->username, pmc->pmc_username, sizeof(onq->username) - 1);
	while (*p){
		//if(*p <= 0x39 && *p >= 0x30) //if it is a number, copy it
		if(isdigit(*p)){
			onq->queue[i][j] = *p;
			p++;
			j++;
			if(*p == ' ' || *p == '\0'){
				onq->queue[i][j] = '\0';
				if(*p == '\0') break;
				else{
					p++;
					i++;
					strcpy(onq->queue[i],ON_COMMAND" "); // a new command
					j=strlen(ON_COMMAND)+1;
					onq->number++;
				}
			}
		}
		else p++;
	}	

	if(strstr(pmc->pmc_cmd,CYCLE_COMMAND)){ //if command is a cycle
		while(i > 0){ //shift all on commands one position ahead
			strcpy(onq->queue[i],onq->queue[i-1]);
			i--;
		}
		strcpy(onq->queue[0],OFF_COMMAND" "); //and finally inserts the off for all outlets
		strcat(onq->queue[0],pmc->pmc_cmd + (strlen(CYCLE_COMMAND)));//in the first position
	}
	else
		memset(onq->queue[i],0,sizeof(onq->queue[i]));
	//onq->last = onq->queue[i-1]; 
	onq->index = 0;

	//[RK]Oct/18/05 - save the original status
	onq->orig_status = pmc->pmc_status;
}


/***************************************************************************************************
 queuedons() - return the next on command waiting in the queue, or NULL if none

 index - Tonqueue index in vet_onq
 username - buffer that will receive the username for which the on command will be issued
 **************************************************************************************************/
static char *queuedons(int index, char *username)
{
	struct Tonqueue *onq = vet_onq[index];

	if (onq == NULL) return NULL;

	if (onq->index < 0) {
		return NULL;
	} else if(onq->queue[onq->index][0]){
		strcpy(username, onq->username);
		return onq->queue[onq->index++];
	} else {
		onq->index = -1;
		return NULL; //no queued ons
	}
}

/***************************************************************************************************
 comma2space - converts all the commas ( , ) found in a string into spaces. This acessory function
 is used to compatibilize the processing when multidevice menu is used, in which the outlets are 
 separated for spaces in the pmoutlet parameter, with the older menu which deals directly with
 the serial where the PM is connected
 **************************************************************************************************/
static void comma2space( char *string)
{
	char *p = string;

	while(*p){
		if(*p == ',') *p = ' '; //changes the comma for the space
		p++; 
	}
}
#endif
