/*
 * Module: pmd.h
 *
 * Author:	Henrique Gobbi <henrique.gobbi@cyclades.com>
 *
 * Copyright:	(c) 1999-2002 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * $Log: pmd.h,v $
 * Revision 1.14  2006/06/21 13:22:05  regina
 * Removed (using ifdef) the feature ON sequence and multi-oultet control
 *
 * Revision 1.13  2006/06/08 13:57:35  regina
 * Port from ACS - pm45
 *
 * Revision 1.16  2006/04/15 03:03:00  edson
 * BTS-6865: memory allocation error
 *
 * Revision 1.15  2006/04/06 22:44:06  fujima
 * Fix in the PMD serial driver to consider that the serial port can receive NULL characters from the IPDU
 *
 * Revision 1.14  2005/09/08 00:52:17  fujima
 * Include support for the OEM version of the KVM
 *
 * Revision 1.13.2.1  2005/08/31 19:11:25  fujima
 * Include support for OEM3's IPDU
 *
 * Revision 1.13  2005/03/16 12:06:40  regina
 * MVL31Pro - fixed syntaxe error
 *
 * Revision 1.12  2004/08/16 14:00:00  regina
 * Implemented the PMD Syslog Messages, and debug messages
 *
 * Revision 1.11  2004/03/05 19:03:48  regina
 * Implemented access control by pmd
 *
 * Revision 1.10  2003/10/22 16:35:11  regina
 * SNMP Proxy for PM - implementation
 *
 * Revision 1.9  2003/04/14 12:26:38  fujima
 * Bugs #464, 442 and 441 fixed. The main changes in this version are: (1) - The menu interface allows entering other parameters than the outlet numbers - current, username, ... (2) - The PMD state machine is more stable, (3) - The PM administrator password is configurable in the pslave.conf, (4) - By adding the option -t the trace of the communication with the IPDUs and with the clients can be monitored, (5) - By adding the option -d <level> the system logs until the level configured appear in the stdout.
 *
 * Revision 1.8  2003/03/21 17:13:04  fujima
 * PMD messages now use syslog
 *
 * Revision 1.7  2003/03/10 19:30:48  fujima
 * Removed debug and trace messages
 *
 * Revision 1.6  2003/03/10 19:09:49  fujima
 * Support for daisy chain and syslog in AlterPath PM
 *
 * Revision 1.5  2003/02/21 19:57:05  fujima
 * Added comments and layout changes
 *
 * Revision 1.4  2003/02/20 20:45:04  fujima
 * Removing debug macros
 *
 * Revision 1.3  2003/02/20 20:43:15  fujima
 * New PMD
 *
 * Revision 1.2  2003/01/31 01:09:36  henrique
 * It define the IPDULIST_REFRESH_TIME and create the defines for the routines
 * updateInfoFile and populateIpduList.
 *
 * Revision 1.1  2003/01/13 21:14:55  henrique
 * This is the Power Management software initial implementation.
 *
*/

# include <pmheaders.h>
# include <server.h>
# include <ipdu.h>

/******************************************************************************
 *                      GLOBAL VARIABLES
 *****************************************************************************/
#ifdef PMD
ipduInfo * ipduList[MAXLINES];            // All the IPDU specific information
//[RK]Aug/09/04 - Debug and trace - syslog message
int debugFlg = 0;	
int debugFd = -1;	
int syslogFlg = 1;
#else
extern ipduInfo * ipduList[MAXLINES];
extern int debugFlg, syslogFlg, debugFd;
#endif

/******************************************************************************
 *                      GLOBAL DEFINES
 *****************************************************************************/
//[RK]Aug/09/04 - debug and trace
#define dodebug(fmt, arg...)  if (debugFlg) debugprintf(fmt,##arg)
#define dotrace(fmt, arg...)  if (debugFlg) traceprintf(fmt,##arg)

#define MAXIPDU MAXLINES
#define enter	"\r"

#ifdef ON_SEQ
struct Tonqueue *vet_onq[MAXPMDACCESS];
#endif

/******************************************************************************
 *                      Routines Section
 *****************************************************************************/

// Serial
void serialFinish(ipduInfo * ipdu);
int serialStart( ipduInfo * ipdu);
int IpduSendCommand (ipduInfo *ipdu, pmcInfo *pmc);
int IpduSendPassword (ipduInfo *ipdu, pmcInfo *pmc);
int IpduSendUpgrade (ipduInfo *ipdu, pmcInfo *pmc);
void IpduHandlePort (ipduInfo * ipdu);
void traceprintf(unsigned int type, unsigned int port, unsigned char *s1, int len);
void debugprintf(const char *fmt, ...);
