/*****************************************************************
* File: pmfwupgrade.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
*
****************************************************************/

// Linux includes
# include <stdio.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <errno.h>
# include <unistd.h>
# include <termios.h>
# include <linux/ioctl.h>
# include <stdlib.h>
# include <string.h>
# include <sys/types.h>
# include <dirent.h>
# include <ctype.h>
# include <signal.h>
# include <time.h>

#include <linux/shm.h>
#include<linux/sem.h>
#include<server.h>

extern char * optarg;
extern int optind;

#define alias serverfarm

#define true	1
#define false	0

//The first PM version to have firmware upgrade
#define VERSIONUPGRADE 0x110
#define VERSIONCHAIN 0x122 

// Version of this software
char version[] = "@(#) pmfwupgrade v_1.1.1 (May/16/06)";

//[RK]May/16/07 - event notificatio
extern void writeevt(int,char *, ...);

int verbose = 0;
int force = 0;
int retries = 0;

#ifdef ONS
int auxflag = 0;
#endif

#define MAX_DEV         cy_shm->max_dev

#define pmprintf(x)	if (verbose) printf(x)

#define UPGNOTAVAIL 	0
#define UPGAVAIL  	1
#define UPGCHAINAVAIL   2

typedef struct {
	char flg;	
	char ver[32];
	int  num;
	} VersionSt;

int serialPort;      // Number of the serial port where PM is connected
#define MAXIPDU	16
int	numUnits;
VersionSt unitVer[MAXIPDU];

VersionSt fileVer; 
char * fileverstr;	// version of the file without date [RK]May/16/06
char * fileData;	// memory with the firmware
int	bytesTx;	// number of bytes that were transmited
int 	fileSize;	// size of the firmware
int	sectorNumber;	// number of the sector
int	unitUpg;	// unit number that do the upgrade
int 	unitNumber;	// argument unit number
int	startTime;

// PMC struct
pmcInfo *pmc;
int 	pmcindex;

char upgCmd[20];

// ASCII Constants  == XMODEM Protocol
#define      SOH  	001 
#define      EOT	004
#define      ACK  	006
#define      NAK  	025
#define	     CAN	030

// XMODEM Constants
#define      NAKMAX	120   // maximum times to wait for initial NAK when sending
#define      RETRYMAX  	5     // maximum retries to be made certain handshaking routines
#define      EOTMAX	10    // maximum times sender will send an EOT to end transfer
#define	     BBUFSIZE	128   // buffer size


/* Which software interrupts were detected */
int got_sig;
# define GOT_SIGUSR1	1	/* It is signalled by PMD, indicating that the
				   PMD is exiting */
# define GOT_SIGUSR2	2	/* It is signalled by PMD, indicating that 
				   there is data to be read from the shared
				   memory. */
# define GOT_SIGINT	4	/* It exits the pm/pmCommand process */

// ==========================================================================
// state machine
// states
#define STIDLE		0
#define STWAITVER	1
#define STWAITMENU	2
#define STWAITFRST	3
#define STWAITBUFOK	4
#define STWAITLASTOK	5
#define STWAITEOTOK	6

int upgState=0;
int event = 0;

// events
// 1 - receive PM version
// 2 - receive menu (answer for "upgrade" or "reboot" command :" 1-upgrade or 2-boot"
// 3 - receive first XModem packet 
// 4 - receive status PMC_ANSOK (pmc_outlet[0] has code error)
// 5 - timeout 
#define EVPMVERSION	1
#define EVPMMENU	2
#define EVPMFRST	3
#define EVPMSTATUS	4
#define EVTIMEOUT	5

/* ============================================================================
 *  InitPmcEntry() - This routine looks for an empty entry in cy_shm->pmc_list.
 *  If found, a pmcInfo structure is allocated, initialized and inserted in 
 *  this entry.
 * ============================================================================
 * */
static int InitPmcEntry(int ipduport)
{
	char * buf;
	int pmindex;
	// Try to find an empty PMC entry in the shared memory
	for (pmindex = 0; pmindex < MAXPMDACCESS; pmindex++) {
		if (cy_shm->pmc_list[pmindex] == 0)
		break;
	}

	// If not found, return error
	if (pmindex == MAXPMDACCESS) {
		printf ("\n[Error] - PMC empty entry was not found.\n");
		exit(-1);
	}

	// Initialize the session: allocate the entry and set initial
	// values.
	pmc = (pmcInfo *)shm_malloc(sizeof(pmcInfo), TYPE_SHM_PMC);
	buf = shm_malloc(1280, TYPE_SHM_PMCBUF);

	if (!pmc || !buf) {
		if (pmc) shm_free_str((char *)pmc);
		if (buf) shm_free_str(buf);
		printf ("\n[Error] - PMC entry could not be allocated.\n");
		exit(-1);
	}

	shm_signal(CY_SEM_SHM);
	pmc->idx_pmclist = pmindex;
	pmc->pmc_pid = getpid();
#ifdef ONS
	pmc->pmc_aux = auxflag;
#endif
	pmc->pmc_ipdu = ipduport-1;
	pmc->pmc_status = PMC_IDLE;
	pmc->pmc_sizeAnswerBuf = 1280;
	pmc->pmc_answerBuf = buf;
	strcpy(pmc->pmc_username,"root");
	pmc->next = NULL;
	cy_shm->pmc_list[pmindex] = pmc;
	shm_unsignal(CY_SEM_SHM);
	pmcindex = pmindex;
        return TRUE;
}

/* ============================================================================
 *  EndPmcEntry() - This routine deallocates the pmcInfo structure and resets
 *  the corresponding entry.
 * ============================================================================
 * */
static void EndPmcEntry(void)
{
	shm_signal(CY_SEM_SHM);
	cy_shm->pmc_list[pmcindex] = NULL;
	if (pmc) {
		if (pmc->pmc_answerBuf) 
			shm_free_str(pmc->pmc_answerBuf);
		shm_free_str((char *)pmc);
		pmc = NULL;
	}
	shm_unsignal(CY_SEM_SHM);
	if (fileData) { 
		free(fileData);
		fileData = 0;
	}
}

/* ============================================================================
 *  treatsignals() - It is the handler routine for the signals. At this point,
 *  only the got_sig flag is activated according to the signals received. The 
 *  real treatment will be performed in the main loop.
 * ============================================================================
 * */
static void treatSignals( int sig ) {
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));
	switch (sig) {
		case SIGUSR1:
			got_sig |= GOT_SIGUSR1;
			break;
		case SIGUSR2:
			got_sig |= GOT_SIGUSR2;
			break;
		case SIGINT:
			got_sig |= GOT_SIGINT;
			break;
	}
        act.sa_handler = treatSignals;
        sigaction(sig, &act, 0);
        unblock(sig);
}

/* ============================================================================
 *  initSigTreatment() - It starts the reception of SIGINT, SIGUSR1 and 
 *  SIGUSR2. If it fails it returns -1.
 * ============================================================================
 * */
static int initSigTreatment() {
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));

	// Setting up the act structure
	act.sa_handler = treatSignals;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;

	// Trying to register the signals. If one of them fail it returns error
	if ( sigaction(SIGUSR1, &act, 0) == -1 ) return -1;
	if ( sigaction(SIGUSR2, &act, 0) == -1 ) return -1;
	if ( sigaction(SIGINT, &act, 0)  == -1 ) return -1;
	unblock(SIGUSR1);
	unblock(SIGUSR2);
	unblock(SIGINT);

	// Success
	return 0;
}

/* ============================================================================
 * sendCommand() - This routine sends a SIGIO to the PMD process, indicating 
 * that there is a command to be processed.
 * ============================================================================
 * */
static void sendCommand (int status)
{
	pmc->pmc_status = status;
	kill(cy_shm->PMD_pid, SIGIO);
}

static void sendCommandCancel(void)
{
	pmc->pmc_answerBuf[0] = CAN;
	pmc->pmc_answerBuf[1] = CAN;
	pmc->pmc_answerBuf[2] = CAN;
	pmc->pmc_answerBuf[3] = CAN;
	pmc->pmc_answerBuf[4] = CAN;
	pmc->pmc_answerBuf[5] = 0x00;
	pmc->pmc_sizeAnswerBuf = 5;
	
	sendCommand(PMC_UPGRADE_END);
}

static void sendCommandEot(void)
{
	pmc->pmc_answerBuf[0] = EOT;
	pmc->pmc_answerBuf[1] = 0x00;
	pmc->pmc_sizeAnswerBuf = 1;
	
	sendCommand(PMC_UPGRADE_END);
}

static void sendOptionUpgrade(void)
{
	pmc->pmc_answerBuf[0] = '1';
	pmc->pmc_answerBuf[1] = '\n';
	pmc->pmc_answerBuf[2] = 0x00;
	pmc->pmc_sizeAnswerBuf = 2;
	if (force == 2) pmc->pmc_outlet[1] = 0xff;
	sendCommand(PMC_UPGRADE);
}

static void sendCommandUpgrade(void)
{
	sprintf(pmc->pmc_answerBuf,upgCmd,unitUpg+1);
	pmc->pmc_sizeAnswerBuf= strlen(pmc->pmc_answerBuf);
	sendCommand(PMC_UPGRADE);
}

// Building the packet. return size in destination buffer
int mountXmodemBuf(char *destination, char * source, int size, int sectnum)
{
	char *src, *dest;
	int cont=0;
	unsigned short checksum;// holds the checksum for a package
	
	dest = destination;
	src = source;

	*dest++ = SOH;        // Start of header byte
	*dest++ = sectnum;    // The packet number
	*dest++ = ~sectnum;   // The 1's complement of the packate number

	// Copying data to blockbuf and calculating checksum
	checksum = 0;
	for (cont = 0; cont < size; cont++) {
		*dest++  = *src;
		checksum = ((checksum + *src++) & 0xff);
	} 
	if (size < BBUFSIZE) {
		for (; cont < BBUFSIZE ; cont++) {
			*dest++ = 0xff;
			checksum = ((checksum + 0xff) & 0xff);
		}
	}
	*dest = checksum;
	return cont+4;
}

// return true when sends the last buffer
int treatXmodemSend(void) 
{
	int size,ret=true;
	
	if (upgState == STWAITFRST) {
		// initialize global vars
		pmprintf("\nInitializing transfer\n");
		bytesTx = 0;
		sectorNumber = 1;
		retries = 0;
		startTime = time((time_t *) 0);
	}

	if ((size = (fileSize - bytesTx)) > BBUFSIZE)
		size = BBUFSIZE;
	else
		ret = false;	// last buffer
	
	pmc->pmc_sizeAnswerBuf = mountXmodemBuf(pmc->pmc_answerBuf, 	// destination
						(fileData + bytesTx),	// source
						size,
						sectorNumber);
	
	bytesTx += size;
	sectorNumber++;
	retries = 0;	
	if (verbose) {
		int i;
		printf("\rProgress: |");

		if (ret == true) {
			size = (((sectorNumber - 2) * BBUFSIZE) * 100) / fileSize;
		} else {
			size = 100;
		}

		for (i=0;i<100;i+=2)
			printf("%c",(size>i)? '=' : ' ');
		printf("| %d %% ", size);
	
	}
	
	return ret;
}

void treatEndUpgrade(void)
{
	sendCommandEot();

	if (verbose) {
		printf("\n\nTransfer complete. %d bytes were sent in %d seconds\n", fileSize, 
				(int)(time(NULL) - startTime));
	}

	//[RK]May/11/06 - event notification
	writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, serialPort,
		unitUpg+1,1,fileverstr);

	if ((unitNumber == 0) && (unitUpg > 0)) {
	       unitUpg--;
      	} else {
 		unitUpg = -1;
	}		
}

int checkUpgAvail (void) 
{
	int i, val,resp=0,ans=0;
		
	if (unitUpg < 0) return false;
	
	for (i = 0; i < unitUpg ; i++) {
		if (unitVer[i].flg != UPGCHAINAVAIL) {
			printf("\nThe PM unit number #%d does not support upgrade for daisy-chain\n",i+1);
			exit(-1);
		}
	}

	if (verbose || !force) {
		printf("\nUnit %d is running firmware version: %s",
			unitUpg+1,unitVer[unitUpg].ver);
		fflush(stdout);
	}

	if (force) {
		//[RK]May/11/06 - event notification
		writeevt(32,"usiiss",cy_shm->line_conf[serialPort-1].alias, serialPort,unitUpg+1,
			unitVer[unitUpg].ver,fileverstr);
		return true;  // not send questions to user
	}

	val = fileVer.num - unitVer[unitUpg].num;
	printf("\n");
	
	// Tell user what he is about to do
	if (val > 0) {
		printf("\nAre you sure you want to upgrade from version\n %s to version %s ?\n",
			unitVer[unitUpg].ver,fileVer.ver);
	} else {
		if (val == 0) {
			printf( "It seems you are trying to upload a firmware that is already\n"
				"running in your unit.\n"  //[GY]2005/Nov/22  BUG#5658
				"Are you sure you want to proceed ?\n");
		} else {
			printf("You are about to downgrade your unit from version\n");  //[GY]2005/Nov/22  BUG#5658
			printf(" %s to version %s\n",unitVer[unitUpg].ver,fileVer.ver);
			printf("Are you sure you want to proceed ?\n");
		}
	}

	// Wait for user confirmation
	printf("Please answer 'y' (yes) or 'n' (no):");
	fflush(stdout);
	do {
		resp = getchar();
		if ((char)resp != '\n') ans=resp;
	} while (resp != '\n'); 

	// Check user answer
	if (((char)ans & 0x5f) == 'Y') {
		//[RK]May/11/06 - event notification
		writeevt(32,"usiiss",cy_shm->line_conf[serialPort-1].alias, serialPort,unitUpg+1,
			unitVer[unitUpg].ver,fileverstr);
		return true;
	} else {
		EndPmcEntry();
		exit(-1);
	}
}

void treatVersions(char *vers)
{
	char cmd[50], *pe, *pini;
	int j,ver0,ver1,ver2;

	pini = vers;
	for (j=0; j < numUnits; j++) {
		pe = strstr(pini,"max Sw V ");
		if (!pe) break;
		if (sscanf(pe,"max Sw V %s ",cmd)==1){
			sscanf(cmd,"%d.%d.%d",&ver0,&ver1,&ver2);
			unitVer[j].num = (ver0*0x100) + (ver1*0x10) + ver2;
			strcpy(unitVer[j].ver,cmd);
			if (unitVer[j].num < VERSIONUPGRADE) {
				unitVer[j].flg = UPGNOTAVAIL;
			} else {
				if (unitVer[j].num < VERSIONCHAIN) {
					unitVer[j].flg = UPGAVAIL;
				} else {
					unitVer[j].flg = UPGCHAINAVAIL;
				}
			}
		}
		while (*pe && (*pe != '\n')) pe++;
		if (*pe) pe++;
		pini = pe;
	}

	switch (unitVer[0].flg) {
		case UPGNOTAVAIL:
			printf("\n[Error] - the master PM unit does not support upgrade.\n");
			EndPmcEntry();		
			exit(-1);
			break;
		case UPGAVAIL :
			strcpy(upgCmd,"reboot\n");
			break;
		case UPGCHAINAVAIL :
			strcpy(upgCmd,"upgrade %d\n");
			break;
	}

	if (unitNumber == 0) { 	// upgrade to all chain
		for (j=0; j < numUnits; j++) {
			if (unitVer[j].flg != UPGCHAINAVAIL) {
				break;
			}
		}
		if (j != numUnits) {
			unitUpg = j;
		} else {
			unitUpg = j-1;
		}
	} else {
		unitUpg = unitNumber - 1;
	}
}

unsigned int checkFile(char *filename) 
{
	struct stat fstat;
	FILE *fp;
	char version[200],*pe;
	int i1,i2,i3;
	char str[8];

	if (stat(filename,&fstat) == -1) {
		perror("[Error] - stat:");
		return false;
	}

	if ((fp = fopen (filename, "r")) == NULL) {
		perror("[Error] - open:");
		return false;
	}

	if ((fileData = (char *) malloc(fstat.st_size)) == NULL) {
		perror("[Erro] - malloc");
	       return false;
      	}

	if ((i1 = fread(fileData,fstat.st_size,1,fp)) == 0) {
		perror("[Erro] - read file : ");
		return false;
	}
	fileSize = fstat.st_size;
	
	fclose(fp);
	memset(version,0,200);
	sprintf(version,"what %s > /tmp/filevers",filename);
	system(version);

	if ((fp = fopen("/tmp/filevers","r")) == NULL) {
		perror("[Error] - after what: ");
		return false;
	}
	fread(version,200,1,fp);
	fclose(fp);
	system("rm /tmp/filevers");

	str[0] = '@';
   str[1] = '(';
   str[2] = '#';
   str[3] = ')';
   str[4] = 'V';
   str[5] = ' ';
   str[6] = 0;
	pe = strstr(version, str);
	sscanf(pe,"%s%s ", str, version);
	strncpy(fileVer.ver,pe+6,31);
//	strcpy(fileVer.ver,version);
	fileverstr = strdup(version); //[RK]May/16/06
	sscanf(version,"%d.%d.%d",&i1,&i2,&i3);
	fileVer.num = (i1 * 0x100) + (i2 * 0x10) + i3;

	if (verbose) {
		printf("\nFile %s (size=%d bytes) contains firmware version: %s",
			filename,fileSize,version);
	}
	return true;
}

void usage()
{
#ifdef ONS
	printf("\nUsage: pmfwupgrade [options] <port>[:<unit number>] [<filename>]\n");
#else
	printf("\nUsage: pmfwupgrade [options] <serial port>[:<unit number>] [<filename>]\n");
#endif
	printf("This program does the upgrade of all Intelligent Power Distribution\n");  //[GY]2005/Nov/22  BUG#5658
	printf("Units (IPDU) equipments.\n");  //[GY]2005/Nov/22  BUG#5658
	printf("\n -h   Show this help message and exit\n");
	printf(" -f   Do the upgrade without asking any question\n");
	printf(" -F   Do the upgrade without logical connection with PM\n");
	printf(" -v   Print messages about the execution\n");
	printf(" <serial port>: the serial port number where the PM unit is connected\n");
	printf(" <unit number>: the unit number when PM's in daisy-chain\n");
	printf(" <filename>:  The new firmware to upload to the PM unit. Default:/tmp/pmfirmware\n\n");
}

/* ============================================================================
 * got_sigusr2() - This routine treats SIGUSR2. It reads the buffer written by
 * the PMD and do the action
 * ============================================================================
 * */
static int got_sigusr2()
{
	if (upgState == STIDLE) return true;

	if (pmc->pmc_status != PMC_ANSOK) {
		return true;
	}

	if (upgState <= STWAITMENU) {
		switch (upgState) {
			case STWAITVER : 
				if ((strstr(pmc->pmc_answerBuf,"IPDU is not ready")) ||
				     (strstr(pmc->pmc_answerBuf,"Port is not configured as IPDU"))) {
					sendOptionUpgrade();
					upgState = STWAITFRST;
					break;
				}
					
				treatVersions(pmc->pmc_answerBuf);
				if (checkUpgAvail() == true) {
					sendCommandUpgrade();
					upgState = STWAITMENU;
				} else {
					upgState = STIDLE;
				}
				break;
					
			case STWAITMENU :
				if (strstr(pmc->pmc_answerBuf,"IPDU is not ready")) {
					sleep(2);
					sendOptionUpgrade();
					break;
				}
					
				if (strstr(pmc->pmc_answerBuf,"Enter choice (1 or 2):")) {
					sendOptionUpgrade();
					upgState = STWAITFRST;
				} else {
					if (retries++ < RETRYMAX ) {
						sendCommandUpgrade();
					} else {
						printf("\n[Error] - PM with problem is not responding\n");
                                                //return for "pmfwupgrade -f"  [GY]2006/Jun/06
                                                if( force == 1 )
                                                   exit(-1);
						return false;
					}
				}
				break;

			case STIDLE :
				pmc->pmc_status = PMC_IDLE;
				break;
		}
		return true;
	} 
	switch (upgState) {
		case STWAITFRST :
			switch (pmc->pmc_outlet[0]) {
				case PMC_UPG_OK :
					sendCommandCancel();
					printf("\n[Error] - Too many errors in transmission \n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
				case PMC_UPG_NAK :
					treatXmodemSend();
					upgState = STWAITBUFOK;
					sendCommand(PMC_UPGRADE);
					break;
				case PMC_UPG_CAN :
					printf("\n[Error] - Transfer canceled by PM\n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
			}
			break;
			
		case STWAITBUFOK :
			switch (pmc->pmc_outlet[0]) {
				case PMC_UPG_NAK :
					if (retries++ > RETRYMAX) {
						sendCommandCancel();
						printf("\n[Error] - Too many errors in transmission \n");
						writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
							serialPort,unitUpg+1,2,fileverstr);
						exit(-1);
					}  	
					sendCommand(PMC_UPGRADE);
                                        //return for "pmfwupgrade -f"  [GY]2006/Jun/06
                                        if( force == 1 )
                                           exit(-1);
					return false;
				case PMC_UPG_CAN :
					printf("\n[Error] - Transfer canceled by PM\n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
				case PMC_UPG_OK :
					if (treatXmodemSend()) {
						upgState = STWAITBUFOK;
					} else {
						upgState = STWAITLASTOK;
					}
					sendCommand(PMC_UPGRADE);
					break;
			}
			break;
			
		case STWAITLASTOK :
			switch (pmc->pmc_outlet[0]) {
				case PMC_UPG_ERROR :
					sendCommandCancel();
					printf("\n[Error] - Too many errors in transmission \n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
				case PMC_UPG_NAK :
					printf("\n[Error] - PM not responding\n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
				case PMC_UPG_CAN :
					printf("\n[Error] - Transfer canceled by PM\n");
					writeevt(33,"siiis",cy_shm->line_conf[serialPort-1].alias, 
						serialPort,unitUpg+1,2,fileverstr);
					exit(-1);
				case PMC_UPG_OK :
					treatEndUpgrade();
					upgState = STWAITEOTOK;
					break;
			} 
			break;

		case STWAITEOTOK :
			if (checkUpgAvail()) {
				sleep(2);
				retries = 0;
				sendCommandUpgrade();
				upgState = STWAITMENU;
			} else {
				pmc->pmc_sizeAnswerBuf = 1280;
				return false;
			}
			break;
	}
	return true;
}

int main (int argc, char ** argv) 
{
	char filename[50];           // Pointer to the firmware file name
        int opt,count;                   // aux variable
	char *aux;
	
	fileData = 0;
	strcpy(upgCmd,"reboot\n");

	if (shm_init(-1) < 0) {
		printf("\n[Error] - Could not start shared memory.\n");
		return -1;
	}
	
	if (! cy_shm->PMD_pid) {
		printf ("\n[Error] - PMD has not started.\n");
		return -1;
	}

	while ((opt = getopt(argc, argv, "hvfF")) != EOF) {
		switch (opt) {
			case 'h':           // help
				usage();
				return(-1);
				break;
			case 'f':           // don't perform checks
				force = 1;
				break;
			case 'v':           // show messages
				verbose = 1;
				break;
			case 'F':           // don't perform checks
				force = 2;
				break;
		}
	}
        argc -= optind;
        argv += optind;

	if ( argc == 0 ) {
		printf("[Error] - Missing arguments\n");
		usage(); // exit
		return -1;
	}
#ifdef ONS
	if (!memcmp(argv[0], "s", 1)) {
		auxflag = 0;
		serialPort = strtol(argv[0]+1,&aux,10);
	} else if (!memcmp(argv[0], "a", 1)) {
		auxflag = 1;
		serialPort = strtol(argv[0]+1,&aux,10) + AUX_OFFSET;
	} else {
		printf(" Error in syntax\n");
		usage();
		return -1;
	}
#else
	serialPort = strtol(argv[0],&aux,10);
#endif
	if ( (serialPort < 1) || (serialPort > MAX_DEV)) {
		printf("[Error] -  Invalid serial port number\n");
		return -2;
	}
	
	if (*aux == ':') {
		aux ++;
		unitNumber = atoi(aux);
	} else {
		unitNumber = 0;
	}
		
	if (argc == 2) {
		strcpy(filename,argv[1]);
	} else {
		// default file
		strcpy(filename,"/tmp/pmfirmware");
	}

	// Zeroing output buffer (for fast printings) and clearing the screen
	setbuf(stdout, NULL);
	setbuf(stdin, NULL);

	// check serial port and unit number
	if (cy_shm->line_conf[serialPort-1].protocol != 105) {
		printf("[Error] - The serial port is not configured as Power Management.\n");
		return -1;
	}
	numUnits = (int)cy_shm->PMD_pms[serialPort-1];
	if (numUnits == 0) numUnits = 1;
	if (verbose && (numUnits > 1)) {
		printf("\n[IMPORTANT WARNING]This software detected that you have %d unit connected in daisy-chain. \n",numUnits);  //[GY]2005/Nov/22  BUG#5658
	}
	if (unitNumber && (unitNumber > numUnits)) {
		printf("[Error] - The chain has only %d units.\n",numUnits);
		return -1;
	} 

	if (checkFile(filename) == false) {
		return -1;
	}

	if( InitPmcEntry(serialPort) == FALSE )  //[GY]2006/Jun/06
           return -1;

	// init signals
	if( initSigTreatment() )  //[GY]2006/Jun/06
           return -1;
	
	// Exit routine
	atexit(EndPmcEntry);
	
	// get PM version
	if (force != 2) {
		strcpy(pmc->pmc_cmd,"ver");
		upgState = STWAITVER;
		sendCommand(PMC_REQUEST);
	} else {
		upgState = STWAITFRST;
		sendOptionUpgrade();
	}
	
	while (TRUE) {
		sleep(2);
		// Signal Treatment
		if (got_sig & GOT_SIGINT) {
			EndPmcEntry();
			return -1;
			break;
		}
		if (got_sig & GOT_SIGUSR1) {
			printf("[Error] - PMD is closing\n");
                        //return for "pmfwupgrade -f"  [GY]2006/Jun/06
                        if( force == 1 )
                           return -1;
			break;
		}
		// Check if the pmcInfo structure is still alive.
		if ((pmc = cy_shm->pmc_list[pmcindex]) == NULL) {
			printf("[Error] - PMC structure was lost\n");
                        //return for "pmfwupgrade -f"  [GY]2006/Jun/06
                        if( force == 1 )
                           return -1;
			break;
		}
		if (got_sig & GOT_SIGUSR2) {
			got_sig &= ~GOT_SIGUSR2;
			if (!got_sigusr2()) {
				break;
			}
		}
	}
	if (verbose || !force) {
		count = 0;
GETVER:
		while (sleep(1) != 0);
		
		if (numUnits && (count++ < 60) && (numUnits != (int)cy_shm->PMD_pms[serialPort-1]))
			goto GETVER;
		
		strcpy(pmc->pmc_cmd,"ver");
		memset(pmc->pmc_answerBuf,0,1280);
		pmc->pmc_sizeAnswerBuf = 1280;
		upgState = STIDLE;
		sendCommand(PMC_REQUEST);
		while (1) {
		      if (pmc->pmc_status == PMC_ANSOK) break;
	      	      sleep(2);
		}
		printf("\nChecking the upgrade :\n");	
		if (strstr(pmc->pmc_answerBuf,"IPDU restarted")||
		    strstr(pmc->pmc_answerBuf,"IPDU is not ready")) {
			goto GETVER;
		}
		printf(pmc->pmc_answerBuf);
	}
	EndPmcEntry();
	return 0;
}
