/*
 * Module: serial.c
 *
 * Copyright:	(c) 1999-2002 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License or, at your option, any later version.
 *
*
*/

# include <stdio.h>
# include <sys/types.h>
# include <ctype.h>
# include <stdlib.h>
# include <sys/stat.h>
#include <stdarg.h>
# include <fcntl.h>
# include <termios.h>
# include <unistd.h>
# include <linux/ioctl.h>
# include <sys/ioctl.h>
# include <sys/time.h>
# include <time.h>
# include <string.h>
# include <signal.h>
#include <grp.h> 

# include <linux/shm.h>
# include <linux/sem.h>
# include <server.h>
# include <pmd.h>

#define ADMIN_GROUP             "admin"
#define ADMIN_USER              "admin"

#define alias	serverfarm
struct termios old_tio;

//[RK]May/11/06 - event notification
extern void writeevt(int, char *, ...);

int maxflgsendenter = 120; 
int printlog = 1;
static unsigned char *outletCommands[] = {OUTLET_COMMANDS, NULL};
static void IpduFinishCommands (ipduInfo *ipdu, unsigned char *pmsg); 


void FillPmcOutlets(pmcInfo *pmc,char * outlets);
void DeleteOutlets( char *command );
void GetSideNumbers(int *leftn, int *rightn, char **leftposition, char **rightposition, \
			char *slashpos);
void GetFullSequence(char *sequence, char **leftpos, char **rightpos, char *slashpos);
void ExpandOutlets(char *cmdsubstr);
int isNameCommand(char *cmd);

/* Trace types */
enum {
	TRACE_IPDU_TO,	// Output to IPDU
	TRACE_IPDU_FROM,  // Input from IPDU
	TRACE_PMC_TO,	// Output to the PMD client
	TRACE_PMC_FROM	// Input from the PMD client
};


// Message received from PM : ALARM: IPDU ....
static void mount_sysmsg(ipduInfo *ipdu, char * s)
{
	int numipdu,i;
	char curr[10], thre[10], *aux, phase[10];

	sscanf(s,"ALARM: IPDU %d:",&numipdu);

	i = 0;		
	aux = s;
	while (*aux && *aux != '(') aux++;
	aux++;
	while (*aux && *aux != ')') curr[i++]=*aux++;
	curr[i] = 0x00; i=0;
	while (*aux && *aux != '(') aux++;
	aux++;
	while (*aux && *aux != ')') thre[i++]=*aux++;
	thre[i] = 0x00;

	phase[0] = 0;
	if ((aux = strstr(s, " phase ")) != NULL) {
#ifdef ONS
		strncpy(phase, aux, 8);
		phase[8] = 0;
#else
		phase[0] = *(aux+7);
		phase[1] = 0;
	} else {
		strcpy(phase,"none");		
#endif
	}

	if (strstr(s,"over threshold")) {
#ifdef ONS
		syslog(LOG_ALERT,"[PMD]-%s Port %d: OVER CURRENT on IPDU %d%s (current:%s threshold:%s).",
			ipdu->auxflg ? "Aux" : "Serial",
			ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
			numipdu, phase, curr, thre);
#else
		//[RK]May/11/06 - event notification
		writeevt(27,"siiss",cy_shm->line_conf[ipdu->serialPort-1].alias, ipdu->serialPort,numipdu,phase,curr);
		//syslog(LOG_ALERT,"[PMD]-Serial Port %d: OVER CURRENT on IPDU %d%s (current:%s threshold:%s).",
		//	ipdu->serialPort, numipdu, phase, curr,thre);
#endif
		return;
	}		

	if (strstr(s,"fell below")) {
#ifdef ONS
		syslog(LOG_WARNING,"[PMD]-%s Port %d: Current is now back to normal on IPDU %d%s (current:%s threshold:%s).",
			ipdu->auxflg ? "Aux" : "Serial",
			ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
			numipdu, phase, curr, thre);
#else
		//[RK]May/11/06 - event notification
		writeevt(28,"siiss",cy_shm->line_conf[ipdu->serialPort-1].alias, ipdu->serialPort,numipdu,phase,curr);
		//syslog(LOG_WARNING,"[PMD]-Serial Port %d: Current is now back to normal on IPDU %d%s (current:%s threshold:%s).",
		//	ipdu->serialPort, numipdu, phase, curr, thre);
#endif
		return;
	}		
	syslog(LOG_ALERT,"[PMD]-Serial Port %d:%s.",ipdu->serialPort,s);
}

// verify the answer to client and mount the syslog message
static void answer_mount_sysmsg(pmcInfo *pmc)
{
	int nipdu;
	char aux[10],*next;

	next = pmc->pmc_answerBuf;
	while (*next && (*next == '\n')) next++;
	while (*next) {
		if (sscanf(next,"IPDU #%d: Overcurrent protection turned %s.",&nipdu,aux) == 2) {
			aux[strlen(aux)-1]= 0x00;
#ifdef ONS
			syslog(LOG_WARNING,"[PMD]-%s Port %d: Overcurrent protection turned %s on IPDU# %d by user %s.",
				pmc->pmc_aux ? "Aux" : "Serial",
				pmc->pmc_aux ? (pmc->pmc_ipdu+1-AUX_OFFSET) : (pmc->pmc_ipdu+1),
				aux,nipdu,pmc->pmc_username);
#else
			syslog(LOG_WARNING,"[PMD]-Serial Port %d: Overcurrent protection turned %s on IPDU# %d by user %s.",
				pmc->pmc_ipdu+1,aux,nipdu,pmc->pmc_username);
#endif
		} else {
			if (sscanf(next,"%d: Outlet turned %s.",&nipdu,aux) == 2) {
				aux[strlen(aux)-1]= 0x00;
#ifdef ONS
			syslog(LOG_ALERT,"[PMD]-%s Port %d: Outlet %d has been turned %s by user %s.",
				pmc->pmc_aux ? "Aux" : "Serial",
				pmc->pmc_aux ? (pmc->pmc_ipdu+1-AUX_OFFSET) : (pmc->pmc_ipdu+1),
				nipdu, aux,pmc->pmc_username);
#else
				//[RK]May/11/06 - event notifications
				if (strcmp(aux,"on") == 0) {
					if (pmc->iscycle & 2) {
						writeevt(26,"sii",pmc->pmc_username,pmc->pmc_ipdu+1,nipdu); 
					} else {
						writeevt(24,"sii",pmc->pmc_username,pmc->pmc_ipdu+1,nipdu); 
					}
				} else {
					if (pmc->iscycle & 2) {
						writeevt(41,"sii",pmc->pmc_username,pmc->pmc_ipdu+1,nipdu); 
					} else {
						writeevt(25,"sii",pmc->pmc_username,pmc->pmc_ipdu+1,nipdu); 
					}
				}
				//syslog(LOG_ALERT,"[PMD]-Serial Port %d: Outlet %d has been turned %s by user %s.",
				//	pmc->pmc_ipdu+1, nipdu,aux,pmc->pmc_username);
#endif
			}
		}
		while (*next && (*next != '\n')) next++;
		if (*next) next++;
	}
}

static void IpduTreatsDcdOFF(ipduInfo *ipdu)
{
	int pindex;
	pmcInfo *pmc;
	char mess[]="Ipdu is not available. Detected DCD OFF";
	
#ifdef ONS
	dodebug("%s[%d]: IpduTreatsDcdOff\n",
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort);
#else
	dodebug("Serial[%d]: IpduTreatsDcdOff\n",ipdu->serialPort);
#endif

	if (cy_shm->PMD_pms[ipdu->serialPort-1] != 0) { 
		if (syslogFlg) { 
#ifdef ONS
			if (printlog) syslog(LOG_INFO,"[PMD]-%s Port %d: IPDU was removed. DCD OFF.",
				ipdu->auxflg ? "Aux" : "Serial",
				ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort);
#else
			//[RK]May/11/06 - event notification
			writeevt(23,"si",cy_shm->line_conf[ipdu->serialPort-1].alias,ipdu->serialPort);
			//if (printlog) syslog(LOG_INFO,"[PMD]-Serial Port %d: IPDU was removed. DCD OFF.",ipdu->serialPort);
#endif
		}
		cy_shm->PMD_pms[ipdu->serialPort-1] = 0;
		cy_shm->PMD_outlets[ipdu->serialPort-1] = 0;
	}

	if (ipdu->logoutAfterCommandComplete) {
		ipdu->state = IPDU_INIT;
	} else if (ipdu->needAuth) {
		ipdu->state = IPDU_WAITUSERNAMESTR;
	} else {
		ipdu->state = IPDU_WAITPROMPTSTR;
	}

	pindex = ipdu->pmindex;
	ipdu->pmindex = -1;
	
	while (pindex != -1) {
		pmc = cy_shm->pmc_list[pindex];
		if (pmc) {
			strcpy(pmc->pmc_answerBuf,mess);
			dotrace(TRACE_PMC_TO, pmc->pmc_pid, 
				pmc->pmc_answerBuf, strlen(pmc->pmc_answerBuf));
			pmc->pmc_status = PMC_ANSOK;
			//dolog(LOG_INFO, "SIGUSR2 sent to process ID %d",pmc->pmc_pid);
#ifdef ONS
			dodebug("%s[%d]: IpduTreatsDcdOff SIGUSR2 sent to process ID %d\n",
				ipdu->auxflg ? "Aux" : "Serial",
				ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
				pmc->pmc_pid);
#else
			dodebug("Serial[%d]: IpduTreatsDcdOff SIGUSR2 sent to process ID %d\n",ipdu->serialPort,pmc->pmc_pid);
#endif
			kill(pmc->pmc_pid, SIGUSR2); /* Asks for the client to read the answer */
		}
		if (ipdu->command_get != ipdu->command_put) {
			pindex = ipdu->commands[ipdu->command_get].pmindex;
			ipdu->command_get = (ipdu->command_get + 1) % MAXIPDUQUEUE;
		} else {
			break;
		}
	}
}

/****************************************************************************
 * It sends a string to the IPDU
 ***************************************************************************/
static int IpduSendBin ( ipduInfo * ipdu, char *s, int len) 
{
	int signal=0, dcd = 0;
	
	if ((!ipdu->dcdDetectOff) && (ipdu->state != IPDU_UPGRADE) && 
		(ipdu->state != IPDU_WAITUPGRADEANS) &&
		(ipdu->state != IPDU_WAITUSERNAMESTR) && 
		(ipdu->state != IPDU_WAITRESTART)) {
		// check DCD status
		ioctl(ipdu->fd,TIOCMGET,&signal);
		dcd = (signal & TIOCM_CD);
		//syslog(LOG_INFO,"[PMD]-DCD is %d, Status is %d.",dcd, ipdu->dcdStatus);
		if ( dcd != ipdu->dcdStatus) {
			signal &= ~TIOCM_DTR;
			ioctl(ipdu->fd,TIOCMSET,&signal);
			usleep(1000);
			signal |= TIOCM_DTR;
			ioctl(ipdu->fd,TIOCMSET,&signal);
			ipdu->dcdStatus = dcd;
			printlog = 1;
		} else {
			printlog = 0;
		}
		if (!dcd) { // DCD is OFF
			IpduTreatsDcdOFF(ipdu);
			return 0; // erro
		}
	}

	if (!len) return 1;

	dotrace(TRACE_IPDU_TO, ipdu->serialPort, s, len);
	write (ipdu->fd, s, len);
	return 1;
}


/****************************************************************************
 * IpduSendStr - Sends a string, followed by an enter, to the IPDU. It also 
 * sets a timer.
 ***************************************************************************/
static int IpduSendStr (ipduInfo *ipdu, char *str)
{
	ipdu->bufindex = 0;
	ipdu->start_time = time(NULL);
	ipdu->retry = 0;
	if (str) {
		if (! IpduSendBin(ipdu, str,strlen(str))) {
			return 0;
		}
		//dotrace(TRACE_IPDU_TO, ipdu->serialPort, str, strlen(str));
	} else { 
		//dotrace(TRACE_IPDU_TO, ipdu->serialPort, enter, 1);
	}
	if (! IpduSendBin(ipdu, enter,1)) {
		return 0;
	}
	ipdu->flgSendEnter = time(NULL);
	return 1;
}

/****************************************************************************
 * IpduTimeoutCheck - Checks if a timeout for waiting a response was reached.
 ***************************************************************************/
static int IpduTimeoutCheck (ipduInfo *ipdu, int timeout, int maxtries, 
	int to_state)
{
	time_t current_time, time_cnt;
	current_time = time(NULL);
	time_cnt = current_time - ipdu->start_time;
	if (time_cnt >= timeout) {
		//dolog(LOG_DEBUG, "IPDU[%d] - Timeout", ipdu->serialPort);
#ifdef ONS
		dodebug("%s[%d]:IpduTimeoutCheck state=%d timeout(start[%d] current[%d] timeout[%d])\n",
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
		ipdu->state,ipdu->start_time,current_time, timeout);
#else
		dodebug("Serial[%d]:IpduTimeoutCheck state=%d timeout(start[%d] current[%d] timeout[%d])\n", ipdu->serialPort,ipdu->state,ipdu->start_time,current_time, timeout);
#endif
		ipdu->retry ++;
		if (ipdu->retry >= maxtries) {
			//dolog(LOG_DEBUG, "IPDU[%d] - Maximum of tries - changing state",ipdu->serialPort);
			dodebug("Maximum of tries[%d/%d] - changing state %d\n",ipdu->retry,maxtries,to_state);
			ipdu->state = to_state;
			ipdu->retry = 0;
			if (ipdu->state != IPDU_INIT) {
				ipdu->start_time = current_time;
			} else if (ipdu->logoutAfterCommandComplete && ipdu->gotNumOutlets) {
				IpduFinishCommands(ipdu, "[Error] - IPDU could not be logged in");
			} else if (IpduSendBin(ipdu,enter,1)) {
				if (ipdu->needAuth) {
					ipdu->state = IPDU_WAITUSERNAMESTR;
				} else {
					ipdu->state = IPDU_WAITPROMPTSTR;
				}
				ipdu->start_time = current_time;
			}
			return 1;
		} else {
			ipdu->bufindex = 0;
			ipdu->start_time = current_time;
			IpduSendBin(ipdu, enter,1);
		}
	}
	return 0;
}

/****************************************************************************
 * This routine tries to transmit an information to the client.
 ***************************************************************************/
static void RequestPasswordToUser (ipduInfo *ipdu)
{
	int pmindex = ipdu->pmindex;
	pmcInfo *pmc;
	char mess[]="[Error] - one command from PM need return by another user";
	char penter[]="\n";
	char *cmd;
	
	if (pmindex == -1) {
		return;
	}
	
	pmc = cy_shm->pmc_list[pmindex];
	if (!pmc) {
		ipdu->pmindex = 0;
		if (ipdu->admPasswd) {
			cmd = ipdu->admPasswd;
		} else {
			cmd = penter;
		}
		if (IpduSendStr(ipdu, cmd)) {
			ipdu->state = IPDU_WAITPROMPTSTR;
		}
		return;
	}
		
	pmc->pmc_status = PMC_PASSWD;
	//dolog(LOG_INFO, "SIGUSR2 sent to process ID %d",pmc->pmc_pid);
#ifdef ONS
	dodebug("%s[%d]:RequestPasswordToUser to process ID %d\n",
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
		pmc->pmc_pid);
#else
	dodebug("Serial[%d]:RequestPasswordToUser to process ID %d\n",ipdu->serialPort,pmc->pmc_pid);
#endif
	kill(pmc->pmc_pid, SIGUSR2);

	ipdu->state = IPDU_WAITUSERANS;

	do {
		if (ipdu->command_get != ipdu->command_put) {
			pmindex = ipdu->commands[ipdu->command_get].pmindex;
			ipdu->command_get = (ipdu->command_get + 1) % MAXIPDUQUEUE;
		} else {
			break;
		}
		pmc = cy_shm->pmc_list[pmindex];
		if (pmc) {
			strcpy(pmc->pmc_answerBuf,mess);
			dotrace(TRACE_PMC_TO, pmc->pmc_pid, 
				pmc->pmc_answerBuf,  
				strlen(pmc->pmc_answerBuf));
			pmc->pmc_status = PMC_ANSOK;
			//dolog(LOG_INFO, "SIGUSR2 sent to process ID %d",pmc->pmc_pid);
			dodebug("Serial[%d]:SIGUSR2 sent to process ID %d\n",ipdu->serialPort,pmc->pmc_pid);
			kill(pmc->pmc_pid, SIGUSR2); /* Asks for the client to read the answer */
		}
	} while (pmindex != -1);
}

static int SendToUser (int pmindex, char * s, int finish) {
	pmcInfo *pmc;
#ifdef ON_SEQ
	struct Tonqueue *onq = vet_onq[pmindex];
#endif
	int strsize, bufsize;
	if (pmindex == -1) {
		return 0;
	}

	pmc = cy_shm->pmc_list[pmindex];

	if (!pmc) {
		return 0;
	}
	//[RK]Nov/30/05 - fixed segmentation fault (pmc==0)
	strsize = strlen(s);
	bufsize = strlen(pmc->pmc_answerBuf);

	if ((strsize + bufsize) >= pmc->pmc_sizeAnswerBuf) {
		//dolog (LOG_NOTICE, "Answer in process %d ID is too long[%s]", pmindex,s);
		dodebug("Answer in process %d ID is too long [%s]==>[%s]\n", pmindex,pmc->pmc_answerBuf,s);
		strsize = pmc->pmc_sizeAnswerBuf-bufsize;
	}

	if (strsize > 0) {
		strncpy(pmc->pmc_answerBuf + bufsize, s, strsize);
		pmc->pmc_answerBuf[bufsize+strsize] = 0;
	}

	if (finish) {
#ifdef ON_SEQ
		if(pmc->iscycle){
			pmc->iscycle &= ~1;
			return 1;
		}
		if(onq != NULL && onq->number){
			onq->number--;
			if(onq->number > 0){ //was not the last "on"
				return 1;
			}
		}
#endif
		dotrace(TRACE_PMC_TO, pmc->pmc_pid, pmc->pmc_answerBuf, 
			strlen(pmc->pmc_answerBuf));
		if (syslogFlg) answer_mount_sysmsg(pmc);
		pmc->pmc_status = PMC_ANSOK;
		//dolog(LOG_INFO, "SIGUSR2 sent to process ID %d", pmc->pmc_pid);
#ifdef ONS
		dodebug("%s[%d]: SIGUSR2 sent to process ID %d\n", 
			pmc->pmc_aux ? "Aux" : "Serial",
			pmc->pmc_aux ? (pmc->pmc_ipdu-AUX_OFFSET) : pmc->pmc_ipdu,
			pmc->pmc_pid);
#else
		dodebug("Serial[%d]: SIGUSR2 sent to process ID %d\n", pmc->pmc_ipdu,pmc->pmc_pid);
#endif
		kill(pmc->pmc_pid, SIGUSR2); /* Asks for the client to read 
						the answer */
	}
	return 1;
}

/****************************************************************************
 * This routine should release all the commands and send a message for each 
 * of them. This will happen when there is a failed attempt to log in and 
 * logoutAfterCommandComplete is set.
 ***************************************************************************/
static void IpduFinishCommands (ipduInfo *ipdu, unsigned char *pmsg) 
{
	int pmindex = ipdu->pmindex;
	if (pmindex != -1) {
		SendToUser(pmindex, pmsg, 1);
	}
	while (1) {
		if (ipdu->command_get == ipdu->command_put) {
			ipdu->pmindex = -1;
			break;
		}
		pmindex = ipdu->commands[ipdu->command_get].pmindex;
		SendToUser(pmindex, pmsg, 1);
		ipdu->command_get = (ipdu->command_get + 1) % MAXIPDUQUEUE;
	}
	ipdu->pmupgidx = -1;
}
	
/****************************************************************************
 * This routine should get the next command and start treating it.
 ***************************************************************************/
static void IpduTreatNextCommand (ipduInfo *ipdu) {
	char *command;
	int pmindex;

	while (1) {
		if (ipdu->command_get == ipdu->command_put) {
			ipdu->pmindex = -1;
			if (ipdu->logoutAfterCommandComplete || 
				ipdu->gotNumOutlets == 0) {
				ipdu->start_time = time(NULL);
			}
			break;
		}

		command = ipdu->commands[ipdu->command_get].command;
		pmindex = ipdu->commands[ipdu->command_get].pmindex;
		ipdu->command_get = (ipdu->command_get + 1) % MAXIPDUQUEUE;
	
		switch (ipdu->state) {
		case IPDU_READY:
			strncpy(ipdu->comm, command, IPDUCOMMSIZE);
			ipdu->pmindex = pmindex;
			if (IpduSendStr (ipdu, ipdu->comm)) {
				ipdu->state = IPDU_WAITECHO;
			}
			return;
		case IPDU_INIT:
			/* Return error, and start authentication process */
			if (IpduSendStr (ipdu, NULL)) {
				if (ipdu->needAuth) {
					ipdu->state = IPDU_WAITUSERNAMESTR;
				} else {
					ipdu->state = IPDU_WAITPROMPTSTR;
				}
			}
		default:
			/* Send error to the client */
			SendToUser(pmindex, "[Error] - IPDU is not ready\r\n", 1);
			break;
		}
	}
}

/***************************************************************************************************
 **************************************************************************************************/
void DeleteOutlets( char *command )
{
	char *pcom = command;
	while (*pcom && !isdigit(*pcom)) pcom++;
	*pcom = '\0';	
}

//[RK]Oct/24/05 - I only tried to fix one problem, the logic used with the pmc_outlet need to be revised

int IpduCheckUserAccess(ipduInfo *ipdu, pmcInfo *pmc,int chg)
{
	IpduUserList *plist = ipdu->userlist;
	int j,k,found,maxout;
	char outs[MAXOutletsPS], aux[MAXOutletsPS],virg;
	char interv_str[10]; //[RK]Jun/26/06

	//[RK]Jun/26/06 	
	memset(interv_str,0,10);
	if(!strncmp(pmc->pmc_cmd,"interval",8)) {
		char *paux = pmc->pmc_cmd + 8;
		while (*paux && (*paux != ' ')) paux++;
		if (*paux) {
			paux++;
			while (*paux && (*paux != ' ')) paux++;
			strncpy(interv_str,paux,10);
			*paux = 0x00;
		}
	}

	memset(aux,0,MAXOutletsPS);
	FillPmcOutlets(pmc,aux);

	if(isNameCommand(pmc->pmc_cmd)) aux[1] = 0;
	if (!plist) return 0;

	maxout = cy_shm->PMD_outlets[ipdu->serialPort-1];
	
	//find for the username associate to this user in the list of users
	for (plist=ipdu->userlist; plist->username[0]; plist++) {
		if (!strcmp(plist->username,pmc->pmc_username))
			break; //ok, now we have the IpduUserList structure for this user
				//which contains a list of the outlets this user can manage
	}
	if (!plist->username[0]) return 0; // access deny, user not found

		
	if (aux[0] == 0) { // all
		char *temp;
		memcpy(outs,plist->outlets,MAXOutletsPS);
		//[RK]Oct/26/05
		// remove the word "all" of the command
		temp = strstr(pmc->pmc_cmd,"all");
		if (temp) *temp = 0x00; //[RK]Oct/31/05 - segmentation fault
	} else {
		//only one outlet in the command
		if (!aux[1]) {
			if (plist->outlets[(int)aux[0]]) {
				return 1; //user has permission for using this outlet
			} else {
				return 0;
			}
		}

		memset(outs,0,MAXOutletsPS);
		for (j=0; aux[j] != 0; j++) {
			k = aux[j];
			outs[k] = plist->outlets[k];
		}
	}

	memset(aux,0,MAXOutletsPS);

	//command will be rebuild based on the permissions of the user
	//if a mix of outlets for which the user has and doesnt have permission is in the command
	//only those for which he is allowed access will remain 

	//first, remove all outlets from the command
	//notice: this info remains in pmc->pmc_outlets
	DeleteOutlets(pmc->pmc_cmd);
	//[RK]Jun/26/06
	if (interv_str[0]) {
		strcat(pmc->pmc_cmd,interv_str);
	}

	// mount outstring
	k = found = 0;
	virg = ' ';
	for (j=1; j <= maxout; j++) {
		if (outs[j]) {
			if (!k) 
				k=found=j;
			else 
				found = j;
		} else {
			if (found) {
				if (k != found)
					sprintf(aux,"%c%d-%d",virg,k,found);
				else
					sprintf(aux,"%c%d",virg,k);
				if (chg) strcat(pmc->pmc_cmd,aux); // add outlets
				
				virg = ',';
				k=found=0;
			}
		}
	}
	if (found) {
		if (k != found)
			sprintf(aux,"%c%d-%d",virg,k,found);
		else
			sprintf(aux,"%c%d",virg,k);
		if (chg) strcat(pmc->pmc_cmd,aux); // add outlets
		virg = ',';
	}

	if (virg == ',') {
#ifdef CommandAdjustmentsAfterOutlet
		CommandAdjustmentsAfterOutlet(pmc->pmc_cmd);
#endif
		return 1;
	} else {
		return 0;
	}
}


/***************************************************************************************************
 isNameCommand - returns 1 if the ipdu command passed to him is a name outlet command, otherwise
 return 0

 cmd - string with the command to be verifyed
 **************************************************************************************************/
int isNameCommand(char *cmd)
{
	return ( strncmp(cmd,"name",4) ? 0 : 1);
}

/****************************************************************************
 * This routine should be called whenever a command is received from the PMD
 * client. This routine will put the command in the queue and schedule the 
 * command treatment. 
 ***************************************************************************/
int user_not_admin(char * username)
{
	struct group *grpa;
	char *pname,**pmem;

	if ((grpa = getgrnam(ADMIN_GROUP)) == NULL) {
		return 1;
	}
	
	pmem = grpa->gr_mem;
	while((pname = *pmem++)) {
		if (!strcmp(pname, username))
			return 0;
	}
	return 1;
}

/***************************************************************************************************
 FillPmcOultets - this function fills the array pmc->pmc_outlets, which is supposed to contain, in
 each position, the numerical value corresponding to each outlet that the command stored in 
 pmc->pmc_command is using

 pmc -> pointer to the structure containing pmc_command, which is the command for the ipdu in
 string format, and pmc_outlets, which is the list we are going to fullfill with each outlet
 number 
 **************************************************************************************************/
void FillPmcOutlets(pmcInfo *pmc,char *outlets)
{
	char obuf[5] = "";
	char mycom[1000];
	int pmcoi = 0; //pmc_outlet index
	char *pcom = pmc->pmc_cmd; 
	char *pbuf = obuf;

	bzero(mycom, sizeof(mycom));
	strncpy(mycom, pcom, sizeof(mycom));
	
	ExpandOutlets(mycom); //if there are intervals, expand it
	strcpy(pmc->pmc_cmd, mycom);
	
	while(!isdigit(*pcom)){ //find the first number 
		pcom++;
		if(!*pcom) return; //what? no digit found? is there any outlet number?
	}
	while(*pcom){
		while(isdigit(*pcom)){ //copies the outlet number to the temporary buffer
			*pbuf = *pcom;
			pbuf++;
			pcom++;
		}
		//found other thing than number, so we have the outlet number
		//closes the string with the necessary null terminator
		*pbuf = '\0';

		//converts it to integer and puts it into its proper location in 
		//pmc->pmc_outlets
		outlets[ pmcoi ] = atoi(obuf);

		//go to next position to insert outlet number in this array
		pmcoi++;

		//reset our buffer pointer
		pbuf = obuf;

		//and looks for the next digit, what will mark the beggining of
		//the next outlet number
		while(!isdigit(*pcom))
			if(*pcom) pcom++;
			else return; //we are finished with the outlets when this line is reached
	}

}


/***************************************************************************************************
 ExpandOutlets - if given a command with outlets intervals into its sole argument (like, for example,
 on 1,2,3-10,11,14-17), it will expand these intervals and rewrite the command with the full outlets
 listing in place of each interval.

 cmdsubstr - buffer which the command. It will be rewritten which the intervals substituted by
 full listings
 **************************************************************************************************/
void ExpandOutlets(char *cmdsubstr)
{
	char *slash=0, *leftp=0, *rightp=0;
	char seq[1000];
	char ibuf[1000];

	bzero(ibuf, sizeof(ibuf));
	bzero(seq, sizeof(seq));
	if(! (slash = strstr(cmdsubstr, "-"))) return;
	else{
		//gets into seq the full outlets sequence, separated by spaces
		GetFullSequence(seq, &leftp, &rightp, slash);

		//the left side of the command, before the interval
		strncpy(ibuf, cmdsubstr, leftp - cmdsubstr + 1);

		//the place were the interval was will now contain the full sequence
		strncat(ibuf, seq, sizeof(ibuf) - strlen(ibuf) - 1);

		//the right side of the command, after the interval
		strncat(ibuf, rightp, sizeof(ibuf) - strlen(ibuf) - 1);

		strcpy(cmdsubstr, ibuf);

		//keep expanding until there is no interval left
		ExpandOutlets(cmdsubstr);
	}
}

/***************************************************************************************************
 GetFullSequence - stores into the "sequence" argument the full listing of outlets belonging to an
 interval. It will, also, sore into the pointers leftpos and rightpos the the limits of the interval
 string (pointers for one position before it starts and one position after it into the original
 buffer)

 sequence - will be written which the full outlets listing for the interval
 leftpos - will be written with the addres of one position before the interval into the original 
 	   buffer
 rightpos - will be written with the address of one position after the interval into the original
 	   buffer
 slashpos - pointer for the slash position into the original buffer 
 
 **************************************************************************************************/
void GetFullSequence(char *sequence, char **leftpos, char **rightpos, char *slashpos)
{
	char buf[1000], sbuf[10];
	int outletn;
	int outlet1, outlet2;
	bzero(buf, sizeof(buf));
	
	GetSideNumbers(&outlet1, &outlet2, leftpos, rightpos, slashpos);	

	for(outletn = outlet1; outletn <= outlet2; outletn++){
		//we will insert the outlet plus a space, unless it is the last one
		snprintf(sbuf, sizeof(sbuf), (outletn == outlet2 ? "%d" : "%d "), outletn); 
		strncat(buf,sbuf, sizeof(buf) - strlen(buf) - 1);
	}

	strcpy(sequence, buf);
}

/***************************************************************************************************
 GetSideNumbers - get the numerical values for the outlets at the left and right side for an 
 interval

 leftn - integer pointer for the buffer where the left outlet value will be stored
 rightn - integer pointer for the buffer where the right outlet value will be stored
 leftposition - will be written with the pointer for the rightside substring into the original 
 		buffer
 rightposition - same as above, but this time is the rightside ending position
 slashpos - the only input value, the position of the slash into the original buffer
 **************************************************************************************************/
void GetSideNumbers(int *leftn, int *rightn, \
		    char **leftposition, char **rightposition, char *slashpos)
{

	char buf1[10];
	char *p, *sini, *sfinal;

	bzero(buf1,sizeof(buf1));
	p = slashpos;

	while(!isdigit(*p) ) p--; //find the ending of the left number
	sfinal = p;
	
	while(isdigit(*p) )  p--; //find the begining of the left number
	*leftposition = p;
	sini = p + 1;
	
	strncpy(buf1, sini, sfinal - sini + 1); //get the left number from the string
	*leftn = atoi(buf1);

	p = slashpos;
	bzero(buf1, sizeof(buf1));

	while(!isdigit(*p) && *p ) p++; //find the begining of the right number
	sini = p;
	
	while(isdigit(*p) && *p ) p++; //find the ending of the right number
	*rightposition = p;
	sfinal = p - 1;

	if(!isdigit(*sfinal)) return; //error, could not find right number

	strncpy(buf1, sini, sfinal - sini + 1); //get the left number from the string
	*rightn = atoi(buf1);
}
	
	

	

int IpduSendCommand (ipduInfo *ipdu, pmcInfo *pmc) {
	int ncommands;
	int pmindex = pmc->idx_pmclist;
#ifdef ON_SEQ
	struct Tonqueue *onq = vet_onq[pmindex];
#endif
	int auth_req;
	int chg = 0; // include or not the outlets in the command
	int i;

	if (pmc->pmc_status & PMD_NOAUTH) {
		auth_req = 0;
		pmc->pmc_status &= ~PMD_NOAUTH;
	} else {
		auth_req = 1;
	}

	if (pmc->pmc_status != PMC_REQUEST) {
		return 0;
	}

	if (!ipdu) {
		SendToUser(pmindex, "[Error] - Port is not configured as IPDU\r\n", 1);
		return 0;
	}

	// check for valid outlet
	for (i = 0; i < PMC_OUTLETS && pmc->pmc_outlet[i]; i++) {
		if (pmc->pmc_outlet[i] < 0 ||
			pmc->pmc_outlet[i] > cy_shm->PMD_outlets[ipdu->serialPort-1]) {
			SendToUser(pmindex, "[Error] - Outlet number is out of range\r\n", 1);
			return 0;
		}
	}

	dotrace(TRACE_PMC_FROM, pmc->pmc_pid, pmc->pmc_cmd, 
		strlen(pmc->pmc_cmd));

	if (ipdu->logout && ! strcmp(pmc->pmc_cmd, ipdu->logout)) {
		SendToUser(pmindex, "[Error] - This command cannot be issued\r\n", 1);
		return 0;
	}

	// check the IPDU state -- Event : PMC_REQUEST
	switch (ipdu->state) {
		case IPDU_INIT :
		case IPDU_WAITUSERNAMESTR :
		case IPDU_WAITPASSWDSTR :
		case IPDU_WAITPROMPTSTR :
			if (ipdu->logoutAfterCommandComplete) {
				break;
			}

		case IPDU_LOGINFAILED:
			SendToUser(pmindex,"[Error] - IPDU is not ready",1);
			return 0;
			
			
		case IPDU_WAITUSERANS :
			if ((ipdu->pmindex!=-1) && (pmindex != ipdu->pmindex)) {
				SendToUser(pmindex,"[Error] - IPDU is waiting answer from another user",1);
				return 0;
			}
			
		case IPDU_READY :
		case IPDU_WAITECHO :
		case IPDU_WAITSYSLOG :
		case IPDU_WAITANSWER :
			break;
			
		case IPDU_UPGRADE :
		case IPDU_WAITUPGRADEANS :
			if ((ipdu->pmindex != -1) && (pmindex != ipdu->pmindex)) {
				SendToUser(pmindex,"[Error] - Administrator is doing Upgrade Firmware",1);
				return 0;
			}
			SendToUser(pmindex,"[Error] - Upgrade Firmware NOK",1);
			ipdu->state = IPDU_READY;
			return 0;
			
		case IPDU_WAITRESTART :
			if ((ipdu->pmindex != -1) && (pmindex != ipdu->pmindex)) {
				SendToUser(pmindex,"[Error] - Administrator is doing Upgrade Firmware",1);
				return 0;
			}
			ipdu->state = IPDU_READY;
			break;

		default :
			SendToUser(pmindex,"[Error] - state machine wrong - state",1);
			return 0;
	}

	// check user access
	if (auth_req & user_not_admin(pmc->pmc_username)) { 
		chg = 0;
		for (i = 0; outletCommands[i] != NULL; i ++) {
			chg = chg || !strncmp(pmc->pmc_cmd, 
				outletCommands[i], strlen(outletCommands[i]));
		}
		if (!IpduCheckUserAccess(ipdu,pmc,chg)) {
#ifdef ON_SEQ
			if(strstr(pmc->pmc_cmd, ON_COMMAND)){
				struct Tonqueue *onq = vet_onq[pmindex];
				if (onq != NULL && --onq->number > 0) {
					pmc->pmc_status = PMC_IDLE;  //was not the last "on"
				}
                                else {  //[GY]2006/Mar/02  BUG#6622
                                   if( strlen(pmc->pmc_answerBuf) == 0 )
                                      SendToUser(pmindex,"[Error] - Not have permission to manage the outlet(s)",1);
                                   else
                                      SendToUser(pmindex," ",1);
                                }
				return 1;

			} else {
				SendToUser(pmindex,"[Error] - Not have permission to manage the outlet(s)",1);
				return 0;
			}
#else
			SendToUser(pmindex,"[Error] - Not have permission to manage the outlet(s)",1);
			return 0;
#endif
		}
	}

	if (ipdu->state == IPDU_INIT) {
		if (ipdu->logoutAfterCommandComplete) {
			if (IpduSendStr (ipdu, NULL)) {
				ipdu->state = IPDU_WAITUSERNAMESTR;
			} else {
				return 1;
			}
		}
	} else if (ipdu->state == IPDU_READY) {
		//if state is ready, send command immediately...
		pmc->pmc_status = PMC_WAITANS;	
		strncpy(ipdu->comm, pmc->pmc_cmd, IPDUCOMMSIZE);
		ipdu->pmindex = pmindex;
		if (IpduSendStr (ipdu, ipdu->comm)) {
			ipdu->state = IPDU_WAITECHO;
		}
		return 1;
	}

	//...otherwise put it in the queue to be sent later
	
	// Check the queue
	ncommands = (MAXIPDUQUEUE + ipdu->command_put - ipdu->command_get) % MAXIPDUQUEUE;
	if (ncommands >= MAXIPDUQUEUE - 1) {
		/* Queue is full; send error to client */
		SendToUser(pmindex, "[Error] - Command queue is full\r\n", 1);
		return 0;
	}

	pmc->pmc_status = PMC_WAITANS;	

	ipdu->commands[ipdu->command_put].pmindex = pmc->idx_pmclist;

#ifdef ON_SEQ
	//on commands receive spetial processing since it is broken into multiple commands
	if(onq != NULL && onq->index > 0) {
		ipdu->commands[ipdu->command_put].command = onq->queue[ onq->index - 1]; //index was incremented just after the array element be passed here
	} else {
		ipdu->commands[ipdu->command_put].command = pmc->pmc_cmd;
	}
#else
	ipdu->commands[ipdu->command_put].command = pmc->pmc_cmd;
#endif
	ipdu->command_put = (ipdu->command_put + 1) % MAXIPDUQUEUE;
	return 1;
}

int SendUpgAnswer(int pmcindex, int code)
{
	pmcInfo* pmc = cy_shm->pmc_list[pmcindex];

	if (!pmc) {
		return 0;
	}

	pmc->pmc_outlet[0] = (char) code;
	pmc->pmc_status = PMC_ANSOK;
	kill(pmc->pmc_pid, SIGUSR2); 
	return 1;
}

int IpduSendPassword (ipduInfo *ipdu, pmcInfo *pmc) 
{
	int pmindex = pmc->idx_pmclist;
	
	//dolog (LOG_DEBUG,"IpduSendPassword pmc=%d ipduState=%d",pmc->pmc_status,ipdu->state);
	dodebug("Serial[%d]:IpduSendPassword pmc=%d ipduState=%d\n",ipdu->serialPort,pmc->pmc_status,ipdu->state);
	if ((ipdu->pmindex != pmindex) || (ipdu->state != IPDU_WAITUSERANS)) {
		SendToUser(pmindex,"[Error]: Password was not required in this port",1);
		return FALSE;
	}
	if (IpduSendStr(ipdu,pmc->pmc_cmd)) {
		ipdu->state = IPDU_WAITANSWER;
	}
	return TRUE;
}

int IpduSendUpgrade (ipduInfo *ipdu, pmcInfo *pmc) 
{
	int pmindex = pmc->idx_pmclist;
	
	//dolog (LOG_DEBUG,"IpduSendUpgrade pmc=%d ipduState=%d",pmc->pmc_status,ipdu->state);
	dodebug("Serial[%d]: IpduSendUpgrade pmc=%d ipduState=%d\n",ipdu->serialPort,pmc->pmc_status,ipdu->state);
	// check the IPDU state -- Event : PMC_UPGRADE
	switch (ipdu->state) {
		case IPDU_INIT :
			if (ipdu->logoutAfterCommandComplete && 
				ipdu->pmupgidx == -1) { 
				int signal = 0;
				ipdu->pmindex = pmindex;
				ioctl(ipdu->fd,TIOCMGET, &signal);
				if (!(signal & TIOCM_CD) && 
					pmc->pmc_outlet[1] == 0xff) {
					ipdu->state = IPDU_WAITUSERNAMESTR;
					break;
				} else if (IpduSendStr (ipdu, NULL)) {
					ipdu->state = IPDU_WAITUSERNAMESTR;
					ipdu->pmupgidx = pmindex;
					pmc->pmc_status = PMC_WAITANS;
					return 1;
				} else {
					return 0;
				}
			}
			pmc->pmc_sizeAnswerBuf = 500;
			SendToUser(pmindex,"[Error] - IPDU is not ready",1);
			return 0;

		case IPDU_WAITPASSWDSTR :
		case IPDU_WAITPROMPTSTR :
			pmc->pmc_sizeAnswerBuf = 500;
			SendToUser(pmindex,"[Error] - IPDU is not ready",1);
			return 0;

		case IPDU_LOGINFAILED:
			pmc->pmc_sizeAnswerBuf = 500;
			SendToUser(pmindex,"[Error] - IPDU is not ready",1);
			return 0;
			
		case IPDU_WAITUSERANS :
			if ((ipdu->pmindex!=-1) && (pmindex != ipdu->pmindex)) {
				pmc->pmc_sizeAnswerBuf = 500;
				SendToUser(pmindex,"[Error] - IPDU is waiting answer from another user",1);
				return 0;
			}
			
		case IPDU_READY :
		case IPDU_WAITECHO :
		case IPDU_WAITSYSLOG :
		case IPDU_WAITANSWER :
			break;
			
		case IPDU_WAITUSERNAMESTR :
			if (pmc->pmc_outlet[1] != 0xff) {
				pmc->pmc_sizeAnswerBuf = 500;
				SendToUser(pmindex,"[Error] - IPDU is not ready",1);
				return 0;
			}
		case IPDU_UPGRADE :
			if ((ipdu->pmindex != -1) && (pmindex != ipdu->pmindex)) {
				pmc->pmc_sizeAnswerBuf = 500;
				SendToUser(pmindex,"[Error] - Administrator is doing Upgrade Firmware",1);
				return 0;
			}
			break;
			
		case IPDU_WAITRESTART :
			if ((ipdu->pmindex!=-1) && (pmindex != ipdu->pmindex)) {
				pmc->pmc_sizeAnswerBuf = 500;
				SendToUser(pmindex,"[Error] - Administrator is doing Upgrade Firmware",1);
			}
			break;

		case IPDU_WAITUPGRADEANS :
			if ((ipdu->pmindex!=-1) && (pmindex != ipdu->pmindex)) {
				SendToUser(pmindex,"[Error] - Administrator is doing Upgrade Firmware",1);
			}
			return 0;
			
		default :
			SendToUser(pmindex,"[Error] - state machine wrong",1);
			return 0;
	}
	
	ipdu->pmindex = pmc->idx_pmclist;
	ipdu->bufindex = 0;
	ipdu->start_time = time(NULL);
	ipdu->retry = 0;
	ipdu->flgSendEnter = time(NULL);
	if (! IpduSendBin(ipdu, pmc->pmc_answerBuf,pmc->pmc_sizeAnswerBuf)) {
		return 1;
	}
	if (pmc->pmc_status == PMC_UPGRADE_END) {
		ipdu->state = IPDU_WAITRESTART;
	} else {
		ipdu->state = IPDU_WAITUPGRADEANS;
	}
	pmc->pmc_status = PMC_WAITANS;
	//dolog (LOG_DEBUG,"IpduSendUpgrade new ipduState=%d",ipdu->state);
#ifdef ONS
	dodebug("%s[%d]:IpduSendUpgrade new ipduState=%d\n",
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
		ipdu->state);
#else
	dodebug("serial[%d]:IpduSendUpgrade new ipduState=%d\n",ipdu->serialPort,ipdu->state);
#endif
	return 1;
}

void treatUpgOption(ipduInfo *ipdu, int pmindex)
{
	pmcInfo *pmc;

	pmc = cy_shm->pmc_list[pmindex];
	
	if (!pmc) {
		ipdu->state = IPDU_READY;
		return;
	}

	if (pmc->pmc_outlet[1] == 0xff) {
		// force the upgrade
		write(ipdu->fd,"1\n",2);
		ipdu->state = IPDU_WAITUPGRADEANS;
		return;
	}
	strcpy(pmc->pmc_answerBuf,"Enter choice (1 or 2): ");
	SendUpgAnswer(pmindex,PMC_UPG_OK);
	ipdu->state = IPDU_UPGRADE;
}

#ifdef UPLOAD_COMMAND
static void checkUploadPre (ipduInfo * ipdu)
{
	char *s;
	if (findstr(ipdu->buffer, UPLOADCONFIRM, ipdu->bufidx)) {
		IpduSendStr(ipdu, "YES");
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
	} else if (findstr(ipdu->buffer, UPLOADMENUHEADER, ipdu->bufidx) && 
		findstr(ipdu->buffer, UPLOADMENUPROMPT, ipdu->bufidx)) {
		unsigned char speed[2];
		/* We will leave the option 2, which stands for 9600 bps*/
		speed[0] = '0' + 2;
		speed[1] = 0;
		IpduSendStr(ipdu, speed);
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
	} else if ((s = findstring(ipdu->buffer, UPLOADSTART1, ipdu->bufidx)) != NULL 
		&& findstring(ipdu->buffer, UPLOADSTART2, ipdu->bufidx)) {
		/* This part is not working yet (why?) */
		//int speed;
		//struct termios tio;
		//speed = atoi(s + 20);
		//tcgetattr(ipdu->fd, &tio);
		//if (speed == 2400) {
		//	speed = B2400;
		//} else if (speed == 9600) {
		//	speed = B9600;
		//} else if (speed == 19200) {
		//	speed = B19200;
		//} else if (speed == 38400) {
		//	speed = B38400;
		//} else {
		//	speed = ipdu->speed;
		//}
		//if (ipdu->speed != speed) {
		//	cfsetispeed(&tio, speed);
		//	cfsetospeed(&tio, speed);
		//	tcsetattr(ipdu->fd, TCSANOW, &tio);
		//	sleep(1);
		//}
		IpduSendStr(ipdu, NULL);
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
	} else if ((s = findstring(ipdu->buffer, UPLOADSTART3, ipdu->bufidx)) != NULL) {
		strcpy(ipdu->buffer, s + 30);
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
		ipdu->state = IPDU_WAITUPGRADEANS;
	}
}
#endif


/****************************************************************************
 * This routine looks like the strstr function in the strings library; the 
 * only difference is that the haystack is not necessarily a string, and 
 * the needle will be searched until the full length of the haystack, even
 * if a NULL is found in the middle of it
 ***************************************************************************/
static char *findstring(char *haystack, char *needle, int len)
{
	char *p, *q;
	int i;
	if (len < strlen(needle)) {
		return NULL;
	}
	for (i = 0; i <= len - strlen(needle); i ++) {
		for (p = haystack + i, q = needle; *q && *p == *q; p++, q++);
		if (! *q) return (haystack + i);
	}
	return NULL;
}

/****************************************************************************
 * This routine is called by the PMD main loop, and it should check the 
 * timeouts and the information received from the IPDU.
 ***************************************************************************/
void IpduHandlePort (ipduInfo * ipdu)
{
	int cnt, old_state = ipdu->state, events = 0;
	unsigned char *s, *s1, *ppwd, *pprt, *pecho, *pcon,*ptmostr;
	int pmindex = ipdu->pmindex;
	int curtime,aux;
	char aux1;

	curtime=time(NULL);

	ppwd = pprt = pecho= pcon = ptmostr = s = s1 = NULL;

	cnt = IPDUBUFSIZE - ipdu->bufindex - 1;
	if (cnt <= 20) {
		//dolog(LOG_NOTICE, "Input buffer overflow in port %d", ipdu->serialPort);
#ifdef ONS
		dodebug("%s[%d] : IpduHandlePort : Input buffer overflow\n",
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort);
#else
		dodebug("Serial[%d] : IpduHandlePort : Input buffer overflow\n", ipdu->serialPort);
#endif
		cnt = ipdu->bufindex-1;
		while (ipdu->buffer[cnt] != '\n') cnt--;
		if (cnt != ipdu->bufindex-1) {
			aux1 = ipdu->buffer[cnt+1];
		} else {
			aux1 = 0x00;
		}
		ipdu->buffer[cnt+1] = 0x00;
		SendToUser(pmindex, ipdu->buffer, 0);
		ipdu->buffer[cnt+1] = aux1;
		strcpy(ipdu->buffer, &ipdu->buffer[cnt+1]);
		ipdu->bufindex -= cnt+1; 
		cnt = IPDUBUFSIZE - ipdu->bufindex - 1;
	}
	cnt = read(ipdu->fd, ipdu->buffer + ipdu->bufindex, cnt);
	// check Events => IPDU_GOTUPGANS
	if ((cnt == 1) && (ipdu->state == IPDU_WAITUPGRADEANS)) {
		char cbin = ipdu->buffer[ipdu->bufindex];
		int ret=0;
		
		events = 0;
		
		switch (cbin & 0x7f) {
			case 0x18 :
				ret = SendUpgAnswer(pmindex,PMC_UPG_CAN);
				events = 1;
				break;
			case 0x15 :
				ret = SendUpgAnswer(pmindex,PMC_UPG_NAK);
				events = 1;
				break;
			case 0x06 :
				ret = SendUpgAnswer(pmindex,PMC_UPG_OK);
				events = 1;
				break;
			case 0x43 :
				ret = SendUpgAnswer(pmindex,PMC_UPG_CRC);
				events = 1;
				break;
		}
		if (events) {
			if (ret) 
				ipdu->state = IPDU_UPGRADE;
			else
				ipdu->state = IPDU_READY;

			ipdu->bufindex = 0;
			ipdu->start_time = time(NULL);
			return;
		}
	}

	if (ipdu->state == IPDU_WAITRESTART) {
		if ((ipdu->buffer[ipdu->bufindex] & 0x7F) == 0x15) {
			char eot[2];
			eot[0] = 4;
			IpduSendBin(ipdu,eot,1);
			ipdu->bufindex = 0;
			ipdu->start_time = time(NULL);
			return;
		}
	}	

	// check another events
	if (cnt > 0) {
		ipdu->bufindex += cnt;

	//	s = s1 = ipdu->buffer;
		dotrace(TRACE_IPDU_FROM, ipdu->serialPort, 
			ipdu->buffer + ipdu->bufindex - cnt, cnt);
#ifdef MENUPROMPT
		if ((s=findstring(ipdu->buffer, MENUPROMPT, ipdu->bufindex)) != NULL) {
			IpduSendBin(ipdu, MENUPROMPTANSWER, 
				strlen(MENUPROMPTANSWER));
			ipdu->bufindex = 0;
			ipdu->start_time = time(NULL);
			ipdu->state = IPDU_WAITUSERNAMESTR;
		}
#endif
#ifdef MENUPROMPT2
		if ((s=findstring(ipdu->buffer, MENUPROMPT2, ipdu->bufindex)) != NULL) {
			IpduSendBin(ipdu, MENUPROMPT2ANSWER, 
				strlen(MENUPROMPT2ANSWER));
			ipdu->bufindex = 0;
			ipdu->start_time = time(NULL);
			ipdu->state = IPDU_WAITUSERNAMESTR;
		}
#endif
#ifdef UPLOAD_COMMAND
		if (ipdu->state == IPDU_UPLOADPRE) {
			checkUploadPre(ipdu);
		}
#else
		// check Upgrade Firmware messages 
		if ((s=findstring(ipdu->buffer,"Enter choice (1 or 2):", ipdu->bufindex))) {
			events |= IPDU_GOTUPGOPTION;
		}
		if ((s=findstring(ipdu->buffer, "Please send file in 10 seconds.", ipdu->bufindex))) {
			events |= IPDU_GOTUPGSTART;
		}
		if ((s=findstring(ipdu->buffer,"DCD went ON on", ipdu->bufindex))) {
			s1 = s;
			while (*s1 && (*s1!='\r')) s1++;
			if (*s1) events |= IPDU_GOTUPGDONE;
		}
#endif
		// Got number of ipdus and number of outlets
		if (ipdu->numberIPDUs && 
			(s = findstring(ipdu->buffer, ipdu->numberIPDUs, ipdu->bufindex)) != NULL) {
			for (cnt=strlen(ipdu->numberIPDUs); 
				s[cnt] && isdigit(s[cnt]);
				cnt++); 
			if (s[cnt] == '\r') { 
				s[cnt++]= 0x00;
				cy_shm->PMD_pms[ipdu->serialPort-1] = atoi(s+12);
				dodebug("Got number of IPDUS : %s\n", s + 12);
#ifdef OEM3
				if (ipdu->state != IPDU_WAITANSWER) 
#endif
				{
					strcpy(s, s+cnt);
					ipdu->bufindex -= cnt;
				}
			}
		}
		if (ipdu->numberOutlets && 
			(s = findstring(ipdu->buffer, ipdu->numberOutlets, ipdu->bufindex)) != NULL) {
			char number[8];
			strncpy (number, s + strlen(ipdu->numberOutlets), 8);
			for (cnt=0; number[cnt] && isdigit(number[cnt]);cnt++);
			if (number[cnt] == '\r') { 
				number[cnt++]= 0x00;
				aux = cy_shm->PMD_outlets[ipdu->serialPort-1];
				cy_shm->PMD_outlets[ipdu->serialPort-1] = atoi(number);
				dodebug("Got number of outlets : %s\n", number);
				ipdu->gotNumOutlets = 1;
#ifdef OEM3
				if (ipdu->state != IPDU_WAITANSWER) 
#endif
				{
					cnt += strlen(ipdu->numberOutlets);
					strcpy(s, s + cnt);
					ipdu->bufindex -= cnt;
				}
				if (! ipdu->numberIPDUs) {
					cy_shm->PMD_pms[ipdu->serialPort-1] = 1;
				}
				if (syslogFlg) {
#ifdef ONS
					if (aux == 1) {
						syslog(LOG_NOTICE,"[PMD]-%s Port %d: "
						"PMD has started on this port."
						" The chain has %d IPDUs and %d outlets",
						ipdu->auxflg ? "Aux" : "Serial",
						ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
						cy_shm->PMD_pms[ipdu->serialPort-1],
						cy_shm->PMD_outlets[ipdu->serialPort-1]);
					} else {
						syslog(LOG_NOTICE,"[PMD]-%s Port %d: "
						"One or more IPDUs were %s the chain."
						" The chain has now %d IPDUs and %d outlets",
						ipdu->auxflg ? "Aux" : "Serial",
						ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort, 
						((aux > cy_shm->PMD_outlets[ipdu->serialPort-1])?
						 "removed from" : "added to"),
						cy_shm->PMD_pms[ipdu->serialPort-1],
						cy_shm->PMD_outlets[ipdu->serialPort-1]);
					}
#else
					//[RK]May/11/06 - event notification
					writeevt(22,"siii",cy_shm->line_conf[ipdu->serialPort-1].alias,
					ipdu->serialPort, cy_shm->PMD_pms[ipdu->serialPort-1],
                                               cy_shm->PMD_outlets[ipdu->serialPort-1]);
#endif
				}
			}
		}

		// check syslog message
		if (ipdu->syslog) {
			aux = 0;
			while ((s = findstring(ipdu->buffer, ipdu->syslog, ipdu->bufindex)) != 
				NULL) {
				// Syslog header found. Check for the end of it
				for (s1 = s; (int)s1 - (int)(ipdu->buffer) < ipdu->bufindex && *s1 != '\n'; s1 ++);
				if ((int)s1 - (int)(ipdu->buffer) < ipdu->bufindex) {
					/* The end of the syslog message
					 *  was found. Send it to 
					 *  syslog-ng */
					*s1 = 0;
					mount_sysmsg(ipdu,s); 

					/* Remove the syslog from the 
					 * buffer */
					strcpy(s, s1+1);
					ipdu->bufindex -= s1+1-s;
					if (ipdu->state == IPDU_WAITSYSLOG) {
						aux = IPDU_ENDSYSLOG;
					}
				} else {
					aux = IPDU_GOTSYSLOG;
					break;
				}
			}
			events |= aux;
		}
		
		if (ipdu->username && ((s = findstring(ipdu->buffer, ipdu->username, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTUSERNAMESTR;
		}
		if (ipdu->password && ((ppwd = findstring(ipdu->buffer, ipdu->password, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTPASSWDSTR;
		}
		if (ipdu->confirm && ((pcon = findstring(ipdu->buffer, ipdu->confirm, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTCONFIRM;
		}
		if (ipdu->prompt && ((pprt = findstring(ipdu->buffer, ipdu->prompt, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTPROMPTSTR;
		}
		if (ipdu->timeout && ((ptmostr = findstring(ipdu->buffer, ipdu->timeout, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTTIMEOUTSTR;
		}
		if (ipdu->comm[0] && ((pecho = findstring(ipdu->buffer, ipdu->comm, ipdu->bufindex)) != NULL)) {
			events |= IPDU_GOTECHO;
		}

		if (events) {
			//dolog(LOG_DEBUG, "IPDU[%d] - Got Events =0x%x", ipdu->serialPort, events);
#ifdef ONS
			dodebug("%s[%d]: IpduHandlePort: state=%d - Events=0x%x\n",
				ipdu->auxflg ? "Aux" : "Serial", 
				ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
			ipdu->state, events);
#else
			dodebug("Serial[%d]: IpduHandlePort: state=%d - Events=0x%x\n", ipdu->serialPort, ipdu->state, events);
#endif
		}
	}

	// check the IPDU state --  Event - IPDU_GOTECHO
	if (events & IPDU_GOTECHO) {
		switch (ipdu->state) {
			case IPDU_WAITECHO :
#ifdef UPLOAD_COMMAND
				if (! strcmp(ipdu->comm, UPLOAD_COMMAND)) {
					ipdu->state = IPDU_UPLOADPRE;
					cnt = ((int)pecho-(int)ipdu->buffer)+ 
						strlen(ipdu->comm);
					strcpy(ipdu->buffer, ipdu->buffer+cnt);
					ipdu->bufindex -= cnt;
					checkUploadPre(ipdu);
					break;
				}
#endif
				ipdu->state = IPDU_WAITANSWER;
				cnt = strlen(ipdu->comm);
				strcpy(pecho, pecho+cnt);
				ipdu->bufindex -= cnt;
				break;
				
			case IPDU_WAITUPGRADEANS :
				cnt = strlen(ipdu->comm);
				strcpy(pecho, pecho+cnt+1);
				ipdu->bufindex -= cnt+1;
				break;
				
			case IPDU_WAITRESTART :
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_WAITUSERANS :
			case IPDU_READY :
			case IPDU_WAITSYSLOG :
			case IPDU_WAITANSWER :
			case IPDU_UPGRADE :
				break;
		}
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTUSERNAMESTR
	if (events & IPDU_GOTUSERNAMESTR) {
		switch (ipdu->state) {
			/* IPDU_UPGRADE IPDU_WAITSYSLOG IPDU_READY IPDU_WAITUSERANS :
			   IPDU_WAITPASSWDSTR IPDU_WAITUSERNAMESTR : break; */
			
			case IPDU_INITCOMM :
			case IPDU_WAITPROMPTSTR :
				/* Username string gotten - the authentication was not
			 	 * completed */
				IpduFinishCommands(ipdu, "[Error] - IPDU authentication error. Check if username/password is correct\r\n");
				if (ipdu->timeAfterFailedLogin) {
					ipdu->state = IPDU_LOGINFAILED;
					ipdu->start_time = time(NULL);
					//[RK]May/11/06 - event notification
					writeevt(23,"",cy_shm->line_conf[ipdu->serialPort-1].alias,
							ipdu->serialPort);

				} else {
					ipdu->state = IPDU_INIT;
				}
				break;
				
			case IPDU_WAITECHO :
			case IPDU_WAITANSWER :
				IpduFinishCommands(ipdu, "[Error] - IPDU restarted");
				//[RK]May/11/06 - event notification
				writeevt(23,"",cy_shm->line_conf[ipdu->serialPort-1].alias,
						ipdu->serialPort);

				ipdu->state = IPDU_INIT;
				break;
				
			case IPDU_WAITRESTART :
				SendUpgAnswer(pmindex,PMC_UPG_OK);
				break;

			case IPDU_WAITUPGRADEANS :
				SendUpgAnswer(pmindex,PMC_UPG_CAN);
				break;

			default :
				break;
			
		}
		if ((ipdu->state == IPDU_INIT &&
			ipdu->logoutAfterCommandComplete && 
			ipdu->gotNumOutlets) || 
			ipdu->state == IPDU_LOGINFAILED
#ifdef BUFFERIZED_OUTPUT
			|| ipdu->state == IPDU_WAITPASSWDSTR
#endif
		) {
			/* don't do anything */
		} else {
			if (IpduSendStr(ipdu, ipdu->admUser)) {
				if (ipdu->admPasswd == NULL) {
					ipdu->state = IPDU_WAITPROMPTSTR;
				} else {
					ipdu->state = IPDU_WAITPASSWDSTR;
				}
			}
			ipdu->bufindex = 0;
			ipdu->start_time = time(NULL);
		}
	}

	// check the IPDU state --  Event - IPDU_GOTPASSWDSTR
	if (events & IPDU_GOTPASSWDSTR) {
		switch (ipdu->state) {
			case IPDU_WAITPASSWDSTR :
				if (ipdu->admPasswd == NULL ||
					IpduSendStr(ipdu, ipdu->admPasswd)) {
					ipdu->state = IPDU_WAITPROMPTSTR;
				}
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;
				
			case IPDU_WAITANSWER :
				RequestPasswordToUser(ipdu);
				ipdu->state = IPDU_WAITUSERANS;
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;
				
			default: 
				cnt = strlen(ipdu->password);
				strcpy(ppwd, ppwd+cnt+1);
				ipdu->bufindex -= cnt+1;
				break;
		}
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTPROMPTSTR
	if (events & IPDU_GOTPROMPTSTR) {
		switch (ipdu->state) {
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
#ifdef UPLOAD_COMMAND
			case IPDU_UPLOADPRE:
#endif
				// the session is Ready
				ipdu->bufindex = 0;
				ipdu->state = IPDU_READY;
				if (ipdu->pmupgidx != -1) {
					pmcInfo* pmc = cy_shm->pmc_list[ipdu->pmupgidx];
					IpduSendUpgrade (ipdu, pmc);
					ipdu->pmupgidx = -1;
				} else {
					IpduTreatNextCommand(ipdu);
				}
				break;
				
			case IPDU_WAITRESTART :
#ifdef UPLOAD_COMMAND
				SendUpgAnswer(pmindex,PMC_UPG_OK);
				/* This part is not working yet (why?) */
				//{ 
				//	int speed;
				//	struct termios tio;
				//	tcgetattr(ipdu->fd, &tio);
				//	speed = cfgetispeed(&tio);
				//	if (ipdu->speed != speed) {
				//		cfsetispeed(&tio, ipdu->speed);
				//		cfsetospeed(&tio, ipdu->speed);
				//		tcsetattr(ipdu->fd, TCSANOW, 
				//			&tio);
				//	}
				//}
#else
				break;
#endif

			case IPDU_INITCOMM :
			case IPDU_WAITPROMPTSTR :
				ipdu->bufindex = 0;
				if (ipdu->initcomm && 
					ipdu->state != IPDU_INITCOMM) {
					if (IpduSendStr(ipdu, ipdu->initcomm)) {
						ipdu->state = IPDU_INITCOMM;
					}
				} else {
					ipdu->state = IPDU_READY;
					if (ipdu->pmupgidx != -1) {
						pmcInfo* pmc = cy_shm->pmc_list[ipdu->pmupgidx];
						IpduSendUpgrade (ipdu, pmc);
						ipdu->pmupgidx = -1;
					} else {
						IpduTreatNextCommand(ipdu);
					}
				}
				break;
				
			case IPDU_WAITUSERANS :
			case IPDU_WAITANSWER : 
				s = findstring(ipdu->buffer, ipdu->prompt, ipdu->bufindex);
				*s = 0;
				s = ipdu->buffer;
				while (*s == '\r' || *s == '\n') s++;
				SendToUser(pmindex, s, 1);
				ipdu->bufindex = 0;
				ipdu->comm[0] = 0x00;
				ipdu->state = IPDU_READY;
				IpduTreatNextCommand (ipdu);
				break;

			case IPDU_READY :
			case IPDU_WAITECHO :
			case IPDU_WAITSYSLOG :
			case IPDU_UPGRADE :
			case IPDU_WAITUPGRADEANS :
				cnt = strlen(ipdu->prompt);
				strcpy(ipdu->buffer, pprt + cnt);
				ipdu->bufindex -= (int)pprt - (int)ipdu->buffer + cnt;
				break;
		}
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTUPGOPTION
	if (events & IPDU_GOTUPGOPTION) {
		switch (ipdu->state) {
			case IPDU_WAITECHO :
			case IPDU_WAITANSWER :
				SendToUser(pmindex,"[Error] - IPDU restarted",1);

			case IPDU_WAITUSERNAMESTR :
				if (ipdu->pmupgidx != -1) {
					pmcInfo* pmc = cy_shm->pmc_list[ipdu->pmupgidx];
					IpduSendUpgrade (ipdu, pmc);
					ipdu->pmupgidx = -1;
					
					break;
				}
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_READY :
			case IPDU_WAITUSERANS :
			case IPDU_WAITSYSLOG :
			case IPDU_WAITRESTART :
				if (IpduSendBin(ipdu,"2\n",2)) {
					ipdu->state = IPDU_WAITUSERNAMESTR;
				}
				break;
				
			case IPDU_UPGRADE :
			case IPDU_WAITUPGRADEANS :
				treatUpgOption(ipdu,pmindex);
				break;
		}
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTUPGSTART
	if (events & IPDU_GOTUPGSTART) {
		switch (ipdu->state) {
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_WAITUSERANS :
			case IPDU_READY :
			case IPDU_WAITSYSLOG :
			case IPDU_WAITUPGRADEANS :
			case IPDU_WAITRESTART :
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;
				
			case IPDU_WAITECHO :
			case IPDU_WAITANSWER :
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;

			case IPDU_UPGRADE :
				//???????
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;
		}
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTUPGDONE
	if (events & IPDU_GOTUPGDONE) {
		switch (ipdu->state) {
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_WAITUSERANS :
			case IPDU_READY :
			case IPDU_WAITSYSLOG :
			case IPDU_WAITUPGRADEANS :
			case IPDU_WAITECHO :
			case IPDU_WAITANSWER :
			case IPDU_UPGRADE :
				break;
				
			case IPDU_WAITRESTART :
				ipdu->state = IPDU_READY;
				SendUpgAnswer(pmindex,PMC_UPG_OK);
				ipdu->bufindex = 0;
				ipdu->start_time = time(NULL);
				break;
		}
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - IPDU_GOTCONFIRM
	if (events & IPDU_GOTCONFIRM) {
		switch (ipdu->state) {
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_WAITUSERANS :
			case IPDU_READY :
			case IPDU_WAITECHO :
			case IPDU_WAITSYSLOG :
			case IPDU_UPGRADE :
			case IPDU_WAITUPGRADEANS :
			case IPDU_WAITRESTART :
				break;
			case IPDU_WAITANSWER :
				/* Send Confirmation */
				IpduSendStr(ipdu, ipdu->admConfirm);
				strncpy(ipdu->comm, ipdu->admConfirm, IPDUCOMMSIZE);
				break;
		}
		cnt = strlen(ipdu->confirm);
		strcpy(pcon, pcon+cnt+1);
		ipdu->bufindex -= cnt+1;
	}
	
	// check the IPDU state --  Event - IPDU_GOTTIMEOUTSTR
	if (events & IPDU_GOTTIMEOUTSTR) {
		switch (ipdu->state) {
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
			case IPDU_WAITPROMPTSTR :
			case IPDU_WAITUSERANS :
			case IPDU_WAITSYSLOG :
			case IPDU_READY :
				// It logged out by timeout. Proceeed with the authentication process
				if (IpduSendStr (ipdu, NULL)) {
					if (ipdu->needAuth) {
						ipdu->state = IPDU_WAITUSERNAMESTR;
					} else {
						ipdu->state = IPDU_WAITPROMPTSTR;
					}
				}
				break;
				
			case IPDU_WAITECHO :
			case IPDU_WAITANSWER :
				SendToUser(pmindex,"[Error] - IPDU restarted",1);
				break;
		
			case IPDU_UPGRADE :
			case IPDU_WAITUPGRADEANS :
			case IPDU_WAITRESTART :
				//???????
				break;
		}
		ipdu->bufindex = 0;
		ipdu->start_time = time(NULL);
		ipdu->flgSendEnter = curtime;
	}

	// check the IPDU state --  Event - TIMEOUT 
	if (!events) {
		switch (ipdu->state) {
			case IPDU_LOGINFAILED :
				if (curtime - ipdu->start_time < 
					ipdu->timeAfterFailedLogin) {
					break;
				}
				if (ipdu->logoutAfterCommandComplete) {
					ipdu->state = IPDU_INIT;
					break;
				}
				if (IpduSendBin(ipdu,enter,1)) {
					if (ipdu->needAuth) {
						ipdu->state = IPDU_WAITUSERNAMESTR;
					} else {
						ipdu->state = IPDU_WAITPROMPTSTR;
					}
					ipdu->start_time = curtime;
				}
				break;
				
			case IPDU_WAITUSERNAMESTR :
			case IPDU_WAITPASSWDSTR :
				IpduTimeoutCheck(ipdu,PASSWDTO,1,IPDU_INIT);
				break;

			case IPDU_INITCOMM :
			case IPDU_WAITPROMPTSTR :
				IpduTimeoutCheck(ipdu,PROMPTTO,1,IPDU_INIT);
				break;
				
			case IPDU_WAITUSERANS :
				if (IpduTimeoutCheck(ipdu,360,1,IPDU_INIT)) {
					SendToUser(pmindex,"[Error] - Timeout waiting the new password",1);
					if (ipdu->admPasswd) {
						if (IpduSendStr(ipdu, ipdu->admPasswd)) {
							ipdu->state = IPDU_READY;
						}
					} else {
						if (IpduSendBin(ipdu,enter,1)) {
							ipdu->flgSendEnter = curtime;
							ipdu->state = IPDU_READY;
						}
					}
				}
				break;
				
			case IPDU_READY :
				if ((ipdu->logoutAfterCommandComplete || 
					ipdu->gotNumOutlets == 0) && 
					curtime - ipdu->start_time < 2) {
					if (IpduSendStr (ipdu, ipdu->logout)) {
						int i;
						for (i = 0; i < ipdu->entersAfterLogout; i ++) {
							usleep(200000);
							IpduSendBin(ipdu, enter,1);
						}
						usleep(200000);
						if (ipdu->gotNumOutlets) {
							ipdu->state = IPDU_INIT;
							break;
						}
						if (IpduSendBin(ipdu,enter,1)) {
							if (ipdu->needAuth) {
								ipdu->state = IPDU_WAITUSERNAMESTR;
							} else {
								ipdu->state = IPDU_WAITPROMPTSTR;
							}
							ipdu->start_time = curtime;
						}
					}
					break;
				}
				if ((curtime - ipdu->flgSendEnter) >= maxflgsendenter) {
					//dolog(LOG_DEBUG,"IDLE - Send enter");
#ifdef ONS
					dodebug("%s[%d]: IpduHandlePort: Send enter (curtime=%d, max=%d)\n",
					ipdu->auxflg ? "Aux" : "Serial",
					ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
					curtime, maxflgsendenter);
#else
					dodebug("Serial[%d]: IpudHandlePort: Send enter (curtime=%d, max=%d)\n",ipdu->serialPort,curtime, maxflgsendenter);
#endif
					if (IpduSendBin(ipdu,enter,1)) {
						ipdu->flgSendEnter = curtime;
					}
					break;
				}
				//[RK]Jun/13/06 - check DCD status
				if (!ipdu->dcdDetectOff) {
					IpduSendBin(ipdu, NULL, 0);
				}
				break;

			case IPDU_WAITECHO :
				if (IpduTimeoutCheck(ipdu, ECHOTO,1,IPDU_INIT)) {
					/* Timeout was reached. Send error */
					SendToUser(pmindex, "[Error] - Timeout waiting "
						"for a response (echo) from the IPDU.\r\n", 1);
					ipdu->state = IPDU_READY;
					IpduTreatNextCommand (ipdu);
				}
				break;
				
			case IPDU_WAITSYSLOG :
			case IPDU_WAITANSWER :
				if (IpduTimeoutCheck(ipdu, COMTO,1,IPDU_INIT)) {
					/* Timeout was reached. Send error */
					SendToUser(pmindex, "[Error] - Timeout waiting "
						"for a response from the IPDU.\r\n", 1);
					ipdu->state = IPDU_READY;
					IpduTreatNextCommand (ipdu);
				}
				// the PM sent part of the answer
                                if (ipdu->bufindex != 0) {
                                        ipdu->start_time = time(NULL);
                                }

				break;

			case IPDU_UPGRADE :
				break;
				
			case IPDU_WAITUPGRADEANS :
				if (IpduTimeoutCheck (ipdu, 600, 4, IPDU_UPGRADE)) {
					//dolog(LOG_ERR,"Upgrade timeout");
#ifdef ONS
					dodebug("%s[%d]: IpduHandlePort : Upgrade timeout\n",
					ipdu->auxflg ? "Aux" : "Serial",
					ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort);
#else
					dodebug("Serial[%d]: IpduHandlePort : Upgrade timeout\n",ipdu->serialPort);
#endif
					if (SendUpgAnswer(pmindex,PMC_UPG_CAN)) 
						ipdu->state = IPDU_UPGRADE;
					else
						ipdu->state = IPDU_READY;
				}
				break;
		}
	}
		
	if (ipdu->state != old_state) {
		//dolog(LOG_DEBUG, "serial.c:handle_port() - IPDU[%d] - State changed from " "%d to %d", ipdu->serialPort, old_state, ipdu->state);
#ifdef ONS
		dodebug("%s[%d]: IpduHandlePort - State changed from %d to %d\n", 
		ipdu->auxflg ? "Aux" : "Serial",
		ipdu->auxflg ? (ipdu->serialPort-AUX_OFFSET) : ipdu->serialPort,
		old_state, ipdu->state);
#else
		dodebug("Serial[%d]: IpduHandlePort - State changed from %d to %d\n", 
			ipdu->serialPort, old_state, ipdu->state);
#endif
	}
}

/*****************************************************************************
 * This routine closes the serial port. 
 ****************************************************************************/
void serialFinish(ipduInfo * ipdu) 
{
	int dtr;
	
	int i;
#ifdef ONS
	dodebug("%s[%d]: serialFinish\n",
		ipdu->auxflg ? "Aux" : "Serial", ipdu->serialPort);
#else
	dodebug("Serial[%d]: serialFinish\n",ipdu->serialPort);
#endif
	if (! ipdu->logoutAfterCommandComplete) {
		IpduSendBin(ipdu, ipdu->logout,strlen(ipdu->logout));
		IpduSendBin(ipdu, enter,1);
		for (i = 0; i < ipdu->entersAfterLogout; i ++) {
			usleep(200000);
			IpduSendBin(ipdu, enter,1);
		}
		sleep(1);
		tcflush(ipdu->fd, TCIOFLUSH);
	}
	ioctl(ipdu->fd,TIOCMGET,&dtr);
	dtr &= ~TIOCM_DTR;
	ioctl(ipdu->fd,TIOCMSET,&dtr);

	serial_close(ipdu->fd,&old_tio);
	shm_free_str(cy_shm->tty_user[ipdu->serialPort-1]);
	cy_shm->tty_user[ipdu->serialPort-1] = NULL;
	cy_shm->vet_pid[ipdu->serialPort-1] = 0;

}	

/*****************************************************************************
 * This routine opens the IPDU serial port, logs in to the IPDU system,
 * using pmadmin and pmpasswd configured in the pslave.conf, and wait for 
 * a prompt string. 
 ****************************************************************************/
int serialStart(ipduInfo * ipdu) 
{
	int port = ipdu->serialPort;
	struct line_cfg * ipduPort = &cy_shm->line_conf[port-1];
	int ipduFd;
	int dtr;
#ifdef UPLOAD_COMMAND
	struct termios tio;
#endif
	
	// Check if the ipduPort structure is valid
	if (ipduPort == NULL) {
		//dolog(LOG_ERR, "serial.c:serialStart() - Could not get serial " "configuration for port %d", port);
#ifdef ONS
		dodebug("%s[%d]: serialStart - Could not get serial configuration for port\n", ipdu->auxflg ? "Aux" : "Serial", ipdu->auxflg ? (port-AUX_OFFSET) : port);
#else
		dodebug("Serial[%d]: serialStart - Could not get serial configuration for port\n", port);
#endif
		return -1;
	}

	// Opening the serial port
	ipduFd = serial_open(ipduPort, 1, &old_tio);
	if ( ipduFd == -1 ) {
		//dolog (LOG_ERR, "serial.c:serialStart() - Could not open serial port ""%d", port);
		dodebug("Serial[%d]: serialStart - Could not open serial port\n", port);
		return -1;
	}
	ipdu->fd = ipduFd;
	ioctl(ipdu->fd,TIOCMGET,&dtr);
	dtr &= ~TIOCM_DTR;
	ioctl(ipdu->fd,TIOCMSET,&dtr);
	tcflush(ipdu->fd, TCIOFLUSH);
	sleep(1);
	ioctl(ipdu->fd,TIOCMGET,&dtr);
	dtr |= TIOCM_DTR;
	ioctl(ipdu->fd,TIOCMSET,&dtr);
	
#ifdef UPLOAD_COMMAND
	tcgetattr(ipdu->fd, &tio);
	ipdu->speed = cfgetispeed(&tio);
#endif

	dodebug("Serial[%d]: serialStart - serial port OK\n", port);
	// Authentication
	if (ipdu->numberOutlets) { 
		ipdu->gotNumOutlets = 0;
	} else {
		ipdu->gotNumOutlets = 1;
	}
	ipdu->state = IPDU_INIT;
	if (ipdu->logoutAfterCommandComplete && ipdu->gotNumOutlets) {
	} else if (IpduSendStr (ipdu, NULL)) {
		if (ipdu->needAuth) {
			ipdu->state = IPDU_WAITUSERNAMESTR;
		} else {
			ipdu->state = IPDU_WAITPROMPTSTR;
		}
	}
	return ipduFd;
}


/*****************************************************************************
 * This routine traces the pmd communication channels
 ****************************************************************************/
void debugprintf(const char *fmt, ...)
{
	char buf[1024];
	va_list ap;

	if (debugFd < 0) {
		if ((debugFd = open("/dev/console",O_WRONLY,0)) <= 0){
			return;
		}
	}

	va_start(ap,fmt);
	vsnprintf(buf,sizeof(buf),fmt,ap);
	write(debugFd,buf,strlen(buf));
	va_end(ap);
}

void traceprintf(unsigned int type, unsigned int port, unsigned char *s1, 
	int size)
{
	unsigned char ch;
	int i, j;

	
	switch (type) {
		case TRACE_IPDU_TO :
			debugprintf("DATA TO IPDU[%d]\n", port);
			break;
		case TRACE_IPDU_FROM :
			debugprintf("DATA FROM IPDU[%d]\n",port);
			break;
		case TRACE_PMC_TO :
			debugprintf("DATA TO CLIENT[%d]\n",port);
			break;
		case TRACE_PMC_FROM :
			debugprintf("DATA FROM CLIENT[%d]\n",port);
	}
	
	i = 0;
	while (1) {
		if (i * 16 >= size) {
			break;
		}
		for (j = 0; j < 16; j ++) {
			if (i * 16 + j >= size) {
				debugprintf("   ");
			} else {
				debugprintf("%02x ", s1[i * 16 + j]);
			}
		}
		debugprintf("   ");
		for (j = 0; j < 16; j ++) {
			if (i * 16 + j >= size) {
				break;
			}
			ch = s1[i * 16 + j];
			if (ch >= 0x20 && ch <= 0x7e) {
				debugprintf("%c", ch);
			} else {
				debugprintf(".");
			}
		}
		debugprintf("\n");
		i ++;
	}
}

