/*
 * libpsr.c	LibPortSlaveRadius.
 *		Shared library to be linked with pppd-2.2.0f
 *		Takes care of the portslave side of things.
 *
 * Version	@(#)libpsr.c 1.03  09-Jan-1998  miquels@cistron.nl
 *
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <pppd.h>
#include <fsm.h>
#include <lcp.h>
#include <ipcp.h>

#include <../pslave_cfg.h>
#include <server.h>

#ifdef CYCLADES
#define CBCP_SERVER_SUPPORT
#include <cbcp.h>
#undef CBCP_SERVER_SUPPORT
#endif

//[RK]May/12/06 - event configuration
extern void writeevt(int, char* , ...);

struct auth thisauth;
struct auth *pai = &thisauth;

#ifndef CYCLADES
static int rad_logged_in;
int setipaddr(char *, char **, int);
#else
int socket_tty_conv(void)
{
	return(3);
}
#endif

int ul_login(char *, char *, char **, struct wordlist **, struct wordlist **);
void ul_logout (void);
void ul_ppp_ipup (void);
void ul_ppp_ipdown (void);
void rad_make_wordlist (u_int32_t, struct wordlist **);

/* [RK]Mar/06/03
 * routine moved from auth.c
 */
/*
 * Invoke 'route' command to update "Framed-Route".
 */
static int update_framed_route(struct auth *ai, int islogin)
{
	char cmd[1024], buf[sizeof(ai->framed_route[0])];
	char *net, *gw, *metric;
	int x;

	if (ai->frn == 0) return 0;
	
	if (islogin) {
		nsyslog(LOG_INFO, "Adding routes: %d.", ai->frn);
		x = 0;
	} else {
		nsyslog(LOG_INFO, "Deleting routes: %d.", ai->frn);	
		x = ai->frn - 1;
	}
		
	while (x < ai->frn && x >= 0) {	
		memcpy(buf, ai->framed_route[x], sizeof(buf));
		net = strtok (buf, " ");
		gw = strtok ((char *) 0, " ");
		metric = strtok ((char *) 0, " ");
							
		snprintf(cmd, sizeof(cmd) - 1, "exec %s %s -net %s gw %s metric %s >/dev/null 2>&1",
			PATH_ROUTE, islogin ? "add" : "del", net, gw, metric);
		system(cmd);
		
		islogin ? x++ : x--;		//FIFO add, LIFO del
	} 

	return 0;
}

int plugin_init (void) {

	nsyslog (LOG_DEBUG, "ul_init () called");
	pap_auth_hook = ul_login;
	pap_logout_hook = ul_logout;
	ip_up_hook = ul_ppp_ipup;
	ip_down_hook = ul_ppp_ipdown;
#ifndef CYCLADES
	rad_logged_in = 0;
#endif
	return 0;
}

int ul_login(char *user, char *passwd, char **msg, struct wordlist **addrs,
    struct wordlist **opts) {
	char tmp[255];
	char *s;

	if (rad_init (-2, &thisauth, devnam, 0) < 0)
	{
		exit(1);
	}

	nsyslog(LOG_DEBUG, "ul_login(%c) called - port=%d", thisauth.proto,thisauth.nasport);

	if ((s = getenv ("CONNECT_INFO")) != NULL) {
		strcpy (thisauth.conn_info, s);
	}

	snprintf (thisauth.login, sizeof (thisauth.login), "%s", user);
	snprintf (thisauth.passwd, sizeof (thisauth.passwd), "%s", passwd);
#ifdef CYCLADES
	if (do_local_or_server_authentication(&thisauth, 1)) {
		return (0);
	}
	update_utmp("%L",lineconf[thisauth.nasport].utmp_from, 
		&thisauth, lineconf[thisauth.nasport].syswtmp); //[RK]Dec/26/02 Bug#129 - utmp
	//[RK]Jun/12/096
	writeevt(5,"ssii",thisauth.login, lineconf[thisauth.nasport].serverfarm, (thisauth.nasport + 1), 5);
	if (thisauth.next_token) {
		return(0);
	}
#else
	if (rad_client (&thisauth, 1) < 0) {
		return (0);
	}
#endif
//[RK]Jun/12/06 - port from tslinux_mv21
//	if (thisauth.proto != P_PPP && thisauth.proto == P_PPP_ONLY) {
//		return (0);
//	}

#ifdef CYCLADES
	/* CallBack Server */
	if (lcp_hisoptions[0].neg_cbcp) {	/* callback negociated in LCP */
		cbcp_protent.enabled_flag = 1;
		cbcp[0].us_cbcp = 1;	/* callback server */
		if (thisauth.sess_type) { /* callback session - set by Radius Server */
			if (thisauth.callback_number[0]) {
				/* callback number is defined by administrator */
				cbcp[0].us_type = (1 << CB_CONF_ADMIN);
				cbcp[0].us_number = thisauth.callback_number;
			} else {
				cbcp[0].us_type = (1 << CB_CONF_USER);
			}
		} else {
			/* reset callback, it is a normal session to Radius Server */
			cbcp[0].us_type = (1 << CB_CONF_NO);
		}
	}
#endif
	
	if (thisauth.netmask && thisauth.netmask != 0xFFFFFFFF) {
		netmask = thisauth.netmask;
	}
	if (thisauth.mtu) {
		lcp_allowoptions[0].mru = thisauth.mtu;
	}
	if (thisauth.mru) {
		lcp_wantoptions[0].mru = thisauth.mru;
		lcp_wantoptions[0].neg_mru = 1;
	}
	strcpy (tmp, dotted (thisauth.localip));
	strcat (tmp, ":");
	if (thisauth.address != 0xFFFFFFFF) {
		strcat(tmp, dotted (thisauth.address));
		rad_make_wordlist (thisauth.address, addrs);
	} else {
		*addrs = NULL;
	}
	if (setipaddr(tmp,NULL,1) < 0) {
		nsyslog (LOG_ERR, "bad IP address %s", tmp);
		return (0);
	}
	*opts = NULL;
	if (thisauth.idletime > 0) {
		idle_time_limit = thisauth.idletime;
	}
	if (thisauth.sessiontime > 0) {
		maxconnect = thisauth.sessiontime;
	}
	setenv ("LOGNAME", user, 1);
#ifndef CYCLADES
	rad_acct (&thisauth, 1);
	rad_logged_in = 1;
//#else
//	if (thisauth.do_acct) {
//		rad_acct (&thisauth, 1);
//	}
#endif
	nsyslog (LOG_NOTICE, "user %s logged in", user);
	return (1);
}

void ul_logout (void) {
	nsyslog (LOG_DEBUG, "ul_logout () called");
#ifdef CYCLADES
	do_pam_close();
	nsyslog (LOG_NOTICE, "user %s logged out", thisauth.login);
	clear_utmp_entry(getpid());	//[RK]Dec/26/02 Bug#129 - utmp 
	//[RK]Jun/12/096
	writeevt(6,"ssii",thisauth.login, lineconf[thisauth.nasport].serverfarm, (thisauth.nasport + 1), 5);
#else
	if (rad_logged_in) {
		rad_acct(&thisauth, 0);
		rad_logged_in = 0;
		nsyslog (LOG_NOTICE, "user %s logged out", thisauth.login);
	}
#endif
}

void ul_ppp_ipup (void) {
	nsyslog (LOG_DEBUG, "ul_ipup () called");
	update_framed_route(&thisauth, 1); 
}

void ul_ppp_ipdown (void) {
	struct pppd_stats io_stats;
	
	nsyslog (LOG_DEBUG, "ul_ppp_ifdown () called");
	get_ppp_stats (ifunit, &io_stats);
	if (io_stats.bytes_out) {
		thisauth.sent_bytes = io_stats.bytes_out;
	}
	if (io_stats.bytes_in) {
		thisauth.recv_bytes = io_stats.bytes_in;
	}
}

void rad_make_wordlist (u_int32_t addr, struct wordlist **addrs) {
	char *ipaddr;
	struct wordlist *ap;

	ipaddr = ip_ntoa (addr);
	ap = (struct wordlist *) malloc(sizeof(struct wordlist) + strlen(ipaddr) + 1);
	if (ap == NULL) {
		nsyslog (LOG_DEBUG, "rad_make_wordlist: NOMEM: authorized addresses");
	}
	ap->word = (char *) (ap + 1);   
	ap->next = NULL;
	strcpy(ap->word, ipaddr);
	*addrs = ap;
}
