/*
 *  Module:  2way-raw.h (raw data socket client/server application header file)
 *
 *  Author:  Marcelo Peccin <marcelo.peccin@cyclades.com>
 *
 *  Copyright:  (c) 2004 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  September 16, 2004
 *
 *  Comments:  The first release of this module is intended to
 *             Telia's project. The project involves a main Application
 *             that pools and receive asynchronous alarms from
 *             a number of remote serial devices.
 *             This software is based on modem-replace.c.
 */

#ifndef RAW_2_WAY_H
#define RAW_2_WAY_H


/**************************************************/
/**********     Constant Definitions     **********/
/**************************************************/
#undef  FALSE
#define FALSE      0
#undef  TRUE
#define TRUE    (!FALSE)
#undef  NULL
#define NULL   ((void*)0)

#define ERROR     (-1)
#define ERRCLOSE  (-2)  
#define OK          0
#define PROBLEM     1

#define PARITY_NONE  1
#define PARITY_ODD   2
#define PARITY_EVEN  3

#define FLOW_NONE  0
#define FLOW_HARD  1
#define FLOW_SOFT  2

#define RAW2W_NO_EVENT     0
#define ALARM_RECEIVED     1
#define SOCK_CONN_ACCEPT   2

#define NO_SOCK        (-1)

#define SBUFFSIZE      1024
#define TCPBUFFSIZE    1024
#define SBUFFLIMIT    (SBUFFSIZE - 24)


/**********************************************/
/**********     Type Definitions     **********/
/**********************************************/
typedef unsigned char  uchar;

typedef struct
{
	int    port;
	int    speed;
	int    datasize;
	int    parity;
	int    stopbits;
	int    flowctrl;
	int    media;
	uchar  dev[15];
}
sconf;

typedef struct
{
	sconf  serial;
	int    localip;
	int    localport;
	int    remoteip;
	int    remoteport;	
	int    maxidle;
	int    tx_interval;
}
raw2wconf;


/*************************************************/
/**********     Function Prototypes     **********/
/*************************************************/
static void  mpfprint(const char *fmt, ...);
static int   config_port(int fd, sconf *pconf);
static int   ssend(int fds, uchar *buff, int tosend);
static int   tcpsend();


/***********************************************/
/**********     Macro Definitions     **********/
/***********************************************/
#undef Max
#define Max(x,y)  (((x) >= (y)) ? (x) : (y))


#endif /* RAW_2_WAY_H */

/* End of 2way-raw.h */
