/*
 *  Module:  billing.c (Billing feature main file)
 *
 *  Author:  Marcelo Peccin <mpeccin@cyclades.com>
 *
 *  Copyright:  (c) 2002 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  January 16, 2003
 *
 *  Comments:  
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/vfs.h>
#include <sys/ioctl.h>
#include <string.h>
#include "server.h"
#include "billing.h"
#include "rot_nports.h"
#include <asm/prxk.h>

// Control variables
static int    billing_debug = FALSE;
static int    fd  = -1;
static int    ffd = -1;
static uchar  filen[FILENLEN];
static uchar  ffilen[FILENLEN];
time_t        lasttime;

static struct MachineType  mt;
static int    serial_port;
static char*  billing_path;


/*****************************/
int main(int argc, char **argv)   // Billing feature entry point
/*****************************/
{

	billingconf       config;
  	int               fdserial, fdready, exitstatus = PROBLEM, retries;
	uchar             sdevbase[] = "/dev/ttyS";
	uchar             collector_name_data[25];
	fd_set            serial_in, testfd;
	uchar             default_eor[] = "\n";
	uchar             default_path[] = "/var/run/DB";
	uchar             bbuff[BBUFFSIZE];
	uchar             bfbuff[BBUFFSIZE];
	uchar             syscmd[120];
	FILE             *currfile = NULL;
	int               endrn, rec, aux;
	int               records = 0, waitdata = 0, pending = FALSE, chrs = 0;
	struct stat       statinfo;
	struct statfs     fsinfo;
  	struct timeval    timeout;

	billing_path = default_path; //to start with...


	// get machine type
	get_machine_type(&mt);


	// take command line arguments
	if ((argc < 2) || ((config.serial.port = atoi(argv[1])) < 1) || (config.serial.port > mt.nports)) {
		mpfprint("Billing must be called with a port (1..%d) as argument.\n", mt.nports);
		goto end_billing;		
	}
	serial_port = config.serial.port; //serial_port has a bigger scope
	if ((mt.board_type == BOARD_TS1H) || (mt.board_type == BOARD_TS110)) {
		// TS100/110 has its own zImage...
		config.serial.port = 0; //the shared console and serial port
		sprintf(config.serial.dev, "%s%d", sdevbase, config.serial.port);
	} else {
		sprintf(config.serial.dev, "%s%d", sdevbase, config.serial.port);
		config.serial.port--;
	}


    // initialize Shared Memory
	if (shm_init(-1) < 0) {
		mpfprint("problems initializing shared memomy\n");
		goto end_billing;
	}
	while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) == 0) {
		sleep(1);
	}


	// copy Configuration Information
	config.serial.speed    = cy_shm->line_conf[config.serial.port].speed; 
	config.serial.datasize = cy_shm->line_conf[config.serial.port].DataSize;
	config.serial.parity   = cy_shm->line_conf[config.serial.port].Parity;
	config.serial.stopbits = cy_shm->line_conf[config.serial.port].StopBits;
	config.serial.flowctrl = cy_shm->line_conf[config.serial.port].flow;
	config.serial.media	   = cy_shm->line_conf[config.serial.port].media;
	config.maxrecords = cy_shm->line_conf[config.serial.port].billing_records;
	config.timeout    = cy_shm->line_conf[config.serial.port].billing_timeout * 60;
	config.record_end = cy_shm->line_conf[config.serial.port].billing_eor;
	config.path       = cy_shm->line_conf[config.serial.port].billing_dir;
	config.collector_name = cy_shm->line_conf[config.serial.port].serverfarm;
	// get number of chars in termitation sequence
	if (!config.record_end || !(*config.record_end)) {
		config.record_end = default_eor;
		mpfprint("invalid termination sequence: using \"\\n\"\n");
	}
	endrn = strlen(config.record_end);
	// check application directory
	if (!config.path) {
		config.path = default_path;
		mpfprint("invalid directory: using \"/var/run/DB\"\n");
	} else if (stat(config.path, &statinfo) == ERROR) {
		mpfprint("cannot stat (%s): using \"/var/run/DB\"\n", config.path);
		config.path = default_path;
	} else if (!S_ISDIR(statinfo.st_mode)) {
		mpfprint("invalid directory (%s): using \"/var/run/DB\"\n", config.path);
		config.path = default_path;
	}
	billing_path = config.path; //billing_path has a bigger scope
	// check collector name
	if (!config.collector_name) {
		sprintf(collector_name_data, "ttyS%d", serial_port);
		config.collector_name = collector_name_data;
	}


	// can we run?
	if (!config.maxrecords) {
		mpfprint("Billing STOPPED\n");
		while(1) sleep(60); //avoid cy_ras keeps restarting the application...
	}


	if (billing_debug)
		mpfprint("Billing settings for %s...\n\
config.serial.speed    = %d\n\
config.serial.datasize = %d\n\
config.serial.parity   = %d (1,odd,even)\n\
config.serial.stopbits = %d\n\
config.serial.flowctrl = %d (0,Hw,Sw)\n\
config.serial.media    = %d (232=0)\n\
config.maxrecords      = %d\n\
config.timeout         = %d(seg) = %d(min)\n\
config.path            = %s\n",
				 config.collector_name,
				 config.serial.speed,
				 config.serial.datasize,
				 config.serial.parity,
				 config.serial.stopbits,
				 config.serial.flowctrl,
                 config.serial.media,
				 config.maxrecords,
				 config.timeout,
				 config.timeout/60,
				 config.path);


    // open and configure Serial Port
	if ((fdserial = open(config.serial.dev, O_RDWR | O_NOCTTY | O_NONBLOCK)) == ERROR) {
		mpfprint("error opening serial port: %s (%d)\n", strerror(errno), errno);
		goto end_billing;
	}	
	if (config_port(fdserial, &config.serial) == ERROR) {
		mpfprint("problems configuring serial port\n");
		goto close_serial;
	}
	if (billing_debug)
		mpfprint("...serial port is ready.\n");


	FD_ZERO(&serial_in);
	FD_SET(fdserial, &serial_in);


	// Main Loop
	while (1) {
			
  		testfd = serial_in;

		// check available space and wait, if needed
		if (statfs(config.path, &fsinfo)) {
			mpfprint("error verifying RAMDISK space: %s (%d)\n", strerror(errno), errno);
			break;
		}
		if (fsinfo.f_bfree < MINRDISK) {
			if (billing_debug)
				mpfprint("file system full (free space < %d 1k-blocks), RTS is being activeted\n",
						 MINRDISK);
			if (!config.serial.media) { //interface is RS232
				aux = TIOCM_DTR;
				ioctl(fdserial, TIOCMSET, &aux);
			}

			while (fsinfo.f_bfree < RDISKOK) { //some hysteresis is good
				sleep(10);
				if (statfs("/", &fsinfo)) {
					mpfprint("error verifying RAMDISK space: %s (%d)\n", strerror(errno), errno);
					goto close_serial;
				}
			}

			if (billing_debug)
				mpfprint("file system has at least %d 1k-blocks free, RTS is being deactivated\n",
						 fsinfo.f_bfree);
		}
		if (!config.serial.media) { //interface is RS232
			aux = TIOCM_DTR | TIOCM_RTS;
			ioctl(fdserial, TIOCMSET, &aux);
		}

		// check timeout only in case a temporary file is opened and "timeout" is in use
		if (currfile && config.timeout) {
			if ((timeout.tv_sec = (config.timeout - (time(NULL) - lasttime))) <= 0)
				goto timeout_entry;
		} else {
			timeout.tv_sec = 0;
		}

		// listen to the serial port, considering timeout
  		if (timeout.tv_sec) {
			timeout.tv_usec = 0;
  			if ((fdready = select(fdserial+1, &testfd, NULL, NULL, &timeout)) == ERROR) {
				mpfprint("error in select: %s (%d)\n", strerror(errno), errno);
				break;
			}
  		} else {
			if ((fdready = select(fdserial+1, &testfd, NULL, NULL, NULL)) == ERROR) {
				mpfprint("error in select: %s (%d)\n", strerror(errno), errno);
				break;
			}
		}

		if (!fdready) {
		//***********************
		//*****   Timeout   *****
		//***********************
			if (currfile) {
timeout_entry:
				if (billing_debug)
					mpfprint("closing billing file: timeout (%d min)\n", config.timeout/60);
				if (fclose(currfile)) {
					mpfprint("error closing billing file: %s (%d)\n", strerror(errno), errno);
					break;
				}
				sprintf(syscmd, "mv %s %s", filen, ffilen);
				if (system(syscmd)) { //rename file from .TMP to .TXT
					mpfprint("problems renaming billing file\n");
					break;
				}
				currfile = NULL;
			}
			continue;
		}
		
		//***************************************
		//*****   Activity on serial port   *****
		//***************************************
		if (billing_debug)
			mpfprint("activity on serial port\n");

		retries = 0;
		// reading serial data
		do {

			if ((rec = read(fdserial, bbuff, BBUFFSIZE)) == ERROR) {
				if (retries >= 50) {
					mpfprint("error readind serial data: %s (%d)\n", strerror(errno), errno);
					goto close_serial;
				}
				usleep(100);
				retries++;
				continue;
				
			} else if (!rec) {
				break;
				
			} else { //data successfully received
				int i = 0, offset = 0;

				aux = rec;
				while (aux-- && ((waitdata+i) < BBUFFSIZE)) {

					pending = TRUE;

					bfbuff[waitdata+i] = bbuff[offset+i]; //copy char

					i++; chrs++;

					if (((waitdata + i - endrn) >= 0) &&
						(!strncmp((bfbuff+waitdata+i-endrn), config.record_end, endrn))) {

						if (chrs > endrn) //one more complete and non-zero-lenth record
							records++;

						// in case it's the first time or the file has been closed...
						if (!currfile) {
							setbfname(config.collector_name);
							if (!(currfile = fopen(filen,"w+"))) {
								mpfprint("problems opening %s file\n", filen);
                                goto close_serial;
							}
						}

						// write to temporary file
						if (fwrite(bfbuff, sizeof(uchar), (waitdata+i), currfile) != (waitdata+i)) {
							mpfprint("error writing billing file (%d)\n");
							goto close_serial;
						}
						if (fflush(currfile)) {
							mpfprint("error flushing billing file (%d)\n");
							goto close_serial;
						}

						if (records == config.maxrecords) {
							//*****************************************************
							//*****   Maximun number of records was reached   *****
							//*****************************************************
							if (billing_debug)
								mpfprint("closing billing file: %d records\n", records);
							if (fclose(currfile)) {
								mpfprint("error closing billing file: %s (%d)\n", strerror(errno), errno);
								goto close_serial;
							}
							sprintf(syscmd, "mv %s %s", filen, ffilen);
							if (system(syscmd)) { //rename file from .TMP to .TXT
								mpfprint("problems renaming billing file\n");
							}
							currfile = NULL;
							records = 0;
						}

						offset += i;
						pending = FALSE;
						waitdata = 0;
						i = 0, chrs = 0;

					} //if record complete


				} //while data

				if (pending) {
					if ((waitdata += i) >= BBUFFSIZE) {
						mpfprint("billing record exceeds %d bytes\n", BBUFFSIZE);
						goto close_serial;
					}
				}

			} //if read()

		} while (rec == BBUFFSIZE); //do read()

  	} //main loop



close_serial:
	if (close(fdserial) == ERROR) exitstatus = ERRSCLOSE;

end_billing:
	if (billing_debug) mpfprint("..................................... exit status: %d\n", exitstatus);


	exit(exitstatus);

	
} /* main() */



/********************************************/
/**********     Print to ttyS0     **********/
/********************************************/
void mpprint(const char *fmt, ...)
{
	va_list ap;
	char buf[1024];

	if (fd < 0) {
		if ((fd = open("/dev/ttyS0", O_WRONLY, 0)) <= 0) {
			return;
		}
	}

	va_start(ap, fmt);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	write(fd, buf, strlen(buf));
	va_end(ap);
}
/*******************************************************************/


/*********************************************/
/**********   Print to a log file   **********/
/*********************************************/
void mpfprint(const char *fmt, ...)
{
	va_list ap;
	char buf[1024];
	char filen[64];

	if (serial_port)
		sprintf(filen, "%s/billing-%02d.log", billing_path, serial_port);
	else
		sprintf(filen, "%s/billing.log", billing_path);

	if (ffd < 0) {
		if ((ffd = open(filen, O_WRONLY | O_CREAT | O_TRUNC, 0)) <= 0) {
			mpprint("error opening mpfprint default file: %s !!!\r\n", strerror(errno));
			return;
		}
	}

	va_start(ap, fmt);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	write(ffd, buf, strlen(buf));
	va_end(ap);
}
/**********************************************/


/*****************************************************************************
 * This routine configures the Serial Port
 ****************************************************************************/
int config_port(int fd, sconf *pconf)
{ 
	struct termios term;
	int aux;

	
	// Getting the port configuration
	if (tcgetattr(fd, &term) == ERROR) {
		return ERROR;
	}

	// Setting media type
	aux = pconf->media | CYC_SET_MEDIA; //[RK]Jul/29/03
	if (ioctl(fd, TIOCSERCYC, &aux) == ERROR) {
		return ERROR;
	}

	// Setting raw mode
	cfmakeraw(&term);

	// Setting speed
	switch (pconf->speed) {
	case 50:
		aux = B50;
		break;
	case 75:
		aux = B75;
		break;
	case 110:
		aux = B110;
		break;
	case 134:
		aux = B134;
		break;
	case 150:
		aux = B150;
		break;
	case 200:
		aux = B200;
		break;
	case 300:
		aux = B300;
		break;
	case 600:
		aux = B600;
		break;
	case 1200:
		aux = B1200;
		break;
	case 1800:
		aux = B1800;
		break;
	case 2400:
		aux = B2400;
		break;
	case 4800:
		aux = B4800;
		break;
	case 14400:
		aux = B14400;
		break;
	case 19200:
		aux = B19200;
		break;
	case 28800:
		aux = B28800;
		break;
	case 38400:
		aux = B38400;
		break;
	case 57600:
		aux = B57600;
		break;
	case 76800:
		aux = B76800;
		break;
	case 115200:
		aux = B115200;
		break;
	case 230400:
		aux = B230400;
		break;
	case 460800:
		aux = B460800;
		break;
	case 921600:
		aux = B921600;
		break;
	default:
		aux = B9600;
	}
	cfsetospeed(&term, aux);
	cfsetispeed(&term, aux);

	// Setting data size
	term.c_cflag &= ~CSIZE;
	switch (pconf->datasize) {
	case 5:
		term.c_cflag |= CS5;
		break;
	case 6:
		term.c_cflag |= CS6;
		break;
	case 7:
		term.c_cflag |= CS7;
		break;
	case 8:
	default:
		term.c_cflag |= CS8;
	}

	// Setting parity
	switch (pconf->parity) {
	case PARITY_ODD:
		term.c_cflag |= (PARENB | PARODD);
		break;
	case PARITY_EVEN:
		term.c_cflag |= (PARENB);
		term.c_cflag &= ~(PARODD);
		break;
	default:
		term.c_cflag &= ~(PARENB); //none
	}

	// Setting number of stop bits
	if (pconf->stopbits == 2) {
		term.c_cflag |= (CSTOPB);
	}
	else {
		term.c_cflag &= ~(CSTOPB);
	}

	// Setting flow control
	switch (pconf->flowctrl) {
	case FLOW_HARD:
		term.c_iflag &= ~(IXON | IXOFF);
		term.c_cflag |= (CRTSCTS);
		break;
	case FLOW_SOFT:
		term.c_iflag |= (IXON | IXOFF);
		term.c_cflag &= ~(CRTSCTS);
		break;
	default:
		term.c_iflag &= ~(IXON | IXOFF); //none
		term.c_cflag &= ~(CRTSCTS);
	}

	// not canonical
	term.c_lflag &= ~(ICANON);

	// Disabling modem signal
	term.c_cflag |= CLOCAL;

	// Enabling modem hang up
	term.c_cflag |= HUPCL;

	// flush the port
	tcflush(fd, TCIOFLUSH);

	// Setting the port configuration
	if (tcsetattr(fd, TCSANOW, &term) == ERROR) {
		return ERROR;
	}

	return OK;
}


/********************************************************************************
 * This routine prepares the next billing file name (filen & ffilen) according
 * to the following format:
 *                            cyXXXXX-YYMMDD.hhmmss.tmp (& .txt)
 * where:
 *        XXXXX  - is the serverfarm or "tty name" (collector name)
 *        YYMMDD - is the year/month/day
 *        hhmmss - is the hour:min:seg
 *
 *******************************************************************************/
void setbfname(uchar *collector)
{
	struct tm  *tstr;


	if (!collector) return;

	lasttime = time(NULL);
	tstr = localtime(&lasttime);

	sprintf(filen, "%s/cy%s-%02d%02d%02d.%02d%02d%02d.tmp", billing_path, collector,
			tstr->tm_year-100, tstr->tm_mon+1, tstr->tm_mday,
			tstr->tm_hour, tstr->tm_min, tstr->tm_sec);
	sprintf(ffilen, "%s/cy%s-%02d%02d%02d.%02d%02d%02d.txt", billing_path, collector,
			tstr->tm_year-100, tstr->tm_mon+1, tstr->tm_mday,
			tstr->tm_hour, tstr->tm_min, tstr->tm_sec);
	
}	


/* End of modbusd.c */
