#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <asm/cyc_config.h>
#include <asm/flash.h>
#include <asm/prxk.h>

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <linux/if_ether.h>

#include <setjmp.h>

#include "rot_nports.h" 
#include "bootconf.h"

#include "../pslave_cfg.h" 

#define CHAR_ESC			27
#define CHAR_CTRL_C			3
#define CHAR_CR				'\r'
#define CHAR_BS				'\b'

#define FLASH_BLOCK	(64*1024)

struct termios raw_term = {0}, std_term;

jmp_buf my_env;

static unsigned char hw_mac[6] = {0x00, 0x60, 0x2e, 0x00, 0x00, 0x01};

static int om_flag = 0, config_size, boot_ori_size;

OM_INFO				save_om_info = {{0}};
static struct type0000		save_type0000 = {{0}};
static struct HW_SETUP		save_hw_setup = {0};
static struct BOOT_MASK	save_boot_mask = {0};

OM_INFO				*om_info = NULL;
static struct type0000		*type0000;
static struct HW_SETUP		*hw_setup;
static struct BOOT_MASK	*boot_mask;
static char			buffer[FLASH_BLOCK];
static char			om_buf[2*FLASH_BLOCK];
static char			aux[160], tmp[160];

// The first two bytes are the default
static char yes_no			[] = {'Y', 'Y', 'N', 'N', 0xff, 0xff};
static char save_op		[] = {'N', 'N', 'Y', 'Y', 'Q', 'Q', 0xff, 0xff};
static char wdt_op			[] = {'A', 1, 'I', 0, 0xff, 0xff};
static char boot_op		[] = {'F', 0, 'N', 1, 0xff, 0xff};
static char boot_type		[] = {'H', 3, 'B', 1, 'T', 2, 0xff, 0xff};
static char op_code_crc	[] = {'P', 1, 'S', 0, 0xff, 0xff};
static char mem_test		[] = {'F', 2, 'S', 0, 'Q', 1, 0xff, 0xff};
static char mii_op			[] = {'A', 0, '1', 1, 'T', 2, 'F', 3, 'H', 4, 0xff, 0xff};

static struct MachineType mt;

static int	 speeda[] = {4800, 9600, 19200, 38400, 57600, 115200};

static char *access_type(FLASH_CONFIG_VECTOR_HEADER *p, int type, int tries);

static int GetKeyPressed (int flag_jmp)
{
	int ch;
	fd_set rfds;

	FD_ZERO (&rfds);
	FD_SET (0, &rfds);
	select (1, &rfds, NULL, NULL, NULL);

	if ((ch = getchar()) == CHAR_CTRL_C) {
		exit(-1);	// ctrl-C
	}

	if (ch == CHAR_ESC && flag_jmp == 0) {
		longjmp(my_env, 1);
	}

	return(ch);
}

static char seek_val(int flag_seek, char val, char *op)
{
	char *p = op;

	while (*p != 0xff) {
		if (p[flag_seek] == val) {
			op = p;
			break;
		}
		p+=2;
	}
	return(flag_seek ? op[0] : op[1]); 
}

static int get_param(char *f, int n, int *p)
{
	char	*r = tmp;
	char	local[3];
	int	i, ret = 0, cnt = 0;

	fflush(stdout);

	while((*r = GetKeyPressed(0)) != CHAR_CR) {
		if (*r != CHAR_BS) {
			if (*r >= 32 && *r < 127) {
				printf ("%c", *r);
			} else {
				printf ("?");
			}
			fflush(stdout);
			cnt++;
			r++;
		} else {
			if (cnt) {
				printf ("\b \b");
				fflush(stdout);
				--cnt;
				--r;
			}
		}
	}

	if (cnt == 0) {
		return(0);
	}

	*r = 0;
	r = tmp;
	
	if (n == 6) { // MAC address
		if (r[2] == ':') { // using format NN:NN:NN:NN:NN:NN
			ret = sscanf(r, f, &p[0], &p[1], &p[2], &p[3], &p[4], &p[5]);
		}
		else { // using format NNNNNNNNNNNN
			for (i = 0; i < n; i++) {
				local[0] = r[i*2];
				local[1] = r[i*2+1];
				local[2] = 0;
				ret = ret + sscanf(local, "%x", &p[i]);
			}
		}
	}
	if (n == 4) { // IP address
		ret = sscanf(r, f, &p[0], &p[1], &p[2], &p[3]);
	}
	if (n == 1) { // string or speed
		ret = sscanf(r, f, &p[0]);
	}
	if (n == 2) {
		strcpy(&p[0], r);
		ret = n;
	}

	if (ret != n) {
		return(0);
	}

	return(ret);
}

static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
		int j;

		while (byte_n--) {
				crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
				for(j = 0; j < 8; j++) {
						if (crc & 0x8000)  {
								crc <<= 1;
								crc ^= 0x1021;
						} 
			else {
								crc <<= 1;
						}
		}
	}
	return(crc);
}

static void read_om_info_from_flash(char *buf)
{
	FLASH_CONFIG_VECTOR_HEADER	*v;
	int	n, fd;

	if ((fd = open("/proc/flash/boot_ori", O_RDONLY)) == -1) {
		printf("\r\nError %d openning flash (boot_ori) !!!\r\n", errno);
		goto set_default;
	}
	
	if (mt.board_type == BOARD_TS1H ||
		mt.board_type == BOARD_TS110 ||
		mt.board_type == BOARD_SM100 ||
		mt.board_type == BOARD_NL1K ||
		mt.board_type == BOARD_TS3K) {
		n = OM_ADDRESS1;
	} else {
		n = OM_ADDRESS2;
	}

	if ((boot_ori_size = read(fd, buf, 2*FLASH_BLOCK-4)) <= 0) {
		close(fd);
		printf("\r\nError %d reading flash (boot_ori) !!!\r\n", errno);
		goto set_default;
	}

	close(fd);

	om_info = (OM_INFO *)
			access_type((FLASH_CONFIG_VECTOR_HEADER *)&buf[n], TYPE9000, 1);

	if (om_info != NULL) {
		save_om_info = *om_info;
		return;
	}

set_default:

	// TYPE9000
	v = (FLASH_CONFIG_VECTOR_HEADER	*)&buf[n];
	om_info = (OM_INFO *)v->data;
	v->type = TYPE9000;
	v->size = sizeof(OM_INFO);
	memset(v->data, 0, v->size);
	memcpy(om_info->mac_addr, hw_mac, 6);
	v->crc = calc_crc(v->size, 0, v->data); 
}

static int read_configuration_from_flash(char *buf)
{
	int	n, fd;

	if ((fd = open("/proc/flash/config", O_RDONLY)) == -1) {
		printf("\r\nError %d reading flash (config) !!!\r\n", errno);
		return(0);
	}

	n = read(fd, buf, 64*1024);

	close(fd);
	return(n);
}

static char *access_type(FLASH_CONFIG_VECTOR_HEADER *p, int type, int tries)
{
	while (tries--) {
		if (p->type == 9999) {
			break;
		}
		if (p->type > 9999 || p->size == 0 || p->size > 10000) {
			break;
		}
		if (p->crc != calc_crc(p->size, 0, p->data)) {
			break;
		}
		if (p->type == type) {
			return(p->data);
		}
		p = (FLASH_CONFIG_VECTOR_HEADER *)((char *)p + p->size + 6);
	}
	return(NULL);
}

static void save_om_info_into_flash(char *buf)
{
	int fd, bytes;
	FLASH_CONFIG_VECTOR_HEADER *v;

	if (!memcmp(&save_om_info, om_info, sizeof(*om_info))) {
		return;
	}

	// recalculates CRC after the changes
	v = (FLASH_CONFIG_VECTOR_HEADER *)((char *)om_info - 6);
	v->crc = calc_crc(v->size, 0, v->data);

	if ((fd = open("/proc/flash/boot_ori", O_WRONLY)) == -1) {
		printf("\r\nError when trying to write boot_ori into flash!");
		return;
	}

	if ((bytes = write(fd, buf, boot_ori_size)) != boot_ori_size) {
		printf("\r\nWritten %d instead of %d into boot_ori", bytes, boot_ori_size);
	}

	close(fd);
}

static int save_config_into_flash(char *buffer, int n)
{
	int	fd;
	int	bytes;

	if (!memcmp(&save_type0000, type0000, sizeof(*type0000)) &&
		!memcmp(&save_boot_mask, boot_mask, sizeof(*boot_mask)) &&
		!memcmp(&save_hw_setup, hw_setup, sizeof(*hw_setup))) {
		return(0);
	}

	if ((fd = open("/proc/flash/config", O_WRONLY)) == -1) {
		printf("\r\nError when trying to write config into flash!");
		return(0);
	}

	if ((bytes = write(fd, buffer, n)) != n) {
		printf("\r\nWritten %d instead of %d into config", bytes, n);
	}

	close(fd);
	printf("\r\nConfiguration saved!");

	return(1);
}

static int set_default_config(void)
{
	FLASH_CONFIG_VECTOR_HEADER	*v;

	// TYPE0000
	type0000 = (struct type0000 *)&buffer[6];
	v = (FLASH_CONFIG_VECTOR_HEADER	*)buffer;
	v->type = TYPE0000;
	v->size = sizeof(struct type0000);
	memset(v->data, 0, v->size);
	strcpy(v->data, "CYCLADES");
	v->crc = calc_crc(v->size, 0, v->data); 

	// TYPE0068
	v = (FLASH_CONFIG_VECTOR_HEADER *)&buffer[6 + v->size];
	v->type = TYPE0068;
	v->size = sizeof(struct HW_SETUP);

	hw_setup = (struct HW_SETUP *)(v->data);
	memset(v->data, 0, v->size);

	memcpy(hw_setup->physical_addr, om_info->mac_addr, 6);
	hw_setup->itf_ip_addr[0] = 192;
	hw_setup->itf_ip_addr[1] = 168;
	hw_setup->itf_ip_addr[2] = 160;
	hw_setup->itf_ip_addr[3] = 10;

	hw_setup->enable_wdt = DEFAULT_WDT_OPT;
	hw_setup->console_speed = 1;
	hw_setup->op_code_crc = 0;
	hw_setup->mem_test = 0;
	hw_setup->boot_option = 0;
	strcpy(hw_setup->tftp_path, DEFAULT_TFTP_PATH);
	v->crc = calc_crc(v->size, 0, v->data); 

	// TYPE0015
	v = (FLASH_CONFIG_VECTOR_HEADER *)&buffer[12 + sizeof(struct type0000) + v->size];
	v->type = TYPE0015;
	v->size = sizeof(struct BOOT_MASK);

	boot_mask = (struct BOOT_MASK *)(v->data);
	memset(v->data, 0, v->size);
	boot_mask->boot_type = 2;
	boot_mask->bootserver[0] = 192;
	boot_mask->bootserver[1] = 168;
	boot_mask->bootserver[2] = 160;
	boot_mask->bootserver[3] = 1;
	v->crc = calc_crc(v->size, 0, v->data); 

	v = (FLASH_CONFIG_VECTOR_HEADER *)&buffer[18 + sizeof(struct type0000) + sizeof(struct HW_SETUP) + v->size];
	v->type = 9999;
	v->size = 0;
	v->crc = 0;

	return(24 + sizeof(struct type0000) +
				sizeof(struct HW_SETUP) +
				sizeof(struct BOOT_MASK)); // new size of the file
}

static void show_config(void)
{
	int i;

	if (om_flag) {
		printf("\r\nOEM Banner [%s]", om_info->banner);
	}

	printf("\r\nMAC address assigned to Ethernet [");
	for (i = 0; i < EP_ALEN; i++) {
		printf("%.2x", hw_setup->physical_addr[i]);
		if (i == (EP_ALEN - 1)) {
			printf("]");
		}
		else {
			printf(":");
		}
	}

	printf("\r\nIP address assigned to Ethernet interface [");
	for (i = 0; i < IP_ALEN; i++) {
		printf("%d", hw_setup->itf_ip_addr[i]);
		if (i == (IP_ALEN - 1)) { // last part
			printf("]");
		}
		else {
			printf(".");
		}
	}

	printf("\r\nWatchdog timer ((A)ctive or (I)nactive) [%c]",
		seek_val(1, hw_setup->enable_wdt, wdt_op));

	printf("\r\nFirmware boot from ((F)lash or (N)etwork) [%c]",
		seek_val(1, hw_setup->boot_option, boot_op));

	printf("\r\nBoot type ((B)ootp,(T)ftp or Bot(H)) [%c]",
		seek_val(1, boot_mask->boot_type, boot_type));

	printf("\r\nBoot File Name [%s]", hw_setup->tftp_path);

	printf("\r\nServer's IP address [");
	for (i = 0; i < IP_ALEN; i++) {
		printf("%d", boot_mask->bootserver[i]);
		if (i == (IP_ALEN - 1)) { // last part
			printf("]");
		}
		else {
			printf(".");
		}
	}

	printf("\r\nConsole speed [%d]", speeda[hw_setup->console_speed]);

	printf("\r\n(P)erform or (S)kip Flash test [%c]",
		seek_val(1, hw_setup->op_code_crc, op_code_crc));

	printf("\r\n(S)kip, (Q)uick or (F)ull RAM test [%c]",
		seek_val(1, hw_setup->mem_test, mem_test));

	if (mt.board_type != BOARD_PR3K) {
		printf("\r\nFast Ethernet ((A)uto Neg, (1)00 BtH, 100 Bt(F), "
			"10 B(t)F, 10 Bt(H)) [%c]",
			seek_val(1, hw_setup->mii_operation, mii_op));
		printf("\r\nFast Ethernet Maximum Interrupt Events [%d]", hw_setup->max_int_work);
	}
	printf("\r\n\n");
}

static void check_speed_console(int console_speed)
{
	int fd, file_speed, cnt;

	if ((fd = open("/etc/getty_ttyS0", O_RDWR|O_CREAT)) < 0) {
		perror("Could not read/create file /etc/getty_ttyS0");
		return;
	}

	if ((cnt = read(fd, aux, sizeof(aux))) <= 0) {
		goto update_speed;
	}

	aux[cnt] = 0;
	file_speed = atoi(aux);

	if (speeda[console_speed] == file_speed) {
		return;
	}

update_speed:

	lseek(fd, 0, SEEK_SET);

	sprintf(aux, "%d", speeda[console_speed]);

	cnt = strlen(aux);

	if (write(fd, aux, cnt) != cnt) {
		perror("Could not write file /etc/getty_ttyS0");
		return;
	}

	ftruncate(fd, cnt);
	close(fd);

//	system("sh /bin/updatefiles /etc/getty_ttyS0 > /dev/ttyS0"); mp[Fev/2005] Bug fix 1482

}

static char my_getchar(char *fmt, char cur, char *op, int flag_any)
{
	char *p;
	int cnt;

	while(1) {
		printf("\r\n%s [%c] : ", fmt, cur);
		fflush(NULL);
		fflush(stdout);
		p = tmp;
		cnt = 0;

		while((*p = GetKeyPressed(flag_any&2)) != CHAR_CR) {
			if (*p != CHAR_BS) {
				if (*p >= 32 && *p < 127) {
					printf ("%c", *p);
				} else {
					printf ("?");
				}
				fflush(stdout);
				cnt++;
				p++;
			} else {
				if (cnt) {
					printf ("\b \b");
					fflush(stdout);
					--cnt;
					--p;
				}
			}
		}
		*p = 0;

		if (!(cnt)) {
			return(cur);
		}
		if (cnt > 1) {
			printf("\r\nERROR : Only one char should be typed...");
			continue;
		}

		if (*tmp >= 'a' && *tmp <= 'z')
			*tmp &= 0xdf;

		for(p = op; *p != 0xff; p+=2) {
			if (*tmp == *p) {
				return(*tmp);
			}
		}
		if (flag_any & 1) {
			return(cur);
		}
		printf ("\r\nERROR : Only [");
		for (p = op+2; *p != 0xff; p+=2)
			if (p == op+2)
				 printf("%c", *(p-2));
			else
				 printf(", %c", *(p-2));
		printf (" or %c] are accepted!", *(p-2));
	}
}

static void exit_rot(void)
{
	tcsetattr(0, TCSANOW, &std_term);
	printf("\r\n");
}

int main(int argc, char *argv[])
{
	int				i, j, x[6], k, flag;
	char				*p;
	FLASH_CONFIG_VECTOR_HEADER	*v;

	tcgetattr (0, &std_term);
	raw_term.c_cflag = (std_term.c_cflag & CBAUD) | CS8 | CREAD;
	raw_term.c_iflag = IGNPAR | IGNBRK;
	atexit(exit_rot);
	tcsetattr(0, TCSADRAIN, &raw_term);

	//Check if user is root, else do not allow access
        if (getgid() != 0) {
                printf("\r\n Only root user has access to bootconf!!!\r\n");
                exit(0);
        }

	if (argc == 2 && !strcmp(argv[1], "-O")) {
		om_flag = 1;
	}

	get_machine_type(&mt);

	read_om_info_from_flash(om_buf);

	config_size = read_configuration_from_flash(buffer);

	if (config_size > 0) {
		x[0] = 'n';
		type0000 = (struct type0000 *)
			access_type((FLASH_CONFIG_VECTOR_HEADER *)buffer, TYPE0000, 1000);
		hw_setup = (struct HW_SETUP *)
			access_type((FLASH_CONFIG_VECTOR_HEADER *)buffer, TYPE0068, 1000);
		boot_mask = (struct BOOT_MASK *)
			access_type((FLASH_CONFIG_VECTOR_HEADER *)buffer, TYPE0015, 1000);

		if (hw_setup == (struct HW_SETUP *)NULL ||
			boot_mask == (struct BOOT_MASK *)NULL ||
			type0000 == (struct type0000 *)NULL) {
			x[0] = 'Y';
		} else {
			if (!memcmp(om_info->mac_addr, hw_mac, 6)) {
				memcpy(om_info->mac_addr, hw_setup->physical_addr, 6);
			}
			save_type0000 = *type0000;
			save_boot_mask = *boot_mask;
			save_hw_setup = *hw_setup;
		}
	}
	else {
		x[0] = 'Y';
	}

	if (x[0] == 'Y') {
		x[0] = 'n';
		config_size = set_default_config();
	} else {
		x[0] = my_getchar("Set to defaults (y/n)", 'N', yes_no, 1);
	}

	if (x[0] == 'Y') {
		config_size = set_default_config();

		if (setjmp(my_env)) {
			goto L3;
		} else {
			goto L2;
		}
	} else {

		// not defaults
		// Just to see how it is configured
		printf("\r\n\nCurrent configuration\r\n");
		show_config();

		if (setjmp(my_env)) {
			goto L3;
		}
	}

L1:
	if (om_flag) {
		printf("\r\nOEM Banner [%s]: ", om_info->banner);
		j = get_param("%s", 2, (int *)aux);
		if (j > 0) {
			if (*(aux+1) == 0 && *aux == ' ') {
				memset(om_info->banner, 0, sizeof(om_info->banner)-1);
			} else {
				strncpy(om_info->banner, aux, sizeof(om_info->banner)-1);
				om_info->banner[sizeof(om_info->banner)-1] = 0;
			}
		}
	}

	printf("\r\nMAC address assigned to Ethernet [");
	for (i = 0; i < EP_ALEN; i++) {
		printf("%.2x", hw_setup->physical_addr[i]);
		if (i == (EP_ALEN - 1)) { // last part
			printf("]: ");
		}
		else {
			printf(":");
		}
	}
	j = get_param("%x:%x:%x:%x:%x:%x", 6, x);
	for (i = 0; i < j; i++) {
		om_info->mac_addr[i] = hw_setup->physical_addr[i] = (unsigned char)x[i];
	}

	printf("\r\nIP address assigned to Ethernet interface [");
	for (i = 0; i < IP_ALEN; i++) {
		printf("%d", hw_setup->itf_ip_addr[i]);
		if (i == (IP_ALEN - 1)) { // last part
			printf("]: ");
		}
		else {
			printf(".");
		}
	}
	j = get_param("%d.%d.%d.%d", 4, x);
	for (i = 0; i < j; i++) {
		hw_setup->itf_ip_addr[i] = (unsigned char)x[i];
	}

	x[1] = seek_val(1, hw_setup->enable_wdt, wdt_op);
	x[0] = my_getchar("Watchdog timer ((A)ctive or (I)nactive)",
				x[1], wdt_op, 0);
	hw_setup->enable_wdt = seek_val(0, x[0], wdt_op);

	x[1] = seek_val(1, hw_setup->boot_option, boot_op);
	x[0] = my_getchar("Firmware boot from ((F)lash or (N)etwork)",
				x[1], boot_op, 0);
	hw_setup->boot_option = seek_val(0, x[0], boot_op);

	if (hw_setup->boot_option) {
		x[1] = seek_val(1, boot_mask->boot_type, boot_type);
		x[0] = my_getchar("Boot type ((B)ootp,(T)ftp or Bot(H))",
					x[1], boot_type, 0);
		//[RK]Jun/13/05 - fixed bug#4016
		boot_mask->boot_type = seek_val(0, x[0], boot_type);

		printf("\r\nBoot File Name [%s]: ", hw_setup->tftp_path);
		j = get_param("%s", 1, (int *)aux);
		if (j > 0) {
			strncpy(hw_setup->tftp_path, aux, PATH_LEN - 1);
		}
		hw_setup->tftp_path[PATH_LEN - 1] = 0;

		printf("\r\nServer's IP address [");
		for (i = 0; i < IP_ALEN; i++) {
			printf("%d", boot_mask->bootserver[i]);
			if (i == (IP_ALEN - 1)) { // last part
				printf("]: ");
			}
			else {
				printf(".");
			}
		}
		j = get_param("%d.%d.%d.%d", 4, x);
		for (i = 0; i < j; i++) {
			boot_mask->bootserver[i] = (unsigned char)x[i];
		}
	}

	flag = 0;
	while (!flag) {
		printf("\r\nConsole speed [%d]: ", speeda[hw_setup->console_speed]);
		j = get_param("%d", 1, x);
		if (j > 0) {
			for (k = 0; k < 6; k++) {
				if (speeda[k] == x[0]) {
					hw_setup->console_speed = k;
					flag = 1;
					break;
				}
			}
		}
		else {
			flag = 1;
		}
	}

	x[1] = seek_val(1, hw_setup->op_code_crc, op_code_crc);
	x[0] = my_getchar("(P)erform or (S)kip Flash test",
				x[1], op_code_crc, 0);
	hw_setup->op_code_crc = seek_val(0, x[0], op_code_crc);

	x[1] = seek_val(1, hw_setup->mem_test, mem_test);
	x[0] = my_getchar("(S)kip, (Q)uick or (F)ull RAM test",
				x[1], mem_test, 0);
	hw_setup->mem_test = seek_val(0, x[0], mem_test);

	if (mt.board_type != BOARD_PR3K) {
		x[1] = seek_val(1, hw_setup->mii_operation, mii_op);
		x[0] = my_getchar("Fast Ethernet ((A)uto Neg, (1)00 BtH, "
					"100 Bt(F), 10 B(t)F, 10 Bt(H))", x[1], mii_op, 0); 
		hw_setup->mii_operation = seek_val(0, x[0], mii_op);

		printf("\r\nFast Ethernet Maximum Interrupt Events [%d]: ", hw_setup->max_int_work);
		if (get_param("%d", 1, x)) {
			hw_setup->max_int_work = x[0];
		}
	}

L3:
	// recalculates CRC after the changes
	p = (char *)hw_setup;
	v = (FLASH_CONFIG_VECTOR_HEADER *)(p - 6);
	v->crc = calc_crc(v->size, 0, v->data);
	p = (char *)boot_mask;
	v = (FLASH_CONFIG_VECTOR_HEADER *)(p - 6);
	v->crc = calc_crc(v->size, 0, v->data);

L2:
	// Just to see how it is configured
	printf("\r\n\nNew configuration to be saved as\r\n");
	show_config();

	x[0] = my_getchar("Do you confirm these changes in flash "
				"( (Y)es, (N)o (Q)uit )", 'N', save_op, 2);

	if (x[0] == 'Q') {
		exit(0);
	}

	if (x[0] == 'Y') {
		tcsetattr(0, TCSANOW, &std_term);
		printf("\r\n");
		fflush(stdout);
		save_om_info_into_flash(om_buf);
		save_config_into_flash(buffer, config_size);
		printf("\r\n\n");
		check_speed_console(hw_setup->console_speed);
		printf("\r\n\n");

		if (mt.board_type != BOARD_PR3K) {

			struct ifreq ifr;
			int sockfd;

			if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
				perror("sockfd open:");
				exit(0);
			}	

			strcpy(ifr.ifr_name, "eth0");

			ifr.ifr_ifru.ifru_flags = 1;
			ifr.ifr_ifru.ifru_ivalue = hw_setup->mii_operation;

			if (ioctl(sockfd, SIOCDEVPRIVATE, &ifr) < 0) {
				perror("ioctl (mii OP) :");
				exit(0);
			}

			ifr.ifr_ifru.ifru_ivalue = hw_setup->max_int_work;

			if (ioctl(sockfd, SIOCDEVPRIVATE+1, &ifr) < 0) {
				perror("ioctl (mii Max Intr) :");
				exit(0);
			}

			close(sockfd);
		}

		sprintf(aux, "config_eth0 ip %d.%d.%d.%d mac %x:%x:%x:%x:%x:%x\n"
			"sh " PATH_NET "/st_routes",
			hw_setup->itf_ip_addr[0], 
			hw_setup->itf_ip_addr[1],
			hw_setup->itf_ip_addr[2],
			hw_setup->itf_ip_addr[3],
			hw_setup->physical_addr[0],
			hw_setup->physical_addr[1],
			hw_setup->physical_addr[2],
			hw_setup->physical_addr[3],
			hw_setup->physical_addr[4],
			hw_setup->physical_addr[5]);
		system(aux);

		exit(0);
	}
	printf("\r\n");
	goto L1;
}
