/*****************************************************************
* File: cy_ras.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description: Main part of the portslave. Handles
*              configuration changes.
*
****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define XDEBUG

#include "server.h"

//[RK]Jun/08/06 - event configuration
extern void writeevt(int, char* , ...);

typedef union _x_types {
	char				**xpc;
	struct realm_def	**xr;
} x_types;

typedef union _y_types {
	int					*yi;
	char				*yc;
	char				**ypc;
	struct realm_def	**yr;
	x_types				yx;
} y_types;

static time_t vet_time[MAXVLINES];
static int vet_flag[MAXVLINES];

struct main_cfg save_mainconf;
struct line_cfg save_lineconf[MAXVLINES];
struct line_cfg save_allconf;

static char *pg_name;

static int last_ip = 0;

static void got_usr1(int flag);
static void init_socket_server(void);

static void kill_session_nowait(CY_SESS *sess, int flag)
{
	if (sess->vet_pid) {
		kill(sess->vet_pid, SIGINT);
		if (flag)
			sess->vet_pid = 0;
	}
	return;
}

static void kill_session_list_nowait (int port_num, int flag)
{
	CY_SESS *sess = cy_shm->sess_list[port_num];

	while (sess) {
		if (sess->layout) break; // IPDU
		kill_session_nowait(sess, flag);
		sess = sess->next;
	}
	return;
}

static void read_st_routes(char **st_rt)
{
	int  fd, sz;
	char fname[64];

	sprintf(fname, "%s/st_routes", PATH_NET);

	 if ((fd = open(fname, O_RDONLY)) < 0) { 
		XPRINTF("%s : could not open file %s\r\n", pg_name, fname);
		return;
	}

	if ((sz = lseek(fd, 0, SEEK_END)) <= 0) {  
		close(fd);
		return;
	}

	lseek(fd, 0, SEEK_SET);	

	if ((*st_rt = shm_malloc(sz, TYPE_SHM_STR)) == NULL) {
		close(fd);
		return;
	}

	read(fd, *st_rt, sz);

	close(fd);
}

static int check_string(char *str1, char *str2)
{
	if ((!str1 && str2) || (str1 && !str2) ||
		(str1 && str2 && strcmp(str1, str2))) {
		return(1);
	}
	return(0);
}

static void ip_to_asc(char *p, int ipno)
{
	p += sprintf(p, "%d", (ipno >> 24) & 0xff);
	p += sprintf(p, ".%d", (ipno >> 16) & 0xff);
	p += sprintf(p, ".%d", (ipno >> 8) & 0xff);
	sprintf(p, ".%d", ipno & 0xff);
}

static void add_remove_pool_itf_and_route(int flag)
{
	char ip_addr[20];
	char cmd[255];
	char *p;
	struct line_cfg *pconf = &lineconf[0];
	int ix_ip = 0, i, j;
	int ip_local[MAXLINES];

	for (i=0; i < MAXLINES; i++, pconf++) {
		if (pconf->tty == NULL || !(*pconf->tty) || pconf->virtual ||
			(pconf->protocol != P_SOCKET_SERVER &&
		 	pconf->protocol != P_SOCKET_SERVER_RAW &&
		 	pconf->protocol != P_SOCKET_SSH &&
		 	pconf->protocol != P_MODEM &&
		 	pconf->protocol != P_2WRAW) ||
			!pconf->pool_ipno) {
			continue;
		}

		for(j=0; j < ix_ip && ip_local[j] != pconf->pool_ipno; j++) ;

		if (j < ix_ip) continue;

		ip_local[ix_ip++] = pconf->pool_ipno;

		if (flag) {
			/* Call ifconfig to create the interface. */
			ip_to_asc(ip_addr, pconf->pool_ipno);
			p += sprintf((p=cmd), "exec %s eth0:%d %s netmask 255.255.255.255 broadcast 0.0.0.0",
			PATH_IFCONFIG, ix_ip+MAXLINES, ip_addr);
		} else {
			/* remove the interface and route */
			sprintf(cmd, "exec %s eth0:%d down",
				PATH_IFCONFIG, ix_ip+MAXLINES);
		}
		system(cmd);
	}
}

static void add_remove_socket_itf_and_route(int flag, struct line_cfg *pconf)
{
	char ip_addr[20], ip_mask[20], ip_broad[20];
	char cmd[255];
	unsigned int aux;
	char *p;

	if (pconf->tty == NULL || !(*pconf->tty) || pconf->virtual ||
		(pconf->protocol != P_SOCKET_SERVER &&
		 pconf->protocol != P_SOCKET_SERVER_RAW &&
		 pconf->protocol != P_SOCKET_SSH &&
		 pconf->protocol != P_MODEM &&
		 pconf->protocol != P_2WRAW) ||
		!pconf->ipno) {
		return;
	}

	if (flag) {
		/* Call ifconfig to create the interface. */
		ip_to_asc(ip_addr, pconf->ipno);
		ip_to_asc(ip_mask, pconf->netmask);
		aux = pconf->ipno & pconf->netmask;
		aux |= ~pconf->netmask;
		ip_to_asc(ip_broad, aux);
		p += sprintf((p=cmd), "exec %s eth0:%d %s netmask %s broadcast %s",
		PATH_IFCONFIG, pconf->this+1, ip_addr, ip_mask, ip_broad);
	} else {
		/* remove the interface and route */
		sprintf(cmd, "exec %s eth0:%d down",
		PATH_IFCONFIG, pconf->this+1);
	}
	system(cmd);
}

static int check_config_equal(struct conf *pc, char *pa, char *po)
{
	int	rtn = 1;
	y_types	ya, yo;

	for( ; pc->name != NULL && rtn; pc++) {
		ya.yc = pa+pc->offs;
		yo.yc= po+pc->offs;
		switch(pc->type) {
			case C_MSTR: //[RK]Nov/25/02 Bug#97
			case C_STR:
			case C_CHAT:
				if (!*(ya.yx.xpc) && !*(yo.yx.xpc)) {
					break;
				}
				if (!*(ya.yx.xpc) || !*(yo.yx.xpc)) {
					rtn = 0;
					break;
				}
				if (strcmp(*(ya.yx.xpc), *(yo.yx.xpc))) {
					rtn = 0;
				}
				break;
			case C_IPNO:
			case C_IPDY:
			case C_HOST:
				XPRINTF4("%s va[%x] %d.%d.%d.%d vo[%x] %d.%d.%d.%d \r\n", pc->name, ya.yi, ya.yc[0], ya.yc[1], ya.yc[2], ya.yc[3], yo.yi, yo.yc[0], yo.yc[1], yo.yc[2], yo.yc[3]);  
				if (*(ya.yi) != *(yo.yi)) {
					rtn = 0;
				}
				break;
			case C_INT:
			case C_LIST:
			case C_INDY:
			case C_CHAR:
				XPRINTF4("%s va[%x] %d vo[%x] %d \r\n", pc->name, ya.yi, *(ya.yi), yo.yi, *(yo.yi));  
				if (*(ya.yi) != *(yo.yi)) {
					rtn = 0;
				}
				break;
			case C_REALM:
				if (!*(ya.yx.xr) && !*(yo.yx.xr)) {
					break;
				}
				if (!*(ya.yx.xr) || !*(yo.yx.xr)) {
					rtn = 0;
					break;
				}
				if (!check_config_equal(&realm_cfg[0], *(ya.yx.xpc), *(yo.yx.xpc))) {
					rtn = 0;
				}
				break;
		}
	}
	return(rtn);
}

static void free_mem_config(struct conf *pc, char *p)
{
	y_types	y, x;

	for( ; pc->name != NULL; pc++) {
		y.yc = p+pc->offs;
		switch(pc->type) {
			case C_LSTR:
				while (*(y.yx.xpc)) {
					x.yc = *(y.yx.xpc);
					XPRINTF4("free_mem_cfg lstr %s : I=%x PC=%x N=%x\r\n",
						pc->name, y.yi, (int)*(y.yx.xpc), (int)*(x.yi));  
					shm_free_str(*(y.yx.xpc));
					y.yc = x.yc;
				}
				break;
			case C_MSTR:
			case C_XSTR:
			case C_STR:
			case C_CHAT:
				if (*(y.yx.xpc)) {
					XPRINTF4("free_mem_cfg str %s : I=%x PC=%x\r\n", pc->name,
						y.yi, (int)*(y.yx.xpc));  
					shm_free_str(*(y.yx.xpc));
				}
				break;
			case C_REALM:
				if (*(y.yx.xr)) {
					XPRINTF4("free_mem_cfg realm %s : I=%x PC=%x\r\n", pc->name,
						y.yi, (int)*(y.yx.xpc));  
					free_mem_config(&realm_cfg[0], *(y.yx.xpc));
					shm_free_str(*(y.yx.xpc));
				}
				break;
		}
	}
}

static void dup_config(void)
{
	int	i;

	XPRINTF8("dup_config free mainconf\r\n");
	free_mem_config(&main_cfg[0], (char *)&mainconf);
	XPRINTF8("dup_config free allconf\r\n");
	free_mem_config(&line_cfg[0], (char *)&allconf);
	for(i=0; i < MAXVLINES; i++) {
		XPRINTF8("dup_config free lineconf[%d]\r\n", i);
		free_mem_config(&line_cfg[0], (char *)&lineconf[i]);
	}

	XPRINTF8("dup_config copy mainconf\r\n");
	mainconf = save_mainconf;
	XPRINTF8("dup_config copy allconf\r\n");
	allconf = save_allconf;
	for(i=0; i < MAXVLINES; i++) {
		XPRINTF8("dup_config copy lineconf[%d]\r\n", i);
		lineconf[i] = save_lineconf[i];
	}

	XPRINTF8("dup_config clear save_mainconf\r\n");
	memset(&save_mainconf, 0, sizeof(save_mainconf));
	XPRINTF8("dup_config clear save_allconf\r\n");
	memset(&save_allconf, 0, sizeof(save_allconf));
	XPRINTF8("dup_config clear save_lineconf\r\n");
	memset(&save_lineconf[0], 0, sizeof(save_lineconf));

}

static void kill_process(struct line_cfg *pconf, int flag)
{
	if (cy_shm->vet_pid[pconf->this] == cy_shm->PMD_pid) return;

	switch (pconf->protocol) {
	case P_SOCKET_SERVER:
	case P_SOCKET_SERVER_RAW:
	case P_SOCKET_SSH:
             	if (cy_shm->vet_pid[pconf->this]) {
			// was fast
			kill(cy_shm->vet_pid[pconf->this], SIGINT);
			cy_shm->vet_pid[pconf->this] = 0;
		} else {
			kill_session_list_nowait(pconf->this, flag);
		}
		add_remove_socket_itf_and_route(0, pconf);
		break;
	case P_SOCKET_CLIENT:
		if (cy_shm->vet_pid[pconf->this]) {
			kill(cy_shm->vet_pid[pconf->this], SIGINT);
			if (flag)
				cy_shm->vet_pid[pconf->this] = 0;
		}
		break;
	default:
		if (cy_shm->vet_pid[pconf->this]) {
			if (cy_shm->vet_ppp[pconf->this]) {
				kill(cy_shm->vet_pid[pconf->this], SIGINT);
			} else {
				kill(cy_shm->vet_pid[pconf->this], SIGKILL);
			}
			if (flag)
				cy_shm->vet_pid[pconf->this] = 0;
		}
		break;
	}
}

/*
 *	  Relay the SIGKILL to our process group.
 */
static void got_int(int sig)
{
	int i, st;

	for(i=0; i < MAXVLINES; i++) {
		if (lineconf[i].tty == NULL || !(*lineconf[i].tty)) {
			continue;
		}
		kill_process(&lineconf[i], 1);
	}

	while (wait(&st) > 0) ;

}

static void init_socket_server(void)
{
	int i;

	for(i=0; i < MAXVLINES; i++) {
		if (lineconf[i].tty == NULL || !(*lineconf[i].tty)) {
			continue;
		}
		if ((lineconf[i].protocol == P_SOCKET_SERVER ||
			 lineconf[i].protocol == P_SOCKET_SERVER_RAW) &&
			lineconf[i].socket_port) {
			kill_session_list_nowait(i, 0);
		}
	}
}

void signal_inetd(void)
{
	FILE *f;
	int pid, ret;

	if ((f = fopen("/var/run/inetd.pid", "r")) == NULL) {
		goto exit_err;
	}

	ret = fscanf(f, "%d", &pid);
	fclose(f);

	if (ret != 1) {
		goto exit_err;
	}

	if (!kill(pid, SIGHUP)) {
		return;
	}

exit_err:

	XPRINTF("%s : could not send SIGHUP to inetd(%d) [%d]: %s\r\n", pg_name,
		pid, errno, strerror(errno));
}

/*
 *	  Relay the SIGHUP to our process group.
 */

static void got_hup(int sig)
{
	FILE *f;
	int i, flag = 0, flagi = 0, flagp = 0, flag_alias, flag_eth, flag_st;
	char cmd[128];
	struct stat st;

	if (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) == 2) {
		return;
	}

	XPRINTF8("got_hup %d\r\n", sig);

	memset(&cy_shm->vet_chg[0], 0, sizeof(cy_shm->vet_chg));

	shm_get_set_val(2, &cy_shm->st_cy_ras, 1, CY_SEM_CONFIG);
		
	initcfg(&save_mainconf, &save_allconf, &save_lineconf[0]);
	readcfg(&save_mainconf, &save_allconf, &save_lineconf[0]);

	read_st_routes(&save_mainconf.st_routes);

	if (sig) {
		flag_st = check_string(mainconf.st_routes, save_mainconf.st_routes);
		flag_alias = (mainconf.eth_ip_alias != save_mainconf.eth_ip_alias ||
			mainconf.eth_mask_alias != save_mainconf.eth_mask_alias);

		//[RK]Jun/05/03 Bug#566 - first check dhcpc 
		if (mainconf.dhcpcd || save_mainconf.dhcpcd) { // DHCPCD
			if (mainconf.dhcpcd == save_mainconf.dhcpcd) {
				flag_eth = 0;
			} else {
				flag_eth = 2;
			}
		} else {
			flag_eth = (mainconf.eth_ip != save_mainconf.eth_ip ||
				    mainconf.netmask != save_mainconf.netmask ||
				    mainconf.mtu != save_mainconf.mtu);
		}

	} else {
		flag_eth = 0; // boot - does not restart eth0	
		flag_alias = 0; 
		flag_st = 0; 
	}

	if (check_string(mainconf.nfs_data_buffering,
			save_mainconf.nfs_data_buffering)) {
		cy_shm->DB_status |= DB_STATUS_NFS_CHANGED;
		flag = 1;
	}

	if (mainconf.DB_facility != save_mainconf.DB_facility) {
		cy_shm->DB_status |= DB_STATUS_FAC_CHANGED;
		flag = 1;
	}

	for (i=0; i < MAXVLINES; i++) {
		if (lineconf[i].this == i &&
			save_lineconf[i].this == i) {

			if (check_config_equal(&line_cfg[0], (char *)&lineconf[i],
					(char *)&save_lineconf[i])) {
				XPRINTF1("HUP port %d config equal\r\n", i);
				continue;
			}

		} else {
			if (lineconf[i].this != i &&
				save_lineconf[i].this != i) {
				XPRINTF1("HUP port %d inactive\r\n", i);
				continue;
			}
			XPRINTF1("HUP port %d active/inactive\r\n", i);
		}

// [Fuji - Feb/02/2003] - vet_chg will be treated now by masks.
// 1 will be related to cy_buffering; 2 will be pmd.

		if (lineconf[i].tty != NULL && *lineconf[i].tty &&
			(((lineconf[i].protocol == P_SOCKET_SERVER) ||
			  (lineconf[i].protocol == P_SOCKET_SERVER_RAW) ||
		 	  (lineconf[i].protocol == P_SOCKET_SSH))  
			 || //[RK]Dec/06/02 Bug#97 
			  ((save_lineconf[i].protocol == P_SOCKET_SERVER) ||
			   (save_lineconf[i].protocol == P_SOCKET_SERVER_RAW) ||
		 	   (save_lineconf[i].protocol == P_SOCKET_SSH)))) {
			flag = 1;
			flagi = 1;
			cy_shm->vet_chg[i] |= 1;
		}
	
// [Fuji - Feb/02/2003] - IPDU protocol change
		if (lineconf[i].tty != NULL && *lineconf[i].tty &&
			(lineconf[i].protocol == P_IPDU ||  
			  save_lineconf[i].protocol == P_IPDU)) {
 			//[RK]June/03/04 - SSH/Telnet to IPDU
 			if ((lineconf[i].pmsessions != PMSESS_NONE) ||
 			    (save_lineconf[i].pmsessions != PMSESS_NONE)) {
 				flagi = 1;
 			}
			flagp = 1;
			cy_shm->vet_chg[i] |= 2;
		}
	
		//[RK]Aug/07/03 Bug#690
		if (lineconf[i].this != i) { // port was inactive
			flag = 1;
			flagi = 1;
			cy_shm->vet_chg[i] |= 1;
		}
		       	
		XPRINTF1("HUP port %d : killing related process(es)\r\n",i);
		kill_process(&lineconf[i], 0);
		add_remove_socket_itf_and_route(0, &lineconf[i]);
	}

	add_remove_pool_itf_and_route(0);

	XPRINTF8("got_hup dup_config\r\n", sig);
	dup_config();

	if (flag_eth) {
		if (stat("/var/aidp.lock", &st))
			system("killall aidpd > /dev/null 2>&1");
		XPRINTF8("got_hup config_eth0\r\n");
		system("sh /etc/network/intf_cfg_eth0 restart");
	}

	if (flag_alias) {
		XPRINTF8("got_hup config_eth0_alias\r\n");
		system("sh /etc/network/intf_cfg_eth0:0 restart");
	}

	if (mainconf.dhcpcd) {
		if (!flag_eth) {
			got_usr1(0);
		}
		if (flag_st && !flag_eth) {
			system("sh /bin/handle_udhcpc.sh rt");
		}
	} else {
		if (flag_st || flag_eth) {
			if (!flag_eth) {
				system("ifconfig eth0 down");
				system("ifconfig eth0 up");
			}
			system("sh " PATH_NET "/st_routes");
		}
	}

	if (mainconf.hostname1) {
		if (mainconf.hostname) {
			shm_free_str(mainconf.hostname);
		}
		mainconf.hostname = mainconf.hostname1;
		snprintf(cmd, sizeof(cmd), "exec /bin/hostname %s", mainconf.hostname);
		system(cmd);
		if ((f = fopen("/etc/hostname", "w+")) == NULL) {
			XPRINTF("%s : could not create file /etc/hostname\r\n", pg_name);
		} else {
			fprintf(f, mainconf.hostname);
			fclose(f);
		}
	} else {
		snprintf(cmd, sizeof(cmd), "exec /bin/hostname -F /etc/hostname");
		if (system(cmd) == 0 && gethostname(cmd, sizeof (cmd)) == 0) {
			if (mainconf.hostname) {
				shm_free_str(mainconf.hostname);
			}
			mainconf.hostname = shm_strdup(cmd);
		}
	}

	for (i=0; i < MAXVLINES; i++) {
		if (lineconf[i].tty == NULL || *lineconf[i].tty == 0) {
			continue;
		}
		//[RK]Dec/10/02 Bug#127 syslog_buffering max value 255
		if (lineconf[i].syslog_buffering > 255) {
			lineconf[i].syslog_buffering = 255;
		}
	
		add_remove_socket_itf_and_route(1, &lineconf[i]);
	}

	add_remove_pool_itf_and_route(1);

	if (flag) {
		XPRINTF8("got_hup active/inactive buffering ports\r\n");
		if (cy_shm->DB_pid) {
			kill(cy_shm->DB_pid, SIGUSR2);
		}
	}
	if (flagi) {
		XPRINTF8("got_hup signalling inetd\r\n");
		signal_inetd();
	}
	if (flagp) {
		XPRINTF8("got_hup active/inactive pmd ports\r\n");
		if (cy_shm->PMD_pid) {
			kill(cy_shm->PMD_pid, SIGUSR2);
		}
	}
	
	shm_get_set_val(1, &cy_shm->st_cy_ras, 1, CY_SEM_CONFIG);

}

/*
 *	  Relay the SIGUSR to our process group.
 */

static char dhcp_fname[] = "/etc/dhcpcd-eth0.save";

// DHCPCD UP/NEW
static void got_usr1(int flag)
{
	FILE *fp;
	char buf[128], *p;
	int ip = 0;

	if (flag) {
		XPRINTF2("%s : got_usr1 called via sigusr1\r\n", pg_name);
	} else {
		XPRINTF2("%s : got_usr1 called in init\r\n", pg_name);
	}

	if (!mainconf.dhcpcd) { // dhcpcd is inactive
		XPRINTF("Signal USR without dhcp client configured\r\n");
		return;
	}

	// dhcpcd is active 

	if ((fp = fopen(dhcp_fname, "r")) == NULL) {
		XPRINTF("Could not open file %s\r\n", dhcp_fname, strerror(errno));
		return;
	}
	while (fgets(buf, sizeof(buf)-2, fp)) {
		if (buf[0] == '\n') continue; 
		p = buf + strlen(buf) - 1;
		if (*p == '\r') *p = 0;
		if ((p=strchr(buf, '=')) == NULL) continue;
		*p++ = 0;
		if (strcmp(buf, "IPADDR")) continue;
		if (!(ip = inet_addr(p)) || ip == -1) {
			break;
		}
		XPRINTF2("%s : dhcpcd has gotten ip %s\r\n", pg_name, p);
		mainconf.eth_ip = ip;
		mainconf.ipno = ip;
		last_ip = ip;
		init_socket_server();
	}
	fclose(fp);
}

#define SIG_INT		0x01
#define SIG_HUP		0x02
#define SIG_GEN		0x04
#define SIG_USR1	0x08
#define SIG_ALRM	0x10

static int last_sig = 0;
static volatile int got_signal = 0;

static void got_sig(int sig)
{
	switch (sig) {
		case SIGUSR1:
			got_signal |= SIG_USR1;
			break;
		case SIGALRM:
			got_signal |= SIG_ALRM;
			break;
		case SIGHUP:
			got_signal |= SIG_HUP;
			break;
		case SIGINT:
		case SIGTERM:
			got_signal |= SIG_INT;
			break;
		default:
			last_sig = sig;
			got_signal |= SIG_GEN;
			break;
	}
}

static void init_log(void)
{
	int fac;

	fac = LOG_FAC(LOG_LOCAL0) + mainconf.facility;
	nopenlog("cy_ras", LOG_CONS | LOG_NDELAY, LOG_MAKEPRI(fac, 0));

	nsyslog(LOG_INFO, "Cy_ras Started");
}

/*
 *	Main loop for all channels.
 */
int main(int argc, char **argv)
{
	pid_t	pid;
	int		i, st;
	char	nport[4];
	struct sigaction sa = {0};

	pg_name = argv[0];

	memset(&save_mainconf, 0, sizeof(save_mainconf));
	memset(&save_lineconf, 0, sizeof(save_lineconf));
	memset(&save_allconf,  0, sizeof(save_allconf));

	if (argc >= 2) {
		set_xprintf(atoi(argv[1]));
	}

	if (shm_init(-1) < 0) {
		return(0);
	}

	while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) == 0) {
		sleep(1);
		continue;
	}

	/* Redirect stdin, stdout, and stderr to /dev/null. */
	freopen("/dev/null", "r", stdin);
	freopen("/dev/null", "w", stdout);
	freopen("/dev/null", "w", stderr);


	/*
 	 *	  try to catch all signals first
 	 */
	sa.sa_handler = got_sig;

	for (i=1; i < 64; i++) {
		if (i == SIGPIPE) continue;
		if (i == SIGWINCH) continue;
		if (i == SIGSEGV) continue;
		if (i == SIGFPE) continue;
		if (i == SIGILL) continue;
		if (i == SIGKILL) continue;
		if (i == SIGCHLD) continue;
		sigaction(i,  &sa, NULL);
		unblock(SIGINT);
	}

	/*
	 *	  Read the config file. (alike SIGHUP)
	 */

	init_log();

	if (getppid() != 1) {
		XPRINTF("%s : restarted ppid %d\r\n", pg_name, getppid());
	}

	got_hup(0);

	for(i=0; i < MAXLINES; i++) {
		if (cy_shm->vet_pid[i] && kill(cy_shm->vet_pid[i], 0) < 0) {
			cy_shm->vet_pid[i] = 0;
		}
	}

	while (1) {
		alarm(0);
		if (got_signal & SIG_GEN) {
			got_signal &= ~SIG_GEN;
			XPRINTF("%s : received signal %d (ignored)\r\n", pg_name, last_sig);
		}

		if (got_signal & SIG_USR1) {
			got_signal &= ~SIG_USR1;
			XPRINTF2("%s : received signal USR1\r\n", pg_name);
			got_usr1(1);
			XPRINTF2("%s : received signal USR1-END\r\n", pg_name);
		}

		if (got_signal & SIG_HUP) {
			got_signal &= ~SIG_HUP;
			XPRINTF2("%s : received signal HUP\r\n", pg_name);
			got_hup(SIGHUP);
			XPRINTF2("%s : received signal HUP-END\r\n", pg_name);
			got_signal &= ~SIG_HUP;
			continue;
		}

		if (got_signal & SIG_INT) {
			got_signal &= ~SIG_INT;
			XPRINTF2("%s : received signal INT\r\n", pg_name);
			got_int(SIGINT);
			XPRINTF2("%s : received signal INT-END\r\n", pg_name);
			break;
		}

		if (got_signal & SIG_ALRM) {
			got_signal &= ~SIG_ALRM;
			for (i=0; i < MAXLINES; i++) {
				if (!vet_flag[i]) {
					continue;
				}
				if (cy_shm->vet_pid[i] &&
					(vet_time[i] + 20) < time(NULL)) {
					vet_flag[i] = 0;
					cy_shm->vet_pid[i] = 0;
				}
			}
		}

		for (i=0; i < MAXLINES; i++) {
			if (lineconf[i].this != i ||
				lineconf[i].tty == NULL ||
				*lineconf[i].tty == 0) {
				continue;
			}

		
			if (lineconf[i].virtual == 0 &&
				lineconf[i].protocol != P_SOCKET_SSH &&
				lineconf[i].protocol != P_SOCKET_SERVER &&
				lineconf[i].protocol != P_IPDU &&
				lineconf[i].protocol != P_BILLING &&
#ifdef TS1H
				lineconf[i].protocol != P_MODBUS &&
#endif
				lineconf[i].protocol != P_MODEM &&
				lineconf[i].protocol != P_2WRAW &&
				lineconf[i].protocol != P_SOCKET_SERVER_RAW &&
				cy_shm->vet_pid[i] == 0 &&	//[RK]Dec/13/02 Bug#145
				(cy_shm->st_data_buffering[i] == DB_ST_INACTIVE)) {
				if ((pid = fork())) {
					vet_time[i] =  time(NULL);
					vet_flag[i] =  0;
					cy_shm->vet_pid[i] = pid;
				} else {
					sprintf(nport, "%d", i);
					if (argc >= 2) {
						execl("/bin/portslave", "/bin/portslave",
							nport, argv[1], lineconf[i].tty, NULL);
					} else {
						execl("/bin/portslave", "/bin/portslave",
							nport, lineconf[i].tty, NULL);
					}
					/* It is not supposed to reach this point */
					exit(-EINVAL);
				}
			}
			if (lineconf[i].virtual == 0 &&
				lineconf[i].protocol == P_BILLING &&
				cy_shm->vet_pid[i] == 0 &&
				cy_shm->st_data_buffering[i] == DB_ST_INACTIVE) {
				if ((pid = fork())) {
					vet_time[i] =  time(NULL);
					vet_flag[i] =  0;
					cy_shm->vet_pid[i] = pid;
				} else {
					char port[10];
					sprintf(port, "%d", i+1);
					execl("/bin/billing", "billing", port, NULL);
					/* It is not supposed to reach this point */
					exit(-EINVAL);
				}
			}
			if (lineconf[i].virtual == 0 &&
				lineconf[i].protocol == P_MODEM &&
				cy_shm->vet_pid[i] == 0 &&
				cy_shm->st_data_buffering[i] == DB_ST_INACTIVE) {
				if ((pid = fork())) {
					vet_time[i] =  time(NULL);
					vet_flag[i] =  0;
					cy_shm->vet_pid[i] = pid;
				} else {
					char port[10];
					sprintf(port, "%d", i+1);
					execl("/bin/modem-replace", "modem-replace", port, NULL);
					/* It is not supposed to reach this point */
					exit(-EINVAL);
				}
			}
			if (lineconf[i].virtual == 0 &&
				lineconf[i].protocol == P_2WRAW &&
				cy_shm->vet_pid[i] == 0 &&
				cy_shm->st_data_buffering[i] == DB_ST_INACTIVE) {
				if ((pid = fork())) {
					vet_time[i] =  time(NULL);
					vet_flag[i] =  0;
					cy_shm->vet_pid[i] = pid;
				} else {
					char port[10];
					sprintf(port, "%d", i+1);
					execl("/bin/2way-raw", "2way-raw", port, NULL);
					/* It is not supposed to reach this point */
					exit(-EINVAL);
				}
			}
		}
		
		if (got_signal) {
			continue;
		}

		// Silva (Mar-26-2001)
		alarm(5);
		pid = waitpid (-1, &st, 0); /* check if any child has exited */
		alarm(0);
		if (pid <= 1 && !got_signal) {
			if (errno != ECHILD)
				XPRINTF("Waitpid failed: %s\r\n", sys_errlist[errno]);
			sleep(5);
			got_signal |= SIG_ALRM;
			continue;
		}
		if (pid > 0) {
			for(i=0; i < MAXLINES; i++) {
				if (cy_shm->vet_pid[i] == pid) {
					//[RK]Jun/09/06 - fixed problem with dial-in
					if (cy_shm->tty_user[i]) {
						//[RK]May/12/06 - event notification
						writeevt(6,"ssii",cy_shm->tty_user[i],
							cy_shm->line_conf[i].serverfarm,
							i+1,4);
						shm_free_str(cy_shm->tty_user[i]);
						cy_shm->tty_user[i] = NULL;
					}

					if ((vet_time[i] + 20) > time(NULL)) {
						if (vet_flag[i]) break;
						vet_flag[i] = 1;
						XPRINTF("Port S%d is respawning too fast. Waiting 20 sec before restarting.\r\n",i+1);
						break;
					} 
					XPRINTF1("Wait port %d pid %d\r\n",i+1, pid);
					cy_shm->vet_pid[i] = 0;
					break;
				}
			}
			if (i >= MAXLINES) {
				XPRINTF("Wait pid %d not found\r\n", pid);
			}
		}
	}
	return(0);
}

