/*
 * lib1.c         Generic functions.
 *
 * Version:      @(#)lib1.c  1.36  13-Jan-1998  miquels@cistron.nl
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <values.h>
#include "server.h"

static int rad_portno(char *tty, int port, int fproto);


/*
 *	Initialize struct and read config file.
 */
int rad_init(int port, struct auth *ai, char *tty, int fproto)
{
	static char ident[64] = "portslave";
	int fac, oport, ppp = 0;

	/*
	 *	attach shared memory
	 */
	if (shm_init(-1) < 0) {
		return -1;
	}

	while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
		sleep(1);
		continue;
	}

	/*
	 *	Find out port number if needed.
	 */
	oport = port;
	if (tty && (port <= -2 && port >= -6)) /* look for TTY */
	{
		port = rad_portno(tty, port, fproto);
		if (oport == -2) /* PPP daemon */
			ppp = 1;
	}

	if (port < 0) {
		return(port);
	}

	/*
	 *	Open the logfile.
	 */

#ifdef CYCLADES
	fac = LOG_FAC(LOG_LOCAL0) + mainconf.facility;
	if (lineconf[port].serverfarm && *lineconf[port].serverfarm)
		snprintf(ident, sizeof (ident), "S%d.%s",
			port+1, lineconf[port].serverfarm);
	else
		snprintf(ident, sizeof (ident), "S%d.%s",
			port+1, lineconf[port].tty);
	nopenlog(ident, LOG_NDELAY, LOG_MAKEPRI(fac, 0)); //[RK]SYSLOG-NG
#else
	if (mainconf.syslog) setlogremote(mainconf.syslog, 1);
	if (ppp != 1)
	{
		fac = LOG_FAC(LOG_LOCAL0) + mainconf.facility;
		snprintf(ident, sizeof (ident), "port[S%d]", (port < 0) ? 9999 : port);
		nopenlog(ident, LOG_CONS|LOG_NDELAY, LOG_MAKEPRI(fac, 0));
	}
#endif

#ifndef CYCLADES
	if (port >= 0 && lineconf[port].debug == 0) {
		if (oport == -2) /* PPP daemon */
			nsetlogmask(LOG_UPTO(LOG_NOTICE));
		else
			nsetlogmask(LOG_UPTO(LOG_INFO));
	}
#endif

	/*
	 *	Fill in the auth struct for now.
	 */

	memset(ai, 0, sizeof(struct auth));
	strcpy(ai->login, "NONE");
	ai->nasport  = lineconf[port].this;
	ai->proto    = lineconf[port].protocol;
	ai->address  = lineconf[port].ipno;
	ai->host     = lineconf[port].host;
	ai->netmask  = lineconf[port].netmask;
	ai->mtu      = lineconf[port].mtu;
	ai->mru      = lineconf[port].mru;
	ai->porttype = lineconf[port].porttype;
	ai->localip  = mainconf.ipno;

	if (ai->netmask == 0) ai->netmask = 0xFFFFFFFF;

	if (ppp && ai->proto != P_PPP && ai->proto == P_PPP_ONLY) {
		ai->proto = P_AUTOPPP;
	}

	return port;
}


/*
 * get the first port free from the pool,
 * lc points to the first port of the pool.
 */

static int get_port_free(struct line_cfg *lc)
{
	struct line_cfg *lp = lc;
	int i = lc->this;

	for( ; i < MAXVLINES; i++, lp++) {
		if (!lp->tty)
			continue;
		if (lp->protocol != lc->protocol)
			continue;
		if ((!lp->pool_serverfarm && lc->pool_serverfarm) ||
			(lp->pool_serverfarm && !lc->pool_serverfarm))
			continue;
		if (lc->pool_serverfarm &&
			strcmp(lp->pool_serverfarm, lc->pool_serverfarm))
			continue;
		if (lp->pool_socket_port != lc->pool_socket_port)
			continue;
		if (lp->pool_ipno != lc->pool_ipno)
			continue;
		if (count_sessions(lp->this))
			continue;		/* port already in use */
		if (!cy_shm->vet_pool[lp->this]) {
			cy_shm->vet_pool[lp->this] = getpid();
			return(lp->this);
		}
	}

	return(-2);
}

/*
 *	Get the port number belonging to this tty
 *	(the pppd uses this).
 */
static int rad_portno(char *tty, int port, int fproto)
{
	int i;

	if (port == -2 || port == -6)
		if (strncmp(tty, "/dev/", 5) == 0) tty += 5;

	for(i = 0; i < MAXVLINES; i++) {
		if (!lineconf[i].tty) continue;

		if (fproto == 1) {
			//[RK]June/03/04 - SSH/Telnet to IPDU
			if ((lineconf[i].protocol == P_IPDU) && 
			    (lineconf[i].pmsessions & PMSESS_TELNET)) {
			} else {	
				if (lineconf[i].protocol != P_SOCKET_SERVER &&
				    lineconf[i].protocol != P_SOCKET_SERVER_RAW)
					continue;
			}
		} else if (fproto == 2) {
			//[RK]June/03/04 - SSH/Telnet to IPDU
			if ((lineconf[i].protocol == P_IPDU) && 
			    (lineconf[i].pmsessions & PMSESS_SSH)) {
			} else {	
				if (lineconf[i].protocol != P_SOCKET_SSH) {
					//[RK]Dez/23/04 Bug 2906 - Fixed problem with invalid ipno
					if ((port == -4) && (lineconf[i].ipno == (int)tty)) {
						return -3;
					}
					continue;
				}
			}
		}

		switch(port) {
		case -5:
			if (lineconf[i].serverfarm &&
			    !strcmp(lineconf[i].serverfarm, tty))
				return i;
			if (lineconf[i].pool_serverfarm &&
			    !strcmp(lineconf[i].pool_serverfarm, tty))
				return (get_port_free(&lineconf[i]));
			break;
		case -3:
			if (lineconf[i].socket_port == (int)tty)
				return i;
			if (lineconf[i].pool_socket_port == (int)tty)
				return (get_port_free(&lineconf[i]));
			break;
		case -4:
			if (lineconf[i].ipno == (int)tty)
				return i;
			if (lineconf[i].pool_ipno == (int)tty)
				return (get_port_free(&lineconf[i]));
			break;
		case -2:
		case -6:
			if (strcmp(lineconf[i].tty, tty) == 0)
				return i;
			break;
		}
	}
	return -1;
}

