/*
 *  Module:  modem-replace.h (DCE-like behavior header file)
 *
 *  Author:  Marcelo Peccin <marcelo.peccin@cyclades.com>
 *
 *  Copyright:  (c) 2003 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  July 22, 2003
 *
 *  Comments:  The first release of this module is intended to solve the
 *             ISDN Modem substitution using TS family for NSW RTA (Australia).
 *             The project involves video cameras that communicate to 
 *             another equipment in a remote central office.
 */

#ifndef MODEM_REPLACE_H
#define MODEM_REPLACE_H


/**************************************************/
/**********     Constant Definitions     **********/
/**************************************************/
#undef  FALSE
#define FALSE      0
#undef  TRUE
#define TRUE    (!FALSE)
#undef  NULL
#define NULL   ((void*)0)

#define ERROR     (-1)
#define ERRCLOSE  (-2)  
#define OK          0
#define PROBLEM     1

#define PARITY_NONE  1
#define PARITY_ODD   2
#define PARITY_EVEN  3

#define FLOW_NONE  0
#define FLOW_HARD  1
#define FLOW_SOFT  2

#define MODEMREP_NO_EVENT  0
#define SER_CONN_REQUEST   1
#define SOCK_CONN_ACCEPT   2

#define NO_SOCK        (-1)
#define MODEMREP_PORT  7999

#define MAX_CMD_LENGTH  80
#define SBUFFSIZE      1024
#define TCPBUFFSIZE    1024
#define SBUFFLIMIT    (SBUFFSIZE - 24)

#define CHAR_CR  13


/**********************************************/
/**********     Type Definitions     **********/
/**********************************************/
typedef unsigned char  uchar;

typedef struct
{
	int    port;
	int    speed;
	int    datasize;
	int    parity;
	int    stopbits;
	int    flowctrl;
	int    media;
	uchar  dev[15];
}
sconf;

typedef struct
{
	sconf  serial;
	int    socket_ip;
	int    tx_interval;
}
modemrepconf;


/*************************************************/
/**********     Function Prototypes     **********/
/*************************************************/
static void  mpfprint(const char *fmt, ...);
static int   config_port(int fd, sconf *pconf);
static int   ssend(int fds, uchar *buff, int tosend);
static int   tcpsend();


/***********************************************/
/**********     Macro Definitions     **********/
/***********************************************/
#undef Max
#define Max(x,y)  (((x) >= (y)) ? (x) : (y))


#endif /* MODEM_REPLACE_H */

/* End of modem-replace.h */
