/*
 * Module: pmc.c
 *
 * Copyright:	(c) 1999-2002 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * $Log: pmc.c,v $
 * Revision 1.18  2006/06/08 13:58:56  regina
 * Port from ACS - event notification, pmd
 *
 * Revision 1.17  2004/10/21 21:36:25  regina
 * The command interval can have 2 outlets
 *
 * Revision 1.16  2004/10/20 12:02:28  sulamita
 * fix bug 2752
 *
 * Revision 1.15  2004/08/17 11:54:45  regina
 * Implemented PMD Syslog Messages
 *
 * Revision 1.16  2004/03/05 19:06:31  regina
 * Implemented access control by pmd
 *
 * Revision 1.15  2004/01/15 16:07:24  fujima
 * bug #1181 fixed
 *
 * Revision 1.14  2003/12/17 02:12:06  erik
 * Addition of new hotkey menu for IPMI feature
 *
 * Revision 1.13  2003/10/10 01:12:48  mangesh
 * Fix for Bug 946.
 *
 * Revision 1.12  2003/08/01 19:28:23  henrique
 * Please see bug 692
 *
 * Revision 1.11  2003/07/07 16:17:44  fujima
 * Bug #676/677 fixed
 *
 * Revision 1.10  2003/06/10 22:21:52  fujima
 * Bug #585/586 fixed
 *
 * Revision 1.9  2003/04/14 12:34:06  fujima
 * Bug #441 fixed. The size of the answerBuf in cy_shm.h was increased. Besides, the pmc_outlet is a number type and the argument is passed in the pmc_cmd parameter with the command
 *
 * Revision 1.8  2003/03/24 13:12:54  fujima
 * Restored bug fix 382 (backspace treatment)
 *
 * Revision 1.7  2003/03/20 21:03:51  fujima
 * Bug #354 and 381 fixed
 *
 * Revision 1.5  2003/03/14 22:15:18  fujima
 * Bug fixed - the PM session didn't recognize the 2nd, 3rd... outlets when they were entered from the Others option
 *
 * Revision 1.4  2003/03/14 19:03:45  fujima
 * Bug #354 fixed
 *
 * Revision 1.3  2003/02/25 17:14:59  fujima
 * Allows treatment for outlets with different IPDU types
 *
 * Revision 1.2  2003/02/21 19:55:57  fujima
 * Changed menu layout for PM
 *
 * Revision 1.1  2003/02/21 16:30:14  fujima
 * New PMD
 *
 * Revision 1.1  2003/02/20 20:47:11  fujima
 * New PMD
 *
 *
*/

// Linux headers
# include <stdio.h>
# include <stdlib.h>
# include <signal.h>
# include <string.h>
# include <unistd.h>
# include <ctype.h>
# include <linux/shm.h>
# include <linux/sem.h>
# include <sys/types.h>
# include <grp.h>

// Cyclades Headers
# include "server.h"	// Global definitions for cy_buffering shared memory

# define PMDCLIENT
# include "pmc.h"
#include <ipdu.h>

#define ADMIN_GROUP		"admin"
#define ADMIN_USER		"admin"

#define XDEBUG

int no_auth_req;	//[SB]Apr/15/05 - parameter pmusers == all || 
			// member of group admin 

void (*pmc_write)(char *line,int cnt);

static void pmc_error_write(char *error, int cnt) {
	pmc_write(pmcerrorbegin, cnt);
	pmc_write(error, cnt);
	pmc_write(pmcerrorend, cnt);
}

/*****************************************************************
 * This routines adds the outlet outNumber to outset.
 ****************************************************************/
void addOutlet (char * outset, int outNumber) 
{
	int i;

	for (i=0; i < PMC_OUTLETS; i++) {
		if (!outset[i]) {
			outset[i] = outNumber;
			return;
		}
	}
}

/*****************************************************************
 * This routines checks if the user is part of the admin group,
 * giving them acess to all ports.  If so, they do not need to
 * be authenticated by pmd.
 ****************************************************************/
int user_in_admin_group(char * username)
{
        struct group *grpa;
        char *pname,**pmem; 
        if ((grpa = getgrnam(ADMIN_GROUP)) == NULL) {
                return 0;
        }
	
        pmem = grpa->gr_mem;
        while((pname = *pmem++)) {
                if (!strcmp(pname, username))
                        return 1;
        }
        return 0;
}

/****************************************************************** 
 * This routine allocs memory to struct pmcinfo. Register this
 * structure on the vector cy_shm->pmc_list and fill this structure
 * with the proper values. It returns a NULL in case anything goes
 * wrong.
 ******************************************************************/
static pmcInfo* pmc_alloc_pmcinfo(CY_SESS * sess, int ipdu, int outlet,char other) {
	int idx;
	pmcInfo * pmc,*old;
	char *buf;
	
	// lock access to shm buffers list
	shm_signal(CY_SEM_SHM);

	// Find a free position on the vector cy_shm->pmc_list
	// If no position is found return error
	for (pmc = sess->pmc; pmc != NULL; pmc = pmc->next) {
		if (pmc->pmc_ipdu == ipdu) {
			addOutlet(pmc->pmc_outlet, outlet);
			shm_unsignal(CY_SEM_SHM);
			return pmc;
		}
	}
	for (idx=0; idx < (MAXPMDACCESS); idx++) {
		if (!cy_shm->pmc_list[idx]) {
			break;
		}
	}
	if (idx == (MAXPMDACCESS)) {
		shm_unsignal(CY_SEM_SHM);
		return NULL;
	}

	// Try to allocate memory. Return error if it fails
	if (!(pmc = (pmcInfo *) shm_malloc(sizeof(pmcInfo), TYPE_SHM_STR))){
		shm_unsignal(CY_SEM_SHM);
		return NULL;
	}
	
	buf = shm_malloc(200,TYPE_SHM_STR);
	// Fill the structure
	pmc->idx_pmclist = idx;
	pmc->pmc_pid = getpid();
	pmc->pmc_ipdu = ipdu;
 	pmc->pmc_status = PMC_IDLE;
	pmc->pmc_answerBuf = buf;
	pmc->pmc_sizeAnswerBuf = 200;
	addOutlet(pmc->pmc_outlet, outlet);
	shm_unsignal(CY_SEM_SHM);
	
	if (sess->pmc) {
		for (old = sess->pmc; old->next != NULL; old = old->next);
		old->next = pmc;
	} else {
		sess->pmc = pmc;
	}
	pmc->next = 0;
	cy_shm->pmc_list[idx] = pmc;	
	
	no_auth_req = 0;	//[RK]Aug/09/04 
	strcpy(pmc->pmc_username,sess->tty_user);  
	if (!other) {
		if (cy_shm->line_conf[sess->port].pmusers && *cy_shm->line_conf[sess->port].pmusers) { 
			if (!strcmp(cy_shm->line_conf[sess->port].pmusers,"all")) {
				no_auth_req = PMD_NOAUTH;
			}
		}
	}
	
	if (user_in_admin_group(pmc->pmc_username)) {
		no_auth_req = PMD_NOAUTH;
	}
	return pmc;
}


/****************************************************************** 
 * This routine frees the struct pmcinfo and remove this structure
 * from the vector cy_shm->pmc_list.
 ******************************************************************/ 
static void pmc_free_pmcinfo(pmcInfo * pmc) {
	if (!pmc) return;
	// free register in pmclist
	cy_shm->pmc_list[pmc->idx_pmclist] = 0; 
	shm_signal(CY_SEM_SHM);
	if (pmc->pmc_answerBuf)
		shm_free_str(pmc->pmc_answerBuf);
	shm_free_str((char *)pmc);
	shm_unsignal(CY_SEM_SHM);
}
/*
 * This routine treats the logout of the PM session
 */
void pmc_logout(CY_SESS *sess) {
	pmcInfo *pmc = sess->pmc,*paux;
	while (pmc) {
		paux = pmc;
		pmc = pmc->next;
		pmc_free_pmcinfo(paux);
	}
	sess->pmc = 0;	
	sess->sess_flag &= ~(CY_SESS_FLAG_PMD);
}

/*****************************************************************
 * This routine is called whenever a user presses pmhotkey on a
 * telnet/ssh session. It receives as parameter:
 *   -  *sess: pointer to the user session;
 *   -  *pmoutlet: the pmoutlet variable for this server
 *   -  other: ???
 *   -  write_line: pointer to the routine that sends data
 *                  to the user;
 * This routine will:
 *  0 - Set the global variable pmc_write
 *  1 - Check if the server is ready
 *  2 - Check if there is another user accessing the port
 *  3 - While there is a valid pair ipdu.outlet
 *    3.2 - Try to allocate a structure to store information
 *          about the this ipdu.outlet.
 *  4 - 
 *    4.1 - Create a menu for this user and store it at
 *          sess->pmc->pmc_menu. This procedure is done by 
 *          pmc_mount_menu
 *    4.2 - Send menu to the user
 *    4.3 - Set flag CY_SESS_FLAG_PMD on sess->sess_flag
 ****************************************************************/
void pmc_login(CY_SESS * sess, char *pmoutlet, char other, void write_line(char * line,int cnt)) 
{
	int ipdu=-1, outlet;
	pmcInfo *pmc;
	char *paux;

	// 0 - The the pointer to the routine that sends that to the user
	pmc_write = write_line;
	
	// 1 - Checking if server is ready
	if (!cy_shm->PMD_pid) {
		pmc_error_write(servernotready,0);
		return;
	}

	// 2 - Checking if there is another session (??)
	if (cy_shm->sess_list[sess->port] != sess) {
		pmc_error_write(anothersession,0);
		return;
	}

	// 3 - The loop through all pairs ipdu.outlet on the variable
	//     pmoutlet.
	if (pmoutlet && *pmoutlet) {
		paux = pmoutlet;
		while (*paux) {
			// Getting the ipdu number and testing if
			// ipdu is a valid number
			ipdu = strtol(paux, &paux, 10);
			// Check if ipdu is a valid number
			if ( ipdu >= MAXLINES ) {
				unsigned char message[64];
				snprintf(message, 64, "The ipdu number %d is not valid.\n"
					 "This is your pmoutlet variable: %s", ipdu + 1, pmoutlet);
				pmc_error_write(message, 0);
				pmc_logout(sess);
				ipdu = -1;
				break;
			}

			// Check if the serial port is configured as IPDU
			if (cy_shm->line_conf[ipdu - 1].protocol != P_IPDU) {
				unsigned char message[64];
				snprintf(message, 64, "The serial port #%d is not configured as IPDU.", ipdu + 1);
				pmc_error_write(message, 0);
				pmc_logout(sess);
				ipdu = -1;
				break;
			}

			// Check if the next char is a dot
			if ((*paux != '.')) {
				unsigned char message[64];
				snprintf(message, 64, "The outlet list has a wrong syntax.\n"
					 "Expected '.' not found.\n"
					 "This is your pmoutlet variable: %s", pmoutlet);
				pmc_error_write(message, 0);
				pmc_logout(sess);
				ipdu = -1;
				break;
			}

			// Getting outlet number
			paux++;
			outlet = strtol(paux, &paux, 10);
			ipdu--;
			// Checking outlet number
			if (outlet == 0 || outlet > cy_shm->PMD_outlets[ipdu]) {
				unsigned char message[64];
				snprintf(message, 64, "Outlet %d.%d is out of range.", ipdu + 1, outlet);
				pmc_error_write(message, 0);
				pmc_logout(sess);
				ipdu = -1;
				break;
			}
			// 3.2 - Trying to allocate pmcinfo for this ipdu.outlet
			if (!(pmc = pmc_alloc_pmcinfo(sess, ipdu,outlet,other))){
				pmc_logout(sess);
				pmc_write(internalerror,0);
				return;
			}
			// Skipping all spaces
			while (isspace(*paux)) paux++;
		}	
	}	
	if (sess->pmc) {
		// 4.1 - Mount menu
		pmc_mount_menu(sess, other);
		// 4.2 - Send menu
		pmc_write(sess->pmc->pmc_menu,0);
		// Update sess->sess_flag
		sess->sess_flag |= CY_SESS_FLAG_PMD;
		return;
	}
}

/**************************************************************************
 * This routine mount the PM session menu (union between all the IPDU 
 * used by this server). It returns the number of items on the resultant
 * menu and it save all menuItems on the position pmc_menu.
 *************************************************************************/
#define PMC_MENUMAX	18
static int pmc_mount_union_menu(CY_SESS * sess, menuItem *pmc_menu) {
	pmcInfo *pmc;
	int pmc_menumax = 0;
	ipduInfo2 *info;
	int i, j;
	for (pmc = sess->pmc; pmc != NULL; pmc = pmc->next) {
		info = (ipduInfo2 *) cy_shm->sess_list[pmc->pmc_ipdu];
		for (i = 0; i < info->menuMax; i++) {
			for (j = 0; j < pmc_menumax; j ++) {
				if (strcmp(pmc_menu[j].label, 
					info->menuItems[i].label) == 0) {
					break;
				}
			}
			if (j == pmc_menumax) {
				pmc_menu[pmc_menumax++] = info->menuItems[i];
				if (pmc_menumax == PMC_MENUMAX) {
					break;
				}
			}
		}
		if (pmc_menumax == PMC_MENUMAX) {
			break;
		}
	}
	return pmc_menumax;
}

/**************************************************************************
 * This routine mount the PM session menu and save it on 
 * sess->pmc->pmc_menu.
 **************************************************************************/
static void pmc_mount_menu(CY_SESS * sess, char other) {
	pmcInfo *pmc = sess->pmc;
	char *menu, *label, out[10];
	int idx, i;
	menuItem pmc_menu[PMC_MENUMAX];
	int pmc_menumax = pmc_mount_union_menu(sess, pmc_menu);

	menu = pmc->pmc_menu;

	idx = 0;
	idx += sprintf(&menu[idx], "\r\n%s%s", menuHeaderLine, menuHeader);
	
	if (other) {
		while (1) {
			for (i=0;pmc->pmc_outlet[i];i++) {
				if (i == 0) {
					sprintf(out,"%d",pmc->pmc_outlet[i]);
				} else {
					sprintf(out,",%d",pmc->pmc_outlet[i]);
				}
				strcat(pmc->pmc_cmd,out);
			}
			idx += sprintf(menu + idx, "                   IPDU %d Outlet %s\r\n", pmc->pmc_ipdu+1, pmc->pmc_cmd);
			if (! pmc->next) {
				break;
			}
			pmc = pmc->next;
		}
	}
	idx += sprintf(&menu[idx], menuHeaderLine);
	// Inserting the commands
	for (i = 0; i < pmc_menumax + 3; i++) {
		if (i == 0) {
			label = "Exit";
		} else if (i == 1) {
			label = "Help";
		} else if (i == pmc_menumax + 2){
			label = "Other";
		} else {
			label = pmc_menu[i-2].label;
		}
		if ((i%3) == 0) {
			idx += sprintf(&menu[idx], "       ");
		}
		idx += sprintf(&menu[idx]," %2d - %-8s", i + 1, label);
		if ((i%3) == 2) {
			idx += sprintf(&menu[idx], "\r\n");
		}
	}
	if ((i%3) != 0) {
		idx += sprintf(&menu[idx], "\r\n");
	}

#define MAX_OPT_SIZE	50
	memset(pmc->pmc_cmd, 0, MAX_OPT_SIZE);
	idx += sprintf(&menu[idx],"\r\n%s", chooseAnOption);
	return ;
}

/*
 * This routine treats the data from socket connection to PM session
 */
void pmc_treat_input(CY_SESS * sess, void write_data(char *data,int cnt)) {
	char echo_str[MAX_OPT_SIZE], aux = 0, *popt, *pecho;
	int  option,size;
	pmcInfo *pmc;

	if (sess->InBuffer.RdPos == sess->InBuffer.WrPos) {
		// there is nothing to read
		return;
	}
	
	if (!sess->pmc) { // not have PM session
		pmc_error_write(noserversess,0);
		sess->sess_flag &= ~(CY_SESS_FLAG_PMD);
		return;
	}
	
	pmc = sess->pmc;
	memset(echo_str,0,MAX_OPT_SIZE);
	size = strlen(pmc->pmc_cmd);
	popt = &pmc->pmc_cmd[size];
	pecho = echo_str;
	while (sess->InBuffer.RdPos != sess->InBuffer.WrPos) {
       		aux = sess->InBuffer.Buffer[sess->InBuffer.RdPos];
		sess->InBuffer.RdPos = (sess->InBuffer.RdPos + 1) % BufferSize;

		if ((aux == 0x08) || (aux == 0x7f)) {
			if (size) {
				popt--; // backspace
				*popt = 0x00;
				size --;
				*pecho++ = 0x08;
				*pecho++ = ' ';
				*pecho++ = 0x08;
			}
			continue;
		}
		if (aux == '\r') {
			char lix;
			*pecho++ = aux;
       			lix=sess->InBuffer.Buffer[sess->InBuffer.RdPos];
			if (lix == '\n') {
				sess->InBuffer.RdPos = 
					(sess->InBuffer.RdPos + 1) % BufferSize;
			} 
			*pecho++ = '\n';
			break;
		}
		if (aux == '\n') {
			*pecho++ = aux;
			break;
		}
		if (size < MAX_OPT_SIZE) {
			size++;
			*popt++ = aux;
			*pecho++ = aux;
		}
	}

	pmc_write(echo_str,strlen(echo_str));	// echoing the data

	if (((aux != '\n') && (aux != '\r')) ||
		(!strlen(pmc->pmc_cmd))) {
		return; // wait the ended line char
	}
	
	while (pmc) {
		if (pmc->pmc_status != PMC_IDLE) {
			return;
		}
		pmc = pmc->next;
	}

	if (!cy_shm->PMD_pid) { // server not ready
		pmc_error_write(servernotready,0);
		return;
	}
	pmc = sess->pmc;
	popt = pmc->pmc_cmd;
	if (pmc->pmc_ipdu == -1) { // enter outlet
		unsigned char outletbuf[PMC_CMDSIZE];

		if (*popt == 'h') { // help line
			pmc_write(outletHelp,0);
			pmc_write(outletquestion,0);
			*popt = 0x00;
			return;
		}
		if (*popt == 'c') { // return to menu
			pmc_logout(sess);
			pmc_login(sess, cy_shm->line_conf[sess->port].pmoutlet,
				  0, pmc_write);
			return;
		}
		if (*popt == 'l') { // logout
			pmc_logout(sess);
			pmc_write(endpmsession,0);
			return;
		}
		// enter outlet list
		strncpy(outletbuf, popt, PMC_CMDSIZE);
		pmc_logout(sess);
		pmc_login(sess, outletbuf, 1, pmc_write);
		return;
	}
	option = strtol(pmc->pmc_cmd,&popt,10); // convert option
	memset(pmc->pmc_cmd, 0, MAX_OPT_SIZE);
	if (popt == pmc->pmc_cmd) {
		pmc_write(invalidOption,0);
		pmc_write(sess->pmc->pmc_menu,0);
		return;
	}
		
	pmc_treat_option(sess, option, pmc_write);		
}

/*
 * This routine sends commands to PM server
 */
static void pmc_treat_option(CY_SESS * sess, int option, void write_data(char *data,int cnt)) {
	pmcInfo *pmc, *paux;
	ipduInfo2 *info;
	int i,j;
	char buffer[96];
	menuItem pmc_menu[PMC_MENUMAX];
	int pmc_menumax = pmc_mount_union_menu(sess, pmc_menu);

	pmc_write = write_data;

	// is option valid
	if (option == 0 || option > pmc_menumax + 3) {
		pmc_write(invalidOption,0);
		pmc_write(sess->pmc->pmc_menu,0);
		return;
	}
	if (option == 1) { // logout
		pmc_logout(sess);
		pmc_write(endpmsession,0);
		return;
	}
	if (option == 2) { // help
		info = (ipduInfo2 *) cy_shm->sess_list[sess->pmc->pmc_ipdu];
		pmc_write("\r\n", 0);
		snprintf(buffer, 96, "%-16s - %s\r\n", 
			"Exit", "Exits the Power Management Session");
		pmc_write(buffer,0);
		snprintf(buffer, 96, "%-16s - %s\r\n", 
			"Help", "Shows this message");
		pmc_write(buffer,0);
		for (i = 0; i < pmc_menumax; i++) {
			snprintf(buffer, 96, "%-16s - %s\r\n", 
				pmc_menu[i].label, pmc_menu[i].help);
			pmc_write(buffer,0);
		}
		snprintf(buffer, 96, "%-16s - %s\r\n", 
			"Other", "Allows the user to manage other outlets");
		pmc_write(buffer,0);
		pmc_write(sess->pmc->pmc_menu,0);
		return;
	}
	// is OTHER
	if (option == pmc_menumax + 3) { // OTHER option
		pmc = sess->pmc->next;
		while (pmc != NULL) {
			paux = pmc->next;
			pmc_free_pmcinfo(pmc);
			pmc = paux;
		}
		pmc = sess->pmc;
		pmc->next = 0;
		pmc->pmc_ipdu = -1;
		pmc->pmc_status = PMC_IDLE;
		pmc_write(outletquestion,0);
		return;
	}

	// Look for the command	
	for (pmc = sess->pmc; pmc != NULL; pmc = pmc->next) {
		info = (ipduInfo2 *) cy_shm->sess_list[pmc->pmc_ipdu];
		for (i = 0; i < info->menuMax; i++) {
			if (strcmp(pmc_menu[option - 3].label, info->menuItems[i].label) == 0) {
				strcpy(pmc->pmc_cmd, info->menuItems[i].command);
				if (info->menuItems[i].args[0] == ARG_OUTLET) {
					//[RK]Oct/21/05 - the logic in PMD was changed
					sprintf(pmc->pmc_cmd + strlen(pmc->pmc_cmd)," %d", pmc->pmc_outlet[0]);
					for (j=1; pmc->pmc_outlet[j] ; j++) {
						sprintf(pmc->pmc_cmd + strlen(pmc->pmc_cmd),",%d", pmc->pmc_outlet[j]);
					}
				}
				pmc->pmc_status = PMC_REQUEST + no_auth_req; //[RK]Aug/09/04
				break;
			}
		}
		if (i == info->menuMax) {
			char buffer[128];
			snprintf(buffer, 128, "\r\nIPDU %d \r\n" 
				"Command [%s] is not available for this IPDU."
				"\r\n\n", pmc->pmc_ipdu + 1, pmc_menu[option - 3].label);
			/* Command not available for this IPDU */
			pmc_write(buffer, 0);
		}
	}
	kill(cy_shm->PMD_pid, SIGIO);	
}

/*
 * This routine treats the SIGUSR2 signal 
 */
void pmc_treat_sigusr2(CY_SESS * sess, void write_data(char*data,int cnt))
{
	pmcInfo *pmc;
	int done = TRUE;
	char out[10];
	int i;
	
	pmc_write = write_data;
	
	pmc = sess->pmc;
	if (!pmc) {
		// the PM session was ended
		pmc_logout(sess);
		return;
	}

	for (; pmc != NULL; pmc = pmc->next) {
		if (pmc->pmc_status == PMC_IDLE) {
			continue;
		} else if (pmc->pmc_status == PMC_ANSOK) {
			char buffer[128];
                        int buf_len = 125;
			sprintf(buffer, "\r\nIPDU %d Outlet ", pmc->pmc_ipdu + 1);
			buf_len -= strlen(buffer);
			for (i=0; pmc->pmc_outlet[i]; i++) {
				if (i==0) {
					sprintf(out,"%d",pmc->pmc_outlet[i]);
				} else {
					sprintf(out,",%d",pmc->pmc_outlet[i]);
				}
				buf_len -= strlen(out);

				if (buf_len > 0 ) {	
					strcat(buffer,out);
				}
			}
			strcat(buffer,"\r\n");
			pmc_write(buffer, 0);
			pmc->pmc_status = PMC_IDLE;
			pmc_write(pmc->pmc_answerBuf, 0);
			memset(pmc->pmc_answerBuf, 0, pmc->pmc_sizeAnswerBuf);
			memset(pmc->pmc_cmd, 0, PMC_CMDSIZE);
		} else {
			done = FALSE;
		}
	}
	
	if (done) {
		pmc_write(sess->pmc->pmc_menu, 0);

		if (sess->InBuffer.RdPos != sess->InBuffer.WrPos){
			pmc_treat_input(sess, pmc_write);
		}
	}
}

 
