/*
	socket: RFC 2217 compliant serial port redirector
	Version 1.1.7, 20 September 1999
	Copyright (C) 1999 InfoTecna di Cesana D. & C. s.n.c. 

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

	To contact the autor:

	 Denis Sbragion
	 InfoTecna
	 Tel, Fax: +39 039 2324054
	 URL: http://infotecna.home.dhs.org
	 E-Mail: sbragion@email.com

*/

/* Return NoError, which is 0, on success */

/* Standard library includes */
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>
#include <pwd.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <termio.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>

#define XDEBUG
#include "server.h"
#include "pmc.h"

//#define DEBUG_CNT

#ifdef DEBUG_CNT
int  tot_read=0, tot_write=0, tty_read=0, tty_write=0, tty_bwrite=0;
#endif

extern int DeviceFd;
extern struct line_cfg		*pconf;
struct auth 		*pai;

extern struct termios		PortSettings;

/* Version id */
#define SocketVersion "Version 1.0.0, Oct/05/2000"

/* Locking constants */
#define LockOk 0
#define Locked 1
#define LockKo 2

/* Error conditions constants */
#define NoError 0
#define OpenError -1

/* Maximum length of temporary strings */
#define TmpStrLen 255

/* Output buffer size */
#define OutBufferSize 4096	// was 512 - performance improv.
//#define OutBufferSize BufferSize

#define MIN(x,y) ((x) > (y) ? (y) : (x))
#define MAX(x,y) ((x) < (y) ? (y) : (x))

#define ZMIN(x,y) (!(x) ? (y) : (!(y) ? (x) : MIN(x,y)))

/* Base Telnet protocol constants (STD 8) */
#define TNSE ((unsigned char) 240)
#define TNNOP ((unsigned char) 241)
#define TNDM ((unsigned char) 242)
#define TNEL ((unsigned char) 248)
#define TNEC ((unsigned char) 247)
#define TNAYT ((unsigned char) 246)
#define TNAO ((unsigned char) 245)
#define TNIP ((unsigned char) 244)
#define TNEOR ((unsigned char) 239)
#define TNBREAK ((unsigned char) 243)
#define TNSB ((unsigned char) 250)
#define TNWILL ((unsigned char) 251)
#define TNWONT ((unsigned char) 252)
#define TNDO ((unsigned char) 253)
#define TNDONT ((unsigned char) 254)
#define TNIAC ((unsigned char) 255)

/* Base Telnet protocol options constants (STD 27, STD 28, STD 29) */
#define TN_TRANSMIT_BINARY ((unsigned char) 0)
#define TN_ECHO ((unsigned char) 1)
#define TN_SUPPRESS_GO_AHEAD ((unsigned char) 3)
#define TN_LINEMODE ((unsigned char) 34)

/* Base Telnet Com Port Control (CPC) protocol constants (RFC 2217) */
#define TNCOM_PORT_OPTION ((unsigned char) 44)

/* CPC Client to Access Server constants */
#define TNCAS_SIGNATURE ((unsigned char) 0)
#define TNCAS_SET_BAUDRATE ((unsigned char) 1)
#define TNCAS_SET_DATASIZE ((unsigned char) 2)
#define TNCAS_SET_PARITY ((unsigned char) 3)
#define TNCAS_SET_STOPSIZE ((unsigned char) 4)
#define TNCAS_SET_CONTROL ((unsigned char) 5)
#define TNCAS_NOTIFY_LINESTATE ((unsigned char) 6)
#define TNCAS_NOTIFY_MODEMSTATE ((unsigned char) 7)
#define TNCAS_FLOWCONTROL_SUSPEND ((unsigned char) 8)
#define TNCAS_FLOWCONTROL_RESUME ((unsigned char) 9)
#define TNCAS_SET_LINESTATE_MASK ((unsigned char) 10)
#define TNCAS_SET_MODEMSTATE_MASK ((unsigned char) 11)
#define TNCAS_PURGE_DATA ((unsigned char) 12)
#define TNCAS_SET_MEDIA ((unsigned char) 20)

/* CPC Access Server to Client constants */
#define TNASC_SIGNATURE ((unsigned char) 100)
#define TNASC_SET_BAUDRATE ((unsigned char) 101)
#define TNASC_SET_DATASIZE ((unsigned char) 102)
#define TNASC_SET_PARITY ((unsigned char) 103)
#define TNASC_SET_STOPSIZE ((unsigned char) 104)
#define TNASC_SET_CONTROL ((unsigned char) 105)
#define TNASC_NOTIFY_LINESTATE ((unsigned char) 106)
#define TNASC_NOTIFY_MODEMSTATE ((unsigned char) 107)
#define TNASC_FLOWCONTROL_SUSPEND ((unsigned char) 108)
#define TNASC_FLOWCONTROL_RESUME ((unsigned char) 109)
#define TNASC_SET_LINESTATE_MASK ((unsigned char) 110)
#define TNASC_SET_MODEMSTATE_MASK ((unsigned char) 111)
#define TNASC_PURGE_DATA ((unsigned char) 112)
#define TNASC_SET_MEDIA ((unsigned char) 120)


/* Modem state effective change mask */
#define ModemStateECMask ((unsigned char) 240)

/* Default modem state polling in milliseconds (1000 msec should be enough) */
//[RK] in rwconf #define ModemStatePolling 1000

/* Input buffer */
BufferType InBuffer;
BufferType *pInBuffer = NULL;

/* Output buffer */
unsigned char OutBuffer[OutBufferSize];

/* Break Sequence buffer and flags */
static char seq_pending[32];		//buffer holding break sequence string
static int seqPendingIndx = 0;	//sequence pending index
static int seq = 0;					//break sequence flag 
static int line_cleared = 0;		//line cleared signaling a new prompt

/* Session Pointer */
CY_SESS *this_sess = NULL;

/* Syslog buffer */
extern LOG_DATA_BUFFER log_buffer;

/* Output buffer write position */
unsigned int OutBPos = 0;

static unsigned char char_cmd_int[10] =
	{TNIAC,TNSB,TNCOM_PORT_OPTION, 0, 0, 0, 0, 0, TNIAC,TNSE};
static unsigned char char_cmd_byte[7] =
	{TNIAC,TNSB,TNCOM_PORT_OPTION, 0, 0, TNIAC,TNSE};
static unsigned char char_cmd[6] =
	{TNIAC,TNSB,TNCOM_PORT_OPTION, 0, TNIAC,TNSE};

/* Com Port Control enabled flag */
Boolean TCPCEnabled = False;

/* Status enumeration for IAC escaping and interpretation */
typedef enum { IACNormal, IACReceived, IACComReceiving } IACState;

/* Effective status for IAC escaping and interpretation */
static IACState IACEscape = IACNormal;

/* Same as above during signature reception */
static IACState IACSigEscape;

/* Current IAC command begin received */
static unsigned char IACCommand[TmpStrLen];

/* Position of insertion into IACCommand[] */
static size_t IACPos;

/* Modem state mask set by the client */
unsigned char ModemStateMask = ((unsigned char) 255);

/* Line state mask set by the client */
static unsigned char LineStateMask = ((unsigned char) 0);

/* Current status of the modem control lines */
unsigned char ModemState = ((unsigned char) 0);
unsigned char FirstModemStateSent = ((unsigned char) 0);

/* Break state flag */
Boolean BreakSignaled = False;

/* Input flow control flag */
Boolean InputFlow = True;

/* These variables used only by EscWriteBuffer */
unsigned char Buf[2 * OutBufferSize];
int Bufwithbytes = 0;
int inputPend = 0;
static int IACCommandPend = 0;
struct TmpBuf *tmp = NULL;
struct TmpBuf tmp_buf;

/* Function prototypes */

static void check_break_sequence(int data_count);

/* Retrieves the port speed from PortFd */
static unsigned long int GetPortSpeed(int PortFd);

/* Retrieves the data size from PortFd */
static unsigned char GetPortDataSize(int PortFd);

/* Retrieves the parity settings from PortFd */
static unsigned char GetPortParity(int PortFd);

/* Retrieves the stop bits size from PortFd */
static unsigned char GetPortStopSize(int PortFd);
  
/* Retrieves the flow control status, including DTR and RTS status,
from PortFd */
static unsigned char GetPortFlowControl(int PortFd, unsigned char Which);

/* Return the status of the modem control lines (DCD, CTS, DSR, RNG) */
unsigned char GetModemState(int PortFd,unsigned char PMState);

/* Set the serial port data size */
static void SetPortDataSize(int PortFd, unsigned char DataSize);

/* Set the serial port parity */
static void SetPortParity(int PortFd, unsigned char Parity);

/* Set the serial port stop bits size */
static void SetPortStopSize(int PortFd, unsigned char StopSize);

/* Set the port flow control and DTR and RTS status */
static void SetPortFlowControl(int PortFd,unsigned char How);

/* Set the serial port speed */ 
static void SetPortSpeed(int PortFd, unsigned long BaudRate);

/* Send the signature Sig to the client */
static void SendSignature(int SockFd, char * Sig);

/* Write a char to SockFd performing IAC escaping */
static void EscWriteChar(int SockFd, unsigned char C);

/* Redirect char C to PortFd checking for IAC escape sequences */
void EscRedirectChar(int SockFd, int PortFd, BufferType *B);

/* Handle Telnet Commands */
int HandleChar(int SockFd, int PortFd, char C);

/* Send the specific telnet option to SockFd using Command as command */
static void SendTelnetOption(int SockFd, unsigned char Command, char Option);
  
/* Send a string to SockFd performing IAC escaping */
static void SendStr(int SockFd, char * Str);

/* Send the baud rate BR to SockFd */
static void SendCPCIntCommand(int SockFd, unsigned char Command, unsigned long int BR);

/* Send the flow control command Command */
static void SendCPCFlowCommand(int SockFd, unsigned char Command);

/* Send the CPC command Command using Parm as parameter */
void SendCPCByteCommand(int SockFd, unsigned char Command, unsigned char Parm);

/* Send a string to the client */
static void SendString(int SockFd, char *str, int len);

/* Handling of COM Port Control specific commands */
static void HandleCPCCommand(int SockFd, int PortFd, unsigned char * Command, size_t CSize);

/* Common telnet IAC commands handling */
static void HandleIACCommand(int SockFd, int PortFd, unsigned char * Command, size_t CSize);

/* The 'more' methods for a socket_server connection*/
int  read_a_char(int nonblock_flag);
void write_a_line(char *);
void write_data(char *, int);

static int got_int_flag = 0;	//[RK]Oct/30/02 BUG#73

/*
 * Cyclades - to check DCD signal 
 * ret 1 - DCD ON
 * ret 0 - DCD OFF
 */
static int cyc_check_dcd(int thisport)
{
	if (pconf->fast) {
		int signals;

		ioctl(DeviceFd,TIOCMGET,&signals);
		if (!(signals & TIOCM_CD) ) { // DCD is OFF
			return(0);
		} else {
			return(1);
		}
	} 
	if(!cy_shm->flag_dcd[thisport]) {
		cy_shm->flag_dcd[thisport] = 1;
		return(cy_shm->dcd_status[thisport]);
	}
	cy_shm->flag_dcd[thisport] = 1;
	return 1;
}

/* Retrieves the port speed from PortFd */
/* [RK]Mar/06/03
 * routines moved from rot_sess1.c
 */

static void send_msg_this_sess(char *buf, int cnt, CY_SESS *sess)
{
	int i;

	if (!sess || sess->vet_pid == 0) {
		return;
	}

	i = 0;
	shm_signal(sess->port);  /* semaphore number: 0 = first port */
	
	while (i < cnt) {
		if (IsBufferFull(&sess->OutBuffer, 2)) {
			sess->flow_flag = 1;
			break;
		}
		PutInBuffer(&sess->OutBuffer, buf[i]);
		i++;
	}
	
	shm_unsignal(sess->port);  /* semaphore number: 0 = first port */

	if (i < cnt)
		nsyslog(LOG_ERR,"Sess on ttyS%d busy cnt = %d i = %d", sess->port+1,cnt, i);
}

static int read_rw_msg(CY_SESS *sess, char *buf, int count)
{
	int cnt = 0;

	if (sess->sess_flag & 4) {
		return(0);
	}
 
	shm_signal(sess->port);  /* semaphore number: 0 = first port */

/*  	while (!IsBufferEmpty(&sess->OutBuffer) && (cnt < count)) { */
/*  		buf[cnt++] = GetFromBuffer(&sess->OutBuffer); */
/*  	} */
//mp: May, 2003 - Improve copy from Shared Memory
	if ((cnt = CharsInBuffer(&sess->OutBuffer))) {
		cnt = MIN(cnt, count);
		GetBlockFromBuffer(buf, &sess->OutBuffer, cnt);
	}

	shm_unsignal(sess->port);  /* semaphore number: 0 = first port */

	//[RK]May/26/03 - Performance Improvement
	if (sess->flow_flag && !IsBufferFull(&sess->OutBuffer, BufferSize/20)) {
		sess->flow_flag = 2;
	}

	return(cnt);
}

void write_a_line (char *line)
{
	EscWriteBuffer(STDOUT_FILENO, line, strlen(line), 0, -1, 1);
}

void write_data (char *p, int cnt)
{
	if (cnt) {
		EscWriteBuffer(STDOUT_FILENO, p, cnt, 0, -1, 1);
	} else {
		EscWriteBuffer(STDOUT_FILENO, p, strlen(p), 0, -1, 1);
        }
}
 
 void pmcommand_call(struct auth *pai)
 {
 	char *argv[6];
 	char cmd[20];
 	int i;
         uid_t	uid;
 	struct passwd *pwui;
 
 	pwui = getpwnam(pai->login);
 
 	if (pwui) {
 		uid = pwui->pw_uid;
 	} else {
 		exit(1);
 	}
 
 	//close the descriptors
 	for (i=3;i<64;i++)
 		close(i);
 
 	setuid(uid);
 
 	argv[0] = "/bin/pmCommand";
 	sprintf(cmd,"%d",pai->nasport+1);
 	argv[1] = strdup(cmd);
 	argv[2] = NULL;
 	
 	signal(SIGPIPE,SIG_DFL);
 
 	execve("/bin/pmCommand", argv,NULL);
 	exit(1);
 }
 
 /* Redirect char C to PortFd checking for IAC escape sequences */
 void sentPMandEcho(int SockFd, int PortFd, BufferType *B)
 {
 	unsigned char C=0, *pe;
 	int cnt;
 
 	if (IACCommandPend) {
 		HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
 	}
 	
 	cnt = 0;
 	pe = &OutBuffer[OutBPos];
 
 	while (B->RdPos != B->WrPos) {
 		C = GetFromBuffer(B);
 		if (!HandleChar(SockFd, PortFd, C)) {
 			continue;
 		}
 
 		if (!C) continue;
 
 		if ((C == 0x7f) || (C == '\b')) { // backspace
 			*pe++ = '\b';
 			*pe++ = ' ';
 			*pe++ = '\b';
 			cnt +=3;
 			if (tmp->cnt_buf) {
 				tmp->cnt_buf--;
 			}
 			tmp->tmp_buf[tmp->cnt_buf] = 0x00;
 			continue;
 		} 		
 		
 		if (C == '\r') {
 			C = GetFromBuffer(B);
 			C = tmp->tmp_buf[tmp->cnt_buf++] = '\n';
 			*pe++ = '\n';
 			cnt++;
 			break;
 		} else {
 			tmp->tmp_buf[tmp->cnt_buf++] = C;
 			*pe++ = C;
 			cnt++;
 		}
 
 #ifdef DEBUG_CNT
 		tty_bwrite++;
 #endif
 		if (tmp->cnt_buf >= sizeof(tmp->tmp_buf)) {
 			break;
 		}
 	}
 
 	if (cnt) {
 		OutBPos += cnt;
 		EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
 	}
 
 	if (C == '\n') {
 		write_pending_buffer(PortFd);
 	}
 }
 
static void got_int(int sig)
{
	switch (sig) {
		case SIGHUP :
		case SIGIO :
		case SIGALRM :
		case SIGUSR2 :
		case SIGUSR1 :
			got_int_flag = sig;
			break;
		case SIGINT :
		case SIGTERM :
		case SIGPIPE :
			syslog(LOG_NOTICE,"Receive Signal %d", sig);
			got_int_flag = sig;
			break;
		default :
			syslog(LOG_NOTICE,"Receive Signal %d", sig);
			got_int_flag = sig;
			break;
	}
}
 
//[RK]Sep/21/04 - Problem with telnet - the pmCommand does not free the pmcentry
void sock_free_pmcentry(int child)
{
	int i;
	pmcInfo *pmc;

	for (i=0; i < MAXPMDACCESS; i++) {
		pmc = cy_shm->pmc_list[i];
		if ((pmc != 0) &&
		    (pmc->pmc_pid == child)) {
			shm_signal(CY_SEM_SHM);
			cy_shm->pmc_list[i]=NULL;
			if (pmc->pmc_answerBuf)
				shm_free_str(pmc->pmc_answerBuf);
			shm_free_str((char *)pmc);
			shm_unsignal(CY_SEM_SHM);
		}
	}
}

 int main_routine_pmsocket(struct auth *pai)
 {
 	int pid_child;
 	int pin[2], pout[2],perr[2];
 	int fdin, fdout,fderr;
 	int max_fd, count, rtn, sec = 0;
	fd_set InFdSet;
 	fd_set FdExSet;
 	fd_set OutFdSet;
 	struct sigaction sa = {0};
 	/* Remote flow control flag */
 	int RemoteFlowOff = 0;
 
 	/* Allocate pipes for communicating with the program. */
 	if (pipe(pin) < 0 || pipe(pout) < 0 || pipe(perr) < 0) {
 		nsyslog(LOG_NOTICE,"Could not create pipes: %.100s",strerror(errno));
 	}
 	/* Fork the child. */
 	if ((pid_child = fork()) == 0) {
 		// Redirect stdin.  We close the parent side of the socket
 		// pair, and make the child side the standard input.
 		close(pin[1]);
 		if (dup2(pin[0], 0) < 0)
 			perror("dup2 stdin");
 		close(pin[0]);
 		/* Redirect stdout. */
 		close(pout[0]);
 		if (dup2(pout[1], 1) < 0)
 			perror("dup2 stdout");
 		close(pout[1]);
 		/* Redirect stderr. */
 		close(perr[0]);
 		if (dup2(perr[1], 2) < 0)
 			perror("dup2 stderr");
 		close(perr[1]);
 
 		tcflush(0,TCIOFLUSH);
 		tcflush(1,TCIOFLUSH);
 
 		// call pmCommand : execve
 		pmcommand_call(pai);
 	}
 
 	// Close the child sides of the pipes. 
 	close(pin[0]);
 	close(pout[1]);
 	close(perr[1]);
 	
 	fdout = pin[1];	// data to pmCommand
 	fdin = pout[0];	// data from pmCommand
 	fderr = perr[0];	// error 
 	
 	max_fd = MAX(STDIN_FILENO, STDOUT_FILENO);
 	max_fd = MAX(max_fd, fdin);
 	max_fd = MAX(max_fd, fdout);
 	max_fd = MAX(max_fd, fderr);
 
 	sa.sa_handler = got_int;
 	for (count=1; count < 32; count++) {
 		if (count == SIGSEGV || count == SIGWINCH) continue;
 		sigaction(count,  &sa, NULL);
 	}
 
 	pInBuffer = &InBuffer;
 	InitBuffer(pInBuffer);
 
 	OutBPos = 0;
 	Bufwithbytes = 0; // no more accumalated bytes in "local" buffer
 
 	tmp = &tmp_buf;
 	tmp_buf.ix_buf = 0;
 	tmp_buf.cnt_buf = 0;
 
 	FD_ZERO(&InFdSet);
 	FD_SET(STDIN_FILENO,&InFdSet);
 	FD_SET(fdin,&InFdSet);
 	FD_ZERO(&OutFdSet);
 	FD_ZERO(&FdExSet);
 	FD_SET(STDIN_FILENO,&FdExSet);
 	FD_SET(fderr,&FdExSet);
 
 	while (1) {
 		if ((rtn = select(max_fd+1,&InFdSet,&OutFdSet,&FdExSet,NULL)) <= 0) {
 			if (got_int_flag) {
 				nsyslog(LOG_NOTICE,"Receive Signal %d", got_int_flag);
 				got_int_flag = 0;
 			}
 			if (rtn) {
 				nsyslog(LOG_NOTICE,"Select error : %m");
 			}
 			break;
 		}
 		if (FD_ISSET(fdin,&InFdSet)) { // data from pmCommand
 			count = OutBufferSize-OutBPos;
 			count = read(fdin,&OutBuffer[OutBPos],count);
 			OutBuffer[count] = 0x00;
 			if (count > 0) {
 #ifdef DEBUG_CNT
 				tty_read += count;
 #endif
 				OutBPos += count;
 				EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
 			} else {
 				break;
 			}
 		}
 
 		if (FD_ISSET(STDIN_FILENO,&InFdSet)) { // data from the remote client
 			count = MaxContiguosFree(pInBuffer);
 			count = read(STDIN_FILENO,&pInBuffer->Buffer[pInBuffer->WrPos], count);
 			if (count <= 0) { // socket sess is stopped
 				nsyslog(LOG_NOTICE,"Communication closed (input)");
 				kill(pid_child,SIGKILL);
 				OutBPos = 0;
 				break;
 			}
 			pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
 			sentPMandEcho(STDOUT_FILENO,fdout,pInBuffer);
 		} else {
 			/* Check if the buffer is not full and remote flow is off */
 			if (TCPCEnabled && RemoteFlowOff &&
 				IsBufferFull(pInBuffer, BufferSize/2) == False) {
 				/* Send a flow control resume command */
 				SendCPCFlowCommand(STDOUT_FILENO,TNASC_FLOWCONTROL_RESUME);
 				RemoteFlowOff = 0;
 			}
 		}
 		
 		/* Resets the fd sets */
 		FD_ZERO(&InFdSet);	
 
 		/* Check if the buffer is not full */
 		if (IsBufferFull(pInBuffer, 1) == False) {
 			FD_SET(STDIN_FILENO,&InFdSet);
 		} else {
 			if (TCPCEnabled && !RemoteFlowOff) {
 				/* Send a flow control suspend command */
 				SendCPCFlowCommand(STDOUT_FILENO,TNASC_FLOWCONTROL_SUSPEND);
 				RemoteFlowOff = 1;
 			}
 		}
 
 		/* If input flow has been disabled from the remote client don't read from the device */
 		if (InputFlow == True) {
 			FD_SET(fdin,&InFdSet);
 		}
 
 		FD_ZERO(&FdExSet);
 		FD_SET(STDIN_FILENO,&FdExSet);
 	}
 	tcflush(fdin,TCIOFLUSH);
 	tcflush(fdout,TCIOFLUSH);
	kill(pid_child,SIGTERM);
	//[RK]Sep/21/04 - Problem with telnet - the pmCommand does not free the pmcentry
	sleep(2);
	sock_free_pmcentry(pid_child);
 	print_counters(0);
 	return(0);
}
 

int read_a_char (int nonblock_flag)
{
	char c;
	struct timeval BTimeout;
	int	max_fd;
	fd_set InFdSet;

	BTimeout.tv_sec = 0;
	BTimeout.tv_usec = 100 * 1000;
	max_fd = STDIN_FILENO + 1;
	FD_ZERO(&InFdSet);
	FD_SET(STDIN_FILENO,&InFdSet);

	while (1) {

		if (nonblock_flag) {
			if (select(max_fd,&InFdSet, NULL, NULL, &BTimeout) <= 0) {
				break;
			}
		}

		if (read(STDIN_FILENO,&c,1) == 1) {

#ifdef DEBUG_CNT
			tot_read++;
#endif
			if (HandleChar(STDOUT_FILENO, DeviceFd, c)) {
				return (c);
			}
		} else {
			break;
		}
	}
	return (-1);
}
		
/* Get a byte from a buffer */
/* Retrieves the port speed from PortFd */
static unsigned long int GetPortSpeed(int PortFd)
  {
	struct termios PortSettings;
	speed_t Speed;

	tcgetattr(PortFd,&PortSettings);
	Speed = cfgetospeed(&PortSettings);

	return(get_baud_table_speed(Speed));

  }

/* Retrieves the data size from PortFd */
static unsigned char GetPortDataSize(int PortFd)
  {
	struct termios PortSettings;
	tcflag_t DataSize;

	tcgetattr(PortFd,&PortSettings);
	DataSize = PortSettings.c_cflag & CSIZE;

	switch (DataSize)
	 {
	case CS5:
		return((unsigned char) 5);
	case CS6:
		return((unsigned char) 6);
	case CS7:
		return((unsigned char) 7);
	case CS8:
		return((unsigned char) 8);
	default:
		return((unsigned char) 0);
	 }
  }

/* Retrieves the parity settings from PortFd */
static unsigned char GetPortParity(int PortFd)
  {
	struct termios PortSettings;

	tcgetattr(PortFd,&PortSettings);

	if ((PortSettings.c_cflag & PARENB) == 0)
	 return((unsigned char) 1);

	if ((PortSettings.c_cflag & PARENB) != 0 &&
	 (PortSettings.c_cflag & PARODD) != 0)
	 return((unsigned char) 2);
	 
	return((unsigned char) 3);
  }

/* Retrieves the stop bits size from PortFd */
static unsigned char GetPortStopSize(int PortFd)
  {
	struct termios PortSettings;

	tcgetattr(PortFd,&PortSettings);

	if ((PortSettings.c_cflag & CSTOPB) == 0)
	 return((unsigned char) 1);
	else
	 return((unsigned char) 2);
  }
  
/* Retrieves the flow control status, including DTR and RTS status,
  from PortFd */
static unsigned char GetPortFlowControl(int PortFd, unsigned char Which)
  {
	struct termios PortSettings;
	int MLines;

	/* Gets the basic informations from the port */
	tcgetattr(PortFd,&PortSettings);
	ioctl(PortFd,TIOCMGET,&MLines);

	/* Check which kind of information is requested */
	switch (Which)
	 {
	/* Com Port Flow Control Setting (outbound/both) */
	case 0:
		if ((PortSettings.c_iflag & IXON) == 0)
		  return((unsigned char) 2);
		if ((PortSettings.c_cflag & CRTSCTS) == 0)
		  return((unsigned char) 3);
		return((unsigned char) 0);
	break;
	
	/* BREAK State  */
	case 4:
		if (BreakSignaled == True)
		  return((unsigned char) 5);
		else
		  return((unsigned char) 6);
	break;
	
	/* DTR Signal State */
	case 7:
		if ((MLines & TIOCM_DTR) == 0)
		  return((unsigned char) 9);
		else
		  return((unsigned char) 8);
	break;

	/* RTS Signal State */
	case 10:
		if ((MLines & TIOCM_RTS) == 0)
		  return((unsigned char) 12);
		else
		  return((unsigned char) 11);
	break;

	/* Com Port Flow Control Setting (inbound) */
	case 13:
		if ((PortSettings.c_iflag & IXOFF) == 0)
		  return((unsigned char) 15);
		if ((PortSettings.c_cflag & CRTSCTS) == 0)
		  return((unsigned char) 16);
		return((unsigned char) 14);
	break;

	default:
		if ((PortSettings.c_iflag & IXON) == 0)
		  return((unsigned char) 2);
		if ((PortSettings.c_cflag & CRTSCTS) == 0)
		  return((unsigned char) 3);
		return((unsigned char) 0);
	break;
	 }
  }

/* Return the status of the modem control lines (DCD, CTS, DSR, RNG) */
unsigned char GetModemState(int PortFd,unsigned char PMState)
  {
	int MLines;
	unsigned char MState = (unsigned char) 0;
	
	ioctl(PortFd,TIOCMGET,&MLines);

	if ((MLines & TIOCM_CAR) != 0)
	 MState += (unsigned char) 128;
	if ((MLines & TIOCM_RNG) != 0)
	 MState += (unsigned char) 64;
	if ((MLines & TIOCM_DSR) != 0)
	 MState += (unsigned char) 32;
	if ((MLines & TIOCM_CTS) != 0)
	 MState += (unsigned char) 16;
	if (!FirstModemStateSent) {
		MState += (unsigned char) 15; //mp[Fev/2004] Bug3112
		FirstModemStateSent = 1;
	} else {
		if ((MState & 128) != (PMState & 128))
			MState += (unsigned char) 8;
		if ((MState & 64) != (PMState & 64))
			MState += (unsigned char) 4;
		if ((MState & 32) != (PMState & 32))
			MState += (unsigned char) 2;
		if ((MState & 16) != (PMState & 16))
			MState += (unsigned char) 1;
	}

	return(MState);
  }

/* Set the serial port data size */
static void SetPortDataSize(int PortFd, unsigned char DataSize)
  {
	struct termios PortSettings;
	tcflag_t PDataSize;

	switch (DataSize)
	 {
	case 5:
		PDataSize = CS5;
	break;
	case 6:
		PDataSize = CS6;
	break;
	case 7:
		PDataSize = CS7;
	break;
	case 8:
		PDataSize = CS8;
	break;
	default:
		PDataSize = CS8;
	break;
	 }

	tcgetattr(PortFd,&PortSettings);
	PortSettings.c_cflag = (PortSettings.c_cflag & ~CSIZE) | PDataSize;
	tcsetattr(PortFd,TCSADRAIN,&PortSettings);
  }

/* Set the serial port parity */
static void SetPortParity(int PortFd, unsigned char Parity)
  {
	struct termios PortSettings;

	tcgetattr(PortFd,&PortSettings);

	switch (Parity)
	 {
	case 1:
		PortSettings.c_cflag = PortSettings.c_cflag & ~PARENB;
	break;
	case 2:
		PortSettings.c_cflag = PortSettings.c_cflag | PARENB | PARODD;
	break;
	case 3:
		PortSettings.c_cflag = (PortSettings.c_cflag | PARENB) & ~PARODD;
	break;
	/* There's no support for MARK and SPACE parity so sets no parity */
	default:
		nsyslog(LOG_WARNING,"Requested unsupported parity. Set to no parity");
		PortSettings.c_cflag = PortSettings.c_cflag & ~PARENB;
	break;
	 }

	tcsetattr(PortFd,TCSADRAIN,&PortSettings);
  }

/* Set the serial port stop bits size */
static void SetPortStopSize(int PortFd, unsigned char StopSize)
  {
	struct termios PortSettings;

	tcgetattr(PortFd,&PortSettings);

	switch (StopSize)
	 {
	case 1:
		PortSettings.c_cflag = PortSettings.c_cflag & ~CSTOPB;
	break;
	case 2:
		PortSettings.c_cflag = PortSettings.c_cflag | CSTOPB;
	break;
	case 3:
		PortSettings.c_cflag = PortSettings.c_cflag & ~CSTOPB;
		nsyslog(LOG_WARNING,"Requested unsupported 1.5 bits stop size. "
		  "Set to 1 bit stop size");
	break;
	default:
		PortSettings.c_cflag = PortSettings.c_cflag & ~CSTOPB;
	break;
	 }

	tcsetattr(PortFd,TCSADRAIN,&PortSettings);
  }

/* Set the port flow control and DTR and RTS status */
static void SetPortFlowControl(int PortFd,unsigned char How)
  {
	struct termios PortSettings;
	int MLines;

	/* Gets the base status from the port */
	tcgetattr(PortFd,&PortSettings);
	ioctl(PortFd,TIOCMGET,&MLines);

	/* Check which settings to change */
	switch (How)
	 {
	/* No Flow Control (outbound/both) */
	case 1:
		PortSettings.c_iflag = PortSettings.c_iflag & ~IXON;
		PortSettings.c_iflag = PortSettings.c_iflag & ~IXOFF;
		PortSettings.c_cflag = PortSettings.c_cflag & ~CRTSCTS;
	break;
	/* XON/XOFF Flow Control (outbound/both) */
	case 2:
		PortSettings.c_iflag = PortSettings.c_iflag | IXON;
		PortSettings.c_iflag = PortSettings.c_iflag | IXOFF;
	break;
	/* HARDWARE Flow Control (outbound/both) */
	case 3:
		PortSettings.c_cflag = PortSettings.c_cflag | CRTSCTS;
	break;
	/* BREAK State ON */
	case 5:
		//[RK]May/05/03 - break interval
		nsyslog(LOG_INFO,"Client sent a break to %s - interval=%d", pconf->tty, pconf->break_interval);
		tcsendbreak(PortFd, pconf->break_interval?pconf->break_interval:500);
		BreakSignaled = True;
	break;
	/* BREAK State OFF */
	case 6:
		tcsendbreak(PortFd,0);
		BreakSignaled = False;
	break;
	/* DTR Signal State ON */
	case 8:
		MLines = MLines | TIOCM_DTR;
	break;
	/* DTR Signal State OFF */
	case 9:
		MLines = MLines & ~TIOCM_DTR;
	break;
	/* RTS Signal State ON */
	case 11:
		MLines = MLines | TIOCM_RTS;
	break;
	/* RTS Signal State OFF */
	case 12:
		MLines = MLines & ~TIOCM_RTS;
	break;
	/* No Flow Control (inbound) */
	case 14:
		PortSettings.c_iflag = PortSettings.c_iflag & ~IXOFF;
	break;
	/* XON/XOFF Flow Control (inbound) */
	case 15:
		PortSettings.c_iflag = PortSettings.c_iflag | IXOFF;
	break;
	/* HARDWARE Flow Control (inbound) */
	case 16:
		PortSettings.c_cflag = PortSettings.c_cflag | CRTSCTS;
	break;
	default:
		nsyslog(LOG_WARNING,"Requested unsupported flow control, "
		  "setting to no flow control");
		PortSettings.c_iflag = PortSettings.c_iflag & ~IXON;
		PortSettings.c_iflag = PortSettings.c_iflag & ~IXOFF;
		PortSettings.c_cflag = PortSettings.c_cflag & ~CRTSCTS;
	break;
	 }
	
	tcsetattr(PortFd,TCSADRAIN,&PortSettings);
	ioctl(PortFd,TIOCMSET,&MLines);
  }

/* Set the serial port speed */ 
static void SetPortSpeed(int PortFd, unsigned long BaudRate)
  {
	struct termios PortSettings;
	speed_t Speed;

	Speed = get_baud_table_index(BaudRate);

	tcgetattr(PortFd,&PortSettings);
	cfsetospeed(&PortSettings,Speed);
	cfsetispeed(&PortSettings,Speed);
	tcsetattr(PortFd,TCSADRAIN,&PortSettings);
  }
  
/* Send the signature Sig to the client */
static void SendSignature(int SockFd, char * Sig)
  {
	char_cmd[3] = TNASC_SIGNATURE;

	if (SockFd == -1) {
		send_msg_this_sess(char_cmd, 4, this_sess);
	} else {
		write(SockFd,char_cmd,4);
	}
	SendStr(SockFd,Sig);
	if (SockFd == -1) {
		send_msg_this_sess(&char_cmd[4], 2, this_sess);
	} else {
		write(SockFd,&char_cmd[4],2);
	}
  }

/* Write a char to SockFd performing IAC escaping */
static void EscWriteChar(int SockFd, unsigned char C)
  {
	if (pconf->protocol == P_SOCKET_SERVER_RAW ||
	   (pconf->protocol == P_SOCKET_CLIENT && pconf->socket_port != TELNET_PORT)) { // Nei - 08/14/02
		if (SockFd == -1) {
			send_msg_this_sess(&C, 1, this_sess);
		} else {
		 	write(SockFd,&C,1);
		}
	}
	else {
		if (C == TNIAC) {
			if (SockFd == -1) {
				send_msg_this_sess(&C, 1, this_sess);
				send_msg_this_sess(&C, 1, this_sess);
			} else {
				write(SockFd,&C,1); 
				write(SockFd,&C,1);
			}
	 	}
		else {
			if (SockFd == -1) {
				send_msg_this_sess(&C, 1, this_sess);
			} else {
	 			write(SockFd,&C,1);
			}
		}
	}
  }

int closing_fd = 0;
static int nretries = 0;

int write_pending_buffer(int PortFd)
{
	int cnt;

	if (!tmp)
		return (0);

	if (tmp->cnt_buf) {
		cnt = write(PortFd,&tmp->tmp_buf[tmp->ix_buf],tmp->cnt_buf);
		if (cnt > 0) {
			//[RK]June/04/03 - SSH/Telnet to IPDU
			if (pconf->protocol != P_IPDU) {
				send_msg_rw_sess(&tmp->tmp_buf[tmp->ix_buf], cnt, 1, pconf->this);
			}
			tmp->ix_buf += cnt;
			tmp->cnt_buf -= cnt;

#ifdef DEBUG_CNT
			tty_write += cnt;
#endif
			if (tmp->cnt_buf > 0) {
				return(1);
			}
		} else {
			if (closing_fd) {
				if (nretries++ < 3) {
					sleep(1);
					return(1);
				}
				tmp->cnt_buf = 0;
				return(0);
			}
			return(1);
		}
	}

	tmp->ix_buf = 0;
	tmp->cnt_buf = 0;

	return(0);
}

/* Redirect char C to PortFd checking for IAC escape sequences */
void EscRedirectChar(int SockFd, int PortFd, BufferType *B)
  {
	unsigned char C;

	if ((SockFd != -2) && !tmp)
		return;

	if ((SockFd != -2) && write_pending_buffer(PortFd)) {
		return;
	}

	if (pconf->protocol == P_SOCKET_SERVER) {  
		if (IACCommandPend) {
			HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
		}
	}

	while (B->RdPos != B->WrPos) {
		C = GetFromBuffer(B);
		if ((pconf->protocol == P_SOCKET_SERVER ||
			 pconf->protocol == P_SOCKET_CLIENT) &&
			 !HandleChar(SockFd, PortFd, C)) {
			continue;
		}

		if (SockFd == -2){ // PM sess
			continue;
		}
		tmp->tmp_buf[tmp->cnt_buf++] = C;
#ifdef DEBUG_CNT
		tty_bwrite++;
#endif
		if (tmp->cnt_buf >= sizeof(tmp->tmp_buf)) {
			break;
		}
	}
	if (SockFd != -2) {
		write_pending_buffer(PortFd);
	}
   }

int HandleChar(int SockFd, int PortFd, char C)
  {
	if ((pconf->protocol == P_SOCKET_SERVER_RAW) ||
	   (pconf->protocol == P_SOCKET_CLIENT && pconf->socket_port != TELNET_PORT) || // Nei - 08/14/02
		(pconf->protocol == P_SOCKET_SSH)) {
		return(1);
	}
	/* Check the IAC escape status */
	switch (IACEscape)
	 {
	/* Normal status */
	case IACNormal:
		if (C == TNIAC)
		  IACEscape = IACReceived;
		else
		  return(1);
	break;

	/* IAC previously received */
	case IACReceived:
		if (C == TNIAC)
		  {
			IACEscape = IACNormal;
			return(1);
		  }
		else
		  {
			IACCommand[0] = TNIAC;
			IACCommand[1] = C;
			IACPos = 2;
			IACEscape = IACComReceiving;
			IACSigEscape = IACNormal;
			if (C == TNBREAK ||
				C == TNNOP ||
				C == TNEL ||
				C == TNEC ||
				C == TNAO ||
				C == TNAYT ||
				C == TNIP ||
				C == TNEOR ||
				C == TNDM) {
				HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
			}
		  }
	break;

	/* IAC Command reception */
	case IACComReceiving:
		/* Telnet suboption, could be only CPC */
		if (IACCommand[1] == TNSB)
		  {
			/* Get the suboption signature */
			if (IACPos < 4)
		{
			IACCommand[IACPos] = C;
			IACPos++;
		}
			else
		{
			/* Check which suboption we are dealing with */
			switch (IACCommand[3])
			  {
				/* Signature, which needs further escaping */
				case TNCAS_SIGNATURE:
			switch (IACSigEscape)
				{
				  case IACNormal:
					if (C == TNIAC)
				IACSigEscape = IACReceived;
					else
				{
					if (IACPos < TmpStrLen)
					  {
						IACCommand[IACPos] = C;
						IACPos++;
					  }
				}
				  break;

				  case IACComReceiving:
					IACSigEscape = IACNormal;
				  break;
	
				  case IACReceived:
					if (C == TNIAC)
				{
					if (IACPos < TmpStrLen)
					  {
						IACCommand[IACPos] = C;
						IACPos++;
					  }
					IACSigEscape = IACNormal;
				}
					else
				{
					if (IACPos < TmpStrLen)
					  {
						IACCommand[IACPos] = TNIAC;
						IACPos++;
					  }
	
					if (IACPos < TmpStrLen)
					  {
						IACCommand[IACPos] = C;
						IACPos++;
					  }
	
					HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
				}
				  break;
				}
				break;
	
				/* Set baudrate */
				case TNCAS_SET_BAUDRATE:
				case TNCAS_SET_MEDIA:
			IACCommand[IACPos] = C;
			IACPos++;
	
			if (IACPos == 10)
				{
				  HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
				}
				break;
				
				/* Flow control command */
				case TNCAS_FLOWCONTROL_SUSPEND:
				case TNCAS_FLOWCONTROL_RESUME:
			IACCommand[IACPos] = C;
			IACPos++;
	
			if (IACPos == 6)
				{
				  HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
				}
				break;
				
				/* Normal CPC command with single byte parameter */
				default:
			IACCommand[IACPos] = C;
			IACPos++;
	
			if (IACPos == 7)
				{
				  HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
				}
				break;
			  }					   
		}
		}
	else
		{
		  /* Normal 3 byte IAC option */
		  IACCommand[IACPos] = C;
		  IACPos++;

		  if (IACPos == 3)
			{
		HandleIACCommand(SockFd,PortFd,IACCommand,IACPos);
			}
		}
	break;
	 }
   return(0);
  }

static unsigned char IACseq[4] = {TNIAC, 0,0,0};

/* Send the specific telnet option to SockFd using Command as command */
static void SendTelnetOption(int SockFd, unsigned char Command, char Option)
  {
	IACseq[1] = Command;
	IACseq[2] = Option;

	if (SockFd == -1) {
		send_msg_this_sess(IACseq, 3, this_sess);
	} else {
		write(SockFd, IACseq, 3);
	}
  }

/* Send a string to SockFd performing IAC escaping */
static void SendStr(int SockFd, char * Str)
  {
	size_t I;
	size_t L;

	L = strlen(Str);

	for (I = 0; I < L;I++)
	 EscWriteChar(SockFd,(unsigned char) Str[I]);
  }

/* Send the CPC command Command using Parm(int) as parameter */
static void SendCPCIntCommand(int SockFd, unsigned char Command, unsigned long int BR)
  {
	char_cmd_int[3] = Command;
	*(unsigned int *)&char_cmd_int[4] = htonl(BR);
	
	if (SockFd == -1) {
		send_msg_this_sess(char_cmd_int, 10, this_sess);
	} else {
		write(SockFd,char_cmd_int,10);
	}
  }

/* Send the flow control command Command */
static void SendCPCFlowCommand(int SockFd, unsigned char Command)
  {
	char_cmd[3] = Command;
	
	if (pconf->protocol == P_SOCKET_SERVER_RAW ||
	    //[RK]June/03/04 - SSH/Telnet to IPDU
	    (pconf->protocol == P_IPDU) ||
	   (pconf->protocol == P_SOCKET_CLIENT && pconf->socket_port != TELNET_PORT)) { // Nei - 08/14/02
		return;
	}
	if (SockFd == -1) {
		send_msg_this_sess(char_cmd, 6, this_sess);
	} else {
		write(SockFd,char_cmd,6);
	}

	if (Command == TNASC_FLOWCONTROL_SUSPEND)
	 nsyslog(LOG_DEBUG,"Sent flow control suspend command");
	else
	 nsyslog(LOG_DEBUG,"Sent flow control resume command");
	
  }
  
/* Send the CPC command Command using Parm as parameter */
void SendCPCByteCommand(int SockFd, unsigned char Command, unsigned char Parm)
  {
	char_cmd_byte[3] = Command;
	char_cmd_byte[4] = Parm;

	if (SockFd == -1) {
		send_msg_this_sess(char_cmd_byte, 7, this_sess);
	} else {
		write(SockFd,char_cmd_byte,7);
	}
  }

/* Send a string to the client */
static void SendString(int SockFd, char *str, int len)
{
	if (SockFd == -1) {
		send_msg_this_sess(str, len, this_sess);
	} else {
		write(SockFd,str, len);
	}
}

/* Handling of COM Port Control specific commands */
static void HandleCPCCommand(int SockFd, int PortFd, unsigned char * Command, size_t CSize)
  {
	char SigStr[TmpStrLen];
	unsigned long int BaudRate;
	unsigned char DataSize;
	unsigned char Parity;
	unsigned char StopSize;
	unsigned char FlowControl;
	
	/* Check which command has been requested */
	switch (Command[3])
	 {
	/* Signature */
	case TNCAS_SIGNATURE:
		if (CSize == 6)
		  {
			/* Void signature, client is asking for our signature */
			sprintf(SigStr,"socket /dev/%s", pconf->tty);
			SendSignature(SockFd,SigStr);
			nsyslog(LOG_INFO,"Sent signature: %s",SigStr);
		  }
		else
		  {
			/* Received client signature */
			strncpy(SigStr,&Command[4],CSize - 6);
			nsyslog(LOG_INFO,"Received client signature: %s",SigStr);
		  }
	break;

	/* Set serial baud rate */
	case TNCAS_SET_BAUDRATE:
		/* Retrieve the baud rate which is in network order */
		BaudRate = ntohl(*((unsigned long int *) &Command[4]));

		if (BaudRate == 0)
		  {
			/* Client is asking for current baud rate */
			nsyslog(LOG_DEBUG,"Baud rate notification received");			
		  }
		else
		  {
			/* Change the baud rate */
			nsyslog(LOG_DEBUG,"Port baud rate change to %lu requested",BaudRate);
			SetPortSpeed(PortFd,BaudRate);		
		  }

		/* Send confirmation */
		BaudRate = GetPortSpeed(PortFd);
		SendCPCIntCommand(SockFd,TNASC_SET_BAUDRATE,BaudRate);
		nsyslog(LOG_DEBUG,"Port baud rate: %lu",BaudRate);
	break;
	
	/* Set serial data size */
	case TNCAS_SET_DATASIZE:
		if (Command[4] == 0)
		  {
			/* Client is asking for current data size */
			nsyslog(LOG_DEBUG,"Data size notification requested");		   
		  }
		else
		  {
			/* Set the data size */
			nsyslog(LOG_DEBUG,"Port data size change to %u requested", (unsigned int) Command[4]);
			SetPortDataSize(PortFd,Command[4]);		   
		  }

		/* Send confirmation */
		DataSize = GetPortDataSize(PortFd);
		SendCPCByteCommand(SockFd,TNASC_SET_DATASIZE,DataSize);
		nsyslog(LOG_DEBUG,"Port data size: %u",(unsigned int) DataSize);
	break;

	/* Set the serial parity */
	case TNCAS_SET_PARITY:
		if (Command[4] == 0)
		  {
			/* Client is asking for current parity */
			nsyslog(LOG_DEBUG,"Parity notification requested");		  
		  }
		else
		  {
			/* Set the parity */
			nsyslog(LOG_DEBUG,"Port parity change to %u requested", (unsigned int) Command[4]);
			SetPortParity(PortFd,Command[4]);		   
		  }

		/* Send confirmation */
		Parity = GetPortParity(PortFd);
		SendCPCByteCommand(SockFd,TNASC_SET_PARITY,Parity);
		nsyslog(LOG_DEBUG,"Port parity: %u",(unsigned int) Parity);
	break;
	
	/* Set the serial stop size */
	case TNCAS_SET_STOPSIZE:
		if (Command[4] == 0)
		  {
			/* Client is asking for current stop size */
			nsyslog(LOG_DEBUG,"Stop size notification requested");		   
		  }
		else
		  {
			/* Set the stop size */
			nsyslog(LOG_DEBUG,"Port stop size change to %u requested", (unsigned int) Command[4]);
			SetPortStopSize(PortFd,Command[4]);
		  }

		/* Send confirmation */
		StopSize = GetPortStopSize(PortFd);
		SendCPCByteCommand(SockFd,TNASC_SET_STOPSIZE,StopSize);
		nsyslog(LOG_DEBUG,"Port stop size: %u",(unsigned int) StopSize);
	break;

	case TNCAS_NOTIFY_MODEMSTATE:
		ModemState = GetModemState(DeviceFd,ModemState) & ModemStateMask;
		SendCPCByteCommand(SockFd,TNASC_NOTIFY_MODEMSTATE, ModemState);
		break;

	/* Flow control and DTR/RTS handling */
	case TNCAS_SET_CONTROL:
		if (Command[4] == 0 || Command[4] == 4 || Command[4] == 7 ||
		  Command[4] == 10 || Command[4] == 13)
		  {
			/* Client is asking for current flow control or DTR/RTS status */
			nsyslog(LOG_DEBUG,"Flow control notification requested");
			FlowControl = GetPortFlowControl(PortFd,Command[4]);
			SendCPCByteCommand(SockFd,TNASC_SET_CONTROL,FlowControl);
			nsyslog(LOG_DEBUG,"Port flow control: %u", (unsigned int) FlowControl);
		  }
		else
		  {
			/* Set the flow control */
			nsyslog(LOG_DEBUG,"Port flow control change to %u requested", (unsigned int) Command[4]);
			SetPortFlowControl(PortFd,Command[4]);

			/* Flow control status confirmation */
			if (Command[4] <= 16)
		FlowControl = Command[4];
			else
		FlowControl = GetPortFlowControl(PortFd,0);
			SendCPCByteCommand(SockFd,TNASC_SET_CONTROL,FlowControl);
			nsyslog(LOG_DEBUG,"Port flow control: %u", (unsigned int) FlowControl);
		  }
	break;
	
	/* Set the line state mask */
	case TNCAS_SET_LINESTATE_MASK:		
		nsyslog(LOG_DEBUG,"Line state set to %u",(unsigned int) Command[4]);
		/* Only break notification supported */
		LineStateMask = Command[4] & (unsigned char) 16;
		SendCPCByteCommand(SockFd,TNASC_SET_LINESTATE_MASK,LineStateMask);
	break;

	/* Set the modem state mask */
	case TNCAS_SET_MODEMSTATE_MASK:
		nsyslog(LOG_DEBUG,"Modem state mask set to %u", (unsigned int) Command[4]);
		ModemStateMask = Command[4];
		SendCPCByteCommand(SockFd,TNASC_SET_MODEMSTATE_MASK,ModemStateMask);
		//ModemState = GetModemState(DeviceFd,ModemState) & ModemStateMask; mp[Fev/2004] Bug3112
	break;

	/* Port flush requested */
	case TNCAS_PURGE_DATA:
		nsyslog(LOG_DEBUG,"Port flush %u requested",(unsigned int) Command[4]);
		switch (Command[4])
		  {
			/* Inbound flush */
			case 1:
		tcflush(PortFd,TCIFLUSH);
			break;
			/* Outbound flush */
			case 2:
		tcflush(PortFd,TCOFLUSH);
			break;
			/* Inbound/outbound flush */
			case 3:
		tcflush(PortFd,TCIOFLUSH);
			break;
		  }

		SendCPCByteCommand(SockFd,TNASC_PURGE_DATA,Command[4]);
	break;

	/* Suspend output to the client */
	case TNCAS_FLOWCONTROL_SUSPEND:
		nsyslog(LOG_DEBUG,"Flow control suspend requested");
		InputFlow = False;
	break;
	
	/* Resume output to the client */
	case TNCAS_FLOWCONTROL_RESUME:
		nsyslog(LOG_DEBUG,"Flow control resume requested");
		InputFlow = True;
	break;

	case TNCAS_SET_MEDIA:
		BaudRate = ntohl(*((unsigned long int *) &Command[4]));

		if (BaudRate & 0x20000) {
			//[RK]Jul/29/03 - not used
			//ioctl(PortFd, TIOCSERCONFIG, &BaudRate);
			break;
		}

		//[RK]Jul/29/03 
		if (BaudRate == RS_QUERY_MEDIA) {
			/* Client is asking for current media type*/
			nsyslog(LOG_DEBUG,"Media type notification requested");		   
		} else {
			/* Set the data size */
			nsyslog(LOG_DEBUG,"Port Media type change to %lu requested", (unsigned int) BaudRate);
			BaudRate |= CYC_SET_MEDIA;
			if (ioctl(PortFd, TIOCSERCYC, &BaudRate) < 0) {
				nsyslog(LOG_ERR, "TIOCSERCYC: %m");
			}
		}

		BaudRate = CYC_GET_MEDIA;
		/* Send confirmation */
		ioctl(PortFd, TIOCSERCYC, &BaudRate);
		SendCPCIntCommand(SockFd,TNASC_SET_MEDIA,BaudRate);
		nsyslog(LOG_DEBUG,"Port Media type %u", (unsigned int) BaudRate);
		break;

	/* Unknown request */
	default:
		nsyslog(LOG_DEBUG,"Unhandled request %u",(unsigned int) Command[3]);
	break;
	 }
  }

/* Common telnet IAC commands handling */
static void HandleIACCommand(int SockFd, int PortFd, unsigned char * Command, size_t CSize)
  {
//	if (PortFd == -1 && (((cy_shm->sess_list[pconf->this+1])->sess_flag) & CY_SESS_FLAG_PMD)  {
	if (PortFd == -1 ) {
		IACCommandPend = 0;
		IACEscape = IACNormal;
		return;
	}

	if ((SockFd!=-2) &&write_pending_buffer(PortFd)) {
		IACCommandPend = 1;
		return;
	}

	IACCommandPend = 0;
	IACEscape = IACNormal;

	/* Check which command */
	switch(Command[1])
	 {
	case TNAYT:
		{
			unsigned char ch[128];
			int len;
			len = snprintf(ch, sizeof(ch), "\r\n[%s : yes]\r\n", mainconf.hostname);
			SendString(SockFd, ch, len);
		}
		break;
	case TNAO:
		{
			unsigned char ch[4];
			ch[0] = TNIAC;
			ch[1] = TNDM;
			ch[2] = 0;
			SendString(SockFd, ch, 2);
		}
		break;
	case TNNOP:
	case TNEL:
	case TNEC:
	case TNIP:
	case TNEOR:
	case TNDM:
		nsyslog(LOG_DEBUG,"Client sent a <%d> to %s", Command[1], pconf->tty);
		break;

	case TNBREAK:
		nsyslog(LOG_INFO,"Client sent a break to %s (interval=%d)", pconf->tty,pconf->break_interval);
		tcsendbreak(PortFd, pconf->break_interval?pconf->break_interval:500);
		break;

	/* Suboptions */
	case TNSB:
		switch (Command[2])
		  {
			/* RFC 2217 COM Port Control Protocol option */
			case TNCOM_PORT_OPTION:
				//[RK]June/03/04 - SSH/Telnet to IPDU
				if (pconf->protocol != P_IPDU) 
					HandleCPCCommand(SockFd,PortFd,Command,CSize);
				break;
			
			default:
		nsyslog(LOG_DEBUG,"Unknown suboption received: %u", (unsigned int) Command[2]);
			break;
		  }
	break;

	/* Requests for options */
	case TNWILL:
		switch (Command[2])
		  {
			/* COM Port Control Option */
			case TNCOM_PORT_OPTION:
		nsyslog(LOG_INFO,"Telnet COM Port Control Enabled (WILL)");
			//[RK]June/03/04 - SSH/Telnet to IPDU
			if (pconf->protocol != P_IPDU) {
				TCPCEnabled = True;			
				SendTelnetOption(SockFd,TNDO,TNCOM_PORT_OPTION);
			} else {
				SendTelnetOption(SockFd,TNDONT,Command[2]);
			}
			break;

			/* Telnet Binary mode */
			case TN_TRANSMIT_BINARY:
		//nsyslog(LOG_INFO,"Telnet Binary Transfer Enabled (WILL)");
		//SendTelnetOption(SockFd,TNDO,TN_TRANSMIT_BINARY);
			break;

			/* Echo request not handled */
			case TN_ECHO:
		//nsyslog(LOG_INFO,"Rejecting Telnet Echo Option (WILL)");
		//SendTelnetOption(SockFd,TNDONT,TN_TRANSMIT_BINARY);
			break;

			/* No go ahead needed */
			case TN_SUPPRESS_GO_AHEAD:
		nsyslog(LOG_INFO,"Suppressing Go Ahead characters (WILL)");
		SendTelnetOption(SockFd,TNDO,TN_TRANSMIT_BINARY);
			break;

			/* Reject everything else */
			default:
		nsyslog(LOG_DEBUG,"Rejecting option WILL: %u", (unsigned int) Command[2]);
		SendTelnetOption(SockFd,TNDONT,Command[2]);
			break;
		  }
	break;

	/* Confirmations for options */
	case TNDO:
		switch (Command[2])
		  {
			/* COM Port Control Option */
			case TNCOM_PORT_OPTION:
		nsyslog(LOG_INFO,"Telnet COM Port Control Enabled (DO)");
			//[RK]June/03/04 - SSH/Telnet to IPDU
			if (pconf->protocol != P_IPDU) {
				TCPCEnabled = True;			
				/* TNWILL already sent on startup */
			}
			break;
			/* Telnet Binary mode */
			case TN_TRANSMIT_BINARY:
		//nsyslog(LOG_INFO,"Telnet Binary Transfer Enabled (DO)");
		//SendTelnetOption(SockFd,TNWILL,TN_TRANSMIT_BINARY);
			break;

			/* Echo reques1t not handled */
			case TN_ECHO:
		//nsyslog(LOG_INFO,"Rejecting Telnet Echo Option (DO)");
		//SendTelnetOption(SockFd,TNWONT,TN_TRANSMIT_BINARY);
			break;

			/* No go ahead needed */
			case TN_SUPPRESS_GO_AHEAD:
		//nsyslog(LOG_INFO,"Suppressing Go Ahead characters (DO)");
		//SendTelnetOption(SockFd,TNWILL,TN_TRANSMIT_BINARY);
			break;

			/* Reject everything else */
			default:
		nsyslog(LOG_DEBUG,"Rejecting option DO: %u", (unsigned int) Command[2]);
		SendTelnetOption(SockFd,TNWONT,Command[2]);
			break;
		  }
	break;
	
	/* Notifications of rejections for options */
	case TNWONT:
	case TNDONT:
		if (Command[2] == TNCOM_PORT_OPTION)
		  {
			nsyslog(LOG_INFO,"Client doesn't support Telnet COM Port "
		"Protocol Option (RFC 2217)");
		  }
		else
		  {
			nsyslog(LOG_DEBUG,"Received rejection for option: %u", (unsigned int) Command[2]);
		  }
	break;
	 }
  }

/* Write a buffer to SockFd without IAC escaping */
static void WriteBuffer(int SockFd, unsigned char * Buffer, unsigned int BSize)
{
	int I;	//[RK]Oct/30/02 BUG#73

	if (BSize > 0) {
		if (SockFd == -1) {
			return;
		} else {
GOTO_WBUF:
			I = write(SockFd,Buffer,BSize);
			if (I > 0) {
				OutBPos -= I;
				if (OutBPos)
					memcpy(Buffer, &Buffer[I], BSize - I);
			}else{	//[RK]Oct/30/02 BUG#73
				if (got_int_flag == SIGIO) {
					got_int_flag = 0;
					goto GOTO_WBUF;
				}
			}
#ifdef DEBUG_CNT
			tot_write += I;
#endif
		}

	}
}

/* Write a buffer to SockFd with IAC escaping.
   Also, part of this routine implements the probing mechanism
   to allow BIOS output. Specifically, the all.auto_answer_input
   and all.auto_answer_output parameters. */
int EscWriteBuffer(int SockFd, unsigned char * Buffer, unsigned int BSize, unsigned int isOutBPos, int ttyFd, int sess)
{
	unsigned int I;
	unsigned char *TmpBuf;
	unsigned int BPos = 0;

	if (BSize > 0) {
		// this logic below tries to transmit any accumulated bytes in "local" buffer
		if (Bufwithbytes > 0) {
			I = write(SockFd,Buf,Bufwithbytes);
			if (I != Bufwithbytes) {
				if (I >= 0) {
					memcpy(Buf, &Buf[I], Bufwithbytes = Bufwithbytes - I);
				}
			}
			else {
				Bufwithbytes = 0; // no more accumalated bytes in "local" buffer
			}
			return 0;
		}

		// no accumulated bytes in "local" buffer; tries to just transmit the "global" buffer then
		if (pconf->protocol == P_SOCKET_SERVER_RAW ||
		    //[RK]Aug/30/06 - SSH - without sess ==> 
		    // treatment of autoanswer
		     (sess && (pconf->protocol == P_SOCKET_SSH)) ||
 		    //[RK]June/03/04 - SSH/Telnet to IPDU
 		    (pconf->protocol == P_IPDU) ||
		   (pconf->protocol == P_SOCKET_CLIENT && pconf->socket_port != TELNET_PORT)) { // Nei - 08/14/02
			TmpBuf = Buffer;
			BPos = BSize;
			if (SockFd == -1) {
				return send_msg_rw_sess(TmpBuf, BPos, 2, pconf->this);
			} 
			else {
				I = write(SockFd,TmpBuf,BPos);
				if (isOutBPos) {
					if (I != BPos) {
						if (I >= 0) {
							OutBPos -= I;
							memcpy(Buffer, &Buffer[I], BPos - I);
						}
					} 
					else {
						OutBPos = 0;
					}
				}
#ifdef DEBUG_CNT
				tot_write += I;
#endif
			}
		} 
		else {
			TmpBuf = Buf;
			for (I = 0;I < BSize; I++) {
				// if there is TNIAC (Telnet Interpret as Command),
				// insert an extra TNIAC to escape from the characters' special meaning
				if (Buffer[I] == TNIAC)
				{
					TmpBuf[BPos] = TNIAC;
					TmpBuf[BPos + 1] = TNIAC;
					BPos += 2;
				}
				// Implemention of probing mechanism to send configured output string
				// to the tty if configured input string is found. Jen - 11/26/02
				else if (!sess && pconf->auto_answer_input && Buffer[I] == pconf->auto_answer_input[inputPend]) 
				{
					inputPend++;
					TmpBuf[BPos] = Buffer[I];
					BPos++;
					if (!pconf->auto_answer_input[inputPend]) { // input string match with incoming data
						if (pconf->auto_answer_output) {
							//nsyslog(LOG_INFO, "Sending '%s' to tty", pconf->auto_answer_output);
							I = write(ttyFd, pconf->auto_answer_output, strlen(pconf->auto_answer_output));
							if (isOutBPos) {
								if (I != strlen(pconf->auto_answer_output)) {
									if (I >= 0) {
										OutBPos -= I;
										memcpy(Buffer, &Buffer[I], strlen(pconf->auto_answer_output) - I);
									}
								}
								else {
									OutBPos = 0;
								}
							}
						}
						inputPend = 0;  // reset bc there was a match
					}
				}
				// Copy incoming data into TmpBuf and send to socket
				else
				{
					TmpBuf[BPos] = Buffer[I];
					BPos++;
					inputPend = 0;
				}
			} //end of FOR loop
	
			if (SockFd == -1) {
				return send_msg_rw_sess(TmpBuf, BPos, 2, pconf->this);
			} 
			else {
				I = write(SockFd,TmpBuf,BPos);
				if (isOutBPos) {
					if (I != BPos) {
						if (I >= 0) {
							OutBPos = 0; // all bytes transferred to the "local" buffer
							memcpy(Buf, &Buf[I], Bufwithbytes = BPos - I);
						}
					} 
					else {
						OutBPos = 0;
						Bufwithbytes = 0; // no accumulated bytes in "local" buffer
					}
				}
#ifdef DEBUG_CNT
				tot_write += I;
#endif
			}
		}
	}
	return 0;
}


static void get_send_ModemState(int DeviceFd, int wait_flag)
{
	unsigned char ModemSt = ModemState;

	/* Check the port state and notify the client if it's changed */
	if (!TCPCEnabled || !(wait_flag || InputFlow)) {
		return;
	}

	ModemState = GetModemState(DeviceFd,ModemState) & ModemStateMask;

	if (ModemSt != ModemState) {
		SendCPCByteCommand(STDOUT_FILENO,TNASC_NOTIFY_MODEMSTATE, ModemState);
		nsyslog(LOG_DEBUG,"Sent modem state: %x", (unsigned int)ModemState);
		if (wait_flag) {
			sleep(2);
		}
	}
}

static char op3_txt[] =
"\r\nA non-empty Data Buffering File was found. Choose which action \r\n"
"should be performed ( (I)gnore or (D)isplay ) : ";
static char op0_txt[] =
"\r\nA non-empty Data Buffering File was found. Choose which action \r\n"
"should be performed ( (I)gnore, (D)isplay, (E)rase or (S)how and erase ) : ";
static char inv_txt[] = "\r\n'?' is an invalid option";
static char disp_txt[] = "?\r\nFile content:\r\n";
//[RK]Mar/14/03 static char ignore_txt[] = "\r\nFile content was ignored\r\n";
static char del_txt[] = "?\r\nFile erased !\r\n";

int check_buffering_option(BufferType *InBuffer, int fd, int DeviceFd, CY_SESS *sess)
{
	int rtn = 0;
	char ch_op, flag = 0;

	if ((pconf->dont_show_db_op == 1) && (pconf->DB_mode != LIN_mode)) { // no display menu and ignore buffer
//[RK]Mar/14/03 EscWriteBuffer(STDOUT_FILENO, ignore_txt, strlen(ignore_txt), 0, -1, 1);
		goto exit_rot;
	}

	if (pconf->dont_show_db_op == 2 || pconf->DB_mode == LIN_mode) { // no display menu and display buffer
		if (pconf->DB_mode != LIN_mode && pconf->DB_mode != DLS_mode) {
			EscWriteBuffer(STDOUT_FILENO, &disp_txt[1], strlen(disp_txt)-1, 0, -1, 1);
			lseek(fd, 0, SEEK_SET);
			more (fd, DeviceFd, read_a_char, write_a_line,0);
			EscWriteBuffer(STDOUT_FILENO, "\r\n", 2, 0, -1, 1);
			lseek(fd, 0, SEEK_END);
		}		
		goto exit_rot;
	}
	
	while (InBuffer->RdPos < InBuffer->WrPos) {
		ch_op = InBuffer->Buffer[InBuffer->RdPos++];
		if (!HandleChar(STDOUT_FILENO, DeviceFd, ch_op)) {
			continue;
		}
		flag = 1;
		switch (ch_op &= 0xdf) {
			case 'S':
				if (pconf->dont_show_db_op == 3) {
					inv_txt[3] = ch_op;
					EscWriteBuffer(STDOUT_FILENO, inv_txt, strlen(inv_txt), 0, -1, 1);
					break;
				}
			case 'D':
				disp_txt[0] = ch_op;
				EscWriteBuffer(STDOUT_FILENO, disp_txt, strlen(disp_txt), 0, -1, 1);

				lseek(fd, 0, SEEK_SET);

				more (fd, DeviceFd, read_a_char, write_a_line,0);

				if (ch_op == 'D') {
					EscWriteBuffer(STDOUT_FILENO, "\r\n", 2, 0, -1, 1);
					lseek(fd, 0, SEEK_END);
					goto exit_rot;
				}

				EscWriteBuffer(STDOUT_FILENO, del_txt+1, strlen(del_txt)-1, 0, -1, 1);
				lseek(fd, 0, SEEK_SET);
				ftruncate(fd, 0); 
				goto exit_rot;

			case 'E':
				if (pconf->dont_show_db_op == 3) {
					inv_txt[3] = ch_op;
					EscWriteBuffer(STDOUT_FILENO, inv_txt, strlen(inv_txt), 0, -1, 1);
					break;
				}
				del_txt[0] = ch_op;
				EscWriteBuffer(STDOUT_FILENO, del_txt, strlen(del_txt), 0, -1, 1);
				lseek(fd, 0, SEEK_SET);
				ftruncate(fd, 0); 
				goto exit_rot;

			case 'I':
				disp_txt[0] = ch_op;
				EscWriteBuffer(STDOUT_FILENO, disp_txt, 3, 0, -1, 1);
				goto exit_rot;

			default:
				inv_txt[3] = ch_op;
				EscWriteBuffer(STDOUT_FILENO, inv_txt, strlen(inv_txt), 0, -1, 1);
				break;
		}
	}
	if (!InBuffer->RdPos || flag) {
		if (pconf->dont_show_db_op == 0) {
			EscWriteBuffer(STDOUT_FILENO, op0_txt, strlen(op0_txt), 0, -1, 1);
		} else {
			EscWriteBuffer(STDOUT_FILENO, op3_txt, strlen(op3_txt), 0, -1, 1);
		}
	}
	InitBuffer(InBuffer);
	rtn = 1;
exit_rot:
	if (sess) {
		if (rtn)
			sess->sess_flag |= CY_SESS_FLAG_DBMENU;
		else
			sess->sess_flag &= ~CY_SESS_FLAG_DBMENU;
	}
	return(rtn);
}

void print_counters(int flag)
{
#ifdef DEBUG_CNT
	nsyslog(LOG_NOTICE,"Total ReadS %d  writeS %d ReadT %d WriteT %d WriteTb %d",
		tot_read, tot_write, tty_read, tty_write, tty_bwrite);
	if (flag) {
		tot_read = tot_write = tty_read = tty_write = tty_bwrite = 0;
	}
#endif
}

int main_routine_socket(CY_SESS *sess)
{   
	/* Input fd set */
	fd_set InFdSet;
	
	/* Exceptions fd set */
	fd_set FdExSet;
	
	/* Output fd set */
	fd_set OutFdSet;

	/* Base timeout for stream reading */
	struct timeval BTimeout;

	/* Pointer to timeout structure to set */
	struct timeval * ETimeout = NULL;

	/* Remote flow control flag */
	int RemoteFlowOff = 0;

	/* Out buffer ticks count */
	clock_t OutBTicks, OutBTicksTx;

	/* Out buffer clock ticks limit */
	clock_t MaxBTicksTx = 0, MaxBTicks = 0, MinBTicks = 50;

	/* Redefines SIGINT & SIGTERM */
	struct sigaction sa = {0};

	int	max_fd, count, rtn, sec = 0;

	int	msTicks = 10;	/* How many ms in one time tick !!! */
						/* I don't know any smart calculation for that */

	int	RetentionMode = 0;
	unsigned long idletimeout = 0, max_idle = 0;

	int data_count;	/* Menu treatment */
	int cmd_pend = 0;

	int fd_buf = -1;

	if (!sess) {
		tmp = &tmp_buf;
	}

	OutBTicks = OutBTicksTx = times(NULL);

	if (pconf->idletimeout) {
		max_idle = pconf->idletimeout * 60 * 1000;
	}

	sa.sa_handler = got_int;
#if 0
	sigaction(SIGINT,  &sa, NULL);
	sigaction(SIGTERM, &sa, NULL);
	sigaction(SIGHUP,  &sa, NULL);
	sigaction(SIGALRM,  &sa, NULL);
	sigaction(SIGIO,  &sa, NULL);	//[RK]Sep/04/02 Dell
	sigaction(SIGPIPE,  &sa, NULL);  //[RK] socket session is stopped (receive RST)
	sigaction(SIGUSR2,  &sa, NULL);
	sigaction(SIGUSR1,  &sa, NULL);
#else
	for (count=1; count < 64; count++) {
		if (count == SIGPIPE) continue;
		if (count == SIGWINCH) continue;
		if (count == SIGSEGV) continue;
		if (count == SIGFPE) continue;
		if (count == SIGILL) continue;
		if (count == SIGKILL) continue;
		sigaction(count,  &sa, NULL);
	}
#endif

	if (DeviceFd != -1) {	
		max_fd = DeviceFd + 1;
	} else {
		max_fd = MAX(STDIN_FILENO, STDOUT_FILENO)+1;
	}

	/* Write a little usage information */
	nsyslog(LOG_INFO, "socket: RFC 2217 compliant serial port redirector %s"
, SocketVersion);

	/* Retrieve the polling interval */
	if (pconf->poll_interval) {
		MaxBTicks = MAX(pconf->poll_interval, 100);
	}

	/* Retrieve the tx interval */
	if (pconf->tx_interval) {
		MaxBTicksTx = MAX(pconf->tx_interval, 50);
	}

	if (max_idle || MaxBTicksTx || MaxBTicks) {

		/* Set up timeout for select */
		ETimeout = &BTimeout;
		MinBTicks = ZMIN(MinBTicks, MaxBTicks);
		MinBTicks = ZMIN(MinBTicks, MaxBTicksTx);
		MinBTicks = ZMIN(max_idle, MinBTicks);
	} else {
		if (sess) {
			ETimeout = &BTimeout;
		}
	}

	/* Logs socket start */
	nsyslog(LOG_NOTICE,"SRedird started");

	/* Logs the polling interval */
	nsyslog(LOG_INFO,"Poll interval %u ms Tx interval %u ms Idle %d min",
		MaxBTicks, MaxBTicksTx, pconf->idletimeout);
	
	makeRawOrStty(DeviceFd, pconf);
	
	/* Reset the device fd to blocking mode */
	if (DeviceFd != -1) {
		if (fcntl(DeviceFd,F_SETFL,fcntl(DeviceFd,F_GETFL) & ~(O_NDELAY)) == OpenError)
			nsyslog(LOG_ERR,"Unable to reset device to non blocking mode, ignoring");
	}

	/* Initialize the input buffer */
	if (sess) {
		pInBuffer = &sess->InBuffer;
		this_sess = sess;
	} else {
		pInBuffer = &InBuffer;
	}
	InitBuffer(pInBuffer);

	tmp_buf.ix_buf = 0;
	tmp_buf.cnt_buf = 0;

	/* Set up fd sets */
	FD_ZERO(&InFdSet);
	FD_SET(STDIN_FILENO,&InFdSet);
	if (!sess) {	// SOCKET_CLIENT
		FD_SET(DeviceFd,&InFdSet);
	}
	FD_ZERO(&OutFdSet);
	FD_ZERO(&FdExSet);
	FD_SET(STDIN_FILENO,&FdExSet);

	//ModemState = GetModemState(DeviceFd, ModemStateECMask) & ModemStateMask; mp[Fev/2004] Bug3112

	if (sess && (RetentionMode = check_buffering_retention_mode(pconf, &fd_buf, sess))) {
		RetentionMode = check_buffering_option(pInBuffer, fd_buf, DeviceFd, sess);
		if (!RetentionMode) {
			close(fd_buf);
		}
	}
	curr_log = &log_buffer;
	log_buffer.tot_buf = 0;

	
	/* Main loop with fd's control */
	while (True) {
		if (MinBTicks) {
			if (MinBTicks > 10000) {
				BTimeout.tv_sec = MinBTicks / 1000;
				BTimeout.tv_usec = 0;
			} else {
				BTimeout.tv_sec = 0;
				BTimeout.tv_usec = MinBTicks * 1000;
			}
		}

		if (pconf->dcd) {
			if(!cyc_check_dcd(pconf->this)) {
				write_a_line ("Error reading from device, DCD is off.");
				nsyslog(LOG_NOTICE,"Error reading from device, DCD is off.");
				break;
			}
		}

		// check for data in shared memory (treatment done by cy_buffering)
		if (sess && !(sess->sess_flag & CY_SESS_FLAG_PMD)) { // sess is not PM session
			// amount available to read from the common area to the local buffer
			count = OutBufferSize-OutBPos; 
			// count is the REAL number of bytes read in
			count = read_rw_msg(sess, &OutBuffer[OutBPos], count); 
			if (count > 0) { 
#ifdef DEBUG_CNT
				tty_read += count;
#endif
				idletimeout = 0;

				if (!RetentionMode) {
					OutBPos += count;
				}

				if (MaxBTicksTx > 0) {
					if (OutBPos >= (OutBufferSize - 15)) {
						WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
						OutBTicksTx = times(NULL);
					}
				} else {
					WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
				}
				//[RK]May/26/03 - Performance Improv.
				//[RK]Nov/24/03 - Bug#1075
				if (sess->flow_flag == 2 && cy_shm->DB_pid) {
					kill(cy_shm->DB_pid, SIGIO);
				}
			}
		}

		if ((rtn = select(max_fd,&InFdSet,&OutFdSet,&FdExSet,ETimeout)) > 0) {

			idletimeout = 0;

			/* Check for data on the device fd */
			if (!sess && FD_ISSET(DeviceFd,&InFdSet)) {	//SOCKET_CLIENT or fast
				count = OutBufferSize-OutBPos;
				count = read(DeviceFd,&OutBuffer[OutBPos],count);
				if (count > 0) {
#ifdef DEBUG_CNT
					tty_read += count;
#endif
				//	if(pconf->fast && (fd_buf > 0) || (pconf->alarm == 1) || (pconf->alarm == 3) ) {  //[RK]ALARM  [GY]2006/May/31
				//		check_file_size(fd_buf, 
				//				&OutBuffer[OutBPos], count, 
				//				(RetentionMode!=0), pconf, 0);
				//	}

					if (!RetentionMode) {
						OutBPos += count;
					}

					if (MaxBTicksTx > 0) {
						if (OutBPos == OutBufferSize) {
							if (pconf->fast) {
								EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
							} else {
								WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
							}
							OutBTicksTx = times(NULL);
						}
					} else {
						if (pconf->fast) {
							EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
						} else {
							WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
						}
					}
				} else {
					nsyslog(LOG_NOTICE,"Error reading from device, may be power lost");
					get_send_ModemState(DeviceFd, 1);
					break;
				}
			}

			/* Check for data on stdin, i.e the socket from the remote client */
			if (FD_ISSET(STDIN_FILENO,&InFdSet)) { 
				count = MaxContiguosFree(pInBuffer);
				if (!count) 
					XPRINTF("ISSET and CT=%d w=%d r=%d\n", count, pInBuffer->WrPos, pInBuffer->RdPos);
				if (count > 0) {
					// Reading the data from the user 
					count = read(STDIN_FILENO,&pInBuffer->Buffer[pInBuffer->WrPos], count);

					if (count <= 0) { // socket sess is stopped
				  		nsyslog(LOG_NOTICE,"Communication closed (input)");
						OutBPos = 0;
						if (sess && (sess->sess_flag & CY_SESS_FLAG_PMD)) {
							// exit from PM session
							pmc_logout(sess);
						}
						break;
					}

					if (!sess && !pconf->fast) { // treat socket_client
						pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
						goto SOCK_SKIP_CAS;
					}

					{ // begin CAS session treatment		
						unsigned char *pd,escape_char = 0, break_seq=0;
						if (sess && sess->sess_flag & CY_SESS_FLAG_PMD) {
							// user is in PM session
							for (data_count = 0, pd=&pInBuffer->Buffer[pInBuffer->WrPos];
								data_count < count; data_count++, pd++) {
								if (*pd == TNIAC) {
									pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
									kill(cy_shm->DB_pid, SIGIO);
									break;
								}
							}
						if (data_count == count) {
							pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
							pmc_treat_input(sess, write_data);
						}
							continue; 	// return to main loop
						} 
						// user is in CAS session
									
						if (pconf->break_seq && *pconf->break_seq != 0) {
							break_seq = 1;	// check break_seq
						}

						if (sess && pconf->protocol == P_SOCKET_SERVER && 
							sess->sess_flag & CY_SESS_FLAG_MENU) {
							escape_char = pconf->escape_char;
						} 
								
						for (data_count = 0, pd=&pInBuffer->Buffer[pInBuffer->WrPos]; 
							data_count < count; data_count++, pd++) {
							// Socket_server and has menu -> check escape_char
							if(escape_char && (*pd == escape_char)) { 
								cmd_pend = 1;
								count = data_count;
								break;
							}
							// CAS and has PM hotkey 
							if (pconf->pmkey && (*pd == pconf->pmkey)) {
								count = data_count;
								pmc_login(sess,pconf->pmoutlet, 0, write_data);
								pInBuffer->RdPos = pInBuffer->WrPos;
								break;
							}
							// CAS and has break sequence 
							if (break_seq && (pconf->protocol != P_SOCKET_SERVER_RAW) ) {
								check_break_sequence(data_count);
							}
						}
							
#ifdef DEBUG_CNT
						tot_read += count;
#endif
						// Linefeed suppression. If the last bytes consist of \r\n, eliminate
						// the \n. Solves problem where telnet from Win98 gives extra prompt
						// line when you press Enter.
						// Nei: added raw data with this feature  
						// 08/25/03 Nei: lf_suppression = 1 (CRLF expected; LF suppressed)
						if ((pconf->protocol == P_SOCKET_SERVER || pconf->protocol == P_SOCKET_SERVER_RAW)  && 
							pconf->lf_suppress == 1 && count >= 2 &&
							pInBuffer->Buffer[pInBuffer->WrPos + count-2] == '\r' &&
							pInBuffer->Buffer[pInBuffer->WrPos+count-1] == '\n') {
							count--;
						}
						else { // 08/25/03 Nei: lf_suppression = 2 (CRNULL expected; NULL suppressed)
							if ((pconf->protocol == P_SOCKET_SERVER || pconf->protocol == P_SOCKET_SERVER_RAW)  && 
								pconf->lf_suppress == 2 && count >= 2 &&
								pInBuffer->Buffer[pInBuffer->WrPos + count-2] == '\r' &&
								pInBuffer->Buffer[pInBuffer->WrPos+count-1] == 0) {
								count--;
							}
						}

						/* Updating the write position
							If the break sequence did not match the one configured, output 
							the current break sequence buffer; otherwise, output current stdin */
						if (!seq) {	
							if (seqPendingIndx) {
								seq_pending[seqPendingIndx++] = pInBuffer->Buffer[pInBuffer->WrPos];
								if ((count = MaxContiguosFree(pInBuffer)) > 0) {
									count = MIN(count, seqPendingIndx);
									memcpy(&pInBuffer->Buffer[pInBuffer->WrPos], seq_pending, count);
									pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
								}
							} else {
								pInBuffer->WrPos = (pInBuffer->WrPos + count) % BufferSize;
							}
							//Clear the break sequence pending index
							seqPendingIndx = 0;
						}

						if (RetentionMode) {
							RetentionMode = check_buffering_option(pInBuffer, fd_buf, DeviceFd, sess);
							if (!RetentionMode) {
								close(fd_buf);
							}
						}
						//[RK]Oct/04/02 Dell  - BugTrack #19
						//[RK]NOv/24/03 - Bug#1075
						if (sess && pInBuffer->WrPos != pInBuffer->RdPos && cy_shm->DB_pid) {
							kill(cy_shm->DB_pid, SIGIO);
						}
						if (cmd_pend) {
							break;
						}
					} // end of CAS session treatment
				}
			} else {
				/* Check if the buffer is not full and remote flow is off */
				if (TCPCEnabled && RemoteFlowOff &&
					IsBufferFull(pInBuffer, BufferSize/2) == False) {
					/* Send a flow control resume command */
					SendCPCFlowCommand(STDOUT_FILENO,TNASC_FLOWCONTROL_RESUME);
					RemoteFlowOff = 0;
				}
			}
SOCK_SKIP_CAS : 
			/* Check for output availability on the device */
			if (!sess && FD_ISSET(DeviceFd,&OutFdSet)) { //SOCKET_CLIENT
				EscRedirectChar(STDOUT_FILENO,DeviceFd,pInBuffer);
			}
			
			/* Check for output buffer timeout */
			if (MaxBTicksTx > 0 &&
				(times(NULL) - OutBTicksTx) * msTicks >= MaxBTicksTx) {
				if (!sess) {
					if (pconf->fast) {
						EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
					} else {
						WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
					}
				} else {
					if (!(sess->sess_flag & CY_SESS_FLAG_PMD)) {
						WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
					}
				}
				OutBTicksTx = times(NULL);
			}

		} else {
			//[RK]Sep/04/02 Dell
			if (got_int_flag == SIGIO) {
				got_int_flag = 0;
				continue;
			}

			// CAS and it is in PM session
			if (sess && (sess->sess_flag & CY_SESS_FLAG_PMD)) {
				if (got_int_flag == SIGUSR1) { // pmd is not ready
					got_int_flag = 0;
					pmc_logout(sess);	// exit PM session
					write_data(servernotready,0); // send to user PMD not ready
					continue;
				}
				if (got_int_flag == SIGUSR2) { // pmd answered request
					got_int_flag = 0;
					pmc_treat_sigusr2(sess,write_data);
					continue;
				}
			}

			if (got_int_flag && got_int_flag != SIGIO && rtn) {
				nsyslog(LOG_NOTICE,"Receive Signal %d", got_int_flag);
				if (!sess) get_send_ModemState(DeviceFd, 1);
				break;
			}
			if (rtn) {
				nsyslog(LOG_NOTICE,"Select error : %m");
				break;
			}

			if (pconf->idletimeout) {
				idletimeout += MinBTicks;
				if (idletimeout >= max_idle) {
					nsyslog(LOG_NOTICE,"Inactivity Timeout");
					if (!sess) get_send_ModemState(DeviceFd, 1);
					break;
				}
			}

			got_int_flag = 0;

			if (MaxBTicksTx > 0 &&
				(times(NULL) - OutBTicksTx) * msTicks >= MaxBTicksTx) {
		  		/* Polling timeout, send output buffer */
				if (!sess) {
					if (pconf->fast) {
						EscWriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos, 1, -1, 1);
					} else {
						WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
					}
				} else {
					if (!(sess->sess_flag & CY_SESS_FLAG_PMD)) {
						WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
					}
				}
		  		OutBTicksTx = times(NULL);
			}
		}

		if (MaxBTicks > 0 &&
				(times(NULL) - OutBTicks) * msTicks >= MaxBTicks) {
			//[RK]Dec/15/04 - Send RFC2217 Motification of Modem State
			if (!sess) get_send_ModemState(DeviceFd, 0);
		  	OutBTicks = times(NULL);
		}

#if 0
		nsyslog(LOG_NOTICE,"Total ReadS %d  writeS %d ReadT %d WriteT %d WriteTb %d",
			tot_read, tot_write, tty_read, tty_write, tty_bwrite);
#endif
		/* Resets the fd sets */
		FD_ZERO(&InFdSet);	

		/* Check if the buffer is not full */
		if (IsBufferFull(pInBuffer, 1) == False) {
			FD_SET(STDIN_FILENO,&InFdSet);
		} else {
			if (TCPCEnabled && !RemoteFlowOff) {
				/* Send a flow control suspend command */
				SendCPCFlowCommand(STDOUT_FILENO,TNASC_FLOWCONTROL_SUSPEND);
				RemoteFlowOff = 1;
			}
		}

		/* If input flow has been disabled from the remote client don't read from the device */
		if (InputFlow == True && !sess) {
			FD_SET(DeviceFd,&InFdSet);
		}

		FD_ZERO(&OutFdSet);

		/* Check if there are characters available to write */
		if (!sess) {
			if (IsBufferEmpty(pInBuffer) == False || tmp->cnt_buf) {
				FD_SET(DeviceFd,&OutFdSet);
			}
		}

		FD_ZERO(&FdExSet);
		FD_SET(STDIN_FILENO,&FdExSet);

	}   

	closing_fd = 1;

	if (sess && (sess->sess_flag & CY_SESS_FLAG_PMD)) {
		pmc_logout(sess);
	}

	if (!sess) {
		do {
			if (OutBPos) {
				WriteBuffer(STDOUT_FILENO,OutBuffer,OutBPos);
			}
			if (!IsBufferEmpty(pInBuffer) || (tmp && tmp->cnt_buf)) {
				EscRedirectChar(STDOUT_FILENO,DeviceFd,pInBuffer);
			}
		} while(OutBPos || !IsBufferEmpty(pInBuffer) || (tmp && tmp->cnt_buf));
	}

	if (cmd_pend) {
		sess->sess_flag |= 4;
		return(1);
	}

	if (!sess) { 	//[RK]
		alarm(4);
		tcdrain(STDOUT_FILENO);
		alarm(0);
	}

	if (DeviceFd != -1) {
		alarm(10);
		if (tcdrain(DeviceFd) < 0) {
			XPRINTF("tcdrain ttys error:%s", strerror(errno));
		}
		alarm(0);
	} else {
		while (!IsBufferEmpty(pInBuffer)) {
			sleep(1);
			sec++;
		}
		nsyslog(LOG_NOTICE,"Total seconds waiting buffer empty %d", sec);
		print_counters(0);
	}

	return(0);
}

/* Check for break sequence to send break */
static void check_break_sequence(int data_count) {

		/* If current stdin is ENTER */
		if ((pInBuffer->Buffer[pInBuffer->WrPos+data_count] == '\r' ||
			pInBuffer->Buffer[pInBuffer->WrPos+data_count] == '\n')) {
			if (seqPendingIndx)
				seqPendingIndx = 0;
			seq = 0;
			line_cleared = 1;
			return;
		}

		/* If current stdin is part of the break sequence string, and it is at the beginning 
			of a new prompt (line was cleared) */
		if (pInBuffer->Buffer[pInBuffer->WrPos+data_count] == pconf->break_seq[seqPendingIndx] && line_cleared == 1) {
			seq_pending[seqPendingIndx++] = pInBuffer->Buffer[pInBuffer->WrPos+data_count];	
			seq = 1;
			if (!pconf->break_seq[seqPendingIndx]) {
				if (!pai->sess) {
					nsyslog(LOG_INFO,"Client sent a break to %s", pconf->tty);
					tcsendbreak(DeviceFd, 
						pconf->break_interval?pconf->break_interval:500);
					line_cleared = 0;			
				} else {
					if (pai->st_sess == 1) {
						nsyslog(LOG_INFO,"Client sent a break to %s", pconf->tty);
						pai->sess->sendbreak = (pconf->break_interval)? pconf->break_interval : 500;
						line_cleared = 0;			
						if (cy_shm->DB_pid) { //[RK]Nov/24/03 Bug#1075
							kill(cy_shm->DB_pid, SIGIO);
						}
					}
				}
			}
		} else {		// If stdin is not part of the break sequence
			seq = 0;
			// If the previous stdin wasn't part of the break sequence, reset sequence index
			if (!line_cleared)			
				seqPendingIndx = 0;
			line_cleared = 0;
		}
}

void do_telnet_options(void)
{
	/* Socket setup flag */
	int SockParmEnable = 1;
	
	/* Generic socket parameter */
	int SockParm;

	syslog(LOG_NOTICE,"do telnet options");
	//[RK]June/03/04 - SSH/Telnet to IPDU
	if ((pconf->protocol != P_SOCKET_SERVER_RAW) &&
	    (pconf->protocol != P_IPDU)) {
		/* Ask the client for COM Port Control portocol */
		SendTelnetOption(STDOUT_FILENO,TNWILL,TNCOM_PORT_OPTION);
		SendTelnetOption(STDOUT_FILENO,TNWILL,TN_ECHO);
		SendTelnetOption(STDOUT_FILENO,TNWILL,TN_SUPPRESS_GO_AHEAD);
		SendTelnetOption(STDOUT_FILENO,TNWILL,TN_TRANSMIT_BINARY);
		SendTelnetOption(STDOUT_FILENO,TNDO,TN_TRANSMIT_BINARY);
	} else {
		if (pconf->protocol == P_IPDU) {
			SendTelnetOption(STDOUT_FILENO,TNWONT,TNCOM_PORT_OPTION);
			SendTelnetOption(STDOUT_FILENO,TNWILL,TN_ECHO);
			SendTelnetOption(STDOUT_FILENO,TNWILL,TN_SUPPRESS_GO_AHEAD);
			SendTelnetOption(STDOUT_FILENO,TNWONT,TN_TRANSMIT_BINARY);
			SendTelnetOption(STDOUT_FILENO,TNDONT,TN_TRANSMIT_BINARY);
		}
	}

	/* Setup sockets for low latency and automatic keepalive;
 	doesn't check if anything fails because failure doesn't prevent
 	correct functioning but only provides slightly worse behaviour */
	SockParm = IPTOS_LOWDELAY;
	setsockopt(STDIN_FILENO, SOL_SOCKET, SO_OOBINLINE,&SockParmEnable,sizeof(SockParmEnable));
	setsockopt(STDIN_FILENO,SOL_SOCKET,SO_KEEPALIVE,&SockParmEnable,sizeof(SockParmEnable));
	setsockopt(STDIN_FILENO,SOL_TCP,TCP_NODELAY,&SockParmEnable,sizeof(SockParmEnable));
	setsockopt(STDIN_FILENO,SOL_IP,IP_TOS,&SockParm,sizeof(SockParm));
	setsockopt(STDOUT_FILENO,SOL_SOCKET,SO_KEEPALIVE,&SockParmEnable,sizeof(SockParmEnable));
	setsockopt(STDOUT_FILENO,SOL_TCP,TCP_NODELAY,&SockParmEnable,sizeof(SockParmEnable));
	setsockopt(STDOUT_FILENO,SOL_IP,IP_TOS,&SockParm,sizeof(SockParm));
}


/*
 *  The 'more' methods for a socket_server connection
 */

