#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <termio.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>

#include "server.h"

LOG_DATA_BUFFER log_buffer = {0};

extern int     read_a_char(int);
extern void    write_a_line(char *);
extern void    write_data(char *, int);

struct termios old_tio;

void do_telnet_options(void);
int main_routine_socket(CY_SESS *sess);
int main_routine_pmsocket(struct auth *);

extern struct line_cfg		*pconf;
struct auth ai;
struct auth *pai = &ai;

/* Device file descriptor */
int DeviceFd = -1;

//[RK]Jun/08/06 - event notification
extern void writeevt(int, char*, ...);

/* Function executed when the program exits */
void ExitFunction(void)
  {
	CY_SESS *sess = pai->sess;
	CY_SESS *first_rw;
	int flag,flow_flag;
	int pid = getpid();
	int evt = 6;

    if ((!sess && !pconf->fast) || (sess && (sess->vet_pid != pid))) { //[RK]Dec/04/02 BUG#76
        nsyslog(LOG_INFO, "Socket exit on port %d (%s)",
            (pai->nasport + 1), pconf->tty); 
		//[RK]Dec/13/02 Bug#129
		if (!sess) { // socket_client
			clear_utmp_entry(pid);
		}
        return;
    }

	if (pconf->fast && (cy_shm->vet_pid[ai.nasport] != pid)) {
		return;
	}

	if (pconf->fast) {
		if (cy_shm->vet_pid[pai->nasport] == pid)
			cy_shm->vet_pid[pai->nasport] = 0;
	} else {
		flag = sess->sess_flag;
		flow_flag = sess->flow_flag;
		sess->vet_pid = 0;
	}
	
	if (cy_shm->vet_pool[pai->nasport] == pid) {
		cy_shm->vet_pool[pai->nasport] = 0;
	}
	
	/* Program termination notification */

#ifdef USE_PAM
	do_pam_close();
#else
	if (pai->do_acct) {
		pai->do_acct = 0;
		rad_acct(pai, 0);   
	}
#endif

	if (sess) {
		if (pai->st_sess == 1 || pai->st_sess == 3) {
			if (sess->sess_flag & 3) {
				write_a_line("\r\nSession killed by the administrator...\r\n");
			}
		}
	}

	close(0);
	close(1);
	close(2);

	if (DeviceFd != -1) {
		serial_close(DeviceFd, &old_tio);
	}

        nsyslog(LOG_INFO, "Socket exit on port %d (%s)",(pai->nasport + 1), pconf->tty); 
	if (sess) {
		if (pai->st_sess == 0 || pai->st_sess == 2) {
			clean_session_entry (sess);
			evt = 42;
			goto call_writeevt;
		}

		clean_session_entry (sess);

		if (!(first_rw = first_rw_session(pai->nasport))) {
			//There are no more read/write sessions 
			if (!(flag & 1)) {
				//cy_shm->vet_pool[pai->nasport] = 0;
				//No one is assuming the main session - kill all the sniff sessions
				kill_session_list(pai->nasport, 2);
			}
		} else {
			evt = 42;
		}
		if (flow_flag && cy_shm->DB_pid) { //[RK]Nov/24/03 Bug#1075
			kill(cy_shm->DB_pid,SIGIO);
		}
//[RK]Nov/26/02 - W-CAS	clear_utmp_entry(getpid()); //[FW]Oct/25/02 Bug #40. Handles telnet 
				    //to slave's serial port.
	}

call_writeevt:

	//[RK]May/10/06 - event notification
	writeevt(evt,"ssii",pai->login, pconf->serverfarm, (pai->nasport+1),
			(pconf->protocol == P_SOCKET_SERVER_RAW) ? 3 : 2);

  }
  
/* Main function */

int got_signal_int = 0;

static void got_int(int sig)
{
	if (sig != SIGIO) { //[RK]Oct/31/02 BUG#76
		got_signal_int = 1;
	}
}

int do_socket_auth(struct auth *pai)
{
    struct line_cfg *lc = &lineconf[pai->nasport];
    char s[1024];
	int  len, i, r;
   
	if (lc->authtype == AUTH_NONE ||
		(pai->proto != P_SOCKET_SERVER &&
		 pai->proto != P_SOCKET_SERVER_RAW)) {
		if ( ! (pai->proto == P_IPDU && (lc->pmsessions & PMSESS_TELNET))) {
	//[RK]Nov/26/02 - W-CAS
        //update_utmp("%L", lineconf [pai->nasport].utmp_from, pai,
	//		lineconf[pai->nasport].syswtmp);
		//[RK]Jun/08/06 - event notification
		writeevt(5,"ssii",pai->login, pconf->serverfarm, (pai->nasport+1), 4);
		return(0);
		}
	}

	/*
	 *  Show the banner and say hello.
	 */
	if (lc->issue && lc->issue[0]) {
		expand_format(s, sizeof(s), lc->issue, pai);
		if (s) {
			len = strlen(s);
			write(STDOUT_FILENO, s, len);
		}
	}

	for(i=0; i < 3; i++) {
#ifndef USE_PAM
		r = do_login1(pai);

		if (r == 2)	continue;

		if (r < 0)	return(1);

		/*
		 *	Do authentication if needed.
		 */
#endif
		if (!do_local_or_server_authentication(pai, 0)) {
			break;
		}
		//[RK]Jun/08/06 - event notification
		writeevt(7,"ssii",pai->login, pconf->serverfarm, (pai->nasport+1),
				(pconf->protocol == P_SOCKET_SERVER_RAW)? 3: 2);

	}

	if (i >= 3) {
		/*
		 *	3 login failures - exit!
		 */
		usleep(250000);
		maketermsane(0);
		return(1);
	}

	//[RK]Jun/08/06 - event notification
	writeevt(5,"ssii",pai->login, pconf->serverfarm, (pai->nasport+1), 4);

#ifndef USE_PAM
	if (pai->next_token) {
		if (pai->message[0][0]) {
			for (r = 0; r < pai->msn; r++) {
				write(STDOUT_FILENO, pai->message[r], strlen(pai->message[r]));
				pai->message[r][0] = 0;
			}
			pai->msn = 0;
			write(STDOUT_FILENO, "\r\n", 2);
		} else {
			write(STDOUT_FILENO, "\r\nWait for the next token !!! \r\n\r\n", 34);
		}
		do_next_token(pai, 1);
		if (do_local_or_server_authentication(pai, 2)) {
			return(1);
		}
	}

	if (pai->do_acct == 1) {
		/*
		 *	While messing around with accounting, we have to
		 *	block, but not ignore, SIGHUP and friends.
		 */
		block(SIGHUP);
		block(SIGTERM);

		rad_acct(pai, 1);

		unblock(SIGHUP);
		unblock(SIGTERM);

	}
#endif
	return(0);
}

int do_socket(struct auth *ai)
{
	int	addr_len, client_fd, signals;
	struct sockaddr_in	sock_client;
	struct sigaction sa = {0};
	char *p;
	int fd;

	atexit(ExitFunction);

	sa.sa_handler = got_int;
	sigaction(SIGINT,  &sa, NULL);
	sigaction(SIGTERM, &sa, NULL);
	sigaction(SIGIO, &sa, NULL);   //[RK]Oct/31/02 BUG#76
	unblock(SIGINT);
	unblock(SIGTERM);
	unblock(SIGIO);		//[RK]Oct/31/02 BUG#76

	if (pconf->protocol == P_SOCKET_SERVER ||
 		pconf->protocol == P_SOCKET_SERVER_RAW || 
 		//[RK]June/03/04 SSH/Telnet to IPDU
 		(pconf->protocol == P_IPDU && (pconf->pmsessions & PMSESS_TELNET))) {

		do_telnet_options();

		if (do_socket_auth(pai)) {
			nsyslog(LOG_NOTICE,"Authentication Failure");
			return(0);
		}

		set_remote_address(STDIN_FILENO, pai);

		//[RK]June/03/04 SSH/Telnet to IPDU
		if (pconf->protocol == P_IPDU) {
			title(2, NULL, "-RW_srv %s", get_tty_text(pconf));
			cy_shm->vet_pid[pai->nasport] = getpid();
			cy_shm->tty_user[pai->nasport] = shm_strdup(pai->login);
			cy_shm->st_time[pai->nasport] = time(0);
			main_routine_pmsocket(pai);
			return(0);
		}

		p = term_in_use(pconf, 3);

		if (pconf->fast) {

			if (p) {
				nsyslog(LOG_NOTICE,"Port in use");
				return(0);
			}

			if ((fd = getty(ai, 1, &old_tio)) < 0) {
				nsyslog(LOG_ERR, "Serial [%s] Open error %m",
						lineconf[ai->nasport].tty);
				return(0);
			}

			title(2, NULL, "-RW_srv %s", get_tty_text(pconf));
			cy_shm->vet_pid[ai->nasport] = getpid();
			cy_shm->tty_user[ai->nasport] = shm_strdup(ai->login);
			cy_shm->st_time[ai->nasport] = time(0);

			DeviceFd = fd;
			main_routine_socket(NULL);
			return(0);

		} else {

			check_mult_session(ai, 3,p, read_a_char, write_data, main_routine_socket);
			return(0);

		}
	}

	sock_client.sin_family = AF_INET;
	sock_client.sin_addr.s_addr = pconf->host;
	if (ai->loginport) {
		sock_client.sin_port = ai->loginport;
	} else {
		sock_client.sin_port = pconf->socket_port ?
			pconf->socket_port : TELNET_PORT;
	}
	addr_len = sizeof(sock_client);

	DeviceFd = dup(0);
	close(0);
	close(1);
	close(2);

	if ((client_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		nsyslog(LOG_ERR, "Create client socket error: %s", strerror(errno));
		return(0);
	}

	dup(client_fd);
	dup(client_fd);

	if (connect(client_fd, (struct sockaddr *)&sock_client, addr_len) < 0) {
		nsyslog(LOG_ERR, "Connect socket error: %s", strerror(errno));
		return(0);
	}

	nsyslog(LOG_INFO, "socket: connected to %s port %d device /dev/%s\n",
		inet_ntoa(sock_client.sin_addr), sock_client.sin_port, pconf->tty);
	update_utmp("%L", lineconf [pai->nasport].utmp_from, pai,
		lineconf[pai->nasport].syswtmp);

	if (sock_client.sin_port==TELNET_PORT)
		do_telnet_options();

	main_routine_socket(NULL);
	return(0);
}

