#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#define MAXMSG	900
#define MAXNAME	64

char domain_name[MAXNAME] = {0};
char host_name[MAXNAME] = {0};

char *toaddr=NULL;	// TO: <name>[,<name>]
char *ccaddr=NULL;	// CC: <name>[,<name>]
char *bccaddr=NULL;	// BCC: <name>[,<name>]
char *rpaddr=NULL;	// Reply-To: <name>[,<name>]
char *fromaddr=NULL;	// From: <name>
char *subject=NULL;	// Subject: <text#>
char *message=NULL;	// <bodys message>
char *smtp_server=NULL;	// <IP or name of smtp server>
int smtp_port=0;	// smtp port
int sock_fd = 0;

int get_socket(void)
{
	struct hostent *hostp;
	struct servent *servp;
	int achou=0;
	struct sockaddr_in addr;
	char * pname = smtp_server;
	struct in_addr ip_addr;

	//verify smtp_server is name or ip address
	if (isdigit(*pname)) { // is digit
		if (pname=strchr(smtp_server, (int)'.')) {
			if (isdigit(*pname)) { // after i is digit
				if (inet_aton(smtp_server, &ip_addr) == 0) {
					fprintf(stderr,"sendmail: server SMTP invalid (%d):%s\n",errno,strerror(errno));
					return 0;	// IP address is invalid
				}
				addr.sin_addr = ip_addr;
				achou = 1;
			}
		}
	}
	if (!achou) {
		if ((hostp = gethostbyname(smtp_server)) == NULL) {
			fprintf(stderr,"sendmail: server SMTP invalid (%d):%s\n",errno,strerror(errno));
			return 0;	// name is invalid
		}
		addr.sin_addr = *((struct in_addr *)hostp->h_addr);
	}

	if (smtp_port == 0) {
		if ((int)(servp = getservbyname("smtp","tcp")) == -1) {
			smtp_port = 25;	//Default value
		} else {
			smtp_port = servp->s_port;
		}
	}

	if ((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr,"sendmail: socket (%d):%s\n",errno,strerror(errno));
		return 0; // not open socket
	}
	
	addr.sin_family = AF_INET;
	addr.sin_port = htons(smtp_port);
	bzero(&addr.sin_zero,8);

	if (connect(sock_fd, (struct sockaddr *)&addr, sizeof(struct sockaddr)) == -1) {
		fprintf(stderr,"sendmail: connect (%d):%s\n",errno,strerror(errno));
		close(sock_fd);
		return 0; // not connect
	}
	getdomainname(domain_name,MAXNAME);
	gethostname(host_name,MAXNAME);
	if (strcmp(domain_name,"(none)") == 0) {
		domain_name[0] = 0x00;
	}
	return 1;
}

int wait_answer(char * match) 
{
	int nread, ret, prev;
	fd_set inputs;
	struct timeval tv;
	char in_buf[MAXMSG];
	int num_bytes;
  
	nread = prev = 0;
  
	memset(in_buf, 0, MAXMSG);

	FD_ZERO (&inputs);
	FD_SET (sock_fd, &inputs);

	tv.tv_sec = 10;		// 10 secs
	tv.tv_usec = 0;

	/* wait for data to arrive on the line */
	if (select(FD_SETSIZE, &inputs, NULL, NULL, &tv) <= 0)  {
		fprintf(stderr,"sendmail: select (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	
	ioctl(sock_fd, FIONREAD, &nread);
	while (nread != prev) {
		sleep(1);	// wait message
		prev = nread;
		ioctl(sock_fd, FIONREAD, &nread);
	}
	if (nread > MAXMSG) nread=MAXMSG;
	if ((nread=read(sock_fd, in_buf,nread )) <=0) {
		fprintf(stderr,"sendmail: read (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	in_buf[nread] = 0x00;
	ret = (int) strstr(in_buf, match);

	if (nread != prev) {
		nread=read(sock_fd, in_buf,prev-nread );
	}
	return (!ret);	 
}

int send_rcpts(char * names)
{
	char out_buf[MAXNAME], *pname, *p1;

	pname = p1 = names;

	while (*pname && p1) {
		if ((p1 = strchr(pname, (int)',')) != NULL) { 
			*p1 = 0x00;
		}
		sprintf(out_buf, "RCPT TO: <%s>\r\n", pname);
		if( write(sock_fd, out_buf, strlen(out_buf)) < 1) {	
			fprintf(stderr,"sendmail: write rcpto (%d):%s\n",errno,strerror(errno));
			return 1;
		}
		// wait answer to FROM message = 250
		if (wait_answer("250")) {
			fprintf(stderr,"sendmail: RCPT-TO answer 250 : error\n");
			return 1;
		}
		if (p1) {
			*p1 = ',';
			pname = p1+1;
		}
	}
	return 0;
}

int send_mail(void)
{
	int i;
	char out_buf[MAXMSG];
	time_t msgtime;  //[RK]Jan/29/03 Bug#198/199

	//wait message with 220 - server anounce 
	if (wait_answer("220")) {
		fprintf(stderr,"sendmail: first message 220 : error\n");
		return 1;	// not received
	}

	if (domain_name[0]) {
		sprintf(out_buf, "HELO %s.%s\r\n", host_name,domain_name);
	}else{
		sprintf(out_buf, "HELO %s\r\n",host_name);
	}
	if (write(sock_fd, out_buf, strlen(out_buf)) < 1) {
		fprintf(stderr,"sendmail: write HELO (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	//wait answer to HELO message = 250
	if (wait_answer("250")) {
		fprintf(stderr,"sendmail: HELO answer 250 : error\n");
		return 1;
	}
		
	sprintf(out_buf, "MAIL FROM: <%s>\r\n", fromaddr);
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1) {	
		fprintf(stderr,"sendmail: write MAIL FROM (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	// wait answer to FROM message = 250
	if (wait_answer("250")) {
		fprintf(stderr,"sendmail: FROM answer 250 : error\n");
		return 1;
	}
		
	if (toaddr && send_rcpts(toaddr)) {
		return 1;
	}

	if (ccaddr && send_rcpts(ccaddr)) {
		return 1;
	}

	if (bccaddr && send_rcpts(bccaddr)) {
		return 1;
	}
  
	sprintf(out_buf,"DATA\r\n");
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write DATA (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	// wait answer to DATA message = 354
	if (wait_answer("354")) {
		fprintf(stderr,"sendmail: DATA answer 354 : error\n");
		return 1;
	}

	// mount message 
	// comment From : ... To: ...
	sprintf(out_buf,"From: %s\r\nTo: %s\r\n",fromaddr,toaddr);
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write Com FROM/TO (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	
	if (ccaddr) { // comment Cc:
		sprintf(out_buf,"Cc: %s\r\n",ccaddr);
		if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
			fprintf(stderr,"sendmail: write Com Cc (%d):%s\n",errno,strerror(errno));
			return 1;
		}
	}

	if (bccaddr) { // comment Bcc:
		sprintf(out_buf,"Bcc: %s\r\n",bccaddr);
		if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
			fprintf(stderr,"sendmail: write Com Bcc (%d):%s\n",errno,strerror(errno));
			return 1;
		}
	}

	if (rpaddr) { // comment Reply-To
		sprintf(out_buf,"Reply-To: %s\r\n",rpaddr);
		if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
			fprintf(stderr,"sendmail: write Reply-To (%d):%s\n",errno,strerror(errno));
			return 1;
		}
	}

	if (subject) { // comment Subject
		sprintf(out_buf,"Subject: %s\r\n",subject);
		if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
			fprintf(stderr,"sendmail: write Subject (%d):%s\n",errno,strerror(errno));
			return 1;
		}
	}

	//[RK]Jan/29/03 Bug#198/199
	msgtime = time(NULL);
	strftime(out_buf, MAXMSG-10, "Date: %a, %d %b %Y %H:%M:%S %z\r\n",gmtime(&msgtime));
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write Date (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	sprintf(out_buf,"Message-ID: <%d@%s>\r\n",msgtime+getpid(),host_name);
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write Message-ID (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	sprintf(out_buf,"X-Mailer: sendmail \r\n");
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write Mailer (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	sprintf(out_buf,"\r\n%s\r\n",message); //[RK]Jan/29/03 Bug#198/199
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write body (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	// end the message
	sprintf(out_buf,"\r\n.\r\n");
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1){
		fprintf(stderr,"sendmail: write fim (%d):%s\n",errno,strerror(errno));
		return 1;
	}

	// wait answer to message = 250
	if (wait_answer("250")) {
		fprintf(stderr,"sendmail: MESSAGE answer 250 : error\n");
		return 1;
	}

	sprintf(out_buf,"QUIT\r\n");
	if( write(sock_fd, out_buf, strlen(out_buf)) < 1) {
		fprintf(stderr,"sendmail: write QUIT (%d):%s\n",errno,strerror(errno));
		return 1;
	}
	if (wait_answer("221")) {
		fprintf(stderr,"sendmail: QUIT answer 221 : error\n");
		return 1;
	}

	return 0;
}

void print_usage(void)
{
	printf ("\nsendmail ver. 1.0.1\n");
	printf ("\nUsage:\n");
	printf ("sendmail -t toaddr [-c ccaddr] [-b bccaddr] [-r rpaddr] -f fromaddr -s subject -m body -h server [-p port]\n");
	printf ("\nWhere: \n");
	printf ("       toaddr  = field 'TO'. format: name@domain[,name@domain]\n");
	printf ("       ccaddr  = field 'CC'. format: name@domain[,name@domain] (opt)\n");
	printf ("       bccaddr = field 'BCC'. format: name@domain[,name@domain] (opt)\n");
	printf ("       rpaddr = field 'Reply-To'. format: name@domain (opt)\n");
	printf ("       fromaddr = field 'FROM'. format: name@domain\n");
	printf ("       subject = field 'Subject'. format: \"text\"\n");
	printf ("       body = body of the message (max. %d chars). format: \"text\"\n",MAXMSG);
	printf ("       server  = SMTP server name or IP address\n");
	printf ("       port   = TCP port on the server (if non-standard)\n");
}

int main(int argc, char *argv[])
{
	int i;

	setbuf(stderr,0);

	if (argc < 11) {
		print_usage(); 
		exit(1);
	}

	for(i=1; (i+1)<argc; i+=2){
	
		if (*argv[i] != '-') continue;

		switch(*(argv[i]+1)) {
			case 't':	// TO:
				toaddr = (char *)strdup(argv[i+1]); break;

			case 'c': 	// CC:
				ccaddr = (char *)strdup(argv[i+1]); break;
     
			case 'b': 	// BCC:
				bccaddr = (char *)strdup(argv[i+1]); break;
     
			case 'r': 	// Reply-To:
				rpaddr = (char *)strdup(argv[i+1]); break;
     
			case 'f': 	// From:
				fromaddr = (char *)strdup(argv[i+1]); break;
     
			case 's': 	// Subject:
				subject = (char *)strdup(argv[i+1]); break;
     
			case 'm': 	// Message
				message = (char *)strdup(argv[i+1]); break;
     
			case 'h': 	// SMTP server
				smtp_server = (char *)strdup(argv[i+1]); break;
     
			case 'p': 	// Port
				smtp_port = atoi(argv[i+1]); break;
     
			default: print_usage(); exit(1);
		}
	}

	if ((toaddr==NULL) || (fromaddr==NULL) || (subject==NULL) || 
		(message==NULL) || (smtp_server==NULL)) {
		print_usage();
		exit(1);
	}

	if (strlen(message) > MAXMSG) {
		print_usage();
		exit(1);
	}

	if (get_socket() == 0) {
		exit(1);	// dont open socket
	}

	i = send_mail();
	close(sock_fd);
	exit(i);
}
