#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <../../linux/include/linux/cy_sm.h>

static int got_sig_term = 0;
u_char bmc_msg_counter = 0x10;

void unblock(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_UNBLOCK, &set, NULL);
}

static void term_handler(int sig)
{
	got_sig_term = 1;
	signal(sig, SIG_IGN);
}

int ATXDetectConnections(int *f, unsigned char *status)
{
	int fl, fw;
	char fan[32], ch[8] = {0};

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	strcpy(fan, "/dev/sm_atx");

	/* open atx device */
	if ((fw = open(fan, O_RDWR|O_NONBLOCK)) < 0) {
		close(fw);
		return 0;
	}
	*f = fw;

	/* read atx port */
	read(fw, &ch[0], 1);
	*status = ch[0];

	/* write atx port */
	ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
	ch[1] = SM100_POWER_ON|MB_POWER_ENABLE;						/* set value 1 and disable MB */
	write(fw, &ch[0], 2);

	/* sleep 100 ms */
	usleep(100000);

	/* read atx port */
	read(fw, &ch[0], 1);
	
	if (!(ch[0] & POWER_ON_MB)) {
		close(fw);
		return 0;		/* port not in loop */
	}

	/* write atx port */
	ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
	ch[1] = MB_POWER_ENABLE;									/* set value 0 and disable MB */
	write(fw, &ch[0], 2);

	/* sleep 100 ms */
	usleep(100000);

	/* read atx port */
	read(fw, &ch[0], 1);
	
	if (ch[0] & POWER_ON_MB) {
		close(fw);
		return 0;		/* port not in loop */
	}

	return(1);
}

int ATXTestPortsOnce(int fw)
{
	char ch[8] = {0};
	
	/* write atx port */
	ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
	ch[1] = SM100_POWER_ON|MB_POWER_ENABLE;						/* set value 1 and disable MB */
	write(fw, &ch[0], 2);

	/* sleep 100 ms */
	usleep(100000);

	/* read atx port */
	read(fw, &ch[0], 1);
	
	if (!(ch[0] & POWER_ON_MB)) {
		return 0;		/* port not in loop */
	}

	/* write atx port */
	ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
	ch[1] = MB_POWER_ENABLE;									/* set value 0 and disable MB */
	write(fw, &ch[0], 2);

	/* sleep 100 ms */
	usleep(100000);

	/* read atx port */
	read(fw, &ch[0], 1);
	
	if (ch[0] & POWER_ON_MB) {
		return 0;		/* port not in loop */
	}

	return(1);
}

void ATXResetPort (int fw, unsigned char status)
{
	char ch[8] = {0};
	
	/* write atx port */
	ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
	ch[1] = status;												/* set original value */
	write(fw, &ch[0], 2);

	close(fw);

}

int IPMITestPortsOnce(int fw, char device)
{
	char ch[8] = {0};

	/* read I2C device via ipmi port */
	ch[0] = device;				/* address on the target device */
	ch[1] = 0xfe;				/* device ID address */
	read(fw, &ch[0], 1);
	
	if (ch[0] != 0x0a) {
		return 0;				/* device ID does not match */
	}

	ch[0] = device;				/* address on the target device */
	ch[1] = 0xff;				/* manufacture ID address */
	read(fw, &ch[0], 1);
	
	if (ch[0] != 0x4d) {
		return 0;				/* manufacture ID does not match */
	}

	ch[0] = device;				/* address on the target device */
	ch[1] = 0x14;				/* RRHL2 address */
	read(fw, &ch[0], 1);
	
	if (ch[0] != 0x7f) {
		return 0;		/* RRHL2 default value does not match */
	}
	
	return(1);
}


int IPMIOpenPort(int *f)
{
	int fl, fw;
	char fan[32];

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	strcpy(fan, "/dev/sm_sensor");

	/* open ipmi device */
	if ((*f = fw = open(fan, O_RDWR|O_NONBLOCK)) < 0) {
		close(fw);
		return 0;
	}
	return 1;
}


void IPMIResetPort (int fw)
{
	close(fw);
}


