#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "cyconst.h"
#include "macro.h"
#include "cyclades.h"
#include "cystruct.h"
#include "cyfunc.h"
#include "version.h"

#include <rot_nports.h>

#define CMD_RTS		1
#define CMD_RTS0	16
#define CMD_DTR		2
#define CMD_DTR0	32
#define CMD_BREAK	4
#define CMD_PCONV	8

#define FLOW_NONE	0
#define FLOW_HARD	1
#define FLOW_SOFT	2

#define PARITY_NONE	1
#define PARITY_EVEN	3
#define PARITY_ODD	2

int interactive = 1;
int commands = 0;
char *str_chat = NULL;
int portNumber = -1;
int DataSize = CS8;
int StopBits = 1;
int Parity = PARITY_NONE;
int flow = FLOW_NONE;

int bInterval = 0;
int tInterval = 0;
int rInterval = 0;
int flag_open_close = 0;

struct termios old_tio;

#define DEFAULTBAUDRATE B9600

struct MachineType mt;

int baudRate = DEFAULTBAUDRATE;

/** board structure */
static cyboard_t board;

// Dec/21/01 - Alex Ura: Added new speeds: 230400, 460800, 921600 
static int validRates[] = {110, 150, 300, 600, 1200, 2400, 4800, 
		    9600, 14400, 19200, 28800, 38400, 57600, 76800, 115200, 230400,
		    460800, 500000, 576000, 921600, -1};

// Dec/21/01 - Alex Ura: Added new speeds: 230400, 460800, 921600 
static int baudRates[] = {
 B110, B150, B300, B600, B1200, B2400, B4800,
 B9600, B14400, B19200, B28800, B38400, B57600, B76800, 
 B115200, B230400, B460800, B500000, B576000, B921600, 0 };

int numberOfBoards = 0;

struct termios raw_term, std_term;

/** global variables that are set during argument passing */
int noQuit = 0; /* show Quit option in the main menu or not? */

struct option Args[] = {
    {"noquit", 0, NULL, 1},
    {"speed", 1, NULL, 3},
    {NULL, 0, NULL, 0},
};

char *WarnTitle =	"\tHW Test/Linux\r\n\n"
			"\tThis tool is for internal use ONLY!\r\n\n"
			"\tIt should not be used if the serial port is\r\n"
			"\tconfigured and/or running.\r\n";

char *mainMenu[] = { 
    "\t1. Single Port Operations\r\n", 
    "\t2. Test Asynchronous Ports\r\n",
#ifdef CONFIG_SM100
    "\t3. Test ATX Ports\r\n",
    "\t4. Test IPMI Ports\r\n",
    "\t5. Test ALL Ports\r\n",
    "\t6. Quit\r\n"
#else
#ifdef CONFIG_TS1H
    "\t3. Test ANALOG Ports\r\n",
    "\t4. Test DIGITAL Ports\r\n",
    "\t5. Test ALL Ports\r\n",
    "\t6. Quit\r\n"
#else
    "\t3. Quit\r\n"
#endif
#endif
};

char *functionMenu[] = {
    "\t1. Port Conversation\r\n",
    "\t2. Test Signals Manually\r\n"
};

static int last_fd = -1;
static int saveMLines;

void serial_close(int fd)
{
	int MLines;

	if (fd == -1) {
		return;
	}

	if (!(flag_open_close & 2)) {
		if ((mt.board_type == 12) || (mt.board_type == 16)) {	/* TS100 or SM100 */
			MLines = 0x20000;
			ioctl(fd, TIOCSERCONFIG, &MLines);
		}
		tcsetattr(fd, TCSANOW, &old_tio);
		ioctl(fd,TIOCMSET,&saveMLines);
	}
	close(fd);
	last_fd = -1;
}

/*
 *	Open the serial line
 */
int serial_open(int portNumber)
{
	int fd;
	struct termios tio;
	char tty[64];
	int MLines;

	snprintf(tty, sizeof(tty), "/dev/ttyS%d", portNumber);

	if ((fd = open(tty, O_RDWR|O_NOCTTY|O_NONBLOCK)) < 0) {
		return -1;
	}

	tcgetattr(fd, &tio);
	tcgetattr(fd, &old_tio);
	ioctl(fd,TIOCMGET,&MLines);
	saveMLines = MLines;

	if (flag_open_close & 1) {
		last_fd = fd;
		return(fd);
	}

	MLines = MLines | TIOCM_DTR;
	ioctl(fd,TIOCMSET,&MLines);

	if ((mt.board_type == 12) || (mt.board_type == 16)) {	/* TS100 or SM100 */
		MLines = 0x20001;
		ioctl(fd, TIOCSERCONFIG, &MLines);
	}

	tio.c_iflag&=~(IGNBRK|IGNCR|INLCR|ICRNL|IUCLC|IXANY|IXON|IXOFF|INPCK|ISTRIP);
	tio.c_oflag&=~(IGNBRK|IGNCR|INLCR|ICRNL|IUCLC|IXANY|IXON|IXOFF|INPCK|ISTRIP);
	tio.c_iflag|=(BRKINT|IGNPAR);
	tio.c_oflag&=~(OPOST|ONLCR);
	tio.c_lflag&=~(XCASE|ECHONL|NOFLSH|ICANON|ISIG|ECHO);

	tio.c_cflag = CREAD|CS8|CLOCAL;

	switch (flow) {
	case FLOW_HARD:
		tio.c_cflag |= CRTSCTS;
		break;
	case FLOW_SOFT:
		tio.c_iflag |= IXON|IXOFF;
		tio.c_cc[VSTART] = 17;
		tio.c_cc[VSTOP]  = 19;
		break;
	case FLOW_NONE:
	default:
		break;
	}

	switch (StopBits) {
	case 2:
		tio.c_cflag = tio.c_cflag | CSTOPB;
		break;
	case 1:
	default:
		/* there is no support to 1.5 stop bits defaults to 1 */
		tio.c_cflag = tio.c_cflag & ~CSTOPB;
		break;
	}

	switch (Parity) {
	case 2:
		tio.c_cflag = tio.c_cflag | PARENB | PARODD;
		break;
	case 3:
		tio.c_cflag = (tio.c_cflag | PARENB) & ~PARODD;
		break;
	case 1:
	default:
		/* There's no support for MARK/SPACE parity so sets no parity */
		tio.c_cflag = tio.c_cflag & ~PARENB;
		break;
	}

	tio.c_cflag = tio.c_cflag & ((tio.c_cflag & ~CSIZE) | DataSize);

	// Setting speed
	cfsetospeed(&tio, baudRate);
	cfsetispeed(&tio, baudRate);
		
	tcsetattr(fd, TCSANOW, &tio);

	last_fd = fd;

	return fd;
}

static int valid_speed(int portSpeed)
{
	int i;
	for (i = 0; validRates[i] != -1; i++) {
	    if (validRates[i] == portSpeed) {
			baudRate = baudRates[i];
			return(1);
		}
	}
	return(0);
}

/**
 * =======================================================================
 * int 
 * GetMenuSelection (char* title, char** menu,
 *		     int numberOfItems, int allowEscape, board_t* b)
 *
 * This presents a menu passed to it as an array of char* items and returns
 * the number that user selected. The number is in the range from 1 to
 * numberOfItems, which is the third argument denoting the number of items
 * in the array menu. If fourth argument is not NULL, then the information
 * about the board is printed above the menu. The layout in this case is like
 * this:
 *
 * BOARD INFO (3) lines
 *
 * MENU TITLE
 *
 * MENU entries.
 *
 * The function does not return until a valid choice is entered.
 *
 * LIMITATIONS:
 * Currently only one-degit choices are supported. That means that the 
 * maximum number of entries in the menu is 9.
 * =======================================================================
 */
int
GetMenuSelection (char *title, char **menu, int numberOfItems, 
		  int allowEscape, cyboard_t *b)
{
    int i;
    int choice;

    do {
	CLS ();
	printf ("\n\n\n\n\n\r");

	printf ("%s", title);
	printf ("\n\n\r");

	for (i = 0; i < numberOfItems; i++)
	    printf (menu[i]);

	if (numberOfItems == 0) {
	    printf ("\n\r\tPress any key to continue or <ESC> to cancel ==> ");
	} else
	if (!allowEscape) {
	    printf ("\n\r\tSelect option ==> ");
	} else {
	    printf ("\n\r\tSelect option or press <ESC> to return ==> ");
	}
	fflush (stdout);
	choice = GetKeyPressed ();
	ClearInput ();
	if ((choice == ESC_CHAR) && allowEscape) {
	    return 0;
	}
	if (numberOfItems == 0) {
		return(1);
	}
	choice = choice - '0';
    }
    while (!IN_RANGE (choice, 1, numberOfItems));

    printf ("%d", choice);
    fflush (stdout);
    return choice;
}

/* 
 * =======================================================================
 * int Initialize (void)
 * 
 * This function is called once and it performs some initial configuration.
 * In particular, it tries to open MEMDEV (which is defined as /dev/mem) file
 * and sets the terminal in the raw mode. The return value is 1 on success.
 * The current implementation terminates the whole program in any of the steps
 * failed. This behaviour is likely to change in future. This function does
 * not take any arguments.
 * =======================================================================
 */
int
Initialize (void)
{
    /*
     * Save the standard stdin config. and change it to raw mode 
     */
    if (tcgetattr (0, &std_term) == -1) {
		printf ("Cannot get TERM attributes.\n\r");
		exit (-1);
    }
    memset (&raw_term, 0, sizeof (raw_term));
    raw_term.c_cflag = (std_term.c_cflag & CBAUD) | CS8 | CREAD;
    raw_term.c_iflag = IGNPAR | IGNBRK;
    SET_RAW_MODE_INPUT ();
	if (interactive) {
    	printf ("Please wait while program is examining the hardware.\r\n");
	}
    fflush (stdout);
    return 1;
}


/* 
 * =======================================================================
 * int DetectBoards (void)
 * This function is just a simple wrapper for the calls to board detection
 * functions (see detect.c). It returns the number of boards detected. As 
 * a "side effect" it fills the array board_t board[MAX_BOARDS]. 
 * Sequential invokations of this function will overwrite the element of 
 * the board array, since the array is always filled starting with 0 element.
 * =======================================================================
 */
int
DetectBoards (void)
{
    if (numberOfBoards > 0)
	return numberOfBoards;

	board.nports = mt.nports;
	strcpy(board.name, mt.name);

    numberOfBoards = DetectAsyncZbusBoards (&board);
    return numberOfBoards;
}

int
PortTest (void)
{
    int n;
    int finished = 0;
    int were_errors = 0;
    asyncboard_t *currentBoard;

    CLS ();
    
    printf ("Detecting loopbacks on the present boards:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    currentBoard = &(board.type.async);
    fflush (stdout);
    n = YDetectConnections (currentBoard);
    printf ("%d port(s) selected for testing.\r\n", n);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (100000L);
    if (n == 0) {
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    if (were_errors) {
	printf ("There were errors. Press any key to continue testing. ");
	GetKeyPressed ();
	ClearInput ();
    }
    n = 0;     	/*
		 * this variable is now used to count the number of passes 
		 */
    currentBoard->porterrors = 0;
    do {
	YTestPortsOnce (currentBoard);
	switch (WasKeyPressed ()) {
	    case -1: /* no input */
		break;
	    case ESC_CHAR:
		finished = 1;
	    default:
		ClearInput ();
		break;
	}
	n++;
	CLS ();
	PrintStatsHeader ();
	PrintBoardStats (currentBoard, 0, n);
	printf ("\n\r\tPress <ESC> to stop the test.\r");
	fflush (stdout);
    } while (!finished);

    for (n = 0; n < currentBoard->channels; n++) {
	ResetPort (currentBoard, n);
    }
    currentBoard->porterrors = 0;
    printf ("\tPress any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}

#ifdef CONFIG_SM100
int
ATX_Test (void)
{
    int n, fw;
	int finished = 0, rx = 0, errors = 0;
	unsigned char status;

    CLS ();
    
    printf ("Detecting cabling on the ATX port:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);
    n = ATXDetectConnections (&fw, &status);
    printf ("%d port(s) selected for testing.\r\n", n);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (100000L);
    if (n == 0) {
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    if (errors) {
	printf ("There were errors. Press any key to continue testing. ");
	GetKeyPressed ();
	ClearInput ();
    }
    n = 0;	/* number of passes */
    rx = 0;
    errors = 0;
    do {
	if (ATXTestPortsOnce (fw)) {
		rx++;
	} else {
		errors++;
	}
	switch (WasKeyPressed ()) {
	    case -1: /* no input */
		break;
	    case ESC_CHAR:
		finished = 1;
	    default:
		ClearInput ();
		break;
	}
	n++;
	CLS ();
	PrintStatsATXHeader ();
	PrintATXStats (0, 0, n, rx, errors);
	printf ("\n\r\tPress <ESC> to stop the test.\r\n");
	fflush (stdout);
	sleep(1);
    } while (!finished);

	ATXResetPort (fw, status);
    printf ("\tPress any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}

int
IPMI_Test (void)
{
    int n, fw;
	int finished = 0, rx[2] = {0}, errors[2] = {0};

    CLS ();
    
    printf ("Detecting cabling on the IPMI port:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (IPMIOpenPort(&fw)) {
    n =  IPMITestPortsOnce(fw, 0x30);
    n += IPMITestPortsOnce(fw, 0x54);
    }
    printf ("%d port(s) selected for testing.\r\n", n);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (100000L);
    if (n == 0) {
	IPMIResetPort (fw);
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    if (errors[0]) {
	printf ("There were errors. Press any key to continue testing. ");
	GetKeyPressed ();
	ClearInput ();
    }
    n = 0;	/* number of passes */
    rx[0] = 0;
    rx[1] = 0;
    errors[0] = 0;
    errors[1] = 0;
    do {
	if (IPMITestPortsOnce (fw, 0x30)) {
		rx[0]++;
	} else {
		errors[0]++;
	}
	if (IPMITestPortsOnce (fw, 0x54)) {
		rx[1]++;
	} else {
		errors[1]++;
	}
	switch (WasKeyPressed ()) {
	    case -1: /* no input */
		break;
	    case ESC_CHAR:
		finished = 1;
	    default:
		ClearInput ();
		break;
	}
	n++;
	CLS ();
	PrintStatsIPMIHeader ();
	PrintIPMIStats (0, 2, n, rx, errors);
	printf ("\n\r\tPress <ESC> to stop the test.\n\r\t");
	fflush (stdout);
	sleep(1);
    } while (!finished);

	IPMIResetPort (fw);
    printf ("Press any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}

int
ALL_Test (void)
{
    int m, n, o, p, fa, fi;
    int finished = 0, rx_atx = 0, errors_atx = 0, rx_ipmi[2] = {0}, errors_ipmi[2] = {0};
    int were_errors = 0;
    asyncboard_t *currentBoard;
	unsigned char status;

    CLS ();
    
	/* serial port */
    printf ("Detecting cabling on the serial port:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    currentBoard = &(board.type.async);
    fflush (stdout);
    n = YDetectConnections (currentBoard);
    printf ("%d serial port(s) selected for testing.\r\n", n);

	/* atx port */
    printf ("Detecting cabling on the ATX port:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);
    o = ATXDetectConnections (&fa, &status);
    printf ("%d ATX port(s) selected for testing.\r\n", o);

	/* ipmi port */
    printf ("Detecting cabling on the IPMI port:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (IPMIOpenPort(&fi)) {
    p =  IPMITestPortsOnce(fi, 0x30);
    p += IPMITestPortsOnce(fi, 0x54);
    }
    printf ("%d IPMI port(s) selected for testing.\r\n", p);
	
    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (100000L);
    if (n == 0 && o == 0 && p == 0) {
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    if (were_errors) {
	printf ("There were errors. Press any key to continue testing. ");
	GetKeyPressed ();
	ClearInput ();
    }
    m = 0;
    currentBoard->porterrors = 0;
    rx_atx = 0;
    errors_atx = 0;
    rx_ipmi[0] = 0;
    rx_ipmi[1] = 0;
    errors_ipmi[0] = 0;
    errors_ipmi[1] = 0;
    do {
	if (n) { 
		/* serial port was detected, let's test it */
		YTestPortsOnce (currentBoard);
	}
	if (o) {
		/* atx port was detected, let's test it */
		if (ATXTestPortsOnce (fa)) {
			rx_atx++;
		} else {
			errors_atx++;
		}
	}
	if (p) {
		/* ipmi port was detected, let's test it */
		if (IPMITestPortsOnce (fi, 0x30)) {
			rx_ipmi[0]++;
		} else {
			errors_ipmi[0]++;
		}
		if (IPMITestPortsOnce (fi, 0x54)) {
			rx_ipmi[1]++;
		} else {
			errors_ipmi[1]++;
		}
	}
	switch (WasKeyPressed ()) {
	    case -1: /* no input */
		break;
	    case ESC_CHAR:
		finished = 1;
	    default:
		ClearInput ();
		break;
	}
	m++;
	CLS ();
	if (n) {
		PrintStatsHeader ();
		PrintBoardStats (currentBoard, 0, m);
		printf ("\n\r\n\r");
	}
	if (o) {
		PrintStatsATXHeader ();
		PrintATXStats (0, 0, m, rx_atx, errors_atx);
		printf ("\n\r\n\r");
	}
	if (p) {
		PrintStatsIPMIHeader ();
		PrintIPMIStats (0, 2, m, rx_ipmi, errors_ipmi);
		printf ("\n\r\tPress <ESC> to stop the test.\n\r\t");
	}
	else {
		printf ("\n\r\tPress <ESC> to stop the test.\n\r");
	}
	fflush (stdout);
	sleep(1);
    } while (!finished);

	/* close serial port */
    for (m = 0; n && (m < currentBoard->channels); m++) {
	ResetPort (currentBoard, m);
    }
    currentBoard->porterrors = 0;
	/* close atx */
	if (o)
	ATXResetPort (fa, status);
	/* close ipmi */
	if (p)
	IPMIResetPort (fi);

	if (!p) printf("\t"); 
    printf ("Press any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}
#endif

#ifdef CONFIG_TS1H
int ANALOG_Test (void)
{
    int n=0, fw[2], first_value[2], curr_value[2], max[2], min[2];
	int finished = 0, rx[2] = {0}, errors[2] = {0};

    CLS ();
    
    printf ("Detecting cabling on ANALOG ports:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (OpenAnalogPort(&fw)) {
    first_value[0] = TestAnalogPortsOnce(fw[0]);	// physical address ADC1
    first_value[1] = TestAnalogPortsOnce(fw[1]);	// physical address ADC2
    if (fw[0] >= 0) n += 1;
    if (fw[1] >= 0) n += 1;
    }
    printf ("%d port(s) selected for testing.\r\n", n);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (1000000L);
    if (n == 0) {
	ResetAnalogPort (&fw);
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    n = 0;	/* number of passes */
	max[0] = first_value[0] + 0x100;	// less than 1% error out of 0x8000 
	min[0] = first_value[0] - 0x100;
	max[1] = first_value[1] + 0x100;	// less than 1% error out of 0x8000 
	min[1] = first_value[1] - 0x100;
    do {
		curr_value[0] = TestAnalogPortsOnce (fw[0]);
		if ((curr_value[0] > max[0]) || (curr_value[0] < min[0])) {
			errors[0]++;
		} else {
			rx[0]++;
		}
		curr_value[1] = TestAnalogPortsOnce (fw[1]);
		if ((curr_value[1] > max[1]) || (curr_value[1] < min[1])) {
			errors[1]++;
		} else {
			rx[1]++;
		}
		switch (WasKeyPressed ()) {
		    case -1: /* no input */
			break;
		    case ESC_CHAR:
			finished = 1;
		    default:
			ClearInput ();
			break;
		}
		n++;
		CLS ();
		PrintStatsAnalogHeader ();
		PrintAnalogStats (0, 2, n, first_value, curr_value, rx, errors);
		printf ("\n\r\tPress <ESC> to stop the test.\n\r\t");
		fflush (stdout);
		sleep(1);
    } while (!finished);

	ResetAnalogPort (&fw);
    printf ("Press any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}

int DIG_Test (void)
{
    int n=0, fw, first_value[8], curr_value[8], aux, i;
	int finished = 0, rx[8] = {0}, errors[8] = {0};

    CLS ();
    
    printf ("Detecting cabling on DIGITAL ports:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (OpenDigitalPort(&fw)) {
    	aux = TestDigitalPortsOnce(fw);
		for (n=0; n<8; n++) {
    		first_value[n] = ((aux & (1 << n)) ? 1 : 0);
    	}
    	if (fw < 0) n = 0;
    }
    printf ("%d port(s) selected for testing.\r\n", n);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (1000000L);
    if (n == 0) {
	ResetDigitalPort (fw);
	printf ("No ports to test. Press any key to return to Main Menu. ");
	fflush (stdout);
	GetKeyPressed ();
	ClearInput ();
	return -1;
    }
    n = 0;	/* number of passes */
    do {
		aux = TestDigitalPortsOnce (fw);
		for (i=0; i<8; i++) {
			curr_value[i] = ((aux & (1 << i)) ? 1 : 0);
			if (curr_value[i] != first_value[i]) {
				errors[i]++;
			} else {
				rx[i]++;
			}
		}
		switch (WasKeyPressed ()) {
		    case -1: /* no input */
			break;
		    case ESC_CHAR:
			finished = 1;
		    default:
			ClearInput ();
			break;
		}
		n++;
		CLS ();
		PrintStatsDigitalHeader ();
		PrintDigitalStats (0, 8, n, first_value, curr_value, rx, errors);
		printf ("\n\r\tPress <ESC> to stop the test.\n\r\t");
		fflush (stdout);
		sleep(1);
    } while (!finished);

	ResetDigitalPort (fw);
    printf ("Press any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}

int ALL_Test (void)
{
    int n=0, fw_a[2], first_value_a[2], curr_value_a[2], max[2], min[2];
	int finished = 0, rx_a[2] = {0}, errors_a[2] = {0};

    int d=0, fw_d, first_value_d[8], curr_value_d[8], aux, i;
	int rx_d[8] = {0}, errors_d[8] = {0};

    CLS ();
    
	/* Analog Ports */
    printf ("Detecting cabling on ANALOG ports:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (OpenAnalogPort(&fw_a)) {
    first_value_a[0] = TestAnalogPortsOnce(fw_a[0]);	// physical address ADC1
    first_value_a[1] = TestAnalogPortsOnce(fw_a[1]);	// physical address ADC2
    if (fw_a[0] >= 0) n += 1;
    if (fw_a[1] >= 0) n += 1;
    }
    printf ("%d port(s) selected for testing.\r\n", n);
    fflush (stdout);
    if (n == 0) {
		ResetAnalogPort (&fw_a);
    }

	/* Digital Ports */
    printf ("Detecting cabling on DIGITAL ports:\r\n");
    fflush (stdout);
    switch (WasKeyPressed ()) {
	case -1: /* no input */
	    break;
	case ESC_CHAR:
	    return 0;
	default:
	    ClearInput ();
    }
    fflush (stdout);

    if (OpenDigitalPort(&fw_d)) {
    	aux = TestDigitalPortsOnce(fw_d);
		for (d=0; d<8; d++) {
    		first_value_d[d] = ((aux & (1 << d)) ? 1 : 0);
    	}
    	if (fw_d < 0) d = 0;
    }
    printf ("%d port(s) selected for testing.\r\n", d);

    printf ("\n\rDone.\n\r");
    fflush (stdout);
    usleep (1000000L);
    if (d == 0) {
		ResetDigitalPort (fw_d);
	}
	if ((n == 0) && (d == 0)) {
		printf ("No ports to test. Press any key to return to Main Menu. ");
		fflush (stdout);
		GetKeyPressed ();
		ClearInput ();
		return -1;
    }

    n = 0;	/* number of passes */
	max[0] = first_value_a[0] + 0x100;	// less than 1% error out of 0x8000 
	min[0] = first_value_a[0] - 0x100;
	max[1] = first_value_a[1] + 0x100;	// less than 1% error out of 0x8000 
	min[1] = first_value_a[1] - 0x100;
    do {
		curr_value_a[0] = TestAnalogPortsOnce (fw_a[0]);
		if ((curr_value_a[0] > max[0]) || (curr_value_a[0] < min[0])) {
			errors_a[0]++;
		} else {
			rx_a[0]++;
		}
		curr_value_a[1] = TestAnalogPortsOnce (fw_a[1]);
		if ((curr_value_a[1] > max[1]) || (curr_value_a[1] < min[1])) {
			errors_a[1]++;
		} else {
			rx_a[1]++;
		}

		aux = TestDigitalPortsOnce (fw_d);
		for (i=0; i<8; i++) {
			curr_value_d[i] = ((aux & (1 << i)) ? 1 : 0);
			if (curr_value_d[i] != first_value_d[i]) {
				errors_d[i]++;
			} else {
				rx_d[i]++;
			}
		}


		switch (WasKeyPressed ()) {
		    case -1: /* no input */
			break;
		    case ESC_CHAR:
			finished = 1;
		    default:
			ClearInput ();
			break;
		}
		n++;
		CLS ();
		PrintStatsAnalogHeader ();
		PrintAnalogStats (0, 2, n, first_value_a, curr_value_a, rx_a, errors_a);
		printf ("\n\r\n\r");
		PrintStatsDigitalHeader ();
		PrintDigitalStats (0, 8, n, first_value_d, curr_value_d, rx_d, errors_d);
		printf ("\n\r\tPress <ESC> to stop the test.\n\r\t");
		fflush (stdout);
		sleep(1);
    } while (!finished);

	ResetAnalogPort (&fw_a);
	ResetDigitalPort (fw_d);
    printf ("Press any key to return to the Main Menu. ");
    fflush (stdout);
    GetKeyPressed ();
    ClearInput ();
    return 0;
}
#endif

void
do_portConversation(asyncboard_t* chosenBoard, int portNumber)
{
    CLS ();
    PrintAsyncBoardInfo (chosenBoard);
	if (str_chat && *str_chat) {
    	printf ("Init Chat [%s]\r\n\n", str_chat);
	}
    printf ("Hit Ctrl-Q to exit the terminal emulation test.\r\n");
    fflush (NULL);
    YPortConversation (chosenBoard, portNumber, str_chat);
    baudRate = DEFAULTBAUDRATE;
}


int
PortConversation (asyncboard_t* chosenBoard) {
    int portNumber, portSpeed;

    CLS ();
    PrintAsyncBoardInfo (chosenBoard);

    while (1) {
	printf ("\tEnter port number : ");
	portNumber = GetInteger();
	if (portNumber == -1) return 0; /* ESC was pressed */
	if (IN_RANGE (portNumber, 1, chosenBoard->channels)) break;
	printf ("\tINVALID PORT NUMBER.\r\n");
    }
    portNumber --;

    while (1) {

	printf ("\tEnter baud rate (bps) : ");
	portSpeed = GetInteger ();
	if (portSpeed == -1) return 0; /* ESC was pressed */
	if (valid_speed(portSpeed))
		break;
	printf ("\tInvalid baud rate for the port.\r\n");
    }
	do_portConversation(chosenBoard, portNumber);
	return(0);
}

int TestSignalsManually (asyncboard_t* chosenBoard) {
    int portNumber;

    CLS ();
    PrintAsyncBoardInfo (chosenBoard);
    while (1) {
	printf ("\tEnter port number : ");
	portNumber = GetInteger();
	if (portNumber == -1) return 0; /* ESC was pressed */
	if (IN_RANGE (portNumber, 1, chosenBoard->channels)) break;
	printf ("\tINVALID PORT NUMBER.\r\n");
    }
    portNumber --;
    YTestSignalsManually (chosenBoard, portNumber);
    return 0;
}

int
TestSingleAsyncPort (asyncboard_t* chosenBoard, int menuChoice) {
    switch (menuChoice) {
    case MENU_CONVERSATION:
	PortConversation (chosenBoard);
	break;
    case MENU_SIGNALS:
	TestSignalsManually (chosenBoard);
	break;
    case 0: /* Escape is pressed */
	return 0;
    }
    return 0;
}

int
TestSinglePort (cyboard_t* board) {
    int menuChoice;

    do {
	int menuSize = MENU_SIGNALS;
	asyncboard_t* chosenBoard = &(board->type.async);

	menuChoice = GetMenuSelection ("", functionMenu, menuSize, 
							ALLOW_ESCAPE, board);
	if (!menuChoice) return -1; /* ESC pressed */
	TestSingleAsyncPort (chosenBoard, menuChoice);
    } while (1);
}

static void execute_cmds(void)
{
	asyncboard_t* chosenBoard = &(board.type.async);
    port_t *port;
	int fd;

	if (portNumber == -1) {
		printf("A port number between [1-%d] should be specified !!!\n", mt.nports);
		return;
	}

	if ((fd = serial_open(portNumber)) == -1) {
		printf("Error openning serial device /dev/ttyS%d !!!\n", portNumber);
		return;
	}

    port = &(chosenBoard->ports[--portNumber]);
    port->cyctfd = fd;

	if (commands & CMD_BREAK) {
		tcsendbreak(fd, bInterval * 2);
	}

	if (commands & CMD_DTR0) {
		YSetDTRLow (chosenBoard, portNumber);
	}

	if (commands & CMD_DTR) {
		if (YGetDTR (chosenBoard, portNumber)) {
		    YSetDTRLow (chosenBoard, portNumber);
		}
		else {
		    YSetDTRHigh (chosenBoard, portNumber);
		}
		if (tInterval) {
			sleep(tInterval);
			if (YGetDTR (chosenBoard, portNumber)) {
		    	YSetDTRLow (chosenBoard, portNumber);
			}
			else {
		    	YSetDTRHigh (chosenBoard, portNumber);
			}
		}
	}

	if (commands & CMD_RTS0) {
		YSetRTSLow (chosenBoard, portNumber);
	}

	if (commands & CMD_RTS) {
		if (YGetRTS (chosenBoard, portNumber)) {
		    YSetRTSLow (chosenBoard, portNumber);
		}
		else {
		    YSetRTSHigh (chosenBoard, portNumber);
		}
		if (rInterval) {
			sleep(rInterval);
			if (YGetRTS (chosenBoard, portNumber)) {
		    	YSetRTSLow (chosenBoard, portNumber);
			}
			else {
		    	YSetRTSHigh (chosenBoard, portNumber);
			}
		}
	}

	if (commands & CMD_PCONV) {
		do_portConversation(chosenBoard, portNumber);
	}

	serial_close(fd);
	++portNumber;
}

void exit_prog(void)
{
    printf ("\n\r");
    SET_STD_MODE_INPUT ();
	serial_close(last_fd);
}

static struct sigaction sa = {0};

static void got_sig(int sig)
{
	exit_prog();
	
	sa.sa_handler = SIG_DFL;
	sigaction(sig,  &sa, NULL);
	kill(getpid(), sig);
}

/**
 * ===============================================================
 * Main function of the program. It provides the template for and
 * overall design for the Cyctest/Linux program.
 * ===============================================================
 */
int
main (int argc, char **argv)
{
    unsigned int menuChoice;
    int menuSize = MENU_QUIT;
	extern char *optarg;
	extern int optind;
	int opt, val;
	int i;

    numberOfBoards = 0;

	get_machine_type(&mt);

#ifdef CONFIG_TS1H
	/* adjust menu options for TS100 family */
	if (mt.board_type != 18) {	// BOARD_TS110
		menuSize -= 3;
		mainMenu[2] = "\t3. Quit\r\n";		
	}
#endif

    /* Parse command-line arguments. */
    while ((opt = getopt(argc, argv, "l:s:p:f:d:B:T:R:I:CcIibtrh")) != EOF) {
        switch (opt) {
		case 'c':
			interactive = 0;
			flag_open_close |= 1;
			break;
		case 'C':
			interactive = 0;
			flag_open_close |= 2;
			break;
		case 'T':
			interactive = 0;
			commands |= CMD_DTR;
			val = atoi(optarg);
			if (val >= 1 && val <= 5) {
				tInterval = val;
				break;
			}
			goto xdefault;

		case 't':
			interactive = 0;
			tInterval = 0;
			commands |= CMD_DTR0;
			break;
		case 'R':
			interactive = 0;
			commands |= CMD_RTS;
			val = atoi(optarg);
			if (val >= 1 && val <= 5) {
				rInterval = val;
				break;
			}
			goto xdefault;

		case 'r':
			interactive = 0;
			rInterval = 0;
			commands |= CMD_RTS0;
			break;
		case 'I':
			interactive = 0;
			commands |= CMD_PCONV;
			str_chat = optarg;
			break;
		case 'i':
			interactive = 0;
			commands |= CMD_PCONV;
			break;
		case 'B':
			interactive = 0;
			commands |= CMD_BREAK;
			val = atoi(optarg);
			if (val >= 1 && val <= 5) {
				bInterval = val;
				break;
			}
			goto xdefault;

		case 'b':
			interactive = 0;
			bInterval = 0;
			commands |= CMD_BREAK;
            break;
		case 'd':
			val = atoi(optarg);
			switch (val) {
			case 5:
				DataSize = CS5;
				break;
			case 6:
				DataSize = CS6;
				break;
			case 7:
				DataSize = CS7;
				break;
			default:
				DataSize = CS8;
				break;
			}
            break;
		case 'f':
			if (strcmp(optarg, "none")) {
				Parity = FLOW_NONE;
            	break;
			}
			if (strcmp(optarg, "hard")) {
				Parity = FLOW_HARD;
            	break;
			}
			if (strcmp(optarg, "soft")) {
				Parity = FLOW_SOFT;
            	break;
			}
			goto xdefault;

		case 'p':
			if (strcmp(optarg, "odd")) {
				Parity = PARITY_ODD;
            	break;
			}
			if (strcmp(optarg, "even")) {
				Parity = PARITY_EVEN;
            	break;
			}
			if (strcmp(optarg, "none")) {
				Parity = PARITY_NONE;
            	break;
			}
			goto xdefault;

		case 'l':
			val = atoi(optarg);
			if (IN_RANGE (val, 1, mt.nports)) {
				portNumber = val;
				break;
			}
			goto xdefault;

		case 's':
			if ((val = atoi(optarg)) && valid_speed(val)) {
            	break;
			}
			goto xdefault;

        case 'h':
		default:
xdefault:
			printf("\nUSAGE: %s [ options ]\n\nWhere options could be:\n\n"
"-l #port      - Serial port number [1 to %d]\n"
"-s speed      - Baud rate\n"
"-p parity     - Parity even, odd, none\n"
"-f flow       - Flow control hard, soft, none\n"
"-d DataLength - Number of bits from 5 to 8\n"
"-b            - Send break 0.25 to 0.5 seconds long\n"
"-B interval   - Send break [1-5] seconds long\n"
"-T interval   - Toggle DTR [1-5] seconds long\n"
"-t            - Toggle DTR forever\n"
"-R interval   - Toggle RTS [1-5] seconds long\n"
"-r            - Toggle RTS forever\n"
"-i            - Port conversation\n"
"-I <initchat> - Port conversation\n"
"-c            - doesn't change tty configuration/signals on open\n"
"-C            - doesn't restore tty configuration/signals on close\n"
				"\n", argv[0], mt.nports);
			return(0);
		}
	}

    Initialize ();

	sa.sa_handler = got_sig;

	for (i=1; i < 64; i++) {
		if (i == SIGPIPE) continue;
		if (i == SIGWINCH) continue;
		if (i == SIGSEGV) continue;
		if (i == SIGFPE) continue;
		if (i == SIGILL) continue;
		if (i == SIGKILL) continue;
		if (i == SIGCHLD) continue;
		sigaction(i,  &sa, NULL);
	}

	atexit(exit_prog);

    if (DetectBoards ()) {
		if (interactive) {
			printf("Detected %s board %d ports in the system.\n\r",
				mt.name, mt.nports); 
			fflush(stdout);
		}
    }

	if (!interactive) {
		execute_cmds();
		return(0);
	}

	if (!GetMenuSelection (WarnTitle, NULL, 0, ALLOW_ESCAPE, NULL)) {
		CLS ();
		return(0);
	}

    do {
	menuChoice = GetMenuSelection (VERSION, mainMenu, menuSize, 
				       NO_ESCAPE, NULL);
	switch (menuChoice) {
	    case MENU_SELECT_BOARD:
                TestSinglePort (&board);
		break;

	    case MENU_ASYNC_PORT_TEST:
		PortTest ();
		break;

#ifdef CONFIG_SM100
	    case MENU_ATX_PORT_TEST:
		ATX_Test ();
		break;

	    case MENU_IPMI_PORT_TEST:
		IPMI_Test ();
		break;

	    case MENU_ALL_PORT_TEST:
		ALL_Test ();
		break;
#endif
#ifdef CONFIG_TS1H
	    case MENU_ANALOG_PORT_TEST:
		if (mt.board_type == 18) ANALOG_Test (); //BOARD_TS110
		CLS ();
		break;

	    case MENU_DIG_PORT_TEST:
		if (mt.board_type == 18) DIG_Test (); //BOARD_TS110
		CLS ();
		break;

	    case MENU_ALL_PORT_TEST:
		if (mt.board_type == 18) ALL_Test (); //BOARD_TS110
		CLS ();
		break;
#endif
	    case MENU_QUIT:
	    default:
		CLS ();
		break;
	}
    }
    while (menuChoice != menuSize);	// check for MENU_QUIT

    return 0;
}

