/*
 * Various utility functions that proved to be useful ;)
 */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "cyconst.h"
#include "macro.h"
#include "cystruct.h"
#include "cyfunc.h"

#define CLRBUF_SIZE	1024

static int linesPrinted = 0;

void 
ClearInput (void)
{
    char *buf[CLRBUF_SIZE];

    while (fread ((void *) buf, 1, CLRBUF_SIZE, stdin) != 0);
}

int 
GetKeyPressed (void)
{
    fd_set rfds;

    FD_ZERO (&rfds);
    FD_SET (0, &rfds);
    select (1, &rfds, NULL, NULL, NULL);
    return getchar ();
}


int 
WasKeyPressed (void)
{
    fd_set rfds;
    struct timeval tv;
    int c;

    FD_ZERO (&rfds);
    FD_SET (0, &rfds);
    tv.tv_sec = 0;
    tv.tv_usec = 0L;	/*
				 * Wait for input for 10ms 
				 */
    c = select (1, &rfds, NULL, NULL, &tv);
    if (c)
	return getchar ();
    else
	return -1;
}

/**
 * Note: the following function does NOT understand negative numbers
 * It will return -1 if the user pressed ESC
 */

int 
GetInteger (void) 
{
    int n = 0;
    int c;
    int counter = 0;

    fflush (NULL);
    do {
	c = GetKeyPressed (); 
	if (isdigit (c)) {
	    printf ("%c", c);
	    c = c - '0'; /* convert from ASCII code to digit */
	    n = n*10 + c;
	    counter ++;
	} else if ( (c == BS_CHAR) && (counter > 0)) {
	    n = (n - (n % 10)) / 10;
	    printf ("\b \b");
	    counter --;
	} else if (c == ESC_CHAR) {
	    return -1;
	}
	fflush (NULL);
    } while (c != ENTER_CHAR);
    printf ("\r\n");

    return n;
}

void 
PrintStatsHeader (void)
{
    printf ("\t                <---- PACKETS --->          <---- ERRORS ---->\r\n");
    printf ("\tBd. From    To  Sent      Received  Passes  Data CTS  DCD  DSR\n\r");
    printf ("\t--------------------------------------------------------------\n\r");
    linesPrinted = 3; /* this assumes that the CLS() was called before */
    return;
}

void 
PrintStatsATXHeader (void)
{
    printf ("\t                <---- PACKETS --->          <---- ERRORS ---->\r\n");
    printf ("\tATX From    To  Sent      Received  Passes         Data       \n\r");
    printf ("\t--------------------------------------------------------------\n\r");
    linesPrinted = 3; /* this assumes that the CLS() was called before */
    return;
}

PrintStatsIPMIHeader (void)
{
    printf ("\t                <---- PACKETS --->          <---- ERRORS ---->\r\n");
    printf ("\tIPMI From   To  Sent      Received  Passes         Data       \n\r");
    printf ("\t--------------------------------------------------------------\n\r");
    linesPrinted = 3; /* this assumes that the CLS() was called before */
    return;
}

PrintStatsAnalogHeader (void)
{
    printf ("\t        <--------- VALUE --------->         <---- ERRORS ---->\r\n");
    printf ("\tANALOG  Initial   Current   Correct  Passes        Data       \n\r");
    printf ("\t--------------------------------------------------------------\n\r");
    linesPrinted = 3; /* this assumes that the CLS() was called before */
    return;
}

PrintStatsDigitalHeader (void)
{
    printf ("\t        <--------- VALUE --------->         <---- ERRORS ---->\r\n");
    printf ("\tDIGITAL Initial   Current   Correct  Passes        Data       \n\r");
    printf ("\t--------------------------------------------------------------\n\r");
    linesPrinted = 3; /* this assumes that the CLS() was called before */
    return;
}

void 
PrintBoardStats (asyncboard_t * board, int n, int npass)
{
    int i;
    port_t *port;

    for (i = 0; i < board->channels; i++) {
	port = &(board->ports[i]);
	if (port->selected) {
	    /* we need 2 lines to print [\r\nPress <ESC> to stop the test.] */
	    //if (linesPrinted++ > (SCREEN_HEIGHT - 4)) break;
	    printf ("\t%-3d  %-2d <-> %-2d  %-8d  %-8d  %-6d  %-4d %-4d %-4d %-4d\n\r",
		    n+1, i+1, port->connectedToNumber+1,
		    port->txtotalpackets,
		    port->rxtotalpackets,
		    npass,
		    port->errors, 
		    port->ctserrors, 
		    port->dcderrors,
		    port->dsrerrors);
	}
    }
    if (board->porterrors) {
	printf("\tDetected errors on ports :");
	for (i = 0; i < board->channels; i++) {
	    if (board->porterrors & (1 << i)) {
		printf(" %d", i + 1);
	    }
	}
    }
    fflush (NULL);
    return;
}

void 
PrintATXStats (int port, int n, int npass, int rx, int errors)
{
    printf ("\t     %-2d <-> %-2d  %-8d  %-8d  %-6d         %-4d\n\r",
		    port+1, port+1,
		    npass,
		    rx,
		    npass,
		    errors);
    if (errors) {
	printf("\tDetected errors on ports %d", port+1);
    }
    fflush (NULL);
    return;
}

void 
PrintIPMIStats (int port, int n, int npass, int *rx, int *errors)
{
	int i, flag = 1;

    for (i = port; i < n; i++) {
	    printf ("\t     %-2d <-> %-2d  %-8d  %-8d  %-6d         %-4d\n\r",
			    i+1, i+1,
			    npass,
			    rx[i],
			    npass,
			    errors[i]);
	}
    for (i = 0; i < n; i++) {
	    if (errors[i]) {
	    	if (flag) {
		    flag = 0;
				printf("\tDetected errors on ports");
	    	}
			printf(" %d", i+1);
		}
	}

    fflush (NULL);
    return;
}

void
PrintAnalogStats (int port, int n, int npass, int *first, int *curr, int *rx, int *errors)
{
	int i, flag = 1;

    for (i = port; i < n; i++) {
	    printf ("\t  %-2d      %04x      %04x       %-8d %-7d      %-4d\n\r",
			    i+1, 
			    first[i],
			    curr[i],
			    rx[i],
			    npass,
			    errors[i]);
	}
    for (i = 0; i < n; i++) {
	    if (errors[i]) {
	    	if (flag) {
		    flag = 0;
				printf("\tDetected errors on ports");
	    	}
			printf(" %d", i+1);
		}
	}

    fflush (NULL);
    return;
}

void
PrintDigitalStats (int port, int n, int npass, int *first, int *curr, int *rx, int *errors)
{
	int i, flag = 1;

    for (i = port; i < n; i++) {
	    printf ("\t  %-2d        %-4d      %-4d     %-8d %-6d       %-4d\n\r",
			    i+1, 
			    first[i],
			    curr[i],
			    rx[i],
			    npass,
			    errors[i]);
	}
    for (i = 0; i < n; i++) {
	    if (errors[i]) {
	    	if (flag) {
		    flag = 0;
				printf("\tDetected errors on ports");
	    	}
			printf(" %d", i+1);
		}
	}

    fflush (NULL);
    return;
}

void
PrintAsyncBoardInfo (asyncboard_t* board) {
    printf ("\tTYPE        PORTS\n\r");
    printf ("\t-----------------------------------------------\n\r");
    printf ("\t%-10s  %-5d\n\n\r",
	    board->name,
	    board->channels);
    fflush (NULL);
    return;
}


