/*
 * ====================================================================
 * Y-board specific functions.
 * ====================================================================
 */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>

#include "cyconst.h"
#include "cystruct.h"
#include "cyfunc.h"
#include "macro.h"
#include <rot_nports.h>

extern int baudRate;

extern struct termios old_tio;

void 
YTestPortsOnce (asyncboard_t * board)
{
    int i, packet_nok;
    ucchar *data = (ucchar*) calloc (1024, sizeof (char));
    port_t* port;
    int bytesReceived; /* this variable is for debugging purposes */
	int nports;

    for (nports=0, i=0; i < board->channels; i++) {
	port = &(board->ports[i]);
	if (!port->selected) continue;
	if (port->cyctfd == -1) continue;
	write(port->cyctfd, DATA, strlen (DATA));
	port->txtotalpackets ++;
	nports++;
    }
	nports < 4 ? sleep(1) : usleep(100000);
    for (i = 0; i < board->channels; i++) {
	port = &(board->ports[i]);
	if (!port->selected) continue;
	if (port->cyctfd == -1) continue;
	memset (data, 0, 1024);
	packet_nok = 0;
	bytesReceived = read(port->cyctfd, data, 1024);
	if ( (!*data) ||
	     (memcmp (data, DATA, strlen (DATA)) != 0) ||
	     (bytesReceived != (int)strlen (DATA)) ) {
		packet_nok = 1;
	}
	if (packet_nok) {
	    port->errors ++;
	    board->porterrors |= (1 << i);
	}
	else {
	    port->rxtotalpackets ++;
	}
    }
    for (i = 0; i < board->channels; i ++) {
	port = &(board->ports[i]);
	if (!port->selected) continue;
	if (port->cyctfd == -1) continue;
	YTestSignals (board,  i);
    }

    free (data);
    return;
}

int YSetRTS (asyncboard_t* board, int portNumber, int level) {
    int signals;
    port_t *port;
	struct MachineType mt;
    port = &(board->ports[portNumber]);

    ioctl(port->cyctfd, TIOCMGET, &signals);

    if (level)  signals |= TIOCM_RTS;
    else 	signals &= ~TIOCM_RTS;

    ioctl(port->cyctfd, TIOCMSET, &signals);
 
    /*
     * wait until the changes will take effect 
     */
	get_machine_type(&mt);
    if ((mt.board_type == 12) || (mt.board_type == 16)) {	/* TS100 or SM100 */
        usleep (50000);
	}
	else {
        usleep (500);
    }
    return 0;
}

int YSetDTR (asyncboard_t* board, int portNumber, int level) {
    int signals;
    port_t *port;
	struct MachineType mt;
    port = &(board->ports[portNumber]);

    ioctl(port->cyctfd, TIOCMGET, &signals);
    if (level)  signals |= TIOCM_DTR;
    else 	signals &= ~TIOCM_DTR;
    ioctl(port->cyctfd, TIOCMSET, &signals);
 
    /*
     * wait until the changes will take effect 
     */
	get_machine_type(&mt);
    if ((mt.board_type == 12) || (mt.board_type == 16)) {	/* TS100 or SM100 */
        usleep (50000);
	}
	else {
        usleep (500);
    }

    return 0;
}

int YSetRTSHigh (asyncboard_t* board, int portNumber) {
    return YSetRTS (board, portNumber, 1);
}

int YSetRTSLow (asyncboard_t* board, int portNumber) {
    return YSetRTS (board, portNumber, 0);
}

int YSetDTRHigh (asyncboard_t* board, int portNumber) {
    return YSetDTR (board, portNumber, 1);
}

int YSetDTRLow (asyncboard_t* board, int portNumber) {
    return YSetDTR (board, portNumber, 0);
}

int YGetInputSignal (asyncboard_t* board, int portNumber, int signal) {
    int signals, level = 0;
    port_t *port;
    port = &(board->ports[portNumber]);

    ioctl(port->cyctfd, TIOCMGET, &signals);
    switch (signal) {
	case CyDCD:
	    level = (signals & TIOCM_CD) ? 1 : 0;
	    break;
	case CyDSR:
	    level = (signals & TIOCM_DSR) ? 1 : 0;
	    break;
	case CyCTS:
	    level = (signals & TIOCM_CTS) ? 1 : 0;
	    break;
	case CyDTR:
	    level = (signals & TIOCM_DTR) ? 1 : 0;
	    break;
	case CyRTS:
	    level = (signals & TIOCM_RTS) ? 1 : 0;
	    break;
	default:
	    level = 0;
	    break;
    }
    return level;
}

int YGetDCD (asyncboard_t* board, int portNumber) {
    return YGetInputSignal (board, portNumber, CyDCD);
}

int YGetDSR (asyncboard_t* board, int portNumber) {
    return YGetInputSignal (board, portNumber, CyDSR);
}

int YGetCTS (asyncboard_t* board, int portNumber) {
    return YGetInputSignal (board, portNumber, CyCTS);
}

int YGetDTR (asyncboard_t* board, int portNumber) {
    return YGetInputSignal (board, portNumber, CyDTR);
}

int YGetRTS (asyncboard_t* board, int portNumber) {
    return YGetInputSignal (board, portNumber, CyRTS);
}

int 
YDetectConnections (asyncboard_t * board)
{
    int loopbacks = 0;
    port_t *port;
    ucchar* data = (ucchar*) calloc (1024, sizeof (char));
    int channel, channel_ok;
    int datalength;
    int portNumber, bytesReceived;

    /* Open all the ports */
    for (channel = 0; channel < board->channels; channel ++) {
	port = &(board->ports[channel]);
	if ((port->cyctfd = serial_open(channel + board->first_port)) == -1) continue; 
	usleep(1000);
    }
    sleep(1);
    /* Set DTR and RTS low */
    for (channel = 0; channel < board->channels; channel ++) {
	port = &(board->ports[channel]);
	if (port->cyctfd == -1) continue;
	YSetDTRLow (board, channel);
	YSetRTSLow (board, channel);
    }
    sleep(1);
    /* Initialize and write to all the ports */
    for (channel = 0; channel < board->channels; channel ++) {
	port = &(board->ports[channel]);
	if (port->cyctfd == -1) continue;
	datalength = sprintf (data, "%d", channel);
	write(port->cyctfd, data, datalength);
	port->selected = 1;
	port->connectedTo = port;
	port->connectedToNumber = channel;
    }
    sleep(1);

    for (channel = 0; channel < board->channels; channel ++) {
	port = &(board->ports[channel]);
	memset (data, 0, 1024);
	channel_ok = 0;
	if (port->cyctfd == -1) continue;
	bytesReceived = read(port->cyctfd, data, 1024);
	if (*data) { /* not empty */
	    portNumber = atoi (data);
	    port->connectedTo = &(board->ports[portNumber]);
	    port->connectedToNumber = portNumber;
	    port->txtotalchars = port->rxtotalchars = 0;
	    port->txchars = port->rxchars = 0;
	    port->selected = 1;
	    loopbacks ++;
	    channel_ok = 1;
	}
	if (!channel_ok) {
	    port->connectedTo = NULL;
	    port->connectedToNumber = -1;
	    port->selected = 0;
	    tcsetattr(port->cyctfd, TCSANOW, &old_tio); /* restore termios */
	    close (port->cyctfd);
	    port->cyctfd = -1;
	}
    }
    free (data);
    return loopbacks;
}

int YTestSignals (asyncboard_t* board, int portNumber) {
    port_t* port = &(board->ports[portNumber]);
    int flag;

    flag = 0;
    YSetDTRHigh (board, portNumber);
    if (YGetDSR (board, port->connectedToNumber) == 0) flag = 1;
    YSetDTRLow (board, portNumber);
    if (YGetDSR (board, port->connectedToNumber) != 0) flag = 1;
    port->dsrerrors += flag;
    board->porterrors |= (flag << portNumber);

    flag = 0;
    YSetDTRHigh (board, portNumber);
    if (YGetDCD (board, port->connectedToNumber) == 0) flag = 1;
    YSetDTRLow (board, portNumber);
    if (YGetDCD (board, port->connectedToNumber) != 0) flag = 1;
    port->dcderrors += flag;
    board->porterrors |= (flag << portNumber);

    flag = 0;
    YSetRTSHigh (board, portNumber);
    if (YGetCTS (board, port->connectedToNumber) == 0) flag = 1;
    YSetRTSLow (board, portNumber);
    if (YGetCTS (board, port->connectedToNumber) != 0) flag = 1;
    port->ctserrors += flag;
    board->porterrors |= (flag << portNumber);

    return 0;
}


int YTestSignalsManually (asyncboard_t* board, int portNumber) {
    int c = 0;
    int rts, dtr, dcd, dsr, cts;
    port_t *port;

    port = &(board->ports[portNumber]);
    if ((port->cyctfd = serial_open(portNumber + board->first_port)) == -1) return(0); 
    usleep(1000);

    YSetRTSLow (board, portNumber);
    YSetDTRLow (board, portNumber);
    do {
    usleep(50000);
	dtr = YGetDTR (board, portNumber);
	dcd = YGetDCD (board, portNumber);
	dsr = YGetDSR (board, portNumber);
	rts = YGetRTS (board, portNumber);
	cts = YGetCTS (board, portNumber);
	if (c > -1) {
	    CLS ();
	    PrintAsyncBoardInfo (board);
	    // printf ("SERIAL PORT SIGNAL TESTING\r\n");
	    printf ("\n\tCTRL-Q - Return to main menu\r\n");
	    printf ("\tCTRL-D - Toggle DTR\r\n\tCTRL-R - Toggle RTS\r\n\n");
	    printf ("\t\t\tPORT %d\r\n\n", portNumber+1);
	    printf ("\tSTATE | DTR   DCD   DSR   RTS   CTS\r\n");
	    printf ("\t------+----------------------------\r\n");
	    printf ("\tON    |  %c     %c     %c     %c     %c\r\n",
		    dtr ? 'X' : ' ',
		    dcd ? 'X' : ' ',
		    dsr ? 'X' : ' ',
		    rts ? 'X' : ' ',
		    cts ? 'X' : ' ');
	    printf ("\tOFF   |  %c     %c     %c     %c     %c\r\n",
		    dtr ? ' ' : 'X',
		    dcd ? ' ' : 'X',
		    dsr ? ' ' : 'X',
		    rts ? ' ' : 'X',
		    cts ? ' ' : 'X');
	    fflush (stdout);
	}
	c = WasKeyPressed ();
	switch (c) {
	    case CTRLD_CHAR:
		if (YGetDTR (board, portNumber)) {
		    YSetDTRLow (board, portNumber);
		}
		else {
		    YSetDTRHigh (board, portNumber);
		}
		break;

	    case CTRLR_CHAR:
		if (YGetRTS (board, portNumber)) {
		    YSetRTSLow (board, portNumber);
		}
		else {
		    YSetRTSHigh (board, portNumber);
		}
		break;

	    case CTRLQ_CHAR:
	    case CTRLE_CHAR:
    	c = CTRLQ_CHAR;
		break;
	    default:
		break;
	}
    } while (c != CTRLQ_CHAR);
    serial_close (port->cyctfd);
    port->cyctfd = -1;
    return 0;
}

#define	Y_TX_BUFSIZE	 0x0400

void YPortConversation (asyncboard_t* board, int n, char *str_chat) {
    int c;
    char* data = (char*) calloc (1024, sizeof (char));
    int rx_bytes, tx_bytes, i;
    ucchar txbuf[Y_TX_BUFSIZE];
    port_t *port;

    port = &(board->ports[n]);
    if ((port->cyctfd = serial_open(n + board->first_port)) == -1) return; 

    /* Setting a signal LOW will ASSERT that pin */
    YSetDTRLow (board, n);
    YSetRTSLow (board, n);
    YSetDTRHigh (board, n);
    YSetRTSHigh (board, n);

	if (str_chat && *str_chat) {
		char *s;
		tstr(0, str_chat); 
		while (s = strchr(str_chat, '\r')) {
			write(port->cyctfd, str_chat, ++s - str_chat);
			str_chat = s;
			memset (data, 0, 1024);
			sleep(1);
			rx_bytes = read(port->cyctfd, data, 1024);
			printf ("%s", data);
		}
		if (*str_chat) {
			write(port->cyctfd, str_chat, strlen(str_chat));
			write(port->cyctfd, "\r", 1);
		}
	}

    do {
	memset (data, 0, 1024);
	rx_bytes = read(port->cyctfd, data, 1024);
	printf ("%s", data);
	fflush (stdout);
	if ( (c = WasKeyPressed ()) != -1) {
	    if (c != CTRLQ_CHAR) {
		for (txbuf[0] = c, tx_bytes = 1 ;
			(i = fread((void *)&txbuf[tx_bytes], 1,
			 (Y_TX_BUFSIZE - tx_bytes), stdin)) != 0 ;
			 tx_bytes += i);
		write(port->cyctfd, txbuf, tx_bytes);
	    }
	}
    } while (c != CTRLQ_CHAR);

    serial_close (port->cyctfd);
    port->cyctfd = -1;
    free (data);
}


