#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else				/* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>
#include <ucd-snmp/ucd-snmp-agent-includes.h>
#endif				/* !IN_UCD_SNMP_SOURCE */

#include "cyACSConf.h"
#include "cycommon.h"

// ptr. aux buffer
#define CY_BUFAUX_SIZE	255
static unsigned char bufaux[CY_BUFAUX_SIZE + 1];
static char *paux;

/* 
 * cyACSConf_variables_oid:
 *   this is the top level oid that we want to register under.  This
 *   is essentially a prefix, with the suffix appearing in the
 *   variable below.
 */

oid cyACSConf_variables_oid[] = { 1, 3, 6, 1, 4, 1, 2925, 4, 2 };

/* 
 * variable7 cyACSConf_variables:
 *   this variable defines function callbacks and type return information 
 *   for the cyACSConf mib section 
 */

struct variable4 cyACSConf_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define   CYHOSTNAME            1
	{CYHOSTNAME, ASN_OCTET_STR, RWRITE, var_cyACSConf, 1, {1}},
#define   CYCONSOLEBANNER       2
	{CYCONSOLEBANNER, ASN_OCTET_STR, RWRITE, var_cyACSConf, 1, {2}},
#define   CYMOTD                3
	{CYMOTD, ASN_OCTET_STR, RWRITE, var_cyACSConf, 1, {3}},
#define   CYETHDHCPC            4
	{CYETHDHCPC, ASN_INTEGER, RWRITE, var_cyACSConf, 2, {4, 1}},
#define   CYETHIPADDR           5
	{CYETHIPADDR, ASN_IPADDRESS, RWRITE, var_cyACSConf, 2, {4, 2}},
#define   CYETHIPMASK           6
	{CYETHIPMASK, ASN_IPADDRESS, RWRITE, var_cyACSConf, 2, {4, 3}},
#define   CYETHMTU              7
	{CYETHMTU, ASN_INTEGER, RWRITE, var_cyACSConf, 2, {4, 4}},
#define   CYETHIPADDR2          8
	{CYETHIPADDR2, ASN_IPADDRESS, RWRITE, var_cyACSConf, 2, {4, 5}},
#define   CYETHIPMASK2          9
	{CYETHIPMASK2, ASN_IPADDRESS, RWRITE, var_cyACSConf, 2, {4, 6}},
#define   CYRESOLVERORDER       10
	{CYRESOLVERORDER, ASN_OCTET_STR, RWRITE, var_cyACSConf, 2, {5, 1}},
#define   CYMULTIPLEIP          11
	{CYMULTIPLEIP, ASN_OCTET_STR, RWRITE, var_cyACSConf, 2, {5, 2}},
#define   CYDNSPRISERV          12
	{CYDNSPRISERV, ASN_IPADDRESS, RWRITE, var_cyACSConf, 3, {5, 3, 1}},
#define   CYDNSSECSERV          13
	{CYDNSSECSERV, ASN_IPADDRESS, RWRITE, var_cyACSConf, 3, {5, 3, 2}},
#define   CYDNSDOMAIN           14
	{CYDNSDOMAIN, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {5, 3, 3}},
#define   CYSERIALINCLUDE       15
	{CYSERIALINCLUDE, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 1}},
#define   CYSERIALNFS           16
	{CYSERIALNFS, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 2}},
#define   CYSERIALLOCKDIR       17
	{CYSERIALLOCKDIR, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 3}},
#define   CYSERIALRLOGIN        18
	{CYSERIALRLOGIN, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 4}},
#define   CYSERIALPPPD          19
	{CYSERIALPPPD, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 5}},
#define   CYSERIALTELNET        20
	{CYSERIALTELNET, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 6}},
#define   CYSERIALSSH           21
	{CYSERIALSSH, ASN_OCTET_STR, RWRITE, var_cyACSConf, 3, {6, 1, 7}},
#define   CYSERIALLOCALLOGINS   22
	{CYSERIALLOCALLOGINS, ASN_INTEGER, RWRITE, var_cyACSConf, 3, {6, 1, 8}},
#define   CYSERIALFACILITY      23
	{CYSERIALFACILITY, ASN_INTEGER, RWRITE, var_cyACSConf, 3, {6, 1, 9}},
#define   CYSERIALDBFACILITY    24
	{CYSERIALDBFACILITY, ASN_INTEGER, RWRITE, var_cyACSConf, 3, {6, 1, 10}},

};

/*
 * init_cyACSConf():
 *   Initialization routine.  This is called when the agent starts up.
 *   At a minimum, registration of your variables should take place here.
 */
void init_cyACSConf(void)
{
	/* register ourselves with the agent to handle our mib tree */
	REGISTER_MIB("cyACSConf", cyACSConf_variables, variable4,
		     cyACSConf_variables_oid);

	memset(bufaux, 0, CY_BUFAUX_SIZE + 1);
}

/*
 * var_cyACSConf():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *var_cyACSConf(struct variable *vp,
			     oid * name,
			     size_t * length,
			     int exact,
			     size_t * var_len, WriteMethod ** write_method)
{
	/* variables we may use later */
	static long long_ret;
	u_long size;
	struct in_addr aux;

	if (header_generic(vp, name, length, exact, var_len, write_method)
	    == MATCH_FAILED)
		return NULL;

	*var_len = sizeof(long);
	long_ret = 0;

	switch (vp->magic) {

	case CYHOSTNAME:
		*write_method = write_cyHostName;
		*var_len = cy_rw_parse("/etc/hostname", NULL, bufaux,
				       CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYCONSOLEBANNER:
		*write_method = write_cyConsoleBanner;
		*var_len = cy_rw_parse("/etc/issue", NULL, bufaux,
				       CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYMOTD:
		*write_method = write_cyMotd;
		*var_len =
		    cy_rw_parse("/etc/motd", NULL, bufaux, CY_BUFAUX_SIZE,
				0);
		return (unsigned char *) bufaux;

	case CYETHDHCPC:
		*write_method = write_cyEthDhcpc;
		size = cy_rwsp_conf("conf", "dhcp_client", bufaux,
				 CY_BUFAUX_SIZE, 0);
		if (size) long_ret = atoi(bufaux);
		return (unsigned char *) &long_ret;

	case CYETHIPADDR:
		*write_method = write_cyEthIPaddr;
		size = cy_rwsp_conf("conf", "eth_ip", bufaux, CY_BUFAUX_SIZE,
				 0);
		inet_aton(bufaux, &aux);
		long_ret = aux.s_addr;
		return (unsigned char *) &long_ret;

	case CYETHIPMASK:
		*write_method = write_cyEthIPmask;
		size =
		    cy_rwsp_conf("conf", "eth_mask", bufaux,
				 CY_BUFAUX_SIZE, 0);
		inet_aton(bufaux, &aux);
		long_ret = aux.s_addr;
		return (unsigned char *) &long_ret;

	case CYETHMTU:
		*write_method = write_cyEthMTU;
		long_ret = cy_rwsp_conf("conf", "eth_mtu", bufaux, 
					CY_BUFAUX_SIZE, 0);
		if (long_ret) long_ret = atoi(bufaux);
		return (unsigned char *) &long_ret;

	case CYETHIPADDR2:
		*write_method = write_cyEthIPaddr2;
		size = cy_rwsp_conf("conf", "eth_ip_alias", bufaux,
				 CY_BUFAUX_SIZE, 0);
		if (size) {
			inet_aton(bufaux, &aux);
			long_ret = aux.s_addr;
		} else {
			long_ret = 0;
		}
		return (unsigned char *) &long_ret;

	case CYETHIPMASK2:
		*write_method = write_cyEthIPmask2;
		size = cy_rwsp_conf("conf", "eth_mask_alias", bufaux,
				    CY_BUFAUX_SIZE, 0);
		if (size) {
			inet_aton(bufaux, &aux);
			long_ret = aux.s_addr;
		} else {
			long_ret = 0;
		}
		return (unsigned char *) &long_ret;

	case CYRESOLVERORDER:
		*write_method = write_cyResolverOrder;
		*var_len = cy_rw_parse("/etc/host.conf", "order", bufaux,
				      CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYMULTIPLEIP:
		*write_method = write_cyMultipleIP;
		*var_len = cy_rw_parse("/etc/host.conf", "multi", bufaux,
				   CY_BUFAUX_SIZE, 0);
		return (unsigned char *)bufaux;

	case CYDNSPRISERV:
		*write_method = write_cyDNSpriserv;
		size = cy_rw_parse("/etc/resolv.conf", "nameserver", bufaux,
				 CY_BUFAUX_SIZE, 0);
		if (size) {
			inet_aton(bufaux, &aux);
			long_ret = aux.s_addr;
		} else {
			long_ret = 0;
		}
		return (unsigned char *) &long_ret;

	case CYDNSSECSERV:
		*write_method = write_cyDNSsecserv;
		size = cy_rw_parse("/etc/resolv.conf", "2nameserver", bufaux,
				 CY_BUFAUX_SIZE, 0);
		if (size) {
			inet_aton(bufaux, &aux);
			long_ret = aux.s_addr;
		} else {
			long_ret = 0;
		}
		return (unsigned char *) &long_ret;

	case CYDNSDOMAIN:
		*write_method = write_cyDNSdomain;
		*var_len = cy_rw_parse("/etc/resolv.conf", "domain", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALINCLUDE:
		*write_method = write_cySerialInclude;
		*var_len = cy_rwsp_conf("conf", "include", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALNFS:
		*write_method = write_cySerialNFS;
		*var_len = cy_rwsp_conf("conf", "nfs_data_buffering", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALLOCKDIR:
		*write_method = write_cySerialLockDir;
		*var_len = cy_rwsp_conf("conf", "lockdir", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALRLOGIN:
		*write_method = write_cySerialRlogin;
		*var_len = cy_rwsp_conf("conf", "rlogin", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALPPPD:
		*write_method = write_cySerialPppd;
		*var_len = cy_rwsp_conf("conf", "ppp", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALTELNET:
		*write_method = write_cySerialTelnet;
		*var_len = cy_rwsp_conf("conf", "telnet", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALSSH:
		*write_method = write_cySerialSsh;
		*var_len = cy_rwsp_conf("conf", "ssh", bufaux,
				 CY_BUFAUX_SIZE, 0);
		return (unsigned char *) bufaux;

	case CYSERIALLOCALLOGINS:
		*write_method = write_cySerialLocalLogins;
		long_ret = cy_rwsp_conf("conf", "locallogins", bufaux, 
					CY_BUFAUX_SIZE, 0);
		if (long_ret) long_ret = atoi(bufaux);
		return (unsigned char *) &long_ret;

	case CYSERIALFACILITY:
		*write_method = write_cySerialFacility;
		long_ret = cy_rwsp_conf("conf", "facility", bufaux, 
					CY_BUFAUX_SIZE, 0);
		if (long_ret) long_ret = atoi(bufaux);
		return (unsigned char *) &long_ret;

	case CYSERIALDBFACILITY:
		*write_method = write_cySerialDBFacility;
		long_ret = cy_rwsp_conf("conf", "DB_facility", bufaux, 
					CY_BUFAUX_SIZE, 0);
		if (long_ret) long_ret = atoi(bufaux);
		return (unsigned char *) &long_ret;

	default:
	}
	return NULL;
}

int
write_cyHostName(int action,
		 u_char * var_val,
		 u_char var_val_type,
		 size_t var_val_len,
		 u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 40) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/hostname", NULL, var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyConsoleBanner(int action,
		      u_char * var_val,
		      u_char var_val_type,
		      size_t var_val_len,
		      u_char * statP, oid * name, size_t name_len)
{

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			fprintf(stderr,
				"write to cyConsoleBanner not ASN_OCTET_STR\n");
			return SNMP_ERR_WRONGTYPE;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/issue", NULL, var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyMotd(int action,
	     u_char * var_val,
	     u_char var_val_type,
	     size_t var_val_len,
	     u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 250) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/motd", NULL, var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthDhcpc(int action,
		 u_char * var_val,
		 u_char var_val_type,
		 size_t var_val_len,
		 u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_INTEGER) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > sizeof(long)) {
			return SNMP_ERR_WRONGLENGTH;
		}
		if (*((long *) var_val) > 2) {
			return SNMP_ERR_WRONGVALUE;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		sprintf(bufaux, "%d", *((long *) var_val));
		cy_rwsp_conf("conf", "dhcp_client", bufaux, strlen(bufaux),
			     1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthIPaddr(int action,
		  u_char * var_val,
		  u_char var_val_type,
		  size_t var_val_len,
		  u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			fprintf(stderr,
				"write to cyEthIPaddr not ASN_IPADDRESS\n");
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "eth_ip", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthIPmask(int action,
		  u_char * var_val,
		  u_char var_val_type,
		  size_t var_val_len,
		  u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			fprintf(stderr,
				"write to cyEthIPmask not ASN_IPADDRESS\n");
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "eth_mask", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthMTU(int action,
	       u_char * var_val,
	       u_char var_val_type,
	       size_t var_val_len,
	       u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_INTEGER) {
			fprintf(stderr,
				"write to cyEthMTU not ASN_INTEGER\n");
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > sizeof(long)) {
			fprintf(stderr, "write to cyEthMTU: bad length\n");
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		sprintf(bufaux, "%d", *((long *) var_val));
		cy_rwsp_conf("conf", "eth_mtu", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthIPaddr2(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "eth_ip_alias", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyEthIPmask2(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "eth_mask_alias", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyResolverOrder(int action,
		      u_char * var_val,
		      u_char var_val_type,
		      size_t var_val_len,
		      u_char * statP, oid * name, size_t name_len)
{
	int size;
	unsigned char *order_list[] = {
		"", "hosts", "bind", "hosts,bind", "bind,hosts"
	};

	switch (action) {

	case RESERVE1:
		if (var_val_len > 15) {
			return SNMP_ERR_WRONGLENGTH;
		}
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len) {
			strncpy(bufaux, var_val, var_val_len);
			bufaux[var_val_len] = 0x00;

			for (size = 0; size < 5; size++) {
				if (!strcmp(bufaux, order_list[size])) {
					break;
				}
			}
			if (size == 5) {
				return SNMP_ERR_WRONGVALUE;
			}
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/host.conf", "order", var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyMultipleIP(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 4) {
			return SNMP_ERR_WRONGLENGTH;
		}
		if (var_val_len) {
			strncpy(bufaux, var_val, var_val_len);
			bufaux[var_val_len] = 0x00;
			if (strcmp(bufaux, "ON")) {
				if (strcmp(bufaux, "OFF")) {
					return SNMP_ERR_WRONGVALUE;
				}
			}
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/host.conf", "multi",var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyDNSpriserv(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/resolv.conf", "nameserver",
				   bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyDNSsecserv(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	struct in_addr aux;

	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_IPADDRESS) {
			return SNMP_ERR_WRONGTYPE;
		}
		aux.s_addr = *((ulong *) var_val);
		paux = inet_ntoa(aux);
		if (!inet_aton(paux, &aux)) {
			return SNMP_ERR_WRONGVALUE;
		}
		strcpy(bufaux, paux);
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/resolv.conf", "2nameserver",
				   bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cyDNSdomain(int action,
		  u_char * var_val,
		  u_char var_val_type,
		  size_t var_val_len,
		  u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 40) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rw_parse("/etc/resolv.conf", "domain", var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}


int
write_cySerialInclude(int action,
		      u_char * var_val,
		      u_char var_val_type,
		      size_t var_val_len,
		      u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 40) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "include", var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialNFS(int action,
		  u_char * var_val,
		  u_char var_val_type,
		  size_t var_val_len,
		  u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "nfs_data_buffering", var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}


int
write_cySerialLockDir(int action,
		      u_char * var_val,
		      u_char var_val_type,
		      size_t var_val_len,
		      u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "lockdir",var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialRlogin(int action,
		     u_char * var_val,
		     u_char var_val_type,
		     size_t var_val_len,
		     u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "rlogin", var_val, var_val_len,1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialPppd(int action,
		   u_char * var_val,
		   u_char var_val_type,
		   size_t var_val_len,
		   u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "ppp", var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialTelnet(int action,
		     u_char * var_val,
		     u_char var_val_type,
		     size_t var_val_len,
		     u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "telnet", var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialSsh(int action,
		  u_char * var_val,
		  u_char var_val_type,
		  size_t var_val_len,
		  u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_OCTET_STR) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > 50) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		cy_rwsp_conf("conf", "ssh", var_val, var_val_len, 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialLocalLogins(int action,
			  u_char * var_val,
			  u_char var_val_type,
			  size_t var_val_len,
			  u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_INTEGER) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > sizeof(long)) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		sprintf(bufaux, "%d", *((long *)var_val));
		cy_rwsp_conf("conf", "locallogins", bufaux,
				 strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialFacility(int action,
		       u_char * var_val,
		       u_char var_val_type,
		       size_t var_val_len,
		       u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_INTEGER) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > sizeof(long)) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		sprintf(bufaux, "%d", *((long*)var_val));
		cy_rwsp_conf("conf", "facility", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}

int
write_cySerialDBFacility(int action,
			 u_char * var_val,
			 u_char var_val_type,
			 size_t var_val_len,
			 u_char * statP, oid * name, size_t name_len)
{
	switch (action) {
	case RESERVE1:
		if (var_val_type != ASN_INTEGER) {
			return SNMP_ERR_WRONGTYPE;
		}
		if (var_val_len > sizeof(long)) {
			return SNMP_ERR_WRONGLENGTH;
		}
		break;

	case RESERVE2:
		break;

	case FREE:
		break;

	case ACTION:
		sprintf(bufaux, "%d", *((long*)var_val));
		cy_rwsp_conf("conf", "DB_facility", bufaux, strlen(bufaux), 1);
		break;

	case UNDO:
		break;

	case COMMIT:
		break;
	}
	return SNMP_ERR_NOERROR;
}
