#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>
#include <ucd-snmp/ucd-snmp-agent-includes.h>
#include "cyPM.h"

extern int cy_numports;

oid cyNumberOfPM_oid[] = { PROD_OID, 5 };
oid cyPMTable_oid[] = { PROD_OID, 5 , 2};
oid cyPMUnitTable_oid[] = { PROD_OID, 5 , 3};
oid cyOutletTable_oid[] = { PROD_OID, 5, 4 };

#define CYPM_INDEX	PROD_OID_SIZE + 4
#define CYPMUnit_INDEX	PROD_OID_SIZE + 5
#define CYOUTLET_INDEX	PROD_OID_SIZE + 5

struct variable2 cyNumberOfPM_variables[] = {
#define CYNumberOfPM	1
	{CYNumberOfPM, ASN_INTEGER, RONLY, var_cyNumberOfPM, 1, { 1}},
};

struct variable4 cyPMTable_variables[] = {
#define CYPMSerialPortN	1
	{CYPMSerialPortN, ASN_INTEGER, RONLY, var_cyPMTable, 2, {1, 1}},
#define CYPMNumOutlets	2
	{CYPMNumOutlets, ASN_INTEGER, RONLY, var_cyPMTable, 2, {1, 2}},
#define CYPMNumUnits	3
	{CYPMNumUnits, ASN_INTEGER, RONLY, var_cyPMTable, 2, {1, 3}},
#define CYPMCurrent	4
	{CYPMCurrent, ASN_OCTET_STR, RONLY, var_cyPMTable, 2, {1, 4}},
#define CYPMVersion	5
	{CYPMVersion, ASN_OCTET_STR, RONLY, var_cyPMTable, 2, {1, 5}},
#define CYPMTemperature	6
	{CYPMTemperature, ASN_OCTET_STR, RONLY, var_cyPMTable, 2, {1, 6}},
#define CYPMCommand	7
	{CYPMCommand, ASN_OCTET_STR, RWRITE, var_cyPMTable, 2, {1, 7}}
};

struct variable4 cyPMUnitTable_variables[] = {
#define CYPMUnitVersion	2
	{CYPMUnitVersion, ASN_OCTET_STR, RONLY, var_cyPMUnitTable, 2, {1, 2}},
#define CYPMUnitOutlets	3
	{CYPMUnitOutlets, ASN_INTEGER, RONLY, var_cyPMUnitTable, 2, {1, 3}},
#define CYPMUnitFirstOutlet	4
	{CYPMUnitFirstOutlet, ASN_INTEGER, RONLY, var_cyPMUnitTable, 2, {1, 4}},
#define CYPMUnitCurrent	5
	{CYPMUnitCurrent, ASN_INTEGER, RONLY, var_cyPMUnitTable, 2, {1, 5}},
#define CYPMUnitMaxCurrent	6
	{CYPMUnitMaxCurrent, ASN_INTEGER, RWRITE, var_cyPMUnitTable, 2, {1, 6}},
#define CYPMUnitTemperature	7
	{CYPMUnitTemperature, ASN_INTEGER, RONLY, var_cyPMUnitTable, 2, {1, 7}},
#define CYPMUnitMaxTemperature	8
	{CYPMUnitMaxTemperature, ASN_INTEGER, RWRITE, var_cyPMUnitTable, 2, {1, 8}},
};

struct variable4 cyOutletTable_variables[] = {
#define CYOUTLETName	2
	{CYOUTLETName, ASN_OCTET_STR, RWRITE, var_cyOutletTable, 2, {1, 2}},
#define CYOUTLETServer	3
	{CYOUTLETServer, ASN_OCTET_STR, RONLY, var_cyOutletTable, 2, {1, 3}},
#define CYOUTLETPower	4
	{CYOUTLETPower, ASN_INTEGER, RWRITE, var_cyOutletTable, 2, {1, 4}},
#define CYOUTLETLock	5
	{CYOUTLETLock, ASN_INTEGER, RWRITE, var_cyOutletTable, 2, {1, 5}},
#define CYOUTLETInterval	6
	{CYOUTLETInterval, ASN_INTEGER, RWRITE, var_cyOutletTable, 2, {1, 6}}
};

static struct CY_SHM_SNMP * cy_shm;

static void cypm_system(unsigned char *cmd, unsigned int nps,
	int outlet, unsigned char *resp, int max_size)
{
	FILE *FRESP;
	int len=0;
	unsigned char tempname[50];
	unsigned char cypm[256];

	sprintf(tempname,"/tmp/temp.snmp%d",nps);

	switch (outlet) {
		case -1 :
			sprintf(cypm,"/bin/pmCommand %d %s >%s 2>&1",nps,cmd,tempname);
			break;
		case 0 :
			sprintf(cypm,"/bin/pmCommand %d %s all >%s 2>&1",nps,cmd,tempname);
			break;
		default :
			if (!max_size && resp) {
				sprintf(cypm,"/bin/pmCommand %d %s %d %s >%s 2>&1",
						nps,cmd,outlet,resp,tempname);
			} else {
				sprintf(cypm,"/bin/pmCommand %d %s %d >%s 2>&1",
						nps,cmd,outlet,tempname);
			}
			break;
	}
	
	system(cypm);

	if (max_size) {
		if (FRESP = fopen(tempname,"r")) {
			while (fgets(cypm,256,FRESP)) {
				if ((len = strlen(cypm)) > 2) {
					strncat(resp,cypm,len - 2);
					strncat(resp,"  ",2);
				}
			}
			fclose(FRESP);
		}
	}

	sprintf(cypm,"rm %s",tempname);
	system(cypm);
}

void init_cyPM(void) 
{
	int shm_id;
	
	DEBUGMSGTL(("cyPM","Initializing\n"));

	if ((shm_id = shmget(CY_ID_SHM,0,SHM_PERM)) < 0) {
		return;
	}
	
	cy_shm = (struct CY_SHM_SNMP *)shmat(shm_id,(void *) CY_ADDR_SHM,0);

	REGISTER_MIB("cyNumberOfPM", cyNumberOfPM_variables, variable2, cyNumberOfPM_oid);
	/** Initializes the cyPMTable module */
	REGISTER_MIB("cyPMTable", cyPMTable_variables, variable4, cyPMTable_oid);
	
	/** Initializes the cyPMUnitTable module */
	REGISTER_MIB("cyPMUnitTable", cyPMUnitTable_variables, variable4,
		       	cyPMUnitTable_oid);
	
	/** Initializes the cyOutletTable module */
	REGISTER_MIB("cyOutletTable", cyOutletTable_variables, variable4, 
			cyOutletTable_oid);
}

unsigned char *var_cyNumberOfPM(struct variable *vp,
				 oid * name,
				 size_t * length,
				 int exact,
				 size_t * var_len,
				 WriteMethod ** write_method)
{
	/*
	 * variables we may use later 
	 */
	static long long_ret;
	int i;
	
	if (header_generic(vp, name, length, exact, var_len, write_method)
			== MATCH_FAILED)
		return NULL;
	 
	if (vp->magic == CYNumberOfPM) {
		long_ret = 0;
		for (i = 0; i < cy_numports ; i++) {
			if (cy_shm->PMD_pms[i]) 
				long_ret++;
		}
		if (write_method) *write_method = 0;
		if (var_len) *var_len = sizeof(long);
		return (unsigned char *) &long_ret;
	} else {
		return NULL;
	}
}

/* Find out the next PMD serial port number */
static int cy_find_nsp(int first) 
{
	int i;

	for (i=first; i < cy_numports ; i++) {
		if (cy_shm->PMD_pms[i])
			return i+1;
	}
	return 0;
}

/* Compare 'name' to vp->name for the best match or an exact match
 * Also check that 'name' is valid PM port serial number.
 * Store a successful match in 'name', and increment the OID instance if the
 * match was not exact.
 */
static int header_PMTable(struct variable *vp, oid * name, size_t * length,
			int exact, size_t * var_len, WriteMethod ** write_method)
{
	int i, rtest,nsp;
	oid newname[MAX_OID_LEN];

	for (i=0, rtest=0; (i < (int)vp->namelen) && (i < (int)(*length)) &&
		 !rtest; i++) {
		if (name[i] != vp->name[i]) {
			if (name[i] < vp->name[i])
				rtest = -1;
			else
				rtest = 1;
		}
	}

	// find the first serial port with PM connected
	nsp = cy_find_nsp(0);

	if ((rtest > 0) || 
		((exact == 1) && (rtest || ((int)*length != (int)(vp->namelen+1))))) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memset(newname, 0, sizeof(newname));

	if (((int)*length <= (int)vp->namelen) || (rtest == -1)) {
		memmove(newname, vp->name,(int)vp->namelen * sizeof(oid));
		newname[vp->namelen] = nsp; 
		*length = vp->namelen + 1;
	} else {
		if ((int)*length > ((int)vp->namelen+1)) {
			// exact case checked earlier
			*length = vp->namelen + 1;
			memmove(newname, name, (int)*length * sizeof(oid));
			if (name[*length - 1] < ULONG_MAX) {
				nsp = cy_find_nsp(name[*length - 1]);
				newname[*length - 1] = nsp;
			} else {
				nsp = newname[*length - 1] = name[*length - 1];
			}
		} else {
			*length = vp->namelen + 1;
			memmove(newname,name, (*length) * sizeof(oid));
			if (!exact) {
				if (name[*length - 1] < ULONG_MAX) {
					nsp = cy_find_nsp(name[*length - 1]);
					newname[*length - 1] = nsp;
				} else {
					nsp = newname[*length - 1] = name[*length - 1];
				}
			}
		}
	}

	if (nsp == 0) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memmove(name, newname, (*length) * sizeof(oid));
	// default values
	if (write_method) *write_method = 0;
	if (var_len) *var_len = sizeof(long);
	
	return MATCH_SUCCEEDED;
}		


/*
 * var_cyPMTable():
 */
unsigned char *var_cyPMTable(struct variable *vp,
				 oid * name,
				 size_t * length,
				 int exact,
				 size_t * var_len,
				 WriteMethod ** write_method)
{
	/*
	 * variables we may use later 
	 */
	static long long_ret;
	static int size;
	static unsigned char string[1024];
	int cy_nsp;

	if (cy_shm->PMD_pid == 0) { // PMD is not OK
		return NULL;
	}

	// verify the PM serial port index
	if (header_PMTable(vp, name, length, exact, var_len, write_method)
	    == MATCH_FAILED)
		return NULL;

	memset(string, 0, 1024);

	cy_nsp = name[CYPM_INDEX];

	if (!cy_shm->PMD_pms[cy_nsp-1]) {
		return NULL; 	// it is not a IPDU
	}
	

	/*
	 * * this is where we do the value assignments for the mib results.
	 */
	switch (vp->magic) {
		case CYPMSerialPortN :
			long_ret = cy_nsp;
			return (unsigned char *) &long_ret;
			break;

		case CYPMNumOutlets:
			long_ret = cy_shm->PMD_outlets[cy_nsp - 1];
			return (unsigned char *) &long_ret;
			break;

		case CYPMNumUnits:
			long_ret = cy_shm->PMD_pms[cy_nsp - 1];
			return (unsigned char *) &long_ret;
			break;

		case CYPMCurrent:
			cypm_system("current", cy_nsp,-1,string,1024);
			*var_len = strlen(string);
			return (unsigned char *) &string;
			break;
			
		case CYPMVersion:
			cypm_system("ver", cy_nsp,-1,string,1024);
			*var_len = strlen(string);
			return (unsigned char *) &string;
			break;
			
		case CYPMTemperature:
			cypm_system("temperature", cy_nsp,-1,string,1024);
			*var_len = strlen(string);
			return (unsigned char *) &string;
			break;
			
		case CYPMCommand:
			*var_len = 0;
			*write_method = write_cyPM_command;
			return (unsigned char *) &string;
			break;
			
		default:
			break;
	}
	return NULL;
}

int write_cyPM_command(int action, u_char * var_val, u_char var_val_type,
		   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp;

	cy_nsp = name[CYPM_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_OCTET_STR) {
				fprintf(stderr,"write to cyPMCommand not ASN_OCTET_STR\n");
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > 100) {
				return SNMP_ERR_WRONGLENGTH;
			}
			break;

		case ACTION:
			*(var_val + var_val_len) = 0x00;
			cypm_system(var_val, cy_nsp,-1,NULL,0);
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

/* Compare 'name' to vp->name for the best match or an exact match
 * Also check that 'name' is valid PM port serial number.
 * Store a successful match in 'name', and increment the OID instance if the
 * match was not exact.
 */
static int header_PMUnitTable(struct variable *vp, oid * name, size_t * length,
		int exact, size_t * var_len, WriteMethod ** write_method)
{
	int i, rtest,nsp,ipdu;
	oid newname[MAX_OID_LEN];

	for (i=0, rtest=0; (i < (int)vp->namelen) && (i < (int)(*length)) &&
		 !rtest; i++) {
		if (name[i] != vp->name[i]) {
			if (name[i] < vp->name[i])
				rtest = -1;
			else
				rtest = 1;
		}
	}

	// find the first serial port with PM connected
	nsp = cy_find_nsp(0);
	ipdu = 1;

	if ((rtest > 0) || 
		((exact == 1) && (rtest || ((int)*length != (int)(vp->namelen+2))))) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memset(newname, 0, sizeof(newname));

	if (((int)*length <= (int)vp->namelen) || (rtest == -1)) {
		memmove(newname, vp->name,(int)vp->namelen * sizeof(oid));
		newname[vp->namelen] = nsp; 
		newname[vp->namelen + 1] = ipdu; 
		*length = vp->namelen + 2;
	} else {
		if ((int)*length > ((int)vp->namelen+2)) {
			// exact case checked earlier
			*length = vp->namelen + 2;
			memmove(newname, name, (int)*length * sizeof(oid));
			if (name[*length - 1] < ULONG_MAX) {
				nsp = name[*length - 2];
				ipdu = newname[*length - 1] = name[*length-1] + 1;
				if (ipdu > cy_shm->PMD_pms[nsp-1]) {
					nsp = cy_find_nsp(nsp);
					newname[*length - 2] = nsp;
					ipdu = newname[*length - 1] = 1;
				} 
			} else {
				nsp = newname[*length - 2] = name[*length - 2];
				ipdu = newname[*length - 1] = name[*length - 1];
			}
		} else {
			*length = vp->namelen + 2;
			memmove(newname,name, (*length) * sizeof(oid));
			if (!exact) {
				if (name[*length - 1] < ULONG_MAX) {
					nsp = name[*length - 2];
					ipdu = newname[*length - 1] = name[*length-1] + 1;
					if (ipdu > cy_shm->PMD_pms[nsp-1]) {
						nsp = cy_find_nsp(nsp);
						newname[*length - 2] = nsp;
						ipdu = newname[*length - 1] = 1;
					} 
				} else {
					nsp = newname[*length - 2] = name[*length - 2];
					ipdu = newname[*length - 1] = name[*length - 1];
				}
			}
		}
		
	}

	if ((nsp == 0) || (ipdu > cy_shm->PMD_pms[nsp-1])) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memmove(name, newname, (*length) * sizeof(oid));
	// default values
	if (write_method) *write_method = 0;
	if (var_len) *var_len = sizeof(long);
	
	return MATCH_SUCCEEDED;
}		

// parse routines

// command ver :
// IPDU <number> Hw with <outlets> outlets <number> AMPs max Sw V <version>
//  
static int cypm_parse_units(unsigned char *status, int ipdu, int field)
{
	unsigned char *aux,*s;
	int i, outlets = 0, max;

	aux = status;

	max = (field == CYPMUnitOutlets) ? ipdu + 1 : 2;
	
	for (i = 1; i < max; i++) {
		if ((s = strstr(aux,"Hw with ")) == NULL) {
			break;
		} 
		aux = s += 8;
		while (*s && isdigit(*s)) s++;
		*s = 0x00;
		outlets += atoi(aux);
	}
	return outlets;			
}

// command current :
// IPDU <number>: True RMS current : <value x.x>A. Maximum current: <value x.x>A
//  
static int cypm_parse_current(unsigned char *status, int field)
{
	unsigned char *aux,*s;
	int a, b ;

	aux = status;

	if (field == CYPMUnitCurrent) {
		s = strstr(aux,"RMS current: ");
		b = 13;
	} else {
		s = strstr(aux,"Maximum current: ");
		b = 17;
	}
	
	if (s == NULL) return 0;
	
	aux = s += b;
	while (*s && isdigit(*s)) s++;
	if (*s == '.') {
		b = *(s+1) - 0x30;
	} else {
		b = 0;
	}
	*s++ = 0x00;
	a = atoi(aux);
	return ((a*10) + b);			
}

// command temperature :
// IPDU <number>: Temperature detected: <value>F (<value> C). Maximum temperature detected: <value>F (<value> C)
// PM version 2.1.0 : the format of the message was changed
// IPDU <number>: Temperature: <value>oC (<value>oF). Maximum: <value>oC (<value>oC)
//  
static int cypm_parse_temp(unsigned char *status, int field)
{
	unsigned char *aux,*s;
	int a,b;

	aux = status;

	if (field == CYPMUnitTemperature) {
		s = strstr(aux,"Temperature: ");
	} else {
		s = strstr(aux,"Maximum: ");
	}
	
	if (s == NULL) return 0;
	
	while (*s && !isdigit(*s)) s++;
	aux = s ;

	while (*s && isdigit(*s)) s++;
	if (*s == '.') {
		b = *(s+1) - 0x30;
	} else {
		b = 0;
	}
	*s++ = 0x00;
	a = atoi(aux);
	return (a*10)+b;			
}

/*
 * var_cyPMUnitTable():
 */
unsigned char *var_cyPMUnitTable(struct variable *vp,
				 oid * name,
				 size_t * length,
				 int exact,
				 size_t * var_len,
				 WriteMethod ** write_method)
{
	/*
	 * variables we may use later 
	 */
	static long long_ret;
	static int size;
	static unsigned char string[1024];
	int cy_nsp,cy_pm;

	if (cy_shm->PMD_pid == 0) { // PMD is not OK
		return NULL;
	}

	// verify the PM serial port index
	if (header_PMUnitTable(vp, name, length, exact, var_len, write_method)
	    == MATCH_FAILED)
		return NULL;

	memset(string, 0, 1024);

	cy_nsp = name[CYPM_INDEX];
	cy_pm = name[CYPMUnit_INDEX];

	if (!cy_shm->PMD_pms[cy_nsp-1] || 
	    (cy_pm > cy_shm->PMD_pms[cy_nsp-1])) {
		return NULL; 
	}
	
	/*
	 * * this is where we do the value assignments for the mib results.
	 */
	switch (vp->magic) {
		case CYPMUnitVersion:
			cypm_system("ver", cy_nsp,cy_pm,string,1024);
			*var_len = strlen(string);
			return (unsigned char *) &string;
			break;
			
		case CYPMUnitOutlets:
			cypm_system("ver", cy_nsp,cy_pm,string,1024);
			long_ret = cypm_parse_units(string,cy_pm,CYPMUnitOutlets);
			return (unsigned char *) &long_ret;
			break;
			
		case CYPMUnitFirstOutlet:
			if (cy_pm == 1) {
				long_ret = 1;
			} else {
				cypm_system("ver", cy_nsp,-1,string,1024);
				long_ret = cypm_parse_units(string,cy_pm,CYPMUnitFirstOutlet) + 1;
			}
			return (unsigned char *) &long_ret;
			
		case CYPMUnitCurrent :
			cypm_system("current",cy_nsp,cy_pm,string,1024);
			long_ret = cypm_parse_current(string,CYPMUnitCurrent);
			return (unsigned char *) &long_ret;

		case CYPMUnitMaxCurrent :
			cypm_system("current",cy_nsp,cy_pm,string,1024);
			long_ret = cypm_parse_current(string,CYPMUnitMaxCurrent);
			*write_method = write_cyPMUnit_current;
			return (unsigned char *) &long_ret;

		case CYPMUnitTemperature :
			cypm_system("temperature",cy_nsp,cy_pm,string,1024);
			long_ret = cypm_parse_temp(string,CYPMUnitTemperature);
			return (unsigned char *) &long_ret;

		case CYPMUnitMaxTemperature :
			cypm_system("temperature",cy_nsp,cy_pm,string,1024);
			long_ret = cypm_parse_temp(string,CYPMUnitMaxTemperature);
			*write_method = write_cyPMUnit_temp;
			return (unsigned char *) &long_ret;

		default:
			break;
	}
	return NULL;
}

int write_cyPMUnit_current(int action, u_char * var_val, u_char var_val_type,
	   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_pm;

	cy_nsp = name[CYPM_INDEX];
	cy_pm = name[CYPMUnit_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_INTEGER) {
				fprintf(stderr,"write to cyOutletPower not ASN_INTEGER\n");
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > sizeof(long)) {
				return SNMP_ERR_WRONGLENGTH;
			}
			if (*(long*)var_val) {
					return SNMP_ERR_WRONGVALUE;
			}
			break;

		case ACTION:
			cypm_system("current reset",cy_nsp,-1,NULL,0);
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

int write_cyPMUnit_temp(int action, u_char * var_val, u_char var_val_type,
		   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_pm;

	cy_nsp = name[CYPM_INDEX];
	cy_pm = name[CYPMUnit_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_INTEGER) {
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > sizeof(long)) {
				return SNMP_ERR_WRONGLENGTH;
			}
			if (*(long*)var_val) {
					return SNMP_ERR_WRONGVALUE;
			}
			break;

		case ACTION:
			cypm_system("temperature reset",cy_nsp,-1,NULL,0);
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

/* Compare 'name' to vp->name for the best match or an exact match
 * Also check that 'name' is valid PM port serial number.
 * Store a successful match in 'name', and increment the OID instance if the
 * match was not exact.
 */
static int header_OutletTable(struct variable *vp, oid * name, size_t * length,
			int exact, size_t * var_len, WriteMethod ** write_method)
{
	int i, rtest,nsp,outlet;
	oid newname[MAX_OID_LEN];

	for (i=0, rtest=0; (i < (int)vp->namelen) && (i < (int)(*length)) &&
		 !rtest; i++) {
		if (name[i] != vp->name[i]) {
			if (name[i] < vp->name[i])
				rtest = -1;
			else
				rtest = 1;
		}
	}

	// find the first serial port with PM connected
	nsp = cy_find_nsp(0);
	outlet = 1;

	if ((rtest > 0) || 
		((exact == 1) && (rtest || ((int)*length != (int)(vp->namelen+2))))) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memset(newname, 0, sizeof(newname));

	if (((int)*length <= (int)vp->namelen) || (rtest == -1)) {
		memmove(newname, vp->name,(int)vp->namelen * sizeof(oid));
		newname[vp->namelen] = nsp; 
		newname[vp->namelen + 1] = outlet; 
		*length = vp->namelen + 2;
	} else {
		if ((int)*length > ((int)vp->namelen+2)) {
			// exact case checked earlier
			*length = vp->namelen + 2;
			memmove(newname, name, (int)*length * sizeof(oid));
			if (name[*length - 1] < ULONG_MAX) {
				nsp = name[*length - 2];
				outlet = newname[*length - 1] = name[*length-1] + 1;
				if (outlet > cy_shm->PMD_outlets[nsp-1]) {
					nsp = cy_find_nsp(nsp);
					newname[*length - 2] = nsp;
					outlet = newname[*length - 1] = 1;
				} 
			} else {
				nsp = newname[*length - 2] = name[*length - 2];
				outlet = newname[*length - 1] = name[*length - 1];
			}
		} else {
			*length = vp->namelen + 2;
			memmove(newname,name, (*length) * sizeof(oid));
			if (!exact) {
				if (name[*length - 1] < ULONG_MAX) {
					nsp = name[*length - 2];
					outlet = newname[*length - 1] = name[*length-1] + 1;
					if (outlet > cy_shm->PMD_outlets[nsp-1]) {
						nsp = cy_find_nsp(nsp);
						newname[*length - 2] = nsp;
						outlet = newname[*length - 1] = 1;
					} 
				} else {
					nsp = newname[*length - 2] = name[*length - 2];
					outlet = newname[*length - 1] = name[*length - 1];
				}
			}
		}
		
	}

	if ((nsp == 0) || (outlet > cy_shm->PMD_outlets[nsp-1])) {
		if (var_len)
			*var_len = 0;
		return MATCH_FAILED;
	}

	memmove(name, newname, (*length) * sizeof(oid));
	// default values
	if (write_method) *write_method = 0;
	if (var_len) *var_len = sizeof(long);
	
	return MATCH_SUCCEEDED;
}		

static int cypm_parse_status(unsigned char *status, int field)
{
	unsigned char *pname = 0, *aux;
	int i;

	aux = status;

	// find the outlet number
	while (*aux && !isdigit(*aux)) aux++;

	// skip the outlet number
	while (*aux && isdigit(*aux)) aux++;

	for (i=0; ((i < 3) && *aux); i++) {
		while (*aux && isspace(*aux)) aux++;
		pname = aux;
		while (*aux && !isspace(*aux)) aux++;
		if (*aux) *aux++ = 0x00;
	
		// is there a name 
		if (!strcmp(pname,"Locked")) {
			if (field == CYOUTLETLock) {
				return 1;
			}
			continue;
		}
		if (!strcmp(pname,"Unlocked")) {
			if (field == CYOUTLETLock) {
				return 0;
			}
			continue;
		}
		if (!strcmp(pname,"ON")) {
			if (field == CYOUTLETPower) {
				return 1;
			}
			continue;
		}
		if (!strcmp(pname,"OFF")) {
			if (field == CYOUTLETPower) {
				return 0;
			}
			continue;
		}
		if ((i==0) &&(field == CYOUTLETName)){
			strcpy(status,pname);
			return 0;
		}
			       
	}
	*status = 0x00;
	return -1;
}

// command interval :
// 	<outlet number>: Outlet cycle interval is <value x.x> S
//  
static int cypm_parse_interval(unsigned char *status)
{
	unsigned char *aux,*s;
	int a, b ;

	aux = status;

	s = strstr(aux,"interval is ");
	
	if (s == NULL) return 0;
	
	aux = s += 12;
	while (*s && isdigit(*s)) s++;
	if (*s == '.') {
		b = *(s+1) - 0x30;
	} else {
		b = 0;
	}
	*s++ = 0x00;
	a = atoi(aux);
	return ((a*10) + b);			
}

/*
 * var_cyOutletTable():
 */
unsigned char *var_cyOutletTable(struct variable *vp,
				 oid * name,
				 size_t * length,
				 int exact,
				 size_t * var_len,
				 WriteMethod ** write_method)
{
	/*
	 * variables we may use later 
	 */
	static long long_ret;
	static int size;
	static unsigned char string[1024];
	int cy_nsp,cy_outlet;

	if (cy_shm->PMD_pid == 0) { // PMD is not OK
		return NULL;
	}

	// verify the PM serial port index
	if (header_OutletTable(vp, name, length, exact, var_len, write_method)
	    == MATCH_FAILED)
		return NULL;

	memset(string, 0, 1024);

	cy_nsp = name[CYPM_INDEX];
	cy_outlet = name[CYOUTLET_INDEX];

	if (!cy_shm->PMD_pms[cy_nsp-1] || 
	    (cy_outlet > cy_shm->PMD_outlets[cy_nsp-1])) {
		return NULL; 
	}
	
	/*
	 * * this is where we do the value assignments for the mib results.
	 */
	switch (vp->magic) {
		case CYOUTLETName:
			if (cy_outlet && cy_shm->PMD_outlets[cy_nsp-1]) {
				cypm_system("status", cy_nsp,cy_outlet,string,1024);
				if (cypm_parse_status(string,CYOUTLETName)==-1) {
					sprintf(string,"OUTLET-%d",cy_outlet);
				}
			} else {
				strcpy(string,"all");
			}
			*write_method = write_cyOutlet_name;
			*var_len = strlen(string); 
			return (unsigned char *) &string;
			break;

		case CYOUTLETServer:
			if (cy_outlet) {
				cypm_system("getname", cy_nsp,cy_outlet,string,1024);
			} else {
				strcpy(string,"all");
			}
			*var_len = strlen(string);
			return (unsigned char *) &string;
			break;
			
		case CYOUTLETPower:
			if (cy_outlet && cy_shm->PMD_outlets[cy_nsp-1]) {
				cypm_system("status", cy_nsp,cy_outlet,string,1024);
				if ((long_ret = cypm_parse_status(string,
						CYOUTLETPower)) == -1) {
					long_ret = 3; // unknow
				}

			} else {
				long_ret = 3;
			}
			*write_method = write_cyOutlet_power;
			return (unsigned char *) &long_ret;
			break;
			
		case CYOUTLETLock:
			if (cy_outlet && cy_shm->PMD_outlets[cy_nsp-1]) {
				cypm_system("status", cy_nsp,cy_outlet,string,1024);
				if ((long_ret = cypm_parse_status(string,
						CYOUTLETLock)) == -1) {
					long_ret = 2; // unknow
				}
			} else {
				long_ret = 2;
			}
			*write_method = write_cyOutlet_lock;
			return (unsigned char *) &long_ret;
			
		case CYOUTLETInterval:
			if (cy_outlet && cy_shm->PMD_outlets[cy_nsp-1]) {
				cypm_system("interval", cy_nsp,cy_outlet,string,1024);
				if ((long_ret = cypm_parse_interval(string)) == -1) {
					long_ret = 0; // unknow
				}
			} else {
				long_ret = 0;
			}
			*write_method = write_cyOutlet_interval;
			return (unsigned char *) &long_ret;
			
		default:
			break;
	}
	return NULL;
}

int write_cyOutlet_name(int action, u_char * var_val, u_char var_val_type,
		size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_outlet;

	cy_nsp = name[CYPM_INDEX];
	cy_outlet = name[CYOUTLET_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_OCTET_STR) {
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > 8) {
				return SNMP_ERR_WRONGLENGTH;
			}
			break;

		case ACTION:
			*(var_val + var_val_len) = 0x00;
			cypm_system("name",cy_nsp,cy_outlet,var_val,0);
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;
	}
	return SNMP_ERR_NOERROR;
}

int write_cyOutlet_power(int action, u_char * var_val, u_char var_val_type,
		   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_outlet;

	cy_nsp = name[CYPM_INDEX];
	cy_outlet = name[CYOUTLET_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_INTEGER) {
				fprintf(stderr,"write to cyOutletPower not ASN_INTEGER\n");
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > sizeof(long)) {
				return SNMP_ERR_WRONGLENGTH;
			}
			if (*(long*)var_val > 2) {
					return SNMP_ERR_WRONGVALUE;
			}
			break;

		case ACTION:
			switch (*(long *) var_val) {
				case 0 : // Power OFF
					cypm_system("off",cy_nsp,cy_outlet,NULL,0);
					break;
				case 1 : // Power ON
					cypm_system("on",cy_nsp,cy_outlet,NULL,0);
					break;
				case 2 : // Power CYCLE
					cypm_system("cycle",cy_nsp,cy_outlet,NULL,0);
					break;
				default :
					return SNMP_ERR_WRONGVALUE;
					break;
			}
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

int write_cyOutlet_lock(int action, u_char * var_val, u_char var_val_type,
		   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_outlet;

	cy_nsp = name[CYPM_INDEX];
	cy_outlet = name[CYOUTLET_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_INTEGER) {
				fprintf(stderr,"write to cyOutletLock not ASN_INTEGER\n");
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > sizeof(long)) {
				return SNMP_ERR_WRONGLENGTH;
			}
			if (*(long*)var_val > 1) {
					return SNMP_ERR_WRONGVALUE;
			}
			break;

		case ACTION:
			switch (*(long *) var_val) {
				case 0 : // UNLOCK
					cypm_system("unlock",cy_nsp,cy_outlet,NULL,0);
					break;
				case 1 : // LOCK
					cypm_system("lock",cy_nsp,cy_outlet,NULL,0);
					break;
				default :
					return SNMP_ERR_WRONGVALUE;
					break;
			}
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

int write_cyOutlet_interval(int action, u_char * var_val, u_char var_val_type,
		   size_t var_val_len, u_char * statP, oid * name, size_t name_len)
{
	int cy_nsp, cy_outlet;
	unsigned char interv[20];

	cy_nsp = name[CYPM_INDEX];
	cy_outlet = name[CYOUTLET_INDEX];

	switch (action) {
		case RESERVE1:
			if (var_val_type != ASN_INTEGER) {
				return SNMP_ERR_WRONGTYPE;
			}
			if (var_val_len > sizeof(long)) {
				return SNMP_ERR_WRONGLENGTH;
			}
			if (*(long*)var_val > 655360) {
					return SNMP_ERR_WRONGVALUE;
			}
			if (*(long*)var_val < 5) {
					return SNMP_ERR_WRONGVALUE;
			}
			break;

		case ACTION:
			sprintf(interv,"%.1f",*(long*)var_val / 10);
			cypm_system("interval",cy_nsp,cy_outlet,interv,0);
			break;

		case RESERVE2:
		case COMMIT:
		case FREE:
		case UNDO:
			break;

	}
	return SNMP_ERR_NOERROR;
}

