/*
 * appl3.c -- Routines for the web pages (LINUX version)
 *
 */

/******************************** Description *********************************/

/*
 *	Routines for the web pages (asp/form/cgi)
 *	This module manages the web pages for information tables.
 */

/********************************* Includes ***********************************/

#include	"appl.h"
#include	<signal.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/wait.h>

/******************************* Definitions **********************************/
#define CONFIG_VERTICAL_TABLE    1 /* The information will be show in an inverted table */
#define CONFIG_ONE    			 2 /* There is only one line to read */
#define CONFIG_ADMIN			 4 /* The table will be used for administration */
#define CONFIG_MULTIPLE			 8 /* More than one entry can be chosen */
#define CONFIG_CFG			16 /* The table will be used for configuration */
#define CONFIG_REFRESH			32 /* The table will be refreshed each
					    30 seconds */

#define CONFIG_HEADER 1 /* The item should be shown as a header (in bold) */
#define CONFIG_ALIGN  6 /* Mask for alignment type */
#define CONFIG_RIGHT  2 /* The item should be shown with alignment to right */
#define CONFIG_LEFT   4 /* The item should be shown with alignment to left */
#define CONFIG_STAT	  8 /* The table will be for statistics */
#define CONFIG_PARSE 16 /* The item will be gotten by a proprietary routine in item->end */

/********** Routine types to be defined ************/
typedef int convert (unsigned char **, unsigned char *);
typedef unsigned char *getitem (unsigned char **, unsigned char *);

/************ Structures to be defined ***********/
/* List of items */
struct showitem {
    unsigned char *name;
    convert       *convchar;
    unsigned char *value;
    unsigned char *end;
    unsigned char config;
};

/* Table structure */
struct showtable {
    unsigned char   *filename;
    unsigned char   *title;
    unsigned char   *showorder;
    struct showitem *item;
    unsigned char    nitems;
    unsigned char    config;
    unsigned char    nheader;
    unsigned char   *formname;
    unsigned char   **inputstr;
};

/* Routines for conversion*/

/*****************************************************************************
 * Hex2IPAddr - Converts a number in HEX(as shown in /proc/net/route) to
 * IP Address.
 */
//static int Hex2IPAddr (unsigned char **s, unsigned char *d)
//{
//    struct in_addr a;
//    if (sscanf(*s, "%x", &(a.s_addr)) < 1) return 0;
//    strcpy (d, inet_ntoa(a));
//    return 1;
//}

/*****************************************************************************
 * Hex2Flag - Converts a number in HEX(as shown in /proc/net/route) to
 * set of active flags.
 */
//static unsigned char routeflags[] = {"UGHRDMAC!"};
//static int Hex2Flag (unsigned char **s, unsigned char *d)
//{
//    unsigned int flag;
//    int i;
//    if (!sscanf (*s, "%x", &flag)) return 0;
//    for (i=0; i<9; i++) {
//        if (flag & 1<<i) {
//            *d++ = routeflags[i];
//        }
//    }
//    *d=0;
//    return 1;
//}

/*****************************************************************************
 * Hex2HWType - Converts a number in HEX (as shown in /proc/net/arp) to
 * Hardware Type.
 */
static unsigned char *shwtypes[] = {"0x1", "0x7", "0x4", "0x3", "0x0"};
static unsigned char *shwstr[] = {"ether", "arc-net", "pro-net", "ax25", "netrom"};
static int Hex2HWType (unsigned char **s, unsigned char *d)
{
    int i;
    for (i=0; i<5; i++) {
        if (strcmp (*s, shwtypes[i]) == 0) {
            strcpy (d, shwstr[i]);
            return 1;
        }
    }
    return 0;
}

/*****************************************************************************
 * Hex2FlagARP - Converts a number in HEX(as shown in /proc/net/arp) to
 * set of active flags.
 */
static unsigned char arpflags[] = {"CMP"};
static int Hex2FlagARP (unsigned char **s, unsigned char *d)
{
    unsigned int flag;
    int i;
    if (!sscanf (*s+2, "%x", &flag)) return 0;
    for (i=0; i<3; i++) {
        if (flag & 1<<i) {
            *d++ = arpflags[i];
        }
    }
    *d=0;
    return 1;
}

/*****************************************************************************
 * Hex2IPAddrMask - Converts a number in HEX(as shown in /proc/net/ip_fwchains
 * to IP Address/Mask.
 */
static int Hex2IPAddrMask (unsigned char **s, unsigned char *d)
{
    struct in_addr a, c;
    unsigned char j;
    int i;
    unsigned int mask;

    if (sscanf(*s, "%x/%x", &(a.s_addr), &(c.s_addr)) < 2) return 0;
    mask=0;
    for (i=0;; i++) {
        j=0x80>>(i%8);
        if (c.s_addr == mask) {
            sprintf(d, "%s/%d", inet_ntoa(a), i);
            return 1;
        }
        if (i==32) break;
        *((unsigned char *)&mask + i/8) += j;
    }
    return 0;
}

static int AddHiValue (unsigned char **s, unsigned char *d)
{
    int hival, loval;
    unsigned long value[4];
    unsigned char digits[64];
    int i, rest, n;
    unsigned char *t = *((unsigned char **)((unsigned char *)s - sizeof(struct showitem)));
    unsigned char *u = *s;
    if (! sscanf (t, "%d", &hival)) return 0;
    if (! sscanf (u, "%d", &loval)) return 0;
    value[0] = (unsigned long)loval % 0x10000;
    value[1] = (unsigned long)loval / 0x10000;
    value[2] = (unsigned long)hival % 0x10000;
    value[3] = (unsigned long)hival / 0x10000;
    n=0;
    while (value[0] || value[1] || value[2] || value[3]) {
        rest=0;
        for (i=3; i>=0; i--) {
            value[i] += rest * 0x10000;
            rest = value[i]%10;
            value[i] /= 10;
        }
        digits[n++]=rest;
    }
    if (n==0) {
        *d = '0';
        *(d+1) = 0;
    } else {
        for (i=0; i<n; i++) {
            *(d+i) = '0' + digits[n-1-i];
        }
        *(d+n) = 0;
    }
    return 1;
}

/*****************************************************************************
 * ShowProtocol - Seek a list of protocols and writes it if found.
 */
unsigned char *protocollist[] = {"0", "Any", "1", "ICMP", "6", "TCP", "17", "UDP", NULL};
static int ShowProtocol (unsigned char **s, unsigned char *d)
{
    int i=0;
    for (i=0; protocollist[2*i]; i++) {
        if (strcmp (*s, protocollist[2*i]) == 0) {
            strcpy (d, protocollist[2*i+1]);
            return 1;
        }
    }
    strcpy (d, *s);
    return 1;
}

/*****************************************************************************
 * General routines
 */
/*****************************************************************************
 * GetItem - Gets an item from the buffer.
 */
unsigned char *GetItem (struct showitem *item, unsigned char *s)
{
    unsigned char *ch;
    if (item->config & CONFIG_PARSE)
    	return (*((getitem *)(item->end))) (&(item->value), s);
    while (isspace(*s)) s++;
    item->value=s;
    if ((ch = item->end) == NULL) {
        while (*s && !(isspace(*s))) s++;
        if (*s) *s++=0;
    } else {
        while (*s && strncmp(s, ch, strlen(ch))) s++;
        if (*s) {
            *s=0;
            s+=strlen(ch);
        }
    }
    return s;
}

/*****************************************************************************
 * WriteItem - Write an item in the table.
 */
static void WriteItem (webs_t wp, struct showitem *item)
{
    unsigned char back[48];
    back[0] = 0;
    if (item->convchar) {
        (*(item->convchar)) (&(item->value), back);
        item->value=back;
    }
    websWrite (wp, T("<t%s"), (item->config & CONFIG_HEADER)?"h class=\"st\"":"d");
    switch (item->config & CONFIG_ALIGN) {
    case CONFIG_RIGHT:
        websWrite(wp, " align=\"right\"");
        break;
    case CONFIG_LEFT:
        websWrite(wp, " align=\"left\"");
        break;
    }
    if (item->config & CONFIG_STAT) {
        websWrite(wp, " width=\"70\"");
    }
    websWrite (wp, T(">%s%s%s%s%s</t%c>"), (item->config & CONFIG_LEFT)? "<b>":"", (*item->value)?"":"&nbsp;", item->value,  (item->config & CONFIG_LEFT)?"":"</b>",(item->config & CONFIG_LEFT)?":":"", (item->config & CONFIG_HEADER)?'h':'d');
}

/*****************************************************************************
 * Finds in a buffer the n item
 * This routine interprets strings as terminated by {00,09 or 20}hex where 20h
 * must be followed by {00,09 or 20}.
*/
static void findHorizontalItemName(char *dst, char item, char *buffer)
{
	char str_ct = 0;
	char *pstr = 0;
	
	while(*buffer != 0x00) {
	    // Find the initial char of the string
	    while ((*buffer == 0x09)||(*buffer == 0x20)) {
	    	buffer++;
	    }
	    if (item == str_ct) {
	    	pstr = buffer;
	    }
	    		
		// Next token
		while ((*buffer != 0x00)&&(*buffer != 0x09)&&(*buffer != 0x20)) {
		 	buffer++;
		}
	    if (pstr) {
	    	strncpy(dst,pstr,(buffer-pstr));
	    	dst[buffer-pstr] = 0; 	// Marks the end of string
	    	return;
	    }
	    str_ct++;
	}
}

/*****************************************************************************
 * ShowInfoTable - Shows the Information Table in the page.
 */
static int ShowInfoTable (webs_t wp, struct showtable *table)
{
    FILE *arq;
    unsigned char buffer[512];
    char wordbuf[64];
    unsigned char *s, *v;
    int i, nentries=0, ncols;
    unsigned char config = table->config;
    unsigned char **inputstr = table->inputstr;
	 struct cyc_webs_header hdr;
    
    if (websGetPriority(wp) < ((config & CONFIG_CFG)?PRIV_FULL: PRIV_ADMIN)) {
        config &= ~(CONFIG_ADMIN | CONFIG_MULTIPLE);
        inputstr = NULL;
    }
    *buffer = 0;

    /* Get file size, allocate a buffer and copies the contents of the file
     * to that buffer*/
    
	websCycHdrInit(&hdr);
	hdr.need_header = 1;
	hdr.title = table->title;
	hdr.form = table->formname;
	if (table->config & CONFIG_REFRESH) {
		hdr.refresh_secs = 10;
	}
	websPageStart(wp, &hdr);
				
    if ((arq = fopen(table->filename, "r"))!= NULL) {
       	websWrite (wp, T("<center><table align=\"center\" border>"));
       	if (config & CONFIG_ONE) {
       		ncols=2;
       	} else {
	        if (config & CONFIG_VERTICAL_TABLE) {
	        	ncols=1;
                rewind(arq);
                for (i=0; i<table->nheader; i++) {
                    fgets(buffer, 512, arq);
                }
                while (fgets(buffer, 512, arq)) {
                	ncols++;
                }
	        } else {
	            if (config & (CONFIG_ADMIN | CONFIG_MULTIPLE)) {
	            	ncols=1;
	            } else {
	            	ncols=0;
	            }
   	            for (v=table->showorder; *v!=0xff; v++) ncols++;
	        }
       	}
       	websWrite (wp, T("<tr><th class=\"tt\" colspan=\"%d\">%s</th></tr>"), ncols, table->title);
        if (config & CONFIG_VERTICAL_TABLE) {
            for (v=table->showorder; *v!=0xff; v++) {
                websWrite (wp, T("<tr>"));
                rewind(arq);
                /* Discard the header or trash lines */
                for (i=0; i<table->nheader; i++) {
                    fgets(buffer, 512, arq);
                }

				s = (table->item + *v)->name;
   	            if (s == NULL) {
   	            	s = wordbuf;
   	            	findHorizontalItemName(s, *v, buffer);
   	            }
                websWrite (wp, T("<th %salign=\"left\">%s:</th>"),
                	((table->item + *v)->config & CONFIG_HEADER)?"class=\"st\" ":"", s);
                while (fgets(buffer, 512, arq)) {
                    s=buffer;
                    /* Parses the line and gets the values for each parameter */
                    for (i=0; i<table->nitems; i++) {
                        s = GetItem(table->item+i, s);
                        if (i==*v) {
                            WriteItem (wp, table->item+i);
                            break;
                        }
                    }
                    if (config & CONFIG_ONE) break;
                }
                websWrite (wp, T("</tr>"));
            }
        } else {  /// HORIZONTAL TABLE
            /* Discard the header */
            for (i=0; i<table->nheader; i++) {
                fgets(buffer, 512, arq);
            }

            websWrite (wp, T("<tr>"));
            if (config & (CONFIG_ADMIN | CONFIG_MULTIPLE)) {
            	websWrite(wp, "<th class=\"st\">Entry</th>");
            }
            for (v=table->showorder; *v!=0xff; v++) {
   	            s = (table->item + *v)->name;
   	            if (s == NULL) {
		    	if (!table->nheader) continue;
   	            	s = wordbuf;
   	            	findHorizontalItemName(s, *v, buffer);
   	            }
   	            websWrite (wp, T("<th class=\"st\" align=\"left\">%s</th>"), s);
       	    }
            websWrite (wp, T("</tr>"));

            while (fgets(buffer, 512, arq)) {
                websWrite (wp, T("<tr>"));
	            if (config & CONFIG_ADMIN) {
    	        	websWrite(wp, "<td><center><input type=\"radio\" name=\"entry\" value=\"%s\"></center></td>",
    	        		buffer);
        	    }
	            if (config & CONFIG_MULTIPLE) {
    	        	websWrite(wp, "<td><center><input type=\"checkbox\" name=\"entry%d\" value=\"%s\"></center></td>",
    	        		nentries, buffer);
        	    }
                nentries ++;
                s=buffer;
   	            /* Parses the line and gets the values for each parameter */
       	        for (i=0; i<table->nitems; i++) {
           	        s = GetItem(table->item+i, s);
               	}
   	            for (v=table->showorder; *v!=0xff; v++) {
       	            WriteItem (wp, table->item+ *v);
           	    }
               	websWrite (wp, T("</tr>"));
                if (config & CONFIG_ONE) break;
			}
            if (config & CONFIG_MULTIPLE) {
   	        	websWrite(wp, "<input type=\"hidden\" name=\"nentries\" value=\"%d\">",
   	        		nentries);
       	    }
        }
       	websWrite (wp, T("</table></center>"));
        fclose (arq);
    } else {
        websWrite (wp, T("<b class=err><center>Error: File %s does not exist.</center></b>"), table->filename);
    }
    if (inputstr && nentries) {
		websWrite (wp, "<br><center>");
		i=0;
		while (1) {
			if (*(table->inputstr+i) == NULL) break;
			websWrite (wp, "&nbsp;&nbsp;<input type=\"submit\" name=\"opt\" value=\"%s\">&nbsp;&nbsp;", *(table->inputstr+i));
			i++;
		}
		websWrite (wp, "</center>");
    }
	websPageNoNoteEnd(wp, &hdr);
    return 1;
}

/**************************************************************************
 * Definitions for DHCP client host information
 */
static struct showitem dhcpc_items[] = {
    {NULL, NULL,  NULL, "=" , CONFIG_LEFT},
    {NULL, NULL,  NULL, NULL, CONFIG_RIGHT}
};

/* Order in which the items will be shown in the table */
static unsigned char dhcpc_order[] = {0, 1, 0xff};

/* Table structure */
static struct showtable dhcpc_table = {
    "/etc/dhcpc/dhcpcd-eth0.info", "DHCPC - host information", dhcpc_order, dhcpc_items,
    sizeof(dhcpc_items)/sizeof(struct showitem), 0, 0,
    NULL, NULL
};

/* Form routine */
static void ShowDHCPCInfo(webs_t wp, char_t *path, char_t *query)
{
	FILE *arq;

	//Regina Kodato 
	if ((arq = fopen(dhcpc_table.filename, "r")) != NULL) {
		ShowInfoTable (wp, &dhcpc_table);
	} else {
		struct cyc_webs_header hdr;
		websCycHdrInit(&hdr);
		hdr.title = "DHCP Client";
		websPageMessage(wp, &hdr,  "Error: DHCP client is not active.", 
				1);
	}
}

/**************************************************************************
 * Definitions for Routing Table
 */
/* List of items */
static struct showitem routeitems[] = {
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, 0},
};

/* Order in which the items will be shown in the table */
static unsigned char routeshoworder[] = {0, 1, 2, 3, 4, 5, 6, 7, 0xff};

/* Submit options */
static unsigned char *routesubmit[] = {"Add Route", "Delete Route", NULL};

/* Table structure */
static struct showtable showroute = {
    "/tmp/webtemp", "Show Routing Table", routeshoworder, routeitems,
    sizeof(routeitems)/sizeof(struct showitem), CONFIG_ADMIN, 2,
    "adm/HandleRoute", routesubmit
};

/* Form routine */
static void ShowRouteTable(webs_t wp, char_t *path, char_t *query)
{
	system("netstat -rn>/tmp/webtemp");
    ShowInfoTable (wp, &showroute);
   	system("rm -f /tmp/webtemp");
}

/*****************************************************************************
 * Definitions for ARP Table
 */
/* List of items */
static struct showitem arpitems[] = {
    {"IP Address", NULL,        NULL, NULL, CONFIG_RIGHT},
    {"HW Type",    Hex2HWType,  NULL, NULL, CONFIG_RIGHT},
    {"Flags",      Hex2FlagARP, NULL, NULL, CONFIG_RIGHT},
    {"HW Address", NULL,        NULL, NULL, 0},
    {"Mask",       NULL,        NULL, NULL, 0},
    {"Device",     NULL,        NULL, NULL, CONFIG_RIGHT},
};

/* Order in which the items will be shown in the table */
static unsigned char arpshoworder[] = {0, 3, 1, 5, 2, 0xff};

/* Table structure */
static struct showtable showarp = {
    "/proc/net/arp", "Show ARP Cache", arpshoworder, arpitems,
    sizeof(arpitems)/sizeof(struct showitem), 0, 1,
    NULL, NULL
};

/* Form routine */
static void ShowARPTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showarp);
}

/*****************************************************************************
 * Definitions for Interface Statistics
 */
/* List of items */
static struct showitem itfitems[] = {
    {"Interface",     NULL, NULL, ":" , CONFIG_STAT|CONFIG_RIGHT|CONFIG_HEADER},
    {"Rx Bytes",      NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Packets",    NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Errors",     NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Drops",      NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Overruns",   NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Frames",     NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Compressed", NULL, NULL, NULL, CONFIG_RIGHT},
    {"Rx Multicast",  NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Bytes",      NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Packets",    NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Errors",     NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Drops",      NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Underruns",  NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Collisions", NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Carrier",    NULL, NULL, NULL, CONFIG_RIGHT},
    {"Tx Compressed", NULL, NULL, NULL, CONFIG_RIGHT}
};

/* Order in which the items will be shown in the table */
static unsigned char itfshoworder[] = {0, 1, 2, 3, 4, 5, 6, 9, 10, 11,
12, 13, 14, 15, 0xff};

/* Table structure */
static struct showtable showitf = {
    "/proc/net/dev", "Show Interface Statistics", itfshoworder, itfitems,
    sizeof(itfitems)/sizeof(struct showitem), CONFIG_VERTICAL_TABLE | CONFIG_REFRESH, 2,
    NULL, NULL
};

/* Form routine */
static void ShowItfTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showitf);
}

/*****************************************************************************
 * Definitions for IP chains table
 */
/* List of items */
static struct showitem ipchainsitems[] = {
    {"Name",      NULL,       NULL, NULL, CONFIG_LEFT},
    {"Target",    NULL,       NULL, NULL, 0},
    {"RefCount",  NULL,       NULL, NULL, 0},
    {NULL,        NULL,       NULL, NULL, 0},
    {"Packets",   AddHiValue, NULL, NULL, CONFIG_RIGHT},
    {NULL,        NULL,       NULL, NULL, 0},
    {"Bytes",     AddHiValue, NULL, NULL, CONFIG_RIGHT}
};

/* Order in which the items will be shown in the table */
static unsigned char ipchainsshoworder[] = {0, 1, 4, 6, 0xff};

/* Table structure */
static struct showtable showipchains = {
    "/proc/net/ip_fwnames", "IP Chains", ipchainsshoworder, ipchainsitems,
    sizeof(ipchainsitems)/sizeof(struct showitem), 0, 0,
    NULL, NULL
};

/* Form routine */
static void ShowIPChainsTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showipchains);
}

/*****************************************************************************
 * Definitions for the rules table
 */
/* List of items */
static struct showitem iprulesitems[] = {
    {"Chain",         NULL,           NULL, NULL, CONFIG_LEFT},
    {"Source IP",     Hex2IPAddrMask, NULL, "->", 0},
    {"Dest. IP",      Hex2IPAddrMask, NULL, NULL, 0},
    {"Interface",     NULL,           NULL, NULL, 0},
    {"FW flags",      NULL,           NULL, NULL, 0},
    {"Inv FW flags",  NULL,           NULL, NULL, 0},
    {"Protocol",      ShowProtocol,   NULL, NULL, 0},
    {NULL,            NULL,           NULL, NULL, 0},
    {"Packets",       AddHiValue,     NULL, NULL, CONFIG_RIGHT},
    {NULL,            NULL,           NULL, NULL, 0},
    {"Bytes",         AddHiValue,     NULL, NULL, CONFIG_RIGHT},
    {"Source Socket", NULL,           NULL, NULL, 0},
    {"Dest. Socket",  NULL,           NULL, NULL, 0},
    {"TOS",           NULL,           NULL, NULL, 0},
    {"AND/XOR",       NULL,           NULL, NULL, 0},
    {"Redir port",    NULL,           NULL, NULL, 0},
    {"FW mark",       NULL,           NULL, NULL, 0},
    {"Output size",   NULL,           NULL, NULL, 0},
    {"Target",        NULL,           NULL, NULL, 0}
};

/* Order in which the items will be shown in the table */
static unsigned char iprulesshoworder[] = {0, 18, 1, 11, 2, 12, 6, 3, 8, 10, 0xff};

/* Table structure */
static struct showtable showiprules = {
    "/proc/net/ip_fwchains", "IP Rules", iprulesshoworder, iprulesitems,
    sizeof(iprulesitems)/sizeof(struct showitem), 0, 0,
    NULL, NULL
};

/* Form routine */
static void ShowIPRulesTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showiprules);
}
/*****************************************************************************
 * Definitions for the IP statistics table
 */
/* List of items */
static struct showitem ipstatitems[] = {
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, CONFIG_STAT | CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT}
};

/* Order in which the items will be shown in the table */
static unsigned char ipstatshoworder[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 0xff};

/* Table structure */
static struct showtable showipstat = {
    "/proc/net/snmp", "IP Statistics", ipstatshoworder, ipstatitems,
    sizeof(ipstatitems)/sizeof(struct showitem), CONFIG_VERTICAL_TABLE | CONFIG_ONE | CONFIG_REFRESH, 1,
    NULL, NULL
};

/* Form routine */
static void ShowIPStatTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showipstat);
}

/*****************************************************************************
 * Definitions for the ICMP statistics
 */
/* List of items */
static struct showitem icmpstatitems[] = {
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, CONFIG_STAT | CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
};

/* Order in which the items will be shown in the table */
static unsigned char icmpstatshoworder[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 0xff};

/* Table structure */
static struct showtable showicmpstat = {
    "/proc/net/snmp", "ICMP Statistics", icmpstatshoworder, icmpstatitems,
    sizeof(icmpstatitems)/sizeof(struct showitem), CONFIG_VERTICAL_TABLE | CONFIG_ONE | CONFIG_REFRESH, 3,
    NULL, NULL
};

/* Form routine */
static void ShowICMPStatTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showicmpstat);
}

/*****************************************************************************
 * Definitions for the TCP statistics
 */
/* List of items */
static struct showitem tcpstatitems[] = {
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, CONFIG_STAT | CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
};

/* Order in which the items will be shown in the table */
static unsigned char tcpstatshoworder[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
12, 13, 14, 0xff};

/* Table structure */
static struct showtable showtcpstat = {
    "/proc/net/snmp", "TCP Statistics", tcpstatshoworder, tcpstatitems,
    sizeof(tcpstatitems)/sizeof(struct showitem), CONFIG_VERTICAL_TABLE | CONFIG_ONE | CONFIG_REFRESH, 5,
    NULL, NULL
};

/* Form routine */
static void ShowTCPStatTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showtcpstat);
}

/*****************************************************************************
 * Definitions for the UDP statistics
 */
/* List of items */
static struct showitem udpstatitems[] = {
    {NULL, NULL, NULL, NULL, 0},
    {NULL, NULL, NULL, NULL, CONFIG_STAT | CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL, NULL, NULL, CONFIG_RIGHT},
};

/* Order in which the items will be shown in the table */
static unsigned char udpstatshoworder[] = {1, 2, 3, 4, 0xff};

/* Table structure */
static struct showtable showudpstat = {
    "/proc/net/snmp", "UDP Statistics", udpstatshoworder, udpstatitems,
    sizeof(udpstatitems)/sizeof(struct showitem), CONFIG_VERTICAL_TABLE | CONFIG_ONE | CONFIG_REFRESH, 7,
	NULL, NULL
};

/* Form routine */
static void ShowUDPStatTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showudpstat);
}

/*****************************************************************************
 * Definitions for the System Users
 */
/* List of items */
static struct showitem sysusritems[] = {
    {"User Name",  NULL, NULL, ":",  0},
    {NULL,         NULL, NULL, ":",  0},
    {"User ID",    NULL, NULL, ":",  CONFIG_RIGHT},
    {"Group ID",   NULL, NULL, ":",  CONFIG_RIGHT},
    {"Comments",   NULL, NULL, ":",  0},
    {"Home Dir",   NULL, NULL, ":",  0},
    {"Shell Comm", NULL, NULL, "\n", 0}
};

/* Order in which the items will be shown in the table */
static unsigned char sysusrshoworder[] = {0, 2, 3, 4, 5, 6, 0xff};

static unsigned char *sysusrsubmit[] = {"Add User", "Delete User", "Change Password", NULL};
/* Table structure */
static struct showtable showsysusr = {
    "/etc/passwd", "System Users", sysusrshoworder, sysusritems,
    sizeof(sysusritems)/sizeof(struct showitem), CONFIG_ADMIN|CONFIG_CFG, 0,
    "cfg/HandleSysUsr", sysusrsubmit
};

/* Form routine */
static void ShowSysUsrTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showsysusr);
}

/*****************************************************************************
 * Definitions for the System Groups
 */
/* List of items */
static struct showitem sysgrpitems[] = {
    {"Group Name", NULL, NULL, ":",  0},
    {NULL,         NULL, NULL, ":",  0},
    {"Group ID",   NULL, NULL, ":",  CONFIG_RIGHT},
    {NULL,         NULL, NULL, "\n", 0},
};

/* Order in which the items will be shown in the table */
static unsigned char sysgrpshoworder[] = {0, 2, 0xff};

/* Submit options */
static unsigned char *sysgrpsubmit[] = {"Add Group", "Delete Group", NULL};

/* Table structure */
static struct showtable showsysgrp = {
    "/etc/group", "System Groups", sysgrpshoworder, sysgrpitems,
    sizeof(sysgrpitems)/sizeof(struct showitem), CONFIG_ADMIN|CONFIG_CFG, 0,
    "cfg/HandleSysGrp", sysgrpsubmit
};

/* Form routine */
static void ShowSysGrpTable(webs_t wp, char_t *path, char_t *query)
{
    ShowInfoTable (wp, &showsysgrp);
}

/**************************************************************************
 * Definitions for Show Process Status
 */
/* List of items */
/* This items can be renamed according to your preference */
/* For this case I prefered the internal System strings */
static struct showitem processstatusitems[] = {
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, "\r", 0}
};

/* Order in which the items will be shown in the table */
static unsigned char processstatusorder[] = {0, 1, 2, 3, 4, 5,0xff};

/* Submit options */
static unsigned char *procsubmit[] = {"Kill Processes", NULL};

/* Table structure */
static struct showtable processstatustable = {
    "/tmp/webtemp", "Process Status", processstatusorder, processstatusitems,
//    "/tmp/ps.txt", "Process Status", processstatusorder, processstatusitems,
    sizeof(processstatusitems)/sizeof(struct showitem), CONFIG_MULTIPLE, 1,
    "adm/HandleProc", procsubmit
};

/* Form routine */
static void ShowProcessStatus(webs_t wp, char_t *path, char_t *query)
{
	system("ps > /tmp/webtemp");
    ShowInfoTable (wp, &processstatustable);
   	system("rm -f /tmp/webtemp");
}

/**************************************************************************
 * Definitions for Active Sessions
 */
/* List of items */
/* This items can be renamed according to your preference */
/* For this case I prefered the internal System strings */
static struct showitem activesessionsitems[] = {
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, "\r", 0}
};

/* Order in which the items will be shown in the table */
static unsigned char activesessionsorder[] = {0, 1, 2, 3, 4, 5, 6, 7, 0xff};

/* Submit options */
static unsigned char *sesssubmit[] = {"Kill Session", NULL};

/* Table structure */
static struct showtable activesessionstable = {
    "/tmp/webtemp", "Active Sessions", activesessionsorder, activesessionsitems,
//[RK]Oct/09/02 kill session is not implemented
     sizeof(activesessionsitems)/sizeof(struct showitem), CONFIG_ADMIN, 2,
     "adm/HandleSess", sesssubmit
//    sizeof(activesessionsitems)/sizeof(struct showitem), 0, 2,
//    "adm/HandleSess", NULL
};

/* Form routine */
static void ShowActiveSessions(webs_t wp, char_t *path, char_t *query)
{
	system("w_ori > /tmp/webtemp");
    ShowInfoTable (wp, &activesessionstable);
   	system("rm -f /tmp/webtemp");
}


/**************************************************************************
 * Definitions for CAS Sessions
 */
/* List of items */
/* This items can be renamed according to your preference */
/* For this case I prefered the internal System strings */
static struct showitem cassessionsitems[] = {
    {"USER",        NULL,  NULL, NULL, 0},
    {"TTY",         NULL,  NULL, NULL, 0},
    {"FROM",        NULL,  NULL, NULL, 0},
    {"LOGIN@",      NULL,  NULL, NULL, 0},
    {"PID/Command", NULL,  NULL, "\r", 0}
};

/* Order in which the items will be shown in the table */
static unsigned char cassessionsorder[] = {0, 1, 2, 3, 4, 0xff};

/* Submit options */
static unsigned char *cassesssubmit[] = {"Kill Session", NULL};

/* Table structure */
static struct showtable cassessionstable = {
    "/tmp/webtemp", "CAS Active Sessions", cassessionsorder, cassessionsitems,
     sizeof(cassessionsitems)/sizeof(struct showitem), CONFIG_ADMIN, 3,
     "adm/HandleCASSess", cassesssubmit
//[RK]Oct/09/02 kill session is not implemented
//    sizeof(cassessionsitems)/sizeof(struct showitem), 0, 3,
//    NULL, NULL
};

/* Form routine */
static void ShowCASSessions(webs_t wp, char_t *path, char_t *query)
{
    system("w_cas > /tmp/webtemp");
    ShowInfoTable (wp, &cassessionstable);
    system("rm -f /tmp/webtemp");
}


/**************************************************************************
 * Definitions for Ram Disk Usage
 */
/* List of items */
/* This items can be renamed according to your preference */
/* For this case I prefered the internal System strings */
static struct showitem ramdiskitems[] = {
    {NULL, NULL,  NULL, NULL, 0},
    {NULL, NULL,  NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL,  NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL,  NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL,  NULL, NULL, CONFIG_RIGHT},
    {NULL, NULL,  NULL, NULL, 0}
};

/* Order in which the items will be shown in the table */
static unsigned char ramdiskorder[] = {0, 1, 2, 3, 4, 5, 0xff};

/* Table structure */
static struct showtable ramdisktable = {
    "/tmp/webtemp", "Ram Disk Usage", ramdiskorder, ramdiskitems,
    sizeof(ramdiskitems)/sizeof(struct showitem), 0, 1,
    NULL, NULL
};

/* Form routine */
static void ShowRamDiskUsage(webs_t wp, char_t *path, char_t *query)
{
	system("df > /tmp/webtemp");
    ShowInfoTable (wp, &ramdisktable);
   	system("rm -f /tmp/webtemp");
}

/**************************************************************************
 * Definitions for Ram Disk Usage
 */
static unsigned char *GetParameterFromTty (unsigned char **value,
										   unsigned char *buf,
										   unsigned char *base,
										   unsigned char flag)
{
	unsigned char *s = strstr(buf, base);
	if (s) {
		s+=strlen(base);
		*value = s;
		while (*s && !isspace(*s)) s++;
		if (*s) *s++=0;
		return s;
	} else {
		if (flag) {
			*value="0";
		} else {
			*value="";
		}
		return buf;
	}
}

static unsigned char *GetSignalFromTty (unsigned char **value, unsigned char *buf, unsigned char *base)
{
	unsigned char *s = strstr(buf, base);
	if (s) {
		*value="ON";
		return s+strlen(base);
	} else {
		*value="OFF";
		return buf;
	}
}

static unsigned char *GetBaudFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "baud:", 0);
}

static unsigned char *GetTxFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "tx:", 1);
}

static unsigned char *GetRxFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "rx:", 1);
}

static unsigned char *GetFeFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "fe:", 1);
}

static unsigned char *GetPeFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "pe:", 1);
}

static unsigned char *GetBrkFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "brk:", 1);
}

static unsigned char *GetOeFromTty (unsigned char **value, unsigned char *buf)
{
	return GetParameterFromTty (value, buf, "oe:", 1);
}

static unsigned char *GetRTSFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "RTS");
}

static unsigned char *GetCTSFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "CTS");
}

static unsigned char *GetDTRFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "DTR");
}

static unsigned char *GetDSRFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "DSR");
}

static unsigned char *GetCDFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "CD");
}

static unsigned char *GetRIFromTty (unsigned char **value, unsigned char *buf)
{
	return GetSignalFromTty (value, buf, "RI");
}

/* List of items */
static struct showitem ttyinfoitems[] = {
    {"Port",      NULL, NULL, ":",                             0},
    {"Baud Rate", NULL, NULL, (unsigned char *)GetBaudFromTty, CONFIG_PARSE | CONFIG_RIGHT},
    {"Tx bytes",  NULL, NULL, (unsigned char *)GetTxFromTty,   CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"Rx bytes",  NULL, NULL, (unsigned char *)GetRxFromTty,   CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"Frame",     NULL, NULL, (unsigned char *)GetFeFromTty,   CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"Parity",    NULL, NULL, (unsigned char *)GetPeFromTty,   CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"Breaks",    NULL, NULL, (unsigned char *)GetBrkFromTty,  CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"Overrun",   NULL, NULL, (unsigned char *)GetOeFromTty,   CONFIG_PARSE | CONFIG_STAT | CONFIG_RIGHT},
    {"RTS",       NULL, NULL, (unsigned char *)GetRTSFromTty,  CONFIG_PARSE},
    {"CTS",       NULL, NULL, (unsigned char *)GetCTSFromTty,  CONFIG_PARSE},
    {"DTR",       NULL, NULL, (unsigned char *)GetDTRFromTty,  CONFIG_PARSE},
    {"DSR",       NULL, NULL, (unsigned char *)GetDSRFromTty,  CONFIG_PARSE},
    {"CD",        NULL, NULL, (unsigned char *)GetCDFromTty,   CONFIG_PARSE},
    {"RI",        NULL, NULL, (unsigned char *)GetRIFromTty,   CONFIG_PARSE}
};

/* Order in which the items will be shown in the table */
static unsigned char ttyinfoorder[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 0xff};

/* Table structure */
static struct showtable ttyinfotable = {
    "/proc/tty/driver/serial_ttyS",
    "Serial Ports Status and Statistics", ttyinfoorder, ttyinfoitems,
    sizeof(ttyinfoitems)/sizeof(struct showitem), 0, 1,
    NULL, NULL
};

static struct showtable ttyinfotable1h = {
    "/proc/tty/driver/serial",
    "Serial Port Status and Statistics", ttyinfoorder, ttyinfoitems,
    sizeof(ttyinfoitems)/sizeof(struct showitem), 0, 1,
    NULL, NULL
};

/* Form routine */
static void ShowTtyInfo(webs_t wp, char_t *path, char_t *query)
{
    if (numsports == 1) {
        ShowInfoTable (wp, &ttyinfotable1h);
    } else {
        ShowInfoTable (wp, &ttyinfotable);
    }
}

/****************************************************************************
 *  This routine reads meminfo file and print it as a simple table.
 */

int WebGetMemInfo(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    unsigned char buffer[512];
    unsigned char *name;
    unsigned char *value;
    int i;

	if ((arq = fopen("/proc/meminfo", "r"))!= NULL) {
		fgets(buffer, 512, arq);
		fgets(buffer, 512, arq);
		fgets(buffer, 512, arq);
		
		websWrite (wp, T("<tr>"));
        websWrite (wp, T("<th colspan=\"2\" class=\"tt\" align=\"center\">  Memory Information </th>"));
        websWrite (wp, T("</tr>"));
        while (fgets(buffer, 512, arq)) {
	        value = name = buffer;
            for (i = 0; i < 30; i++) {
				if (*value == ':') break;
            	value++;
            }
            if (i == 30) break;
            *value = 0;
            value++;
			websWrite (wp, T("<tr>"));
            websWrite (wp, T("<td><b>%s:</b></td><td>%s</td>"), name, value);
			websWrite (wp, T("</tr>"));
		}	
		fclose(arq);
    }
    return 0;
}

/****************************************************************************
 *  This routine reads cpuinfo file and print it as a simple table.
 */

int WebGetCpuInfo(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    unsigned char buffer[512];
    unsigned char *name, *t;
    unsigned char *value;
    int i, j=0;

	if ((arq = fopen("/proc/cpuinfo", "r"))!= NULL) {
		websWrite (wp, T("<tr>"));
        websWrite (wp, T("<th colspan=\"2\" class=\"tt\" align=\"center\">  CPU Information </th>"));
        websWrite (wp, T("</tr>"));
        while (fgets(buffer, 512, arq)) {
        	if (++j==3) continue; /* Filter one of the "clock" fields */
	        value = name = buffer;
            for (i = 0; i < 30; i++) {
				if (*value == ':') break;
            	value++;
            }
            if (i == 30) break;
            *value = 0;
            value++;
			 // Remove tab at the end of name
            t = name;
            while(*name && *name != '\t')
            {
              name++;
            }
            if (*name) *name++=0;
            //Capitalize the first char of name
          *t &= ~(*t & 0x20);

			websWrite (wp, T("<tr>"));
         websWrite (wp, T("<td><b>%s:</b></td><td>%s</td>"), t, value);
			websWrite (wp, T("</tr>"));
		}	
		fclose(arq);
    }
    return 0;
}


/*****************************************************************************
 *  * This routine get the date in a simple string to web page.
 *  In the form "Tue Jun  5 18:09:01"
*/
int WebGetDate(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    unsigned char buffer[512];
	
	system("date > /tmp/webtemp");
	if ((arq = fopen("/tmp/webtemp", "r"))!= NULL) {
		fgets(buffer, 512, arq);
		buffer[19] = 0;
		websWrite (wp, T("%s"), buffer);
		fclose(arq);
	}
	system("rm -f /tmp/webtemp");
	return 0;
}

/*****************************************************************************
 *  * This routine get the Cyclades version in a simple string to web page.
*/
int WebGetVersion(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    unsigned char buffer[512], *t, *s;
	int i;

	if ((arq = fopen("/proc/version", "r"))!= NULL) {	// "/proc/version"
		fgets(buffer, 512, arq);	// Discard first line.
		/* Get Linux Version */
		t=buffer;
		s=t+1;
		while (*s && *s!='(') s++;
		*(s-1) = 0;
		websWrite (wp, "%s<br>", t);
		/* Get the session where the kernel was generated */
		t=s;
		s++;
		while (*s && *s!='(') s++;
		*(s-1) = 0;
		websWrite (wp, "%s<br>", t);
		/* Get the compiler version */
		t=s;
		s++;
		while (*s && *s!='#') s++;
		while (*s && !isspace(*s)) s++;
		if (*s) *s++=0;
		websWrite (wp, "%s<br>", t);
		/* Get the date of generation*/
		websWrite (wp, "%s<br>", s);
		for (i = 0; i < 512; i++) {
			buffer[i] = 0;
		}
		fgets(buffer, 512, arq);
		buffer[50] = 0;
		websWrite (wp, buffer);
		fclose(arq);
	}
	return 0;
}

/*****************************************************************************
 *  *  This routine get the how long time the system is up in a simple form.
*/

int WebGetUpTime(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    unsigned char buffer[512];
	int i;
	unsigned char *pstr;
	
	pstr = 0;
	system("uptime > /tmp/webtemp");
	if ((arq = fopen("/tmp/webtemp", "r"))!= NULL) {
		fgets(buffer, 512, arq);
		for (i = 0; i < 512; i++) {
			if ((buffer[i] == 'u') && (buffer[i+1] == 'p')) {
				pstr = &buffer[i+2];
				break;
			}
		}
		for (; i < 512; i++) {
			if (buffer[i] == ',') {
				buffer[i] = '\0';
				break;
			}
		}
		if (pstr) {
			websWrite (wp, T("%s"), pstr);
		}
		fclose(arq);
	}
	system("rm -f /tmp/webtemp");
	return 0;
}

/*============================================================================
 * DefineWebAspform3 - This is the only routine which is not static. It's
 * called in main.c.
 *============================================================================
 */
void DefineWebAspForm3() {
    websFormDefine(T("ShowRouteTable"),    ShowRouteTable);
    websFormDefine(T("ShowARPTable"),      ShowARPTable);
    websFormDefine(T("ShowItfTable"),      ShowItfTable);
    websFormDefine(T("ShowDHCPCInfo"),      ShowDHCPCInfo);
    websFormDefine(T("ShowIPChainsTable"), ShowIPChainsTable);
    websFormDefine(T("ShowIPRulesTable"),  ShowIPRulesTable);
    websFormDefine(T("ShowIPStatTable"),   ShowIPStatTable);
    websFormDefine(T("ShowICMPStatTable"), ShowICMPStatTable);
    websFormDefine(T("ShowTCPStatTable"),  ShowTCPStatTable);
    websFormDefine(T("ShowUDPStatTable"),  ShowUDPStatTable);
    websFormDefine(T("ShowRamDiskUsage"),  ShowRamDiskUsage);
    websFormDefine(T("ShowActiveSessions"),ShowActiveSessions);
    websFormDefine(T("ShowCASSessions"),   ShowCASSessions);
    websFormDefine(T("ShowProcessStatus"), ShowProcessStatus);
    websFormDefine(T("ShowSysUsrTable"),   ShowSysUsrTable);
    websFormDefine(T("ShowSysGrpTable"),   ShowSysGrpTable);
    websFormDefine(T("ShowTtyInfo"),       ShowTtyInfo);

	websAspDefine(T("WebGetDate"),    WebGetDate);
	websAspDefine(T("WebGetUpTime"),  WebGetUpTime);
	websAspDefine(T("WebGetVersion"), WebGetVersion);
	websAspDefine(T("WebGetCpuInfo"), WebGetCpuInfo);	
	websAspDefine(T("WebGetMemInfo"), WebGetMemInfo);
}

