/*
 * appl4.c -- Routines for the web pages (LINUX version)
 *
 */

/******************************** Description *********************************/

/*
 *	Routines for the web pages (asp/form/cgi)
 *	This module manages the web pages for administration tools.
 */

/********************************* Includes ***********************************/

#include	"appl.h"
#include	<signal.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<sys/ioctl.h>
#include	<asm/flash.h>
#include	<asm/cyc_config.h>
#include	<net/if.h>

static int     speeda[] = {4800, 9600, 19200, 38400, 57600, 115200};
static unsigned char def_hwmac[6] = {0x00, 0x60, 0x2e, 0x00, 0x00, 0x01};
static unsigned char def_ethip[4] = {192, 162, 160,10};
static unsigned char def_srvip[4] = {192, 162, 160,1};

static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
	int j;

	while (byte_n--) {
		crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
		for(j = 0; j < 8; j++) {
			if (crc & 0x8000)  {
				crc <<= 1;
				crc ^= 0x1021;
			} else {
				crc <<= 1;
			}
		}
	}
	return(crc);
}

static int set_default_config(struct HW_SETUP *hw_setup, struct BOOT_MASK *boot_mask)
{
	unsigned char macaddr[6];
	
	if (memcmp(def_hwmac, hw_setup->physical_addr, 3)) {
		memcpy(macaddr, def_hwmac, 6);
	} else {
		memcpy(macaddr, hw_setup->physical_addr, 6);
	}
	
	memset(hw_setup, 0, sizeof(struct HW_SETUP));
	memcpy(hw_setup->physical_addr, macaddr, 6);
	*(unsigned long *)hw_setup->itf_ip_addr = *(unsigned long *)def_ethip;
#ifdef OEM1
    hw_setup->enable_wdt = 0;
#else
    hw_setup->enable_wdt = 1;
#endif
	hw_setup->console_speed = 1;
	hw_setup->op_code_crc = 1;
	hw_setup->mem_test = 2;
	hw_setup->boot_option = 0;
	strcpy(hw_setup->tftp_path, "zvmppcts.bin");
	
	memset(boot_mask, 0, sizeof(struct BOOT_MASK));
	boot_mask->boot_type = 2;
	*(unsigned long *)boot_mask->bootserver = *(unsigned long *)def_srvip;
}
	

static char *access_type(char *buf, int type)
{
	FLASH_CONFIG_VECTOR_HEADER	*p = (FLASH_CONFIG_VECTOR_HEADER *)buf;

	while (p->type != 9999) {
		FLASH_CONFIG_VECTOR_HEADER	*p = (FLASH_CONFIG_VECTOR_HEADER *)buf;
		if (p->type == 9999) {
			break;
		}
		if (p->crc != calc_crc(p->size, 0, p->data)) {
			break;
		}
		if (p->type == type) {
			return(p->data);
		}
		buf += p->size + 6;
	}
	return(NULL);
}


static int GetBootConfiguration(struct HW_SETUP *hwsetup, struct BOOT_MASK *boot_mask)
{
	unsigned char *buffer, *s1, *s2, *s3;
	int fd, ret=0;
	
	memset((unsigned char *)hwsetup, 0, sizeof(struct HW_SETUP));
	memset((unsigned char *)boot_mask, 0, sizeof(struct BOOT_MASK));
	if ((buffer = malloc(64*1024)) != NULL) {
		if ((fd = open("/proc/flash/config", O_RDONLY|O_CREAT)) != -1) {
			if (read(fd, buffer, 64*1024)>0) {
				if ((s1 = access_type(buffer, TYPE0000)) != NULL &&
					(s2 = access_type(buffer, TYPE0068)) != NULL &&
					(s3 = access_type(buffer, TYPE0015)) != NULL) {
					
					memcpy((unsigned char *)hwsetup, s2, sizeof(struct HW_SETUP));
					memcpy((unsigned char *)boot_mask, s3, sizeof(struct BOOT_MASK));
					ret = 1;
				}
			}
			close(fd);
		}
		free(buffer);
	}
	return ret;
}

static int SetBootConfiguration(struct HW_SETUP *hw_setup, struct BOOT_MASK *boot_mask)
{
	unsigned char *buffer, *s;
	FLASH_CONFIG_VECTOR_HEADER	*v;
	int ret = 0, fd;
	if ((buffer = malloc(64*1024)) != NULL) {
		s = buffer;
		
		// TYPE0000
		v = (FLASH_CONFIG_VECTOR_HEADER	*)s;
		v->type = TYPE0000;
		v->size = sizeof(struct type0000);
		memset(v->data, 0, v->size);
		strcpy(v->data, "CYCLADES");
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;

		// TYPE0068
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = TYPE0068;
		v->size = sizeof(struct HW_SETUP);
		memcpy(v->data, (unsigned char *)hw_setup, v->size);
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;

		// TYPE0015
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = TYPE0015;
		v->size = sizeof(struct BOOT_MASK);
		memcpy(v->data, (unsigned char *)boot_mask, v->size);
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;
		
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = 9999;
		v->size = 0;
		v->crc = 0;
		s += 6 + v->size;
		
		if ((fd = open("/proc/flash/config", O_WRONLY|O_CREAT)) != -1) {
			if (write(fd, buffer, s-buffer)==s-buffer) {
				ret = 1;
			}
			close(fd);
		}
		free(buffer);
	}
	return ret;
}

static void check_speed_console(int console_speed)
{
	FILE *arq;
	int file_speed;

	if ((arq = fopen("/etc/getty_ttyS0", "rw")) != NULL) {
		fscanf(arq, "%d", &file_speed);
	}
	
	if (speeda[console_speed] != file_speed) {
		rewind(arq);
		fprintf(arq, "%d", speeda[console_speed]);
//[RK]		system("sh /bin/updatefiles /etc/getty_ttyS0 > /dev/ttyS0");
	}
	fclose(arq);
}

//[RK] delete this routine
//static int check_ethernet_operation(struct HW_SETUP *hw_setup)
//{
//	struct ifreq ifr;
//	int sockfd, ret=0;
//	unsigned char aux[128];
//	if (!strcmp(mt.name, "CPLD")) {
//		if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) >= 0) {
//			strcpy(ifr.ifr_name, "eth0");
//			ifr.ifr_ifru.ifru_flags = 1;
//			ifr.ifr_ifru.ifru_ivalue = hw_setup->mii_operation;
 //       	if (ioctl(sockfd, SIOCDEVPRIVATE, &ifr) >= 0) {
  //      		ret = 1;
//			}
//		}
//		close(sockfd);
//	}
//	if (ret) {
//		sprintf(aux, "config_eth0 ip %s mac %02x:%02x:%02x:%02x:%02x:%02x",
//			inet_ntoa (*(struct in_addr *)(hw_setup->itf_ip_addr)),
//			hw_setup->physical_addr[0],	hw_setup->physical_addr[1],
//			hw_setup->physical_addr[2],	hw_setup->physical_addr[3],
//			hw_setup->physical_addr[4],	hw_setup->physical_addr[5]);
//		system(aux);
//	}
//	return ret;
//}

static void ShowBootPage(webs_t wp, struct HW_SETUP *hw_setup, struct BOOT_MASK *boot_mask, int flash_ok, int readerr)
{
	int i;
	unsigned char value[32];
	struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
	hdr.need_header = 1;
	hdr.title = "Boot Configuration";
	hdr.form = "cfg/SetBootConf";
	websPageStart(wp, &hdr);
	
	if (!flash_ok) {
		websWrite(wp,"<center><b class=\"err\">%s<br></b><center><br>",
			"The configuration vector is empty. Loading Factory Defaults.");
	}
	if (readerr) {
		websWrite(wp,"<center><b class=\"err\">%s<br></b><center><br>",
			"Configuration Error. The wrong parameters are marked with red.");
	}
	websWrite(wp, "<center><table border align=\"center\"><tr><th class=\"tt\" colspan=\"2\">%s</th></tr>",
		"Boot Configuration");
	sprintf(value, "%02x:%02x:%02x:%02x:%02x:%02x",
		hw_setup->physical_addr[0], hw_setup->physical_addr[1],
		hw_setup->physical_addr[2], hw_setup->physical_addr[3],
		hw_setup->physical_addr[4], hw_setup->physical_addr[5]);
//	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
//		(readerr & 1)?" class=\"err\"":"", "MAC Address assigned to Ethernet");
//	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
//		"text", "ethmac", value, " class=\"tx\"");
//	websWrite(wp, "</td></tr>");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"hidden", "ethmac", value, " class=\"tx\"");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 2)?" class=\"err\"":"", "IP Address assigned to Ethernet:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"text", "ethip", inet_ntoa(*(struct in_addr *)hw_setup->itf_ip_addr), " class=\"tx\"");
	websWrite(wp, "</td></tr>");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 4)?" class=\"err\"":"", "Watchdog Timer:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "wdt", "1", (hw_setup->enable_wdt == 1)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "active");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "wdt", "0", (hw_setup->enable_wdt == 0)?" checked":"");
	websWrite(wp, "&nbsp;%s</td></tr>", "inactive");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 8)?" class=\"err\"":"", "Unit boot from:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "boot", "0", (hw_setup->boot_option == 0)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "flash");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "boot", "1", (hw_setup->boot_option == 1)?" checked":"");
	websWrite(wp, "&nbsp;%s</td></tr>", "network");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 16)?" class=\"err\"":"", "Boot Type:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "btype", "1", (boot_mask->boot_type == 1)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "bootp");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "btype", "2", (boot_mask->boot_type == 2)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "tftp");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "btype", "3", (boot_mask->boot_type == 3)?" checked":"");
	websWrite(wp, "&nbsp;%s</td></tr>", "both");
	
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 32)?" class=\"err\"":"", "Boot File Name:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"text", "bfile", hw_setup->tftp_path, " size=\"32\" class=\"tx\"");
	websWrite(wp, "</td></tr>");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 64)?" class=\"err\"":"", "Server's IP Address:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"text", "srvip", inet_ntoa(*(struct in_addr *) boot_mask->bootserver), " class=\"tx\"");
	websWrite(wp, "</td></tr>");
			
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 128)?" class=\"err\"":"", "Console Speed:");
	websWrite(wp, "<select name=\"%s\">", "consspd");
	for (i=0; i<6; i++) {
		websWrite(wp, "<option value=\"%d\"%s>%d", i, (hw_setup->console_speed == i)?" selected":"", speeda[i]);
	}
	websWrite(wp, "</select></td></tr>");
		
	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 256)?" class=\"err\"":"", "Flash Test:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "ftest", "0", (hw_setup->op_code_crc == 0)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "skip");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "ftest", "1", (hw_setup->op_code_crc == 1)?" checked":"");
	websWrite(wp, "&nbsp;%s</td></tr>", "full");

	websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
		(readerr & 512)?" class=\"err\"":"", "RAM Test:");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "rtest", "0", (hw_setup->mem_test == 0)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "skip");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "rtest", "1", (hw_setup->mem_test == 1)?" checked":"");
	websWrite(wp, "&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;", "quick");
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"radio", "rtest", "2", (hw_setup->mem_test == 2)?" checked":"");
	websWrite(wp, "&nbsp;%s</td></tr>", "full");
				
	get_machine_type(&mt);
	if (!strcmp(mt.name, "CPLD")) {
		websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
			(readerr & 1024)?" class=\"err\"":"", "Fast Ethernet:");
		websWrite(wp, "<select name=\"%s\">", "feth");
		websWrite(wp, "<option value=\"%d\"%s>%s", 0, (hw_setup->mii_operation == 0)?" selected":"", "Auto Negotiation");
		websWrite(wp, "<option value=\"%d\"%s>%s", 1, (hw_setup->mii_operation == 1)?" selected":"", "100BaseT Half-Duplex");
		websWrite(wp, "<option value=\"%d\"%s>%s", 3, (hw_setup->mii_operation == 3)?" selected":"", "100BaseT Full-Duplex");
		websWrite(wp, "<option value=\"%d\"%s>%s", 4, (hw_setup->mii_operation == 4)?" selected":"", "10BaseT Half-Duplex");
		websWrite(wp, "<option value=\"%d\"%s>%s", 2, (hw_setup->mii_operation == 2)?" selected":"", "10BaseT Full-Duplex");
		websWrite(wp, "</select></td></tr>");
		//[RK] Fast Ethernet Maximum Interrupt Events
		websWrite(wp, "<tr><td><b%s>%s</b></td><td>",
			(readerr & 64)?" class=\"err\"":"", "Fast Ethernet Maximum Interrupt Events:");
		websWrite(wp, "<input class=\"tx\" type=\"text\" name=\"%s\" value=\"%d\">",
			 "fe_inte", hw_setup->max_int_work);
		websWrite(wp, "</td></tr>");
	}
	
	websWrite(wp, "</table></center><br>");
	if (websGetPriority(wp) == PRIV_FULL) {
		websWrite(wp, "<center>");
		websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
			"submit", "opt", "Save to Flash", "");
		websWrite(wp, "&nbsp;&nbsp;&nbsp;&nbsp;"
			"&nbsp;&nbsp;&nbsp;&nbsp;");
		websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
			"submit", "opt", "Load Factory Defaults", "");
		websWrite(wp, "&nbsp;&nbsp;&nbsp;&nbsp;"
			"&nbsp;&nbsp;&nbsp;&nbsp;");
		websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
			"submit", "opt", "Cancel", "");
		websWrite(wp, "&nbsp;&nbsp;&nbsp;&nbsp;"
			"&nbsp;&nbsp;&nbsp;&nbsp;");
		websWrite(wp, "<input type=\"%s\" value=\"%s\">",
			"reset",  "Reset", "");
		websWrite(wp, "</center>");
	}
	websPageNoNoteEnd(wp, &hdr);
}

static void ShowBootConf(webs_t wp, char_t *path, char_t *query)
{
	struct HW_SETUP hw_setup;
	struct BOOT_MASK boot_mask;
	int flash_ok = GetBootConfiguration(&hw_setup, &boot_mask);
	if (!flash_ok) {
		set_default_config(&hw_setup, &boot_mask);
	}
	ShowBootPage(wp, &hw_setup, &boot_mask, flash_ok, 0);
}

static void SetBootConf(webs_t wp, char_t *path, char_t *query)
{
	struct HW_SETUP hw_setup;
	struct BOOT_MASK boot_mask;
	unsigned char *value;
	unsigned char error = 0, *message;
	int readerr, writeerr, x[6], i;
	unsigned int aux;
//[RK]	unsigned char eth_chg=0;
	struct cyc_webs_header hdr;
	
	value = websGetVar(wp, "opt", "");
	if (strcmp(value, "Load Factory Defaults") == 0) {
		GetBootConfiguration(&hw_setup, &boot_mask);
		set_default_config(&hw_setup, &boot_mask);
		ShowBootPage(wp, &hw_setup, &boot_mask, 1, 0);
		return;
	}
	if (strcmp(value, "Save to Flash") == 0) {
		GetBootConfiguration(&hw_setup, &boot_mask);
		readerr = (sscanf(websGetVar(wp, "ethmac", ""), "%x:%x:%x:%x:%x:%x",
			&x[0], &x[1], &x[2], &x[3], &x[4], &x[5]))? 0:1;
		for (i=0; i<6; i++) hw_setup.physical_addr[i] = x[i];
		readerr += (inet_aton(websGetVar(wp, "ethip", ""),
			(struct in_addr *)(hw_setup.itf_ip_addr)))? 0:2;
		readerr += (sscanf(websGetVar(wp, "wdt", ""), "%d", &aux)>0 &&
			aux <= 1) ? 0:4;
		hw_setup.enable_wdt = aux;
		readerr += (sscanf(websGetVar(wp, "boot", ""), "%d", &aux)>0 &&
			aux <=1) ? 0:8;
		hw_setup.boot_option = aux;
		readerr += (sscanf(websGetVar(wp, "btype", ""), "%d", &aux)>0 &&
			(aux>=1 && aux<=3)) ? 0:16;
		boot_mask.boot_type = aux;
		readerr += ((value = websGetVar(wp, "bfile", NULL)) == NULL)? 32:0;
		if (value) {
			strncpy(hw_setup.tftp_path, value, PATH_LEN-1);
			hw_setup.tftp_path[PATH_LEN-1] = 0;
		}
		readerr += (inet_aton(websGetVar(wp, "srvip", ""),
			(struct in_addr *)(boot_mask.bootserver)))? 0:64;
		readerr += (sscanf(websGetVar(wp, "consspd", ""), "%d", &aux)>0 &&
			aux<=5) ? 0:128;
		hw_setup.console_speed = aux;
		readerr += (sscanf(websGetVar(wp, "ftest", ""), "%d", &aux)>0 &&
			aux<=1) ? 0:256;
		hw_setup.op_code_crc = aux;
		readerr += (sscanf(websGetVar(wp, "rtest", ""), "%d", &aux)>0 &&
			aux<=2) ? 0:512;
		hw_setup.mem_test = aux;
		get_machine_type(&mt);
		if (!strcmp(mt.name, "CPLD")) {
			readerr += (sscanf(websGetVar(wp, "feth", ""), "%d", &aux)>0 &&
				aux<=4) ? 0:1024;
			hw_setup.mii_operation = aux;
			//[RK] FETH Maximum Interrupt Events
			readerr += (sscanf(websGetVar(wp, "fe_inte", ""), "%d", &aux)>0)?0:1024;
			hw_setup.max_int_work = aux;
		}
		if (readerr) {
			ShowBootPage(wp, &hw_setup, &boot_mask, 0, readerr);
			return;
		} else {
			/* Save configuration in flash */
			writeerr = (SetBootConfiguration(&hw_setup, &boot_mask));
			/* Check Console Speed in /etc/getty_ttyS0 */
			check_speed_console(hw_setup.console_speed);
//[RK]			eth_chg = 1;
//[RK]			/* Check Ethernet */
//[RK]			check_ethernet_operation(&hw_setup);
		}
	}
	
	if (strcmp(value, "Cancel") == 0) {
		message = "Boot configuration was cancelled.";
	} else {
		if (writeerr == 0) {
			message = "ERROR - Configuration could not be written in the flash.";
			error = 1;
		} else {
			message = "Configuration was written successfully.<br>This configuration will be effective after reboot.";
		}
	}
	websCycHdrInit(&hdr);
	hdr.title = "Boot Configuration Result";
	hdr.father = "/goform/ShowBootConf";
	hdr.links = "&nbsp;<a href=\"/goform/ShowBootConf\">"
		"[Boot Configuration]</a>";
	websPageMessage(wp, &hdr, message, error);
}

/*============================================================================
 * DefineWebAspform3 - This is the only routine which is not static. It's
 * called in main.c.
 *============================================================================
 */
void DefineWebAspForm5() {
    websFormDefine(T("cfg/SetBootConf"), SetBootConf);
    websFormDefine(T("ShowBootConf"),    ShowBootConf);
}
