/*
 * umui.c -- User Management GoForm Processing
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 *
 *	$Id: umui.c,v 1.16 2004/05/21 03:42:08 edson Exp $
 */

/******************************** Description *********************************/

/*
 *	This module provides GoForm functions for User management
 */

/********************************* Includes ***********************************/

#include	"LINUX/appl.h"

/********************************* Defines ************************************/

#define		NONE_OPTION		T("<NONE>")

/**************************** Forward Declarations ****************************/
static int 		aspPrintSubmit(int eid, webs_t wp, 
				int argc, char_t **argv);

static void		formAddUser(webs_t wp, char_t *path, char_t *query);
static void		formHandleUser(webs_t wp, char_t *path, char_t *query);
static int		aspGenerateUserList(int eid, webs_t wp, 
									int argc, char_t **argv);

static void		formAddGroup(webs_t wp, char_t *path, char_t *query);
static void		formChgPasswd(webs_t wp, char_t *path, char_t *query);
static void		formHandleGroup(webs_t wp, char_t *path, char_t *query);
static int		aspGenerateGroupList(int eid, webs_t wp, 
									 int argc, char_t **argv);

// Jul/25/2001 Cyclades TSxK Livio: List all groups as checkbox (complete info).
static int		aspGenerateGroupFullList(int eid, webs_t wp, 
				int argc, char_t **argv);

static void		formAddAccessLimit(webs_t wp, char_t *path, 
				char_t *query);
static void		formHandleAccessLimit(webs_t wp, char_t *path, 
				char_t *query);
static int		aspGenerateAccessLimitList(int eid, webs_t wp, 
										   int argc, char_t **argv);

static int		aspGenerateAccessMethodList(int eid, webs_t wp, 
											int argc, char_t **argv);
static int		aspGeneratePrivilegeList(int eid, webs_t wp, 
										 int argc, char_t **argv);

static void		formFileUserManagement(webs_t wp, char_t *path, 
				char_t *query);

/*********************************** Code *************************************/
/*
 *	Set up the User Management form handlers
 */

void formDefineUserMgmt(void)
{
	websAspDefine(T("MakeGroupList"), aspGenerateGroupList);
	websAspDefine(T("MakeGroupFullList"), aspGenerateGroupFullList);
	websAspDefine(T("MakeUserList"), aspGenerateUserList);
	websAspDefine(T("MakeAccessLimitList"), aspGenerateAccessLimitList);
	websAspDefine(T("MakeAccessMethodList"), aspGenerateAccessMethodList);
	websAspDefine(T("MakePrivilegeList"), aspGeneratePrivilegeList);
	websAspDefine(T("PrintSubmit"), aspPrintSubmit);

	websFormDefine(T("um/AddUser"), formAddUser);
	websFormDefine(T("um/ChgPasswd"), formChgPasswd);
	websFormDefine(T("um/HandleUser"), formHandleUser);
	websFormDefine(T("um/AddGroup"), formAddGroup);
	websFormDefine(T("um/HandleGroup"), formHandleGroup);
	websFormDefine(T("um/AddAccessLimit"), formAddAccessLimit);
	websFormDefine(T("um/HandleAccessLimit"), formHandleAccessLimit);

	websFormDefine(T("um/FileUserManagement"), formFileUserManagement);
}

/******************************************************************************/
/*
 *  Add a user
 */

static void formAddUser(webs_t wp, char_t *path, char_t *query)
{
	char_t	*userid, *pass1, *pass2, *group, *enabled, *ok;
	bool_t bDisable;
	int	nCheck;
	char_t message[128], error=1;
	struct cyc_webs_header hdr;

	a_assert(wp);

	userid = websGetVar(wp, T("user"), T("")); 
	pass1 = websGetVar(wp, T("password"), T("")); 
	pass2 = websGetVar(wp, T("passconf"), T("")); 
	group = websGetVar(wp, T("group"), T("")); 
	enabled = websGetVar(wp, T("enabled"), T("")); 
	ok = websGetVar(wp, T("ok"), T("")); 

	if (*ok != 'S') {
		strcpy(message, "Add User Cancelled");
        } else if (*userid == 0) {
                strcpy(message, "ERROR: Invalid Username.");
        } else if (*pass1 == 0) {
                strcpy(message, "ERROR: Invalid Password.");
	} else if (gstrcmp(pass1, pass2) != 0) {
		strcpy(message, "ERROR: Confirmation Password did not match.");
	} else {
		if (enabled && *enabled && (gstrcmp(enabled, T("on")) == 0)) {
			bDisable = FALSE;
		} else {
			bDisable = TRUE;
		}

		nCheck = umAddUser(userid, pass1, group, 0, bDisable);
		if (nCheck != 0) {
			char_t * strError;

			switch (nCheck) {
			case UM_ERR_DUPLICATE:
				strError = T("User already exists.");
				break;

			case UM_ERR_BAD_NAME:
				strError = T("Invalid user name.");
				break;

			case UM_ERR_BAD_PASSWORD:
				strError = T("Invalid password.");
				break;

			case UM_ERR_NOT_FOUND:
				strError = T("Invalid or unselected group.");
				break;

			default:
				strError = T("Error writing user record.");
				break;
			}

			sprintf(message, "ERROR: Unable to add user \"%s\".  %s", userid, strError);
		} else {
			sprintf(message, "User \"%s\" was successfully added.", userid);
			error = 0;
		}
	}
	websCycHdrInit(&hdr);
	hdr.title = "Web User Management - Add a User";
	hdr.father = "/read/webuser.asp";
	hdr.links = "&nbsp;<a href=\"/read/webuser.asp\">[User List]</a>";
	websPageMessage(wp, &hdr, message, error);
}

/******************************************************************************/
/*
 *  Change Password for a User
 */
static void formChgPasswd(webs_t wp, char_t *path, char_t *query)
{
	char_t	*userid, *pass1, *pass2, *opt;
	char_t message[128], error=1;
	struct cyc_webs_header hdr;

	a_assert(wp);

	userid = websGetVar(wp, T("user"), T("")); 
	pass1 = websGetVar(wp, T("password"), T(""));
	pass2 = websGetVar(wp, T("passconf"), T(""));
	opt = websGetVar(wp, T("opt"), T(""));

	if (*opt != 'S') {
		strcpy(message, "ERROR: Change Password cancelled");
        } else if (*userid == 0) {
                strcpy(message, "ERROR: Invalid Username.");
        } else if (*pass1 == 0) {
                strcpy(message, "ERROR: Invalid Password.");
	} else if (gstrcmp(pass1, pass2) != 0) {
		strcpy(message, "ERROR: Confirmation Password did not match.");
	} else {
		if (umUserExists(userid) == FALSE) {
			sprintf(message, "ERROR: User \"%s\" not found", userid);
		} else if (umSetUserPassword(userid, pass1) != 0) {
			sprintf(message, "ERROR: Unable to change password for user \"%s\".", userid);
		} else {
			sprintf(message, "User \"%s\" password was successfully changed.", userid);
			error = 0;
		}
	}
	websCycHdrInit(&hdr);
	hdr.title = "Web User Management - Change Password";
	hdr.father = "/read/webuser.asp";
	hdr.links = "&nbsp;<a href=\"/read/webuser.asp\">[User List]</a>";
	websPageMessage(wp, &hdr, message, error);
}

/******************************************************************************/
/*
 *  Add a group
 */

static void formAddGroup(webs_t wp, char_t *path, char_t *query)
{
	char_t			*group, *enabled, *privilege, *method, *ok, *pChar;
	int				nCheck;
	short			priv;
	accessMeth_t	am;
	bool_t			bDisable;
	char_t message[128], error=1;
	struct cyc_webs_header hdr;

	a_assert(wp);

	group = websGetVar(wp, T("group"), T("")); 
	method = websGetVar(wp, T("method"), T("")); 
	enabled = websGetVar(wp, T("enabled"), T("")); 
	privilege = websGetVar(wp, T("privilege"), T("")); 
	ok = websGetVar(wp, T("ok"), T("")); 

	if (*ok != 'S') {
		strcpy(message, "ERROR: Add Group Cancelled.");
	} else if ((group == NULL) || (*group == 0)) {
		strcpy(message, "ERROR: No Group Name was entered.");
	} else if (umGroupExists(group)) {
		sprintf(message, "ERROR: Group \"%s\" already exists.", group);
	} else {
		if (privilege && *privilege) {
/*
 *			privilege is a mulitple <SELECT> var, and must be parsed.
 *			Values for these variables are space delimited.
 */
//			priv = 0;
//			for (pChar = privilege; *pChar; pChar++) {
//				if (*pChar == ' ') {
//					*pChar = '\0';
//					priv |= gatoi(privilege);
//					*pChar = ' ';
//					privilege = pChar + 1;
//				}
//			}
//			priv |= gatoi(privilege);
			priv = gatoi(privilege);
		} else {
			priv = 0;
		}

		if (method && *method) {
			am = (accessMeth_t) gatoi(method);
		} else {
			am = AM_FULL;
		}

		if (enabled && *enabled && (gstrcmp(enabled, T("on")) == 0)) {
			bDisable = FALSE;
		} else {
			bDisable = TRUE;
		}

		nCheck = umAddGroup(group, priv, am, 0, bDisable);
		if (nCheck != 0) {
			sprintf(message, "ERROR: Unable to add group \"%s\" - code: %d ",
				group, nCheck);
		} else {
			sprintf(message, "Group \"%s\" was successfully added.", group);
			error = 0;
		}
	}
	websCycHdrInit(&hdr);
	hdr.title = "Web User Management - Add a User Group";
	hdr.father = "/read/webgroup.asp";
	hdr.links = "&nbsp;<a href=\"/read/webgroup.asp\">[Group List]</a>";
	websPageMessage(wp, &hdr, message, error);
}

/******************************************************************************/
/*
 *  Add an access limit
 */

static void formAddAccessLimit(webs_t wp, char_t *path, char_t *query)
{
	char_t			*url, *method, *secure, *ok;
	char_t message[128], error=1;

// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifdef CYCLADES_TSxK
	char_t	*priv;
	short	nPriv;
#else 
	char_t	*group;
#endif
	
	int				nCheck;
	accessMeth_t	am;
	short			nSecure;
	struct cyc_webs_header hdr;

	a_assert(wp);

	url = websGetVar(wp, T("url"), T(""));

// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifdef CYCLADES_TSxK
	priv = websGetVar(wp, T("privilege"), T(""));
#else
	group = websGetVar(wp, T("group"), T(""));
#endif

	method = websGetVar(wp, T("method"), T(""));
	secure = websGetVar(wp, T("secure"), T(""));
	ok = websGetVar(wp, T("ok"), T(""));


	if (*ok != 'S') {
		strcpy(message, "ERROR: Add Access Limit Cancelled.");
	} else if ((url == NULL) || (*url == 0)) {
		strcpy(message, "ERROR: No URL was entered.");
	} else if (umAccessLimitExists(url)) {
		sprintf(message, "ERROR: An Access Limit for [%s] already exists.", url);
	} else {
		if (method && *method) {
			am = (accessMeth_t) gatoi(method);
		} else {
			am = AM_FULL;
		}

		if (secure && *secure) {
			nSecure = (short) gatoi(secure);
		} else {
			nSecure = 0;
	}

// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifdef CYCLADES_TSxK
		nPriv = (short) gatoi(priv);
		nCheck = umAddAccessLimit(url, am, nSecure, nPriv);
#else
		nCheck = umAddAccessLimit(url, am, nSecure, group);
#endif
		if (nCheck != 0) {
			sprintf(message, "ERROR: Unable to add Access Limit for [%s].",	url);
		} else {
			sprintf(message, "Access limit for [%s], was successfully added.", url);
			error = 0;
		}
	}
	websCycHdrInit(&hdr);
	hdr.title = "Web User Management - Add an Access Limit";
	hdr.father = "/read/weblimit.asp";
	hdr.links = "&nbsp;<a href=\"/read/weblimit.asp\">[Access Limit List]"
		"</a>";
	websPageMessage(wp, &hdr, message, error);
}

/******************************************************************************/
/*
 *  Delete a user
 */

static void formHandleUser(webs_t wp, char_t *path, char_t *query)
{
	char_t	*userid, *ok;
	char_t message[128], error=1;
	struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
	hdr.father = "/read/webuser.asp";
	hdr.links = "&nbsp;<a href=\"/read/webuser.asp\">"
		"[User List]</a>";

	a_assert(wp);

	userid = websGetVar(wp, T("user"), T("")); 
	ok = websGetVar(wp, T("ok"), T("")); 


	if (*ok == 'A') {
		websRedirect(wp, "/um/adduser.asp", NULL);
	} else if (*ok == 'D') {
		if (umUserExists(userid) == FALSE) {
			sprintf(message, "ERROR: User \"%s\" not found", userid);
		} else if (umGetUserProtected(userid)) {
			sprintf(message, "ERROR: User \"%s\" is delete-protected.", userid);
		} else if (umDeleteUser(userid) != 0) {
			sprintf(message, "ERROR: Unable to delete user \"%s\".", userid);
		} else {
			sprintf(message, "User \"%s\" was successfully deleted.", userid);
			error = 0;
		}
		hdr.title =  "Web User Management - Delete a User";
		websPageMessage(wp, &hdr, message, error);
	} else if (*ok == 'C') {
		if (umUserExists(userid) == FALSE) { //RK
			sprintf(message, "ERROR: User \"%s\" not found", userid);			hdr.title = "Web User Management - Change Password";
			websPageMessage(wp, &hdr, message, error);
			return;
		}
		sprintf(message, "Web User Management - Change Password for %s", userid);
		hdr.need_header = 1;
		hdr.title = message;
		hdr.form = "um/ChgPasswd";
		websPageStart(wp, &hdr);
		websWrite(wp, "<input type=\"hidden\" name=\"user\" value=\"%s\">"
			"<center><table border><tr><th class=\"tt\" colspan=\"2\">%s</th></tr>"
			"<tr><td><b>Password</b></td><td><input class=\"tx\" type=\"password\" name=\"password\"></td></tr>"
			"<tr><td><b>Confirm</b></td><td><input class=\"tx\" type=\"password\" name=\"passconf\"></td></tr>"
			"</table><br><input type=\"submit\" name=\"opt\" value=\"Submit\">"
			"&nbsp;&nbsp;&nbsp;&nbsp;<input type=\"submit\" name=\"opt\" value=\"Cancel\"></center>",
			userid, message);
		websPageEnd(wp, &hdr);
	}
}

/******************************************************************************/
/*
 *  Delete a group
 */

static void formHandleGroup(webs_t wp, char_t *path, char_t *query)
{
	char_t	*group, *ok;
	struct cyc_webs_header hdr;

	a_assert(wp);

	group = websGetVar(wp, T("group"), T("")); 
	ok = websGetVar(wp, T("ok"), T("")); 

	if (*ok == 'A') {
		websRedirect(wp, "/um/addgroup.asp", NULL);
	} else if (*ok == 'D') {
		char_t message[128], error=1;
		if ((group == NULL) || (*group == '\0')) {
			strcpy(message, "ERROR: No group was selected.");
	} else if (umGetGroupProtected(group)) {
			sprintf(message, "ERROR: Group \"%s\" is delete-protected.", group);
	} else if (umGetGroupInUse(group)) {
			sprintf(message, "ERROR: Group \"%s\" is being used.",	group);
	} else if (umDeleteGroup(group) != 0) {
			sprintf(message, "ERROR: Unable to delete group \"%s\".", group);
	} else {
			sprintf(message, "Group \"%s\" was successfully deleted.", group);
			error=0;
		}
		websCycHdrInit(&hdr);
		hdr.need_header = 1;
		hdr.title = "Web User Management - Delete a Group";
		hdr.father = "/read/webgroup.asp";
		hdr.links = "&nbsp;<a href=\"/read/webgroup.asp\">"
			"[Group List]</a>";
		websPageMessage(wp, &hdr, message, error);
	}
}

/******************************************************************************/
/*
 *  Delete an Access Limit
 */

static void formHandleAccessLimit(webs_t wp, char_t *path, char_t *query)
{
	char_t	*url, *ok;
	struct cyc_webs_header hdr;

	a_assert(wp);

	url = websGetVar(wp, T("url"), T(""));
	ok = websGetVar(wp, T("ok"), T(""));

	if (*ok == 'A') {
		websRedirect(wp, "/um/addlimit.asp", NULL);
	} else if (*ok == 'D') {
		char_t message[128], error=1;
		if (umDeleteAccessLimit(url) != 0) {
			sprintf(message, "ERROR: Unable to delete Access Limit for [%s]", url);
		} else {
			sprintf(message, "Access Limit for [%s], was successfully deleted.", url);
			error=0;
		}
		websCycHdrInit(&hdr);
		hdr.need_header = 1;
		hdr.title = "Web User Management - Delete an Access Limit";
		hdr.father = "/read/weblimit.asp";
		hdr.links = "&nbsp;<a href=\"/read/weblimit.asp\">[Access Limit List]</a>";
		websPageMessage(wp, &hdr, message, error);
	}
}

/******************************************************************************/
/*
 *  Save the user management configuration to a file
 */

static void formFileUserManagement(webs_t wp, char_t *path, char_t *query)
{
	char_t	*ok;
	struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
	hdr.father = "/um/umconfig.asp";

	a_assert(wp);

	ok = websGetVar(wp, T("ok"), T(""));

	if (*ok == 'S') {
		hdr.title =  "Web User Management - Save Configuration";
		if (umCommit(NULL) != 0) {
			websPageMessage(wp, &hdr, "ERROR: Unable to save user "
				"configuration.", 1);
		} else {
			websPageMessage(wp, &hdr, "User configuration was "
				"saved successfully.", 0);
		}
	} else if (*ok == 'L') {
		hdr.title = "Web User Management - Load Configuration";
		if (umRestore(NULL) != 0) {
			websPageMessage(wp, &hdr, "ERROR: Unable to load user "
				"configuration.", 1);
		} else {
			websPageMessage(wp, &hdr, "User configuration was "
				"reloaded successfully.", 0);
		}
	}
}

/******************************************************************************/
/*
 *  Generate HTML to create a list box containing the users
 */

// Jul/25/2001 Cyclades TSxK Livio: List all users as checkbox (complete info).
static int aspGenerateUserList(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t	*userid, *temp;
	int		row, nBytesSent, nBytes;
	bool_t	enabled;
	unsigned char count = 1;

	a_assert(wp);

	row = 0;
	userid = umGetFirstUser();
	nBytesSent = 0;

	while (userid) {
		nBytes = websWrite(wp, T("<tr><td>"));
		if (websGetPriority(wp) == PRIV_FULL) {
			nBytes += websWrite(wp, T("<input type=\"radio\" name=\"%s\" value=\"%s\">&nbsp;"), "user", userid);
		}
		nBytes += websWrite(wp, T("%d</td>"), count++);
		nBytes += websWrite(wp, T("<td>%s</td>"), userid);
		temp = umGetUserGroup(userid);
		nBytes += websWrite(wp, T("<td>%s</td>"), temp);
		enabled = umGetUserEnabled(userid);
		nBytes += websWrite(wp, T("<td>%s</td></tr>"), enabled?"Enabled":"Disabled");
		
		userid = umGetNextUser(userid);
		nBytesSent += nBytes;
	}

	return nBytesSent;
}

/******************************************************************************/
/*
 *  Generate HTML to create a list box containing the groups
 */

static int aspGenerateGroupList(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t	*group;
	int		row, nBytesSent, nBytes;

	a_assert(wp);

	row = 0;
	nBytesSent = 0;
	nBytes = websWrite(wp, 
		T("<SELECT NAME=\"group\" SIZE=\"4\" TITLE=\"Select a Group\">"));
/*
 *  Add a special "<NONE>" element to allow de-selection
 */
//	nBytes = websWrite(wp, T("<OPTION VALUE=\"\">[NONE]\n"));

	group = umGetFirstGroup();
	while (group && (nBytes > 0)) {
		nBytes = websWrite(wp, T("<OPTION VALUE=\"%s\">%s\n"), group, group);
		group = umGetNextGroup(group);
		nBytesSent += nBytes;
	}

	nBytesSent += websWrite(wp, T("</SELECT>"));

	return nBytesSent;
}


/******************************************************************************/
/*
 *  Generate HTML to create a list check box containing the groups
 */

// Jul/25/2001 Cyclades TSxK Livio: List all groups as checkbox (complete info).
static int aspGenerateGroupFullList(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t	*group;
	int		row, nBytesSent, nBytes;
	short	privilege;
// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifndef CYCLADES_TSxK
	accessMeth_t	amRet;
#endif	
	bool_t	enabled;
	unsigned char count = 1;
	a_assert(wp);
	row = 0;
	nBytesSent = 0;
	group = umGetFirstGroup();

	while (group) {
		nBytes = websWrite(wp, T("<tr><td>"));
		if (websGetPriority(wp) == PRIV_FULL) {
			nBytes += websWrite(wp, T("<input type=\"radio\" name=\"%s\" value=\"%s\">&nbsp;"), "group", group);
		}
		nBytes += websWrite(wp, T("%d</td>"), count++);
		nBytes += websWrite(wp, T("<td>%s</td>"), group);
		privilege = umGetGroupPrivilege(group);
		nBytes += websWrite(wp, T("<td>%s%s%s%s</td>"), 
				(privilege == PRIV_USER)?"User ":"", 
				(privilege == PRIV_MONITOR)?"Monitor ":"", 
				(privilege == PRIV_ADMIN)?"Administrator ":"",
				(privilege == PRIV_FULL)?"Full ":"");

#ifndef CYCLADES_TSxK
		amRet = umGetGroupAccessMethod(group);
		nBytes += websWrite(wp, T("<td>%s%s%s%s%s%s</td>"),
				(amRet == AM_NONE)?"NONE":"", 
				(amRet == AM_FULL)?"FULL":"", 
				(amRet == AM_BASIC)?"BASIC":"", 
				(amRet == AM_DIGEST)?"DIGEST":"", 
				(amRet == AM_COOKIE)?"COOKIE":"",
				(amRet == AM_INVALID)?"INVALID":"");
#endif

		enabled = umGetGroupEnabled(group);
		nBytes += websWrite(wp, T("<td>%s</td></tr>"), enabled?"Enabled":"Disabled");

		group = umGetNextGroup(group);
		nBytesSent += nBytes;
	}

	return nBytesSent;
}

/******************************************************************************/
/*
 *  Generate HTML to create a list box containing the access limits
 */

// Jul/25/2001 Cyclades TSxK Livio: List all access limit as checkbox (complete info).
static int aspGenerateAccessLimitList(int eid, webs_t wp, 
									  int argc, char_t **argv)
{

// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifdef CYCLADES_TSxK
	short	priv;
#else
	char_t	*group;
#endif
		  
	char_t	*url;
	int		row, nBytesSent, nBytes;
	accessMeth_t amRet;
	short secure;
	unsigned char count = 1;

	a_assert(wp);

	row = nBytesSent = 0;
	url = umGetFirstAccessLimit();

	while (url) {
		nBytes = websWrite(wp, T("<tr><td>"));
		if (websGetPriority(wp) == PRIV_FULL) {
			nBytes += websWrite(wp, T("<input type=\"radio\" name=\"%s\" value=\"%s\">&nbsp;"), "url", url);
		}
		nBytes += websWrite(wp, T("%d</td>"), count++);
		nBytes += websWrite(wp, T("<td>%s</td>"), url);

		// Jul/26/2001 Cyclades TSxk Livio: Config Privilege in Access Limit
#ifdef CYCLADES_TSxK
		priv = umGetAccessLimitPrivilege(url);
		nBytes += websWrite(wp, T("<td>%s%s%s%s</td>"), 
				(priv == PRIV_USER)?"User ":"", 
				(priv == PRIV_MONITOR)?"Monitor ":"", 
				(priv == PRIV_ADMIN)?"Administrator ":"",
				(priv == PRIV_FULL)?"Full ":"");
#else
		group = umGetAccessLimitGroup(url);
		nBytes += websWrite(wp, T("<td>%s</td>"), group);
#endif
		amRet = umGetAccessLimitMethod(url);
		nBytes += websWrite(wp, T("<td>%s%s%s%s%s%s</td>"),
				(amRet == AM_NONE)?"NONE":"", 
				(amRet == AM_FULL)?"FULL":"", 
				(amRet == AM_BASIC)?"BASIC":"", 
				(amRet == AM_DIGEST)?"DIGEST":"", 
				(amRet == AM_COOKIE)?"COOKIE":"",
				(amRet == AM_INVALID)?"INVALID":"");

		secure = umGetAccessLimitSecure(url);
		nBytes += websWrite(wp, T("<td>%s</td></tr>"), secure?"Yes":"No");
		
		url = umGetNextAccessLimit(url);
		nBytesSent += nBytes;
	}

	return nBytesSent;
}

/******************************************************************************/
/*
 *  Generate HTML to create a list box containing the access methods
 */

static int aspGenerateAccessMethodList(int eid, webs_t wp, 
									   int argc, char_t **argv)
{
	int		nBytes;

	a_assert(wp);

	nBytes = websWrite(wp, 
		T("<SELECT NAME=\"method\" SIZE=\"5\" TITLE=\"Select a Method\">"));
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Full Access\n"), 
		AM_FULL);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Basic Access\n"), 
		AM_BASIC);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Digest Access\n"),
		AM_DIGEST);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\" SELECTED>Cookie Access\n"),
		AM_COOKIE);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">No Access\n"),
		AM_NONE);
	nBytes += websWrite(wp, T("</SELECT>"));

	return nBytes;
}
/******************************************************************************/
/*
 *  Generate HTML to create a list box containing privileges
 */

static int aspGeneratePrivilegeList(int eid, webs_t wp, 
									int argc, char_t **argv)
{
	int		nBytes;
	char_t	*type;

	a_assert(wp);

	ejArgs(argc, argv, "%s", &type);
	
	nBytes = websWrite(wp, T("<SELECT NAME=\"privilege\" SIZE=\"4\" "));
	nBytes += websWrite(wp, T("%sTITLE=\"Choose Privilege Level\">"), type);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">User\n"), 
			PRIV_USER);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Monitor\n"), 
			PRIV_MONITOR);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Administrator\n"), 
		PRIV_ADMIN);
	nBytes += websWrite(wp, T("<OPTION VALUE=\"%d\">Full\n"), 
			PRIV_FULL);
	nBytes += websWrite(wp, T("</SELECT>"));

	return nBytes;
}

/******************************************************************************/
/*
 *  Generate HTML to print the submit inputs
 */

static int aspPrintSubmit(int eid, webs_t wp, int argc, char_t **argv)
{
	int		nBytes;
	char_t	*name;
	a_assert(wp);
	ejArgs(argc, argv, "%s", &name);
	if (websGetPriority(wp) == PRIV_FULL) {
		nBytes = websWrite(wp, "<br><center>"
			"<input type=submit name=ok value=\"Add %s\">"
			"&nbsp;&nbsp;&nbsp;&nbsp;"
			"<input type=submit name=ok value=\"Delete %s\">"
			, name, name);
		if (strcmp(name, "User") == 0) {
			nBytes += websWrite(wp, "&nbsp;&nbsp;&nbsp;&nbsp;"
				"<input type=submit name=ok "
				"value=\"Change Password\">");
	}
		nBytes+= websWrite(wp, "</center>");
	}

	return nBytes;
}

/******************************************************************************/

